@@ -29,6 +29,7 @@
 #include <string.h>		/* for strcmp() */
 
 #include "shunk.h"
+#include "names.h"
 
 struct jambuf;
 
@@ -40,6 +41,15 @@ enum {
 	secs_per_day = 24 * secs_per_hour
 };
 
+enum binary {
+	binary_per_kilo = UINT64_C(1024),
+	binary_per_mega = UINT64_C(1024) * binary_per_kilo,
+	binary_per_giga = UINT64_C(1024) * binary_per_mega,
+	binary_per_tera = UINT64_C(1024) * binary_per_giga,
+	binary_per_peta = UINT64_C(1024) * binary_per_tera,
+	binary_per_exa  = UINT64_C(1024) * binary_per_peta, /* 2^64 s 16 Exa */
+};
+
 /*
  * This file was split into internal constants (Libreswan/pluto related),
  * and external constants (defined by IETF, etc.)
@@ -93,14 +103,9 @@ enum {
 
 #define NULL_FD (-1)	/* NULL file descriptor */
 
-#include <inttypes.h>
-
-#include <prcpucfg.h>	/* from nspr4 devel */
-
-#ifndef BITS_PER_BYTE
-# define BITS_PER_BYTE  8
-#endif
-#define BYTES_FOR_BITS(b)   (((b) + BITS_PER_BYTE - 1) / BITS_PER_BYTE)
+/* octet_t / BITS_IN_OCTET anyone? */
+#define BITS_IN_BYTE 8
+#define BYTES_FOR_BITS(b)   (((b) + BITS_IN_BYTE - 1) / BITS_IN_BYTE)
 
 /* clearer shorthand for *cmp functions */
 #define streq(a, b) (strcmp((a), (b)) == 0)
@@ -118,23 +123,6 @@ enum {
 		memeq(l_, r_, sizeof(L));				\
 	})
 
-/* Remember, THING 1 and THING 2 are inseparable */
-#define FOR_EACH_THING(THING, THING1, THING2, ...)			\
-	for (typeof(THING1) things_[] = { THING1, THING2, ##__VA_ARGS__ }, \
-		     *thingp_ = things_, THING;				\
-	     thingp_ < things_ + elemsof(things_) ? (THING = *thingp_, true) : false; \
-	     thingp_++)
-
-#define FOR_EACH_ELEMENT(THING, ARRAY)			\
-	for (typeof(&(ARRAY)[0]) THING = (ARRAY);	\
-	     THING < (ARRAY) + elemsof(ARRAY);		\
-	     THING++)
-
-#define FOR_EACH_ELEMENT_FROM_1(THING, ARRAY)		\
-	for (typeof(&(ARRAY)[1]) THING = &(ARRAY)[1];	\
-	     THING < (ARRAY) + elemsof(ARRAY);		\
-	     THING++)
-
 /*
  * Fill a string field, ensuring that it is padded and terminated with NUL
  * If termination isn't required, strncpy would do.
@@ -184,6 +172,7 @@ enum {
 #define messup(x) messupn((x), sizeof(*(x)))	/* set all bytes to wrong value */
 
 extern const char *bool_str(bool b);	/* bool -> string */
+err_t ttobool(const char *t, bool *b);	/* string -> bool */
 
 /* routines to copy C strings to fixed-length buffers */
 extern char *jam_str(char *dest, size_t size, const char *src);
@@ -207,8 +196,8 @@ extern char *add_str(char *buf, size_t size, char *hint, const char *src);
  *
  * Recommended for determining an enum's validity:
  *
- *   enum_name*() returns the name of an enum value, or NULL if
- *   unnamed.
+ *   enum_name*() returns true when known, and sets enum_buf to a
+ *   non-NULL string.
  *
  * Recommended for logging:
  *
@@ -219,18 +208,8 @@ extern char *add_str(char *buf, size_t size, char *hint, const char *src);
  *   a mashup of the standard prefix and the numeric value.
  *
  * {*}_short() same as for root, but with any standard prefix removed.
- */
-
-typedef const struct enum_names enum_names;
-
-extern const char *enum_name(enum_names *ed, unsigned long val);
-extern const char *enum_name_short(enum_names *ed, unsigned long val);
-
-size_t jam_enum(struct jambuf *, enum_names *en, unsigned long val);
-size_t jam_enum_short(struct jambuf *, enum_names *en, unsigned long val);
-
-/*
- * caller-allocated buffer for str_enum*().
+ *
+ * Caller-allocated buffer
  *
  * Enough space for decimal rep of any unsigned long + "??"  sizeof
  * yields log-base-256 of maximum value.  Multiplying by 241/100
@@ -238,16 +217,33 @@ size_t jam_enum_short(struct jambuf *, enum_names *en, unsigned long val);
  * rounded up a little (instead of 2.40654...).  The addition of 99
  * ensures that the division rounds up to an integer rather than
  * truncates.
+ *
+ * The .name field points either at buf[] or some internal string.  It
+ * is never NULL.  DANGER: enum_buf can't be returned as that moves
+ * the struct invalidating the internal pointer.
  */
-typedef struct {
-	char buf[(sizeof(unsigned long) * 241 + 99) / 100 + sizeof("??")];
-} enum_buf;
-typedef enum_buf esb_buf; /* XXX: TBD */
 
-extern const char *str_enum(enum_names *ed, unsigned long val, enum_buf *);
-extern const char *str_enum_short(enum_names *ed, unsigned long val, enum_buf *);
+#define enum_buf name_buf
+#define esb_buf  name_buf /* XXX: TBD */
 
-#define enum_show str_enum /* XXX: TBD */
+typedef const struct enum_names enum_names;
+
+bool enum_long(const struct enum_names *en, unsigned long val, enum_buf *b);
+const char *str_enum_long(const struct enum_names *en, unsigned long val, enum_buf *b);
+size_t jam_enum_long(struct jambuf *, const struct enum_names *en, unsigned long val);
+
+#define jam_enum jam_enum_long /* legacy */
+#define str_enum str_enum_long /* legacy */
+#define enum_name enum_long /* legacy */
+
+bool enum_short(const struct enum_names *en, unsigned long val, enum_buf *b);
+const char *str_enum_short(const struct enum_names *en, unsigned long val, enum_buf *b);
+size_t jam_enum_short(struct jambuf *, const struct enum_names *en, unsigned long val);
+
+#define enum_name_short enum_short
+
+/* drop prefix + transform [_A-Z]->[-a-z] */
+size_t jam_enum_human(struct jambuf *, enum_names *en, unsigned long val);
 
 /*
  * iterator
@@ -287,7 +283,6 @@ const char *enum_range_name(enum_names *range, unsigned long val, const char *pr
  *
  * Like above:
  *
- * enum_enum_table() returns TABLE's enum_names, or NULL.
  * enum_enum_name() returns TABLE VAL's enum, or NULL.
  *
  * jam_enum_enum() appends TABLE VAL's enum name; if unnamed, append a
@@ -300,9 +295,8 @@ const char *enum_range_name(enum_names *range, unsigned long val, const char *pr
 
 typedef const struct enum_enum_names enum_enum_names;
 
-enum_names *enum_enum_table(enum_enum_names *e, unsigned long table);
-const char *enum_enum_name(enum_enum_names *e, unsigned long table,
-			   unsigned long val);
+bool enum_enum_name(enum_enum_names *e, unsigned long table,
+		    unsigned long val, enum_buf *buf);
 
 const char *str_enum_enum(enum_enum_names *e, unsigned long table,
 			  unsigned long val, enum_buf *buf);
@@ -317,8 +311,6 @@ size_t jam_enum_enum_short(struct jambuf *log, enum_enum_names *een,
 /* XXX: assumes enums are 8-bits!?! */
 #define LOOSE_ENUM_OTHER 255
 
-extern void init_constants(void);
-
 #include "ietf_constants.h"
 #include "pluto_constants.h"
 #include "names_constant.h"
