@@ -16,7 +16,14 @@
 #ifndef IKE_ALG_ENCRYPT_OPS_H
 #define IKE_ALG_ENCRYPT_OPS_H
 
+#include "chunk.h"
+#include "shunk.h"
+
 struct logger;
+enum cipher_op;
+enum cipher_iv_source;
+struct cipher_op_context;
+struct crypt_mac;
 
 struct encrypt_ops {
 	const char *backend;
@@ -24,20 +31,37 @@ struct encrypt_ops {
 	/*
 	 * Delegate responsibility for checking OPS specific fields.
 	 */
-	void (*const check)(const struct encrypt_desc *alg, struct logger *logger);
+	void (*const cipher_check)(const struct encrypt_desc *cipher,
+				   struct logger *logger);
+
+	/*
+	 * Create/delete a crypto context for use by the IKE SA.
+	 */
+	struct cipher_op_context *(*const cipher_op_context_create)(const struct encrypt_desc *cipher,
+								    enum cipher_op,
+								    enum cipher_iv_source,
+								    PK11SymKey *symkey,
+								    shunk_t salt,
+								    struct logger *logger);
+	void (*const cipher_op_context_destroy)(struct cipher_op_context **context,
+						struct logger *logger);
 
 	/*
 	 * Perform simple encryption.
 	 *
 	 * Presumably something else is implementing the integrity.
 	 */
-	void (*const do_crypt)(const struct encrypt_desc *alg,
-			       uint8_t *dat,
-			       size_t datasize,
-			       PK11SymKey *key,
-			       uint8_t *iv,
-			       bool enc,
-			       struct logger *logger);
+	void (*const cipher_op_normal)(const struct encrypt_desc *cipher,
+				       struct cipher_op_context *context,
+				       enum cipher_op op,
+				       enum cipher_iv_source iv_source,
+				       PK11SymKey *symkey,
+				       shunk_t salt,
+				       chunk_t wire_iv,
+				       chunk_t text,
+				       /**/
+				       struct crypt_mac *ikev1_iv,
+				       struct logger *logger);
 
 	/*
 	 * Perform Authenticated Encryption with Associated Data
@@ -51,20 +75,27 @@ struct encrypt_ops {
 	 * tag (which is appended to the text).
 	 *
 	 * All sizes are in 8-bit bytes.
+	 *
+	 * Danger: TEXT and TAG are clearly contigious.
 	 */
-	bool (*const do_aead)(const struct encrypt_desc *alg,
-			      uint8_t *salt, size_t salt_size,
-			      uint8_t *wire_iv, size_t wire_iv_size,
-			      uint8_t *aad, size_t aad_size,
-			      uint8_t *text_and_tag,
-			      size_t text_size, size_t tag_size,
-			      PK11SymKey *key, bool enc,
-			      struct logger *logger);
+
+	bool (*const cipher_op_aead)(const struct encrypt_desc *cipher,
+				     struct cipher_op_context *context,
+				     enum cipher_op op,
+				     enum cipher_iv_source iv_source,
+				     PK11SymKey *symkey,
+				     shunk_t salt,
+				     /**/
+				     chunk_t wire_iv,
+				     shunk_t aad,
+				     chunk_t text_and_tag,
+				     size_t text_size, size_t tag_size,
+				     struct logger *logger);
 };
 
 extern const struct encrypt_ops ike_alg_encrypt_nss_aead_ops;
 extern const struct encrypt_ops ike_alg_encrypt_nss_cbc_ops;
 extern const struct encrypt_ops ike_alg_encrypt_nss_ctr_ops;
-extern const struct encrypt_ops ike_alg_encrypt_nss_gcm_ops;
+extern const struct encrypt_ops ike_alg_encrypt_null_ops;
 
 #endif
