@@ -19,6 +19,7 @@
 #include "chunk.h"
 
 struct logger;
+struct secret_preshared_stuff;
 
 struct prf_ikev1_ops {
 	const char *backend;
@@ -43,7 +44,7 @@ struct prf_ikev1_ops {
 					PK11SymKey *dh_secret,
 					struct logger *logger);
 	PK11SymKey *(*pre_shared_key_skeyid)(const struct prf_desc *prf_desc,
-					     chunk_t pre_shared_key,
+					     const struct secret_preshared_stuff *pre_shared_key,
 					     chunk_t Ni_b, chunk_t Nr_b,
 					     struct logger *logger);
 
