@@ -25,4 +25,5 @@ extern const struct cbc_test_vector *const aes_cbc_tests;
 extern const struct cbc_test_vector *const camellia_cbc_tests;
 
 bool test_cbc_vectors(const struct encrypt_desc *encrypt_desc,
-		      const struct cbc_test_vector *tests);
+		      const struct cbc_test_vector *tests,
+		      struct logger *logger);
