@@ -24,6 +24,7 @@
 struct fd;
 struct logger;
 struct jambuf;
+struct whack_impair;
 
 /*
  * Meddle with the contents of a payload.
@@ -34,7 +35,7 @@ enum impair_emit {
 	IMPAIR_EMIT_OMIT,
 	IMPAIR_EMIT_EMPTY,
 	IMPAIR_EMIT_DUPLICATE,
-	IMPAIR_EMIT_ROOF, /* >= ROOF -> <number> */
+#define IMPAIR_EMIT_ROOF (IMPAIR_EMIT_DUPLICATE+1) /* >= ROOF -> <number> */
 };
 
 /*
@@ -60,6 +61,11 @@ enum impair_v2_transform {
 	IMPAIR_v2_TRANSFORM_OMIT,
 };
 
+struct impair_unsigned {
+	bool enabled;
+	unsigned value;
+};
+
 /*
  * What can be impaired.
  *
@@ -87,25 +93,23 @@ struct impair {
 	enum impair_emit ike_key_length_attribute;
 	enum impair_emit child_key_length_attribute;
 
-	unsigned log_rate_limit;
+	struct impair_unsigned log_rate_limit;
 
 	enum impair_emit v1_hash_payload;
 	enum impair_v1_exchange v1_hash_exchange;
 	bool v1_hash_check;
 
-	unsigned ike_initiator_spi;
-	unsigned ike_responder_spi;
+	struct impair_unsigned ike_initiator_spi;
+	struct impair_unsigned ike_responder_spi;
 
 	bool bust_mi2;
 	bool bust_mr2;
 	bool drop_i2;
-	bool sa_creation;
 	bool jacob_two_two;
 	bool allow_null_none;
 	bool major_version_bump;
 	bool minor_version_bump;
 	bool timeout_on_retransmit;
-	bool delete_on_retransmit;
 	bool suppress_retransmits;
 	bool send_bogus_payload_flag;
 	bool send_bogus_isakmp_flag;
@@ -123,11 +127,10 @@ struct impair {
 
 	enum impair_v2_transform v2_proposal_integ;
 	enum impair_v2_transform v2_proposal_dh;
-	unsigned ikev2_add_ike_transform;
-	unsigned ikev2_add_child_transform;
-	bool replay_duplicates;
-	bool replay_forward;
-	bool replay_backward;
+
+	struct impair_unsigned ikev2_add_ike_transform;
+	struct impair_unsigned ikev2_add_child_transform;
+
 	bool replay_encrypted;
 	bool corrupt_encrypted;
 	bool proposal_parser;
@@ -146,14 +149,15 @@ struct impair {
 	bool tcp_use_blocking_write;
 	bool tcp_skip_setsockopt_espintcp;
 
-	unsigned add_unknown_v2_payload_to;
-	unsigned add_unknown_v2_payload_to_sk;
+	struct impair_unsigned add_unknown_v2_payload_to;
+	struct impair_unsigned add_unknown_v2_payload_to_sk;
 	bool unknown_v2_payload_critical;
 
-	unsigned force_v2_auth_method;
+	struct impair_unsigned force_v2_auth_method;
 
-	bool ignore_v2N_SIGNATURE_HASH_ALGORITHMS;
-	bool omit_v2N_SIGNATURE_HASH_ALGORITHMS;
+	struct impair_unsigned add_v2_notification;
+	struct impair_unsigned ignore_v2_notification;
+	struct impair_unsigned omit_v2_notification;
 
 	bool omit_v2_ike_auth_child;
 	bool ignore_v2_ike_auth_child;
@@ -162,60 +166,86 @@ struct impair {
 	bool ignore_soft_expire;
 	bool ignore_hard_expire;
 
+	bool cannot_ondemand;
+
+	struct impair_unsigned number_of_TSi_selectors;
+	struct impair_unsigned number_of_TSr_selectors;
+
+	bool lifetime;
+
+	bool copy_v1_notify_response_SPIs_to_retransmission;
+
+	struct impair_unsigned v1_remote_quick_id;
+	struct impair_unsigned v1_emit_quick_id;
+
 	enum impair_emit v1_isakmp_delete_payload;
 	enum impair_emit v1_ipsec_delete_payload;
 
-	unsigned v2_delete_protoid;
-	unsigned v2n_rekey_sa_protoid;
-	unsigned v2_proposal_protoid;
+	struct impair_unsigned v2_delete_protoid;
+	struct impair_unsigned v2n_rekey_sa_protoid;
+	struct impair_unsigned v2_proposal_protoid;
+
+	struct impair_unsigned helper_thread_delay;
+
+	bool install_ipsec_sa_inbound_state;
+	bool install_ipsec_sa_inbound_policy;
+	bool install_ipsec_sa_outbound_state;
+	bool install_ipsec_sa_outbound_policy;
+
+	bool record_inbound;
+	bool record_outbound;
 
 	/*
 	 * add more here
 	 */
 
-	unsigned v1_remote_quick_id;
-
 };
 
 extern struct impair impair;
 
-/*
- * What whack sends across the wire for a impair.
- */
-
-struct whack_impair {
-	unsigned what;
-	unsigned biased_value;
-};
-
 enum impair_status {
 	IMPAIR_OK = 1,
 	IMPAIR_HELP,
 	IMPAIR_ERROR,
 };
 
-enum impair_status parse_impair(const char *optarg, struct whack_impair *whack_impair,
+enum impair_status parse_impair(const char *optarg,
+				struct whack_impair *whack_impair,
 				bool enable, struct logger *logger);
 
+enum impair_message_direction {
+	IMPAIR_INBOUND_MESSAGE,
+	IMPAIR_OUTBOUND_MESSAGE,
+};
+
 enum impair_action {
 	CALL_IMPAIR_UPDATE,
+
 	CALL_INITIATE_v2_LIVENESS,
 	CALL_SEND_KEEPALIVE,
+
 	CALL_GLOBAL_EVENT_HANDLER,
 	CALL_STATE_EVENT_HANDLER,
-	CALL_IMPAIR_DROP_INCOMING,
-	CALL_IMPAIR_DROP_OUTGOING,
+	CALL_CONNECTION_EVENT_HANDLER,
+
+	CALL_IMPAIR_MESSAGE_DROP,
+	CALL_IMPAIR_MESSAGE_BLOCK,
+	CALL_IMPAIR_MESSAGE_DRIP,
+	CALL_IMPAIR_MESSAGE_DUPLICATE,
+	CALL_IMPAIR_MESSAGE_REPLAY,
+
 #if 0
-	CALL_IMPAIR_DUP_INCOMING,
-	CALL_IMPAIR_DUP_OUTGOING,
-	CALL_IMPAIR_CORRUPT_INCOMING,
-	CALL_IMPAIR_CORRUPT_OUTGOING,
+	CALL_IMPAIR_CORRUPT_INBOUND,
+	CALL_IMPAIR_CORRUPT_OUTBOUND,
 #endif
 };
 
 bool process_impair(const struct whack_impair *whack_impair,
-		    void (*action)(enum impair_action, unsigned what,
-				   unsigned how, bool background,
+		    void (*action)(enum impair_action impairment_action,
+				   unsigned impairment_param,
+				   bool whack_enable,
+				   unsigned whack_value,
+				   bool background,
 				   struct logger *logger),
 		    bool background, struct logger *logger);
 
