@@ -56,11 +56,8 @@ typedef struct {
 	 * possible (struct in_addr requires htonl() which is run-time
 	 * only).
 	 */
-	struct ip_bytes bytes;
-	/*
-	 * (routing prefix) bits.
-	 */
-	unsigned maskbits;
+	struct ip_bytes lo;
+	struct ip_bytes hi;
 	unsigned ipproto;
 	/*
 	 * For moment, one port
@@ -68,21 +65,22 @@ typedef struct {
 	int hport;
 } ip_selector;
 
-#define PRI_SELECTOR "<selector-%s:IPv%d,%s["PRI_IP_BYTES"]:%u/%u>"
+#define PRI_SELECTOR "<selector-%s:IPv%d,%s["PRI_IP_BYTES".."PRI_IP_BYTES"]:%u>"
 #define pri_selector(S)						\
 		((S)->is_set ? "set" : "unset"),		\
 		(S)->version,					\
 		((S)->ipproto > 255 ? "IPPROTO>255" :		\
 		 protocol_from_ipproto((S)->ipproto)->name),	\
-		pri_ip_bytes((S)->bytes),			\
-		(S)->hport,					\
-		(S)->maskbits
+		pri_ip_bytes((S)->lo),				\
+		pri_ip_bytes((S)->hi),				\
+		(S)->hport
 
 void pexpect_selector(const ip_selector *s, where_t where);
 #define pselector(S) pexpect_selector(S, HERE)
 
-ip_selector selector_from_raw(where_t where, enum ip_version version,
-			      const struct ip_bytes prefix, unsigned prefix_length,
+ip_selector selector_from_raw(where_t where, const struct ip_info *afi,
+			      const struct ip_bytes lo,
+			      const struct ip_bytes hi,
 			      const struct ip_protocol *protocol, const ip_port port);
 
 ip_selector selector_from_address(const ip_address address);
@@ -167,7 +165,9 @@ int selector_hport(const ip_selector selector);
 /* assuming a subnet like XFRM does */
 ip_address selector_prefix(const ip_selector selector);
 ip_address selector_prefix_mask(const ip_selector selector);
-unsigned selector_prefix_bits(const ip_selector selector);
+
+int selector_prefix_len(const ip_selector selector); /* <0 when non-CIDR*/
+int selector_host_len(const ip_selector selector); /* <0 when non-CIDR*/
 
 bool selector_contains_one_address(const ip_selector selector);
 
@@ -188,16 +188,16 @@ bool selector_overlaps_selector(const ip_selector l, const ip_selector r);
 /* printing */
 
 typedef struct {
-	char buf[sizeof(address_buf) + 4/*"/NNN"*/ + 6/*:65535*/];
+	char buf[(sizeof(address_buf) + 4/*/NNN*/ + 6/*/CHAOS*/ + 6/*/65535*/)*2 + 1/*-*/];
 } selector_buf;
 
 const char *str_selector(const ip_selector *selector, selector_buf *out);
-const char *str_selector_subnet(const ip_selector *selector, subnet_buf *buf);
-const char *str_selector_subnet_port(const ip_selector *selector, selector_buf *out);
+const char *str_selector_range(const ip_selector *selector, subnet_buf *buf);
+const char *str_selector_range_port(const ip_selector *selector, selector_buf *out);
 
 size_t jam_selector(struct jambuf *buf, const ip_selector *selector);
-size_t jam_selector_subnet(struct jambuf *buf, const ip_selector *selector);
-size_t jam_selector_subnet_port(struct jambuf *buf, const ip_selector *selector);
+size_t jam_selector_range(struct jambuf *buf, const ip_selector *selector);
+size_t jam_selector_range_port(struct jambuf *buf, const ip_selector *selector);
 
 typedef struct {
 	char buf[sizeof(selector_buf) + sizeof("=UNKNOWN=UNKNOWN=>") + sizeof(selector_buf)];
@@ -222,8 +222,6 @@ size_t jam_selector_pair_sensitive(struct jambuf *buf, const ip_selector *src, c
 
 ip_subnet selector_subnet(const ip_selector selector);
 bool selector_range_eq_selector_range(const ip_selector lhs, const ip_selector rhs);
-bool selector_range_in_selector_range(const ip_selector lhs, const ip_selector rhs);
-bool selector_range_eq_address(const ip_selector selector, const ip_address address);
 bool address_in_selector_range(const ip_address l, const ip_selector r);
 
 #endif
