@@ -34,6 +34,7 @@
 #include "ip_cidr.h"
 #include "lswcdefs.h"
 #include "authby.h"
+#include "end.h"
 
 struct logger;
 
@@ -48,9 +49,14 @@ enum keyword_set {
 	k_default = 2
 };
 
-typedef char *ksf[KW_roof];
-typedef intmax_t knf[KW_roof];
-typedef enum keyword_set keyword_set[KW_roof];
+struct keyword_value {
+	enum keyword_set set;
+	char *string;
+	intmax_t option;
+	deltatime_t deltatime;
+};
+
+typedef struct keyword_value keyword_values[KW_roof];
 
 /*
  * Note: string fields in struct starter_end and struct starter_conn
@@ -65,11 +71,8 @@ struct starter_end {
 	ip_address addr;
 	ip_address nexthop;
 	ip_cidr vti_ip;
-	ip_protoport protoport;
 
-	ksf strings;
-	knf options;
-	keyword_set set;
+	keyword_values values;
 };
 
 /*
@@ -81,16 +84,14 @@ struct starter_conn {
 	TAILQ_ENTRY(starter_conn) link;
 	char *name;
 
-	ksf strings;
-	knf options;
-	keyword_set set;
+	keyword_values values;
 
 	enum ike_version ike_version;
 	struct authby authby;
 	lset_t sighash_policy;
 	enum shunt_policy shunt[SHUNT_KIND_ROOF];
 
-	struct starter_end left, right;
+	struct starter_end end[END_ROOF];
 
 	const struct ip_info *clientaddrfamily;
 
@@ -106,11 +107,7 @@ struct starter_conn {
 };
 
 struct starter_config {
-	struct {
-		ksf strings;
-		knf options;
-		keyword_set set;
-	} setup;
+	keyword_values values;
 
 	/* conn %default */
 	struct starter_conn conn_default;
