@@ -13,6 +13,12 @@
  *
  */
 
-struct logger;
+#ifndef KERNEL_NETLINK_QUERY_H
+#define KERNEL_NETLINK_QUERY_H
 
-int nl_send_query(const struct nlmsghdr *req, int protocol, struct logger *logger);
+struct logger;
+struct nlmsghdr;
+
+int nl_send_query(const struct nlmsghdr *req, int protocol, const struct logger *logger);
+
+#endif
