@@ -22,10 +22,11 @@
 #include <stddef.h>		/* for size_t */
 #include <stdint.h>		/* for uint_fast64_t */
 #include <stdbool.h>
+#include <inttypes.h>		/* for PRI... */
 
 struct enum_names;
 struct jambuf;
-struct sparse_name;
+struct sparse_names;
 
 /*
  * set type with room for at least 64 elements for ALG opts (was 32 in
@@ -33,8 +34,8 @@ struct sparse_name;
  */
 
 typedef uint_fast64_t lset_t;
-#define PRI_LSET "%"PRIxLSET
-#define PRIxLSET    PRIxFAST64
+#define PRI_LSET "%"PRIxFAST64
+
 #define LELEM_ROOF  64	/* all elements must be less than this */
 #define LEMPTY ((lset_t)0)
 #define LELEM(opt) ((lset_t)1 << (opt))
@@ -59,7 +60,7 @@ extern bool test_lset(const struct enum_names *table, lset_t val);
 
 size_t jam_lset(struct jambuf *, const struct enum_names *sd, lset_t val);
 const char *str_lset(const struct enum_names *sd, lset_t val, lset_buf *buf);
-size_t jam_sparse_lset(struct jambuf *, const struct sparse_name *sd, lset_t val);
+size_t jam_sparse_lset(struct jambuf *, const struct sparse_names *sd, lset_t val);
 
 size_t jam_lset_short(struct jambuf *, const struct enum_names *sd,
 		      const char *separator, lset_t val);
