@@ -32,11 +32,8 @@
 #include "where.h"		/* used by macros */
 #include "fd.h"			/* for null_fd */
 #include "impair.h"
-
-#define LOG_WIDTH	((size_t)1024)	/* roof of number of chars in log line */
-
-extern bool log_to_stderr;          /* should log go to stderr? */
-
+#include "pexpect.h"
+#include "fatal.h"
 
 /*
  * Codes for status messages returned to whack.
@@ -50,66 +47,47 @@ extern bool log_to_stderr;          /* should log go to stderr? */
  */
 
 enum rc_type {
-	RC_COMMENT,		/* non-commital utterance with 000 prefix(does not affect exit status) */
-	RC_RAW,			/* ditto, but also suppresses the 000 prefix */
-	RC_LOG,			/* message aimed at log (does not affect exit status) */
-	RC_LOG_SERIOUS,		/* serious message aimed at log (does not affect exit status) */
-	RC_SUCCESS,		/* success (exit status 0) */
-	RC_INFORMATIONAL,	/* should get relayed to user - if there is one */
-	RC_INFORMATIONAL_TRAFFIC, /* status of an established IPSEC (aka Phase 2) state */
 
-	/* failure, but not definitive */
-	RC_RETRANSMISSION = 10,
+	RC_LOG = 2,			/* message aimed at log (does
+					 * not affect exit status) */
 
-	/* improper request */
-	RC_EXIT_FLOOR = 20,
-	RC_DUPNAME = RC_EXIT_FLOOR,	/* attempt to reuse a connection name */
-	RC_UNKNOWN_NAME,	/* connection name unknown or state number */
-	RC_ORIENT,		/* cannot orient connection: neither end is us */
-	RC_CLASH,		/* clash between two Road Warrior connections OVERLOADED */
-	RC_DEAF,		/* need --listen before --initiate */
-	RC_ROUTE,		/* cannot route */
-	RC_RTBUSY,		/* cannot unroute: route busy */
-	RC_BADID,		/* malformed --id */
-	RC_NOKEY,		/* no key found through DNS */
-	RC_NOPEERIP,		/* cannot initiate when peer IP is unknown */
-	RC_INITSHUNT,		/* cannot initiate a shunt-oly connection */
-	RC_WILDCARD,		/* cannot initiate when ID has wildcards */
-	RC_CRLERROR,		/* CRL fetching disabled or obsolete reread cmd */
-	RC_WHACK_PROBLEM,	/* whack-detected problem */
-
-	/* permanent failure */
-	RC_BADWHACKMESSAGE = 30,
-	RC_NORETRANSMISSION,
-	RC_INTERNALERR,
-	RC_OPPOFAILURE,		/* Opportunism failed */
-	RC_CRYPTOFAILED,	/* system too busy to perform required
-				* cryptographic operations */
-	RC_AGGRALGO,		/* multiple algorithms requested in phase 1 aggressive */
-	RC_FATAL,		/* fatal error encountered, and negotiation aborted */
+	RC_SUCCESS = 4,			/* success (exit status 0) */
 
 	/* entry of secrets */
-	RC_ENTERSECRET = 40,
-	RC_USERPROMPT = 41,
+	RC_ENTERSECRET = 10,
+	RC_USERPROMPT = 11,
+
+	RC_EXIT_FLOOR = 20,
+
+	/* improper request */
+	RC_DUPNAME = 20,	/* attempt to reuse a connection name */
+	RC_UNKNOWN_NAME = 21,	/* connection name unknown or state number */
+	RC_ORIENT = 22,		/* cannot orient connection: neither end is us */
+	RC_CLASH = 23,		/* clash between two Road Warrior connections OVERLOADED */
+	RC_DEAF = 24,		/* need --listen before --initiate */
+	RC_ROUTE = 25,		/* cannot route */
+	RC_RTBUSY = 26,		/* cannot unroute: route busy */
+	RC_BADID = 27,		/* malformed --id */
+	RC_NOKEY = 28,		/* no key found through DNS */
+	RC_NOPEERIP = 29,	/* cannot initiate when peer IP is unknown */
+	RC_INITSHUNT = 30,	/* cannot initiate a shunt-oly connection */
+	RC_WILDCARD = 31,	/* cannot initiate when ID has wildcards */
+	RC_CRLERROR = 32,	/* CRL fetching disabled or obsolete reread cmd */
+	RC_WHACK_PROBLEM = 33,	/* whack-detected problem */
+
+	/* permanent failure */
+	RC_BADWHACKMESSAGE = 50,
+	RC_NORETRANSMISSION = 51,
+	RC_INTERNAL_ERROR = 52,
+	RC_OPPOFAILURE = 53,	/* Opportunism failed */
+	RC_CRYPTOFAILED = 54,	/* system too busy to perform required
+				 * cryptographic operations */
+	RC_AGGRALGO = 55,	/* multiple algorithms requested in
+				 * phase 1 aggressive */
+	RC_FATAL = 56,		/* fatal error encountered, and
+				 * negotiation aborted */
 
 	RC_EXIT_ROOF = 100,
-
-	/*
-	 * progress: start of range for successful state transition.
-	 * Actual value is RC_NEW_V[12]_STATE plus the new state code.
-	 */
-	RC_NEW_V1_STATE = RC_EXIT_ROOF,
-	RC_NEW_V2_STATE = 150,
-
-	/*
-	 * Start of range for notification.
-	 *
-	 * Actual value is RC_NOTIFICATION plus code for notification
-	 * that should be generated by this Pluto.  RC_NOTIFICATION.
-	 * Since notifications are two octets, that's 64435+200 which
-	 * overflows the 3-digit prefix, oops.
-	 */
-	RC_NOTIFICATION = 200,	/* as per IKE notification messages */
 };
 
 
@@ -121,39 +99,6 @@ enum rc_type {
 
 struct jambuf;
 
-/*
- * The logging streams used by libreswan.
- *
- * So far three^D^D^D^D^D four^D^D^D^D five^D^D^D^D six^D^D^D
- * seven^D^D^D^D^D five.five streams have been identified; and let's
- * not forget that code writes to STDOUT and STDERR directly.
- *
- * The streams differ in the syslog severity and what PREFIX is
- * assumed to be present and the tool being run.
- *
- *                           PLUTO
- *              SEVERITY  WHACK  PREFIX    TOOLS    PREFIX
- *   default    WARNING    yes    state     -v
- *   log        WARNING     -     state     -v
- *   debug      DEBUG       -     "| "     debug?
- *   error      ERR         -    ERROR     STDERR  PROG:_...
- *   whack         -       yes    NNN      STDOUT  ...
- *   file          -        -      -         -
- *
- * The streams will then add additional prefixes as required.  For
- * instance, the log_whack stream will prefix a timestamp when sending
- * to a file (optional), and will prefix NNN(RC) when sending to
- * whack.
- *
- * For tools, the default and log streams go to STDERR when enabled;
- * and the debug stream goes to STDERR conditional on debug flags.
- * Should the whack stream go to stdout?
- *
- * As needed, return size_t - the number of bytes written - so that
- * implementations have somewhere to send values that should not be
- * ignored; for instance fwrite() :-/
- */
-
 /*
  * By default messages are broadcast (to both log files and whack),
  * mix-in one of these options to limit this.
@@ -162,30 +107,37 @@ struct jambuf;
  * the log files.
  */
 
+#define RC_MASK              0x00fffff	/* rc_type max is 64435+200 */
+#define STREAM_MASK          0x0f00000
+#define LOG_PREFIX_MASK	     0xf000000
+
+enum log_prefix {
+	AUTO_PREFIX =        0x0000000,
+	NO_PREFIX =          0x1000000,
+        ADD_PREFIX =         0x2000000,
+};
+
 enum stream {
-	/* Mask the whack RC; max value is 64435+200 */
-	RC_MASK      = 0x0fffff,
-	STREAM_MASK  = 0xf00000,
-	NO_PREFIX   = 0x1000000,
-	/*                         syslog()                       */
-	/*                         Severity  Whack  Tools  Prefix */
-	ALL_STREAMS  = 0x000000, /* WARNING   yes    err?   <o>   */
-	LOG_STREAM   = 0x100000, /* WARNING   no     err?   <o>   */
-	DEBUG_STREAM = 0x200000, /*  DEBUG    no     err   |<o>   */
-	WHACK_STREAM = 0x300000, /*   N/A     yes    err    <o>   */
-	ERROR_STREAM = 0x400000, /*   ERR     no     err    <o>   */
-	NO_STREAM    = 0xf00000, /*   N/A     N/A                 */
+	/*                                 syslog()                      */
+	/*                                Severity  Whack  Tools  Prefix */
+	ALL_STREAMS        = 0x0000000, /* WARNING   yes    err?   <o>   */
+	LOG_STREAM         = 0x0100000, /* WARNING    no    err?   <o>   */
+	WHACK_STREAM       = 0x0200000, /*   N/A     yes    err    <o>   */
+	DEBUG_STREAM       = 0x0300000, /*  DEBUG     no    err    | <o> */
+	ERROR_STREAM       = 0x0400000, /*   ERR     yes    err    <o>   */
+	PEXPECT_STREAM     = 0x0500000, /*   ERR     yes    err    EXPECTATION FAILED: <o> */
+	PASSERT_STREAM     = 0x0600000, /*   ERR     yes    err    ABORT: ASSERTION_FAILED: <o> */
+	FATAL_STREAM       = 0x0700000, /*   ERR     yes    err    FATAL ERROR: <o> */
+	NO_STREAM          = 0x0f00000, /*   N/A     N/A                 */
 	/*
 	 * <o>: add prefix when object is available
 	 *
-	 * |<o>: add both "| " and prefex when object is available and
+	 * | <o>: add both "| " and prefix when object is available and
          * feature is enabled
 	 *
 	 * err?: write to stderr when enabled (tests log_to_stderr,
-	 * typically via -v).
+	 * typically via -v).  Used by tools such as whack.
 	 */
-#define ERROR_FLAGS (ERROR_STREAM|RC_LOG_SERIOUS)
-#define PRINTF_FLAGS (NO_PREFIX|WHACK_STREAM)
 };
 
 /*
@@ -207,44 +159,42 @@ struct logger_object_vec {
 	const char *name;
 	bool free_object;
 	size_t (*jam_object_prefix)(struct jambuf *buf, const void *object);
-#define jam_logger_prefix(BUF, LOGGER) (LOGGER)->object_vec->jam_object_prefix(BUF, (LOGGER)->object)
-#define jam_logger_prefix_rc(BUF, LOGGER, RC_FLAGS)			\
-	({								\
-		if (((RC_FLAGS) & NO_PREFIX) == LEMPTY &&		\
-		    (((RC_FLAGS) & STREAM_MASK) != DEBUG_STREAM ||	\
-		     DBGP(DBG_ADD_PREFIX))) {				\
-			jam_logger_prefix(BUF, LOGGER);			\
-		}							\
-	})
-	/*
-	 * When opportunistic encryption or the initial responder, for
-	 * instance, some logging is suppressed.
-	 */
-	bool (*suppress_object_log)(const void *object);
-#define suppress_log(LOGGER) (LOGGER)->object_vec->suppress_object_log((LOGGER)->object)
 };
 
-bool suppress_object_log_false(const void *object);
-bool suppress_object_log_true(const void *object);
+/* these omit ": " always */
+typedef struct {
+	char buf[100];/* completely made up size */
+} prefix_buf;
+const char *str_prefix(const struct logger *logger, prefix_buf *buf);
+size_t jam_prefix(struct jambuf *buf, const struct logger *logger);
+
+/* these include ": " when jam_prefix() is non-empty */
+size_t jam_logger_prefix(struct jambuf *buf, const struct logger *logger);
+void jam_logger_rc_prefix(struct jambuf *buf, const struct logger *logger, lset_t rc_flags);
+
 size_t jam_object_prefix_none(struct jambuf *buf, const void *object);
 
 #ifndef GLOBAL_LOGGER
-extern const struct logger global_logger;
-#define GLOBAL_LOGGER
+extern struct logger global_logger;
+#define GLOBAL_LOGGER &global_logger
 #endif
 
 struct logger {
-	struct fd *global_whackfd;
-	struct fd *object_whackfd;
+	/* support up to two whacks */
+	struct fd *whackfd[2];
 	const void *object;
 	const struct logger_object_vec *object_vec;
 	where_t where;
 	/* used by timing to nest its logging output */
 	int timing_level;
+	lset_t debugging;
 };
 
 #define PRI_LOGGER "logger@%p/"PRI_FD"/"PRI_FD
-#define pri_logger(LOGGER) (LOGGER), (LOGGER) == NULL ? NULL : (LOGGER)->global_whackfd, (LOGGER) == NULL ? NULL : (LOGGER)->object_whackfd
+#define pri_logger(LOGGER)						\
+	(LOGGER),							\
+		pri_fd((LOGGER) == NULL ? NULL : (LOGGER)->whackfd[0]), \
+		pri_fd((LOGGER) == NULL ? NULL : (LOGGER)->whackfd[1])
 
 void llog(lset_t rc_flags,
 	  const struct logger *log,
@@ -256,10 +206,15 @@ void llog_va_list(lset_t rc_flags, const struct logger *logger,
 void jambuf_to_logger(struct jambuf *buf, const struct logger *logger, lset_t rc_flags);
 
 #define LLOG_JAMBUF(RC_FLAGS, LOGGER, BUF)				\
-	JAMBUF(BUF)							\
-		for (jam_logger_prefix_rc(BUF, LOGGER, RC_FLAGS);	\
+	/* create the buffer */						\
+	for (struct logjam logjam_, *lbp_ = &logjam_;			\
+	     lbp_ != NULL; lbp_ = NULL)					\
+		/* create the jambuf */					\
+		for (struct jambuf *BUF =				\
+			     jambuf_from_logjam(&logjam_, LOGGER,	\
+						0, NULL, RC_FLAGS);	\
 		     BUF != NULL;					\
-		     jambuf_to_logger(BUF, (LOGGER), RC_FLAGS), BUF = NULL)
+		     logjam_to_logger(&logjam_), BUF = NULL)
 
 void llog_dump(lset_t rc_flags,
 	       const struct logger *log,
@@ -312,7 +267,7 @@ void libreswan_exit(enum pluto_exit_code rc) NEVER_RETURNS;
  *   <log-prefix>ERROR: <message...>...
  */
 
-void log_error(struct logger *logger, int error,
+void log_error(const struct logger *logger, int error,
 	       const char *message, ...) PRINTF_LIKE(3);
 
 #define llog_error(LOGGER, ERRNO, FMT, ...)				\
@@ -321,28 +276,17 @@ void log_error(struct logger *logger, int error,
 		log_error(LOGGER, e_, FMT, ##__VA_ARGS__); \
 	}
 
-/* like log_error() but no ERROR: prefix */
+/* like log_error() but no ERROR: prefix and/or ": " separator */
 void llog_errno(lset_t rc_flags, const struct logger *logger, int error,
 		const char *message, ...) PRINTF_LIKE(4);
 
-/*
- * XXX: The message format is:
- *   FATAL ERROR: <log-prefix><message...>
- * and not:
- *   <log-prefix>FATAL ERROR: <message...>
- */
-void fatal(enum pluto_exit_code rc, const struct logger *logger,
-	   const char *message, ...) PRINTF_LIKE(3) NEVER_RETURNS;
-void fatal_errno(enum pluto_exit_code rc, const struct logger *logger,
-		 int error, const char *message, ...) PRINTF_LIKE(4) NEVER_RETURNS;
-
 /*
  * Log debug messages to the main log stream, but not the WHACK log
  * stream.
  *
  * NOTE: DBG's action can be a { } block, but that block must not
  * contain commas that are outside quotes or parenthesis.
- * If it does, they will be interpreted by the C preprocesser
+ * If it does, they will be interpreted by the C preprocessor
  * as macro argument separators.  This happens accidentally if
  * multiple variables are declared in one declaration.
  *
@@ -352,16 +296,8 @@ void fatal_errno(enum pluto_exit_code rc, const struct logger *logger,
 
 extern lset_t cur_debugging;	/* current debugging level */
 
-#define DEBUG_PREFIX "| "
-
 #define DBGP(cond)	(cur_debugging & (cond))
-
-#define DBGF(COND, MESSAGE, ...)				\
-	{							\
-		if (DBGP(COND)) {				\
-			DBG_log(MESSAGE, ##__VA_ARGS__);	\
-		}						\
-	}
+#define LDBGP(COND, LOGGER) (COND & (cur_debugging | (LOGGER)->debugging))
 
 #define dbg(MESSAGE, ...)					\
 	{							\
@@ -370,32 +306,24 @@ extern lset_t cur_debugging;	/* current debugging level */
 		}						\
 	}
 
-#define ldbg(LOGGER, MESSAGE, ...)			\
-	{						\
-		if (DBGP(DBG_BASE)) {			\
-			llog(DEBUG_STREAM, (LOGGER),	\
-			     MESSAGE, ##__VA_ARGS__);	\
-		}					\
-	}
+void ldbg(const struct logger *logger, const char *message, ...) PRINTF_LIKE(2);
+void pdbg(const struct logger *logger, const char *message, ...) PRINTF_LIKE(2);
 
-#define LDBGP(LOGGER, COND, BUF)					\
-	/* allocate the buffer */					\
-	for (char buf_[LOG_WIDTH],					\
-		     *dbgp_ = (DBGP(COND) ? buf_ : NULL);		\
-	     dbgp_ != NULL;						\
-	     dbgp_ = NULL)						\
-		/* create the jambuf_; no prefix; use jambuf_ */	\
-		for (struct jambuf jambuf_ = ARRAY_AS_JAMBUF(buf_),	\
-			     *BUF = &jambuf_;				\
-		     BUF != NULL;					\
-		     BUF = NULL,					\
-			     jambuf_to_logger(&jambuf_, LOGGER, DEBUG_STREAM))
+void ldbgf(lset_t cond, const struct logger *logger, const char *fmt, ...) PRINTF_LIKE(3);
+void pdbgf(lset_t cond, const struct logger *logger, const char *fmt, ...) PRINTF_LIKE(3);
+
+/* LDBG_JAMBUF() is ambiguous - LDBG_op() or ldbg() ucase? */
+
+#define LDBGP_JAMBUF(COND, LOGGER, BUF)					\
+	for (bool cond_ = LDBGP(COND, LOGGER); cond_; cond_ = false)	\
+		LLOG_JAMBUF(DEBUG_STREAM, LOGGER, BUF)
+#define PDBGP_JAMBUF(COND, LOGGER, BUF)					\
+	for (bool cond_ = LDBGP(COND, LOGGER); cond_; cond_ = false)	\
+		LLOG_JAMBUF(DEBUG_STREAM|ADD_PREFIX, LOGGER, BUF)
 
-#define LDBG(LOGGER, BUF) LDBGP(LOGGER, DBG_BASE, BUF)
 
 /* DBG_*() are unconditional */
 void DBG_log(const char *message, ...) PRINTF_LIKE(1);
-void DBG_va_list(const char *message, va_list ap) VPRINTF_LIKE(1);
 void DBG_dump(const char *label, const void *p, size_t len);
 #define DBG_dump_hunk(LABEL, HUNK)					\
 	{								\
@@ -404,6 +332,46 @@ void DBG_dump(const char *label, const void *p, size_t len);
 	}
 #define DBG_dump_thing(LABEL, THING) DBG_dump(LABEL, &(THING), sizeof(THING))
 
+/*
+ * XXX: unlike dbg_dump() et.al., these don't take a prefix; instead
+ * caller should log that separately.
+ */
+#define LDBG_dump(LOGGER, DATA, LEN)				\
+	{							\
+		llog_dump(DEBUG_STREAM, LOGGER, DATA, LEN);	\
+	}
+#define LDBG_hunk(LOGGER, HUNK)						\
+	{								\
+		const typeof(HUNK) *hunk_ = &(HUNK); /* evaluate once */ \
+		llog_dump(DEBUG_STREAM, LOGGER, hunk_->ptr, hunk_->len); \
+	}
+#define LDBG_thing(LOGGER, THING)					\
+	{								\
+		llog_dump(DEBUG_STREAM, LOGGER, &(THING), sizeof(THING)); \
+	}
+#define ldbg_dump(LOGGER, DATA, LEN)			\
+	{						\
+		if (DBGP(DBG_BASE)) {			\
+			LDBG_dump(LOGGER, DATA, LEN);	\
+		}					\
+	}
+#define ldbg_hunk(LOGGER, HUNK)					\
+	{							\
+		if (DBGP(DBG_BASE)) {				\
+			LDBG_hunk(LOGGER, HUNK);		\
+		}						\
+	}
+#define ldbg_thing(LOGGER, THING)			\
+	{						\
+		if (DBGP(DBG_BASE)) {			\
+			LDBG_thing(LOGGER, THING);	\
+		}					\
+	}
+
+/* LDBG_*(logger, ...) are unconditional wrappers */
+#define LDBG_log(LOGGER, FMT, ...) llog(DEBUG_STREAM, LOGGER, FMT, ##__VA_ARGS__)
+#define LDBG_va_list(LOGGER, FMT, AP) llog_va_list(DEBUG_STREAM, LOGGER, FMT, AP)
+
 /*
  * Code wrappers that cover up the details of allocating,
  * initializing, de-allocating (and possibly logging) a 'struct
@@ -452,47 +420,21 @@ void lswbuf(struct jambuf *log)
 #endif
 
 /*
- * Log an expectation failure message to the error streams.  That is
- * the main log (level LOG_ERR) and whack log (level RC_LOG_SERIOUS).
- *
- * When evaluating ASSERTION, do not wrap it in parentheses as it will
- * suppress the warning for 'foo = bar'.
- *
- * Because static analyzer tools are easily confused, explicitly
- * return the assertion result.
+ * For a switch statements
  */
 
-#ifdef EXAMPLE
-void lswlog_pexpect_example(void *p)
-{
-	if (!pexpect(p != NULL)) {
-		return;
-	}
-}
-#endif
+void bad_case_where(const char *expression, long value, where_t where) NEVER_RETURNS;
+#define bad_case(N) bad_case_where(#N, (N), HERE)
 
-extern void llog_pexpect(const struct logger *logger, where_t where,
-			 const char *message, ...) PRINTF_LIKE(3);
+void bad_enum_where(const struct logger *logger,
+		    const struct enum_names *en,
+		    unsigned long val, where_t where) NEVER_RETURNS;
+#define bad_enum(LOGGER, ENUM_NAMES, VALUE) bad_enum_where(LOGGER, ENUM_NAMES, VALUE, HERE)
 
-#define PEXPECT(LOGGER, ASSERTION)					\
-	({								\
-		/* wrapping ASSERTION in parens suppresses -Wparen */	\
-		bool assertion__ = ASSERTION; /* no parens */		\
-		if (!assertion__) {					\
-			where_t here_ = HERE;				\
-			const struct logger *logger_ = LOGGER;		\
-			llog_pexpect(logger_, here_, "%s", #ASSERTION);	\
-		}							\
-		assertion__; /* result */				\
-	})
-
-#define pexpect(ASSERTION)  PEXPECT(&global_logger, ASSERTION)
-
-/* for a switch statement */
-
-void libreswan_bad_case(const char *expression, long value, where_t where) NEVER_RETURNS;
-
-#define bad_case(N)	libreswan_bad_case(#N, (N), HERE)
+void bad_sparse_where(const struct logger *logger,
+		      const struct sparse_names *sn,
+		      unsigned long val, where_t where) NEVER_RETURNS;
+#define bad_sparse(LOGGER, SPARSE_NAMES, VALUE) bad_sparse_where(LOGGER, SPARSE_NAMES, VALUE, HERE)
 
 #define impaired_passert(BEHAVIOUR, LOGGER, ASSERTION)			\
 	{								\
