@@ -26,6 +26,8 @@
 #include "err.h"		/* for err_t */
 #include "lswcdefs.h"		/* for NEVER_RETURNS PRINTF_LIKE() */
 #include "where.h"
+#include "lset.h"
+#include "logjam.h"
 
 struct logger;
 
@@ -36,26 +38,49 @@ struct logger;
  */
 
 #ifndef GLOBAL_LOGGER
-extern const struct logger global_logger;
-#define GLOBAL_LOGGER
+extern struct logger global_logger;
+#define GLOBAL_LOGGER &global_logger
 #endif
 
+/*
+ * XXX: GCC, at least can't understand that LLOG_PASSERT_JAMBUF()
+ * never returns.  Hence code sometimes calls
+ * passert_jambuf_to_logger() explicitly.
+ */
+
 extern void llog_passert(const struct logger *logger, where_t where,
 			 const char *message, ...) NEVER_RETURNS PRINTF_LIKE(3);
 
-#define PASSERT(LOGGER, ASSERTION)					\
+void passert_logjam_to_logger(struct logjam *buf) NEVER_RETURNS;
+
+#define LLOG_PASSERT_JAMBUF(LOGGER, WHERE, BUF)				\
+	/* create the buffer */						\
+	for (struct logjam logjam_, *bf_ = &logjam_;			\
+	     bf_ != NULL; bf_ = NULL)					\
+		/* create the jambuf */					\
+		for (struct jambuf *BUF =				\
+			     jambuf_from_logjam(&logjam_, LOGGER,	\
+						 0, WHERE, PASSERT_FLAGS); \
+		     BUF != NULL;					\
+		     passert_logjam_to_logger(&logjam_), BUF = NULL)
+
+#define PASSERT_WHERE(LOGGER, WHERE, ASSERTION)				\
 	({								\
 		/* wrapping ASSERTION in parens suppresses -Wparen */	\
 		bool assertion__ = ASSERTION; /* no parens */		\
 		if (!assertion__) {					\
-			where_t here = HERE;				\
 			const struct logger *logger_ = LOGGER;		\
-			llog_passert(logger_, here, "%s", #ASSERTION);	\
+			llog_passert(logger_, WHERE, "%s", #ASSERTION);	\
 		}							\
-		(void) true;							\
+		/* return something so flipping to pexpect() is easy */	\
+		(void) true;						\
 	})
 
-#define passert(ASSERTION)     PASSERT(&global_logger, ASSERTION)
+#define PASSERT(LOGGER, ASSERTION)		\
+	PASSERT_WHERE(LOGGER, HERE, ASSERTION)
+
+#define passert(ASSERTION)				\
+	PASSERT_WHERE(&global_logger, HERE, ASSERTION)
 
 /* evaluate x exactly once; assert that err_t result is NULL; */
 #define happy(x) /* TBD: use ??? */				\
