@@ -11,6 +11,7 @@
  * Copyright (C) 2017 Vukasin Karadzic <vukasin.karadzic@gmail.com>
  * Copyright (C) 2019-2019 Andrew Cagney <cagney@gnu.org>
  * Copyright (C) 2020 Yulia Kuzovkova <ukuzovkova@gmail.com>
+ * Copyright (C) 2020 Nupur Agrawal <nupur202000@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -74,7 +75,6 @@ enum ike_version {
 #define IPSEC_SA_MAX_OPERATIONS_STRING "2^63"		/* how to print IPSEC_SA_MAX_OPERATIONS */
 #define IPSEC_SA_MAX_SOFT_LIMIT_PERCENTAGE 50
 
-#define PLUTO_SHUNT_LIFE_DURATION_DEFAULT (15 * secs_per_minute)
 #define PLUTO_HALFOPEN_SA_LIFE (secs_per_minute )
 
 #define SA_REPLACEMENT_MARGIN_DEFAULT (9 * secs_per_minute) /* IPSEC & IKE */
@@ -538,6 +538,11 @@ enum state_kind {
 	/* Xauth states */
 	STATE_XAUTH_R0,         /* server state has sent request, awaiting reply */
 	STATE_XAUTH_R1,         /* server state has sent success/fail, awaiting reply */
+	STATE_MODE_CFG_SERVER_WAITING_FOR_ACK,	/* server initiated
+						 * SET, waiting for
+						 * ACK */
+	STATE_MODE_CFG_CLIENT_RESPONDING,	/* non-pullclient got
+						 * a likely request */
 	STATE_MODE_CFG_R0,      /* these states are used on the responder */
 	STATE_MODE_CFG_R1,
 	STATE_MODE_CFG_R2,
@@ -558,14 +563,15 @@ enum state_kind {
 	 * Note: order matters.  Larval states come before
 	 * ESTABLISHED_*_SA.
 	 */
-	STATE_IKEv2_FLOOR,
 
 	/* IKE SA */
 
-	STATE_V2_IKE_SA_INIT_I0 = STATE_IKEv2_FLOOR,	/* waiting for KE to finish */
+#define STATE_IKEv2_FLOOR (STATE_V2_UNSECURED_R)
 
+	STATE_V2_UNSECURED_R,
+
+	STATE_V2_IKE_SA_INIT_I0,	/* waiting for KE to finish */
 	STATE_V2_IKE_SA_INIT_I,		/* Initiator sent Request */
-	STATE_V2_IKE_SA_INIT_R0,	/* just starting */
 	STATE_V2_IKE_SA_INIT_R,		/* Responder send Response */
 	STATE_V2_IKE_SA_INIT_IR,	/* Initiator processed Response */
 
@@ -573,6 +579,12 @@ enum state_kind {
 	STATE_V2_IKE_INTERMEDIATE_R,
 	STATE_V2_IKE_INTERMEDIATE_IR,
 
+	STATE_V2_IKE_SESSION_RESUME_I0,
+	STATE_V2_IKE_SESSION_RESUME_I,
+	STATE_V2_IKE_SESSION_RESUME_R0,
+	STATE_V2_IKE_SESSION_RESUME_R,
+	STATE_V2_IKE_SESSION_RESUME_IR,
+
 	STATE_V2_IKE_AUTH_EAP_R,  /* IKE_AUTH EAP negotiation */
 
 	STATE_V2_IKE_AUTH_I,        /* IKE_AUTH: sent auth message, waiting for reply */
@@ -636,7 +648,8 @@ extern const struct enum_names perspective_names;
 				     LELEM(STATE_AGGR_I2) | \
 				     LELEM(STATE_XAUTH_I0) |	\
 				     LELEM(STATE_XAUTH_I1) |	\
-				     LELEM(STATE_MODE_CFG_I1))
+				     LELEM(STATE_MODE_CFG_I1) | \
+				     LELEM(STATE_MODE_CFG_CLIENT_RESPONDING))
 
 #define IS_V1_PHASE1(ST) (STATE_MAIN_R0 <= (ST) && (ST) <= STATE_AGGR_R2)
 
@@ -663,10 +676,12 @@ extern const struct enum_names perspective_names;
 					  LELEM(STATE_AGGR_R2) | \
 					  LELEM(STATE_XAUTH_R0) |	\
 					  LELEM(STATE_XAUTH_R1) |	\
+					  LELEM(STATE_MODE_CFG_SERVER_WAITING_FOR_ACK) |	\
 					  LELEM(STATE_MODE_CFG_R0) |	\
 					  LELEM(STATE_MODE_CFG_R1) |	\
 					  LELEM(STATE_MODE_CFG_R2) |	\
 					  LELEM(STATE_MODE_CFG_I1) |	\
+					  LELEM(STATE_MODE_CFG_CLIENT_RESPONDING) |	\
 					  LELEM(STATE_XAUTH_I0) |	\
 					  LELEM(STATE_XAUTH_I1))
 
