@@ -29,13 +29,17 @@
 #include "constants.h"
 #include "ike_alg.h"
 #include "shunk.h"
+#include "diag.h"
+#include "fips_mode.h"
 
+struct jambuf;
 struct alg_info;
 struct proposal_protocol;
 struct proposal;
 struct proposals;
 struct proposal_policy;
 struct proposal_parser;
+enum stream;
 
 /*
  * XXX: needs to be merged with IKE_ALG_TYPE.
@@ -55,8 +59,7 @@ struct proposal_parser {
 	const struct proposal_protocol *protocol;
 	const struct proposal_param *param;
 	const struct proposal_policy *policy;
-	/* need to eliminate hardwired size */
-	char error[200];
+	diag_t diag;
 };
 
 /*
@@ -65,7 +68,6 @@ struct proposal_parser {
 
 struct proposal_policy {
 	enum ike_version version;
-	unsigned parser_version;
 	bool pfs; /* For CHILD SA, use DH from IKE SA */
 	bool check_pfs_vs_dh;
 	bool ignore_parser_errors;
@@ -78,9 +80,10 @@ struct proposal_policy {
 	 */
 	bool (*alg_is_ok)(const struct ike_alg *alg);
 	/*
-	 * Print a warning.  Signature needs to match libreswan_log.
+	 * logging context
 	 */
-	int (*warning)(const char *fmt, ...) PRINTF_LIKE(1);
+	struct logger *logger;
+	lset_t logger_rc_flags;
 };
 
 /*
@@ -91,8 +94,10 @@ struct proposal_defaults {
 	/*
 	 * Proposals to parse when the parser is called with a NULL
 	 * proposals string.
+	 *
+	 * Code needs FIPS and non-FIPS variants.
 	 */
-	const char *proposals;
+	const char *proposals[FIPS_MODE_ROOF];
 	/*
 	 * Algorithms to add to the proposal when they were not
 	 * specified by the proposal string.
@@ -113,12 +118,8 @@ typedef const struct ike_alg *(alg_byname_fn)(struct proposal_parser *parser,
 
 struct proposal_protocol {
 	const char *name;
-	enum ike_alg_key ikev1_alg_id;
-
-	/*
-	 * Lists of defaults for both IKE version.
-	 */
-	const struct proposal_defaults *defaults[IKE_VERSION_ROOF];
+	enum ike_alg_key alg_id;
+	const struct proposal_defaults *defaults;
 
 	/*
 	 * Is the proposal OK?
@@ -157,19 +158,21 @@ bool proposal_encrypt_aead(const struct proposal *proposal);
 bool proposal_encrypt_norm(const struct proposal *proposal);
 bool proposal_integ_none(const struct proposal *proposal);
 
-unsigned nr_proposals(struct proposals *proposals);
-bool default_proposals(struct proposals *proposals);
+unsigned nr_proposals(const struct proposals *proposals);
+bool default_proposals(const struct proposals *proposals);
 
-extern void proposals_addref(struct proposals **proposals);
-extern void proposals_delref(struct proposals **proposals);
+void free_proposals(struct proposals **proposals);
 
-extern struct proposal *alloc_proposal(struct proposal_parser *parser);
+extern struct proposal *alloc_proposal(const struct proposal_parser *parser);
 extern void free_proposal(struct proposal **proposal);
 
 void free_algorithms(struct proposal *proposal, enum proposal_algorithm algorithm);
 void append_proposal(struct proposals *proposals, struct proposal **proposal);
 void append_algorithm(struct proposal_parser *parser, struct proposal *proposal,
 		      const struct ike_alg *alg, int enckeylen);
+void remove_duplicate_algorithms(struct proposal_parser *parser,
+				 struct proposal *proposal,
+				 enum proposal_algorithm algorithm);
 
 struct proposal_parser *alloc_proposal_parser(const struct proposal_policy *policy,
 					      const struct proposal_protocol *protocol);
@@ -189,9 +192,9 @@ struct child_proposals {
 	struct proposals *p;
 };
 
-void fmt_proposal(struct lswlog *log,
+void jam_proposal(struct jambuf *log,
 		  const struct proposal *proposal);
-void fmt_proposals(struct lswlog *log, const struct proposals *proposals);
+void jam_proposals(struct jambuf *log, const struct proposals *proposals);
 
 /*
  * Iterate through all the proposals and the proposal's algorithms.
@@ -268,4 +271,26 @@ struct v1_proposal {
 
 struct v1_proposal v1_proposal(const struct proposal *proposal);
 
+/*
+ * INTERNAL: tokenize <input> into <delim_before><current><delim_after><input>
+ */
+
+struct proposal_tokenizer {
+	char prev_term;
+	shunk_t this;
+	char this_term;
+	shunk_t next;
+	char next_term;
+	shunk_t input;
+	const char *delims;
+};
+
+struct proposal_tokenizer proposal_first_token(shunk_t input, const char *delim);
+void proposal_next_token(struct proposal_tokenizer *token);
+
+bool proposal_parse_encrypt(struct proposal_parser *parser,
+			    struct proposal_tokenizer *tokens,
+			    const struct ike_alg **encrypt,
+			    int *enckeylen);
+
 #endif /* PROPOSALS_H */
