@@ -24,6 +24,7 @@
 
 #include <pk11pub.h>
 
+#include "refcnt.h"
 #include "lswcdefs.h"
 #include "x509.h"
 #include "id.h"
@@ -73,32 +74,48 @@ enum secret_kind {
 	SECRET_INVALID,
 };
 
+/*
+ * Pre-shared Secrets, and XAUTH stuff.
+ */
+
+struct secret_preshared_stuff {
+	size_t len;
+	uint8_t ptr[];
+};
+
+const struct secret_preshared_stuff *secret_preshared_stuff(const struct secret *);
+
+/*
+ * PKI or raw public/private keys.
+ */
+
 struct secret_pubkey_stuff {
+	struct refcnt refcnt;
 	SECKEYPrivateKey *private_key;
 	struct pubkey_content content;
 };
 
-struct secret_stuff {
-	enum secret_kind kind;
-	/*
-	 * This replaced "int lsw_secretlineno()", which assumes only
-	 * one file (no includes) and isn't applicable to NSS.  For
-	 * NSS it's the entry number.
-	 */
-	int line;
-	union {
-		chunk_t preshared_secret;
-		struct secret_pubkey_stuff pubkey;
-	} u;
+struct secret_pubkey_stuff *secret_pubkey_stuff(const struct secret *s);
 
-	chunk_t ppk;
-	chunk_t ppk_id;
+struct secret_pubkey_stuff *secret_pubkey_stuff_addref(struct secret_pubkey_stuff *, where_t where);
+void secret_pubkey_stuff_delref(struct secret_pubkey_stuff **, where_t where);
+
+/*
+ * PPK
+ */
+
+struct secret_ppk_stuff {
+	shunk_t key;
+	chunk_t id;
+	uint8_t data[];
 };
 
-diag_t secret_pubkey_stuff_to_pubkey_der(struct secret_stuff *pks, chunk_t *der);
+const struct secret_ppk_stuff *secret_ppk_stuff(const struct secret *s);
+const struct secret_ppk_stuff *secret_ppk_stuff_by_id(const struct secret *secrets, shunk_t ppk_id);
+
+diag_t secret_pubkey_stuff_to_pubkey_der(struct secret_pubkey_stuff *pks, chunk_t *der);
 diag_t pubkey_der_to_pubkey_content(shunk_t pubkey_der, struct pubkey_content *pkc);
 
-extern struct secret_stuff *get_secret_stuff(struct secret *s);
 extern struct id_list *lsw_get_idlist(const struct secret *s);
 
 /*
@@ -106,12 +123,16 @@ extern struct id_list *lsw_get_idlist(const struct secret *s);
  * return 0 to return current secret
  * return -1 to return NULL
  */
+
+struct secret_context;
 typedef int (*secret_eval)(struct secret *secret,
-			   struct secret_stuff *pks,
-			   void *uservoid);
+			   enum secret_kind kind,
+			   unsigned line,
+			   struct secret_context *context);
 
 struct secret *foreach_secret(struct secret *secrets,
-			      secret_eval func, void *uservoid);
+			      secret_eval func,
+			      struct secret_context *context);
 
 struct hash_signature {
 	size_t len;
@@ -153,7 +174,7 @@ struct pubkey_signer {
 	const char *name;
 	enum digital_signature_blob digital_signature_blob;
 	const struct pubkey_type *type;
-	struct hash_signature (*sign_hash)(const struct secret_stuff *pks,
+	struct hash_signature (*sign_hash)(const struct secret_pubkey_stuff *pks,
 					   const uint8_t *hash_octets, size_t hash_len,
 					   const struct hash_desc *hash_algo,
 					   struct logger *logger);
@@ -254,7 +275,7 @@ struct pubkey *pubkey_addref_where(struct pubkey *pk, where_t where);
 extern void pubkey_delref_where(struct pubkey **pkp, where_t where);
 #define pubkey_delref(PKP) pubkey_delref_where(PKP, HERE)
 
-bool secret_pubkey_same(struct secret *lhs, struct secret *rhs);
+bool secret_pubkey_same(const struct secret *lhs, const struct secret *rhs);
 
 extern void lsw_load_preshared_secrets(struct secret **psecrets, const char *secrets_file,
 				       struct logger *logger);
@@ -266,14 +287,12 @@ extern struct secret *lsw_find_secret_by_id(struct secret *secrets,
 					    const struct id *his_id,
 					    bool asym);
 
-extern struct secret *lsw_get_ppk_by_id(struct secret *secrets, chunk_t ppk_id);
-
 /* err_t!=NULL -> neither found nor loaded; loaded->just pulled in */
 err_t find_or_load_private_key_by_cert(struct secret **secrets, const struct cert *cert,
-				       const struct secret_stuff **pks, bool *load_needed,
+				       struct secret_pubkey_stuff **pks, bool *load_needed,
 				       struct logger *logger);
 err_t find_or_load_private_key_by_ckaid(struct secret **secrets, const ckaid_t *ckaid,
-					const struct secret_stuff **pks, bool *load_needed,
+					struct secret_pubkey_stuff **pks, bool *load_needed,
 					struct logger *logger);
 
 diag_t create_pubkey_from_cert(const struct id *id,
