@@ -148,6 +148,8 @@ shunk_t shunk_span(shunk_t *input, const char *accept);
  * value.
  */
 err_t shunk_to_uintmax(shunk_t input, shunk_t *cursor, unsigned base, uintmax_t *value);
+err_t shunk_to_decimal(shunk_t input, shunk_t *cursor, uintmax_t *decimal,
+		       uintmax_t *numerator, uintmax_t *denominator);
 
 /*
  * To print, use: printf(PRI_SHUNK, pri_shunk(shunk));
