@@ -26,24 +26,16 @@
 #include "chunk.h"
 #include "shunk.h"
 
-/*
- * new IPv6-compatible functions
- */
-
 /* text conversions */
 
-extern err_t ttoulb(const char *src, size_t srclen, int format,
-		    unsigned long upb, unsigned long *dst);
-
-extern size_t ultot(unsigned long src, int format, char *buf, size_t buflen);
 #define ULTOT_BUF	((64+2)/3 + 1)  /* holds 64 bits in octal + NUL */
 
-extern err_t ttodata(const char *src, size_t srclen, int base, char *buf,
-	      size_t buflen, size_t *needed);
+extern err_t ttodata(const char *src, size_t srclen, int base,
+		     void *buf, size_t buflen, size_t *needed);
 
 extern err_t ttochunk(shunk_t src, int base, chunk_t *chunk);
 
-extern size_t datatot(const unsigned char *src, size_t srclen, int format,
-	       char *buf, size_t buflen);
+extern size_t datatot(const void *src, size_t srclen, int format,
+		      char *buf, size_t buflen);
 
 #endif
