@@ -8,6 +8,7 @@
  * Copyright (C) 2013,2016 Antony Antony <antony@phenome.org>
  * Copyright (C) 2016,2018 Andrew Cagney
  * Copyright (C) 2017 Mayank Totale <mtotale@gmail.com>
+ * Copyright (C) 2020 Nupur Agrawal <nupur202000@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -41,6 +42,7 @@
 #include "encap_proto.h"
 #include "sa_type.h"
 #include "constants.h"		/* for enum keyword_auth; et.al. */
+#include "end.h"
 
 #ifndef DEFAULT_CTL_SOCKET
 # define DEFAULT_CTL_SOCKET IPSEC_RUNDIR "/pluto.ctl"
@@ -77,7 +79,7 @@ struct whack_end {
 	enum keyword_host host_type;
 	ip_address host_addr;
 	unsigned host_ikeport;
-	ip_address host_nexthop;
+	ip_address nexthop;		/* host */
 	char *sourceip;
 	ip_cidr host_vtiip;
 	char *interface_ip;		/* for ipsec-interface */
@@ -93,14 +95,15 @@ struct whack_end {
 	bool xauth_server;	/* for XAUTH */
 	bool xauth_client;
 	char *xauth_username;
-	bool modecfg_server;	/* for MODECFG */
-	bool modecfg_client;
-	bool cat;		/* IPv4 Client Address Translation */
+	enum yn_options cat;		/* IPv4 Client Address Translation */
 	enum certpolicy sendcert;
 	enum eap_options eap;
 	bool send_ca;
 	enum ike_cert_type certtype;
 
+	enum yn_options modecfgserver;	/* for MODECFG */
+	enum yn_options modecfgclient;
+
 	char *host_addr_name;	/* DNS name for host, of hosttype==IPHOSTNAME
 				 * pluto will convert to IP address again,
 				 * if this is non-NULL when conn fails.
@@ -212,8 +215,9 @@ struct whack_message {
 	enum yn_options ignore_peer_dns;	/* install obtained
 						 * DNS servers
 						 * locally */
-	enum yn_options ikepad;		/* pad ike packets to 4 bytes
-					 * or not */
+	enum yna_options ikepad;	/* pad ike packets and
+					 * payloads to 4 bytes or
+					 * not */
 	enum yn_options require_id_on_certificate;
 					/* require certificates to
 					 * have IKE ID on cert SAN */
@@ -231,6 +235,9 @@ struct whack_message {
 	enum yn_options compress;
 	enum type_options type;		/* type=tunnel|transport|SHUNT */
 	enum encap_proto phase2;	/* outer protocol: ESP|AH */
+	enum yn_options session_resumption;	/* for RFC 5723 -
+						 * IKEv2 Session
+						 * Resumption */
 
 	uintmax_t sa_ipsec_max_bytes;
 	uintmax_t sa_ipsec_max_packets;
@@ -238,7 +245,7 @@ struct whack_message {
 	deltatime_t ikelifetime;
 	deltatime_t ipsec_lifetime;
 
-	deltatime_t sa_rekey_margin;
+	deltatime_t rekeymargin; /* which SA? */
 	uintmax_t sa_rekeyfuzz_percent;
 
 	struct {
@@ -310,16 +317,21 @@ struct whack_message {
 	uintmax_t priority;
 	uintmax_t tfc;
 	bool send_no_esp_tfc;
+
+	enum yn_options iptfs;
+	enum yn_options iptfs_fragmentation;
+	uintmax_t iptfs_packet_size; /* 0 for PMTU */
+	uintmax_t iptfs_max_queue_size;
+	uintmax_t iptfs_reorder_window;
+	deltatime_t iptfs_drop_time;
+	deltatime_t iptfs_init_delay;
+
 	reqid_t sa_reqid;
 	int nflog_group;
 
 	char *sec_label;	/* sec_label string (if any) -- decoded by pluto */
 
-	/*  note that each end contains string 2/5.id, string 3/6 cert,
-	 *  and string 4/7 updown
-	 */
-	struct whack_end left;
-	struct whack_end right;
+	struct whack_end end[END_ROOF];
 
 	/* names match field */
 	const struct ip_info *host_afi;
@@ -439,7 +451,7 @@ struct whack_message {
 	enum yn_options vti_shared;	/* use remote %any and skip
 					 * cleanup on down? */
 
-	/* RFC 8784 and draft-smyslov-ipsecme-ikev2-qr-alt-07 */
+	/* RFC 8784 and draft-ietf-ipsecme-ikev2-qr-alt-04 */
 	char *ppk_ids;
 
 	/* for RFC 5685 - IKEv2 Redirect mechanism */
@@ -450,6 +462,9 @@ struct whack_message {
 	char *accept_redirect_to;
 	enum yna_options send_redirect;
 
+	/* for RFC 5723 - IKEv2 Session Resumption */
+	bool whack_suspend;
+
 	/* what metric to put on ipsec routes */
 	int metric;
 
