@@ -27,12 +27,14 @@
 
 #include <cert.h>		/* dep:libnss3-dev */
 
+#include "verbose.h"
 #include "deltatime.h"
 #include "chunk.h"
 #include "err.h"
 #include "constants.h"
 #include "jambuf.h"		/* for typedef jam_bytes_fn */
 #include "asn1.h"
+#include "verbose.h"
 
 struct logger;
 struct pubkey_list;
@@ -77,7 +79,8 @@ extern deltatime_t crl_check_interval;
 
 extern bool same_dn(asn1_t a, asn1_t b);
 extern bool match_dn(asn1_t a, asn1_t b, int *wildcards);
-extern bool match_dn_any_order_wild(const char *prefix, asn1_t a, asn1_t b, int *wildcards);
+extern bool match_dn_any_order_wild(asn1_t a, asn1_t b, int *wildcards,
+				    struct verbose verbose);
 extern bool dn_has_wildcards(asn1_t dn);
 extern err_t atodn(const char *src, chunk_t *dn);
 extern void free_generalNames(generalName_t *gn, bool free_name);
@@ -91,12 +94,12 @@ extern void clear_ocsp_cache(void);
  */
 extern SECItem same_shunk_as_dercert_secitem(shunk_t chunk);
 extern chunk_t get_dercert_from_nss_cert(CERTCertificate *cert);
-extern void select_nss_cert_id(CERTCertificate *cert, struct id *end_id);
 extern bool add_pubkey_from_nss_cert(struct pubkey_list **pubkey_db,
 				     const struct id *keyid,
 				     CERTCertificate *cert,
 				     struct logger *logger);
-extern bool trusted_ca(asn1_t a, asn1_t b, int *pathlen);
+extern bool trusted_ca(asn1_t a, asn1_t b, int *pathlen,
+		       struct verbose verbose);
 extern CERTCertList *get_all_certificates(struct logger *logger);
 
 /*
