@@ -28,11 +28,10 @@
 #include <sys/stat.h>
 #include <sys/types.h>
 
-#include <libreswan.h>
-
 #include "constants.h"
 #include "lswtool.h"
 #include "lswlog.h"
+#include "lswalloc.h"
 
 bool log_to_stderr = true;	/* should log go to stderr? */
 
@@ -42,20 +41,14 @@ static size_t jam_progname_prefix(struct jambuf *buf, const void *object UNUSED)
 {
 	const char *progname = object;
 	if (progname != NULL) {
-		return jam(buf, "%s: ", progname);
+		return jam(buf, "%s", progname);
 	}
 	return 0;
 }
 
-static bool suppress_progname_log(const void *object UNUSED)
-{
-	return false;
-}
-
 const struct logger_object_vec progname_object_vec = {
 	.name = "tool",
 	.jam_object_prefix = jam_progname_prefix,
-	.suppress_object_log = suppress_progname_log,
 };
 
 static struct logger progname_logger = {
@@ -63,54 +56,56 @@ static struct logger progname_logger = {
 	.object = NULL, /* progname */
 };
 
-struct logger *tool_init_log(const char *name)
+struct logger *tool_logger(int argc UNUSED, char *argv[])
 {
-	const char *last_slash = strrchr(name, '/');
-	progname_logger.object = progname = last_slash == NULL ? name : last_slash + 1;
+	const char *last_slash = strrchr(argv[0], '/');
+	const char *last_name = (last_slash == NULL ? argv[0] : last_slash + 1);
+
+	/* need to allocate string then mark as never free */
+	ssize_t name_size = sizeof("ipsec "/*includes \0*/) + strlen(last_name) + 1/*to be sure*/;
+	char *name = alloc_things(char, name_size, "(ignore)progname");
+	/* snprintf() returns length, not size */
+	passert(name_size > snprintf(name, name_size, "ipsec %s", last_name));
+
+	progname_logger.object = progname = name;
+
+	/* redundant? */
+	setbuf(stderr, NULL);
 
-	if (log_to_stderr) {
-		setbuf(stderr, NULL);
-	}
 	return &progname_logger;
 }
 
 void jambuf_to_logger(struct jambuf *buf, const struct logger *logger UNUSED, lset_t rc_flags)
 {
-	enum stream only = rc_flags & STREAM_MASK;
-	switch (only) {
-	case DEBUG_STREAM:
-		jambuf_to_debug_stream(buf);
-		break;
+	enum stream stream = (rc_flags & STREAM_MASK);
+	switch (stream) {
 	case ALL_STREAMS:
 	case LOG_STREAM:
 		if (log_to_stderr) {
 			fprintf(stderr, "%s\n", buf->array);
 		}
-		break;
+		return;
 	case WHACK_STREAM:
-		fprintf(stderr, "%s\n", buf->array);
-		break;
+		/* AKA the console */
+		fprintf(stdout, "%s\n", buf->array);
+		return;
+	case FATAL_STREAM:
+	case DEBUG_STREAM:
 	case ERROR_STREAM:
-		jambuf_to_error_stream(buf);
-		break;
+	case PEXPECT_STREAM:
+		fprintf(stderr, "%s\n", buf->array);
+		return;
+	case PASSERT_STREAM:
+		fprintf(stderr, "%s\n", buf->array);
+		return; /*abort();?*/
 	case NO_STREAM:
 		/*
 		 * XXX: Like writing to /dev/null - go through the
 		 * motions but with no result.  Code really really
 		 * should not call this function with this flag.
 		 */
-		break;
-	default:
-		bad_case(only);
+		return;
 	}
-}
-
-void jambuf_to_debug_stream(struct jambuf *buf)
-{
-	fprintf(stderr, "%s%s\n", DEBUG_PREFIX, buf->array);
-}
-
-void jambuf_to_error_stream(struct jambuf *buf)
-{
-	fprintf(stderr, "%s\n", buf->array);
+	fprintf(stderr, "bad stream %d", stream);
+	abort(); /* not bad_case(stream) as recursive */
 }
