@@ -24,7 +24,7 @@
 #include "lswlog.h"
 #include "proposals.h"
 #include "alg_byname.h"
-#include "lswfips.h"
+#include "fips_mode.h"
 
 #include "ike_alg.h"
 #include "ike_alg_integ.h"
@@ -60,7 +60,7 @@ static bool ah_proposal_ok(struct proposal_parser *parser,
  * IKEv1:
  */
 
-static const char default_v1_ah_proposals[] =
+static const char default_ikev1_ah_proposals[] =
 	"SHA1_96" /*???*/
 	","
 	"SHA2_512"
@@ -68,27 +68,22 @@ static const char default_v1_ah_proposals[] =
 	"SHA2_256"
 	;
 
-const struct proposal_defaults v1_ah_defaults = {
-	.proposals = default_v1_ah_proposals,
+const struct proposal_defaults ikev1_ah_defaults = {
+	.proposals[FIPS_MODE_ON] = default_ikev1_ah_proposals,
+	.proposals[FIPS_MODE_OFF] = default_ikev1_ah_proposals,
 };
 
 /*
  * IKEv2:
  */
 
-static const char default_v2_ah_proposals[] =
+static const char default_ikev2_ah_proposals[] =
 	"SHA2_512_256"
 	","
 	"SHA2_256_128"
-	","
-	/*
-	 * something strongswan might accept; bottom of the preference
-	 * list
-	 */
-	"SHA1_96"
 	;
 
-static const struct ike_alg *default_v2_ah_integ[] = {
+static const struct ike_alg *default_ikev2_ah_integ[] = {
 #ifdef USE_SHA2
 	&ike_alg_integ_sha2_512.common,
 	&ike_alg_integ_sha2_256.common,
@@ -96,27 +91,39 @@ static const struct ike_alg *default_v2_ah_integ[] = {
 	NULL,
 };
 
-const struct proposal_defaults v2_ah_defaults = {
-	.proposals = default_v2_ah_proposals,
-	.integ = default_v2_ah_integ,
+const struct proposal_defaults ikev2_ah_defaults = {
+	.proposals[FIPS_MODE_ON] = default_ikev2_ah_proposals,
+	.proposals[FIPS_MODE_OFF] = default_ikev2_ah_proposals,
+	.integ = default_ikev2_ah_integ,
 };
 
 /*
  * All together now ...
  */
 
-const struct proposal_protocol ah_proposal_protocol = {
+static const struct proposal_protocol ikev1_ah_proposal_protocol = {
 	.name = "AH",
-	.ikev1_alg_id = IKEv1_ESP_ID,
-	.defaults = {
-		[IKEv1] = &v1_ah_defaults,
-		[IKEv2] = &v2_ah_defaults,
-	},
+	.alg_id = IKEv1_IPSEC_ID,
+	.defaults = &ikev1_ah_defaults,
 	.proposal_ok = ah_proposal_ok,
 	.integ = true,
 	.dh = true,
 };
 
+static const struct proposal_protocol ikev2_ah_proposal_protocol = {
+	.name = "AH",
+	.alg_id = IKEv2_ALG_ID,
+	.defaults = &ikev2_ah_defaults,
+	.proposal_ok = ah_proposal_ok,
+	.integ = true,
+	.dh = true,
+};
+
+static const struct proposal_protocol *ah_proposal_protocol[] = {
+	[IKEv1] = &ikev1_ah_proposal_protocol,
+	[IKEv2] = &ikev2_ah_proposal_protocol,
+};
+
 /*
  * ??? why is this called _ah_ when almost everything refers to esp?
  * XXX: Because it is parsing an "ah" line which requires a different
@@ -135,5 +142,5 @@ const struct proposal_protocol ah_proposal_protocol = {
 
 struct proposal_parser *ah_proposal_parser(const struct proposal_policy *policy)
 {
-	return alloc_proposal_parser(policy, &ah_proposal_protocol);
+	return alloc_proposal_parser(policy, ah_proposal_protocol[policy->version]);
 }
