@@ -81,7 +81,7 @@ ckaid_t ckaid_from_secitem(const SECItem *const nss_ckaid)
 	/* ckaid = { .len = min(...), } barfs with gcc 11.2.1 */
 	ckaid_t ckaid = {0};
 	/* should not be truncated but can be */
-	ckaid.len = min(nss_ckaid_len, sizeof(ckaid.ptr/*array*/)),
+	ckaid.len = min(nss_ckaid_len, sizeof(ckaid.ptr/*array*/));
 	pexpect(ckaid.len == nss_ckaid_len);
 	memmove(ckaid.ptr, nss_ckaid->data, ckaid.len);
 	return ckaid;
@@ -111,7 +111,7 @@ err_t string_to_ckaid(const char *string, ckaid_t *ckaid)
 	}
 
 	/* binlen will be "fixed"; ttodata doesn't take void* */
-	const char *err = ttodata(string, 0, 16, (void*)ckaid->ptr, ckaid->len, &ckaid->len);
+	const char *err = ttodata(string, 0, 16, ckaid->ptr, ckaid->len, &ckaid->len);
 	if (err != NULL) {
 		return err;
 	}
