@@ -2083,26 +2083,41 @@ static const char *const v2_notification_status_name[] = {
 #undef S
 };
 
-static const char *const v2_notification_private_name[] = {
+static const char *const v2_notification_private_first_range_name[] = {
 #define S(E) [E - v2N_NULL_AUTH] = #E
-	S(v2N_NULL_AUTH),	/* 40960, used for mixed OE */
+	S(v2N_NULL_AUTH),		/* 40960, used for mixed OE */
 #undef S
 };
 
-static const struct enum_names v2_notification_private_names = {
-	v2N_NULL_AUTH,
-	v2N_NULL_AUTH,
-	ARRAY_REF(v2_notification_private_name),
+static const char *const v2_notification_private_second_range_name[] = {
+#define S(E) [E - v2N_PPK_IDENTITY_KEY] = #E
+	S(v2N_PPK_IDENTITY_KEY),	/* 50208, draft-ietf-ipsecme-ikev2-qr-alt-04 */
+	S(v2N_USE_PPK_INT),		/* 50209, draft-ietf-ipsecme-ikev2-qr-alt-04 */
+#undef S
+};
+
+static const struct enum_names v2_notification_private_second_range_names = {
+	v2N_PPK_IDENTITY_KEY,
+	v2N_USE_PPK_INT,
+	ARRAY_REF(v2_notification_private_second_range_name),
 	"v2N_", /* prefix */
 	NULL
 };
 
+static const struct enum_names v2_notification_private_first_range_names = {
+	v2N_NULL_AUTH,
+	v2N_NULL_AUTH,
+	ARRAY_REF(v2_notification_private_first_range_name),
+	"v2N_", /* prefix */
+	&v2_notification_private_second_range_names
+};
+
 static const struct enum_names v2_notification_status_names = {
 	v2N_INITIAL_CONTACT,
 	v2N_USE_AGGFRAG,
 	ARRAY_REF(v2_notification_status_name),
 	"v2N_", /* prefix */
-	&v2_notification_private_names,
+	&v2_notification_private_first_range_names,
 };
 
 const struct enum_names v2_notification_names = {
