@@ -44,17 +44,27 @@ static struct double_double debug = {
 	.name[N##_IX - DBG_floor_IX] = A "\0", \
 	.help[N##_IX - DBG_floor_IX] = H
 
-	/* start of debug-all */
 	D(DBG_BASE, "debug-base", "enable detailed debug logging"),
+	D(DBG_ROUTING, "debug-routing", "enable routing debug logging"),
+
+	/* end of debug=base */
+
 	D(DBG_CPU_USAGE, "debug-cpu-usage", "estimate cpu used"),
 	D(DBG_REFCNT, "debug-refcnt", "debug reference counts"),
-	/* end of debug-all */
+
+	/* end of debug=all */
+
+	D(DBG_TMI, "debug-tmi", "far too much information"),
+
+	/* end of debug=tmi */
+
+	D(DBG_UPDOWN, "debug-updown", "run updown script with 'set -v -x'"),
 
 	D(DBG_CRYPT, "debug-crypt", "encryption/decryption of messages: DANGER!"),
 	D(DBG_PRIVATE, "debug-private", "displays private information: DANGER!"),
 	D(DBG_WHACKWATCH, "debug-whackwatch", "never let WHACK go"),
 	D(DBG_ADD_PREFIX, "debug-add-prefix", "add the log+state prefix to debug lines"),
-	D(DBG_TMI, "debug-tmi", "far too much information"),
+
 #undef D
 };
 
@@ -66,8 +76,9 @@ const enum_names debug_names = {
 };
 
 struct lmod_alias debug_aliases[] = {
-	{ "all", DBG_ALL, },
-	{ "tmi", DBG_ALL|DBG_TMI, },
+	{ "base", LRANGE(DBG_floor_IX, DBG_base_IX), },
+	{ "all", LRANGE(DBG_floor_IX, DBG_all_IX), },
+	{ "tmi", LRANGE(DBG_floor_IX, DBG_tmi_IX), },
 	{ NULL, LEMPTY, },
 };
 
