@@ -48,7 +48,7 @@ diag_t diag_errno(int error, const char *fmt, ...)
 	va_start(ap, fmt);
 	jam_va_list(&buf, fmt, ap);
 	va_end(ap);
-	jam_string(&buf, ": ");
+	/* NO ": "; not *perror() */
 	jam_errno(&buf, error);
 	return (diag_t)clone_str(output, "diag-errno");
 }
@@ -94,12 +94,6 @@ diag_t diag_jambuf(struct jambuf *buf)
 	return (diag_t) clone_hunk_as_string(msg, "diag jambuf");
 }
 
-diag_t clone_diag(diag_t diag)
-{
-	/* clone_str() clones NULL as NULL */
-	return (diag_t) clone_str((char*)diag, "diag clone");
-}
-
 const char *str_diag(diag_t diag)
 {
 	/* let caller deal with mess */
@@ -118,38 +112,3 @@ size_t jam_diag(struct jambuf *buf, diag_t diag)
 {
 	return jam_string(buf, str_diag(diag));
 }
-
-void llog_diag(lset_t rc_flags, struct logger *logger, diag_t *diag,
-	       const char *fmt, ...)
-{
-	LLOG_JAMBUF(rc_flags, logger, buf) {
-		va_list ap;
-		va_start(ap, fmt);
-		jam_va_list(buf, fmt, ap);
-		va_end(ap);
-		jam_diag(buf, *diag);
-	}
-	pfree_diag(diag);
-}
-
-void fatal_diag(enum pluto_exit_code rc, struct logger *logger, diag_t *diag,
-		const char *fmt, ...)
-{
-	JAMBUF(buf) {
-		/* XXX: The message format is:
-		 *   FATAL ERROR: <log-prefix><message...><diag>
-		 * and not:
-		 *   <log-prefix>FATAL ERROR: <message...><diag>
-		 */
-		jam(buf, "FATAL ERROR: ");
-		jam_logger_prefix(buf, logger);
-		va_list ap;
-		va_start(ap, fmt);
-		jam_va_list(buf, fmt, ap);
-		va_end(ap);
-		jam_diag(buf, *diag);
-		jambuf_to_logger(buf, logger, ERROR_FLAGS);
-	}
-	pfree_diag(diag); /* XXX: bother? */
-	libreswan_exit(rc);
-}
