@@ -50,36 +50,71 @@
 
 long next_enum(enum_names *en, long l)
 {
-	enum_names *p = en;
+	if (l < -1) {
+		llog_passert(&global_logger, HERE, "%ld should be >= -1", l);
+	}
+
 	unsigned long e;
+
+	/*
+	 * Advance L by 1 giving a starting candidate.
+	 */
 	if (l < 0) {
-		e = en->en_first;
-		if (en->en_names[e - p->en_first] != NULL) {
-			return e;
-		}
+		e = 0;
 	} else {
-		e = l;
+		e = l+1;
 	}
 
 	while (true) {
-		while (true) {
-			if (p == NULL) {
-				return -1;
+		/*
+		 * Find a range that puts an upper bound on E.  E may
+		 * fall within the range, but could also fall short.
+		 */
+		const struct enum_names *b = NULL;
+		for (const struct enum_names *p = en;
+		     p != NULL; p = p->en_next_range) {
+
+			if (p->en_last - p->en_first + 1 != p->en_checklen) {
+				llog_passert(&global_logger, HERE,
+					     "p->en_last(%lu) - p->en_first(%lu) + 1 == p->en_checklen(%zu)",
+					     p->en_last, p->en_first, p->en_checklen);
 			}
-			passert(p->en_last - p->en_first + 1 == p->en_checklen);
-			if (p->en_first <= e && e < p->en_last) {
-				e++;
+
+			if (e > p->en_last) {
+				continue;
+			}
+
+			if (e >= p->en_first) {
+				b = p;
 				break;
-			} else if (e == p->en_last && p->en_next_range != NULL) {
-				p = p->en_next_range;
-				e = p->en_first;
-				break;
-			} else {
-				p = p->en_next_range;
+			}
+
+			/* e < .en_first; save when closer */
+			if (b == NULL ||
+			    b->en_first > p->en_first) {
+				b = p;
 			}
 		}
-		if (p->en_names[e - p->en_first] != NULL) {
-			return e;
+		if (b == NULL) {
+			return -1;
+		}
+
+		/*
+		 * If E isn't yet within the range, make it so.
+		 */
+		if (e < b->en_first) {
+			e = b->en_first;
+		}
+
+		/*
+		 * Find the first valid name within the range; if this
+		 * fails, iterate round looking for a new range.
+		 */
+		while (e <= b->en_last) {
+			if (b->en_names[e - b->en_first] != NULL) {
+				return e;
+			}
+			e++;
 		}
 	}
 }
@@ -125,78 +160,78 @@ const char *enum_range_name(const struct enum_names *range, unsigned long val,
 	}
 }
 
-/* look up enum names in an enum_names */
-const char *enum_name(enum_names *ed, unsigned long val)
+static const char *find_enum(const struct enum_names *en, unsigned long val, bool shorten)
 {
 	const char *prefix = NULL;
 	/* can be NULL */
-	const struct enum_names *range = enum_range(ed, val, &prefix);
+	const struct enum_names *range = enum_range(en, val, &prefix);
 	/* can be NULL */
-	return enum_range_name(range, val, prefix, /*shorten?*/false);
+	return enum_range_name(range, val, prefix, shorten);
 }
 
-const char *enum_name_short(enum_names *ed, unsigned long val)
+bool enum_long(enum_names *ed, unsigned long val, enum_buf *b)
 {
-	const char *prefix = NULL;
-	/* can be NULL */
-	const struct enum_names *range = enum_range(ed, val, &prefix);
-	/* can be NULL */
-	return enum_range_name(range, val, prefix, /*shorten?*/true);
-}
-
-size_t jam_enum(struct jambuf *buf, enum_names *en, unsigned long val)
-{
-	const char *name = enum_name(en, val);
-	if (name == NULL) {
-		if (en->en_prefix != NULL) {
-			jam_string(buf, en->en_prefix);
-			jam_string(buf, "_");
-		}
-		return jam(buf, "%lu??", val);
+	/* can be NULL; handled here */
+	b->buf = find_enum(ed, val, /*shorten?*/false);
+	if (b->buf == NULL) {
+		bad_name(val, b);
+		return false;
 	}
-	return jam_string(buf, name);
+
+	return true;
 }
 
-size_t jam_enum_short(struct jambuf *buf, enum_names *en, unsigned long val)
+bool enum_short(enum_names *ed, unsigned long val, enum_buf *b)
 {
-	const char *name = enum_name_short(en, val);
-	if (name == NULL) {
-		if (en->en_prefix != NULL) {
-			jam_string(buf, en->en_prefix);
-			jam_string(buf, "_");
-		}
-		return jam(buf, "%lu??", val);
+	/* can be NULL; handled here */
+	b->buf = find_enum(ed, val, /*shorten?*/true);
+	if (b->buf == NULL) {
+		bad_name(val, b);
+		return false;
 	}
-	return jam_string(buf, name);
+
+	return true;
 }
 
-/*
- * find or construct a string to describe an enum value
- *
- * Note: result may or may not be in b.
- */
-const char *str_enum(enum_names *ed, unsigned long val, enum_buf *b)
+const char *str_enum_long(enum_names *ed, unsigned long val, enum_buf *b)
 {
-	const char *p = enum_name(ed, val);
-
-	if (p == NULL) {
-		snprintf(b->buf, sizeof(b->buf), "%lu??", val);
-		p = b->buf;
-	}
-	return p;
+	enum_long(ed, val, b);
+	return b->buf;
 }
 
 const char *str_enum_short(enum_names *ed, unsigned long val, enum_buf *b)
 {
-	const char *prefix;
-	const struct enum_names *range = enum_range(ed, val, &prefix);
-	/* could be NULL */
-	const char *name = enum_range_name(range, val, prefix, /*shorten?*/true);
-	if (name == NULL) {
-		snprintf(b->buf, sizeof(b->buf), "%lu??", val);
-		name = b->buf;
+	enum_short(ed, val, b);
+	return b->buf;
+}
+
+size_t jam_enum_long(struct jambuf *buf, enum_names *en, unsigned long val)
+{
+	const char *name = find_enum(en, val, /*shorten*/false);
+	if (name != NULL) {
+		return jam_string(buf, name);
 	}
-	return name;
+	return jam_bad(buf, en->en_prefix, val);
+}
+
+size_t jam_enum_short(struct jambuf *buf, enum_names *en, unsigned long val)
+{
+	/* can be NULL; handled here */
+	const char *name = find_enum(en, val, /*shorten*/true);
+	if (name != NULL) {
+		return jam_string(buf, name);
+	}
+	return jam_bad(buf, en->en_prefix, val);
+}
+
+size_t jam_enum_human(struct jambuf *buf, enum_names *en, unsigned long val)
+{
+	/* can be NULL; handled here */
+	const char *name = find_enum(en, val, /*shorten?*/true);
+	if (name != NULL) {
+		return jam_string_human(buf, name);
+	}
+	return jam_bad(buf, en->en_prefix, val);
 }
 
 /*
@@ -271,37 +306,41 @@ int enum_match(enum_names *ed, shunk_t string)
 }
 
 /* choose table from struct enum_enum_names */
-enum_names *enum_enum_table(enum_enum_names *een,
-			    unsigned long table)
+static const struct enum_names *enum_enum_table(enum_enum_names *een,
+						unsigned long table)
 {
-	passert(een->een_last - een->een_first + 1 == een->een_checklen);
+	if (!(een->een_last - een->een_first + 1 == een->een_checklen)) {
+		/* *_{last,first} are longs */
+		llog_passert(&global_logger, HERE,
+			     ".een_last=%lu - .een_first=%lu + 1 == .een_checklen=%zu",
+			     een->een_last, een->een_first, een->een_checklen);
+	}
 
 	if (een->een_first <= table && table <= een->een_last) {
 		return een->een_enum_name[table - een->een_first];
-	} else {
-		return NULL;
 	}
+
+	return NULL;
 }
 
-const char *enum_enum_name(enum_enum_names *een, unsigned long table,
-			   unsigned long val)
+bool enum_enum_name(enum_enum_names *een, unsigned long table,
+		    unsigned long val, enum_buf *b)
 {
 	enum_names *en = enum_enum_table(een, table);
+	if (en == NULL) {
+		snprintf(b->tmp, sizeof(b->tmp), "%lu.%lu", table, val);
+		b->buf = b->tmp;
+		return false;
+	}
 
-	return en == NULL ? NULL : enum_name(en, val);
+	return enum_name(en, val, b);
 }
 
 const char *str_enum_enum(enum_enum_names *een, unsigned long table,
 			  unsigned long val, enum_buf *b)
 {
-	enum_names *en = enum_enum_table(een, table);
-	if (en == NULL) {
-		/* assume the log context implies the table name */
-		snprintf(b->buf, sizeof(b->buf), "%lu??", val);
-		return b->buf;
-	}
-
-	return str_enum(en, val, b);
+	enum_enum_name(een, table, val, b);
+	return b->buf;
 }
 
 const char *str_enum_enum_short(enum_enum_names *een, unsigned long table,
@@ -310,7 +349,8 @@ const char *str_enum_enum_short(enum_enum_names *een, unsigned long table,
 	enum_names *en = enum_enum_table(een, table);
 	if (en == NULL) {
 		/* assume the log context implies the table name */
-		snprintf(b->buf, sizeof(b->buf), "%lu??", val);
+		snprintf(b->tmp, sizeof(b->tmp), "%lu.%lu", table, val);
+		b->buf = b->tmp;
 		return b->buf;
 	}
 
@@ -323,7 +363,7 @@ size_t jam_enum_enum(struct jambuf *buf, enum_enum_names *een,
 	enum_names *en = enum_enum_table(een, table);
 	if (en == NULL) {
 		/* XXX: dump something more meaningful */
-		return jam(buf, "%lu??%lu??", table, val);
+		return jam(buf, "%lu.%lu", table, val);
 	}
 	return jam_enum(buf, en, val);
 }
@@ -334,22 +374,38 @@ size_t jam_enum_enum_short(struct jambuf *buf, enum_enum_names *een,
 	enum_names *en = enum_enum_table(een, table);
 	if (en == NULL) {
 		/* XXX: dump something more meaningful */
-		return jam(buf, "%lu??%lu??", table, val);
+		return jam(buf, "%lu.%lu", table, val);
 	}
 	return jam_enum_short(buf, en, val);
 }
 
-void check_enum_names(enum_names *checklist[], size_t tl)
+void check_enum_names(const struct enum_names_check *checklist)
 {
 	/* check that enum_names are well-formed */
-	size_t i;
-
-	for (i = 0; i != tl; i++) {
+	for (const struct enum_names_check *c = checklist;
+	     c->name != NULL && c->enum_names != NULL; c++) {
 		/*
-		 * enum_name will check all linked enum_names
-		 * if given a value that isn't covered.
-		 * -42 is probably not covered.
+		 * enum_name will check all linked enum_names if given
+		 * a value that isn't covered.  -42 is probably not
+		 * covered.
 		 */
-		(void) enum_name(checklist[i], -42UL);
+		enum_buf b;
+		enum_name(c->enum_names, -42UL, &b);
 	}
 }
+
+void check_enum_enum_names(const struct enum_enum_names_check *checklist)
+{
+	/* check that enum_enum_names are well-formed */
+	for (const struct enum_enum_names_check *c = checklist;
+	     c->name != NULL && c->enum_enum_names != NULL; c++) {
+		/* check v2_transform_ID_enums, the only enum_enum_names */
+		enum_enum_table(c->enum_enum_names, -42UL);
+	}
+}
+
+void init_enum_names(void)
+{
+	check_enum_names(enum_names_checklist);
+	check_enum_enum_names(enum_enum_names_checklist);
+}
