@@ -21,18 +21,21 @@
 #include "lswalloc.h"		/* for clone_bytes() */
 #include "hunk.h"
 
-char *clone_bytes_as_string(const void *ptr, size_t maxlen, const char *name)
+char *raw_clone_as_string(const void *ptr, size_t maxlen, const char *name)
 {
 	if (ptr == NULL) {
 		return NULL;
 	}
 
-	/* Look for terminating NULL, if there is one */
+	/* Don't assume terminating NUL */
 	size_t len = strnlen(ptr, maxlen);
 
+	/* include space for NUL */
 	char *out = alloc_things(char, len + 1, name);
 	memcpy(out, ptr, len);
-	out[maxlen] = '\0'; /* also done by alloc_things() */
+
+	/* explict; but redundant as done by alloc_things() */
+	out[len] = '\0';
 	return out;
 }
 
@@ -58,8 +61,8 @@ int raw_cmp(const void *l_ptr, size_t l_len,
 	return ((int)l_len - (int)r_len);
 }
 
-bool bytes_eq(const void *l_ptr, size_t l_len,
-	      const void *r_ptr, size_t r_len)
+bool raw_eq(const void *l_ptr, size_t l_len,
+	    const void *r_ptr, size_t r_len)
 {
 	/* NULL and EMPTY("") are not the same */
 	if (l_ptr == NULL || r_ptr == NULL) {
@@ -71,8 +74,8 @@ bool bytes_eq(const void *l_ptr, size_t l_len,
 	return memcmp(l_ptr, r_ptr, r_len) == 0;
 }
 
-bool case_eq(const void *l_ptr, size_t l_len,
-	     const void *r_ptr, size_t r_len)
+bool raw_caseeq(const void *l_ptr, size_t l_len,
+		const void *r_ptr, size_t r_len)
 {
 	/* NULL and EMPTY("") are not the same */
 	if (l_ptr == NULL || r_ptr == NULL) {
@@ -84,7 +87,35 @@ bool case_eq(const void *l_ptr, size_t l_len,
 	return strncasecmp(l_ptr, r_ptr, r_len) == 0;
 }
 
-void hton_bytes(uintmax_t h, void *bytes, size_t size)
+bool raw_heq(const void *l_ptr, size_t l_len,
+	     const void *r_ptr, size_t r_len)
+{
+	/* NULL and EMPTY("") are not the same */
+	if (l_ptr == NULL || r_ptr == NULL) {
+		return l_ptr == r_ptr;
+	}
+	if (l_len != r_len) {
+		return false;
+	}
+	const char *l = l_ptr;
+	const char *r = r_ptr;
+	for (unsigned i = 0; i < l_len; i++) {
+		char lc = l[i];
+		char rc = r[i];
+		if (char_tolower(lc) == char_tolower(rc)) {
+			continue;
+		}
+		const char *wild = "-_";
+		if (strchr(wild, lc) != NULL &&
+		    strchr(wild, rc) != NULL) {
+			continue;
+		}
+		return false;
+	}
+	return true;
+}
+
+void raw_hton(uintmax_t h, void *bytes, size_t size)
 {
 	uint8_t *byte = bytes;
 	for (unsigned i = 0; i < size; i++) {
@@ -94,17 +125,17 @@ void hton_bytes(uintmax_t h, void *bytes, size_t size)
 	}
 }
 
-uintmax_t ntoh_bytes(const void *bytes, size_t size)
+uintmax_t raw_ntoh(const void *bytes, size_t size)
 {
 	uintmax_t h = 0;
 	const uint8_t *byte = bytes;
 	for (unsigned i = 0; i < size; i++) {
 		uintmax_t n = (h<<8) + byte[i];
 		if (n < h) {
-			h = UINTMAX_MAX;
-		} else {
-			h = n;
+			/* i.e., went backwards */
+			return UINTMAX_MAX;
 		}
+		h = n;
 	}
 	return h;
 }
@@ -171,3 +202,31 @@ char char_toupper(char c)
 {
 	return char_islower(c) ? c - 'a' + 'A' : c;
 }
+
+bool raw_starteq(const void *ptr, size_t len, const void *eat_ptr, size_t eat_len)
+{
+	if (ptr == NULL || eat_ptr == NULL) {
+		return false;
+	}
+	if (len < eat_len) {
+		return false;
+	}
+	if (strncmp(ptr, eat_ptr, eat_len) != 0) {
+		return false;
+	}
+	return true;
+}
+
+bool raw_casestarteq(const void *ptr, size_t len, const void *eat_ptr, size_t eat_len)
+{
+	if (ptr == NULL || eat_ptr == NULL) {
+		return false;
+	}
+	if (len < eat_len) {
+		return false;
+	}
+	if (strncasecmp(ptr, eat_ptr, eat_len) != 0) {
+		return false;
+	}
+	return true;
+}
