@@ -42,6 +42,7 @@
 #include "certs.h"
 #include "ip_info.h"
 #include "ttodata.h"
+#include "lswnss.h"		/* for clone_secitem_as_chunk() */
 
 const struct id empty_id = {
 	.kind = ID_NONE,
@@ -401,8 +402,8 @@ bool same_id(const struct id *a, const struct id *b)
 
 /* compare two struct id values, DNs can contain wildcards */
 
-bool match_id(const char *prefix, const struct id *a, const struct id *b,
-	      int *wildcards_out)
+bool match_id(const struct id *a, const struct id *b,
+	      int *wildcards_out, struct verbose verbose)
 {
 	bool match;
 	int wildcards;
@@ -415,7 +416,7 @@ bool match_id(const char *prefix, const struct id *a, const struct id *b,
 		wildcards = MAX_WILDCARDS;
 		match = false;
 	} else if (a->kind == ID_DER_ASN1_DN) {
-		match = match_dn_any_order_wild(prefix, a->name, b->name, &wildcards);
+		match = match_dn_any_order_wild(a->name, b->name, &wildcards, verbose);
 	} else if (same_id(a, b)) {
 		wildcards = 0;
 		match = true;
@@ -424,13 +425,11 @@ bool match_id(const char *prefix, const struct id *a, const struct id *b,
 		match = false;
 	}
 
-	if (DBGP(DBG_BASE)) {
-		id_buf buf;
-		DBG_log("%smatch_id a=%s", prefix, str_id(a, &buf));
-		DBG_log("%s         b=%s", prefix, str_id(b, &buf));
-		DBG_log("%sresults  %s wildcards=%d",
-			prefix, match ? "matched" : "fail", wildcards);
-	}
+	id_buf buf;
+	vdbg("match_id a=%s", str_id(a, &buf));
+	vdbg("         b=%s", str_id(b, &buf));
+	vdbg("  result %s wildcards=%d",
+	     match ? "matched" : "fail", wildcards);
 
 	*wildcards_out = wildcards;
 	return match;
@@ -510,3 +509,18 @@ enum ike_id_type id_to_payload(const struct id *id, const ip_address *host, shun
 	*body = tl;
 	return type;
 }
+
+
+/*
+ * choose either subject DN or a subjectAltName as connection end ID
+ */
+struct id id_from_cert(const struct cert *cert)
+{
+	chunk_t name = clone_secitem_as_chunk(cert->nss_cert->derSubject, "cert id");
+	struct id id = {
+		.name = ASN1(name),
+		.scratch = name.ptr,
+		.kind = ID_DER_ASN1_DN,
+	};
+	return id;
+}
