@@ -29,7 +29,7 @@ const struct encrypt_desc ike_alg_encrypt_3des_cbc =
 		.algo_type =     IKE_ALG_ENCRYPT,
 		.id = {
 			[IKEv1_OAKLEY_ID] = OAKLEY_3DES_CBC,
-			[IKEv1_ESP_ID] = ESP_3DES,
+			[IKEv1_IPSEC_ID] = IKEv1_ESP_3DES,
 			[IKEv2_ALG_ID] = IKEv2_ENCR_3DES,
 #ifdef SADB_EALG_3DESCBC
 			[SADB_ALG_ID] = SADB_EALG_3DESCBC,
@@ -44,8 +44,8 @@ const struct encrypt_desc ike_alg_encrypt_3des_cbc =
 	.pad_to_blocksize = true,
 	.wire_iv_size =           DES_CBC_BLOCK_SIZE,
 	.keylen_omitted = true,
-	.keydeflen =        DES_CBC_BLOCK_SIZE * 3 * BITS_PER_BYTE,
-	.key_bit_lengths = { DES_CBC_BLOCK_SIZE * 3 * BITS_PER_BYTE, },
+	.keydeflen =        DES_CBC_BLOCK_SIZE * 3 * BITS_IN_BYTE,
+	.key_bit_lengths = { DES_CBC_BLOCK_SIZE * 3 * BITS_IN_BYTE, },
 	.encrypt_ops = &ike_alg_encrypt_nss_cbc_ops,
 	.encrypt_netlink_xfrm_name = "des3_ede",
 	.encrypt_tcpdump_name = "3des",
