@@ -46,7 +46,7 @@ static void nss_ecp_calc_local_secret(const struct dh_desc *group,
 	 * Get the PK11 formatted EC parameters (stored in static
 	 * data) from NSS.
 	 */
-	DBGF(DBG_CRYPT, "oid %d %x", group->nss_oid, group->nss_oid);
+	ldbgf(DBG_CRYPT, logger, "oid %d %x", group->nss_oid, group->nss_oid);
 
 	/*
 	 * Wrap the raw OID in ASN.1.  SECKEYECParams is just a
@@ -162,7 +162,7 @@ static diag_t nss_ecp_calc_shared_secret(const struct dh_desc *group,
 		 * (see comments in pk11_get_EC_PointLenInBytes()).
 		 */
 		passert(remote_ke.len == local_pubk->u.ec.publicValue.len);
-		DBG_log("passing raw CURVE25519 public key blob to NSS");
+		ldbg(logger, "passing raw CURVE25519 public key blob to NSS");
 		memcpy(remote_pubk.u.ec.publicValue.data, remote_ke.ptr, remote_ke.len);
 	}
 
@@ -190,7 +190,7 @@ static diag_t nss_ecp_calc_shared_secret(const struct dh_desc *group,
 		return diag_nss_error("shared key calculation using ECP failed");
 	}
 
-	dbg_alloc("temp", temp, HERE);
+	ldbg_alloc(logger, "temp", temp, HERE);
 
 	/*
 	 * The key returned above doesn't play well with PK11_Derive()
@@ -202,7 +202,7 @@ static diag_t nss_ecp_calc_shared_secret(const struct dh_desc *group,
 					       0, sizeof_symkey(temp),
 					       HERE, logger);
 
-	release_symkey(__func__, "temp", &temp);
+	symkey_delref(logger, "temp", &temp);
 	SECITEM_FreeItem(&remote_pubk.u.ec.publicValue, PR_FALSE);
 
 	return NULL;
