@@ -38,16 +38,24 @@ const struct prf_desc ike_alg_prf_hmac_md5 = {
 		.algo_type = IKE_ALG_PRF,
 		.id = {
 			[IKEv1_OAKLEY_ID] = OAKLEY_MD5,
-			[IKEv1_ESP_ID] = -1,
+			[IKEv1_IPSEC_ID] = -1,
 			[IKEv2_ALG_ID] = IKEv2_PRF_HMAC_MD5,
 		},
 	},
+	.nss = {
+		.mechanism = CKM_MD5_HMAC,
+	},
 	.prf_key_size = MD5_DIGEST_SIZE,
 	.prf_output_size = MD5_DIGEST_SIZE,
 	.hasher = &ike_alg_hash_md5,
-	.prf_mac_ops = &ike_alg_prf_mac_hmac_ops,
+	.prf_mac_ops = &ike_alg_prf_mac_nss_ops,
+#ifdef USE_NSS_KDF
+	.prf_ikev1_ops = &ike_alg_prf_ikev1_nss_ops,
+	.prf_ikev2_ops = &ike_alg_prf_ikev2_nss_ops,
+#else
 	.prf_ikev1_ops = &ike_alg_prf_ikev1_mac_ops,
 	.prf_ikev2_ops = &ike_alg_prf_ikev2_mac_ops,
+#endif
 	.prf_ike_audit_name = "md5",
 };
 
@@ -58,7 +66,7 @@ const struct integ_desc ike_alg_integ_hmac_md5_96 = {
 		.algo_type = IKE_ALG_INTEG,
 		.id = {
 			[IKEv1_OAKLEY_ID] = OAKLEY_MD5,
-			[IKEv1_ESP_ID] = AUTH_ALGORITHM_HMAC_MD5,
+			[IKEv1_IPSEC_ID] = AUTH_ALGORITHM_HMAC_MD5,
 			[IKEv2_ALG_ID] = IKEv2_INTEG_HMAC_MD5_96,
 #ifdef SADB_AALG_MD5HMAC
 			[SADB_ALG_ID] = SADB_AALG_MD5HMAC,
@@ -67,7 +75,7 @@ const struct integ_desc ike_alg_integ_hmac_md5_96 = {
 	},
 	.integ_keymat_size = MD5_DIGEST_SIZE,
 	.integ_output_size = MD5_DIGEST_SIZE_96,
-	.integ_ikev1_ah_transform = AH_MD5,
+	.integ_ikev1_ah_transform = IKEv1_AH_MD5,
 	.prf = &ike_alg_prf_hmac_md5,
 	.integ_netlink_xfrm_name = "md5",
 	.integ_tcpdump_name = "md5",
