@@ -19,6 +19,7 @@
 #include "ike_alg_encrypt.h"
 #include "ike_alg_integ.h"
 #include "ike_alg_dh.h"
+#include "ike_alg_encrypt_ops.h"
 #include "ike_alg_dh_ops.h"
 #include "lsw-pfkeyv2.h"	/* for SADB_*ALG_* */
 
@@ -36,8 +37,11 @@ const struct encrypt_desc ike_alg_encrypt_null =
 		.algo_type = IKE_ALG_ENCRYPT,
 		.id = {
 			[IKEv1_OAKLEY_ID] = -1,
-			[IKEv1_ESP_ID] = ESP_NULL,
+			[IKEv1_IPSEC_ID] = IKEv1_ESP_NULL,
 			[IKEv2_ALG_ID] = IKEv2_ENCR_NULL,
+#ifdef SADB_EALG_NULL
+			[SADB_ALG_ID] = SADB_EALG_NULL,
+#endif
 		},
 	},
 	.enc_blocksize =  1,
@@ -46,13 +50,11 @@ const struct encrypt_desc ike_alg_encrypt_null =
 	.keylen_omitted = true,
 	.keydeflen = 0,
 	.key_bit_lengths = { 0, },
-#ifdef SADB_EALG_NULL
-	.encrypt_sadb_ealg_id = SADB_EALG_NULL,
-#endif
 	.encrypt_netlink_xfrm_name = "cipher_null",
 	.encrypt_tcpdump_name = "null",
 	.encrypt_ike_audit_name = "null",
 	.encrypt_kernel_audit_name = "NULL",
+	.encrypt_ops = &ike_alg_encrypt_null_ops,
 };
 
 /*
@@ -77,8 +79,12 @@ const struct integ_desc ike_alg_integ_none = {
 			 * down the track.  One fix would be to
 			 * finally add IKEv1_AH_ID.
 			 */
-			[IKEv1_ESP_ID] = AUTH_ALGORITHM_NONE, /* not NULL_KAME? */
+			[IKEv1_IPSEC_ID] = AUTH_ALGORITHM_NONE, /* not NULL_KAME? */
 			[IKEv2_ALG_ID] = IKEv2_INTEG_NONE,
+#ifdef SADB_X_AALG_NULL
+			/* This is from BSD's KAME */
+			[SADB_ALG_ID] = SADB_X_AALG_NULL,
+#endif
 		},
 		/*
 		* Because aes_gcm-null is valid in FIPS mode, "none"
@@ -87,12 +93,8 @@ const struct integ_desc ike_alg_integ_none = {
 		* Other code gets the job of rejecting "none" when not
 		* AEAD.
 		*/
-		.fips = true,
+		.fips.approved = true,
 	},
-#ifdef SADB_X_AALG_NULL
-	/* This is from BSD's KAME */
-	.integ_sadb_aalg_id = SADB_X_AALG_NULL,
-#endif
 	.integ_netlink_xfrm_name = "digest_null",
 	.integ_tcpdump_name = "none",
 	.integ_ike_audit_name = "none",
@@ -109,7 +111,7 @@ const struct dh_desc ike_alg_dh_none = {
 		.algo_type = IKE_ALG_DH,
 		.id = {
 			[IKEv1_OAKLEY_ID] = -1,
-			[IKEv1_ESP_ID] = -1,
+			[IKEv1_IPSEC_ID] = -1,
 			[IKEv2_ALG_ID] = OAKLEY_GROUP_NONE,
 		},
 		/*
@@ -120,7 +122,7 @@ const struct dh_desc ike_alg_dh_none = {
 		 *
 		 * Other code gets the job of rejecting "none".
 		 */
-		.fips = true,
+		.fips.approved = true,
 	},
 	.group = OAKLEY_GROUP_NONE,
 	/*
