@@ -6,6 +6,7 @@
  * Copyright (C) 2013-2019 D. Hugh Redelmeier <hugh@mimosa.com>
  * Copyright (C) 2015-2019 Andrew Cagney <cagney@gnu.org>
  * Copyright (C) 2019 Paul Wouters <pwouters@redhat.com>
+ * Copyright (C) 2020 Nupur Agrawal <nupur202000@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -136,6 +137,21 @@ static PK11SymKey *ike_sa_rekey_skeyseed(const struct prf_desc *prf_desc,
 			    HERE, logger);
 }
 
+/*
+ * SKEYSEED = prf(SK_d (old), "Resumption" | Ni | Nr)
+ */
+
+static PK11SymKey *ike_sa_resume_skeyseed(const struct prf_desc *prf,
+						  PK11SymKey *SK_d_old,
+						  const chunk_t Ni, const chunk_t Nr,
+						  struct logger *logger)
+{
+	return ike_alg_prf_ikev2_mac_ops.ike_sa_resume_skeyseed(prf,
+								SK_d_old,
+								Ni, Nr,
+								logger);
+}
+
 /*
  * Compute: prf+ (SKEYSEED, Ni | Nr | SPIi | SPIr)
  */
@@ -260,12 +276,24 @@ static struct crypt_mac psk_auth(const struct prf_desc *prf_desc,
 	return signed_octets;
 }
 
+static struct crypt_mac psk_resume(const struct prf_desc *prf,
+				   PK11SymKey *SK_px,
+				   chunk_t first_packet,
+				   struct logger *logger)
+{
+	return ike_alg_prf_ikev2_mac_ops.psk_resume(prf, SK_px,
+						    first_packet,
+						    logger);
+}
+
 const struct prf_ikev2_ops ike_alg_prf_ikev2_nss_ops = {
 	.backend = "NSS",
 	.prfplus = prfplus,
 	.ike_sa_skeyseed = ike_sa_skeyseed,
 	.ike_sa_rekey_skeyseed = ike_sa_rekey_skeyseed,
+	.ike_sa_resume_skeyseed = ike_sa_resume_skeyseed,
 	.ike_sa_keymat = ike_sa_keymat,
 	.child_sa_keymat = child_sa_keymat,
 	.psk_auth = psk_auth,
+	.psk_resume = psk_resume,
 };
