@@ -365,7 +365,7 @@ struct impairment impairments[] = {
 	U(v2n_rekey_sa_protoid, "corrupt the IKEv2 REKEY CHILD notify protocol ID"),
 	U(v2_proposal_protoid, "corrupt the IKEv2 proposal substructure protocol ID"),
 
-	U(helper_thread_delay, "pause <unsigned> seconds before starting each helper thread job"),
+	U(helper_thread_delay, "pause <unsigned> seconds before starting each helper thread job; 0 will MS warp the delay"),
 
 	B(install_ipsec_sa_inbound_state, "error after installing the inbound IPsec SA state (but before policy)"),
 	B(install_ipsec_sa_inbound_policy, "error after installing the inbound IPsec SA policy (and state)"),
