@@ -25,12 +25,13 @@
 
 const ip_address unset_address; /* all zeros */
 
-ip_address address_from_raw(where_t where, enum ip_version version,
+ip_address address_from_raw(where_t where,
+			    const struct ip_info *afi,
 			    const struct ip_bytes bytes)
 {
 	ip_address a = {
 		.is_set = true,
-		.version = version,
+		.version = afi->ip_version,
 		.bytes = bytes,
 	};
 	pexpect_address(&a, where);
@@ -54,7 +55,7 @@ diag_t data_to_address(const void *data, size_t sizeof_data,
 
 	struct ip_bytes bytes = unset_ip_bytes;
 	memcpy(bytes.byte, data, afi->ip_size);
-	*dst = address_from_raw(HERE, afi->ip_version, bytes);
+	*dst = address_from_raw(HERE, afi, bytes);
 	return NULL;
 }
 
@@ -62,14 +63,14 @@ ip_address address_from_in_addr(const struct in_addr *in)
 {
 	struct ip_bytes bytes = { .byte = { 0, }, };
 	memcpy(bytes.byte, in, sizeof(*in));
-	return address_from_raw(HERE, IPv4, bytes);
+	return address_from_raw(HERE, &ipv4_info, bytes);
 }
 
 ip_address address_from_in6_addr(const struct in6_addr *in6)
 {
 	struct ip_bytes bytes = { .byte = { 0, }, };
 	memcpy(bytes.byte, in6, sizeof(*in6));
-	return address_from_raw(HERE, IPv6, bytes);
+	return address_from_raw(HERE, &ipv6_info, bytes);
 }
 
 const struct ip_info *address_type(const ip_address *address)
@@ -115,13 +116,9 @@ chunk_t address_as_chunk(ip_address *address)
 
 size_t jam_address(struct jambuf *buf, const ip_address *address)
 {
-	if (address_is_unset(address)) {
-		return jam_string(buf, "<unset-address>");
-	}
-
 	const struct ip_info *afi = address_type(address);
 	if (afi == NULL) {
-		return jam_string(buf, "<unknown-address>");
+		return jam_string(buf, "<unset-address>");
 	}
 
 	return afi->jam.address(buf, afi, &address->bytes);
@@ -129,13 +126,9 @@ size_t jam_address(struct jambuf *buf, const ip_address *address)
 
 size_t jam_address_wrapped(struct jambuf *buf, const ip_address *address)
 {
-	if (address_is_unset(address)) {
-		return jam_string(buf, "<unset-address>");
-	}
-
 	const struct ip_info *afi = address_type(address);
 	if (afi == NULL) {
-		return jam_string(buf, "<unknown-address>");
+		return jam_string(buf, "<unset-address>");
 	}
 
 	return afi->jam.address_wrapped(buf, afi, &address->bytes);
@@ -227,7 +220,7 @@ bool address_is_unset(const ip_address *address)
 
 bool address_is_specified(const ip_address address)
 {
-	const struct ip_info *afi = address_type(&address);
+	const struct ip_info *afi = address_info(address);
 	if (afi == NULL) {
 		return false;
 	}
@@ -256,7 +249,7 @@ bool address_eq_address(const ip_address l, const ip_address r)
 
 bool address_is_loopback(const ip_address address)
 {
-	const struct ip_info *afi = address_type(&address);
+	const struct ip_info *afi = address_info(address);
 	if (afi == NULL) {
 		/* NULL+unset+unknown */
 		return false;
