@@ -1,6 +1,8 @@
 /* ip cidr, for libreswan
  *
- * Copyright (C) 2019-2020 Andrew Cagney <cagney@gnu.org>
+ * Copyright (C) 2019-2024 Andrew Cagney <cagney@gnu.org>
+ * Copyright (C) 2023 Brady Johnson <bradyjoh@redhat.com>
+ * Copyright (C) 2021 Antony Antony <antony@phenome.org>
  *
  * This library is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Library General Public License as published by
@@ -22,7 +24,8 @@
 
 const ip_cidr unset_cidr;
 
-ip_cidr cidr_from_raw(where_t where, enum ip_version version,
+ip_cidr cidr_from_raw(where_t where,
+		      const struct ip_info *afi,
 		      const struct ip_bytes bytes,
 		      unsigned prefix_len)
 {
@@ -30,7 +33,7 @@ ip_cidr cidr_from_raw(where_t where, enum ip_version version,
  	/* combine */
 	ip_cidr cidr = {
 		.is_set = true,
-		.version = version,
+		.version = afi->ip_version,
 		.bytes = bytes,
 		.prefix_len = prefix_len,
 	};
@@ -60,7 +63,7 @@ diag_t data_to_cidr(const void *data, size_t sizeof_data, unsigned prefix_len,
 
 	struct ip_bytes bytes = unset_ip_bytes;
 	memcpy(bytes.byte, data, afi->ip_size);
-	*dst = cidr_from_raw(HERE, afi->ip_version, bytes, prefix_len);
+	*dst = cidr_from_raw(HERE, afi, bytes, prefix_len);
 	return NULL;
 }
 
@@ -72,7 +75,7 @@ ip_cidr cidr_from_address(ip_address address)
 	}
 
 	/* contains both routing-prefix and host-identifier */
-	return cidr_from_raw(HERE, address.version, address.bytes,
+	return cidr_from_raw(HERE, afi, address.bytes,
 			     afi->mask_cnt/*32|128*/);
 
 }
@@ -99,28 +102,28 @@ const struct ip_info *cidr_info(const ip_cidr cidr)
 
 ip_address cidr_address(const ip_cidr cidr)
 {
-	const struct ip_info *afi = cidr_type(&cidr);
+	const struct ip_info *afi = cidr_info(cidr);
 	if (afi == NULL) {
 		return unset_address;
 	}
 
-	return address_from_raw(HERE, cidr.version, cidr.bytes);
+	return address_from_raw(HERE, afi, cidr.bytes);
 }
 
 ip_address cidr_prefix(const ip_cidr cidr)
 {
-	const struct ip_info *afi = cidr_type(&cidr);
+	const struct ip_info *afi = cidr_info(cidr);
 	if (afi == NULL) {
 		return unset_address;
 	}
-	return address_from_raw(HERE, cidr.version,
+	return address_from_raw(HERE, afi,
 				ip_bytes_blit(afi, cidr.bytes,
 					      &keep_routing_prefix,
 					      &clear_host_identifier,
 					      cidr.prefix_len));
 }
 
-unsigned cidr_prefix_len(const ip_cidr cidr)
+int cidr_prefix_len(const ip_cidr cidr)
 {
 	return cidr.prefix_len;
 }
@@ -154,50 +157,6 @@ bool cidr_is_specified(const ip_cidr cidr)
 	return cidr_check(cidr) == NULL;
 }
 
-err_t ttocidr_num(shunk_t src, const struct ip_info *afi, ip_cidr *cidr)
-{
-	*cidr = unset_cidr;
-	err_t err;
-
-	/* split CIDR into ADDRESS/MASK. */
-	char slash;
-	shunk_t address = shunk_token(&src, &slash, "/");
-	shunk_t mask = src;
-	if (slash == '\0') {
-		return "missing mask";
-	}
-	if (mask.len == 0) {
-		return "empty mask";
-	}
-
-	/* parse ADDRESS */
-	ip_address addr;
-	err = ttoaddress_num(address, afi/*possibly NULL */,
-				 &addr);
-	if (err != NULL) {
-		return err;
-	}
-	/* Fix AFI, now that it is known */
-	afi = address_type(&addr);
-	passert(afi != NULL);
-
-	/* parse MASK */
-	uintmax_t maskbits = afi->mask_cnt;/*anything*/
-	/* don't use bound - error is confusing */
-	err = shunk_to_uintmax(mask, NULL, 0, &maskbits);
-	if (err != NULL) {
-		/* not a number */
-		return err;
-	}
-	if (maskbits > (uintmax_t)afi->mask_cnt) {
-		return "mask is too big";
-	}
-
-	/* combine */
-	*cidr = cidr_from_raw(HERE, addr.version, addr.bytes, maskbits);
-	return NULL;
-}
-
 shunk_t cidr_as_shunk(const ip_cidr *cidr)
 {
 	const struct ip_info *afi = cidr_type(cidr);
@@ -221,11 +180,8 @@ chunk_t cidr_as_chunk(ip_cidr *cidr)
 
 size_t jam_cidr(struct jambuf *buf, const ip_cidr *cidr)
 {
-	if (cidr == NULL) {
-		return jam_string(buf, "<null-cidr>");
-	}
-
-	if (!cidr->is_set) {
+	const struct ip_info *afi = cidr_type(cidr);
+	if (afi == NULL) {
 		return jam_string(buf, "<unset-cidr>");
 	}
 
