@@ -25,14 +25,15 @@
 
 const ip_endpoint unset_endpoint; /* all zeros */
 
-ip_endpoint endpoint_from_raw(where_t where, enum ip_version version,
+ip_endpoint endpoint_from_raw(where_t where,
+			      const struct ip_info *afi,
 			      const struct ip_bytes bytes,
 			      const struct ip_protocol *protocol,
 			      ip_port port)
 {
 	ip_endpoint endpoint = {
 		.is_set = true,
-		.version = version,
+		.version = afi->ip_version,
 		.bytes = bytes,
 		.hport = port.hport,
 		.ipproto = protocol->ipproto,
@@ -45,24 +46,30 @@ ip_endpoint endpoint_from_address_protocol_port(const ip_address address,
 						const struct ip_protocol *protocol,
 						ip_port port)
 {
-	return endpoint_from_raw(HERE, address.version, address.bytes,
+	const struct ip_info *afi = address_info(address);
+	if (afi == NULL) {
+		/* NULL+unset+unknown */
+		return unset_endpoint; /* empty_address? */
+	}
+
+	return endpoint_from_raw(HERE, afi, address.bytes,
 				 protocol, port);
 }
 
 ip_address endpoint_address(const ip_endpoint endpoint)
 {
-	const struct ip_info *afi = endpoint_type(&endpoint);
+	const struct ip_info *afi = endpoint_info(endpoint);
 	if (afi == NULL) {
 		/* NULL+unset+unknown */
 		return unset_address; /* empty_address? */
 	}
 
-	return address_from_raw(HERE, endpoint.version, endpoint.bytes);
+	return address_from_raw(HERE, afi, endpoint.bytes);
 }
 
 int endpoint_hport(const ip_endpoint endpoint)
 {
-	const struct ip_info *afi = endpoint_type(&endpoint);
+	const struct ip_info *afi = endpoint_info(endpoint);
 	if (afi == NULL) {
 		/* NULL+unset+unknown */
 		/* not asserting, who knows what nonsense a user can generate */
@@ -75,7 +82,7 @@ int endpoint_hport(const ip_endpoint endpoint)
 
 ip_port endpoint_port(const ip_endpoint endpoint)
 {
-	const struct ip_info *afi = endpoint_type(&endpoint);
+	const struct ip_info *afi = endpoint_info(endpoint);
 	if (afi == NULL) {
 		/* NULL+unset+unknown */
 		/* not asserting, who knows what nonsense a user can generate */
@@ -88,7 +95,7 @@ ip_port endpoint_port(const ip_endpoint endpoint)
 
 ip_endpoint set_endpoint_port(const ip_endpoint endpoint, ip_port port)
 {
-	const struct ip_info *afi = endpoint_type(&endpoint);
+	const struct ip_info *afi = endpoint_info(endpoint);
 	if (afi == NULL) {
 		/* includes NULL+unset+unknown */
 		/* not asserting, who knows what nonsense a user can generate */
@@ -140,7 +147,7 @@ const struct ip_protocol *endpoint_protocol(const ip_endpoint endpoint)
 
 bool endpoint_is_specified(const ip_endpoint endpoint)
 {
-	const struct ip_info *afi = endpoint_type(&endpoint);
+	const struct ip_info *afi = endpoint_info(endpoint);
 	if (afi == NULL) {
 		/* NULL+unset+unknown */
 		return false;
@@ -168,16 +175,15 @@ bool endpoint_is_specified(const ip_endpoint endpoint)
 
 size_t jam_endpoint(struct jambuf *buf, const ip_endpoint *endpoint)
 {
-	if (endpoint_is_unset(endpoint)) {
+	const struct ip_info *afi = endpoint_type(endpoint);
+	if (afi == NULL) {
 		return jam_string(buf, "<unset-endpoint>");
 	}
 
-	const struct ip_info *afi = endpoint_type(endpoint);
-	if (afi == NULL) {
-		return jam_string(buf, "<unknown-endpoint>");
-	}
-
-	return afi->jam.address_wrapped_port(buf, afi, &endpoint->bytes, endpoint->hport);
+	size_t s = 0;
+	s += afi->jam.address_wrapped(buf, afi, &endpoint->bytes);
+	s += jam(buf, ":%d", endpoint->hport);
+	return s;
 }
 
 const char *str_endpoint(const ip_endpoint *endpoint, endpoint_buf *dst)
@@ -216,13 +222,9 @@ const char *str_endpoint_sensitive(const ip_endpoint *endpoint, endpoint_buf *ds
 
 size_t jam_endpoint_address_protocol_port(struct jambuf *buf, const ip_endpoint *endpoint)
 {
-	if (endpoint_is_unset(endpoint)) {
-		return jam_string(buf, "<unset-endpoint>");
-	}
-
 	const struct ip_info *afi = endpoint_type(endpoint);
 	if (afi == NULL) {
-		return jam_string(buf, "<unknown-endpoint>");
+		return jam_string(buf, "<unset-endpoint>");
 	}
 
 	size_t s = 0;
