@@ -43,16 +43,6 @@ static size_t jam_ipv4_address(struct jambuf *buf, const struct ip_info *afi, co
 	return s;
 }
 
-static size_t jam_ipv4_address_wrapped_port(struct jambuf *buf, const struct ip_info *afi,
-					    const struct ip_bytes *bytes, unsigned hport)
-{
-	size_t s = 0;
-	/* N.N.N.N:PORT */
-	s += jam_ipv4_address(buf, afi, bytes);
-	s += jam(buf, ":%u", hport);
-	return s;
-}
-
 /*
  * Find longest run of zero pairs that should be suppressed (need at
  * least two).
@@ -122,16 +112,6 @@ static size_t jam_ipv6_address_wrapped(struct jambuf *buf, const struct ip_info
 	return s;
 }
 
-static size_t jam_ipv6_address_wrapped_port(struct jambuf *buf, const struct ip_info *afi,
-					    const struct ip_bytes *bytes, unsigned hport)
-{
-	size_t s = 0;
-	/* [N:..:N]:PORT */
-	s += jam_ipv6_address_wrapped(buf, afi, bytes);
-	s += jam(buf, ":%u", hport);
-	return s;
-}
-
 static ip_address address_from_ipv4_sockaddr(const ip_sockaddr sa)
 {
 	passert(sa.sa.sa.sa_family == AF_INET);
@@ -176,7 +156,6 @@ const struct ip_info ip_families[IP_INDEX_ROOF] = {
 		/* formatting */
 		.jam.address = jam_ipv4_address,
 		.jam.address_wrapped = jam_ipv4_address,
-		.jam.address_wrapped_port = jam_ipv4_address_wrapped_port,
 
 		/* ip_address - .address.any matches grep */
 		.address.unspec = { .is_set = true, .version = IPv4, }, /* 0.0.0.0 */
@@ -188,11 +167,11 @@ const struct ip_info ip_families[IP_INDEX_ROOF] = {
 
 		/* ip_range - .range.any matches grep */
 		.range.zero = { .is_set = true, .version = IPv4, },
-		.range.all = { .is_set = true, .version = IPv4, .end = IPv4_FF, },
+		.range.all = { .is_set = true, .version = IPv4, .hi = IPv4_FF, },
 
 		/* ip_selector - .selector.any matches grep */
-		.selector.zero = { .is_set = true, .version = IPv4, .maskbits = 32, }, /* 0.0.0.0/0 */
-		.selector.all = { .is_set = true, .version = IPv4, .maskbits = 0, }, /* 0.0.0.0/0 */
+		.selector.zero = { .is_set = true, .version = IPv4, }, /* 0.0.0.0/0 */
+		.selector.all = { .is_set = true, .version = IPv4, .hi = IPv4_FF, }, /* 0.0.0.0/0 */
 
 		/* ike */
 		.ikev1_max_fragment_size = ISAKMP_V1_FRAG_MAXLEN_IPv4,
@@ -236,7 +215,6 @@ const struct ip_info ip_families[IP_INDEX_ROOF] = {
 		/* formatting */
 		.jam.address = jam_ipv6_address,
 		.jam.address_wrapped = jam_ipv6_address_wrapped,
-		.jam.address_wrapped_port = jam_ipv6_address_wrapped_port,
 
 		/* ip_address - .address.any matches grep */
 		.address.unspec = { .is_set = true, .version = IPv6, }, /* :: */
@@ -248,11 +226,11 @@ const struct ip_info ip_families[IP_INDEX_ROOF] = {
 
 		/* ip_range - .range.any matches grep */
 		.range.zero = { .is_set = true, .version = IPv6, },
-		.range.all = { .is_set = true, .version = IPv6, .end = IPv6_FF, },
+		.range.all = { .is_set = true, .version = IPv6, .hi = IPv6_FF, },
 
 		/* ip_selector - .selector.any matches grep */
-		.selector.zero = { .is_set = true, .version = IPv6, .maskbits = 128, }, /* ::/0 */
-		.selector.all = { .is_set = true, .version = IPv6, .maskbits = 0, }, /* ::/0 */
+		.selector.zero = { .is_set = true, .version = IPv6, }, /* ::/0 */
+		.selector.all = { .is_set = true, .version = IPv6, .hi = IPv6_FF, }, /* ::/0 */
 
 		/* ike */
 		.ikev1_max_fragment_size = ISAKMP_V1_FRAG_MAXLEN_IPv6,
@@ -301,7 +279,7 @@ const struct ip_info *aftoinfo(int af)
 	}
 }
 
-const struct ip_info *ip_version_info(unsigned version)
+const struct ip_info *ip_version_info(enum ip_version version)
 {
 	static const struct ip_info *ip_types[] = {
 		[0] = NULL,
