@@ -56,8 +56,7 @@ static const char *unset(const ip_packet *packet)
 ip_address packet_src_address(const ip_packet packet)
 {
 	if (packet.is_set) {
-		return address_from_raw(HERE,
-					 packet.info->ip_version,
+		return address_from_raw(HERE, packet.info,
 					 packet.src.bytes);
 	} else {
 		return unset_address;
@@ -67,8 +66,7 @@ ip_address packet_src_address(const ip_packet packet)
 ip_address packet_dst_address(const ip_packet packet)
 {
 	if (packet.is_set) {
-		return address_from_raw(HERE,
-					 packet.info->ip_version,
+		return address_from_raw(HERE, packet.info,
 					 packet.dst.bytes);
 	} else {
 		return unset_address;
@@ -78,8 +76,7 @@ ip_address packet_dst_address(const ip_packet packet)
 ip_endpoint packet_dst_endpoint(const ip_packet packet)
 {
 	if (packet.is_set) {
-		return endpoint_from_raw(HERE,
-					 packet.info->ip_version,
+		return endpoint_from_raw(HERE, packet.info,
 					 packet.dst.bytes,
 					 packet.protocol,
 					 ip_hport(packet.dst.hport));
@@ -92,9 +89,9 @@ ip_selector packet_src_selector(const ip_packet packet)
 {
 	if (packet.is_set) {
 		return selector_from_raw(HERE,
-					 packet.info->ip_version,
+					 packet.info,
+					 packet.src.bytes,
 					 packet.src.bytes,
-					 packet.info->mask_cnt,
 					 packet.protocol,
 					 ip_hport(packet.src.hport));
 	} else {
@@ -106,9 +103,9 @@ ip_selector packet_dst_selector(const ip_packet packet)
 {
 	if (packet.is_set) {
 		return selector_from_raw(HERE,
-					 packet.info->ip_version,
+					 packet.info,
+					 packet.dst.bytes,
 					 packet.dst.bytes,
-					 packet.info->mask_cnt,
 					 packet.protocol,
 					 ip_hport(packet.dst.hport));
 	} else {
@@ -137,11 +134,13 @@ size_t jam_packet(struct jambuf *buf, const ip_packet *packet)
 		 */
 		s += afi->jam.address_wrapped(buf, afi, &packet->src.bytes);
 	} else {
-		s += afi->jam.address_wrapped_port(buf, afi, &packet->src.bytes, packet->src.hport);
+		s += afi->jam.address_wrapped(buf, afi, &packet->src.bytes);
+		s += jam(buf, ":%u", packet->src.hport);
 	}
 	/* DST port is always valid */
 	s += jam(buf, "-%s->", packet->protocol->name);
-	s += afi->jam.address_wrapped_port(buf, afi, &packet->dst.bytes, packet->dst.hport);
+	s += afi->jam.address_wrapped(buf, afi, &packet->dst.bytes);
+	s += jam(buf, ":%u", packet->dst.hport);
 	return s;
 }
 
