@@ -13,7 +13,10 @@
  * License for more details.
  */
 
+
+#include <netdb.h>		/* for getservbyname() */
 #include <arpa/inet.h>		/* for ntohs() */
+#include <stdlib.h>		/* for strtol() */
 
 #include "jambuf.h"
 #include "constants.h"		/* for thingeq() */
