@@ -63,25 +63,25 @@ err_t ttoprotoport(const char *src, ip_protoport *protoport)
 	}
 
 	/* is there a port wildcard? */
-	unsigned port;
+	ip_port port;
 	bool port_wildcard;
 	if (service_name[0] == '\0') {
 		/* allow N/ and N; different to N/%any */
-		port = 0;
+		port = unset_port;
 		port_wildcard = false;
 	} else if (streq(service_name, "%any")) {
 		if (protocol->ipproto == 0) {
 			return "port wildcard (%any) requires a valid protocol";
 		}
-		port = 0;
+		port = unset_port;
 		port_wildcard = true;
 	} else {
 		/* Port 0-65535 is different to %any */
-		err = ttoport(service_name, &port);
+		err = ttoport(shunk1(service_name), &port);
 		if (err != NULL) {
 			return err;
 		}
-		if (protocol->ipproto == 0 && port != 0) {
+		if (protocol->ipproto == 0 && port.hport != 0) {
 			return "protocol 0 must have 0 port";
 		}
 		port_wildcard = false;
@@ -90,7 +90,7 @@ err_t ttoprotoport(const char *src, ip_protoport *protoport)
 	protoport->is_set = true;
 	protoport->ipproto = protocol->ipproto;
 	protoport->has_port_wildcard = port_wildcard;
-	protoport->hport = port;
+	protoport->hport = port.hport;
 	return NULL;
 }
 
@@ -111,7 +111,7 @@ size_t jam_protoport(struct jambuf *buf, const ip_protoport *protoport)
 	}
 
 	size_t s = 0;
-	s += jam(buf, "%s/", protocol_by_ipproto(protoport->ipproto)->name);
+	s += jam(buf, "%s/", protocol_from_ipproto(protoport->ipproto)->name);
 	if (protoport->has_port_wildcard) {
 		pexpect(protoport->hport == 0);
 		s += jam_string(buf, "%any");
