@@ -32,14 +32,15 @@
 
 const ip_range unset_range; /* all zeros */
 
-ip_range range_from_raw(where_t where, enum ip_version version,
-			const struct ip_bytes start, const struct ip_bytes end)
+ip_range range_from_raw(where_t where, const struct ip_info *afi,
+			const struct ip_bytes lo,
+			const struct ip_bytes hi)
 {
 	ip_range r = {
 		.is_set = true,
-		.version = version,
-		.start = start,
-		.end = end,
+		.version = afi->ip_version,
+		.lo = lo,
+		.hi = hi,
 	};
 	pexpect_range(&r, where);
 	return r;
@@ -47,57 +48,39 @@ ip_range range_from_raw(where_t where, enum ip_version version,
 
 /*
  * Calculate the number of significant bits in the size of the range.
- * floor(lg(|high-low| + 1))
+ * floor(lg(|high-low| + 1)); or -1.
  */
 
 int range_prefix_len(const ip_range range)
 {
-	const struct ip_info *afi = range_type(&range);
+	const struct ip_info *afi = range_info(range);
 	if (afi == NULL) {
 		/* NULL+unset+unknown */
 		return -1;
 	}
 
-	struct ip_bytes diff = ip_bytes_sub(afi, range.end, range.start);
-	int fsb = ip_bytes_first_set_bit(afi, diff);
-	return fsb;
+	return ip_bytes_prefix_len(afi, range.lo, range.hi);
 }
 
 int range_host_len(const ip_range range)
 {
-	const struct ip_info *afi = range_type(&range);
+	const struct ip_info *afi = range_info(range);
 	if (afi == NULL) {
 		/* NULL+unset+unknown */
 		return -1;
 	}
 
-	struct ip_bytes diff = ip_bytes_sub(afi, range.end, range.start);
-	int fsb = ip_bytes_first_set_bit(afi, diff);
-	return (afi->ip_size * 8) - fsb;
+	return ip_bytes_host_len(afi, range.lo, range.hi);
 }
 
 size_t jam_range(struct jambuf *buf, const ip_range *range)
 {
-	if (range_is_unset(range)) {
+	const struct ip_info *afi = range_type(range);
+	if (afi == NULL) {
 		return jam_string(buf, "<unset-range>");
 	}
 
-	const struct ip_info *afi = range_type(range);
-	if (afi == NULL) {
-		return jam_string(buf, "<unknown-range>");
-	}
-
-	size_t s = 0;
-	s += afi->jam.address(buf, afi, &range->start);
-	/* when a subnet, try to calculate the prefix-bits */
-	int prefix_bits = (range->is_subnet ? ip_bytes_prefix_len(afi, range->start, range->end) : -1);
-	if (prefix_bits >= 0) {
-		s += jam(buf, "/%d", prefix_bits);
-	} else {
-		s += jam(buf, "-");
-		s += afi->jam.address(buf, afi, &range->end);
-	}
-	return s;
+	return jam_ip_bytes_range(buf, afi, range->lo, range->hi);
 }
 
 const char *str_range(const ip_range *range, range_buf *out)
@@ -109,25 +92,25 @@ const char *str_range(const ip_range *range, range_buf *out)
 
 ip_range range_from_address(const ip_address address)
 {
-	const struct ip_info *afi = address_type(&address);
+	const struct ip_info *afi = address_info(address);
 	if (afi == NULL) {
 		/* NULL+unset+unknown */
 		return unset_range;
 	}
 
-	return range_from_raw(HERE, address.version,
+	return range_from_raw(HERE, afi,
 			      address.bytes, address.bytes);
 }
 
 ip_range range_from_subnet(const ip_subnet subnet)
 {
-	const struct ip_info *afi = subnet_type(&subnet);
+	const struct ip_info *afi = subnet_info(subnet);
 	if (afi == NULL) {
 		/* NULL+unset+unknown */
 		return unset_range;
 	}
 
-	return range_from_raw(HERE, afi->ip_version,
+	return range_from_raw(HERE, afi,
 			      ip_bytes_blit(afi, subnet.bytes,
 					    &keep_routing_prefix,
 					    &clear_host_identifier,
@@ -169,7 +152,7 @@ bool range_is_unset(const ip_range *range)
 
 bool range_is_zero(const ip_range range)
 {
-	const struct ip_info *afi = range_type(&range);
+	const struct ip_info *afi = range_info(range);
 	if (afi == NULL) {
 		return false;
 	}
@@ -179,7 +162,7 @@ bool range_is_zero(const ip_range range)
 
 bool range_is_all(const ip_range range)
 {
-	const struct ip_info *afi = range_type(&range);
+	const struct ip_info *afi = range_info(range);
 	if (afi == NULL) {
 		return false;
 	}
@@ -187,14 +170,24 @@ bool range_is_all(const ip_range range)
 	return range_eq_range(range, afi->range.all);
 }
 
+bool range_is_cidr(ip_range range)
+{
+	const struct ip_info *afi = range_info(range);
+	if (afi == NULL) {
+		return false;
+	}
+
+	return ip_bytes_prefix_len(afi, range.lo, range.hi) >= 0;
+}
+
 uintmax_t range_size(const ip_range range)
 {
-	const struct ip_info *afi = range_type(&range);
+	const struct ip_info *afi = range_info(range);
 	if (afi == NULL) {
 		return 0;
 	}
 
-	struct ip_bytes diff_bytes = ip_bytes_sub(afi, range.end, range.start);
+	struct ip_bytes diff_bytes = ip_bytes_sub(afi, range.hi, range.lo);
 
 	/* more than uintmax_t-bits of host-prefix always overflows. */
 	unsigned prefix_bits = ip_bytes_first_set_bit(afi, diff_bytes);
@@ -238,10 +231,10 @@ bool range_eq_range(const ip_range l, const ip_range r)
 		return false;
 	}
 
-	return (ip_bytes_cmp(l.version, l.start,
-			     r.version, r.start) == 0 &&
-		ip_bytes_cmp(l.version, l.end,
-			     r.version, r.end) == 0);
+	return (ip_bytes_cmp(l.version, l.lo,
+			     r.version, r.lo) == 0 &&
+		ip_bytes_cmp(l.version, l.hi,
+			     r.version, r.hi) == 0);
 }
 
 bool address_in_range(const ip_address address, const ip_range range)
@@ -262,30 +255,30 @@ bool range_in_range(const ip_range inner, const ip_range outer)
 		return false;
 	}
 
-	return (ip_bytes_cmp(inner.version, inner.start,
-			     outer.version, outer.start) >= 0 &&
-		ip_bytes_cmp(inner.version, inner.end,
-			     outer.version, outer.end) <= 0);
+	return (ip_bytes_cmp(inner.version, inner.lo,
+			     outer.version, outer.lo) >= 0 &&
+		ip_bytes_cmp(inner.version, inner.hi,
+			     outer.version, outer.hi) <= 0);
 }
 
 ip_address range_start(const ip_range range)
 {
-	const struct ip_info *afi = range_type(&range);
+	const struct ip_info *afi = range_info(range);
 	if (afi == NULL) {
 		return unset_address;
 	}
 
-	return address_from_raw(HERE, range.version, range.start);
+	return address_from_raw(HERE, afi, range.lo);
 }
 
 ip_address range_end(const ip_range range)
 {
-	const struct ip_info *afi = range_type(&range);
+	const struct ip_info *afi = range_info(range);
 	if (afi == NULL) {
 		return unset_address;
 	}
 
-	return address_from_raw(HERE, range.version, range.end);
+	return address_from_raw(HERE, afi, range.hi);
 }
 
 bool range_overlaps_range(const ip_range l, const ip_range r)
@@ -296,54 +289,57 @@ bool range_overlaps_range(const ip_range l, const ip_range r)
 	}
 
 	/* l before r */
-	if (ip_bytes_cmp(l.version, l.end,
-			 r.version, r.start) < 0) {
+	if (ip_bytes_cmp(l.version, l.hi,
+			 r.version, r.lo) < 0) {
 		return false;
 	}
 	/* l after r */
-	if (ip_bytes_cmp(l.version, l.start,
-			 r.version, r.end) > 0) {
+	if (ip_bytes_cmp(l.version, l.lo,
+			 r.version, r.hi) > 0) {
 		return false;
 	}
 
 	return true;
 }
 
-err_t addresses_to_nonzero_range(const ip_address start, const ip_address end, ip_range *dst)
+err_t addresses_to_nonzero_range(const ip_address lo, const ip_address hi, ip_range *dst)
 {
 	*dst = unset_range;
 
-	if (address_is_unset(&start)) {
+	const struct ip_info *lo_afi = address_info(lo);
+	if (lo_afi == NULL) {
 		/* NULL+unset+unknown */
 		return "start address invalid";
 	}
 
-	if (address_is_unset(&end)) {
+	const struct ip_info *hi_afi = address_info(hi);
+	if (hi_afi == NULL) {
 		/* NULL+unset+unknown */
 		return "end address invalid";
 	}
 
-	if (start.version != end.version) {
+	if (lo_afi != hi_afi) {
 		return "conflicting address types";
 	}
 
 	/* reject both 0 */
-	if (thingeq(start.bytes, unset_ip_bytes) && thingeq(end.bytes, unset_ip_bytes)) {
+	if (thingeq(lo.bytes, unset_ip_bytes) &&
+	    thingeq(hi.bytes, unset_ip_bytes)) {
 		return "zero address range";
 	}
 
-	if (addrcmp(&start, &end) > 0) {
+	if (addrcmp(&lo, &hi) > 0) {
 		return "out-of-order";
 	}
 
-	*dst = range_from_raw(HERE, start.version, start.bytes, end.bytes);
+	*dst = range_from_raw(HERE, lo_afi, lo.bytes, hi.bytes);
 	return NULL;
 }
 
 err_t range_to_subnet(const ip_range range, ip_subnet *dst)
 {
 	*dst = unset_subnet;
-	const struct ip_info *afi = range_type(&range);
+	const struct ip_info *afi = range_info(range);
 	if (afi == NULL) {
 		return "invalid range";
 	}
@@ -353,12 +349,12 @@ err_t range_to_subnet(const ip_range range, ip_subnet *dst)
 	 * matching leading bits of FROM and TO.  Trailing bits
 	 * (subnet address) must be either all 0 (from) or 1 (to).
 	 */
-	int prefix_bits = ip_bytes_prefix_len(afi, range.start, range.end);
+	int prefix_bits = ip_bytes_prefix_len(afi, range.lo, range.hi);
 	if (prefix_bits < 0) {
 		return "address range is not a subnet";
 	}
 
-	*dst = subnet_from_raw(HERE, afi->ip_version, range.start, prefix_bits);
+	*dst = subnet_from_raw(HERE, afi, range.lo, prefix_bits);
 	return NULL;
 }
 
@@ -366,7 +362,7 @@ err_t range_offset_to_address(const ip_range range, uintmax_t offset, ip_address
 {
 	*address = unset_address;
 
-	const struct ip_info *afi = range_type(&range);
+	const struct ip_info *afi = range_info(range);
 	if (afi == NULL) {
 		return "invalid range";
 	}
@@ -378,7 +374,7 @@ err_t range_offset_to_address(const ip_range range, uintmax_t offset, ip_address
 		unsigned add = offset & 0xff;
 		offset >>= 8;
 		/* update */
-		unsigned val = range.start.byte[j] + add + carry;
+		unsigned val = range.lo.byte[j] + add + carry;
 		carry = val > 0xff;
 		sum.byte[j] = val; /* truncates */
 	}
@@ -391,7 +387,7 @@ err_t range_offset_to_address(const ip_range range, uintmax_t offset, ip_address
 		return "address overflow";
 	}
 
-	ip_address tmp = address_from_raw(HERE, range.version, sum);
+	ip_address tmp = address_from_raw(HERE, afi, sum);
 	if (!address_in_range(tmp, range)) {
 		return "range overflow";
 	}
@@ -404,12 +400,12 @@ err_t address_to_range_offset(const ip_range range, const ip_address address, ui
 {
 	*offset = UINTMAX_MAX;
 
-	const struct ip_info *afi = range_type(&range);
+	const struct ip_info *afi = range_info(range);
 	if (afi == NULL) {
 		return "range invalid";
 	}
 
-	if (address_type(&address) != afi) {
+	if (address_info(address) != afi) {
 		return "address is not from range";
 	}
 
@@ -417,7 +413,7 @@ err_t address_to_range_offset(const ip_range range, const ip_address address, ui
 		return "address out-of-bounds";
 	}
 
-	struct ip_bytes diff = ip_bytes_sub(afi, address.bytes, range.start);
+	struct ip_bytes diff = ip_bytes_sub(afi, address.bytes, range.lo);
 
 	*offset = raw_ntoh(diff.byte, afi->ip_size);
 
@@ -441,7 +437,7 @@ void pexpect_range(const ip_range *r, where_t where)
 
 	if (r->is_set == false ||
 	    r->version == 0 ||
-	    ip_bytes_cmp(r->version, r->start, r->version, r->end) > 0) {
+	    ip_bytes_cmp(r->version, r->lo, r->version, r->hi) > 0) {
 		llog_pexpect(&global_logger, where, "invalid range: "PRI_RANGE, pri_range(r));
 	}
 }
