@@ -25,12 +25,14 @@
 
 const ip_subnet unset_subnet; /* all zeros */
 
-ip_subnet subnet_from_raw(where_t where, enum ip_version version,
-			  const struct ip_bytes bytes, unsigned prefix_len)
+ip_subnet subnet_from_raw(where_t where,
+			  const struct ip_info *afi,
+			  const struct ip_bytes bytes,
+			  unsigned prefix_len)
 {
 	ip_subnet s = {
 		.is_set = true,
-		.version = version,
+		.version = afi->ip_version,
 		.bytes = bytes,
 		.maskbits = prefix_len,
 	};
@@ -45,7 +47,7 @@ ip_subnet subnet_from_address(const ip_address address)
 		return unset_subnet;
 	}
 
-	return subnet_from_raw(HERE, address.version, address.bytes, afi->mask_cnt);
+	return subnet_from_raw(HERE, afi, address.bytes, afi->mask_cnt);
 }
 
 ip_subnet subnet_from_cidr(const ip_cidr cidr)
@@ -55,7 +57,7 @@ ip_subnet subnet_from_cidr(const ip_cidr cidr)
 		return unset_subnet;
 	}
 
-	return subnet_from_raw(HERE, cidr.version,
+	return subnet_from_raw(HERE, afi,
 			       ip_bytes_blit(afi, cidr.bytes,
 					     &keep_routing_prefix,
 					     &clear_host_identifier,
@@ -68,31 +70,31 @@ err_t address_mask_to_subnet(const ip_address address,
 			     ip_subnet *subnet)
 {
 	*subnet = unset_subnet;
-	const struct ip_info *afi = address_type(&address);
+	const struct ip_info *afi = address_info(address);
 	if (afi == NULL) {
 		return "invalid address";
 	}
 
-	if (address_type(&mask) != afi) {
+	if (address_info(mask) != afi) {
 		return "invalid mask";
 	}
 
-	int prefix_bits =  masktocount(&mask);
-	if (prefix_bits < 0) {
+	int prefix_len =  ip_bytes_mask_len(afi, mask.bytes);
+	if (prefix_len < 0) {
 		return "invalid mask";
 	}
 
 	struct ip_bytes prefix = ip_bytes_blit(afi, address.bytes,
 					       &keep_routing_prefix,
 					       &clear_host_identifier,
-					       prefix_bits);
-	*subnet = subnet_from_raw(HERE, afi->ip_version, prefix, prefix_bits);
+					       prefix_len);
+	*subnet = subnet_from_raw(HERE, afi, prefix, prefix_len);
 	return NULL;
 }
 
 ip_address subnet_prefix(const ip_subnet subnet)
 {
-	const struct ip_info *afi = subnet_type(&subnet);
+	const struct ip_info *afi = subnet_info(subnet);
 	if (afi == NULL) {
 		/* NULL+unset+unknown */
 		return unset_address;
@@ -102,7 +104,7 @@ ip_address subnet_prefix(const ip_subnet subnet)
 					       &keep_routing_prefix,
 					       &clear_host_identifier,
 					       subnet.maskbits);
-	return address_from_raw(HERE, afi->ip_version, prefix);
+	return address_from_raw(HERE, afi, prefix);
 }
 
 const struct ip_info *subnet_type(const ip_subnet *subnet)
@@ -136,7 +138,7 @@ bool subnet_is_unset(const ip_subnet *subnet)
 
 bool subnet_is_zero(const ip_subnet subnet)
 {
-	const struct ip_info *afi = subnet_type(&subnet);
+	const struct ip_info *afi = subnet_info(subnet);
 	if (afi == NULL) {
 		/* NULL+unset+unknown */
 		return false;
@@ -147,7 +149,7 @@ bool subnet_is_zero(const ip_subnet subnet)
 
 bool subnet_is_all(const ip_subnet subnet)
 {
-	const struct ip_info *afi = subnet_type(&subnet);
+	const struct ip_info *afi = subnet_info(subnet);
 	if (afi == NULL) {
 		/* NULL+unset+unknown */
 		return false;
@@ -170,7 +172,7 @@ uintmax_t subnet_size(const ip_subnet subnet)
 
 ip_address subnet_prefix_mask(const ip_subnet subnet)
 {
-	const struct ip_info *afi = subnet_type(&subnet);
+	const struct ip_info *afi = subnet_info(subnet);
 	if (afi == NULL) {
 		/* NULL+unset+unknown */
 		return unset_address;
@@ -180,7 +182,7 @@ ip_address subnet_prefix_mask(const ip_subnet subnet)
 					     &set_routing_prefix,
 					     &clear_host_identifier,
 					     subnet.maskbits);
-	return address_from_raw(HERE, afi->ip_version, mask);
+	return address_from_raw(HERE, afi, mask);
 }
 
 unsigned subnet_prefix_bits(const ip_subnet subnet)
@@ -190,13 +192,9 @@ unsigned subnet_prefix_bits(const ip_subnet subnet)
 
 size_t jam_subnet(struct jambuf *buf, const ip_subnet *subnet)
 {
-	if (subnet_is_unset(subnet)) {
-		return jam_string(buf, "<unset-subnet>");
-	}
-
 	const struct ip_info *afi = subnet_type(subnet);
 	if (afi == NULL) {
-		return jam_string(buf, "<unknown-subnet>");
+		return jam_string(buf, "<unset-subnet>");
 	}
 
 	size_t s = 0;
@@ -261,7 +259,7 @@ bool subnet_eq_subnet(const ip_subnet l, const ip_subnet r)
 
 bool subnet_eq_address(const ip_subnet subnet, const ip_address address)
 {
-	const struct ip_info *afi = subnet_type(&subnet);
+	const struct ip_info *afi = subnet_info(subnet);
 	if (afi == NULL) {
 		return false;
 	}
@@ -275,12 +273,12 @@ bool subnet_eq_address(const ip_subnet subnet, const ip_address address)
 
 bool subnet_in_subnet(const ip_subnet l, const ip_subnet r)
 {
-	const struct ip_info *afi = subnet_type(&l);
+	const struct ip_info *afi = subnet_info(l);
 	if (afi == NULL) {
 		return false;
 	}
 
-	if (subnet_type(&r) != afi) {
+	if (subnet_info(r) != afi) {
 		return false;
 	}
 
@@ -300,12 +298,12 @@ bool subnet_in_subnet(const ip_subnet l, const ip_subnet r)
 
 bool address_in_subnet(const ip_address l, const ip_subnet r)
 {
-	const struct ip_info *afi = address_type(&l);
+	const struct ip_info *afi = address_info(l);
 	if (afi == NULL) {
 		return false;
 	}
 
-	if (subnet_type(&r) != afi) {
+	if (subnet_info(r) != afi) {
 		return false;
 	}
 
