@@ -17,6 +17,7 @@
 #include "chunk.h"
 #include "ttodata.h"	/* for datatot() */
 #include "passert.h"
+#include "lswalloc.h"
 
 void llog_base64_bytes(lset_t rc_flags,
 		       const struct logger *logger,
@@ -26,17 +27,19 @@ void llog_base64_bytes(lset_t rc_flags,
 	 * A byte is 8-bits, base64 uses 6-bits (2^6=64).  Plus some
 	 * for \0.  Plus some extra for the trailing === and rounding.
 	 */
-	chunk_t base64 = alloc_chunk(size * 8 / 6 + 1 + 10, "base64");
-	size_t base64_size = datatot(ptr, size, 64, (void*)base64.ptr, base64.len);
+	size_t base64_len = size * 8 / 6 + 1 + 10;
+	char *base64_ptr = alloc_things(char, base64_len, "base64");
+	/* size includes NUL */
+	size_t base64_size = datatot(ptr, size, 64, base64_ptr, base64_len);
 	/* BASE64_SIZE includes '\0' */
-	passert(base64_size <= base64.len);
+	passert(base64_size <= base64_len);
 	passert(base64_size > 0);
-	shunk_t rest = shunk2(base64.ptr, base64_size - 1);
+	shunk_t rest = shunk2(base64_ptr, base64_size - 1 /*drop NUL*/);
 	while (true) {
-		shunk_t line = shunk_slice(rest, 0, min((size_t)64, rest.len));
+		shunk_t line = hunk_slice(rest, 0, min((size_t)64, rest.len));
 		if (line.len == 0) break;
-		rest = shunk_slice(rest, line.len, rest.len);
+		rest = hunk_slice(rest, line.len, rest.len);
 		llog(rc_flags, logger, PRI_SHUNK, pri_shunk(line));
 	}
-	free_chunk_content(&base64);
+	pfree(base64_ptr);
 }
