@@ -29,8 +29,8 @@ bool test_lset(const struct enum_names *en, lset_t val)
 	for (unsigned e = 0; val != 0; e++) {
 		lset_t bit = LELEM(e);
 		if (val & bit) {
-			const char *n = enum_name(en, e);
-			if (n == NULL) {
+			enum_buf b;
+			if (!enum_name(en, e, &b)) {
 				return false;
 			}
 		}
@@ -102,7 +102,7 @@ const char *str_lset_short(enum_names *en, const char *separator,
 	return out->buf;
 }
 
-size_t jam_sparse_lset(struct jambuf *buf, const struct sparse_name *sd, lset_t val)
+size_t jam_sparse_lset(struct jambuf *buf, const struct sparse_names *sd, lset_t val)
 {
 	if (val == LEMPTY) {
 		return jam(buf, "none");
@@ -116,12 +116,12 @@ size_t jam_sparse_lset(struct jambuf *buf, const struct sparse_name *sd, lset_t
 			s += jam_string(buf, sep);
 			sep = LSET_SEPARATOR;
 			/* can return NULL */
-			const char *name = sparse_name(sd, bit);
-			if (name == NULL) {
+			name_buf nb;
+			if (sparse_short(sd, bit, &nb)) {
+				s += jam_string(buf, nb.buf);
+			} else {
 				/* No name for this bit, use hex. */
 				s += jam(buf, "0x" PRI_LSET, bit);
-			} else {
-				s += jam_string(buf, name);
 			}
 		}
 		val &= ~bit;
