@@ -24,14 +24,13 @@
 #include "lswnss.h"
 #include "lswalloc.h"
 #include "lswlog.h"
-#include "lswfips.h"
+#include "fips_mode.h"
 
 static char *lsw_nss_get_password(PK11SlotInfo *slot, PRBool retry, void *arg);
 
-static unsigned flags;
+static struct nss_flags shutdown_flags;
 
-diag_t lsw_nss_setup(const char *configdir, unsigned setup_flags,
-		     struct logger *logger)
+static diag_t setup(const char *configdir, struct nss_flags flags, struct logger *logger)
 {
 	/*
 	 * Turn (possibly NULL) CONFIGDIR into (possibly NULL) nssdir
@@ -54,14 +53,14 @@ diag_t lsw_nss_setup(const char *configdir, unsigned setup_flags,
 		llog(RC_LOG, logger, "Initializing NSS");
 	} else {
 		llog(RC_LOG, logger, "Initializing NSS using %s database \"%s\"",
-			    (flags & LSW_NSS_READONLY) ? "read-only" : "read-write",
-			    nssdir);
+		     (flags.open_readonly ? "read-only" : "read-write"),
+		     nssdir);
 	}
 
 	/*
 	 * save for cleanup
 	 */
-	flags = setup_flags;
+	shutdown_flags = flags;
 
 	/*
 	 * According to the manual, not needed, and all parameters are
@@ -72,23 +71,29 @@ diag_t lsw_nss_setup(const char *configdir, unsigned setup_flags,
 	/*
 	 * Initialize NSS, possibly flipping it to the correct mode.
 	 */
-	enum lsw_fips_mode fips_mode;
+	enum fips_mode fips_mode;
 	if (nssdir != NULL) {
 		SECStatus rv = NSS_Initialize(nssdir, "", "", SECMOD_DB,
-					      (flags & LSW_NSS_READONLY) ? NSS_INIT_READONLY : 0);
+					      (flags.open_readonly ? NSS_INIT_READONLY : 0));
 		if (rv != SECSuccess) {
 			/* NSS: <message...>: <error-string> (SECERR: N) */
 			diag_t d = diag_nss_error("initialization using %s database \"%s\" failed",
-						  (flags & LSW_NSS_READONLY) ? "read-only" : "read-write",
+						  (flags.open_readonly ? "read-only" : "read-write"),
 						  nssdir);
 			pfree(nssdir);
 			return d;
 		}
-		fips_mode = lsw_get_fips_mode(logger);
+		fips_mode = get_fips_mode(logger);
 	} else {
 		NSS_NoDB_Init(".");
-		fips_mode = lsw_get_fips_mode(logger);
-		if (fips_mode == LSW_FIPS_ON && !PK11_IsFIPS()) {
+		fips_mode = get_fips_mode(logger);
+		if (fips_mode == FIPS_MODE_ON && !PK11_IsFIPS()) {
+			/*
+			 * Happens when set_fips_mode(FIPS_MODE_ON) is
+			 * called before calling this function.  For
+			 * instance, in algparse.  Need to flip NSS's
+			 * mode so that it matches.
+			 */
 			SECMODModule *internal = SECMOD_GetInternalModule();
 			if (internal == NULL) {
 				return diag_nss_error("SECMOD_GetInternalModule() failed");
@@ -103,7 +108,7 @@ diag_t lsw_nss_setup(const char *configdir, unsigned setup_flags,
 		}
 	}
 
-	if (fips_mode == LSW_FIPS_UNKNOWN) {
+	if (fips_mode == FIPS_MODE_UNSET) {
 		pfreeany(nssdir);
 		return diag("NSS: FIPS mode could not be determined");
 	}
@@ -132,11 +137,22 @@ diag_t lsw_nss_setup(const char *configdir, unsigned setup_flags,
 	return NULL;
 }
 
-void lsw_nss_shutdown(void)
+void init_nss(const char *configdir, struct nss_flags flags, struct logger *logger)
+{
+	diag_t d = setup(configdir, flags, logger);
+	if (d != NULL) {
+		LLOG_FATAL_JAMBUF(PLUTO_EXIT_FAIL, logger, buf) {
+			jam_string(buf, str_diag(d));
+			pfree_diag(&d);
+		}
+	}
+}
+
+void shutdown_nss(void)
 {
 	NSS_Shutdown();
 	/* this flag is never set anywhere */
-	if (!(flags & LSW_NSS_SKIP_PR_CLEANUP)) {
+	if (!shutdown_flags.skip_pr_cleanup) {
 		PR_Cleanup();
 	}
 }
@@ -145,7 +161,7 @@ PK11SlotInfo *lsw_nss_get_authenticated_slot(struct logger *logger)
 {
 	PK11SlotInfo *slot = PK11_GetInternalKeySlot(); /* refcnt_add() */
 	if (slot == NULL) {
-		llog(RC_LOG_SERIOUS|ERROR_STREAM, logger,
+		llog(ERROR_STREAM, logger,
 		     "no internal key slot");
 		return NULL;
 	}
@@ -155,7 +171,7 @@ PK11SlotInfo *lsw_nss_get_authenticated_slot(struct logger *logger)
 						     lsw_nss_get_password_context(logger));
 		if (status != SECSuccess) {
 			const char *token = PK11_GetTokenName(slot);
-			llog(RC_LOG_SERIOUS|ERROR_STREAM, logger,
+			llog(ERROR_STREAM, logger,
 			     "authentication of \"%s\" failed", token);
 			PK11_FreeSlot(slot); /* refcnt_del() */
 			return NULL;
@@ -201,7 +217,7 @@ static char *lsw_nss_get_password(PK11SlotInfo *slot, PRBool retry, void *arg)
 	const struct lsw_conf_options *oco = lsw_init_options();
 
 	/*
-	 * Easy case, passsword specified on the command line.
+	 * Easy case, password specified on the command line.
 	 */
 	if (oco->nsspassword != NULL) {
 		char *password = PORT_Strdup(oco->nsspassword);
