@@ -14,34 +14,26 @@
  * for more details.
  */
 
+#include <stdlib.h>		/* for abort() */
+
 #include "passert.h"
 #include "lswlog.h"
 
-void lsw_passert_fail(where_t where, const char *fmt, ...)
+void llog_passert(const struct logger *logger, where_t where, const char *fmt, ...)
 {
-	JAMBUF(buf) {
-		jam_string(buf, "ABORT: ASSERTION FAILED: ");
+	struct logjam logjam;
+	struct jambuf *buf = jambuf_from_logjam(&logjam, logger, 0, where, PASSERT_FLAGS);
+	{
 		va_list ap;
 		va_start(ap, fmt);
 		jam_va_list(buf, fmt, ap);
 		va_end(ap);
-		jam(buf, " "PRI_WHERE, pri_where(where));
-		jambuf_to_error_stream(buf); /* XXX: grrr */
 	}
-	abort();
+	passert_logjam_to_logger(&logjam);
 }
 
-void passert_fail(struct logger *logger, where_t where, const char *fmt, ...)
+void passert_logjam_to_logger(struct logjam *logjam)
 {
-	JAMBUF(buf) {
-		jam_string(buf, "ABORT: ASSERTION FAILED: ");
-		jam_logger_prefix(buf, logger);
-		va_list ap;
-		va_start(ap, fmt);
-		jam_va_list(buf, fmt, ap);
-		va_end(ap);
-		jam(buf, " "PRI_WHERE, pri_where(where));
-		jambuf_to_logger(buf, logger, ERROR_FLAGS);
-	}
+	logjam_to_logger(logjam);
 	abort();
 }
