@@ -19,29 +19,15 @@
 
 #include "lswlog.h"
 
-void log_pexpect(where_t where, const char *message, ...)
+void llog_pexpect(const struct logger *logger, where_t where, const char *message, ...)
 {
-	JAMBUF(buf) {
-		jam_string(buf, "EXPECTATION FAILED: ");
-		va_list args;
-		va_start(args, message);
-		jam_va_list(buf, message, args);
-		va_end(args);
-		jam(buf, " "PRI_WHERE, pri_where(where));
-		jambuf_to_error_stream(buf); /* XXX: grrr */
-	}
-}
-
-void pexpect_fail(struct logger *logger, where_t where, const char *message, ...)
-{
-	JAMBUF(buf) {
-		jam_string(buf, "EXPECTATION FAILED: ");
-		jam_logger_prefix(buf, logger);
+	struct logjam logjam;
+	struct jambuf *buf = jambuf_from_logjam(&logjam, logger, 0, where, PEXPECT_FLAGS);
+	{
 		va_list ap;
 		va_start(ap, message);
 		jam_va_list(buf, message, ap);
 		va_end(ap);
-		jam(buf, " "PRI_WHERE, pri_where(where));
-		jambuf_to_logger(buf, logger, ERROR_FLAGS);
 	}
+	logjam_to_logger(&logjam);
 }
