@@ -333,8 +333,9 @@ void append_algorithm(struct proposal_parser *parser,
 		      const struct ike_alg *alg,
 		      int enckeylen)
 {
+	struct logger *logger = parser->policy->logger;
 	if (alg == NULL) {
-		DBGF(DBG_PROPOSAL_PARSER, "no algorithm to append");
+		ldbgf(DBG_PROPOSAL_PARSER, logger, "no algorithm to append");
 		return;
 	}
 	enum proposal_algorithm algorithm = ike_to_proposal_algorithm(alg);
@@ -349,9 +350,9 @@ void append_algorithm(struct proposal_parser *parser,
 		.desc = alg,
 		.enckeylen = enckeylen,
 	};
-	DBGF(DBG_PROPOSAL_PARSER, "appending %s %s algorithm %s[_%d]",
-	     parser->protocol->name, ike_alg_type_name(alg->algo_type), alg->fqn,
-	     enckeylen);
+	ldbgf(DBG_PROPOSAL_PARSER, logger, "appending %s %s algorithm %s[_%d]",
+	      parser->protocol->name, ike_alg_type_name(alg->algo_type), alg->fqn,
+	      enckeylen);
 	*end = clone_thing(new_algorithm, "alg");
 }
 
@@ -605,33 +606,32 @@ bool impair_proposal_errors(struct proposal_parser *parser)
 {
 	passert(parser->diag != NULL);
 	if (impair.proposal_parser) {
-		llog_diag(parser->policy->logger_rc_flags,
-			  parser->policy->logger,
-			  &parser->diag,
-			  "IMPAIR: ignoring proposal error: ");
+		llog(parser->policy->logger_rc_flags, parser->policy->logger,
+		     "IMPAIR: ignoring proposal error: %s", str_diag(parser->diag));
+		pfree_diag(&parser->diag);
 		return true;
-	} else {
-		return false;
 	}
+
+	return false;
 }
 
 struct proposals *proposals_from_str(struct proposal_parser *parser,
 				     const char *str)
 {
 	struct proposals *proposals = alloc_thing(struct proposals, "proposals");
-	unsigned parser_version = parser->policy->parser_version;
-	if (parser_version == 0) {
-		parser_version = parser->policy->version;
-	}
 	if (str == NULL) {
 		proposals->defaulted = true;
 		/* may still be null */
-		str = parser->protocol->defaults[parser->policy->version]->proposals;
+		enum fips_mode fips_mode = get_fips_mode(parser->policy->logger);
+		str = parser->protocol->defaults->proposals[fips_mode];
+		PASSERT(parser->policy->logger, str != NULL);
 	}
-	bool ok;
-	switch (parser_version) {
-	case 2: ok = v2_proposals_parse_str(parser, proposals, shunk1(str)); break;
-	default: ok = v1_proposals_parse_str(parser, proposals, shunk1(str)); break;
+	bool ok = false;
+	switch (parser->policy->version) {
+	case IKEv1: ok = v1_proposals_parse_str(parser, proposals, shunk1(str)); break;
+	case IKEv2: ok = v2_proposals_parse_str(parser, proposals, shunk1(str)); break;
+	default:
+		bad_case(parser->policy->version);
 	}
 	if (!ok) {
 		free_proposals(&proposals);
@@ -691,6 +691,7 @@ bool proposal_parse_encrypt(struct proposal_parser *parser,
 			    const struct ike_alg **encrypt,
 			    int *encrypt_keylen)
 {
+	struct logger *logger = parser->policy->logger;
 	if (tokens->this.len == 0) {
 		proposal_error(parser, "%s encryption algorithm is empty",
 			       parser->protocol->name);
@@ -720,9 +721,9 @@ bool proposal_parse_encrypt(struct proposal_parser *parser,
 		const struct ike_alg *alg = encrypt_alg_byname(parser, ealg,
 							       enckeylen, print);
 		if (alg == NULL) {
-			DBGF(DBG_PROPOSAL_PARSER,
-			     "<ealg>byname('"PRI_SHUNK"') with <eklen>='"PRI_SHUNK"' failed: %s",
-			     pri_shunk(ealg), pri_shunk(eklen), str_diag(parser->diag));
+			ldbgf(DBG_PROPOSAL_PARSER, logger,
+			      "<ealg>byname('"PRI_SHUNK"') with <eklen>='"PRI_SHUNK"' failed: %s",
+			      pri_shunk(ealg), pri_shunk(eklen), str_diag(parser->diag));
 			return false;
 		}
 		/* consume <ealg>-<eklen> */
@@ -778,7 +779,7 @@ bool proposal_parse_encrypt(struct proposal_parser *parser,
 	 * like aes_gcm_16, above lookup should have found the
 	 * algorithm so isn't a problem here.
 	 */
-	shunk_t eklen = shunk_slice(ealg, end, ealg.len);
+	shunk_t eklen = hunk_slice(ealg, end, ealg.len);
 	int enckeylen = parse_proposal_eklen(parser, print, eklen);
 	if (enckeylen <= 0) {
 		passert(parser->diag != NULL);
@@ -789,9 +790,9 @@ bool proposal_parse_encrypt(struct proposal_parser *parser,
 	 * The <eklen> in <ealg><eklen> or <ealg>_<eklen> parsed; trim
 	 * <eklen> from <ealg> and then try the lookup.
 	 */
-	ealg = shunk_slice(ealg, 0, end);
-	if (hunk_char_ischar(ealg, ealg.len-1, "_")) {
-		ealg = shunk_slice(ealg, 0, end-1);
+	ealg = hunk_slice(ealg, 0, end);
+	if (hunk_char(ealg, ealg.len-1) == '_') {
+		ealg = hunk_slice(ealg, 0, end-1);
 	}
 	pfree_diag(&parser->diag); /* zap old error */
 	alg = encrypt_alg_byname(parser, ealg, enckeylen, print);
@@ -829,7 +830,7 @@ void proposal_next_token(struct proposal_tokenizer *tokens)
 	tokens->this = tokens->next;
 	tokens->next = shunk_token(&tokens->input, &tokens->next_term, tokens->delims);
 	if (DBGP(DBG_PROPOSAL_PARSER)) {
-		JAMBUF(buf) {
+		LLOG_JAMBUF(DEBUG_STREAM, &global_logger, buf) {
 			jam(buf, "token: ");
 			if (tokens->prev_term != '\0') {
 				jam(buf, "'%c'", tokens->prev_term);
@@ -860,7 +861,6 @@ void proposal_next_token(struct proposal_tokenizer *tokens)
 			} else {
 				jam(buf, "''");
 			}
-			jambuf_to_logger(buf, &global_logger, DEBUG_STREAM);
 		}
 	}
 }
