@@ -14,9 +14,9 @@
  * for more details.
  */
 
-#include <lswalloc.h>
-#include "nss.h"
-#include "secrets.h"
+#include "lswalloc.h"
+#include "jambuf.h"
+#include "lswnss.h"
 
 SECItem same_chunk_as_secitem(chunk_t chunk, SECItemType type)
 {
@@ -28,13 +28,24 @@ SECItem same_chunk_as_secitem(chunk_t chunk, SECItemType type)
 	return si;
 }
 
+SECItem same_shunk_as_secitem(shunk_t shunk, SECItemType type)
+{
+	SECItem si = {
+		.type = type,
+		.data = DISCARD_CONST(uint8_t *, shunk.ptr),
+		.len = shunk.len,
+	};
+	return si;
+}
+
 chunk_t same_secitem_as_chunk(SECItem si)
 {
-	chunk_t chunk = {
-		.ptr = si.data,
-		.len = si.len,
-	};
-	return chunk;
+	return chunk2(si.data, si.len);
+}
+
+shunk_t same_secitem_as_shunk(SECItem si)
+{
+	return shunk2(si.data, si.len);
 }
 
 chunk_t clone_secitem_as_chunk(SECItem si, const char *name)
@@ -45,15 +56,3 @@ chunk_t clone_secitem_as_chunk(SECItem si, const char *name)
 	};
 	return chunk;
 }
-
-#if 0	/* not used (yet?) */
-SECItem clone_chunk_as_secitem(chunk_t chunk, SECItemType type, const char *name)
-{
-	SECItem si = {
-		.type = type,
-		.len = chunk.len,
-		.data = clone_bytes(chunk.ptr, chunk.len, name),
-	};
-	return si;
-}
-#endif
