@@ -63,38 +63,6 @@
 #include "ike_alg_hash.h"
 #include "certs.h"
 
-/*
- * Build up a diagnostic in a static buffer -- NOT RE-ENTRANT.
- *
- * Although this would be a generally useful function, it is very
- * hard to come up with a discipline that prevents different uses
- * from interfering.  It is intended that by limiting it to building
- * diagnostics, we will avoid this problem.
- * Juggling is performed to allow an argument to be a previous
- * result: the new string may safely depend on the old one.  This
- * restriction is not checked in any way: violators will produce
- * confusing results (without crashing!).
- *
- * @param fmt String format
- * @param ... strings
- * @return err_t
- */
-
-static err_t builddiag(const char *fmt, ...) PRINTF_LIKE(1);	/* NOT RE-ENTRANT */
-static err_t builddiag(const char *fmt, ...)
-{
-	/* longer messages will be truncated */
-	static char mydiag_space[LOG_WIDTH];
-	char t[sizeof(mydiag_space)];	/* build result here first */
-	va_list args;
-	va_start(args, fmt);
-	t[0] = '\0';	/* in case nothing terminates string */
-	vsnprintf(t, sizeof(t), fmt, args);
-	va_end(args);
-	strcpy(mydiag_space, t);
-	return mydiag_space;
-}
-
 struct fld {
 	const char *name;
 	ssize_t offset;
@@ -104,14 +72,59 @@ static void process_secrets_file(struct file_lex_position *flp,
 				 struct secret **psecrets, const char *file_pat);
 
 struct secret {
-	struct secret *next;
+	enum secret_kind kind;
+	/*
+	 * The ipsec.secrets line number, but which ipsec.secrets
+	 * (include means more than one).
+	 *
+	 * For NSS, this is the entry number.
+	 */
+	int line;
+	/*
+	 * List of IDs, is this part of pubkey stuff?
+	 */
 	struct id_list *ids;
-	struct secret_stuff stuff;
+	union /*kind*/ {
+		struct secret_preshared_stuff *preshared;
+		struct secret_ppk_stuff *ppk;
+		struct secret_pubkey_stuff *pubkey;
+	} u;
+	/* hope the list doesn't get too long */
+	struct secret *next;
 };
 
-struct secret_stuff *get_secret_stuff(struct secret *secret)
+const struct secret_preshared_stuff *secret_preshared_stuff(const struct secret *secret)
 {
-	return &secret->stuff;
+	switch (secret->kind) {
+	case SECRET_PSK:
+	case SECRET_XAUTH:
+		/* some sort of PSK */
+		return secret->u.preshared;
+	default:
+		return NULL;
+	}
+}
+
+struct secret_pubkey_stuff *secret_pubkey_stuff(const struct secret *secret)
+{
+	switch (secret->kind) {
+	case SECRET_RSA:
+	case SECRET_ECDSA:
+		/* some sort of PKI */
+		return secret->u.pubkey;
+	default:
+		return NULL;
+	}
+}
+
+const struct secret_ppk_stuff *secret_ppk_stuff(const struct secret *secret)
+{
+	switch (secret->kind) {
+	case SECRET_PPK:
+		return secret->u.ppk;
+	default:
+		return NULL;
+	}
 }
 
 struct id_list *lsw_get_idlist(const struct secret *s)
@@ -182,34 +195,34 @@ const ckaid_t *pubkey_ckaid(const struct pubkey *pk)
 
 const ckaid_t *secret_ckaid(const struct secret *secret)
 {
-	switch (secret->stuff.kind) {
-	case SECRET_RSA:
-	case SECRET_ECDSA:
-		/* some sort of PKI */
-		return &secret->stuff.u.pubkey.content.ckaid;
-	default:
+	const struct secret_pubkey_stuff *pki =
+		secret_pubkey_stuff(secret);
+	if (pki == NULL) {
 		return NULL;
 	}
+
+	/* some sort of PKI */
+	return &pki->content.ckaid;
 }
 
 const keyid_t *secret_keyid(const struct secret *secret)
 {
-	switch (secret->stuff.kind) {
-	case SECRET_RSA:
-	case SECRET_ECDSA:
-		/* some sort of PKI */
-		return &secret->stuff.u.pubkey.content.keyid;
-	default:
+	const struct secret_pubkey_stuff *pki =
+		secret_pubkey_stuff(secret);
+	if (pki == NULL) {
 		return NULL;
 	}
+	/* some sort of PKI */
+	return &pki->content.keyid;
 }
 
 struct secret *foreach_secret(struct secret *secrets,
-			      secret_eval func, void *uservoid)
+			      secret_eval func,
+			      struct secret_context *context)
 {
 	for (struct secret *s = secrets; s != NULL; s = s->next) {
-		struct secret_stuff *pks = &s->stuff;
-		int result = (*func)(s, pks, uservoid);
+		int result = (*func)(s, s->kind,
+				     s->line, context);
 
 		if (result == 0)
 			return s;
@@ -221,32 +234,27 @@ struct secret *foreach_secret(struct secret *secrets,
 }
 
 static struct secret *find_secret_by_pubkey_ckaid_1(struct secret *secrets,
-						    const struct pubkey_type *type,
+						    const struct pubkey_type *type_or_null,
 						    const SECItem *pubkey_ckaid)
 {
 	for (struct secret *s = secrets; s != NULL; s = s->next) {
-		const struct secret_stuff *pks = &s->stuff;
 		id_buf idb;
 		enum_buf kb;
 		dbg("trying secret %s:%s",
-		    str_enum(&secret_kind_names, pks->kind, &kb),
+		    str_enum(&secret_kind_names, s->kind, &kb),
 		    (pexpect(s->ids != NULL) ? str_id(&s->ids->id, &idb) : "<NULL-ID-LIST>"));
-		/* should be == SECRET_PKI */
-		switch (pks->kind) {
-		case SECRET_RSA:
-		case SECRET_ECDSA:
-			/* some sort of PKI */
-			break;
-		default:
+		const struct secret_pubkey_stuff *pki = secret_pubkey_stuff(s);
+		if (pki == NULL) {
+			/* not a pubkey */
 			dbg("  not PKI");
 			continue;
 		}
-		if (type != NULL && pks->u.pubkey.content.type != type) {
+		if (type_or_null != NULL && pki->content.type != type_or_null) {
 			/* need exact or wildcard */
-			dbg("  not %s", type->name);
+			dbg("  not %s", type_or_null->name);
 			continue;
 		}
-		if (!ckaid_eq_nss(&pks->u.pubkey.content.ckaid, pubkey_ckaid)) {
+		if (!ckaid_eq_nss(&pki->content.ckaid, pubkey_ckaid)) {
 			dbg("  wrong ckaid");
 			continue;
 		}
@@ -256,32 +264,24 @@ static struct secret *find_secret_by_pubkey_ckaid_1(struct secret *secrets,
 	return NULL;
 }
 
-bool secret_pubkey_same(struct secret *lhs, struct secret *rhs)
+bool secret_pubkey_same(const struct secret *lhs, const struct secret *rhs)
 {
 	/* should be == SECRET_PKI */
-	switch (lhs->stuff.kind) {
-	case SECRET_RSA:
-	case SECRET_ECDSA:
-		break;
-	default:
+	const struct secret_pubkey_stuff *lpk = secret_pubkey_stuff(lhs);
+	if (lpk == NULL) {
 		return false;
 	}
 
-	/* should be == SECRET_PKI */
-	switch (rhs->stuff.kind) {
-	case SECRET_RSA:
-	case SECRET_ECDSA:
-		break;
-	default:
+	const struct secret_pubkey_stuff *rpk = secret_pubkey_stuff(rhs);
+	if (rpk == NULL) {
 		return false;
 	}
 
-	if (lhs->stuff.u.pubkey.content.type != rhs->stuff.u.pubkey.content.type) {
+	if (lpk->content.type != rpk->content.type) {
 		return false;
 	}
 
-	return lhs->stuff.u.pubkey.content.type->pubkey_same(&lhs->stuff.u.pubkey.content,
-							    &rhs->stuff.u.pubkey.content);
+	return lpk->content.type->pubkey_same(&lpk->content, &rpk->content);
 }
 
 struct secret *lsw_find_secret_by_id(struct secret *secrets,
@@ -307,13 +307,13 @@ struct secret *lsw_find_secret_by_id(struct secret *secrets,
 			id_buf idl;
 			enum_buf kb, skb;
 			DBG_log("line %d: key type %s(%s) to type %s",
-				s->stuff.line,
+				s->line,
 				str_enum(&secret_kind_names, kind, &kb),
 				str_id(local_id, &idl),
-				str_enum(&secret_kind_names, s->stuff.kind, &skb));
+				str_enum(&secret_kind_names, s->kind, &skb));
 		}
 
-		if (s->stuff.kind != kind) {
+		if (s->kind != kind) {
 			dbg("  wrong kind");
 			continue;
 		}
@@ -400,8 +400,8 @@ struct secret *lsw_find_secret_by_id(struct secret *secrets,
 				same = true;
 				break;
 			case SECRET_PSK:
-				same = hunk_eq(s->stuff.u.preshared_secret,
-					       best->stuff.u.preshared_secret);
+				same = hunk_eq(s->u.preshared[0],
+					       best->u.preshared[0]);
 				break;
 			case SECRET_RSA:
 			case SECRET_ECDSA:
@@ -414,7 +414,8 @@ struct secret *lsw_find_secret_by_id(struct secret *secrets,
 				 */
 				break;
 			case SECRET_PPK:
-				same = hunk_eq(s->stuff.ppk, best->stuff.ppk);
+				same = hunk_eq(s->u.ppk->key,
+					       best->u.ppk->key);
 				break;
 			default:
 				bad_case(kind);
@@ -454,7 +455,7 @@ struct secret *lsw_find_secret_by_id(struct secret *secrets,
 			 */
 			if (match > best_match) {
 				dbg("  match "PRI_LSET" beats previous best_match "PRI_LSET" match=%p (line=%d)",
-				    match, best_match, s, s->stuff.line);
+				    match, best_match, s, s->line);
 				/* this is the best match so far */
 				best_match = match;
 				best = s;
@@ -467,7 +468,7 @@ struct secret *lsw_find_secret_by_id(struct secret *secrets,
 
 	dbg("concluding with best_match="PRI_LSET" best=%p (lineno=%d)",
 	    best_match, best,
-	    best == NULL ? -1 : best->stuff.line);
+	    best == NULL ? -1 : best->line);
 
 	return best;
 }
@@ -514,97 +515,92 @@ struct secret *lsw_find_secret_by_id(struct secret *secrets,
  */
 
 /* parse PSK from file */
-static err_t process_psk_secret(struct file_lex_position *flp, chunk_t *psk)
+static diag_t process_preshared_secret(const char *what, size_t minlen,
+				       struct file_lex_position *flp,
+				       struct secret_preshared_stuff **psk)
 {
-	err_t ugh = NULL;
-
 	if (flp->quote == '"' || flp->quote == '\'') {
 		size_t len = strlen(flp->tok);
-		if (len < 8) {
+		if (minlen > 0 && len < minlen) {
 			llog(RC_LOG, flp->logger,
-			     "WARNING: using a weak secret (PSK)");
+			     "WARNING: using a weak secret (%s)", what);
 		}
-		*psk = clone_bytes_as_chunk(flp->tok, len, "PSK");
+		*psk = clone_bytes_as_hunk(struct secret_preshared_stuff,
+					   flp->tok, len);
 		shift(flp);
-	} else {
-		ugh = ttochunk(shunk2(flp->tok, flp->cur - flp->tok), 0, psk);
-		if (ugh != NULL) {
-			/* ttodata didn't like PSK data */
-			ugh = builddiag("PSK data malformed (%s): %s", ugh,
-					flp->tok);
-		} else {
-			shift(flp);
-		}
+		return NULL;
 	}
 
-	return ugh;
-}
-
-/* parse XAUTH secret from file */
-static err_t process_xauth_secret(struct file_lex_position *flp, chunk_t *xauth)
-{
-	err_t ugh = NULL;
-
-	if (flp->quote == '"' || flp->quote == '\'') {
-		*xauth = clone_bytes_as_chunk(flp->tok, strlen(flp->tok), "XAUTH");
-		shift(flp);
-	} else {
-		ugh = ttochunk(shunk2(flp->tok, flp->cur - flp->tok), 0, xauth);
-		if (ugh != NULL) {
-			/* ttodata didn't like PSK data */
-			ugh = builddiag("PSK data malformed (%s): %s", ugh,
-					flp->tok);
-		} else {
-			shift(flp);
-		}
+	chunk_t secret;
+	err_t ugh = ttochunk(shunk2(flp->tok, flp->cur - flp->tok), 0, &secret);
+	if (ugh != NULL) {
+		/* ttodata didn't like PSK data */
+		return diag("%s data malformed (%s): %s",
+			    what, ugh, flp->tok);
 	}
 
-	return ugh;
+	*psk = clone_bytes_as_hunk(struct secret_preshared_stuff,
+				   secret.ptr, secret.len);
+	free_chunk_content(&secret);
+
+	shift(flp);
+	return NULL;
 }
 
 /* parse static PPK */
-static err_t process_ppk_static_secret(struct file_lex_position *flp,
-				       chunk_t *ppk, chunk_t *ppk_id)
+static diag_t process_ppk_static_secret(struct file_lex_position *flp,
+					struct secret_ppk_stuff **ppk)
 {
-	err_t ugh = NULL;
-
-	if (flp->quote == '"' || flp->quote == '\'') {
-		*ppk_id = clone_bytes_as_chunk(flp->tok, strlen(flp->tok), "PPK ID");
-	} else {
-		ugh = "No quotation marks found. PPK ID should be in quotation marks";
-		return ugh;
+	if (flp->quote != '"' && flp->quote != '\'') {
+		return diag("no quotation marks found, PPK ID should be in quotation marks");
 	}
 
+	chunk_t id = clone_bytes_as_chunk(flp->tok, strlen(flp->tok), "PPK ID");
 	if (!shift(flp)) {
-		ugh = "No PPK found. PPK should be specified after PPK ID";
-		free_chunk_content(ppk_id);
-		return ugh;
+		free_chunk_content(&id);
+		return diag("No PPK found. PPK should be specified after PPK ID");
 	}
 
+	chunk_t key;
 	if (flp->quote == '"' || flp->quote == '\'') {
-		*ppk = clone_bytes_as_chunk(flp->tok, strlen(flp->tok), "PPK");
+		key = clone_bytes_as_chunk(flp->tok, strlen(flp->tok), "PPK");
 		shift(flp);
 	} else {
-		ugh = ttochunk(shunk2(flp->tok, flp->cur - flp->tok), 0, ppk);
+		err_t ugh = ttochunk(shunk2(flp->tok, flp->cur - flp->tok), 0, &key);
 		if (ugh != NULL) {
 			/* ttodata didn't like PPK data */
-			ugh = builddiag("PPK data malformed (%s): %s", ugh,
-					flp->tok);
-			free_chunk_content(ppk_id);
-		} else {
-			shift(flp);
+			free_chunk_content(&id);
+			return diag("PPK data malformed (%s): %s", ugh, flp->tok);
 		}
+
+		shift(flp);
 	}
 
-	return ugh;
+	/* merge the fields */
+	(*ppk) = overalloc_thing(struct secret_ppk_stuff, id.len + key.len);
+
+	uint8_t *dst = (*ppk)->data;
+	(*ppk)->id.ptr = dst;
+	(*ppk)->id.len = id.len;
+	memcpy(dst, id.ptr, id.len);
+
+	dst += id.len;
+	(*ppk)->key.ptr = dst;
+	(*ppk)->key.len = key.len;
+	memcpy(dst, key.ptr, key.len);
+
+	free_chunk_content(&key);
+	free_chunk_content(&id);
+
+	return NULL;
 }
 
-struct secret *lsw_get_ppk_by_id(struct secret *s, chunk_t ppk_id)
+const struct secret_ppk_stuff *secret_ppk_stuff_by_id(const struct secret *s, shunk_t ppk_id)
 {
 	while (s != NULL) {
-		struct secret_stuff pks = s->stuff;
-		if (pks.kind == SECRET_PPK && hunk_eq(pks.ppk_id, ppk_id))
-			return s;
+		if (s->kind == SECRET_PPK &&
+		    hunk_eq(s->u.ppk->id, ppk_id))
+			return s->u.ppk;
 		s = s->next;
 	}
 	return NULL;
@@ -695,36 +691,41 @@ static void add_secret(struct secret **slist,
 static void process_secret(struct file_lex_position *flp,
 			   struct secret **psecrets, struct secret *s)
 {
-	err_t ugh = NULL;
+	diag_t ugh = NULL;
 
 	if (tokeqword(flp, "psk")) {
-		s->stuff.kind = SECRET_PSK;
+		s->kind = SECRET_PSK;
 		/* preshared key: quoted string or ttodata format */
-		ugh = !shift(flp) ? "ERROR: unexpected end of record in PSK" :
-			process_psk_secret(flp, &s->stuff.u.preshared_secret);
-		dbg("processing PSK at line %d: %s",
-			s->stuff.line, ugh == NULL ? "passed" : ugh);
+		ugh = (!shift(flp) ? diag("ERROR: unexpected end of record in PSK") :
+		       process_preshared_secret("PSK", 8, flp,
+						&s->u.preshared));
+		ldbg(flp->logger, "processing PSK at line %d: %s",
+		     s->line,
+		     (ugh == NULL ? "passed" : str_diag(ugh)));
 	} else if (tokeqword(flp, "xauth")) {
 		/* xauth key: quoted string or ttodata format */
-		s->stuff.kind = SECRET_XAUTH;
-		ugh = !shift(flp) ? "ERROR: unexpected end of record in PSK" :
-			process_xauth_secret(flp, &s->stuff.u.preshared_secret);
-		dbg("processing XAUTH at line %d: %s",
-			s->stuff.line, ugh == NULL ? "passed" : ugh);
+		s->kind = SECRET_XAUTH;
+		ugh = (!shift(flp) ? diag("ERROR: unexpected end of record in PSK") :
+		       process_preshared_secret("XAUTH", 0, flp,
+						&s->u.preshared));
+		ldbg(flp->logger, "processing XAUTH at line %d: %s",
+		     s->line,
+		     (ugh == NULL ? "passed" : str_diag(ugh)));
 	} else if (tokeqword(flp, "ppks")) {
-		s->stuff.kind = SECRET_PPK;
-		ugh = !shift(flp) ? "ERROR: unexpected end of record in static PPK" :
-			process_ppk_static_secret(flp, &s->stuff.ppk, &s->stuff.ppk_id);
-		dbg("processing PPK at line %d: %s",
-			s->stuff.line, ugh == NULL ? "passed" : ugh);
+		s->kind = SECRET_PPK;
+		ugh = (!shift(flp) ? diag("ERROR: unexpected end of record in static PPK") :
+		       process_ppk_static_secret(flp, &s->u.ppk));
+		ldbg(flp->logger, "processing PPK at line %d: %s",
+		     s->line,
+		     (ugh == NULL ? "passed" : str_diag(ugh)));
 	} else {
-		ugh = builddiag("WARNING: ignored unrecognized keyword: %s", flp->tok);
+		ugh = diag("WARNING: ignored unrecognized keyword: %s", flp->tok);
 	}
 
 	if (ugh != NULL) {
-		llog(RC_LOG, flp->logger,
-			    "\"%s\" line %d: %s",
-			    flp->filename, s->stuff.line, ugh);
+		llog(RC_LOG, flp->logger, "\"%s\" line %d: %s",
+		     flp->filename, s->line, str_diag(ugh));
+		pfree_diag(&ugh);
 		/* free id's that should have been allocated */
 		if (s->ids != NULL) {
 			struct id_list *i, *ni;
@@ -736,7 +737,10 @@ static void process_secret(struct file_lex_position *flp,
 		}
 		/* finally free s */
 		pfree(s);
-	} else if (flushline(flp, "expected record boundary in key")) {
+		return;
+	}
+
+	if (flushline(flp, "expected record boundary in key")) {
 		/* gauntlet has been run: install new secret */
 		add_secret(psecrets, s, "process_secret");
 	}
@@ -808,14 +812,13 @@ static void process_secret_records(struct file_lex_position *flp,
 				flp->tok = NULL;	/* redundant? */
 			}
 		} else {
-			/* expecting a list of indices and then the key info */
+			/*
+			 * Expecting a list of indices and then the
+			 * key info.
+			 */
 			struct secret *s = alloc_thing(struct secret, "secret");
-
-			s->ids = NULL;
-			s->stuff.kind = SECRET_PSK;	/* default */
-			s->stuff.u.preshared_secret = EMPTY_CHUNK;
-			s->stuff.line = flp->lino;
-			s->next = NULL;
+			s->kind = 0;	/* invalid */
+			s->line = flp->lino;
 
 			for (;;) {
 				struct id id;
@@ -864,7 +867,7 @@ static void process_secret_records(struct file_lex_position *flp,
 					id_buf b;
 					enum_buf skb;
 					dbg("id type added to secret(%p) %s: %s",
-					    s, str_enum(&secret_kind_names, s->stuff.kind, &skb),
+					    s, str_enum(&secret_kind_names, s->kind, &skb),
 					    str_id(&id, &b));
 				}
 				if (!shift(flp)) {
@@ -914,7 +917,7 @@ static void process_secrets_file(struct file_lex_position *oflp,
 
 	struct lswglob_context context = {
 		.oflp = oflp,
-		psecrets = psecrets,
+		.psecrets = psecrets,
 	};
 	if (!lswglob(file_pat, "secrets", process_secret_files,
 		     &context, oflp->logger)) {
@@ -940,25 +943,22 @@ void lsw_free_preshared_secrets(struct secret **psecrets, struct logger *logger)
 				free_id_content(&i->id);
 				pfree(i);
 			}
-			switch (s->stuff.kind) {
+			switch (s->kind) {
 			case SECRET_PSK:
-				free_chunk_content(&s->stuff.u.preshared_secret);
+				pfreeany(s->u.preshared);
 				break;
 			case SECRET_PPK:
-				free_chunk_content(&s->stuff.ppk);
-				free_chunk_content(&s->stuff.ppk_id);
+				pfreeany(s->u.ppk);
 				break;
 			case SECRET_XAUTH:
-				free_chunk_content(&s->stuff.u.preshared_secret);
+				pfreeany(s->u.preshared);
 				break;
 			case SECRET_RSA:
 			case SECRET_ECDSA:
-				/* Note: pub is all there is */
-				SECKEY_DestroyPrivateKey(s->stuff.u.pubkey.private_key);
-				s->stuff.u.pubkey.content.type->free_pubkey_content(&s->stuff.u.pubkey.content);
+				secret_pubkey_stuff_delref(&s->u.pubkey, HERE);
 				break;
 			default:
-				bad_case(s->stuff.kind);
+				bad_case(s->kind);
 			}
 			pfree(s);
 		}
@@ -1172,35 +1172,53 @@ static const struct pubkey_type *private_key_type_nss(SECKEYPrivateKey *private_
 	}
 }
 
-static err_t add_private_key(struct secret **secrets, const struct secret_stuff **pks,
-			     SECKEYPublicKey *pubk, SECItem *ckaid_nss,
-			     const struct pubkey_type *type, SECKEYPrivateKey *private_key)
+struct secret_pubkey_stuff *secret_pubkey_stuff_addref(struct secret_pubkey_stuff *pks,
+						       where_t where)
 {
-	struct secret *s = alloc_thing(struct secret, "pubkey secret");
-	s->stuff.kind = type->private_key_kind;
-	s->stuff.line = 0;
-	/* make an unpacked copy of the private key */
-	s->stuff.u.pubkey.private_key = copy_private_key(private_key);
-	err_t err = type->extract_pubkey_content(&s->stuff.u.pubkey.content,
-						 pubk, ckaid_nss);
+	return addref_where(pks, where);
+}
+
+void secret_pubkey_stuff_delref(struct secret_pubkey_stuff **pks, where_t where)
+{
+	struct secret_pubkey_stuff *last = delref_where(pks, &global_logger, where);
+	if (last != NULL) {
+		SECKEY_DestroyPrivateKey(last->private_key);
+		last->content.type->free_pubkey_content(&last->content);
+		pfree(last);
+	}
+}
+
+static err_t add_private_key(struct secret **secrets,
+			     struct secret_pubkey_stuff **pks,
+			     SECKEYPublicKey *pubk, SECItem *ckaid_nss,
+			     const struct pubkey_type *type,
+			     SECKEYPrivateKey *private_key)
+{
+	struct pubkey_content content;
+	err_t err = type->extract_pubkey_content(&content, pubk, ckaid_nss);
 	if (err != NULL) {
-		/* extract should leave pubkey_content clean */
-		SECKEY_DestroyPrivateKey(s->stuff.u.pubkey.private_key); /* allocated above */
-		pfree(s);
 		return err;
 	}
 
-	passert(s->stuff.u.pubkey.content.type == type);
-	pexpect(s->stuff.u.pubkey.content.ckaid.len > 0);
-	pexpect(s->stuff.u.pubkey.content.keyid.keyid[0] != '\0');
+	passert(content.type == type);
+	pexpect(content.ckaid.len > 0);
+	pexpect(content.keyid.keyid[0] != '\0');
+
+	struct secret *s = alloc_thing(struct secret, "pubkey secret");
+	s->kind = type->private_key_kind;
+	s->line = 0;
+	/* make an unpacked copy of the private key */
+	s->u.pubkey = refcnt_alloc(struct secret_pubkey_stuff, HERE);
+	s->u.pubkey->private_key = copy_private_key(private_key);
+	s->u.pubkey->content = content;
 
 	add_secret(secrets, s, "lsw_add_rsa_secret");
-	*pks = &s->stuff;
+	*pks = s->u.pubkey;
 	return NULL;
 }
 
 static err_t find_or_load_private_key_by_cert_3(struct secret **secrets, CERTCertificate *cert,
-						const struct secret_stuff **pks, struct logger *logger,
+						struct secret_pubkey_stuff **pks, struct logger *logger,
 						SECKEYPublicKey *pubk, SECItem *ckaid_nss,
 						const struct pubkey_type *type)
 {
@@ -1216,7 +1234,7 @@ static err_t find_or_load_private_key_by_cert_3(struct secret **secrets, CERTCer
 }
 
 static err_t find_or_load_private_key_by_cert_2(struct secret **secrets, CERTCertificate *cert,
-						const struct secret_stuff **pks, bool *load_needed,
+						struct secret_pubkey_stuff **pks, bool *load_needed,
 						struct logger *logger,
 						SECKEYPublicKey *pubk, SECItem *ckaid_nss)
 {
@@ -1230,7 +1248,7 @@ static err_t find_or_load_private_key_by_cert_2(struct secret **secrets, CERTCer
 	struct secret *s = find_secret_by_pubkey_ckaid_1(*secrets, type, ckaid_nss);
 	if (s != NULL) {
 		dbg("secrets entry for certificate already exists: %s", cert->nickname);
-		*pks = &s->stuff;
+		*pks = s->u.pubkey;
 		*load_needed = false;
 		return NULL;
 	}
@@ -1244,7 +1262,8 @@ static err_t find_or_load_private_key_by_cert_2(struct secret **secrets, CERTCer
 }
 
 static err_t find_or_load_private_key_by_cert_1(struct secret **secrets, CERTCertificate *cert,
-						const struct secret_stuff **pks, bool *load_needed,
+						struct secret_pubkey_stuff **pks,
+						bool *load_needed,
 						struct logger *logger,
 						SECKEYPublicKey *pubk)
 {
@@ -1259,7 +1278,8 @@ static err_t find_or_load_private_key_by_cert_1(struct secret **secrets, CERTCer
 		return "NSS: key ID not found";
 	}
 
-	err_t err = find_or_load_private_key_by_cert_2(secrets, cert, pks, load_needed, logger,
+	err_t err = find_or_load_private_key_by_cert_2(secrets, cert,
+						       pks, load_needed, logger,
 						       /* extracted fields */
 						       pubk, ckaid_nss);
 	SECITEM_FreeItem(ckaid_nss, PR_TRUE);
@@ -1267,7 +1287,7 @@ static err_t find_or_load_private_key_by_cert_1(struct secret **secrets, CERTCer
 }
 
 err_t find_or_load_private_key_by_cert(struct secret **secrets, const struct cert *cert,
-				       const struct secret_stuff **pks, bool *load_needed,
+				       struct secret_pubkey_stuff **pks, bool *load_needed,
 				       struct logger *logger)
 {
 	*load_needed = false;
@@ -1282,7 +1302,8 @@ err_t find_or_load_private_key_by_cert(struct secret **secrets, const struct cer
 		return "NSS: could not determine certificate kind; SECKEY_ExtractPublicKey() failed";
 	}
 
-	err_t err = find_or_load_private_key_by_cert_1(secrets, cert->nss_cert, pks, load_needed, logger,
+	err_t err = find_or_load_private_key_by_cert_1(secrets, cert->nss_cert,
+						       pks, load_needed, logger,
 						       /* extracted fields */
 						       pubk);
 	SECKEY_DestroyPublicKey(pubk);
@@ -1290,7 +1311,7 @@ err_t find_or_load_private_key_by_cert(struct secret **secrets, const struct cer
 }
 
 static err_t find_or_load_private_key_by_ckaid_1(struct secret **secrets,
-						 const struct secret_stuff **pks,
+						 struct secret_pubkey_stuff **pks,
 						 SECItem *ckaid_nss, SECKEYPrivateKey *private_key)
 {
 	const struct pubkey_type *type = private_key_type_nss(private_key);
@@ -1309,7 +1330,7 @@ static err_t find_or_load_private_key_by_ckaid_1(struct secret **secrets,
 }
 
 err_t find_or_load_private_key_by_ckaid(struct secret **secrets, const ckaid_t *ckaid,
-					const struct secret_stuff **pks, bool *load_needed,
+					struct secret_pubkey_stuff **pks, bool *load_needed,
 					struct logger *logger)
 {
 	*load_needed = false;
@@ -1319,7 +1340,7 @@ err_t find_or_load_private_key_by_ckaid(struct secret **secrets, const ckaid_t *
 	struct secret *s = find_secret_by_pubkey_ckaid_1(*secrets, NULL, &ckaid_nss);
 	if (s != NULL) {
 		dbg("secrets entry for ckaid already exists");
-		*pks = &s->stuff;
+		*pks = s->u.pubkey;
 		*load_needed = false;
 		return NULL;
 	}
