@@ -16,6 +16,7 @@
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
  */
+
 #ifndef USE_DNSSEC
 # error this file should only be compiled when using DNSSEC
 #endif
@@ -31,7 +32,7 @@
 #include <errno.h>
 #include <event2/event.h>	/* deb:libevent-dev */
 #include <unbound.h>		/* rpm:unbound-devel deb:libunbound-dev */
-#include <unbound-event.h>	/* see USE_UNBOUND_EVENT_H_COPY */
+#include <unbound-event.h>	/* ditto */
 
 #include "lswglob.h"
 #include "dnssec.h"
@@ -49,7 +50,24 @@ void unbound_ctx_free(void)
 	}
 }
 
-static void unbound_ctx_config(bool do_dnssec, const char *rootfile, const char *trusted,
+static void add_trust_anchors(unsigned count, char **files,
+			      struct lswglob_context *context UNUSED,
+			      struct logger *logger)
+{
+	for (unsigned i = 0; i < count; i++) {
+		const char *file = files[i];
+		int ugh = ub_ctx_add_ta_file(dns_ctx, file);
+		if (ugh != 0) {
+			llog(RC_LOG, logger, "ignored trusted key file %s: %s",
+			     file,  ub_strerror(ugh));
+		} else {
+			ldbg(logger, "added contents of trusted key file %s to unbound resolver context", file);
+		}
+	}
+}
+
+static void unbound_ctx_config(bool do_dnssec, const char *rootfile,
+			       const char *trusted,
 			       struct logger *logger)
 {
 	int ugh;
@@ -62,7 +80,7 @@ static void unbound_ctx_config(bool do_dnssec, const char *rootfile, const char
 	/* lookup from /etc/hosts before DNS lookups as people expect that */
 	ugh = ub_ctx_hosts(dns_ctx, "/etc/hosts");
 	if (ugh != 0) {
-		llog(RC_LOG_SERIOUS, logger,
+		llog(RC_LOG, logger,
 			    "error reading hosts: %s: %s",
 			    ub_strerror(ugh), strerror(errno));
 	} else {
@@ -86,7 +104,7 @@ static void unbound_ctx_config(bool do_dnssec, const char *rootfile, const char
 	if (ugh != 0) {
 		int e = errno;	/* protect value from ub_strerror */
 
-		llog(RC_LOG_SERIOUS, logger,
+		llog(RC_LOG, logger,
 			    "error reading /etc/resolv.conf: %s: [errno: %s]",
 			    ub_strerror(ugh), strerror(e));
 	} else {
@@ -99,7 +117,7 @@ static void unbound_ctx_config(bool do_dnssec, const char *rootfile, const char
 	errno = 0;
 	ugh = ub_ctx_set_option(dns_ctx, "outgoing-port-avoid:", "0-65535");
 	if (ugh != 0) {
-		llog(RC_LOG_SERIOUS, logger,
+		llog(RC_LOG, logger,
 			    "error setting outgoing-port-avoid: %s: %s",
 			    ub_strerror(ugh), strerror(errno));
 	} else {
@@ -109,7 +127,7 @@ static void unbound_ctx_config(bool do_dnssec, const char *rootfile, const char
 	errno = 0;
 	ugh = ub_ctx_set_option(dns_ctx, "outgoing-port-permit:", "32768-60999");
 		if (ugh != 0) {
-		llog(RC_LOG_SERIOUS, logger,
+		llog(RC_LOG, logger,
 			    "error setting outgoing-port-permit: %s: %s",
 			    ub_strerror(ugh), strerror(errno));
 	} else {
@@ -125,13 +143,13 @@ static void unbound_ctx_config(bool do_dnssec, const char *rootfile, const char
 	/* Only DNSSEC related configuration from here */
 	if (rootfile == NULL) {
 		if (trusted == NULL) {
-			llog(RC_LOG_SERIOUS, logger,
+			llog(RC_LOG, logger,
 				    "dnssec-enable=yes but no dnssec-rootkey-file or trust anchors specified.");
-			llog(RC_LOG_SERIOUS, logger,
+			llog(RC_LOG, logger,
 				    "WARNING: DNSSEC validation disabled");
 			return;
 		} else {
-			llog(RC_LOG_SERIOUS, logger,
+			llog(RC_LOG, logger,
 				    "dnssec-enable=yes but no dnssec-rootkey-file specified. Additional trust anchor file MUST include a root trust anchor or DNSSEC validation will be disabled");
 		}
 	} else {
@@ -141,57 +159,22 @@ static void unbound_ctx_config(bool do_dnssec, const char *rootfile, const char
 		if (ugh != 0) {
 			int e = errno;	/* protect value from ub_strerror */
 
-			llog(RC_LOG_SERIOUS, logger,
+			llog(RC_LOG, logger,
 				    "error adding dnssec root key: %s [errno: %s]",
 				    ub_strerror(ugh), strerror(e));
-			llog(RC_LOG_SERIOUS, logger,
+			llog(RC_LOG, logger,
 				    "WARNING: DNSSEC validation disabled");
 		}
 	}
 
 	if (trusted == NULL) {
 		dbg("no additional dnssec trust anchors defined via dnssec-trusted= option");
-	} else {
-		glob_t globbuf;
-		int r = lswglob(trusted, &globbuf, "trusted anchor", logger);
+		return;
+	}
 
-		switch (r) {
-		case 0:	/* success */
-			for (char **fnp = globbuf.gl_pathv; fnp != NULL && *fnp != NULL; fnp++) {
-				ugh = ub_ctx_add_ta_file(dns_ctx, *fnp);
-				if (ugh != 0) {
-					llog(RC_LOG_SERIOUS, logger,
-						    "Ignored trusted key file %s: %s",
-						    *fnp,  ub_strerror(ugh));
-				} else {
-					dbg("added contents of trusted key file %s to unbound resolver context",
-					    *fnp);
-				}
-			}
-			break;
-
-		case GLOB_NOSPACE:
-			llog(RC_LOG_SERIOUS, logger,
-				    "out of space processing dnssec-trusted= argument: %s",
-				    trusted);
-			break;
-
-		case GLOB_ABORTED:
-			/* already logged by globugh_ta */
-			break;
-
-		case GLOB_NOMATCH:
-			llog(RC_LOG_SERIOUS, logger,
-				    "no trust anchor files matched '%s'", trusted);
-			break;
-
-		default:
-			llog(RC_LOG_SERIOUS, logger,
-				    "trusted key file '%s': unknown glob error %d",
-				    trusted, r);
-			break;
-		}
-		globfree(&globbuf);
+	if (!lswglob(trusted, "trusted anchor", add_trust_anchors,
+		     /*lswglob_context*/NULL, logger)) {
+		llog(RC_LOG, logger, "no trust anchor files matched '%s'", trusted);
 	}
 }
 
@@ -232,7 +215,7 @@ void unbound_sync_init(bool do_dnssec, const char *rootfile,
  * af == AF_UNSPEC means default to AF_INET(A/IPv4)
  */
 bool unbound_resolve(char *src, const struct ip_info *afi,
-		     ip_address *ipaddr, struct logger *logger)
+		     ip_address *ipaddr, const struct logger *logger)
 {
 	/* 28 = AAAA record, 1 = A record */
 	const int qtype = (afi == &ipv6_info) ? 28/*AAAA*/ : 1/*A*/;
@@ -298,17 +281,20 @@ bool unbound_resolve(char *src, const struct ip_info *afi,
 	passert(result->data[0] != NULL);
 	passert(result->len != NULL);
 
-	if ((size_t)result->len[0] != afi->ip_size) {
-		llog_pexpect(logger, HERE, "dns record is %u bytes, expecting %zu",
-			     result->len[0], afi->ip_size);
+	/*
+	 * XXX: data_to_address() only requires the length >=
+	 * address-length.
+	 */
+	diag_t diag = data_to_address(result->data[0], (size_t)result->len[0], afi, ipaddr);
+	if (diag != NULL) {
+		llog_pexpect(logger, HERE, "invalid dns address record: %s",
+			     str_diag(diag));
+		pfree_diag(&diag);
 		ub_resolve_free(result);
 		return false;
 	}
 
-	struct ip_bytes bytes = unset_ip_bytes;
-	memcpy(bytes.byte, result->data[0], afi->ip_size);
-	*ipaddr = address_from_raw(HERE, afi->ip_version, bytes);
-	dbg("success for %s lookup", afi->ip_name);
+	ldbg(logger, "success for %s lookup", afi->ip_name);
 	return true;
 }
 
