@@ -6,17 +6,19 @@ BEGIN {
 }
 
 /<manvolnum>/ && !manvolnum {
-    manvolnum = gensub(/[^0-9]*/, "", "g")
+    manvolnum = $0
+    gsub(/[^0-9]*/, "", manvolnum)
     if (debug) print "manvolnum:", manvolnum >> "/dev/stderr"
 }
+
 /<refname>/ {
     refname = $0
     if (debug) print "refname:", refname >> "/dev/stderr"
-    refname = gensub(/[^>]*>/, "", "1", refname)
+    sub(/[^>]*>/, "", refname)
     if (debug) print "refname:", refname >> "/dev/stderr"
-    refname = gensub(/<.*/, "", "1", refname)
+    sub(/<.*/, "", refname)
     if (debug) print "refname:", refname >> "/dev/stderr"
-    refname = gensub(/ /, "_", "g", refname)
+    gsub(/ /, "_", refname)
     if (debug) print "refname:", refname >> "/dev/stderr"
     print refname "." manvolnum
 }
