@@ -28,6 +28,7 @@
 #include "lswnss.h"
 #include "lswtool.h"
 #include "lswalloc.h"
+#include "lswlog.h"		/* for fatal() */
 
 #ifdef __clang__
 /*
@@ -67,7 +68,7 @@ int main(int argc, char *argv[])
 	ssize_t rd;
 	int fin;
 
-	struct logger *logger = tool_init_log(argv[0]);
+	struct logger *logger = tool_logger(argc, argv);
 
 	if (argc != 3) {
 		fatal(PLUTO_EXIT_FAIL, logger, "expecting: <url> <der-size>");
@@ -116,10 +117,7 @@ int main(int argc, char *argv[])
 	}
 
 	const struct lsw_conf_options *oco = lsw_init_options();
-	diag_t d = lsw_nss_setup(oco->nssdir, 0, logger);
-	if (d != NULL) {
-		fatal_diag(PLUTO_EXIT_FAIL, logger, &d, "%s", "");
-	}
+	init_nss(oco->nssdir, (struct nss_flags){0}, logger);
 
 	fin = import_crl(url, tbuf, len);
 
