@@ -29,6 +29,7 @@
 #include "lswalloc.h"
 #include "lswtool.h"
 #include "whack.h"
+#include "sparse_names.h"
 #include "ipsecconf/parser-controls.h"
 #include "ipsecconf/confread.h"
 #include "ipsecconf/confwrite.h"
@@ -42,7 +43,7 @@
 #include "lswseccomp.h"
 #endif
 
-static int verbose = 0;
+#include "optarg.h"
 
 /*
  * make options valid environment variables
@@ -66,8 +67,8 @@ static char *environlize(const char *str)
 static void resolve_default_routes(struct starter_conn *conn UNUSED, struct logger *logger)
 {
 	lset_t verbose_rc_flags = verbose ? (WHACK_STREAM|NO_PREFIX) : LEMPTY;
-	resolve_default_route(&conn->left, &conn->right, verbose_rc_flags, logger);
-	resolve_default_route(&conn->right, &conn->left, verbose_rc_flags, logger);
+	resolve_default_route(&conn->end[LEFT_END], &conn->end[RIGHT_END], verbose_rc_flags, logger);
+	resolve_default_route(&conn->end[RIGHT_END], &conn->end[LEFT_END], verbose_rc_flags, logger);
 }
 
 #ifdef USE_SECCOMP
@@ -171,30 +172,52 @@ static void usage(void)
 	exit(10);
 }
 
-static const struct option longopts[] =
-{
-	{ "config", required_argument, NULL, 'C' },
-	{ "debug", no_argument, NULL, 'D' },
-	{ "verbose", no_argument, NULL, 'D' },
-	{ "addall", no_argument, NULL, 'a' }, /* alias, backwards compat */
-	{ "autoall", no_argument, NULL, 'a' },
-	{ "listall", no_argument, NULL, 'A' },
-	{ "listadd", no_argument, NULL, 'L' },
-	{ "listroute", no_argument, NULL, 'r' },
-	{ "liststart", no_argument, NULL, 's' },
-	{ "listignore", no_argument, NULL, 'i' },
-	{ "varprefix", required_argument, NULL, 'P' },
-	{ "ctlsocket", required_argument, NULL, 'c' },
-	{ "ctlbase", required_argument, NULL, 'c' }, /* backwards compatibility */
-	{ "configsetup", no_argument, NULL, 'T' },
-	{ "liststack", no_argument, NULL, 'S' },
-	{ "checkconfig", no_argument, NULL, 'K' },
-	{ "noexport", no_argument, NULL, 'N' },
-	{ "help", no_argument, NULL, 'h' },
+enum opt {
+	OPT_EOF = -1,
+	OPT_HELP = 'h',
+	OPT_CONFIG = 256,
+	OPT_VERBOSE,
+	OPT_DEBUG,
+	OPT_AUTOALL,
+	OPT_LISTALL,
+	OPT_LISTADD,
+	OPT_LISTROUTE,
+	OPT_LISTSTART,
+	OPT_LISTIGNORE,
+	OPT_VARPREFIX,
+	OPT_CTLSOCKET,
+	OPT_CONFIGSETUP,
+	OPT_LISTSTACK,
+	OPT_CHECKCONFIG,
+	OPT_NOEXPORT,
 	/* obsoleted, eat and ignore for compatibility */
-	{"defaultroute", required_argument, NULL, 'd'},
-	{"defaultroutenexthop", required_argument, NULL, 'n'},
+	OPT_DEFAULTROUTE,
+	OPT_DEFAULTROUTENEXTHOP,
+};
 
+const struct option optarg_options[] =
+{
+	{ "config", required_argument, NULL, OPT_CONFIG, },
+	{ "debug", no_argument, NULL, OPT_DEBUG, },
+	{ "verbose", no_argument, NULL, OPT_VERBOSE, },
+	{ "autoall", no_argument, NULL, OPT_AUTOALL, },
+	{ "addall", no_argument, NULL, OPT_AUTOALL, }, /* alias, backwards compat */
+	{ "listall", no_argument, NULL, OPT_LISTALL, },
+	{ "listadd", no_argument, NULL, OPT_LISTADD, },
+	{ "listroute", no_argument, NULL, OPT_LISTROUTE, },
+	{ "liststart", no_argument, NULL, OPT_LISTSTART, },
+	{ "listignore", no_argument, NULL, OPT_LISTIGNORE, },
+	{ "varprefix", required_argument, NULL, OPT_VARPREFIX, },
+	{ "ctlsocket", required_argument, NULL, OPT_CTLSOCKET, },
+	{ "ctlbase", required_argument, NULL, OPT_CTLSOCKET, }, /* backwards compatibility */
+	{ "configsetup", no_argument, NULL, OPT_CONFIGSETUP, },
+	{ "liststack", no_argument, NULL, OPT_LISTSTACK, },
+	{ "checkconfig", no_argument, NULL, OPT_CHECKCONFIG, },
+	{ "noexport", no_argument, NULL, OPT_NOEXPORT, },
+	{ "help", no_argument, NULL, OPT_HELP, },
+	/* obsoleted, eat and ignore for compatibility */
+	{"defaultroute", required_argument, NULL, OPT_DEFAULTROUTE, },
+	{"defaultroutenexthop", required_argument, NULL, OPT_DEFAULTROUTENEXTHOP, },
 	{ 0, 0, 0, 0 }
 };
 
@@ -202,7 +225,6 @@ int main(int argc, char *argv[])
 {
 	struct logger *logger = tool_logger(argc, argv);
 
-	int opt;
 	bool autoall = false;
 	bool configsetup = false;
 	bool checkconfig = false;
@@ -215,11 +237,10 @@ int main(int argc, char *argv[])
 		listignore = false,
 		listall = false,
 		liststack = false;
-	char *configfile = NULL;
+	const char *configfile = NULL;
 	const char *varprefix = "";
 	int exit_status = 0;
-	struct starter_conn *conn = NULL;
-	char *ctlsocket = clone_str(DEFAULT_CTL_SOCKET, "default control socket");
+	const char *ctlsocket = DEFAULT_CTL_SOCKET;
 
 #if 0
 	/* efence settings */
@@ -229,79 +250,82 @@ int main(int argc, char *argv[])
 	EF_PROTECT_BELOW = 1;
 	EF_PROTECT_FREE = 1;
 #endif
-	while ((opt = getopt_long(argc, argv, "", longopts, 0)) != EOF) {
+	enum opt opt;
+	while ((opt = getopt_long(argc, argv, "", optarg_options, 0)) != EOF) {
 		switch (opt) {
-		case 'h':
+		case OPT_HELP:
 			/* usage: */
 			usage();
 			break;
 
-		case 'a':
+		case OPT_AUTOALL:
 			autoall = true;
 			break;
 
-		case 'D':
-			verbose++;
-			lex_verbosity++;
+		case OPT_VERBOSE:
+			optarg_verbose(LEMPTY);
 			break;
 
-		case 'T':
+		case OPT_DEBUG:
+			optarg_debug(/*enable*/true);
+			break;
+
+		case OPT_CONFIGSETUP:
 			configsetup = true;
 			break;
 
-		case 'K':
+		case OPT_CHECKCONFIG:
 			checkconfig = true;
 			break;
 
-		case 'N':
+		case OPT_NOEXPORT:
 			export = "";
 			break;
 
-		case 'C':
-			configfile = clone_str(optarg, "config file name");
+		case OPT_CONFIG:
+			configfile = optarg;
 			break;
 
-		case 'c':
-			pfree(ctlsocket);
-			ctlsocket = clone_str(optarg, "control socket");
+		case OPT_CTLSOCKET:
+			ctlsocket = optarg;
 			break;
 
-		case 'L':
+		case OPT_LISTADD:
 			listadd = true;
 			dolist = true;
 			break;
 
-		case 'r':
+		case OPT_LISTROUTE:
 			listroute = true;
 			dolist = true;
 			break;
 
-		case 's':
+		case OPT_LISTSTART:
 			liststart = true;
 			dolist = true;
 			break;
 
-		case 'S':
+		case OPT_LISTSTACK:
 			liststack = true;
 			dolist = true;
 			break;
 
-		case 'i':
+		case OPT_LISTIGNORE:
 			listignore = true;
 			dolist = true;
 			break;
 
-		case 'A':
+		case OPT_LISTALL:
 			listall = true;
 			dolist = true;
 			break;
 
-		case 'P':
+		case OPT_VARPREFIX:
 			varprefix = optarg;
 			break;
 
-		case 'd':
-		case 'n':
+		case OPT_DEFAULTROUTE:
+		case OPT_DEFAULTROUTENEXTHOP:
 			fprintf(stderr, "Warning: options --defaultroute and --defaultroutenexthop are obsolete and were ignored\n");
 			break;
 
@@ -310,54 +334,17 @@ int main(int argc, char *argv[])
 		}
 	}
 
-	if (autoall) {
-		/* pluto forks us, we might have to wait on it to create the socket */
-		struct stat sb;
-		int ws = 5; /* somewhat arbitrary */
-
-		while (ws > 0) {
-			int ret = stat(ctlsocket, &sb);
-
-			if (ret == -1) {
-				sleep(1);
-			} else {
-				break;
-			}
-			ws--;
-		}
-		if (ws == 0) {
-			fprintf(stderr, "ipsec addconn: timeout waiting on pluto socket %s - aborted\n",
-				ctlsocket);
-			exit(3);
-		}
-	}
-
 	/* if nothing to add, then complain */
 	if (optind == argc && !autoall && !dolist && !configsetup &&
 	    !checkconfig)
 		usage();
 
-	switch (verbose) {
-	case 0:
-	case 1:
-		break;
-	case 2:
-		cur_debugging = DBG_ALL;
-		break;
-	case 3:
-		cur_debugging = DBG_ALL|DBG_TMI;
-		break;
-	default: /*>=4*/
-		cur_debugging = DBG_ALL|DBG_TMI;
-		yydebug = true;
-		break;
-	}
-
-	/* logged when true! */
-	ldbg(logger, "debugging mode enabled");
+	/* some variables depend on verbose */
+	lex_verbosity = verbose;
+	yydebug = (verbose >= 4);
 
 	if (configfile == NULL) {
-		configfile = clone_str(IPSEC_CONF, "default ipsec.conf file");
+		configfile = IPSEC_CONF;
 	}
 	if (verbose > 0) {
 		printf("opening file: %s\n", configfile);
@@ -375,7 +362,7 @@ int main(int argc, char *argv[])
 	}
 
 #ifdef USE_SECCOMP
-	switch (cfg->setup.options[KBF_SECCOMP]) {
+	switch (cfg->values[KBF_SECCOMP].option) {
 		case SECCOMP_ENABLED:
 			init_seccomp_addconn(SCMP_ACT_KILL, logger);
 		break;
@@ -385,14 +372,14 @@ int main(int argc, char *argv[])
 	case SECCOMP_DISABLED:
 		break;
 	default:
-		bad_case(cfg->setup.options[KBF_SECCOMP]);
+		bad_case(cfg->values[KBF_SECCOMP].option);
 	}
 #endif
 
 #ifdef USE_DNSSEC
-	unbound_sync_init(cfg->setup.options[KBF_DO_DNSSEC],
-			  cfg->setup.strings[KSF_PLUTO_DNSSEC_ROOTKEY_FILE],
-			  cfg->setup.strings[KSF_PLUTO_DNSSEC_ANCHORS],
+	unbound_sync_init(cfg->values[KBF_DO_DNSSEC].option,
+			  cfg->values[KSF_PLUTO_DNSSEC_ROOTKEY_FILE].string,
+			  cfg->values[KSF_PLUTO_DNSSEC_ANCHORS].string,
 			  logger);
 #endif
 
@@ -410,8 +397,9 @@ int main(int argc, char *argv[])
 		if (verbose > 0)
 			printf("  Step #1: Loading auto=add, auto=keep, auto=route, auto=up and auto=start connections\n");
 
-		for (conn = cfg->conns.tqh_first; conn != NULL; conn = conn->link.tqe_next) {
-			enum autostart autostart = conn->options[KNCF_AUTO];
+		for (struct starter_conn *conn = cfg->conns.tqh_first;
+		     conn != NULL; conn = conn->link.tqe_next) {
+			enum autostart autostart = conn->values[KNCF_AUTO].option;
 			switch (autostart) {
 			case AUTOSTART_UNSET:
 			case AUTOSTART_IGNORE:
@@ -450,8 +438,10 @@ int main(int argc, char *argv[])
 		/* load named conns, regardless of their state */
 		int connum;
 
-		if (verbose > 0)
+		if (verbose > 0) {
 			printf("loading named conns:");
+		}
+
 		for (connum = optind; connum < argc; connum++) {
 			const char *connname = argv[connum];
 
@@ -459,13 +449,17 @@ int main(int argc, char *argv[])
 			const char *p2 = "";
 			const char *p3 = "";
 
-			if (verbose > 0)
+			if (verbose > 0) {
 				printf(" %s\n", connname);
+			}
 
 			/* find first name match, if any */
+			struct starter_conn *conn = NULL;
 			for (conn = cfg->conns.tqh_first;
-			     conn != NULL && !streq(conn->name, connname);
-			     conn = conn->link.tqe_next) {
+			     conn != NULL; conn = conn->link.tqe_next) {
+				if (streq(conn->name, connname)) {
+					break;
+				}
 			}
 
 			if (conn == NULL) {
@@ -476,11 +470,9 @@ int main(int argc, char *argv[])
 				p3 = " ";
 
 				for (conn = cfg->conns.tqh_first;
-				     conn != NULL;
-				     conn = conn->link.tqe_next) {
+				     conn != NULL; conn = conn->link.tqe_next) {
 					if (lsw_alias_cmp(connname,
-						conn->strings[KSCF_CONNALIAS]))
-					{
+						conn->values[KSCF_CONNALIAS].string)) {
 						break;
 					}
 				}
@@ -489,37 +481,53 @@ int main(int argc, char *argv[])
 			if (conn == NULL) {
 				/* we found neither name nor alias */
 				exit_status += RC_UNKNOWN_NAME; /* cause non-zero exit code */
-				if (verbose > 0) {
-					printf(" (notfound)\n");
-				}
 				fprintf(stderr, "conn '%s': not found (tried aliases)\n",
 					connname);
-			} else {
-				/* found name or alias */
-				if (conn->state == STATE_ADDED) {
-					fprintf(stderr, "\n%s%s%sconn %s already added\n",
-						p1, p2, p3,
-						conn->name);
-				} else if (conn->state == STATE_FAILED) {
-					fprintf(stderr, "\n%s%s%sconn %s did not load properly\n",
-						p1, p2, p3,
-						conn->name);
-				} else {
-					resolve_default_routes(conn, logger);
-					exit_status = starter_whack_add_conn(ctlsocket,
-									     conn, logger);
-					conn->state = STATE_ADDED;
-				}
+				continue;
 			}
+
+			/* found name or alias */
+			if (conn->state == STATE_ADDED) {
+				fprintf(stderr, "\n%s%s%sconn %s already added\n",
+					p1, p2, p3,
+					conn->name);
+				continue;
+			}
+
+			if (conn->state == STATE_FAILED) {
+				fprintf(stderr, "\n%s%s%sconn %s did not load properly\n",
+					p1, p2, p3,
+					conn->name);
+				continue;
+			}
+
+			/*
+			 * Scrub AUTOSTART; conns will need to be
+			 * started manually.
+			 */
+			enum autostart autostart = conn->values[KNCF_AUTO].option;
+			if (autostart != AUTOSTART_UNSET &&
+			    autostart != AUTOSTART_ADD) {
+				if (verbose) {
+					name_buf nb;
+					printf("  overiding auto=%s with auto=add\n",
+					       str_sparse(&autostart_names, autostart, &nb));
+				}
+				conn->values[KNCF_AUTO].option = AUTOSTART_ADD;
+			}
+
+			resolve_default_routes(conn, logger);
+			exit_status = starter_whack_add_conn(ctlsocket,
+							     conn, logger);
+			conn->state = STATE_ADDED;
 		}
 	}
 
 	if (listall) {
 		if (verbose > 0)
 			printf("listing all conns\n");
-		for (conn = cfg->conns.tqh_first;
-			conn != NULL;
-			conn = conn->link.tqe_next)
+		for (struct starter_conn *conn = cfg->conns.tqh_first;
+		     conn != NULL; conn = conn->link.tqe_next)
 			printf("%s ", conn->name);
 		printf("\n");
 	} else {
@@ -528,10 +536,9 @@ int main(int argc, char *argv[])
 				printf("listing all conns marked as auto=add\n");
 
 			/* list all conns marked as auto=add */
-			for (conn = cfg->conns.tqh_first;
-				conn != NULL;
-				conn = conn->link.tqe_next) {
-				enum autostart autostart = conn->options[KNCF_AUTO];
+			for (struct starter_conn *conn = cfg->conns.tqh_first;
+			     conn != NULL; conn = conn->link.tqe_next) {
+				enum autostart autostart = conn->values[KNCF_AUTO].option;
 				if (autostart == AUTOSTART_ADD)
 					printf("%s ", conn->name);
 			}
@@ -544,10 +551,9 @@ int main(int argc, char *argv[])
 			 * list all conns marked as auto=route or start or
 			 * better
 			 */
-			for (conn = cfg->conns.tqh_first;
-				conn != NULL;
-				conn = conn->link.tqe_next) {
-				enum autostart autostart = conn->options[KNCF_AUTO];
+			for (struct starter_conn *conn = cfg->conns.tqh_first;
+			     conn != NULL; conn = conn->link.tqe_next) {
+				enum autostart autostart = conn->values[KNCF_AUTO].option;
 				if (autostart == AUTOSTART_UP ||
 				    autostart == AUTOSTART_START ||
 				    autostart == AUTOSTART_ROUTE ||
@@ -561,10 +567,9 @@ int main(int argc, char *argv[])
 				printf("listing all conns marked as auto=up\n");
 
 			/* list all conns marked as auto=up */
-			for (conn = cfg->conns.tqh_first;
-				conn != NULL;
-				conn = conn->link.tqe_next) {
-				enum autostart autostart = conn->options[KNCF_AUTO];
+			for (struct starter_conn *conn = cfg->conns.tqh_first;
+			     conn != NULL; conn = conn->link.tqe_next) {
+				enum autostart autostart = conn->values[KNCF_AUTO].option;
 				if (autostart == AUTOSTART_UP ||
 				    autostart == AUTOSTART_START)
 					printf("%s ", conn->name);
@@ -576,10 +581,9 @@ int main(int argc, char *argv[])
 				printf("listing all conns marked as auto=ignore\n");
 
 			/* list all conns marked as auto=up */
-			for (conn = cfg->conns.tqh_first;
-				conn != NULL;
-				conn = conn->link.tqe_next) {
-				enum autostart autostart = conn->options[KNCF_AUTO];
+			for (struct starter_conn *conn = cfg->conns.tqh_first;
+			     conn != NULL; conn = conn->link.tqe_next) {
+				enum autostart autostart = conn->values[KNCF_AUTO].option;
 				if (autostart == AUTOSTART_IGNORE ||
 				    autostart == AUTOSTART_UNSET)
 					printf("%s ", conn->name);
@@ -593,9 +597,9 @@ int main(int argc, char *argv[])
 
 		for (kd = ipsec_conf_keywords; kd->keyname != NULL; kd++) {
 			if (strstr(kd->keyname, "protostack")) {
-				if (cfg->setup.strings[kd->field]) {
+				if (cfg->values[kd->field].string) {
 					printf("%s\n",
-						cfg->setup.strings[kd->field]);
+						cfg->values[kd->field].string);
 				} else {
 					/* implicit default */
 					printf("xfrm\n");
@@ -627,28 +631,28 @@ int main(int argc, char *argv[])
 			case kt_filename:
 			case kt_dirname:
 			case kt_host:
-				if (cfg->setup.strings[kd->field]) {
+				if (cfg->values[kd->field].string) {
 					printf("%s %s%s='%s'\n",
 						export, varprefix, safe_kwname,
-						cfg->setup.strings[kd->field]);
+						cfg->values[kd->field].string);
 				}
 				break;
 
 			case kt_bool:
 				printf("%s %s%s='%s'\n", export, varprefix,
 					safe_kwname,
-					bool_str(cfg->setup.options[kd->field]));
+					bool_str(cfg->values[kd->field].option));
 				break;
 
 			case kt_obsolete:
 				break;
 
 			default:
-				if (cfg->setup.options[kd->field] ||
-					cfg->setup.set[kd->field]) {
+				if (cfg->values[kd->field].option ||
+					cfg->values[kd->field].set) {
 					printf("%s %s%s='%jd'\n",
 						export, varprefix, safe_kwname,
-						cfg->setup.options[kd->field]);
+						cfg->values[kd->field].option);
 				}
 				break;
 			}
@@ -662,8 +666,6 @@ int main(int argc, char *argv[])
 #ifdef USE_DNSSEC
 	unbound_ctx_free();
 #endif
-	pfreeany(ctlsocket);
-	pfreeany(configfile);
 	/*
 	 * Only RC_ codes between RC_EXIT_FLOOR (RC_DUPNAME) and
 	 * RC_EXIT_ROOF (RC_NEW_V1_STATE) are errors Some starter code
