@@ -17,7 +17,7 @@
 #include <stdio.h>
 
 #include "lswtool.h"
-#include "lswfips.h"
+#include "fips_mode.h"
 #include "lswnss.h"
 #include "lswlog.h"			/* for log_to_stderr */
 #include "ike_alg.h"
@@ -128,7 +128,7 @@ static void usage(void)
 int main(int argc, char *argv[])
 {
 	log_to_stderr = false;
-	struct logger *logger = tool_init_log(argv[0]);
+	struct logger *logger = tool_logger(argc, argv);
 
 	if (argc <= 1) {
 		usage();
@@ -151,15 +151,12 @@ int main(int argc, char *argv[])
 
 	/* -fips must come first! */
 	if (strcmp(*argp, "-fips") == 0 || strcmp(*argp, "--fips") == 0) {
-		lsw_set_fips_mode(LSW_FIPS_ON);
+		set_fips_mode(FIPS_MODE_ON);
 		argp++;
 	}
 
 	/* start NSS so crypto works while args are being parsed */
-	diag_t d = lsw_nss_setup(NULL, 0, logger);
-	if (d != NULL) {
-		fatal_diag(1, logger, &d, "%s", "");
-	}
+	init_nss(NULL, (struct nss_flags){0}, logger);
 	init_crypt_symkey(logger);
 
 	for (; *argp != NULL; argp++) {
@@ -267,6 +264,6 @@ int main(int argc, char *argv[])
 		cavp_parser(cavp, logger);
 	}
 
-	lsw_nss_shutdown();
+	shutdown_nss();
 	exit(0);
 }
