@@ -20,8 +20,9 @@
 #include "lswalloc.h"
 #include "ike_alg.h"
 #include "ike_alg_encrypt.h"
-#include "ike_alg_encrypt_ops.h"
+#include "crypt_cipher.h"
 #include "crypt_symkey.h"
+#include "ike_alg_encrypt_ops.h"
 
 #include "cavp.h"
 #include "cavp_entry.h"
@@ -87,7 +88,7 @@ static struct encrypt_desc ike_alg_encrypt_aes_gcm_4 = {
 	.salt_size = AES_GCM_SALT_BYTES,
 	.keydeflen = AES_GCM_KEY_DEF_LEN,
 	.aead_tag_size = 4,
-	.encrypt_ops = &ike_alg_encrypt_nss_gcm_ops,
+	.encrypt_ops = &ike_alg_encrypt_nss_aead_ops,
 };
 
 static struct encrypt_desc ike_alg_encrypt_aes_gcm_13 = {
@@ -105,7 +106,7 @@ static struct encrypt_desc ike_alg_encrypt_aes_gcm_13 = {
 	.salt_size = AES_GCM_SALT_BYTES,
 	.keydeflen = AES_GCM_KEY_DEF_LEN,
 	.aead_tag_size = 13,
-	.encrypt_ops = &ike_alg_encrypt_nss_gcm_ops,
+	.encrypt_ops = &ike_alg_encrypt_nss_aead_ops,
 };
 
 static struct encrypt_desc ike_alg_encrypt_aes_gcm_14 = {
@@ -123,7 +124,7 @@ static struct encrypt_desc ike_alg_encrypt_aes_gcm_14 = {
 	.salt_size = AES_GCM_SALT_BYTES,
 	.keydeflen = AES_GCM_KEY_DEF_LEN,
 	.aead_tag_size = 14,
-	.encrypt_ops = &ike_alg_encrypt_nss_gcm_ops,
+	.encrypt_ops = &ike_alg_encrypt_nss_aead_ops,
 };
 
 static struct encrypt_desc ike_alg_encrypt_aes_gcm_15 = {
@@ -141,7 +142,7 @@ static struct encrypt_desc ike_alg_encrypt_aes_gcm_15 = {
 	.salt_size = AES_GCM_SALT_BYTES,
 	.keydeflen = AES_GCM_KEY_DEF_LEN,
 	.aead_tag_size = 15,
-	.encrypt_ops = &ike_alg_encrypt_nss_gcm_ops,
+	.encrypt_ops = &ike_alg_encrypt_nss_aead_ops,
 };
 
 static const struct encrypt_desc *encrypts[] = {
@@ -157,7 +158,7 @@ static const struct encrypt_desc *encrypts[] = {
 
 static const struct encrypt_desc *lookup_by_taglen(void) {
 	for (const struct encrypt_desc **ep = encrypts; *ep != NULL; ep++) {
-		if ((*ep)->aead_tag_size * BITS_PER_BYTE == taglen) {
+		if ((*ep)->aead_tag_size * BITS_IN_BYTE == taglen) {
 			return *ep;
 		}
 	}
@@ -191,16 +192,13 @@ static void gcm_run_test(struct logger *logger)
 
 	chunk_t text_and_tag = clone_hunk_hunk(ct, tag, "text-and-tag");
 
-	bool result = gcm_alg->encrypt_ops
-		->do_aead(gcm_alg,
-			  salt.ptr, salt.len,
-			  iv.ptr, iv.len,
-			  aad.ptr, aad.len,
-			  text_and_tag.ptr,
-			  ct.len, tag.len,
-			  gcm_key,
-			  false/*encrypt*/,
-			  logger);
+	bool result = cipher_aead(gcm_alg, DECRYPT, USE_WIRE_IV,
+				  HUNK_AS_SHUNK(salt), iv,
+				  HUNK_AS_SHUNK(aad),
+				  text_and_tag,
+				  ct.len, tag.len,
+				  gcm_key,
+				  logger);
 	if (result) {
 		/* plain text */
 		chunk_t pt = {
@@ -211,7 +209,7 @@ static void gcm_run_test(struct logger *logger)
 	} else {
 		print_line("FAIL");
 	}
-	release_symkey(__func__, "GCM-key", &gcm_key);
+	symkey_delref(logger, "GCM-key", &gcm_key);
 	free_chunk_content(&text_and_tag);
 }
 
