@@ -136,7 +136,7 @@ static void UpdateNSS_RNG(int seedbits, struct logger *logger)
 int main(int argc, char *argv[])
 {
 	log_to_stderr = false;
-	struct logger *logger = tool_init_log("ipsec ecdsasigkey");
+	struct logger *logger = tool_logger(argc, argv);
 
 	int opt;
 	int seedbits = DEFAULT_SEED_BITS;
@@ -231,16 +231,12 @@ static void ecdsasigkey(SECOidTag curve, int seedbits,
 	SECItem *ecdsaparams = SEC_ASN1EncodeItem(NULL, NULL, &oiddata->oid,
 						  SEC_ObjectIDTemplate);
 
-	diag_t d = lsw_nss_setup(oco->nssdir, 0, logger);
-
-	if (d != NULL) {
-		fatal_diag(1, logger, &d, "%s", "");
-	}
+	init_nss(oco->nssdir, (struct nss_flags){0}, logger);
 
 	slot = lsw_nss_get_authenticated_slot(logger);
 	if (slot == NULL) {
 		/* already logged */
-		lsw_nss_shutdown();
+		shutdown_nss();
 		exit(1);
 	}
 
@@ -288,7 +284,7 @@ static void ecdsasigkey(SECOidTag curve, int seedbits,
 	if (pubkey != NULL)
 		SECKEY_DestroyPublicKey(pubkey);
 
-	lsw_nss_shutdown();
+	shutdown_nss();
 }
 
 /*
@@ -310,7 +306,7 @@ static void lsw_random(size_t nbytes, unsigned char *buf, struct logger *logger)
 
 	ndone = 0;
 	llog(RC_LOG, logger, "getting %d random seed bytes for NSS from %s...\n",
-		    (int) nbytes * BITS_PER_BYTE, device);
+		    (int) nbytes * BITS_IN_BYTE, device);
 	while (ndone < nbytes) {
 		got = read(dev, buf + ndone, nbytes - ndone);
 		if (got < 0) {
