@@ -206,9 +206,15 @@ struct addresspool {
 
 static struct addresspool *pluto_pools = NULL;
 
+ip_range addresspool_range(struct addresspool *pool)
+{
+	return pool->r;
+}
+
 static void free_lease_content(struct lease *lease)
 {
 	pfreeany(lease->reusable_name);
+	lease->assigned_to = SOS_NOBODY;
 }
 
 static unsigned hasher(const char *name)
@@ -251,48 +257,55 @@ static err_t pool_lease_to_address(const struct addresspool *pool, const struct
 	return range_offset_to_address(pool->r, lease - pool->leases, address);
 }
 
-static void LDBG_pool(struct logger *logger, bool verbose,
+PRINTF_LIKE(4)
+static void vdbg_pool(struct verbose verbose,
 		      const struct addresspool *pool,
-		      const char *format, ...) PRINTF_LIKE(4);
-
-static void LDBG_pool(struct logger *logger, bool verbose, const struct addresspool *pool,
+		      struct connection *c,
 		      const char *format, ...)
 {
-	LLOG_JAMBUF(DEBUG_STREAM, logger, buf) {
+	LDBGP_JAMBUF(DBG_BASE, verbose.logger, buf) {
+		jam(buf, PRI_VERBOSE, pri_verbose);
 		jam(buf, "pool ");
 		jam_range(buf, &pool->r);
+		if (c != NULL) {
+			jam_connection(buf, c);
+			jam_string(buf, " ");
+		}
 		jam(buf, ": ");
 		va_list args;
 		va_start(args, format);
 		jam_va_list(buf, format, args);
 		va_end(args);
-		if (verbose) {
-			jam(buf, "; pool-refcount %u size %u leases %u in-use %u free %u reusable %u",
-			    refcnt_peek(pool, logger), pool->size, pool->nr_leases,
-			    pool->nr_in_use, pool->free_list.nr, pool->nr_reusable);
-		}
+		jam(buf, "; pool-refcount %u size %u leases %u in-use %u free %u reusable %u",
+		    refcnt_peek(pool, verbose.logger),
+		    pool->size, pool->nr_leases,
+		    pool->nr_in_use, pool->free_list.nr, pool->nr_reusable);
 	}
 }
 
-static void LDBG_lease(struct logger *logger, bool verbose,
-		       const struct addresspool *pool, const struct lease *lease,
-		       const char *format, ...) PRINTF_LIKE(5);
-
-static void LDBG_lease(struct logger *logger, bool verbose,
-		       const struct addresspool *pool, const struct lease *lease,
+PRINTF_LIKE(5)
+static void vdbg_lease(struct verbose verbose,
+		       const struct addresspool *pool,
+		       const struct lease *lease,
+		       const struct connection *c,
 		       const char *format, ...)
 {
-	LLOG_JAMBUF(DEBUG_STREAM, logger, buf) {
+	LDBGP_JAMBUF(DBG_BASE, verbose.logger, buf) {
+		jam(buf, PRI_VERBOSE, pri_verbose);
 		jam(buf, "pool ");
 		jam_range(buf, &pool->r);
 		jam(buf, " lease ");
+		if (c != NULL) {
+			jam_connection(buf, c);
+			jam_string(buf, " ");
+		}
 		ip_address addr;
 		err_t err = pool_lease_to_address(pool, lease, &addr);
 		if (err != NULL) {
 			jam(buf, "["PEXPECT_PREFIX"%s]", err);
 		}
 		jam_address(buf, &addr);
-		if (co_serial_is_set(lease->assigned_to)) {
+		if (lease->assigned_to != COS_NOBODY) {
 			jam(buf, " "PRI_CO, pri_co(lease->assigned_to));
 		} else {
 			jam(buf, " unassigned");
@@ -302,11 +315,9 @@ static void LDBG_lease(struct logger *logger, bool verbose,
 		va_start(args, format);
 		jam_va_list(buf, format, args);
 		va_end(args);
-		if (verbose) {
-			jam(buf, "; leases %u in-use %u free %u reusable %u",
-			    pool->nr_leases, pool->nr_in_use,
-			    pool->free_list.nr, pool->nr_reusable);
-		}
+		jam(buf, "; leases %u in-use %u free %u reusable %u",
+		    pool->nr_leases, pool->nr_in_use,
+		    pool->free_list.nr, pool->nr_reusable);
 	}
 }
 
@@ -405,8 +416,9 @@ static bool client_can_reuse_lease(const struct connection *c)
  * that it still does (it may have been stolen by a newer connection
  * with the same ID).
  */
-static struct lease *connection_lease(struct connection *c, const struct ip_info *afi,
-				      struct logger *logger)
+static struct lease *connection_lease(struct connection *c,
+				      const struct ip_info *afi,
+				      struct verbose verbose)
 {
 	/*
 	 * No point looking for a lease when the connection doesn't
@@ -430,7 +442,7 @@ static struct lease *connection_lease(struct connection *c, const struct ip_info
 	uintmax_t offset;
 	err_t err = address_to_range_offset(pool->r, prefix, &offset);
 	if (err != NULL) {
-		llog_pexpect(logger, HERE, "offset of address in range failed: %s", err);
+		llog_pexpect(verbose.logger, HERE, "offset of address in range failed: %s", err);
 		return NULL;
 	}
 	passert(pool->nr_leases <= pool->size);
@@ -438,20 +450,18 @@ static struct lease *connection_lease(struct connection *c, const struct ip_info
 	struct lease *lease = &pool->leases[offset];
 
 	/*
-	 * Has the lease been "stolen" by a newer connection with the
-	 * same ID?
+	 * Has the lease been "stolen" by a newer connection instance
+	 * with the same ID?
 	 */
-	if (co_serial_cmp(lease->assigned_to, >, c->serialno)) {
-		if (LDBGP(DBG_BASE, logger)) {
-			LDBG_lease(logger, true, pool, lease, "stolen by "PRI_CO, pri_co(lease->assigned_to));
-		}
+	if (lease->assigned_to > c->serialno) {
+		vdbg_lease(verbose, pool, lease, NULL, "stolen by "PRI_CO, pri_co(lease->assigned_to));
 		return NULL;
 	}
 	/*
-	 * The lease is still assigned to this connection (if it weren't the
-	 * connection wouldn't have .has_lease).
+	 * The lease is still assigned to this connection instance (if
+	 * it weren't the connection wouldn't have .has_lease).
 	 */
-	if (!pexpect(co_serial_cmp(lease->assigned_to, ==, c->serialno))) {
+	if (!pexpect(lease->assigned_to == c->serialno)) {
 		return NULL;
 	}
 	return lease;
@@ -470,16 +480,20 @@ static struct lease *connection_lease(struct connection *c, const struct ip_info
  * In that case, we do free the lease since that ID isn't distinctive.
  */
 
-void free_that_address_lease(struct connection *c, const struct ip_info *afi, struct logger *logger)
+void free_that_address_lease(struct connection *c,
+			     const struct ip_info *afi,
+			     struct logger *logger)
 {
+	VERBOSE_DBGP(DBG_BASE, logger, "freeing peer %s lease", afi->ip_name);
+
 	if (!c->remote->child.lease[afi->ip_index].is_set) {
-		ldbg(logger, "connection has no %s lease", afi->ip_name);
+		vdbg("connection has no %s lease", afi->ip_name);
 		return;
 	}
 
-	struct lease *lease = connection_lease(c, afi, logger);
+	struct lease *lease = connection_lease(c, afi, verbose);
 	if (lease == NULL) {
-		ldbg(logger, "connection lost its %s lease", afi->ip_name);
+		vdbg("connection lost its %s lease", afi->ip_name);
 		c->remote->child.lease[afi->ip_index] = unset_address;
 		return;
 	}
@@ -490,27 +504,20 @@ void free_that_address_lease(struct connection *c, const struct ip_info *afi, st
 		/* the lease is reusable, leave it lingering */
 		APPEND(pool, free_list, free_entry, lease);
 		pool->nr_in_use--;
-		if (LDBGP(DBG_BASE, logger)) {
-			connection_buf cb;
-			LDBG_lease(logger, true, pool, lease,
-				   "lingering reusable lease '%s' for connection "PRI_CONNECTION,
-				   lease->reusable_name, pri_connection(c, &cb));
-		}
+		vdbg_lease(verbose, pool, lease, c,
+			   "lingering reusable lease '%s' left lingering",
+			   lease->reusable_name);
 	} else {
 		/* cannot share: free it */
 		PREPEND(pool, free_list, free_entry, lease);
 		pool->nr_in_use--;
-		if (LDBGP(DBG_BASE, logger)) {
-			connection_buf cb;
-			LDBG_lease(logger, true, pool, lease,
-				   "returning one-time lease for connection "PRI_CONNECTION,
-				   pri_connection(c, &cb));
-		}
+		vdbg_lease(verbose, pool, lease, c,
+			   "returning one-time lease");
 	}
 
 	/* break the link */
 	c->remote->child.lease[afi->ip_index] = unset_address;
-	lease->assigned_to = UNSET_CO_SERIAL;
+	lease->assigned_to = COS_NOBODY;
 }
 
 /*
@@ -518,7 +525,8 @@ void free_that_address_lease(struct connection *c, const struct ip_info *afi, st
  */
 
 static struct lease *recover_lease(const struct connection *c, const char *that_name,
-				   const struct ip_info *afi, struct logger *logger)
+				   const struct ip_info *afi,
+				   struct verbose verbose)
 {
 	struct addresspool *pool = c->pool[afi->ip_index];
 	if (pool->nr_leases == 0) {
@@ -540,23 +548,18 @@ static struct lease *recover_lease(const struct connection *c, const char *that_
 			if (IS_INSERTED(lease, free_entry)) {
 				/* unused */
 				REMOVE(pool, free_list, free_entry, lease);
-				pexpect(co_serial_is_unset(lease->assigned_to));
+				pexpect(lease->assigned_to == COS_NOBODY);
 				pool->nr_in_use++;
-				if (LDBGP(DBG_BASE, logger)) {
-					connection_buf cb;
-					LDBG_lease(logger, false, pool, lease,
-						   "recovered by "PRI_CONNECTION" using '%s'; was on free-list",
-						   pri_connection(c, &cb), that_name);
-				}
+				vdbg_lease(verbose, pool, lease, c,
+					   "recovered using '%s'; was on free-list",
+					   that_name);
 			} else {
-				/* still assigned to older connection */
-				pexpect(co_serial_cmp(lease->assigned_to, <, c->serialno));
-				if (LDBGP(DBG_BASE, logger)) {
-					connection_buf cb;
-					LDBG_lease(logger, false, pool, lease,
-						   "recovered by "PRI_CONNECTION" using '%s'; was in use by "PRI_CO,
-						   pri_connection(c, &cb), that_name, pri_co(lease->assigned_to));
-				}
+				/* still assigned to older connection
+				 * instance */
+				pexpect(lease->assigned_to < c->serialno);
+				vdbg_lease(verbose, pool, lease, c,
+					   "recovered using '%s'; was in use by "PRI_CO,
+					   that_name, pri_co(lease->assigned_to));
 			}
 			return lease;
 		}
@@ -564,232 +567,333 @@ static struct lease *recover_lease(const struct connection *c, const char *that_
 	return NULL;
 }
 
-
-err_t lease_that_selector(struct connection *c, const char *xauth_username,
-			  const ip_selector *remote_client, struct logger *logger)
+static err_t grow_addresspool(struct addresspool *pool,
+			      struct verbose verbose)
 {
-	const struct ip_info *afi = selector_type(remote_client);
-	unsigned next_lease_nr = nr_child_leases(c->remote);
-
-	if (c->remote->child.lease[afi->ip_index].is_set &&
-	    connection_lease(c, afi, logger) != NULL) {
-		ldbg(logger, "connection both thinks it has, and really has a lease");
-		return NULL;
+	/* try to grow the address pool */
+	if (pool->nr_leases >= pool->size) {
+		vdbg_pool(verbose, pool, NULL, "address pool exhausted: %u >= %u",
+			  pool->nr_leases, pool->size);
+		return "address pool exhausted";
 	}
 
-	struct addresspool *pool = c->pool[afi->ip_index];
-	if (pool == NULL) {
-		return "no address pool";
+	unsigned old_nr_leases = pool->nr_leases;
+	if (pool->nr_leases == 0) {
+		pool->nr_leases = min(1U, pool->size);
+	} else {
+		pool->nr_leases = min(pool->nr_leases * 2, pool->size);
 	}
+	realloc_things(pool->leases, old_nr_leases, pool->nr_leases, "leases");
 
-	bool reusable = client_can_reuse_lease(c);
-	if (!reusable) {
-		return "lease is not reusable";
+	range_buf rb;
+	llog(RC_LOG, verbose.logger, "pool %s: growing address pool from %u to %u",
+	     str_range(&pool->r, &rb), old_nr_leases, pool->nr_leases);
+
+	/* initialize new leases (and add to free list) */
+	for (unsigned l = old_nr_leases; l < pool->nr_leases; l++) {
+		struct lease *lease = &pool->leases[l];
+		/*
+		 * Danger: must initialize entire
+		 * struct as resize_things(), which
+		 * may use realloc(), can leave the
+		 * data uninitialized.
+		 */
+		*lease = (struct lease) {
+			.free_entry = empty_entry,
+			.reusable_entry = empty_entry,
+			.reusable_bucket = empty_list,
+		};
+		PREPEND(pool, free_list, free_entry, lease);
 	}
-
-	/*
-	 * Combine the ID with the XAUTH_USERNAME so that, when xauth
-	 * with a shared ID is used, the result is still unique.
-	 */
-	id_buf remote_idb; /* same scope as remote_id */
-	const char *remote_id = str_id(&c->remote->host.id, &remote_idb);
-	char thatstr[sizeof(id_buf) + MAX_XAUTH_USERNAME_LEN];
-	jam_str(thatstr, sizeof(thatstr), remote_id);
-	if (xauth_username != NULL && xauth_username[0] != '\0') {
-		add_str(thatstr, sizeof(thatstr), thatstr, xauth_username);
+	/* destroy existing hash table */
+	for (unsigned l = 0; l < old_nr_leases; l++) {
+		struct lease *lease = &pool->leases[l];
+		lease->reusable_entry = empty_entry;
+		lease->reusable_bucket = empty_list;
 	}
-
-	if (LDBGP(DBG_BASE, logger)) {
-		connection_buf cb;
-		LDBG_pool(logger, false, pool, "requesting %s lease for connection "PRI_CONNECTION" with '%s'",
-			  reusable ? "reusable" : "one-time",
-			  pri_connection(c, &cb), thatstr);
+	/* build a new hash table containing old */
+	pool->nr_reusable = 0;
+	for (unsigned l = 0; l < old_nr_leases; l++) {
+		struct lease *lease = &pool->leases[l];
+		if (lease->reusable_name != NULL) {
+			hash_lease_id(pool, lease);
+		}
 	}
 
-	struct lease *new_lease = recover_lease(c, thatstr, afi, logger);
-	if (new_lease == NULL) {
-		return "no lease";
-	}
-
-	ip_address ia;
-	err_t err = pool_lease_to_address(pool, new_lease, &ia);
-	if (err != NULL) {
-		llog_pexpect(logger, HERE, "%s", err);
-		free_that_address_lease(c, afi, logger);
-		return "bogus lease";
-	}
-
-	ip_selector is = selector_from_address(ia);
-	if (!selector_eq_selector(is, *remote_client)) {
-		free_that_address_lease(c, afi, logger);
-		return "wrong address";
-	}
-
-	scribble_remote_lease(c, ia, next_lease_nr, logger, HERE);
-
-	/* back link */
-	new_lease->assigned_to = c->serialno;
-
 	return NULL;
 }
 
-err_t lease_that_address(struct connection *c, const char *xauth_username,
-			 const struct ip_info *afi, struct logger *logger)
+/*
+ * Remove a lease from the .free_list which has the effect of making
+ * it in-use.
+ *
+ * When (*NEW_OWNER), steal the value, setting .reusable_name (and
+ * update hash table).
+ *
+ * Note: this only updates the lease.  The caller still needs to
+ * assign the lease to the connection.
+ */
+
+static bool unfree_lease(struct addresspool *pool,
+			 char **new_owner,
+			 struct lease *new_lease,
+			 struct verbose verbose)
 {
+	bool stolen = false;
+	REMOVE(pool, free_list, free_entry, new_lease);
+	pool->nr_in_use++;
+
+	/* unhash before freeing */
+	if (new_lease->reusable_name != NULL) {
+		/* oops; takeing over this lingering lease */
+		vdbg_lease(verbose, pool, new_lease, NULL,
+			   "stealing reusable lease from '%s'",
+			   new_lease->reusable_name);
+		unhash_lease_id(pool, new_lease);
+		stolen = true;
+	}
+
+	free_lease_content(new_lease);
+	if ((*new_owner) != NULL) {
+		/* steal reference */
+		new_lease->reusable_name = (*new_owner);
+		(*new_owner) = NULL;
+		hash_lease_id(pool, new_lease);
+	}
+
+	return stolen;
+}
+
+/*
+ * returns ERR_T when something fatal happens, else leaves NEW_LEASE
+ * NULL when lease can't be assigned.
+ */
+static err_t assign_requested_lease(struct connection *c,
+				    struct addresspool *pool,
+				    char **reusable_id,
+				    const ip_address *lease_address,
+				    struct lease **new_lease,
+				    const char **story,
+				    struct verbose verbose)
+{
+	address_buf lab;
+	vdbg("assigning the requested lease %s",
+	     str_address_sensitive(lease_address, &lab));
+	verbose.level++;
+	/*
+	 * Determine the requested LEASE_ADDRESS's offset into the
+	 * address pool.
+	 *
+	 * Earlier code has checked that the LEASE_ADDRESS is within
+	 * the address pool's range, hence pexpect.
+	 */
+	uintmax_t offset;
+	err_t err = address_to_range_offset(pool->r, (*lease_address), &offset);
+	if (err != NULL) {
+		llog_pexpect(verbose.logger, HERE, "offset of address in range failed: %s", err);
+		return "confused, address should be within addresspool";
+	}
+
+	vassert(pool->nr_leases <= pool->size);
+	vassert(offset < pool->size); /* by above */
+
+	if (offset < pool->nr_leases) {
+		/*
+		 * Is the lease available?
+		 *
+		 * IKEv1: must fail as Quick Mode has no way to send
+		 * back an alternative lease.
+		 *
+		 * IKEv2: should not fail, instead the proposed lease
+		 * should be ignored and a new one assigned.
+		 *
+		 * Later.
+		 */
+		struct lease *lease = &pool->leases[offset];
+		if (lease->assigned_to != COS_NOBODY) {
+			vdbg_lease(verbose, pool, lease, c, "owned by "PRI_CO,
+				   pri_co(lease->assigned_to));
+			return "lease address is in use";
+		}
+
+		if (lease->reusable_name != NULL) {
+			vdbg_lease(verbose, pool, lease, c, "owned by "PRI_CO,
+				   pri_co(lease->assigned_to));
+			return "lease address is reserved";
+		}
+
+		/*
+		 * This returns true when the lease had a previous
+		 * owner and the story needs to be updated.
+		 */
+		if (vbad(unfree_lease(pool, reusable_id, lease, verbose))) {
+			return "confused, unreserved lease was stolen";
+		}
+
+		vassert((*reusable_id) == NULL); /* ownership transfered to lease */
+		(*new_lease) = lease;
+		(*story) = "request available";
+		return NULL;
+	}
+
+	/* grow the lease if necessary */
+	while (offset >= pool->nr_leases) {
+		grow_addresspool(pool, verbose);
+	}
+
+	vassert(offset < pool->nr_leases);
+	struct lease *lease = &pool->leases[offset];
+	if (!pexpect(lease->assigned_to == COS_NOBODY)) {
+		return "confused, just allocated lease in use";
+	}
+
+	/* fresh lease can't have previous owner */
+	if (vbad(unfree_lease(pool, reusable_id, lease, verbose))) {
+		return "confused, just allocated lease was stolen";
+	}
+
+	vassert((*reusable_id) == NULL); /* ownership transfered to lease */
+	(*new_lease) = lease;
+	(*story) = "request grown";
+	return NULL;
+}
+
+err_t assign_remote_lease(struct connection *c,
+			  const char *xauth_username,
+			  const struct ip_info *afi,
+			  const ip_address *lease_address,
+			  struct logger *logger)
+{
+	VERBOSE_DBGP(DBG_BASE, logger, "%s() xauth=%s family=%s",
+		     __func__, (xauth_username == NULL ? "n/a" : xauth_username),
+		     afi->ip_name);
+
 	if (c->remote->child.lease[afi->ip_index].is_set &&
-	    connection_lease(c, afi, logger) != NULL) {
+	    connection_lease(c, afi, verbose) != NULL) {
 		ldbg(logger, "connection both thinks it has, and really has a lease");
 		return NULL;
 	}
 
 	struct addresspool *pool = c->pool[afi->ip_index];
-	if (pool == NULL) {
-		return "no address pool";
+	if (PBAD(logger, pool == NULL)) {
+		return "confused, no address pool";
 	}
 
 	unsigned next_lease_nr = nr_child_leases(c->remote);
-	bool reusable = client_can_reuse_lease(c);
 
 	/*
-	 * Combine the ID with the XAUTH_USERNAME so that, when xauth
-	 * with a shared ID is used, the result is still unique.
+	 * When the lease is reusable generate a unique ID by
+	 * conbining the ID (from IKE_AUTH et.al.) with the
+	 * XAUTH_USERNAME (from XAUTH) so that, when when a shared ID
+	 * is used with XAUTH the result is still unique.
 	 */
-	id_buf remote_idb; /* same scope as remote_id */
-	const char *remote_id = str_id(&c->remote->host.id, &remote_idb);
-	char thatstr[sizeof(id_buf) + MAX_XAUTH_USERNAME_LEN];
-	jam_str(thatstr, sizeof(thatstr), remote_id);
-	if (xauth_username != NULL && xauth_username[0] != '\0') {
-		add_str(thatstr, sizeof(thatstr), thatstr, xauth_username);
+
+	char *reusable_id = NULL;
+	if (client_can_reuse_lease(c)) {
+		id_buf remote_idb; /* same scope as remote_id */
+		const char *remote_id = str_id(&c->remote->host.id, &remote_idb);
+		reusable_id = alloc_printf("%s%s", remote_id, (xauth_username != NULL ? xauth_username : ""));
 	}
 
-	if (LDBGP(DBG_BASE, logger)) {
-		connection_buf cb;
-		LDBG_pool(logger, false, pool, "requesting %s lease for connection "PRI_CONNECTION" with '%s'",
-			  reusable ? "reusable" : "one-time",
-			  pri_connection(c, &cb), thatstr);
-	}
+	vdbg_pool(verbose, pool, c,
+		  "requesting lease with reusable ID '%s'",
+		  (reusable_id == NULL ? "" : reusable_id));
 
 	struct lease *new_lease = NULL;
-	const char *story;
-	if (reusable) {
-		new_lease = recover_lease(c, thatstr, afi, logger);
-		story = "recovered";
-	}
-	if (new_lease == NULL) {
-		if (IS_EMPTY(pool, free_list)) {
-			/* try to grow the address pool */
-			if (pool->nr_leases >= pool->size) {
-				if (LDBGP(DBG_BASE, logger)) {
-					LDBG_pool(logger, true, pool, "no free address and no space to grow");
-				}
-				return "no free address in addresspool"; /* address pool exhausted */
-			}
-			unsigned old_nr_leases = pool->nr_leases;
-			if (pool->nr_leases == 0) {
-				pool->nr_leases = min(1U, pool->size);
-			} else {
-				pool->nr_leases = min(pool->nr_leases * 2, pool->size);
-			}
-			realloc_things(pool->leases, old_nr_leases, pool->nr_leases, "leases");
-
-			range_buf rb;
-			llog(RC_LOG, logger, "pool %s: growing address pool from %u to %u",
-			     str_range(&pool->r, &rb), old_nr_leases, pool->nr_leases);
-
-			/* initialize new leases (and add to free list) */
-			for (unsigned l = old_nr_leases; l < pool->nr_leases; l++) {
-				struct lease *lease = &pool->leases[l];
-				/*
-				 * Danger: must initialize entire
-				 * struct as resize_things(), which
-				 * may use realloc(), can leave the
-				 * data uninitialized.
-				 */
-				*lease = (struct lease) {
-					.free_entry = empty_entry,
-					.reusable_entry = empty_entry,
-					.reusable_bucket = empty_list,
-				};
-				PREPEND(pool, free_list, free_entry, lease);
-			}
-			/* destroy existing hash table */
-			for (unsigned l = 0; l < old_nr_leases; l++) {
-				struct lease *lease = &pool->leases[l];
-				lease->reusable_entry = empty_entry;
-				lease->reusable_bucket = empty_list;
-			}
-			/* build a new hash table containing old */
-			pool->nr_reusable = 0;
-			for (unsigned l = 0; l < old_nr_leases; l++) {
-				struct lease *lease = &pool->leases[l];
-				if (lease->reusable_name != NULL) {
-					hash_lease_id(pool, lease);
-				}
-			}
-		}
-		new_lease = HEAD(pool, free_list, free_entry);
-		passert(new_lease != NULL);
-		REMOVE(pool, free_list, free_entry, new_lease);
-		pool->nr_in_use++;
-		if (new_lease->reusable_name != NULL) {
-			/* oops; takeing over this lingering lease */
-			if (LDBGP(DBG_BASE, logger)) {
-				LDBG_lease(logger, false, pool, new_lease,
-					   "stealing reusable lease from '%s'",
-					   new_lease->reusable_name);
-			}
-			unhash_lease_id(pool, new_lease);
-			story = "stolen";
-		} else {
-			story = "unused";
-		}
-		free_lease_content(new_lease);
-		if (reusable) {
-			new_lease->reusable_name = clone_str(thatstr, "lease name");
-			hash_lease_id(pool, new_lease);
+	const char *story = NULL;
 
+	/*
+	 * Using the mangled ID (THATSTR), see of there is an existing
+	 * lease.
+	 */
+	if (new_lease == NULL && reusable_id != NULL) {
+		new_lease = recover_lease(c, reusable_id, afi, verbose);
+		if (new_lease != NULL) {
+			story = "recovered";
+			pfreeany(reusable_id); /* not needed */
 		}
 	}
 
 	/*
-	 * convert index i in range to an IP_address
-	 *
-	 * XXX: does this update that.client addr as a side effect?
-	 *
-	 * Can't assert that .assigned_to is unset as this connection
-	 * may be in the process of stealing the lease.
+	 * If the peer's given a prefered address try to assign that.
+	 */
+	if (new_lease == NULL && is_set(lease_address)) {
+		err_t e = assign_requested_lease(c, pool, &reusable_id, lease_address,
+						 &new_lease, &story, verbose);
+		if (e != NULL) {
+			pfreeany(reusable_id);
+			return e;
+		}
+	}
+
+	/*
+	 * Allocate the next lease from the free list; if necessary,
+	 * grow the pool.
+	 */
+	if (new_lease == NULL) {
+		if (IS_EMPTY(pool, free_list)) {
+			err_t e = grow_addresspool(pool, verbose);
+			if (e != NULL) {
+				pfreeany(reusable_id);
+				return e;
+			}
+		}
+
+		/* grab the next lease on the free list */
+		new_lease = HEAD(pool, free_list, free_entry);
+		vassert(new_lease != NULL);
+		story = (unfree_lease(pool, &reusable_id, new_lease, verbose)
+			 ? "stolen"
+			 : "unused");
+		vassert(reusable_id == NULL); /* stolen */
+	}
+
+	/*
+	 * No lease (above should have returned), but play save.
+	 */
+
+	if (vbad(new_lease == NULL)) {
+		pfreeany(reusable_id);
+		return "confused, no lease";
+	}
+
+	if (vbad(story == NULL)) {
+		return "confused, no story";
+	}
+
+	if (vbad(reusable_id != NULL)) {
+		return "confused, leaking memory";
+	}
+
+	/*
+	 * Convert the leases offset into range, into an IP_address.
 	 */
 	ip_address ia;
 	err_t err = pool_lease_to_address(pool, new_lease, &ia);
 	if (err != NULL) {
 		llog_pexpect(logger, HERE, "%s", err);
-		return "bad address";
+		return "confused, bad address";
 	}
 
+	/* assign and back link */
 	scribble_remote_lease(c, ia, next_lease_nr, logger, HERE);
-
-	/* back link */
 	new_lease->assigned_to = c->serialno;
 
-	if (LDBGP(DBG_BASE, logger)) {
-		address_buf ab;
-		connection_buf cb;
-		LDBG_lease(logger, true, pool, new_lease,
-			   "assign %s %s lease to "PRI_CONNECTION" "PRI_CO" with ID '%s' and that.lease %s",
-			   story,
-			   (reusable ? "reusable" : "one-time"),
-			   pri_connection(c, &cb),
-			   pri_co(new_lease->assigned_to),
-			   thatstr,
-			   str_address(&ia, &ab));
-	}
+	address_buf ab;
+	vdbg_lease(verbose, pool, new_lease, c,
+		   "assign %s lease to "PRI_CO" with reusable ID '%s' and that.lease %s",
+		   story,
+		   pri_co(new_lease->assigned_to),
+		   (new_lease->reusable_name == NULL ? "" : new_lease->reusable_name),
+		   str_address(&ia, &ab));
 
 	return NULL;
 }
 
 void addresspool_delref(struct addresspool **poolparty, struct logger *logger)
 {
-	struct addresspool *pool = delref_where(poolparty, logger, HERE);
+	VERBOSE_DBGP(DBG_BASE, logger, "releasing address pool");
+	struct addresspool *pool = delref_where(poolparty, verbose.logger, HERE);
 	if (pool != NULL) {
 		for (struct addresspool **pp = &pluto_pools; *pp != NULL; pp = &(*pp)->next) {
 			if (*pp == pool) {
@@ -802,9 +906,7 @@ void addresspool_delref(struct addresspool **poolparty, struct logger *logger)
 				return;
 			}
 		}
-		if (LDBGP(DBG_BASE, logger)) {
-			LDBG_pool(logger, false, pool, "pool %p not found in list of pools", pool);
-		}
+		vdbg_pool(verbose, pool, NULL, "pool %p not found in list of pools", pool);
 	}
 }
 
@@ -826,7 +928,7 @@ struct addresspool *addresspool_addref(struct addresspool *pool)
  * Otherwise (nothing matches), return NULL and *POOL=NULL.
  */
 
-diag_t find_addresspool(const ip_range pool_range, struct addresspool **pool)
+static diag_t find_addresspool(const ip_range pool_range, struct addresspool **pool)
 {
 	struct addresspool *h;
 
@@ -842,7 +944,7 @@ diag_t find_addresspool(const ip_range pool_range, struct addresspool **pool)
 		if (range_overlaps_range(pool_range, h->r)) {
 			/* bad */
 			range_buf hbuf;
-			return diag("range INEXACTLY OVERLAPS existing address pool %s.",
+			return diag("range inexactly overlaps existing address pool %s",
 				    str_range(&h->r, &hbuf));
 		}
 	}
@@ -853,23 +955,41 @@ diag_t find_addresspool(const ip_range pool_range, struct addresspool **pool)
  * Create an address pool for POOL_RANGE.  Reject invalid ranges.
  */
 
-diag_t install_addresspool(const ip_range pool_range, struct connection *c, struct logger *logger)
+diag_t install_addresspool(const ip_range pool_range,
+			   struct addresspool *addresspool[],
+			   struct logger *logger)
 {
+	range_buf rb;
+	VERBOSE_DBGP(DBG_BASE, logger, "installing address pool %s",
+		     str_range(&pool_range, &rb));
+
 	/* can't be empty */
 	uintmax_t pool_size = range_size(pool_range);
 	if (pool_size == 0) {
+		/*
+		 * Minimum address pool size is 1, so can't happen
+		 * (unless caller fed us an invalid address pool).
+		 */
 		range_buf rb;
-		return diag("address pool %s is empty",
-			    str_range(&pool_range, &rb));
+		llog_pexpect(logger, HERE, "address pool %s is empty",
+			     str_range(&pool_range, &rb));
+		return diag("confused, address pool is empty");
 	}
 
 	if (pool_size >= UINT32_MAX) {
 		/*
+		 * POOL_SIZE is truncated to UINTMAX_MAX when there's
+		 * major overflow.  Hence, it's value isn't always
+		 * correct.
+		 *
 		 * uint32_t overflow, 2001:db8:0:3::/64 truncated to UINT32_MAX
 		 * uint32_t overflow, 2001:db8:0:3:1::/96, truncated by 1
 		 */
+		humber_buf psb;
 		pool_size = UINT32_MAX;
-		ldbg(logger, "WARNING addresspool size overflow truncated to %ju", pool_size);
+		llog(RC_LOG, logger, "warning: limiting the address pool %s to %s addresses",
+		     str_range(&pool_range, &rb),
+		     str_humber(pool_size, &psb));
 	}
 
 	/* can't start at 0 */
@@ -888,16 +1008,16 @@ diag_t install_addresspool(const ip_range pool_range, struct connection *c, stru
 	}
 
 	const struct ip_info *afi = range_info(pool_range);
-	if (c->pool[afi->ip_index] != NULL) {
-		return diag("connection already has a %s address pool", afi->ip_name);
+	if (addresspool[afi->ip_index] != NULL) {
+		llog_pexpect(verbose.logger, HERE,
+			     "connection already has a %s address pool", afi->ip_name);
+		return diag("confused, connection has an address pool");
 	}
 
 	if (existing_pool != NULL) {
 		/* re-use existing pool */
-		if (LDBGP(DBG_BASE, logger)) {
-			LDBG_pool(logger, true, existing_pool, "reusing existing address pool@%p", existing_pool);
-		}
-		c->pool[afi->ip_index] = addresspool_addref(existing_pool);
+		vdbg_pool(verbose, existing_pool, NULL, "reusing existing address pool@%p", existing_pool);
+		addresspool[afi->ip_index] = addresspool_addref(existing_pool);
 		return NULL;
 	}
 
@@ -914,10 +1034,9 @@ diag_t install_addresspool(const ip_range pool_range, struct connection *c, stru
 	new_pool->next = pluto_pools;
 	pluto_pools = new_pool;
 
-	if (LDBGP(DBG_BASE, logger)) {
-		LDBG_pool(logger, false, new_pool, "creating new address pool@%p", new_pool);
-	}
-	c->pool[afi->ip_index] = new_pool;
+	vdbg_pool(verbose, new_pool, NULL, "creating new address pool@%p", new_pool);
+
+	addresspool[afi->ip_index] = new_pool;
 	return NULL;
 }
 
