@@ -23,23 +23,24 @@
 
 struct addresspool;        /* abstract object */
 
-diag_t find_addresspool(const ip_range pool_range, struct addresspool **pool) MUST_USE_RESULT;
-
-diag_t install_addresspool(const ip_range pool_range, struct connection *c,
+diag_t install_addresspool(const ip_range pool_range,
+			   struct addresspool *addresspool[],
 			   struct logger *logger) MUST_USE_RESULT;
+
 void addresspool_delref(struct addresspool **pool, struct logger *logger);
 struct addresspool *addresspool_addref(struct addresspool *pool);
 
-extern err_t lease_that_selector(struct connection *c,
-				 const char *xauth_username/*possibly-NULL|NUL*/,
-				 const ip_selector *selector,
-				 struct logger *logger);
+err_t assign_remote_lease(struct connection *c,
+			  const char *xauth_username/*possibly-NULL|NUL*/,
+			  const struct ip_info *afi,
+			  const ip_address *lease_address,
+			  struct logger *logger);
 
-extern err_t lease_that_address(struct connection *c, const char *xauth_username/*possibly-NULL|NUL*/,
-				const struct ip_info *afi, struct logger *logger);
 extern void free_that_address_lease(struct connection *c, const struct ip_info *afi,
 				    struct logger *logger);
 
+ip_range addresspool_range(struct addresspool *pool);
+
 extern void show_addresspool_status(struct show *s);
 
 #endif /* _ADDRESSPOOL_H */
