@@ -189,8 +189,14 @@ void binlog_state(struct state *st, enum state_kind new_state)
 	{
 		const struct finite_state *save_state = st->st_state;
 		st->st_state = finite_states[new_state];
-		struct state_filter sf = { .where = HERE, };
-		while (next_state(NEW2OLD, &sf)) {
+		struct state_filter sf = {
+			.search = {
+				.order = NEW2OLD,
+				.verbose.logger = &global_logger,
+				.where = HERE,
+			},
+		};
+		while (next_state(&sf)) {
 			connection_state(sf.st, &lc);
 		}
 		st->st_state = save_state;
