@@ -73,11 +73,11 @@ struct task_handler cert_decode_handler = {
 };
 
 void submit_v2_cert_decode(struct ike_sa *ike,
-			struct msg_digest *md, struct payload_digest *cert_payloads,
-			cert_decode_cb *cb, const char *why)
+			   struct msg_digest *md, struct payload_digest *cert_payloads,
+			   cert_decode_cb *cb, where_t where)
 {
 	struct task task = {
-		.root_certs = root_certs_addref(HERE),
+		.root_certs = root_certs_addref(&global_logger),
 		.md = md_addref(md),
 		.cert_payloads = cert_payloads,
 		.cb = cb,
@@ -90,10 +90,10 @@ void submit_v2_cert_decode(struct ike_sa *ike,
 			.crl_strict = crl_strict,
 		},
 	};
-	submit_task(ike->sa.st_logger,
-		    /*resume IKE*/&ike->sa,
+	submit_task(/*callback*/&ike->sa, /*task*/&ike->sa, md,
+		    /*detach_whack*/false,
 		    clone_thing(task, "decode certificate payload task"),
-		    &cert_decode_handler, why);
+		    &cert_decode_handler, where);
 }
 
 static void cert_decode_computer(struct logger *logger,
@@ -131,7 +131,7 @@ static stf_status cert_decode_completed(struct state *st,
 		 */
 		chunk_t fdn = empty_chunk;
 		if (find_crl_fetch_dn(&fdn, ike->sa.st_connection)) {
-			submit_crl_fetch_request(ASN1(fdn), ike->sa.st_logger);
+			submit_crl_fetch_request(ASN1(fdn), ike->sa.logger);
 		}
 		pexpect(task->verified.cert_chain == NULL);
 		pexpect(task->verified.pubkey_db == NULL);
@@ -144,6 +144,7 @@ static stf_status cert_decode_completed(struct state *st,
 
 	pexpect(st->st_remote_certs.verified == NULL);
 	ike->sa.st_remote_certs.verified = task->verified.cert_chain;
+	ike->sa.st_remote_certs.groundhog = task->verified.groundhog;
 	task->verified.cert_chain = NULL;
 
 	pexpect(ike->sa.st_remote_certs.pubkey_db == NULL);
@@ -167,7 +168,7 @@ static stf_status cert_decode_completed(struct state *st,
 		pexpect(ike->sa.st_remote_certs.verified == NULL);
 		pexpect(ike->sa.st_remote_certs.pubkey_db == NULL);
 		/* NSS: already logged details */
-		log_state(RC_LOG, &ike->sa, "X509: certificate payload rejected for this connection");
+		llog_sa(RC_LOG, ike, "X509: certificate payload rejected for this connection");
 		if (ike->sa.st_sa_role == SA_INITIATOR) {
 			/*
 			 * One of the certs was bad; no point switching
@@ -192,6 +193,6 @@ static void cert_decode_cleanup(struct task **task)
 	release_certs(&(*task)->verified.cert_chain);	/* may be NULL */
 	free_public_keys(&(*task)->verified.pubkey_db);	/* may be NULL */
 	md_delref(&(*task)->md);
-	root_certs_delref(&(*task)->root_certs, HERE);
+	root_certs_delref(&(*task)->root_certs, GLOBAL_LOGGER);
 	pfreeany((*task));
 }
