@@ -14,6 +14,7 @@
  * Copyright (C) 2013 Matt Rogers <mrogers@redhat.com>
  * Copyright (C) 2019-2022 Andrew Cagney <cagney@gnu.org>
  * Copyright (C) 2017 Mayank Totale <mtotale@gmail.com>
+ * Copyright (C) 2020 Nupur Agrawal <nupur202000@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -39,6 +40,8 @@
 #include "ip_packet.h"
 #include "refcnt.h"
 #include "encap_mode.h"
+#include "verbose.h"
+#include "end.h"
 
 #include "defs.h"
 #include "proposals.h"
@@ -68,14 +71,6 @@ struct kernel_acquire;
 
 struct connection *connection_by_serialno(co_serial_t serialno);
 
-/*
- * An extract of the original configuration information for
- * the connection's end sent over by whack.
- */
-
-enum left_right { LEFT_END, RIGHT_END, };
-#define END_ROOF 2
-
 /*
  * IKE SA configuration.
  *
@@ -88,6 +83,7 @@ struct host_end_config {
 
 	char *addr_name;	/* string version from whack */
 
+	ip_address nexthop;
 	unsigned ikeport;
 	enum keyword_host type;
 	enum tcp_options iketcp;	/* Allow TCP as fallback,
@@ -99,6 +95,7 @@ struct host_end_config {
 	enum keyword_auth auth;
 	struct authby authby;
 
+	struct id id;			/* or ID_NONE aka %any aka set to host-addr */
 	cert_t cert;			/* end certificate */
 	enum certpolicy sendcert;	/* whether or not to send the certificate */
 	chunk_t ca;			/* CA distinguished name of the end certificate's issuer */
@@ -112,8 +109,6 @@ struct host_end_config {
 		bool client;	/* request address for local end */
 	} modecfg;
 
-	ip_ranges pool_ranges;	/* see also connection .pool */
-
 	struct {
 		bool server;
 		bool client;
@@ -144,6 +139,19 @@ struct child_end_config {
 
 	ip_selectors selectors;
 	ip_addresses sourceip;
+
+	/*
+	 * During orient, this is turned into c->pool
+	 *
+	 * Danger: IKEv1 further updates c->pool during XAUTH using
+	 * something from a file.
+	 *
+	 * Should more than one pool for each address be allowed?
+	 * Should an addresspool combined with selectors be allowed?
+	 */
+	ip_ranges addresspools;
+	struct addresspool *addresspool[IP_INDEX_ROOF]; /* list? */
+
 	/*
 	 * When set, put this on the ipsec-interface.  Should there be
 	 * one for IPv4 and one for IPv6 like sourceip?
@@ -164,7 +172,7 @@ struct child_end_config {
 };
 
 struct config_end {
-	enum left_right index;
+	enum end index;
 	const char *leftright;
 	struct host_end_config host;
 	struct child_end_config child;
@@ -212,7 +220,7 @@ struct config {
 
 	reqid_t sa_reqid;
 
-	/* RFC 8784 and draft-smyslov-ipsecme-ikev2-qr-alt-07 */
+	/* RFC 8784 and draft-ietf-ipsecme-ikev2-qr-alt-04 */
 	char *ppk_ids;
 	struct shunks *ppk_ids_shunks;
 
@@ -260,7 +268,7 @@ struct config {
 
 	bool rekey;				/* rekey state either Phase */
 	bool reauth;				/* IKEv2 only initiate re-authentication */
-	bool ikev2_allow_narrowing;
+	bool narrowing;
 	bool send_no_esp_tfc;
 	bool send_initial_contact;		/* Send INITIAL_CONTACT (RFC-2407) payload? */
 	bool send_vendorid;			/* Send our vendorid? Security vs Debugging help */
@@ -290,8 +298,14 @@ struct config {
 					 * authorize call on ID */
 	bool ignore_peer_dns;		/* install obtained DNS
 					 * servers locally */
-	bool ikepad;			/* pad ike packets to 4 bytes
-					 * or not */
+	struct {
+		bool message;		/* per RFC, pad message to 4
+					 * bytes */
+		bool modecfg;		/* per broken libreswan, pad
+					 * some modecfg packets to 4
+					 * bytes */
+	} v1_ikepad;
+
 	bool require_id_on_certificate;	/* require certificates to
 					 * have IKE ID on cert SAN */
 	bool aggressive;		/* do we do aggressive
@@ -322,6 +336,9 @@ struct config {
 	msgid_t ike_window;		/* IKE v2 window size
 					 * 7296#section-2.3 */
 
+	bool session_resumption;	/* RFC 5723 - IKEv2 Session
+					 * Resumption */
+
 	struct {
 		char *interface;
 		bool routing;		/* should updown perform
@@ -343,6 +360,17 @@ struct config {
 		uint32_t metric;	/* metric for tunnel routes */
 		uint16_t mtu;		/* mtu for tunnel routes */
 		bool ipcomp;
+
+		struct config_iptfs {
+			bool enabled;
+			bool fragmentation;
+			uintmax_t packet_size;
+			uintmax_t max_queue_size;
+			uintmax_t reorder_window;
+			deltatime_t drop_time;
+			deltatime_t init_delay;
+		} iptfs;
+
 		enum encap_proto encap_proto;	/* ESP or AH */
 		enum encap_mode encap_mode;	/* tunnel or transport */
 		bool pfs;			/* use DH */
@@ -378,7 +406,7 @@ struct config {
 		bool insist;
 	} ppk;
 
-	struct {
+	struct ipsec_interface_config {
 		bool enabled;
 		uint32_t id;
 	} ipsec_interface;
@@ -485,11 +513,7 @@ struct config {
 typedef enum { BOTTOM_PRIORITY, } connection_priority_t;
 connection_priority_t connection_priority(const struct connection *c);
 
-typedef struct {
-	char buf[3 + 1 + 3 + 1/*NUL*/ + 10 + 1/*sentinel*/];	/* (10 is to silence GCC) */
-} connection_priority_buf;
-size_t jam_connection_priority(struct jambuf *buf, const struct connection *);
-const char *str_connection_priority(const struct connection *c, connection_priority_buf *buf);
+void jam_connection_priority(struct jambuf *buf, const struct connection *);
 
 struct host_end {
 	const struct host_end_config *config;
@@ -535,7 +559,7 @@ struct child_end {
 	bool has_client;
 #define set_end_child_has_client(C, END, VALUE)				\
 	{								\
-		enum left_right end_ = END;				\
+		enum end end_ = END;				\
 		where_t where_ = HERE;					\
 		bool has_client_ = VALUE;				\
 		struct connection *c_ = C;				\
@@ -577,7 +601,7 @@ struct connection_end {
 	struct child_end child;
 };
 
-void update_end_selector_where(struct connection *c, enum left_right end,
+void update_end_selector_where(struct connection *c, enum end end,
 			       ip_selector s,
 			       const char *excuse, where_t where);
 
@@ -589,15 +613,7 @@ void update_end_selector_where(struct connection *c, enum left_right end,
 
 void append_end_selector(struct connection_end *end,
 			 const struct ip_info *afi, ip_selector s,
-			 struct logger *logger, where_t where);
-
-#define set_end_selector(END, SELECTOR, LOGGER)				\
-	{								\
-		PASSERT(LOGGER, (END)->child.selectors.proposed.list == NULL); \
-		PASSERT(LOGGER, (END)->child.selectors.proposed.len == 0); \
-		append_end_selector(END, selector_info(SELECTOR),	\
-				    SELECTOR, LOGGER, HERE);		\
-	}
+			 const struct logger *logger, where_t where);
 
 struct spd_end {
 	ip_selector client;
@@ -790,6 +806,12 @@ struct connection {
 		struct iface_endpoint *local;
 	} revival;
 
+	/*
+	 * This struct will be used for storing variables required for
+	 * session resumption (when re-animating the IKE SA).
+	 */
+	struct session *session;
+
 	/*
 	 * Private variables for tracking routing.  Only updated by
 	 * routing.c.
@@ -880,7 +902,7 @@ extern bool same_peer_ids(const struct connection *c,
 
 diag_t add_connection(const struct whack_message *wm, struct logger *logger);
 
-void update_hosts_from_end_host_addr(struct connection *c, enum left_right end,
+void update_hosts_from_end_host_addr(struct connection *c, enum end end,
 				     ip_address host_addr, where_t where);
 
 void delete_connection_where(struct connection **cp, where_t where);
@@ -942,7 +964,6 @@ int connection_compare(const struct connection *ca,
 so_serial_t get_newer_sa_from_connection(struct state *st);
 
 diag_t add_end_cert_and_preload_private_key(CERTCertificate *cert,
-					    struct host_end *host_end,
 					    struct host_end_config *host_end_config,
 					    bool preserve_ca,
 					    struct logger *logger);
@@ -963,13 +984,13 @@ struct connection_filter {
 	/*
 	 * Filters.
 	 */
-	enum connection_kind kind;
-	const char *name;
-	const char *alias_root;
-	const struct id *this_id_eq; /* strict; not same_id() */
-	const struct id *that_id_eq; /* strict; not same_id() */
-	struct connection *clonedfrom;
-	enum ike_version ike_version;
+	const enum connection_kind kind;
+	const char *const name;
+	const char *const alias_root;
+	const struct id *const this_id_eq; /* strict; not same_id() */
+	const struct id *const that_id_eq; /* strict; not same_id() */
+	struct connection *const clonedfrom;
+	const enum ike_version ike_version;
 	/*
 	 * host-pair: matches is_template(), is_instance() and
 	 * is_permanent() (i.e., excludes is_group()) and:
@@ -978,9 +999,9 @@ struct connection_filter {
 	 * oriented() + local + 0.0.0.0 or ::; else match oriented() +
 	 * local + remote.
 	 */
-	struct {
-		const ip_address *local;
-		const ip_address *remote;
+	const struct {
+		const ip_address *const local;
+		const ip_address *const remote;
 	} host_pair;
 
 	/*
@@ -999,11 +1020,11 @@ struct connection_filter {
 	/*
 	 * Required fields.
 	 */
-	struct search {
-		enum chrono order;
-		const struct logger *logger;
+	struct /*search*/ {
+		const enum chrono order;
+		struct verbose verbose; /* writable */
 		/* .where MUST BE LAST (See GCC bug 102288) */
-		where_t where;
+		const where_t where;
 	} search;
 };
 
@@ -1066,25 +1087,18 @@ void spd_db_rehash_remote_client(struct spd *sr);
 
 bool dpd_active_locally(const struct connection *c);
 
+ip_address config_end_sourceip(const ip_selector client, const struct child_end_config *end);
 ip_address spd_end_sourceip(const struct spd_end *spde);
 
-PRINTF_LIKE(3)
-void ldbg_connection(const struct connection *c, where_t where,
+PRINTF_LIKE(4)
+void vdbg_connection(const struct connection *c,
+		     struct verbose verbose, where_t where,
 		     const char *message, ...);
 
 void init_connection_spd(struct connection *c, struct spd *spd);
 void alloc_connection_spds(struct connection *c, unsigned nr);
 void discard_connection_spds(struct connection *c);
-void add_connection_spds(struct connection *c, const struct ip_info *host_afi);
-
-/*
- * Format the topology of a connection end, leaving out defaults.
- * Largest left end looks like: client === host : port [ host_id ] ---
- * hop Note: if that==NULL, skip nexthop
- */
-void jam_spd_end(struct jambuf *buf, const struct connection *c,
-		 const struct spd_end *this, const struct spd_end *that,
-		 enum left_right left_right, bool filter_rnh);
+void add_connection_spds(struct connection *c);
 
 /* connections */
 
@@ -1149,4 +1163,9 @@ struct child_policy child_sa_policy(const struct connection *c);
 bool connections_can_share_parent(const struct connection *c,
 				  const struct connection *d);
 
+void add_proposals(struct connection *c, const struct ip_info *host_afi,
+		   struct verbose verbose);
+
+reqid_t child_reqid(const struct config *config, struct logger *logger);
+
 #endif
