@@ -44,7 +44,6 @@
 #include "sysdep.h"
 #include "constants.h"
 #include "defs.h"
-#include "packet.h"
 #include "demux.h"
 #include "crypto.h"
 #include "rnd.h"
@@ -75,14 +74,6 @@ static void jam_dh_local_secret(struct jambuf *buf, struct dh_local_secret *secr
 	jam(buf, "DH secret %s@%p: ", secret->group->common.fqn, secret);
 }
 
-static void free_dh_local_secret(void *obj, where_t where UNUSED)
-{
-	struct dh_local_secret *secret = obj;
-	SECKEY_DestroyPublicKey(secret->pubk);
-	SECKEY_DestroyPrivateKey(secret->privk);
-	pfree(secret);
-}
-
 struct dh_local_secret *calc_dh_local_secret(const struct dh_desc *group, struct logger *logger)
 {
 	SECKEYPrivateKey *privk;
@@ -90,12 +81,11 @@ struct dh_local_secret *calc_dh_local_secret(const struct dh_desc *group, struct
 	group->dh_ops->calc_local_secret(group, &privk, &pubk, logger);
 	passert(privk != NULL);
 	passert(pubk != NULL);
-	where_t here = HERE;
-	struct dh_local_secret *secret = refcnt_alloc(struct dh_local_secret, free_dh_local_secret, here);
+	struct dh_local_secret *secret = refcnt_alloc(struct dh_local_secret, HERE);
 	secret->group = group;
 	secret->privk = privk;
 	secret->pubk = pubk;
-	LSWDBGP(DBG_CRYPT, buf) {
+	LDBGP_JAMBUF(DBG_CRYPT, logger, buf) {
 		jam_dh_local_secret(buf, secret);
 		jam_string(buf, "created");
 	}
@@ -118,9 +108,15 @@ struct dh_local_secret *dh_local_secret_addref(struct dh_local_secret *secret, w
 	return addref_where(secret, where);
 }
 
-void dh_local_secret_delref(struct dh_local_secret **secret, where_t where)
+void dh_local_secret_delref(struct dh_local_secret **secretp, where_t where)
 {
-	delref_where(secret, where);
+	const struct logger *logger = &global_logger;
+	struct dh_local_secret *secret = delref_where(secretp, logger, where);
+	if (secret != NULL) {
+		SECKEY_DestroyPublicKey(secret->pubk);
+		SECKEY_DestroyPrivateKey(secret->privk);
+		pfree(secret);
+	}
 }
 
 struct task {
@@ -152,7 +148,8 @@ static void compute_dh_shared_secret(struct logger *logger,
 								&task->shared_secret,
 								logger);
 	if (diag != NULL) {
-		llog_diag(RC_LOG_SERIOUS, logger, &diag, "%s", "");
+		llog(RC_LOG, logger, "%s", str_diag(diag));
+		pfree_diag(&diag);
 		return;
 	}
 	/*
@@ -164,7 +161,7 @@ static void compute_dh_shared_secret(struct logger *logger,
 			jam(buf, "computed shared DH secret key@%p",
 			    task->shared_secret);
 		}
-		DBG_symkey(logger, "dh-shared ", "g^ir", task->shared_secret);
+		LDBG_symkey(logger, "dh-shared ", "g^ir", task->shared_secret);
 	}
 }
 
@@ -172,7 +169,7 @@ static void cleanup_dh_shared_secret(struct task **task)
 {
 	dh_local_secret_delref(&(*task)->local_secret, HERE);
 	free_chunk_content(&(*task)->remote_ke);
-	release_symkey("DH", "secret", &(*task)->shared_secret);
+	symkey_delref(&global_logger, "DH secret", &(*task)->shared_secret);
 	pfreeany(*task);
 }
 
@@ -184,7 +181,7 @@ static stf_status complete_dh_shared_secret(struct state *task_st,
 	dbg("completing DH shared secret for "PRI_SO"/"PRI_SO,
 	    task_st->st_serialno, dh_st->st_serialno);
 	pexpect(dh_st->st_dh_shared_secret == NULL);
-	release_symkey(__func__, "st_dh_shared_secret", &dh_st->st_dh_shared_secret);
+	symkey_delref(dh_st->logger, "st_dh_shared_secret", &dh_st->st_dh_shared_secret);
 	/* transfer */
 	dh_st->st_dh_shared_secret = task->shared_secret;
 	task->shared_secret = NULL;
@@ -199,14 +196,16 @@ static const struct task_handler dh_shared_secret_handler = {
 	.completed_cb = complete_dh_shared_secret,
 };
 
-void submit_dh_shared_secret(struct state *task_st,
-			     struct state *dh_st, chunk_t remote_ke,
+void submit_dh_shared_secret(struct state *callback_sa,
+			     struct state *dh_st,
+			     struct msg_digest *md,
+			     chunk_t remote_ke,
 			     dh_shared_secret_cb *cb, where_t where)
 {
 	dbg("submitting DH shared secret for "PRI_SO"/"PRI_SO" "PRI_WHERE,
-	    task_st->st_serialno, dh_st->st_serialno, pri_where(where));
+	    callback_sa->st_serialno, dh_st->st_serialno, pri_where(where));
 	if (dh_st->st_dh_shared_secret != NULL) {
-		llog_pexpect(dh_st->st_logger, where,
+		llog_pexpect(dh_st->logger, where,
 			     "in %s expecting st->st_dh_shared_secret == NULL",
 			     __func__);
 	}
@@ -215,5 +214,7 @@ void submit_dh_shared_secret(struct state *task_st,
 	task->local_secret = dh_local_secret_addref(dh_st->st_dh_local_secret, HERE);
 	task->dh_serialno = dh_st->st_serialno;
 	task->cb = cb;
-	submit_task(dh_st->st_logger, task_st, task, &dh_shared_secret_handler, where);
+	submit_task(/*callback*/callback_sa, /*task*/dh_st, md,
+		    /*detach_whack*/false,
+		    task, &dh_shared_secret_handler, where);
 }
