@@ -35,48 +35,21 @@
 #include "state.h"
 #include "crypt_cipher.h"
 
-void calc_v2_keymat(struct state *st,
-		    PK11SymKey *old_skey_d, /* SKEYSEED IKE Rekey */
-		    const struct prf_desc *old_prf, /* IKE Rekey */
-		    const ike_spis_t *new_ike_spis)
+void calc_v2_ike_keymat(struct state *st,
+			PK11SymKey *skeyseed,
+			const ike_spis_t *ike_spis)
 {
-	PK11SymKey *shared = st->st_dh_shared_secret;
-	const ike_spis_t *ike_spis = new_ike_spis;
 	struct logger *logger = st->logger;
 
-	ldbgf(DBG_CRYPT, logger, "NSS: Started key computation");
+	/* now we have to generate the keys for everything */
 
 	const struct encrypt_desc *cipher = st->st_oakley.ta_encrypt;
 	const struct prf_desc *prf = st->st_oakley.ta_prf;
 	const struct integ_desc *integ = st->st_oakley.ta_integ;
 
-	PASSERT(st->logger, cipher != NULL);
-	PASSERT(st->logger, prf != NULL);
-
 	size_t key_size = st->st_oakley.enckeylen / BITS_IN_BYTE;
 	size_t salt_size = cipher->salt_size;
 
-	ldbg(logger, "calculating skeyseed using prf=%s integ=%s cipherkey-size=%zu salt-size=%zu",
-	     prf->common.fqn,
-	     (integ != NULL ? integ->common.fqn : "n/a"),
-	     key_size, salt_size);
-
-	PK11SymKey *skeyseed;
-	if (old_skey_d == NULL) {
-		/* generate SKEYSEED from key=(Ni|Nr), hash of shared */
-		skeyseed = ikev2_ike_sa_skeyseed(prf, st->st_ni, st->st_nr,
-						 shared, logger);
-	}  else {
-		skeyseed = ikev2_ike_sa_rekey_skeyseed(old_prf, old_skey_d,
-						       shared,
-						       st->st_ni, st->st_nr,
-						       logger);
-	}
-
-	passert(skeyseed != NULL);
-
-	/* now we have to generate the keys for everything */
-
 	/* need to know how many bits to generate */
 	/* SK_d needs PRF hasher key bytes */
 	/* SK_p needs PRF hasher*2 key bytes */
@@ -84,14 +57,17 @@ void calc_v2_keymat(struct state *st,
 	/* ..._salt needs salt_size*2 bytes */
 	/* SK_a needs integ's key size*2 bytes */
 
+	PASSERT(logger, st->st_ni.len > 0);
+	PASSERT(logger, st->st_nr.len > 0);
+
 	int skd_bytes = prf->prf_key_size;
 	int skp_bytes = prf->prf_key_size;
 	int integ_size = integ != NULL ? integ->integ_keymat_size : 0;
 	size_t total_keysize = skd_bytes + 2*skp_bytes + 2*key_size + 2*salt_size + 2*integ_size;
 	PK11SymKey *finalkey = ikev2_ike_sa_keymat(prf, skeyseed,
-						   st->st_ni, st->st_nr, ike_spis,
+						   st->st_ni, st->st_nr,
+						   ike_spis,
 						   total_keysize, logger);
-	symkey_delref(logger, "skeyseed", &skeyseed);
 
 	size_t next_byte = 0;
 
@@ -147,12 +123,13 @@ void calc_v2_keymat(struct state *st,
 	next_byte += salt_size;
 
 	/*
-	 * PPK
+	 * PPK and AUTH NULL
 	 */
 
 	st->st_skey_pi_nss = key_from_symkey_bytes("SK_pi", finalkey,
 						   next_byte, skp_bytes,
 						   HERE, logger);
+
 	/* store copy of SK_pi_k for later use in authnull */
 	st->st_skey_chunk_SK_pi = chunk_from_symkey("chunk_SK_pi", st->st_skey_pi_nss, logger);
 	next_byte += skp_bytes;
@@ -160,8 +137,10 @@ void calc_v2_keymat(struct state *st,
 	st->st_skey_pr_nss = key_from_symkey_bytes("SK_pr", finalkey,
 						   next_byte, skp_bytes,
 						   HERE, logger);
+
 	/* store copy of SK_pr_k for later use in authnull */
 	st->st_skey_chunk_SK_pr = chunk_from_symkey("chunk_SK_pr", st->st_skey_pr_nss, logger);
+	next_byte += skp_bytes;
 
 	ldbgf(DBG_CRYPT, logger, "NSS ikev2: finished computing individual keys for IKEv2 SA");
 	symkey_delref(logger, "finalkey", &finalkey);
