@@ -36,7 +36,6 @@
 #include "sysdep.h"
 #include "constants.h"
 #include "defs.h"
-#include "packet.h"
 #include "demux.h"
 #include "crypto.h"
 #include "rnd.h"
@@ -102,13 +101,19 @@ static const struct task_handler ke_and_nonce_handler = {
 	.completed_cb = complete_ke_and_nonce,
 };
 
-void submit_ke_and_nonce(struct state *st, const struct dh_desc *dh,
-			 ke_and_nonce_cb *cb, where_t where)
+void submit_ke_and_nonce(struct state *callback_sa,
+			 struct state *task_sa,
+			 struct msg_digest *md,
+			 const struct dh_desc *dh,
+			 ke_and_nonce_cb *cb,
+			 bool detach_whack,
+			 where_t where)
 {
 	struct task *task = alloc_thing(struct task, "dh");
 	task->dh = dh;
 	task->cb = cb;
-	submit_task(st->st_logger, st, task, &ke_and_nonce_handler, where);
+	submit_task(/*callback*/callback_sa, /*task*/task_sa, md, detach_whack,
+		    task, &ke_and_nonce_handler, where);
 }
 
 /*
@@ -147,7 +152,7 @@ void unpack_KE_from_helper(struct state *st, struct dh_local_secret *local_secre
 			group == st->st_oakley.ta_dh ? "match" : "differ");
 	}
 
-	replace_chunk(g, clone_hunk(dh_local_secret_ke(local_secret), "KE"));
+	replace_chunk(g, dh_local_secret_ke(local_secret), "KE");
 	pexpect(st->st_dh_local_secret == NULL);
 	st->st_dh_local_secret = dh_local_secret_addref(local_secret, HERE);
 }
