@@ -44,8 +44,7 @@
  *      - newest SA
  */
 void show_ike_alg_connection(struct show *s,
-			     const struct connection *c,
-			     const char *instance)
+			     const struct connection *c)
 {
 	if (c->config->ike_proposals.p != NULL
 	    && !default_proposals(c->config->ike_proposals.p)) {
@@ -77,20 +76,23 @@ void show_ike_alg_connection(struct show *s,
 		 * AES_CBC_256+AES_CBC_128-... (which we hope is not
 		 * impossible to parse)?
 		 */
-		SHOW_JAMBUF(RC_COMMENT, s, buf) {
-			jam(buf, "\"%s\"%s:   IKE algorithms: ",
-			    c->name, instance);
+		SHOW_JAMBUF(s, buf) {
+			jam_connection_short(buf, c);
+			jam_string(buf, ":  ");
+			/* algs */
+			jam_string(buf, " IKE algorithms: ");
 			jam_proposals(buf, c->config->ike_proposals.p);
 		}
 	}
 
-	const struct state *st = state_by_serialno(c->newest_ike_sa);
+	const struct state *st = state_by_serialno(c->established_ike_sa);
 
 	if (st != NULL) {
-		SHOW_JAMBUF(RC_COMMENT, s, buf) {
-			jam(buf,
-			    "\"%s\"%s:   %s algorithm newest: ",
-			    c->name, instance,
+		SHOW_JAMBUF(s, buf) {
+			jam_connection_short(buf, c);
+			jam_string(buf, ":  ");
+			/* algs */
+			jam(buf, " %s algorithm newest: ",
 			    st->st_connection->config->ike_info->version_name);
 			const struct trans_attrs *ta = &st->st_oakley;
 			const char *sep = "";
@@ -130,29 +132,34 @@ void show_ike_alg_connection(struct show *s,
 void show_ike_alg_status(struct show *s)
 {
 	show_separator(s);
-	show_comment(s, "IKE algorithms supported:");
+	show(s, "IKE algorithms supported:");
 	show_separator(s);
 
 	for (const struct encrypt_desc **algp = next_encrypt_desc(NULL);
 	     algp != NULL; algp = next_encrypt_desc(algp)) {
 		const struct encrypt_desc *alg = (*algp);
 		if (ike_alg_is_ike(&(alg)->common)) {
-			esb_buf v1namebuf, v2namebuf;
 			passert(alg->common.ikev1_oakley_id >= 0 || alg->common.id[IKEv2_ALG_ID] >= 0);
-			show_comment(s,
-				  "algorithm IKE encrypt: v1id=%d, v1name=%s, v2id=%d, v2name=%s, blocksize=%zu, keydeflen=%u",
-				  alg->common.ikev1_oakley_id,
-				  (alg->common.ikev1_oakley_id >= 0 ? enum_show(&oakley_enc_names,
-										alg->common.ikev1_oakley_id,
-										&v1namebuf)
-				   : "n/a"),
-				  alg->common.id[IKEv2_ALG_ID],
-				  (alg->common.id[IKEv2_ALG_ID] >= 0 ? enum_show(&ikev2_trans_type_encr_names,
-										 alg->common.id[IKEv2_ALG_ID],
-										 &v2namebuf)
-				   : "n/a"),
-				  alg->enc_blocksize,
-				  alg->keydeflen);
+			SHOW_JAMBUF(s, buf) {
+				jam_string(buf, "algorithm IKE encrypt:");
+				jam(buf, " v1id=%d, v1name=", alg->common.ikev1_oakley_id);
+				if (alg->common.id[IKEv1_OAKLEY_ID] >= 0) {
+					jam_enum(buf, &oakley_enc_names, alg->common.ikev1_oakley_id);
+				} else {
+					jam_string(buf, "n/a");
+				}
+				jam_string(buf, ",");
+				jam(buf, " v2id=%d, v2name=", alg->common.id[IKEv2_ALG_ID]);
+				if (alg->common.id[IKEv2_ALG_ID] >= 0) {
+					jam_enum_short(buf, &ikev2_trans_type_encr_names,
+						       alg->common.id[IKEv2_ALG_ID]);
+				} else {
+					jam_string(buf, "n/a");
+				}
+				jam_string(buf, ",");
+				jam(buf, " blocksize=%zu, keydeflen=%u",
+				    alg->enc_blocksize, alg->keydeflen);
+			}
 		}
 	}
 
@@ -160,9 +167,9 @@ void show_ike_alg_status(struct show *s)
 	     algp != NULL; algp = next_prf_desc(algp)) {
 		const struct prf_desc *alg = (*algp);
 		if (ike_alg_is_ike(&(alg)->common)) {
-			show_comment(s,
-				  "algorithm IKE PRF: name=%s, hashlen=%zu",
-				  alg->common.fqn, alg->prf_output_size);
+			show(s,
+			     "algorithm IKE PRF: name=%s, hashlen=%zu",
+			     alg->common.fqn, alg->prf_output_size);
 		}
 	}
 
@@ -171,10 +178,10 @@ void show_ike_alg_status(struct show *s)
 		const struct dh_desc *gdesc = *gdescp;
 		if (gdesc->bytes > 0) {
 			/* nothing crazy like 'none' */
-			show_comment(s,
-				  "algorithm IKE DH Key Exchange: name=%s, bits=%d",
-				  gdesc->common.fqn,
-				  (int)gdesc->bytes * BITS_PER_BYTE);
+			show(s,
+			     "algorithm IKE DH Key Exchange: name=%s, bits=%d",
+			     gdesc->common.fqn,
+			     (int)gdesc->bytes * BITS_IN_BYTE);
 		}
 	}
 }
