@@ -70,12 +70,13 @@ static void connection_check_ddns1(struct connection *c, struct logger *logger)
 		return;
 	}
 
-	if (c->remote->config->child.protoport.has_port_wildcard) {
-		pdbg(c->logger, "pending ddns: skipping connection, remote has wildcard protoport");
+	if (!is_permanent(c)) {
+		pdbg(c->logger, "pending ddns: skipping connection, is not permanent");
 		return;
 	}
 
-	if (id_has_wildcards(&c->remote->host.id)) {
+	/* should have been handled by above */
+	if (pbad(id_has_wildcards(&c->remote->host.id))) {
 		pdbg(c->logger, "pending ddns: skipping connection, remote has wildcard ID");
 		return;
 	}
@@ -137,6 +138,9 @@ static void connection_check_ddns1(struct connection *c, struct logger *logger)
 	/*
 	 * Pull any existing routing based on current SPDs.  Remember,
 	 * per above, the connection isn't established.
+	 *
+	 * Note: disorient() also deletes any SPDs, orient() will put
+	 * them back.
 	 */
 	pdbg(c->logger, "  unrouting");
 	connection_unroute(c, HERE);
@@ -151,8 +155,6 @@ static void connection_check_ddns1(struct connection *c, struct logger *logger)
 	/* propagate remote address */
 	pdbg(c->logger, "  updating hosts");
 	update_hosts_from_end_host_addr(c, c->remote->config->index, new_remote_addr, HERE); /* from DNS */
-	pdbg(c->logger, "  discarding SPDs");
-	discard_connection_spds(c);
 
 	if (c->remote->child.config->selectors.len > 0) {
 		pdbg(c->logger, "  %s.child already has hard-wired selectors; skipping",
@@ -178,9 +180,6 @@ static void connection_check_ddns1(struct connection *c, struct logger *logger)
 				    c->logger, HERE);
 	}
 
-	pdbg(c->logger, "  adding SPDs");
-	add_connection_spds(c, address_info(c->local->host.addr));
-
 	/*
 	 * Caller holds reference.
 	 */
@@ -212,7 +211,7 @@ void connection_check_ddns(struct logger *logger)
 	struct connection_filter cf = {
 		.search = {
 			.order = NEW2OLD,
-			.logger = logger,
+			.verbose.logger = logger,
 			.where = HERE,
 		},
 	};
