@@ -7,6 +7,7 @@
  * Copyright (C) 2018-2019 Andrew Cagney <cagney@gnu.org>
  * Copyright (C) 2017 Mayank Totale <mtotale@gmail.com>
  * Copyright (C) 2020 Yulia Kuzovkova <ukuzovkova@gmail.com>
+ * Copyright (C) 2020 Nupur Agrawal <nupur202000@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -30,6 +31,7 @@
 #include "pluto_timing.h"
 #include "refcnt.h"
 #include "where.h"
+#include "crypt_mac.h"
 
 struct state;   /* forward declaration of tag */
 struct iface_endpoint;
@@ -112,6 +114,7 @@ enum v2_pd {
 	PD_v2N_NO_PROPOSAL_CHOSEN,
 	PD_v2N_NULL_AUTH,
 	PD_v2N_PPK_IDENTITY,
+	PD_v2N_PPK_IDENTITY_KEY,
 	PD_v2N_REDIRECT,
 	PD_v2N_REDIRECTED_FROM,
 	PD_v2N_REDIRECT_SUPPORTED,
@@ -122,8 +125,14 @@ enum v2_pd {
 	PD_v2N_UNSUPPORTED_CRITICAL_PAYLOAD,
 	PD_v2N_UPDATE_SA_ADDRESSES,
 	PD_v2N_USE_PPK,
+	PD_v2N_USE_PPK_INT,
 	PD_v2N_USE_TRANSPORT_MODE,
 	PD_v2N_USE_AGGFRAG,
+	PD_v2N_TICKET_LT_OPAQUE,
+	PD_v2N_TICKET_REQUEST,
+	PD_v2N_TICKET_ACK,
+	PD_v2N_TICKET_NACK,
+	PD_v2N_TICKET_OPAQUE,
 
 	PD_v2_ROOF,
 };
@@ -147,7 +156,9 @@ struct msg_digest {
 	struct isakmp_hdr hdr;			/* message's header */
 	bool encrypted;				/* (v1) was it encrypted? */
 	const struct state_v1_microcode *smc;	/* (v1) microcode for initial state */
-	bool new_iv_set;			/* (v1) */
+	struct crypt_mac v1_decrypt_iv;		/* (v1) needed to
+						 * decrypt the
+						 * message */
 	struct state *v1_st;			/* (v1) current state object */
 	struct logger *logger;			/* logger for this MD */
 
@@ -223,7 +234,7 @@ struct msg_digest {
 	 */
 	struct payload_digest *chain[LELEM_ROOF];
 	struct payload_digest *last[LELEM_ROOF];
-	struct isakmp_quirks quirks;
+	struct isakmp_quirks v1_quirks;
 };
 
 enum ike_version hdr_ike_version(const struct isakmp_hdr *hdr);
