@@ -19,8 +19,6 @@
  *
  */
 
-#if defined(LIBCURL) || defined(LIBLDAP)	/* essentially whole body of file */
-
 #include <pthread.h>    /* Must be the first include file */
 #include <stdlib.h>
 #include <errno.h>
@@ -30,7 +28,6 @@
 #include <cert.h>
 #include <certdb.h>
 
-
 #include "constants.h"
 #include "defs.h"
 #include "log.h"
@@ -49,6 +46,7 @@
 
 #define FETCH_CMD_TIMEOUT       5       /* seconds */
 
+char *curl_iface = NULL;
 deltatime_t curl_timeout = DELTATIME_INIT(FETCH_CMD_TIMEOUT);
 
 static pthread_t fetch_thread_id;
@@ -153,9 +151,11 @@ static err_t fetch_curl(const char *url, chunk_t *blob, struct logger *logger)
 
 #else	/* LIBCURL */
 
-static err_t fetch_curl(const char *url UNUSED,
-			chunk_t *blob UNUSED)
+static err_t fetch_curl(const char *url,
+			chunk_t *blob,
+			struct logger *logger)
 {
+	ldbg(logger, "%s() ignoring %s %p", __func__, url, blob->ptr);
 	return "not compiled with libcurl support";
 }
 
@@ -578,7 +578,3 @@ void free_crl_fetch(void)
 	}
 #endif
 }
-
-#else /* defined(LIBCURL) || defined(LIBLDAP) */
-/* we'll just ignore for now - this is all going away anyway */
-#endif
