@@ -135,12 +135,44 @@ static int subnetcmp(const ip_subnet a, const ip_subnet b)
 	return r;
 }
 
+/*
+ * rangecmp() compares the two ip_range values A and B.
+ *
+ * It returns -1, 0, or +1 if A is, in broad terms, less than, equal
+ * to, or greater than B.
+ */
+
+static int rangecmp(const ip_range a, const ip_range b)
+{
+	/* a.lo vs b.lo? */
+	{
+		ip_address al = range_start(a);
+		ip_address bl = range_start(b);
+		int r = addrcmp(&al, &bl);
+		if (r != 0) {
+			return r;
+		}
+	}
+
+	/* a.hi vs b.hi? */
+	{
+		ip_address ah = range_end(a);
+		ip_address bh = range_end(b);
+		int r = addrcmp(&ah, &bh);
+		if (r != 0) {
+			return r;
+		}
+	}
+
+	/* {a,b}.{lo,hi} are identical */
+	return 0;
+}
+
 static void read_foodgroup(struct file_lex_position *oflp,
 			   struct connection *g,
 			   struct fg_targets **new_targets)
 {
 	const char *fgn = g->name;
-	const ip_subnet lsn = selector_subnet(g->spd->local->client);
 	const struct lsw_conf_options *oco = lsw_init_options();
 	char *fg_path = alloc_printf("%s/%s", oco->policies_dir, fgn); /* must free */
 
@@ -164,50 +196,28 @@ static void read_foodgroup(struct file_lex_position *oflp,
 			continue;
 		}
 
-		/* address or address/mask */
+		/*
+		 * Parse subnet (simple addresses are also accepted).
+		 */
 		ip_subnet sn;
-		if (strchr(flp->tok, '/') == NULL) {
-			/* no /, so treat as /32 or V6 equivalent */
-			ip_address t;
-			err_t err = ttoaddress_num(shunk1(flp->tok), NULL, &t);
-			if (err != NULL) {
-				llog(RC_LOG, flp->logger,
-					    "ignored, '%s' is not an address: %s",
-					    flp->tok, err);
-				flushline(flp, NULL/*shh*/);
-				continue;
-			}
-			sn = subnet_from_address(t);
-		} else {
-			const struct ip_info *afi = strchr(flp->tok, ':') == NULL ? &ipv4_info : &ipv6_info;
-			ip_subnet snn;
-			ip_address nonzero_host;
-			err_t err = ttosubnet_num(shunk1(flp->tok), afi, &snn, &nonzero_host);
-			if (err != NULL) {
-				llog(RC_LOG, flp->logger,
-				     "ignored, '%s' is not a subnet: %s",
-				     flp->tok, err);
-				flushline(flp, NULL/*shh*/);
-				continue;
-			}
-			if (nonzero_host.is_set) {
-				address_buf hb;
-				llog(RC_LOG, flp->logger,
-				     "zeroing non-zero host identifier %s in '%s'",
-				     str_address(&nonzero_host, &hb), flp->tok);
-			}
-			sn = snn;
-		}
+		ip_address nonzero_host;
+		err_t err = ttosubnet_num(shunk1(flp->tok), NULL, &sn, &nonzero_host);
 
-		const struct ip_info *afi = subnet_info(sn);
-		if (afi == NULL) {
+		if (err != NULL) {
 			llog(RC_LOG, flp->logger,
-				    "ignored, unsupported Address Family \"%s\"",
-				    flp->tok);
+			     "ignored, '%s' is not a subnet: %s",
+			     flp->tok, err);
 			flushline(flp, NULL/*shh*/);
 			continue;
 		}
 
+		if (nonzero_host.is_set) {
+			address_buf hb;
+			llog(RC_LOG, flp->logger,
+			     "zeroing non-zero host identifier %s in '%s'",
+			     str_address(&nonzero_host, &hb), flp->tok);
+		}
+
 		const struct ip_protocol *proto = &ip_protocol_all;
 		ip_port sport = unset_port;
 		ip_port dport = unset_port;
@@ -272,6 +282,7 @@ static void read_foodgroup(struct file_lex_position *oflp,
 		pexpect(flp->bdry == B_record || flp->bdry == B_file);
 
 		/* Find where new entry ought to go in new_targets. */
+		ip_range lsn = selector_range(g->spd->local->client);
 		struct fg_targets **pp;
 		int r;
 
@@ -281,7 +292,8 @@ static void read_foodgroup(struct file_lex_position *oflp,
 				r = -1; /* end of list is infinite */
 				break;
 			}
-			r = subnetcmp(lsn, selector_subnet((*pp)->group->spd->local->client));
+
+			r = rangecmp(lsn, selector_range((*pp)->group->spd->local->client));
 			if (r == 0) {
 				r = subnetcmp(sn, (*pp)->subnet);
 			}
@@ -296,13 +308,13 @@ static void read_foodgroup(struct file_lex_position *oflp,
 		}
 
 		if (r == 0) {
-			subnet_buf source;
+			range_buf source;
 			subnet_buf dest;
 			llog(RC_LOG, flp->logger,
 			     "subnet \"%s\", proto %d, sport "PRI_HPORT" dport "PRI_HPORT", source %s, already \"%s\"",
 			     str_subnet(&sn, &dest),
 			     proto->ipproto, pri_hport(sport), pri_hport(dport),
-			     str_subnet(&lsn, &source),
+			     str_range(&lsn, &source),
 			     (*pp)->group->name);
 		} else {
 			struct fg_targets *f = alloc_thing(struct fg_targets,
@@ -313,7 +325,7 @@ static void read_foodgroup(struct file_lex_position *oflp,
 			f->proto = proto;
 			f->sport = sport;
 			f->dport = dport;
-			f->serialno = UNSET_CO_SERIAL;
+			f->serialno = COS_NOBODY;
 			*pp = f;
 		}
 	}
@@ -341,7 +353,7 @@ void load_groups(struct logger *logger)
 		.kind = CK_GROUP,
 		.search = {
 			.order = NEW2OLD,
-			.logger = logger,
+			.verbose.logger = logger,
 			.where = HERE,
 		},
 	};
@@ -362,7 +374,7 @@ void load_groups(struct logger *logger)
 			selector_buf asource;
 			subnet_buf atarget;
 			DBG_log("  %s->%s %s sport "PRI_HPORT" dport "PRI_HPORT" %s",
-				str_selector_subnet_port(&t->group->spd->local->client, &asource),
+				str_selector_range_port(&t->group->spd->local->client, &asource),
 				str_subnet(&t->subnet, &atarget),
 				t->proto->name, pri_hport(t->sport), pri_hport(t->dport),
 				t->group->name);
@@ -373,7 +385,7 @@ void load_groups(struct logger *logger)
 			selector_buf asource;
 			subnet_buf atarget;
 			DBG_log("  %s->%s %s sport "PRI_HPORT" dport "PRI_HPORT" %s",
-				str_selector_subnet_port(&t->group->spd->local->client, &asource),
+				str_selector_range_port(&t->group->spd->local->client, &asource),
 				str_subnet(&t->subnet, &atarget),
 				t->proto->name, pri_hport(t->sport), pri_hport(t->dport),
 				t->group->name);
@@ -421,10 +433,10 @@ void load_groups(struct logger *logger)
 				 */
 				ldbg(op->group->logger,
 				     "transferring "PRI_CO, pri_co(op->serialno));
-				passert(op->serialno != UNSET_CO_SERIAL);
-				passert(np->serialno == UNSET_CO_SERIAL);
+				passert(op->serialno != COS_NOBODY);
+				passert(np->serialno == COS_NOBODY);
 				np->serialno = op->serialno;
-				op->serialno = UNSET_CO_SERIAL;
+				op->serialno = COS_NOBODY;
 				/* free old; advance new */
 				*opp = op->next;
 				pfree_target(&op);
@@ -462,7 +474,7 @@ void load_groups(struct logger *logger)
 							connection_route(t, HERE);
 						}
 						ldbg(g->logger, "setting "PRI_CO, pri_co(t->serialno));
-						passert(np->serialno == UNSET_CO_SERIAL);
+						passert(np->serialno == COS_NOBODY);
 						np->serialno = t->serialno;
 						/* advance new */
 						npp = &np->next;
