@@ -18,13 +18,8 @@
 struct connection;      /* forward declaration */
 struct logger;
 
-extern void add_group(struct connection *c);
-extern void route_group(struct connection *c);
-extern void unroute_group(struct connection *c);
-extern void delete_group(const struct connection *c);
+void load_groups(struct logger *logger);
 
-extern void load_groups(struct logger *logger);
-
-void free_foodgroups(void);
+void remove_from_group(struct connection *c);
 
 #endif
