@@ -23,12 +23,13 @@
 #include "ip_endpoint.h"
 #include "refcnt.h"
 #include "list_entry.h"
+#include "shunk.h"
 
 struct fd;
-struct raw_iface;
+struct kernel_iface;
 struct iface_endpoint;
 struct show;
-struct iface_dev;
+struct iface_device;
 struct logger;
 
 struct iface_packet {
@@ -48,13 +49,13 @@ struct iface_io {
 	struct msg_digest *(*read_packet)(struct iface_endpoint **ifp,
 					  struct logger *logger);
 	ssize_t (*write_packet)(const struct iface_endpoint *ifp,
-				const void *ptr, size_t len,
+				shunk_t packet,
 				const ip_endpoint *remote_endpoint,
 				struct logger *logger);
 	void (*cleanup)(struct iface_endpoint *ifp);
 	void (*listen)(struct iface_endpoint *fip, struct logger *logger);
-	int (*bind_iface_endpoint)(const char *iface, ip_endpoint endpoint, struct logger *logger);
-	bool (*enable_esp_encap)(const struct iface_endpoint *ifp, struct logger *logger);
+	/* returns 0 or ERRNO */
+	int (*enable_esp_encapsulation)(int fd, struct logger *logger);
 };
 
 extern const struct iface_io udp_iface_io;
@@ -73,21 +74,28 @@ extern const struct iface_io iketcp_iface_io; /*IKETCP specific*/
  * interface if they have the same iface_endpoint->iface_alias.
  */
 
-struct iface_dev {
-	struct list_entry ifd_entry;
+struct iface_device {
+	struct list_entry entry;
 	refcnt_t refcnt;
-	char *id_rname; /* real device name */
-	bool id_nic_offload;
-	ip_address id_address;
+	char *real_device_name;
+	bool nic_offload;
+	ip_address local_address;
 	enum { IFD_ADD, IFD_KEEP, IFD_DELETE } ifd_change;
 };
 
-void release_iface_dev(struct iface_dev **id);
-struct iface_dev *find_iface_dev_by_address(const ip_address *address);
+struct iface_device *next_iface_device(struct iface_device *);
+
+struct iface_device *find_iface_device_by_address(const ip_address *address);
+
+struct iface_device *iface_device_addref_where(struct iface_device *ifp, where_t where);
+#define iface_addref(IFP) iface_device_addref_where(IFP, HERE)
+
+void iface_device_delref_where(struct iface_device **ifp, where_t where);
+#define iface_device_delref(IFP) iface_device_delref_where(IFP, HERE)
 
 struct iface_endpoint {
 	refcnt_t refcnt;
-	struct iface_dev *ip_dev;
+	struct iface_device *ip_dev;
 	const struct iface_io *io;
 	ip_endpoint local_endpoint;	/* interface IP address:port */
 	int fd;                 /* file descriptor of socket for IKE UDP messages */
@@ -178,24 +186,34 @@ struct iface_endpoint *iface_endpoint_addref_where(struct iface_endpoint *ifp, w
 void iface_endpoint_delref_where(struct iface_endpoint **ifp, where_t where);
 #define iface_endpoint_delref(IFP) iface_endpoint_delref_where(IFP, HERE)
 
-extern struct iface_endpoint *interfaces;   /* public interfaces */
-
 extern struct iface_endpoint *find_iface_endpoint_by_local_endpoint(ip_endpoint local_endpoint);
 extern void find_ifaces(bool rm_dead, struct logger *logger);
 extern void show_ifaces_status(struct show *s);
 void listen_on_iface_endpoint(struct iface_endpoint *ifp, struct logger *logger);
-struct iface_endpoint *bind_iface_endpoint(struct iface_dev *ifd, const struct iface_io *io,
+
+enum iface_esp_encapsulation {
+	ESP_ENCAPSULATION_ENABLED = 1,
+	ESP_ENCAPSULATION_DISABLED,
+};
+
+enum iface_initiator_port {
+	INITIATOR_PORT_FIXED = 1,
+	INITIATOR_PORT_FLOATS,
+};
+
+struct iface_endpoint *bind_iface_endpoint(struct iface_device *ifd,
+					   const struct iface_io *io,
 					   ip_port port,
-					   bool esp_encapsulation_enabled,
-					   bool float_nat_initiator,
+					   enum iface_esp_encapsulation esp_encapsulation,
+					   enum iface_initiator_port initiator_port,
 					   struct logger *logger);
 
 /* internal */
 struct iface_endpoint *alloc_iface_endpoint(int fd,
-					    struct iface_dev *ifd,
+					    struct iface_device *ifd,
 					    const struct iface_io *io,
-					    bool esp_encapsulation_enabled,
-					    bool float_nat_initiator,
+					    enum iface_esp_encapsulation esp_encapsulation,
+					    enum iface_initiator_port initiator_port,
 					    ip_endpoint local_endpoint,
 					    where_t where);
 
