@@ -61,6 +61,14 @@
 #include "pluto_x509.h"
 #include "ikev1_delete.h"
 #include "pluto_stats.h"
+#include "ikev1_msgid.h"
+#include "ikev1_hash.h"
+#include "ikev1_message.h"
+#include "send.h"
+
+static stf_status send_dpd_notification(struct ike_sa *ike,
+					uint16_t type, const void *data,
+					size_t len);
 
 /**
  * DPD Timeout Function
@@ -109,15 +117,19 @@ void event_v1_dpd_timeout(struct state *tbd_st)
 		ike->sa.st_viable_parent = false; /*needed?*/
 		struct state_filter sf = {
 			.clonedfrom = ike->sa.st_serialno,
-			.where = HERE,
+			.search = {
+				.order = NEW2OLD,
+				.verbose.logger = &global_logger,
+				.where = HERE,
+			},
 		};
-		while (next_state(NEW2OLD, &sf)) {
+		while (next_state(&sf)) {
 			struct child_sa *child = pexpect_child_sa(sf.st);
 			pdbg(logger, "delete IPsec SA "PRI_SO" which is a sibling",
 			     pri_so(child->sa.st_serialno));
 			state_attach(&child->sa, logger);
 			llog_n_maybe_send_v1_delete(ike, &child->sa, HERE);
-			connection_delete_child(&child, HERE);
+			connection_teardown_child(&child, REASON_DELETED, HERE);
 		}
 	}
 
@@ -127,9 +139,13 @@ void event_v1_dpd_timeout(struct state *tbd_st)
 	{
 		struct state_filter sf = {
 			.connection_serialno = c->serialno,
-			.where = HERE,
+			.search = {
+				.order = NEW2OLD,
+				.verbose.logger = &global_logger,
+				.where = HERE,
+			},
 		};
-		while (next_state(NEW2OLD, &sf)) {
+		while (next_state(&sf)) {
 			/* on first pass, ignore established ISAKMP SA's */
 			if (IS_PARENT_SA(sf.st)) {
 				continue;
@@ -142,7 +158,7 @@ void event_v1_dpd_timeout(struct state *tbd_st)
 				established_isakmp_sa_for_state(sf.st, /*viable-parent*/false);
 			llog_n_maybe_send_v1_delete(isakmp, sf.st, HERE);
 			struct child_sa *child = pexpect_child_sa(sf.st);
-			connection_delete_child(&child, HERE);
+			connection_teardown_child(&child, REASON_DELETED, HERE);
 		}
 	}
 
@@ -152,9 +168,13 @@ void event_v1_dpd_timeout(struct state *tbd_st)
 	{
 		struct state_filter sf = {
 			.connection_serialno = c->serialno,
-			.where = HERE,
+			.search = {
+				.order = NEW2OLD,
+				.verbose.logger = &global_logger,
+				.where = HERE,
+			},
 		};
-		while (next_state(NEW2OLD, &sf)) {
+		while (next_state(&sf)) {
 			if (!PEXPECT(logger, IS_PARENT_SA(sf.st))) {
 				continue;
 			}
@@ -166,7 +186,7 @@ void event_v1_dpd_timeout(struct state *tbd_st)
 				established_isakmp_sa_for_state(sf.st, /*viable-parent*/false);
 			llog_n_maybe_send_v1_delete(isakmp, sf.st, HERE);
 			struct ike_sa *ike = pexpect_ike_sa(sf.st);
-			connection_delete_ike(&ike, HERE);
+			connection_teardown_ike(&ike, REASON_DELETED, HERE);
 		}
 	}
 
@@ -259,8 +279,17 @@ stf_status dpd_init(struct state *st)
 			return STF_OK;
 		}
 
-		/* find the IKE SA */
-		struct state *p1st = find_state_ikev1(&st->st_ike_spis, 0);
+		/*
+		 * See if the IKE (ISAKMP) SA that was used to create
+		 * the Child SA is still around.
+		 *
+		 * If it is then it can be used to send the DPD
+		 * message.  If it isn't (for instance peer deleted
+		 * it) then this operation is doomed (technically, the
+		 * a new ISAKMP can be establish and used, but why
+		 * bother).
+		 */
+		struct ike_sa *p1st = find_v1_isakmp_sa(&st->st_ike_spis);
 		if (p1st == NULL) {
 			llog(RC_LOG, st->logger,
 			     "could not find phase 1 state for DPD");
@@ -268,7 +297,7 @@ stf_status dpd_init(struct state *st)
 		}
 
 		if (st->st_v1_dpd_event == NULL ||
-		    deltatime_cmp(monotimediff(st->st_v1_dpd_event->ev_time, mononow()),
+		    deltatime_cmp(monotime_diff(st->st_v1_dpd_event->ev_time, mononow()),
 				  <,
 				  st->st_connection->config->dpd.delay)) {
 			event_delete(EVENT_v1_DPD, st);
@@ -334,7 +363,7 @@ static void dpd_outI(struct ike_sa *p1, struct state *st,
 	monotime_t last = monotime_max(p1->sa.st_last_dpd, st->st_last_dpd);
 
 	monotime_t next_time = monotime_add(last, delay);
-	deltatime_t next_delay = monotimediff(next_time, now);
+	deltatime_t next_delay = monotime_diff(next_time, now);
 
 	/* has there been enough activity of late? */
 	if (deltatime_cmp(next_delay, >, deltatime(0))) {
@@ -407,8 +436,8 @@ static void dpd_outI(struct ike_sa *p1, struct state *st,
 	    str_endpoint(&p1->sa.st_remote_endpoint, &b),
 	    p1->sa.st_serialno);
 
-	if (send_isakmp_notification(&p1->sa, v1N_R_U_THERE,
-				     &seqno, sizeof(seqno)) != STF_IGNORE) {
+	if (send_dpd_notification(p1, v1N_R_U_THERE,
+				  &seqno, sizeof(seqno)) != STF_IGNORE) {
 		llog(RC_LOG, st->logger,
 		     "DPD: could not send R_U_THERE");
 		return;
@@ -549,8 +578,8 @@ stf_status dpd_inI_outR(struct state *p1sa,
 
 	p1->sa.st_dpd_peerseqno = seqno;
 
-	if (send_isakmp_notification(&p1->sa, v1N_R_U_THERE_ACK,
-				     pbs->cur, pbs_left(pbs)) != STF_IGNORE) {
+	if (send_dpd_notification(p1, v1N_R_U_THERE_ACK,
+				  pbs->cur, pbs_left(pbs)) != STF_IGNORE) {
 		llog(RC_LOG, p1->sa.logger,
 		     "DPD: could not send R_U_THERE_ACK");
 		return STF_IGNORE;
@@ -654,3 +683,79 @@ stf_status dpd_inR(struct state *p1sa,
 
 	return STF_IGNORE;
 }
+
+stf_status send_dpd_notification(struct ike_sa *ike,
+				 uint16_t type, const void *data,
+				 size_t len)
+{
+	msgid_t msgid = generate_msgid(&ike->sa);
+
+	struct fragment_pbs_out packet;
+	if (!open_fragment_pbs_out("DPD request", &packet, ike->sa.logger)) {
+		return STF_FATAL;
+	}
+
+	/* HDR* */
+	struct pbs_out rbody;
+
+	{
+		struct isakmp_hdr hdr = {
+			.isa_version = ISAKMP_MAJOR_VERSION << ISA_MAJ_SHIFT |
+				ISAKMP_MINOR_VERSION,
+			.isa_xchg = ISAKMP_XCHG_INFO,
+			.isa_flags = ISAKMP_FLAGS_v1_ENCRYPTION,
+			.isa_msgid = msgid,
+		};
+		hdr.isa_ike_initiator_spi = ike->sa.st_ike_spis.initiator;
+		hdr.isa_ike_responder_spi = ike->sa.st_ike_spis.responder;
+		if (!out_struct(&hdr, &isakmp_hdr_desc, &packet.pbs, &rbody))
+			return STF_INTERNAL_ERROR;
+	}
+
+	struct v1_hash_fixup hash_fixup;
+	if (!emit_v1_HASH(V1_HASH_1, "notification",
+			  IMPAIR_v1_NOTIFICATION_EXCHANGE,
+			  &ike->sa, &hash_fixup, &rbody)) {
+		return STF_INTERNAL_ERROR;
+	}
+
+	/* NOTIFY */
+	{
+		struct pbs_out notify_pbs;
+		struct isakmp_notification isan = {
+			.isan_doi = ISAKMP_DOI_IPSEC,
+			.isan_protoid = PROTO_ISAKMP,
+			.isan_spisize = COOKIE_SIZE * 2,
+			.isan_type = type,
+		};
+		if (!out_struct(&isan, &isakmp_notification_desc, &rbody,
+				&notify_pbs) ||
+		    !out_raw(ike->sa.st_ike_spis.initiator.bytes, COOKIE_SIZE, &notify_pbs,
+			     "notify icookie") ||
+		    !out_raw(ike->sa.st_ike_spis.responder.bytes, COOKIE_SIZE, &notify_pbs,
+			     "notify rcookie"))
+			return STF_INTERNAL_ERROR;
+
+		if (data != NULL && len > 0)
+			if (!out_raw(data, len, &notify_pbs, "notify data"))
+				return STF_INTERNAL_ERROR;
+
+		close_output_pbs(&notify_pbs);
+	}
+
+	fixup_v1_HASH(&ike->sa, &hash_fixup, msgid, rbody.cur);
+
+	/*
+	 * For NOTIFICATION / DELETE messages we don't need to
+	 * maintain a state because there are no retransmissions ...
+	 */
+	struct crypt_mac iv = new_phase2_iv(ike, msgid,
+					    "IKE sending DPD", HERE);
+	if (!close_and_encrypt_v1_message(ike, &rbody, &iv)) {
+		return STF_INTERNAL_ERROR;
+	}
+
+	send_pbs_out_using_state(&ike->sa, "ISAKMP notify", &packet.pbs);
+
+	return STF_IGNORE;
+}
