@@ -28,15 +28,14 @@
 #include "lset.h"
 
 struct id;
+struct authby;
+struct msg_digest;
 
-struct connection *find_next_v1_host_connection(struct connection *c,
-						lset_t req_policy, lset_t policy_exact_mask,
+struct connection *find_v1_main_mode_connection(struct msg_digest *md);
+
+struct connection *find_v1_aggr_mode_connection(struct msg_digest *md,
+						struct authby authby,
+						bool policy_xauth,
 						const struct id *peer_id);
 
-struct connection *find_v1_host_connection(const ip_address local_address,
-					   const ip_address remote_address,
-					   lset_t req_policy,
-					   lset_t policy_exact_mask,
-					   const struct id *peer_id);
-
 #endif
