@@ -16,7 +16,15 @@
 #ifndef IKEV1_MESSAGE_H
 #define IKEV1_MESSAGE_H
 
+#include "shunk.h"
+#include "chunk.h"
+
+struct logger;
+struct msg_digest;
 struct host_end;
+struct pbs_out;
+struct state;
+struct ike_sa;
 
 struct isakmp_ipsec_id build_v1_id_payload(const struct host_end *end, shunk_t *body);
 
@@ -39,4 +47,17 @@ extern bool ikev1_justship_KE(struct logger *logger, chunk_t *g, struct pbs_out
 extern bool ikev1_ship_KE(struct state *st, struct dh_local_secret *local_secret,
 			  chunk_t *g, struct pbs_out *outs);
 
+bool emit_v1_zero_padding(struct pbs_out *pbs);
+
+bool close_and_encrypt_v1_message(struct ike_sa *ike, struct pbs_out *pbs,
+				  struct crypt_mac *iv);
+bool close_v1_message(struct pbs_out *pbs, const struct ike_sa *ike);
+
+/* macros to manipulate IVs in state */
+
+void update_v1_phase_1_iv(struct ike_sa *ike, struct crypt_mac iv, where_t where);
+
+struct crypt_mac new_phase2_iv(const struct ike_sa *ike, const msgid_t msgid,
+			       const char *why, where_t where);
+
 #endif
