@@ -122,10 +122,10 @@ void check_nat_traversal_vid(struct ike_sa *ike, const struct msg_digest *md)
 {
 	ldbg(ike->sa.logger, "sender checking NAT-T: conn %s; VID %d",
 	     (ike->sa.st_connection->config->ikev1_natt == NATT_NONE ? "disabled" : "enabled"),
-	     md->quirks.qnat_traversal_vid);
-	if ((md->quirks.qnat_traversal_vid != VID_none) &&
+	     md->v1_quirks.qnat_traversal_vid);
+	if ((md->v1_quirks.qnat_traversal_vid != VID_none) &&
 	    (ike->sa.st_connection->config->ikev1_natt != NATT_NONE)) {
-		enum natt_method v = nat_traversal_vid_to_method(md->quirks.qnat_traversal_vid);
+		enum natt_method v = nat_traversal_vid_to_method(md->v1_quirks.qnat_traversal_vid);
 
 		ike->sa.hidden_variables.st_nat_traversal = LELEM(v);
 		enum_buf vb;
@@ -337,7 +337,7 @@ static bool emit_one_natoa(struct pbs_out *outs,
 			   const char *nm)
 {
 	struct isakmp_nat_oa natoa = {
-		.isanoa_idtype = address_type(&ip)->id_ip_addr,
+		.isanoa_idtype = address_info(ip)->id_ip_addr,
 	};
 
 	struct pbs_out pbs;
@@ -386,37 +386,38 @@ static void nat_traversal_show_result(struct state *st, uint16_t sport)
 	    rslt);
 }
 
-void ikev1_natd_init(struct state *st, struct msg_digest *md)
+void ikev1_natd_init(struct ike_sa *ike, struct msg_digest *md)
 {
 	lset_buf lb;
-	dbg("init checking NAT-T: conn %s; vid %s",
-	    st->st_connection->config->ikev1_natt == NATT_NONE ? "disabled" : "enabled",
-	    str_lset(&natt_method_names, st->hidden_variables.st_nat_traversal, &lb));
+	ldbg(ike->sa.logger, "init checking NAT-T: conn %s; vid %s",
+	     ike->sa.st_connection->config->ikev1_natt == NATT_NONE ? "disabled" : "enabled",
+	     str_lset(&natt_method_names, ike->sa.hidden_variables.st_nat_traversal, &lb));
 
-	if (st->st_connection->config->ikev1_natt == NATT_NONE) {
-		dbg("Skip NATD payloads due to nat-ikev1-method=none configuration");
+	if (ike->sa.st_connection->config->ikev1_natt == NATT_NONE) {
+		ldbg(ike->sa.logger, "skip NATD payloads due to nat-ikev1-method=none configuration");
 		return;
 	}
 
-	if (st->hidden_variables.st_nat_traversal != LEMPTY) {
-		if (md->v1_st->st_oakley.ta_prf == NULL) {
+	if (ike->sa.hidden_variables.st_nat_traversal != LEMPTY) {
+		if (PBAD(ike->sa.logger, ike->sa.st_oakley.ta_prf == NULL)) {
 			/*
-			 * This connection is doomed - no PRF for NATD hash
-			 * Probably in FIPS trying MD5 ?
+			 * This connection is doomed - no PRF for NATD
+			 * hash Probably in FIPS trying MD5 ?
+			 *
 			 * Nothing will get send, so just do nothing
 			 */
-			log_state(RC_LOG, st,
-				  "Cannot compute NATD payloads without valid PRF");
+			llog(RC_LOG, ike->sa.logger,
+			     "cannot compute NATD payloads without valid PRF");
 			return;
 		}
-		ikev1_natd_lookup(md, st);
+		ikev1_natd_lookup(md, &ike->sa);
 
-		if (st->hidden_variables.st_nat_traversal != LEMPTY) {
-			nat_traversal_show_result(st, endpoint_hport(md->sender));
+		if (ike->sa.hidden_variables.st_nat_traversal != LEMPTY) {
+			nat_traversal_show_result(&ike->sa, endpoint_hport(md->sender));
 		}
 	}
 
-	schedule_v1_nat_keepalive(st);
+	schedule_v1_nat_keepalive(&ike->sa);
 }
 
 /*
