@@ -15,10 +15,7 @@
 
 #include "fd.h"
 
-extern void quick_outI1(struct fd *whack_sock,
-			struct state *isakmp_sa,
-			struct connection *c,
-			lset_t policy,
-			unsigned long try,
-			so_serial_t replacing,
-			shunk_t sec_label);
+struct child_sa *quick_outI1(struct ike_sa *isakmp,
+			     struct connection *c,
+			     const struct child_policy *policy,
+			     so_serial_t replacing);
