@@ -35,6 +35,7 @@
 #include "ip_info.h"
 #include "iface.h"
 #include "demux.h"
+#include "pluto_stats.h"
 
 /*
  * (IKE v1) send fragments of packet.
@@ -64,7 +65,7 @@ static bool send_v1_frags(struct state *st, const char *where)
 	 * max_data_len is the maximum data length that will fit within it.
 	 */
 	const size_t max_data_len =
-		endpoint_type(&st->st_remote_endpoint)->ikev1_max_fragment_size
+		endpoint_info(st->st_remote_endpoint)->ikev1_max_fragment_size
 		-
 		(natt_bonus + NSIZEOF_isakmp_hdr +
 		 NSIZEOF_isakmp_ikefrag);
@@ -168,7 +169,7 @@ static bool should_fragment_v1_ike_msg(struct state *st, size_t len, bool resend
 	 *
 	 * ??? the following test does not account for natt_bonus
 	 */
-	return len >= endpoint_type(&st->st_remote_endpoint)->ikev1_max_fragment_size &&
+	return len >= endpoint_info(st->st_remote_endpoint)->ikev1_max_fragment_size &&
 	    (   (resending &&
 		 st->st_connection->config->ike_frag.allow &&
 		 st->st_v1_seen_fragmentation_supported) ||
@@ -224,7 +225,7 @@ bool resend_recorded_v1_ike_msg(struct state *st, const char *where)
 	if (st->st_state->kind == STATE_XAUTH_R0 &&
 	    !st->st_connection->config->aggressive) {
 		/* Only for Main mode + XAUTH */
-		event_schedule(EVENT_v1_SEND_XAUTH, deltatime_ms(EVENT_v1_SEND_XAUTH_DELAY_MS), st);
+		event_schedule(EVENT_v1_SEND_XAUTH, deltatime_from_milliseconds(EVENT_v1_SEND_XAUTH_DELAY_MS), st);
 	}
 
 	return ret;
