@@ -16,6 +16,7 @@
  * Copyright (C) 2017 Sahana Prasad <sahana.prasad07@gmail.com>
  * Copyright (C) 2020 Yulia Kuzovkova <ukuzovkova@gmail.com>
  * Copyright (C) 2021 Paul Wouters <paul.wouters@aiven.io>
+ * Copyright (C) 2020 Nupur Agrawal <nupur202000@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -72,10 +73,11 @@
 #include "ip_endpoint.h"
 #include "kernel.h"
 #include "iface.h"
-#include "ikev2_notify.h"
+#include "ikev2_notification.h"
 #include "unpack.h"
 #include "pending.h"		/* for release_pending_whacks() */
 #include "ikev2_host_pair.h"
+#include "ikev2_unsecured.h"
 #include "ikev2_ike_sa_init.h"
 #include "ikev2_informational.h"
 #include "ikev2_create_child_sa.h"
@@ -640,7 +642,7 @@ static bool is_duplicate_request_msgid(struct ike_sa *ike,
 	 */
 
 	if (ike->sa.st_v2_msgid_windows.responder.wip == msgid) {
-		llog(LOG_STREAM/*not-whack*/, ike->sa.logger,
+		llog(RC_LOG, ike->sa.logger,
 		     "discarding packet received during asynchronous work (DNS or crypto) in %s",
 		     ike->sa.st_state->name);
 		return true;
@@ -841,7 +843,8 @@ void ikev2_process_packet(struct msg_digest *md)
 	}
 
 	/*
-	 * Handle the unprotected IKE_SA_INIT exchange.
+	 * Handle an unsecured IKE exchange (IKE_SA_INIT or
+	 * IKE_SESSION_RESUME).
 	 *
 	 * Unlike for later exchanges (which requires an existing
 	 * secured IKE SA), the code processing an unsecured
@@ -851,12 +854,17 @@ void ikev2_process_packet(struct msg_digest *md)
 	 * For instance, when a cookie is required, a message with no
 	 * cookie is rejected before the IKE SA is created.
 	 *
-	 * Hence, the unprotected IKE_SA_INIT exchange is given its
-	 * own separate code path.
+	 * Hence, the unsecured exchanges are given their own separate
+	 * code path.
 	 */
 
 	if (ix == ISAKMP_v2_IKE_SA_INIT) {
-		process_v2_IKE_SA_INIT(md);
+		process_v2_UNSECURED_message(md);
+		return;
+	}
+
+	if (ix == ISAKMP_v2_IKE_SESSION_RESUME) {
+		process_v2_UNSECURED_message(md);
 		return;
 	}
 
@@ -888,9 +896,9 @@ void ikev2_process_packet(struct msg_digest *md)
 	}
 
 	/*
-	 * Since the IKE_SA_INIT exchanges have been excluded, the
-	 * only acceptable option is a protected exchange (has SK or
-	 * SKF) using a secured IKE SA.
+	 * Since unsecured exchanges (IKE_SA_INIT, IKE_SESSION_RESUME)
+	 * have been excluded, the only acceptable option is a
+	 * protected exchange (has SK or SKF) using a secured IKE SA.
 	 *
 	 * Narrow things further by ensuring that the IKE SA is,
 	 * indeed, secured.
@@ -941,7 +949,7 @@ void ikev2_process_packet(struct msg_digest *md)
  */
 
 static void complete_protected_but_fatal_exchange(struct ike_sa *ike, struct msg_digest *md,
-						  v2_notification_t n, chunk_t *data)
+						  v2_notification_t n, shunk_t data)
 {
 	PASSERT(ike->sa.logger, md != NULL);
 	enum message_role recv_role = v2_msg_role(md);
@@ -951,7 +959,6 @@ static void complete_protected_but_fatal_exchange(struct ike_sa *ike, struct msg
 	/* starting point */
 	const struct v2_transition undefined_transition = {
 		.story = "suspect message",
-		.from = { state, },
 		.to = finite_states[STATE_UNDEFINED],
 		.recv_role = recv_role,
 		.llog_success = ldbg_v2_success,
@@ -1008,7 +1015,8 @@ static void complete_protected_but_fatal_exchange(struct ike_sa *ike, struct msg
 	switch (v2_msg_role(md)) {
 	case MESSAGE_REQUEST:
 		record_v2N_response(ike->sa.logger, ike, md,
-				    n, data, ENCRYPTED_PAYLOAD);
+				    n, data,
+				    ENCRYPTED_PAYLOAD);
 		break;
 	case MESSAGE_RESPONSE:
 		break;
@@ -1263,9 +1271,9 @@ void process_protected_v2_message(struct ike_sa *ike, struct msg_digest *md)
 	md->encrypted_payloads = ikev2_decode_payloads(ike->sa.logger, md, &sk->pbs,
 						       sk->payload.generic.isag_np);
 	if (md->encrypted_payloads.n != v2N_NOTHING_WRONG) {
-		chunk_t data = chunk2(md->encrypted_payloads.data,
+		shunk_t data = shunk2(md->encrypted_payloads.data,
 				      md->encrypted_payloads.data_size);
-		complete_protected_but_fatal_exchange(ike, md, md->encrypted_payloads.n, &data);
+		complete_protected_but_fatal_exchange(ike, md, md->encrypted_payloads.n, data);
 		return;
 	}
 
@@ -1289,7 +1297,7 @@ void process_protected_v2_message(struct ike_sa *ike, struct msg_digest *md)
 		}
 		pexpect(secured_payload_failed);
 		/* XXX: calls delete_ike_sa() */
-		complete_protected_but_fatal_exchange(ike, md, v2N_INVALID_SYNTAX, NULL);
+		complete_protected_but_fatal_exchange(ike, md, v2N_INVALID_SYNTAX, empty_shunk);
 		return;
 	}
 
@@ -1353,23 +1361,9 @@ static void success_v2_state_transition(struct ike_sa *ike,
 
 	struct connection *c = ike->sa.st_connection;
 
-#if 0
-	/*
-	 * XXX: the transition's from state can lie - it may be
-	 * different to the ST's state!
-	 *
-	 * XXX: this fails.
-	 *
-	 * The problem is that the IKE SA, during IKE_AUTH, gets its
-	 * state changed midway through the transition: after
-	 * authentication but before Child SA processing.  Perhaps
-	 * that is no longer needed?
-	 */
-	pexpect(transition->from == ike->sa.st_state);
-#endif
 	if (DBGP(DBG_BASE)) {
 		LLOG_JAMBUF(DEBUG_STREAM, ike->sa.logger, buf) {
-			jam(buf, "IKE SA in state %s transitioning ",
+			jam(buf, "IKE SA in state %s transitioning to ",
 			    ike->sa.st_state->short_name);
 			jam_v2_transition(buf, transition);
 		}
@@ -1609,7 +1603,7 @@ void start_v2_exchange(struct ike_sa *ike,
 		       const struct v2_exchange *exchange,
 		       where_t where)
 {
-	set_v2_transition(&ike->sa, exchange->initiate, where);
+	set_v2_transition(&ike->sa, exchange->initiate.transition, where);
 	v2_msgid_start(ike, exchange, NULL, HERE);
 }
 
@@ -1699,32 +1693,12 @@ void complete_v2_state_transition(struct ike_sa *ike,
 	/* statistics */
 	pstat(stf_status, result);
 
-#if 0
-	/*
-	 * XXX: this fails.
-	 *
-	 * The problem is that the IKE SA, during IKE_AUTH, gets its
-	 * state changed midway through the transition: after
-	 * authentication but before Child SA processing.
-	 *
-	 * Perhaps that is no longer needed?
-	 *
-	 * Part of the hack is to get the IKE SA established message
-	 * out _before_ the Child SA processing occurs.  Is that the
-	 * only reason?
-	 */
-	pexpect(transition->from == ike->sa.st_state->kind);
-#endif
-
 	LDBGP_JAMBUF(DBG_BASE, ike->sa.logger, buf) {
-		jam(buf, "#%lu complete_v2_state_transition()", ike->sa.st_serialno);
-		if (!v2_transition_from(transition, ike->sa.st_state)) {
-			jam(buf, " in state %s", ike->sa.st_state->short_name);
-		}
-		jam(buf, " ");
-		jam_v2_transition(buf, transition);
-		jam(buf, " with status ");
+		jam(buf, "#%lu complete_v2_state_transition() status ", ike->sa.st_serialno);
 		jam_enum(buf, &stf_status_names, result);
+		jam(buf, " transitioning from state %s to ",
+		    ike->sa.st_state->short_name);
+		jam_v2_transition(buf, transition);
 	}
 
 	switch (result) {
@@ -1790,7 +1764,7 @@ void complete_v2_state_transition(struct ike_sa *ike,
 					 ike->sa.st_v2_msgid_windows.responder.outgoing_fragments);
 		/* do the deed */
 		on_delete(&ike->sa, skip_send_delete);
-		connection_delete_ike_family(&ike, HERE);
+		terminate_ike_family(&ike, REASON_DELETED, HERE);
 		pexpect(ike == NULL);
 		return;
 
@@ -1804,7 +1778,7 @@ void complete_v2_state_transition(struct ike_sa *ike,
 		v2_msgid_finish(ike, md, HERE);
 		/* do the deed */
 		on_delete(&ike->sa, skip_send_delete);
-		connection_delete_ike_family(&ike, HERE);
+		terminate_ike_family(&ike, REASON_DELETED, HERE);
 		/* get out of here -- everything is invalid */
 		pexpect(ike == NULL);
 		return;
@@ -1829,7 +1803,7 @@ void complete_v2_state_transition(struct ike_sa *ike,
 		/* do the deed; record'n'send logs */
 		record_n_send_n_log_v2_delete(ike, HERE);
 		/* do the deed */
-		connection_delete_ike_family(&ike, HERE);
+		terminate_ike_family(&ike, REASON_DELETED, HERE);
 		/* get out of here -- everything is invalid */
 		pexpect(ike == NULL);
 		return;
@@ -1849,7 +1823,7 @@ void complete_v2_state_transition(struct ike_sa *ike,
 				send_recorded_v2_message(ike, "STF_FATAL",
 							 ike->sa.st_v2_msgid_windows.responder.outgoing_fragments);
 			} else {
-				llog_pexpect_v2_msgid(ike, "exchange zombie as no response?");
+				llog_pexpect_v2_msgid(ike, "exchange zombie: no FATAL message response was recorded!?!");
 			}
 			break;
 		case NO_MESSAGE:
@@ -1862,7 +1836,7 @@ void complete_v2_state_transition(struct ike_sa *ike,
 		}
 
 		on_delete(&ike->sa, skip_send_delete);
-		connection_delete_ike_family(&ike, HERE);
+		terminate_ike_family(&ike, REASON_DELETED, HERE);
 		pexpect(ike == NULL);
 		return;
 
@@ -1879,7 +1853,7 @@ void complete_v2_state_transition(struct ike_sa *ike,
 		     transition->story,
 		     str_enum(&v2_notification_names, notification, &nb));
 	on_delete(&ike->sa, skip_send_delete);
-	connection_delete_ike_family(&ike, HERE);
+	terminate_ike_family(&ike, REASON_DELETED, HERE);
 }
 
 static void reinitiate_v2_ike_sa_init(const char *story, struct state *st, void *arg)
@@ -1966,14 +1940,18 @@ bool already_has_larval_v2_child(struct ike_sa *ike, const struct connection *c)
 				       LELEM(STATE_V2_NEW_CHILD_R0));
 
 	struct state_filter sf = {
-		.where = HERE,
+		.search = {
+			.order = OLD2NEW,
+			.verbose.logger = &global_logger,
+			.where = HERE,
+		},
 		.ike_version = IKEv2,
 		.ike_spis = &ike->sa.st_ike_spis,
 		/* only children */
 		.clonedfrom = ike->sa.st_serialno,
 	};
 
-	while (next_state(OLD2NEW, &sf)) {
+	while (next_state(&sf)) {
 		struct state *st = sf.st;
 
 		/* larval child state? */
@@ -2024,54 +2002,31 @@ bool accept_v2_notification(v2_notification_t n,
 	return false;
 }
 
-void event_v2_rekey(struct state *st, bool detach_whack)
+void jam_v2_transition(struct jambuf *buf, const struct v2_transition *transition)
 {
-	if (v2_state_is_expired(st, "rekey")) {
+	if (transition == NULL) {
+		jam_string(buf, "<null-transition>");
 		return;
 	}
-
-	struct ike_sa *ike = ike_sa(st, HERE);
-
-	struct child_sa *larval_sa;
-	if (IS_IKE_SA(st)) {
-		larval_sa = submit_v2_CREATE_CHILD_SA_rekey_ike(ike, /*detach_whack*/false);
-	} else {
-		larval_sa = submit_v2_CREATE_CHILD_SA_rekey_child(ike, pexpect_child_sa(st),
-								  detach_whack);
-	}
-
-	llog(RC_LOG, larval_sa->sa.logger,
-	     "initiating rekey to replace %s "PRI_SO" using IKE SA "PRI_SO,
-	     state_sa_name(st),
-	     pri_so(st->st_serialno),
-	     pri_so(ike->sa.st_serialno));
+	jam_string(buf, transition->to->short_name);
+	jam_string(buf, " (");
+	jam_enum(buf, &ikev2_exchange_names, transition->exchange);
+	jam_string(buf, " ");
+	jam_enum(buf, &message_role_names, transition->recv_role);
+	jam_string(buf, ": ");
+	jam_string(buf, transition->story);
+	jam_string(buf, ")");
 }
 
-bool v2_transition_from(const struct v2_transition *transition, const struct finite_state *state)
+bool v2_ike_sa_can_initiate_exchange(const struct ike_sa *ike, const struct v2_exchange *exchange)
 {
-	FOR_EACH_ELEMENT(from, transition->from) {
-		if (*from == state) {
+	const struct finite_state *state = ike->sa.st_state;
+	ldbg(ike->sa.logger, "looking for exchange '%s' in state '%s'",
+	     exchange->subplot, state->short_name);
+	FOR_EACH_ELEMENT(f, exchange->initiate.from) {
+		if (*f == state) {
 			return true;
 		}
 	}
 	return false;
 }
-
-void jam_v2_transition(struct jambuf *buf, const struct v2_transition *transition)
-{
-	if (transition == NULL) {
-		jam_string(buf, "<null-transition>");
-	} else {
-		jam_string(buf, transition->story);
-		const char *sep = " from "; /* at least one */
-		FOR_EACH_ELEMENT(from, transition->from) {
-			if (*from != NULL) {
-				jam_string(buf, sep);
-				jam_string(buf, (*from)->short_name);
-				sep = "+";
-			}
-		}
-		jam_string(buf, " to ");
-		jam_string(buf, transition->to->short_name);
-	}
-}
