@@ -14,6 +14,8 @@
 
 #include "fd.h"
 
+#include "message_role.h"
+
 struct pending;
 struct pluto_crypto_req;
 struct spd;
@@ -23,6 +25,13 @@ struct payload_digest;
 struct ikev2_ipseckey_dns;
 struct hash_signature;
 enum payload_security;
+struct msg_digest;
+struct state;
+struct ike_sa;
+struct child_sa;
+struct pbs_in;
+struct dh_desc;
+struct connection;
 
 typedef stf_status crypto_transition_fn(struct state *st, struct msg_digest *md,
 					struct pluto_crypto_req *r);
@@ -49,6 +58,8 @@ extern void ikev2_derive_child_keys(struct ike_sa *ike, struct child_sa *child);
 bool ikev2_parse_cp_r_body(struct payload_digest *cp_pd, struct child_sa *child);
 
 struct ikev2_expected_payloads {
+	/* For liveness, so that no payloads means no payloads */
+	bool exact_match;
 	/* required payloads: one of each type must be present */
 	lset_t required;
 	/* optional payloads: up to one of each type can be present */
@@ -63,7 +74,6 @@ struct ikev2_expected_payloads {
 
 struct v2_transition {
 	const char *const story;	/* state transition story (not state_story[]) */
-	const struct finite_state *from[2];	/* grow as needed */
 	const struct finite_state *to;
 	struct {
 		bool release_whack;
@@ -109,7 +119,10 @@ struct v2_exchange {
 	const enum ikev2_exchange type;
 	const char *subplot;
 	bool secured;
-	const struct v2_transition *initiate;
+	struct {
+		const struct v2_transition *transition;
+		const struct finite_state *from[3];	/* grow as needed */
+	} initiate;
 	const struct v2_transitions *responder;
 	const struct v2_transitions *response;
 };
@@ -153,10 +166,10 @@ struct v2_exchanges {
 		.type = ISAKMP_v2_##KIND,				\
 		.subplot = SUBPLOT,					\
 		.secured = SECURED,					\
-		.initiate = &v2_##KIND##_initiate_transition,		\
+		.initiate.transition = &v2_##KIND##_initiate_transition, \
+		.initiate.from = { __VA_ARGS__ },			\
 		.responder = &v2_##KIND##_responder_transitions,	\
 		.response = &v2_##KIND##_response_transitions,		\
-		##__VA_ARGS__,						\
 	}
 
 #define V2_STATE(KIND, STORY, CAT, SECURED, ...)			\
@@ -182,8 +195,6 @@ struct v2_exchanges {
 
 extern void init_ikev2(void);
 
-void event_v2_rekey(struct state *st, bool detach_whack);
-
 struct payload_summary ikev2_decode_payloads(struct logger *log,
 					     struct msg_digest *md,
 					     struct pbs_in *in_pbs,
@@ -226,6 +237,7 @@ stf_status next_v2_exchange(struct ike_sa *ike, struct msg_digest *md,
 			    where_t where);
 
 extern void jam_v2_transition(struct jambuf *buf, const struct v2_transition *transition);
-extern bool v2_transition_from(const struct v2_transition *transition, const struct finite_state *state);
+
+bool v2_ike_sa_can_initiate_exchange(const struct ike_sa *ike, const struct v2_exchange *exchange);
 
 #endif
