@@ -40,6 +40,7 @@
 #include "keys.h"
 #include "ikev2_psk.h"
 #include "ikev2_send.h"
+#include "ikev2_notification.h"
 
 struct crypt_mac v2_calculate_sighash(const struct ike_sa *ike,
 				      const struct crypt_mac *idhash,
@@ -116,10 +117,15 @@ struct crypt_mac v2_calculate_sighash(const struct ike_sa *ike,
 
 enum keyword_auth local_v2_auth(struct ike_sa *ike)
 {
+	if (ike->sa.st_v2_resume_session != NULL) {
+		return AUTH_PSK;
+	}
+
 	if (ike->sa.st_peer_wants_null) {
 		/* we allow authby=null and IDr payload told us to use it */
 		return AUTH_NULL;
 	}
+
 	const struct connection *c = ike->sa.st_connection;
 	enum keyword_auth authby = c->local->host.config->auth;
 	pexpect(authby != AUTH_UNSET);
@@ -604,7 +610,7 @@ static stf_status submit_v2_IKE_AUTH_response_signature(struct ike_sa *ike,
 				      &hash_to_sign, hash_algo, signer, cb, HERE)) {
 		dbg("submit_v2_auth_signature() died, fatal");
 		record_v2N_response(ike->sa.logger, ike, md,
-				    v2N_AUTHENTICATION_FAILED, NULL/*no data*/,
+				    v2N_AUTHENTICATION_FAILED, empty_shunk/*no data*/,
 				    ENCRYPTED_PAYLOAD);
 		return STF_FATAL;
 	}
@@ -669,7 +675,7 @@ stf_status submit_v2AUTH_generate_responder_signature(struct ike_sa *ike, struct
 		}
 		if (ike->sa.st_v2_digsig.hash == NULL) {
 			record_v2N_response(ike->sa.logger, ike, md,
-					    v2N_AUTHENTICATION_FAILED, NULL/*no data*/,
+					    v2N_AUTHENTICATION_FAILED, empty_shunk/*no data*/,
 					    ENCRYPTED_PAYLOAD);
 			return STF_FATAL;
 		}
@@ -718,7 +724,7 @@ stf_status submit_v2AUTH_generate_responder_signature(struct ike_sa *ike, struct
 			llog(RC_LOG, ike->sa.logger, "%s", str_diag(d));
 			pfree_diag(&d);
 			record_v2N_response(ike->sa.logger, ike, md,
-					    v2N_AUTHENTICATION_FAILED, NULL/*no data*/,
+					    v2N_AUTHENTICATION_FAILED, empty_shunk/*no-data*/,
 					    ENCRYPTED_PAYLOAD);
 			return STF_FATAL;
 		}
@@ -938,7 +944,7 @@ void v2_IKE_AUTH_initiator_id_payload(struct ike_sa *ike)
 	ike->sa.st_v2_id_payload.mac = v2_hash_id_payload("IDi", ike,
 					    "st_skey_pi_nss",
 					    ike->sa.st_skey_pi_nss);
-	if (ike->sa.st_v2_ike_ppk_enabled && !c->config->ppk.insist) {
+	if (ike->sa.st_v2_ike_ppk == PPK_IKE_AUTH && !c->config->ppk.insist) {
 		/* ID payload that we've build is the same */
 		ike->sa.st_v2_id_payload.mac_no_ppk_auth =
 			v2_hash_id_payload("IDi (no-PPK)", ike,
@@ -989,3 +995,48 @@ struct crypt_mac v2_remote_id_hash(const struct ike_sa *ike,
 	crypt_prf_update_bytes(id_ctx, id_name, id_start, id_size);
 	return crypt_prf_final_mac(&id_ctx, NULL/*no-truncation*/);
 }
+
+/*
+ * Convert the proposed connections into something this responder
+ * might accept.
+ *
+ * + DIGITAL_SIGNATURE code seems a bit dodgy, should this be looking
+ * inside the auth proposal to see what is actually required?
+ *
+ * + the legacy ECDSA_SHA2* methods also seem to be a bit dodgy,
+ * shouldn't they also specify the SHA algorithm so that can be
+ * matched?
+ */
+
+lset_t proposed_v2AUTH(struct ike_sa *ike,
+		       struct msg_digest *md)
+{
+	enum ikev2_auth_method atype =
+		md->chain[ISAKMP_NEXT_v2AUTH]->payload.v2auth.isaa_auth_method;
+	name_buf nb;
+	ldbg(ike->sa.logger, "converting v2AUTH %s into authbys set",
+	     str_enum_short(&ikev2_auth_method_names, atype, &nb));
+
+	switch (atype) {
+	case IKEv2_AUTH_RSA_DIGITAL_SIGNATURE:
+		return LELEM(AUTH_RSASIG);
+	case IKEv2_AUTH_ECDSA_SHA2_256_P256:
+	case IKEv2_AUTH_ECDSA_SHA2_384_P384:
+	case IKEv2_AUTH_ECDSA_SHA2_512_P521:
+		return LELEM(AUTH_ECDSA);
+	case IKEv2_AUTH_SHARED_KEY_MAC:
+		return LELEM(AUTH_PSK);
+	case IKEv2_AUTH_NULL:
+		return LELEM(AUTH_NULL);
+	case IKEv2_AUTH_DIGITAL_SIGNATURE:
+		return LELEM(AUTH_RSASIG) | LELEM(AUTH_ECDSA);
+	default:
+	{
+		name_buf nb;
+		llog(RC_LOG, ike->sa.logger, "auth method %s unrecognized",
+		     str_enum_short(&ikev2_auth_method_names,
+				    atype, &nb));
+		return LEMPTY;
+	}
+	}
+}
