@@ -46,22 +46,22 @@
 
 stf_status emit_v2CERT(const struct connection *c, struct pbs_out *outpbs)
 {
-	const struct cert *mycert = c->local->config->host.cert.nss_cert != NULL ? &c->local->config->host.cert : NULL;
-	bool send_authcerts = c->send_ca != CA_SEND_NONE;
-	bool send_full_chain = send_authcerts && c->send_ca == CA_SEND_ALL;
+	const struct cert *mycert = c->local->host.config->cert.nss_cert != NULL ? &c->local->host.config->cert : NULL;
+	bool send_authcerts = c->config->send_ca != CA_SEND_NONE;
+	bool send_full_chain = send_authcerts && c->config->send_ca == CA_SEND_ALL;
 
 	if (impair.send_pkcs7_thingie) {
-		llog(RC_LOG, outpbs->outs_logger, "IMPAIR: sending cert as PKCS7 blob");
+		llog(RC_LOG, outpbs->logger, "IMPAIR: sending cert as PKCS7 blob");
 		passert(mycert != NULL);
 		SECItem *pkcs7 = nss_pkcs7_blob(mycert, send_full_chain);
 		if (!pexpect(pkcs7 != NULL)) {
 			return STF_INTERNAL_ERROR;
 		}
 		struct ikev2_cert pkcs7_hdr = {
-			.isac_critical = build_ikev2_critical(false, outpbs->outs_logger),
+			.isac_critical = build_ikev2_critical(false, outpbs->logger),
 			.isac_enc = CERT_PKCS7_WRAPPED_X509,
 		};
-		pb_stream cert_pbs;
+		struct pbs_out cert_pbs;
 		if (!out_struct(&pkcs7_hdr, &ikev2_certificate_desc,
 				outpbs, &cert_pbs) ||
 		    !out_hunk(same_secitem_as_chunk(*pkcs7), &cert_pbs, "PKCS7")) {
@@ -88,13 +88,13 @@ stf_status emit_v2CERT(const struct connection *c, struct pbs_out *outpbs)
 	}
 
 	const struct ikev2_cert certhdr = {
-		.isac_critical = build_ikev2_critical(false, outpbs->outs_logger),
+		.isac_critical = build_ikev2_critical(false, outpbs->logger),
 		.isac_enc = cert_ike_type(mycert),
 	};
 
 	/*   send own (Initiator CERT) */
 	{
-		pb_stream cert_pbs;
+		struct pbs_out cert_pbs;
 
 		dbg("sending [CERT] of certificate: %s", cert_nickname(mycert));
 
@@ -111,7 +111,7 @@ stf_status emit_v2CERT(const struct connection *c, struct pbs_out *outpbs)
 	/* send optional chain CERTs */
 	{
 		for (int i = 0; i < chain_len ; i++) {
-			pb_stream cert_pbs;
+			struct pbs_out cert_pbs;
 
 			dbg("sending an authcert");
 
@@ -137,7 +137,6 @@ stf_status emit_v2CERT(const struct connection *c, struct pbs_out *outpbs)
 bool ikev2_send_cert_decision(const struct ike_sa *ike)
 {
 	const struct connection *c = ike->sa.st_connection;
-	const struct end *this = &c->spd.this;
 
 	if (ike->sa.st_peer_wants_null) {
 		/* XXX: only ever true on responder */
@@ -145,32 +144,32 @@ bool ikev2_send_cert_decision(const struct ike_sa *ike)
 		return false;
 	}
 
-	if (c->local->config->host.auth == AUTH_EAPONLY) {
+	if (c->local->host.config->auth == AUTH_EAPONLY) {
 		dbg("IKEv2 CERT: not sending cert: local %sauth==EAPONLY",
 		    c->local->config->leftright)
 		return false;
 	}
 
-	if (!authby_has_digsig(c->local->config->host.authby)) {
+	if (!authby_has_digsig(c->local->host.config->authby)) {
 		authby_buf pb;
 		dbg("IKEv2 CERT: not sending cert: local %sauthby=%s does not have RSA or ECDSA",
 		    c->local->config->leftright,
-		    str_authby(c->local->config->host.authby, &pb));
+		    str_authby(c->local->host.config->authby, &pb));
 		return false;
 	}
 
-	if (this->config->host.cert.nss_cert == NULL) {
+	if (c->local->host.config->cert.nss_cert == NULL) {
 		dbg("IKEv2 CERT: not sending cert: there is no local certificate to send");
 		return false;
 	}
 
-	if (c->local->config->host.sendcert == CERT_SENDIFASKED &&
+	if (c->local->host.config->sendcert == CERT_SENDIFASKED &&
 	    ike->sa.st_v2_ike_seen_certreq) {
 		dbg("IKEv2 CERT: OK to send certificate (send if asked)");
 		return true;
 	}
 
-	if (c->local->config->host.sendcert == CERT_ALWAYSSEND) {
+	if (c->local->host.config->sendcert == CERT_ALWAYSSEND) {
 		dbg("IKEv2 CERT: OK to send a certificate (always)");
 		return true;
 	}
