@@ -28,7 +28,6 @@
 #include <netinet/in.h>
 #include <arpa/inet.h>
 
-
 #include "sysdep.h"
 #include "constants.h"
 
@@ -66,6 +65,7 @@
 #include "ikev2_proposals.h"
 #include "ikev2_parent.h"
 #include "ikev2_states.h"
+#include "ikev2_notification.h"
 
 static bool emit_v2_child_response_payloads(struct ike_sa *ike,
 					    const struct child_sa *child,
@@ -83,7 +83,8 @@ void set_larval_v2_transition(struct child_sa *larval,
 	const struct v2_transition *transition =
 		larval->sa.st_state->v2.child_transition;
 	PASSERT_WHERE(larval->sa.logger, where, transition != NULL);
-	PEXPECT_WHERE(larval->sa.logger, where, v2_transition_from(transition, larval->sa.st_state));
+	PEXPECT_WHERE(larval->sa.logger, where, larval->sa.st_state != NULL);
+	PEXPECT_WHERE(larval->sa.logger, where, larval->sa.st_state->v2.child_transition == transition);
 	PEXPECT_WHERE(larval->sa.logger, where, transition->to == to);
 	set_v2_transition(&larval->sa, transition, where);
 }
@@ -278,6 +279,11 @@ bool emit_v2_child_request_payloads(const struct ike_sa *ike,
 		return false;
 	}
 
+	if (!send_use_transport && cc->config->child_sa.iptfs.enabled &&
+	    !emit_v2N(v2N_USE_AGGFRAG, pbs)) {
+		return false;
+	}
+
 	if (cc->config->send_no_esp_tfc &&
 	    !emit_v2N(v2N_ESP_TFC_PADDING_NOT_SUPPORTED, pbs)) {
 		return false;
@@ -362,6 +368,12 @@ v2_notification_t process_v2_child_request_payloads(struct ike_sa *ike,
 		return v2N_NO_PROPOSAL_CHOSEN;
 	}
 
+	larval_child->sa.st_seen_and_use_iptfs =
+		accept_v2_notification(v2N_USE_AGGFRAG,
+				       larval_child->sa.logger,
+				       request_md,
+				       cc->config->child_sa.iptfs.enabled);
+
 	larval_child->sa.st_kernel_mode = required_mode;
 
 	if (!compute_v2_child_spi(larval_child)) {
@@ -419,10 +431,10 @@ v2_notification_t process_v2_child_request_payloads(struct ike_sa *ike,
 		dbg("policy suggested compression, but peer did not offer support");
 	}
 
-	if (request_md->pd[PD_v2N_ESP_TFC_PADDING_NOT_SUPPORTED] != NULL) {
-		dbg("received ESP_TFC_PADDING_NOT_SUPPORTED");
-		larval_child->sa.st_seen_no_tfc = true;
-	}
+	/* is not not negotiated */
+	larval_child->sa.st_seen_no_tfc = (request_md->pd[PD_v2N_ESP_TFC_PADDING_NOT_SUPPORTED] != NULL);
+	ldbg(larval_child->sa.logger, "received ESP_TFC_PADDING_NOT_SUPPORTED=%s",
+	     bool_str(larval_child->sa.st_seen_no_tfc));
 
 	ikev2_derive_child_keys(ike, larval_child);
 
@@ -538,6 +550,11 @@ bool emit_v2_child_response_payloads(struct ike_sa *ike,
 		return false;
 	}
 
+	if (larval_child->sa.st_seen_and_use_iptfs &&
+	    !emit_v2N(v2N_USE_AGGFRAG, outpbs)) {
+		return false;
+	}
+
 	if (cc->config->send_no_esp_tfc &&
 	    !emit_v2N(v2N_ESP_TFC_PADDING_NOT_SUPPORTED, outpbs)) {
 			return false;
@@ -793,12 +810,20 @@ v2_notification_t process_v2_child_response_payloads(struct ike_sa *ike, struct
 			str_enum(&kernel_mode_stories, accepted_mode, &amb));
  		return v2N_NO_PROPOSAL_CHOSEN;
  	}
+
+	child->sa.st_seen_and_use_iptfs =
+		accept_v2_notification(v2N_USE_AGGFRAG, child->sa.logger, md,
+				       c->config->child_sa.iptfs.enabled);
+
 	enum_buf rmb;
 	ldbg_sa(child, "local policy is %s and received matching notify",
 		str_enum(&kernel_mode_stories, required_mode, &rmb));
 	child->sa.st_kernel_mode = required_mode;
 
-	child->sa.st_seen_no_tfc = md->pd[PD_v2N_ESP_TFC_PADDING_NOT_SUPPORTED] != NULL;
+	child->sa.st_seen_no_tfc = (md->pd[PD_v2N_ESP_TFC_PADDING_NOT_SUPPORTED] != NULL);
+	ldbg(child->sa.logger, "received ESP_TFC_PADDING_NOT_SUPPORTED=%s",
+	     bool_str(child->sa.st_seen_no_tfc));
+
 	if (md->pd[PD_v2N_IPCOMP_SUPPORTED] != NULL) {
 		struct pbs_in pbs = md->pd[PD_v2N_IPCOMP_SUPPORTED]->pbs;
 		size_t len = pbs_left(&pbs);
@@ -1031,10 +1056,10 @@ bool process_any_v2_IKE_AUTH_request_child_payloads(struct ike_sa *ike,
 
 	v2_notification_t cn = process_v2_IKE_AUTH_request_child_sa_payloads(ike, child, md, sk_pbs);
 	if (cn != v2N_NOTHING_WRONG) {
-		connection_delete_child(&ike->sa.st_v2_msgid_windows.responder.wip_sa, HERE);
+		connection_teardown_child(&ike->sa.st_v2_msgid_windows.responder.wip_sa, REASON_DELETED, HERE);
 		if (v2_notification_fatal(cn)) {
 			record_v2N_response(ike->sa.logger, ike, md,
-					    cn, NULL/*no-data*/,
+					    cn, empty_shunk/*no-data*/,
 					    ENCRYPTED_PAYLOAD);
 			return false;
 		}
@@ -1134,7 +1159,7 @@ v2_notification_t process_v2_IKE_AUTH_response_child_payloads(struct ike_sa *ike
 			    ike->sa.st_serialno, child->sa.st_serialno,
 			    pri_connection(child->sa.st_connection, &cb));
 			unpend(ike, child->sa.st_connection);
-			connection_delete_child(&child, HERE);
+			connection_teardown_child(&child, REASON_DELETED, HERE);
 			ike->sa.st_v2_msgid_windows.initiator.wip_sa = child = NULL;
 			/* handled */
 			return v2N_NOTHING_WRONG;
@@ -1157,8 +1182,9 @@ v2_notification_t process_v2_IKE_AUTH_response_child_payloads(struct ike_sa *ike
 		return v2N_TS_UNACCEPTABLE;
 	}
 
+	/* AUTH is ok, we can trust the notify payloads */
+
 	child->sa.st_ikev2_anon = ike->sa.st_ikev2_anon; /* was set after duplicate_state() (?!?) */
-	child->sa.st_seen_no_tfc = response_md->pd[PD_v2N_ESP_TFC_PADDING_NOT_SUPPORTED] != NULL;
 
 	/* examine and accept SA ESP/AH proposals */
 
