@@ -14,6 +14,7 @@
  * Copyright (C) 2015-2019 Andrew Cagney <cagney@gnu.org>
  * Copyright (C) 2017-2018 Sahana Prasad <sahana.prasad07@gmail.com>
  * Copyright (C) 2017 Vukasin Karadzic <vukasin.karadzic@gmail.com>
+ * Copyright (C) 2020 Nupur Agrawal <nupur202000@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -38,6 +39,7 @@
 #include "state.h"
 #include "ikev2.h"
 #include "ikev2_ike_sa_init.h"
+#include "ikev2_notification.h"
 
 /*
  * That the cookie size of 32-bytes happens to match
@@ -96,7 +98,8 @@ bool v2_rejected_initiator_cookie(struct msg_digest *md,
 	/* establish some home truths, but don't barf */
 	if (!pexpect(md->hdr.isa_msgid == 0) ||
 	    !pexpect(v2_msg_role(md) == MESSAGE_REQUEST) ||
-	    !pexpect(md->hdr.isa_xchg == ISAKMP_v2_IKE_SA_INIT) ||
+	    !pexpect((md->hdr.isa_xchg == ISAKMP_v2_IKE_SA_INIT) ||
+		     (md->hdr.isa_xchg == ISAKMP_v2_IKE_SESSION_RESUME)) ||
 	    !pexpect(md->hdr.isa_flags & ISAKMP_FLAGS_v2_IKE_I)) {
 		return true; /* reject cookie */
 	}
@@ -151,8 +154,8 @@ bool v2_rejected_initiator_cookie(struct msg_digest *md,
 
 	/* No cookie? demand one */
 	if (me_want_cookie && cookie_digest == NULL) {
-		llog_md(md, "DOS mode on; responding to IKE_SA_INIT with cookie notification request");
-		send_v2N_response_from_md(md, v2N_COOKIE, &local_cookie);
+		send_v2N_response_from_md(md, v2N_COOKIE, &local_cookie,
+					  "DOS mode is on, initial request must include a COOKIE");
 		return true; /* reject cookie */
 	}
 
@@ -268,3 +271,11 @@ stf_status process_v2_IKE_SA_INIT_response_v2N_COOKIE(struct ike_sa *ike,
 	schedule_reinitiate_v2_ike_sa_init(ike, resume_IKE_SA_INIT_with_cookie);
 	return STF_OK;
 }
+
+stf_status process_v2_IKE_SESSION_RESUME_response_v2N_COOKIE(struct ike_sa *ike,
+							     struct child_sa *child UNUSED,
+							     struct msg_digest *md UNUSED)
+{
+	llog_pexpect(ike->sa.logger, HERE, "not implemented");
+	return STF_FATAL;
+}
