@@ -48,23 +48,23 @@ void ldbg_cp(struct logger *logger, const struct connection *cc, const char *fmt
 		jam_va_list(buf, fmt, ap);
 		va_end(ap);
 		FOR_EACH_THING(lr, LEFT_END, RIGHT_END) {
-			const struct host_end_config *end = &cc->end[lr].config->host;
+			const struct config_end *end = cc->end[lr].config;
 			jam_string(buf, "; ");
 			jam_string(buf, end->leftright);
 			jam_string(buf, ":");
-			if (end == &cc->local->config->host) {
+			if (end == cc->local->config) {
 				jam_string(buf, " local");
 			}
-			if (end == &cc->remote->config->host) {
+			if (end == cc->remote->config) {
 				jam_string(buf, " remote");
 			}
-			if (end->modecfg.client) {
+			if (end->host.modecfg.client) {
 				jam_string(buf, " client");
 			}
-			if (end->modecfg.server) {
+			if (end->host.modecfg.server) {
 				jam_string(buf, " server");
 			}
-			if (end->pool_ranges.len > 0) {
+			if (end->child.addresspools.len > 0) {
 				jam_string(buf, " addresspool");
 			}
 		}
@@ -306,7 +306,9 @@ static bool lease_cp_address(struct child_sa *child, const struct ip_info *afi)
 		return true; /*non-fatal*/
 	}
 
-	err_t e = lease_that_address(cc, /*xauth-username*/NULL, afi, child->sa.logger);
+	err_t e = assign_remote_lease(cc, /*xauth-username*/NULL,
+				      afi, /*remote-address*/NULL,
+				      child->sa.logger);
 	if (e != NULL) {
 		llog_sa(RC_LOG, child, "leasing %s address failed: %s",
 			afi->ip_name, e);
@@ -382,7 +384,7 @@ bool process_v2_IKE_AUTH_request_v2CP_request_payload(struct ike_sa *ike,
 	/* rebuild the SPDs */
 	discard_connection_spds(cc);
 	PEXPECT(cc->logger, oriented(cc));
-	add_connection_spds(cc, address_info(cc->local->host.addr));
+	add_connection_spds(cc);
 
 	return true;
 }
