@@ -57,6 +57,9 @@
 #include "ikev2_parent.h"
 #include "ikev2_delete.h"
 #include "ikev2_states.h"
+#include "ikev2_prf.h"
+#include "crypt_symkey.h"
+#include "ikev2_notification.h"
 
 static ikev2_state_transition_fn process_v2_CREATE_CHILD_SA_request;
 
@@ -197,9 +200,13 @@ static void migrate_v2_children(struct ike_sa *from, struct child_sa *to)
 	/* passert(SPIs should be different) */
 	struct state_filter child = {
 		.clonedfrom = from->sa.st_serialno,
-		.where = HERE,
+		.search = {
+			.order = OLD2NEW,
+			.verbose.logger = &global_logger,
+			.where = HERE,
+		},
 	};
-	while (next_state(OLD2NEW, &child)) {
+	while (next_state(&child)) {
 		migrate_v2_child(from, to, pexpect_child_sa(child.st));
 	}
 }
@@ -222,7 +229,6 @@ static void emancipate_larval_ike_sa(struct ike_sa *old_ike, struct child_sa *ne
 
 	/* complete the state transition */
 	const struct v2_transition *transition = new_ike->sa.st_v2_transition;
-	pexpect(v2_transition_from(transition, new_ike->sa.st_state));
 	pexpect(transition->to == &state_v2_ESTABLISHED_IKE_SA);
 	change_v2_state(&new_ike->sa); /* should trash .st_v2_transition */
 
@@ -350,8 +356,8 @@ static bool find_v2N_REKEY_SA_child(struct ike_sa *ike,
 			str_enum(&ikev2_notify_protocol_id_names, rekey_notify->isan_protoid, &b));
 		record_v2N_spi_response(ike->sa.logger, ike, md,
 					rekey_notify->isan_protoid, &spi,
-					v2N_CHILD_SA_NOT_FOUND,
-					NULL/*empty data*/, ENCRYPTED_PAYLOAD);
+					v2N_CHILD_SA_NOT_FOUND, empty_shunk/*empty data*/,
+					ENCRYPTED_PAYLOAD);
 		return true;
 	}
 
@@ -549,7 +555,6 @@ static void llog_v2_success_rekey_child_request(struct ike_sa *ike)
 
 static const struct v2_transition v2_CREATE_CHILD_SA_rekey_child_initiate_transition = {
 	.story      = "initiate rekey Child_SA (CREATE_CHILD_SA)",
-	.from = { &state_v2_ESTABLISHED_IKE_SA, },
 	.to = &state_v2_ESTABLISHED_IKE_SA,
 	.exchange   = ISAKMP_v2_CREATE_CHILD_SA,
 	.processor  = initiate_v2_CREATE_CHILD_SA_rekey_child_request,
@@ -575,7 +580,6 @@ static const struct v2_transition v2_CREATE_CHILD_SA_rekey_child_responder_trans
 	 */
 
 	{ .story      = "process rekey Child SA request (CREATE_CHILD_SA)",
-	  .from = { &state_v2_ESTABLISHED_IKE_SA, },
 	  .to = &state_v2_ESTABLISHED_IKE_SA,
 	  .flags = { .release_whack = true, },
 	  .exchange   = ISAKMP_v2_CREATE_CHILD_SA,
@@ -597,7 +601,6 @@ static const struct v2_transitions v2_CREATE_CHILD_SA_rekey_child_responder_tran
 static const struct v2_transition v2_CREATE_CHILD_SA_response_transition[] = {
 
 	{ .story      = "process rekey IKE SA response (CREATE_CHILD_SA)",
-	  .from = { &state_v2_ESTABLISHED_IKE_SA, },
 	  .to = &state_v2_ESTABLISHED_IKE_SA,
 	  .exchange   = ISAKMP_v2_CREATE_CHILD_SA,
 	  .recv_role  = MESSAGE_RESPONSE,
@@ -609,7 +612,6 @@ static const struct v2_transition v2_CREATE_CHILD_SA_response_transition[] = {
 	  .timeout_event = EVENT_RETAIN, },
 
 	{ .story      = "process Child SA response (new or rekey) (CREATE_CHILD_SA)",
-	  .from = { &state_v2_ESTABLISHED_IKE_SA, },
 	  .to = &state_v2_ESTABLISHED_IKE_SA,
 	  .flags = { .release_whack = true, },
 	  .exchange   = ISAKMP_v2_CREATE_CHILD_SA,
@@ -622,7 +624,6 @@ static const struct v2_transition v2_CREATE_CHILD_SA_response_transition[] = {
 	  .timeout_event = EVENT_RETAIN, },
 
 	{ .story      = "process CREATE_CHILD_SA failure response (new or rekey Child SA, rekey IKE SA)",
-	  .from = { &state_v2_ESTABLISHED_IKE_SA, },
 	  .to = &state_v2_ESTABLISHED_IKE_SA,
 	  .flags = { .release_whack = true, },
 	  .exchange   = ISAKMP_v2_CREATE_CHILD_SA,
@@ -643,7 +644,8 @@ const struct v2_exchange v2_CREATE_CHILD_SA_rekey_child_exchange = {
 	.type = ISAKMP_v2_CREATE_CHILD_SA,
 	.subplot = "rekey Child SA",
 	.secured = true,
-	.initiate = &v2_CREATE_CHILD_SA_rekey_child_initiate_transition,
+	.initiate.from = { &state_v2_ESTABLISHED_IKE_SA, },
+	.initiate.transition = &v2_CREATE_CHILD_SA_rekey_child_initiate_transition,
 	.responder = &v2_CREATE_CHILD_SA_rekey_child_responder_transitions,
 	.response = &v2_CREATE_CHILD_SA_response_transitions,
 };
@@ -703,7 +705,7 @@ stf_status initiate_v2_CREATE_CHILD_SA_rekey_child_request(struct ike_sa *ike,
 		llog_sa(RC_LOG, larval_child,
 			"IKE SA #%lu no longer viable for rekey of Child SA #%lu",
 			ike->sa.st_serialno, larval_child->sa.st_v2_rekey_pred);
-		connection_delete_child(&larval_child, HERE);
+		connection_teardown_child(&larval_child, REASON_DELETED, HERE);
 		ike->sa.st_v2_msgid_windows.initiator.wip_sa = larval_child = NULL;
 		return STF_OK; /* IKE */
 	}
@@ -728,9 +730,9 @@ stf_status initiate_v2_CREATE_CHILD_SA_rekey_child_request(struct ike_sa *ike,
 		 *
 		 * The older child should have discarded this state.
 		 */
-		llog_sa(LOG_STREAM/*not-whack*/, larval_child,
-			"Child SA to rekey #%lu vanished abort this exchange",
-			larval_child->sa.st_v2_rekey_pred);
+		llog_pexpect(larval_child->sa.logger, HERE,
+			     "Child SA to rekey #%lu vanished abort this exchange",
+			     larval_child->sa.st_v2_rekey_pred);
 		return STF_INTERNAL_ERROR;
 	}
 
@@ -823,8 +825,9 @@ stf_status process_v2_CREATE_CHILD_SA_rekey_child_request(struct ike_sa *ike,
 
 	struct child_sa *predecessor = NULL;
 	if (!find_v2N_REKEY_SA_child(ike, md, &predecessor)) {
-		record_v2N_response(ike->sa.logger, ike, md, v2N_INVALID_SYNTAX,
-				    NULL/*empty data*/, ENCRYPTED_PAYLOAD);
+		record_v2N_response(ike->sa.logger, ike, md,
+				    v2N_INVALID_SYNTAX, empty_shunk/*no-data*/,
+				    ENCRYPTED_PAYLOAD);
 		return STF_FATAL;
 	}
 
@@ -846,7 +849,7 @@ stf_status process_v2_CREATE_CHILD_SA_rekey_child_request(struct ike_sa *ike,
 
 	if (!verify_rekey_child_request_ts(larval_child, md)) {
 		record_v2N_response(ike->sa.logger, ike, md,
-				    v2N_TS_UNACCEPTABLE, NULL/*no data*/,
+				    v2N_TS_UNACCEPTABLE, empty_shunk/*no data*/,
 				    ENCRYPTED_PAYLOAD);
 		delete_child_sa(&larval_child);
 		ike->sa.st_v2_msgid_windows.responder.wip_sa = NULL;
@@ -923,7 +926,6 @@ static void llog_v2_success_new_child_request(struct ike_sa *ike)
 
 static const struct v2_transition v2_CREATE_CHILD_SA_new_child_initiate_transition = {
 	.story      = "initiate new Child SA (CREATE_CHILD_SA)",
-	.from = { &state_v2_ESTABLISHED_IKE_SA, },
 	.to = &state_v2_ESTABLISHED_IKE_SA,
 	.exchange   = ISAKMP_v2_CREATE_CHILD_SA,
 	.processor  = initiate_v2_CREATE_CHILD_SA_new_child_request,
@@ -949,7 +951,6 @@ static const struct v2_transition v2_CREATE_CHILD_SA_new_child_responder_transit
 	 */
 
 	{ .story      = "process create Child SA request (CREATE_CHILD_SA)",
-	  .from = { &state_v2_ESTABLISHED_IKE_SA, },
 	  .to = &state_v2_ESTABLISHED_IKE_SA,
 	  .flags = { .release_whack = true, },
 	  .exchange   = ISAKMP_v2_CREATE_CHILD_SA,
@@ -971,7 +972,8 @@ const struct v2_exchange v2_CREATE_CHILD_SA_new_child_exchange = {
 	.type = ISAKMP_v2_CREATE_CHILD_SA,
 	.subplot = "new Child SA",
 	.secured = true,
-	.initiate = &v2_CREATE_CHILD_SA_new_child_initiate_transition,
+	.initiate.from = { &state_v2_ESTABLISHED_IKE_SA, },
+	.initiate.transition = &v2_CREATE_CHILD_SA_new_child_initiate_transition,
 	.responder = &v2_CREATE_CHILD_SA_new_child_responder_transitions,
 	.response = &v2_CREATE_CHILD_SA_response_transitions,
 };
@@ -1031,7 +1033,7 @@ stf_status initiate_v2_CREATE_CHILD_SA_new_child_request(struct ike_sa *ike,
 		llog_sa(RC_LOG, larval_child,
 			"IKE SA #%lu no longer viable for initiating a Child SA",
 			ike->sa.st_serialno);
-		connection_delete_child(&larval_child, HERE);
+		connection_teardown_child(&larval_child, REASON_DELETED, HERE);
 		ike->sa.st_v2_msgid_windows.initiator.wip_sa = larval_child = NULL;
 		return STF_OK; /* IKE */
 	}
@@ -1103,8 +1105,8 @@ stf_status process_v2_CREATE_CHILD_SA_new_child_request(struct ike_sa *ike,
 	if (!process_v2TS_request_payloads(larval_child, md)) {
 		/* already logged */
 		record_v2N_response(larval_child->sa.logger, ike, md,
-				    v2N_TS_UNACCEPTABLE,
-				    NULL/*no-data*/, ENCRYPTED_PAYLOAD);
+				    v2N_TS_UNACCEPTABLE, empty_shunk/*no-data*/,
+				    ENCRYPTED_PAYLOAD);
 		delete_child_sa(&larval_child);
 		ike->sa.st_v2_msgid_windows.responder.wip_sa = NULL;
 		return STF_OK; /*IKE*/
@@ -1133,12 +1135,12 @@ static stf_status reject_CREATE_CHILD_SA_request(struct ike_sa *ike,
 	 * STF_OK.
 	 */
 	record_v2N_response(ike->sa.logger, ike, md,
-			    n, NULL/*no-data*/,
+			    n, empty_shunk/*no-data*/,
 			    ENCRYPTED_PAYLOAD);
 	/*
 	 * Child could have been partially routed; need to move it on.
 	 */
-	connection_delete_child(larval, where);
+	connection_teardown_child(larval, REASON_DELETED, where);
 	ike->sa.st_v2_msgid_windows.responder.wip_sa = NULL;
 	return v2_notification_fatal(n) ? STF_FATAL : STF_OK; /*IKE*/
 }
@@ -1165,7 +1167,7 @@ stf_status process_v2_CREATE_CHILD_SA_request(struct ike_sa *ike,
 		 * implicitly kills the family.
 		 */
 		record_v2N_response(ike->sa.logger, ike, md,
-				    v2N_INVALID_SYNTAX, NULL/*no-data*/,
+				    v2N_INVALID_SYNTAX, empty_shunk/*no-data*/,
 				    ENCRYPTED_PAYLOAD);
 		delete_child_sa(&larval_child);
 		ike->sa.st_v2_msgid_windows.responder.wip_sa = NULL;
@@ -1281,7 +1283,7 @@ static stf_status process_v2_CREATE_CHILD_SA_request_continue_2(struct state *ik
 	if (larval_child->sa.st_dh_shared_secret == NULL) {
 		log_state(RC_LOG, &larval_child->sa, "DH failed");
 		record_v2N_response(larval_child->sa.logger, ike, request_md,
-				    v2N_INVALID_SYNTAX, NULL,
+				    v2N_INVALID_SYNTAX, empty_shunk,
 				    ENCRYPTED_PAYLOAD);
 		delete_child_sa(&larval_child);
 		ike->sa.st_v2_msgid_windows.responder.wip_sa = NULL;
@@ -1563,6 +1565,35 @@ static stf_status process_v2_CREATE_CHILD_SA_child_response_continue_1(struct st
  * to the message queue.
  */
 
+static bool calc_v2_rekey_ike_keymat(struct ike_sa *old_ike,
+				     struct child_sa *larval_ike,
+				     const ike_spis_t *new_ike_spis,
+				     where_t where)
+{
+	struct logger *logger = larval_ike->sa.logger;
+	PK11SymKey *shared = larval_ike->sa.st_dh_shared_secret;
+
+	const struct prf_desc *old_prf = old_ike->sa.st_oakley.ta_prf;
+	PK11SymKey *old_d = old_ike->sa.st_skey_d_nss;
+	ldbg(logger, "%s() calculating skeyseed using prf %s",
+	     __func__, old_prf->common.fqn);
+
+	PK11SymKey *skeyseed =
+		ikev2_ike_sa_rekey_skeyseed(old_prf, old_d,
+					    shared,
+					    larval_ike->sa.st_ni,
+					    larval_ike->sa.st_nr,
+					    logger);
+	if (skeyseed == NULL) {
+		llog_pexpect(logger, where, "rekey SKEYSEED failed");
+		return false;
+	}
+
+	calc_v2_ike_keymat(&larval_ike->sa, skeyseed, new_ike_spis);
+	symkey_delref(logger, "skeyseed", &skeyseed);
+	return true;
+}
+
 struct child_sa *submit_v2_CREATE_CHILD_SA_rekey_ike(struct ike_sa *ike,
 						     bool detach_whack)
 {
@@ -1630,7 +1661,6 @@ static void llog_v2_success_rekey_ike_request(struct ike_sa *ike)
 
 static const struct v2_transition v2_CREATE_CHILD_SA_rekey_ike_initiate_transition = {
 	.story      = "initiate rekey IKE_SA (CREATE_CHILD_SA)",
-	.from = { &state_v2_ESTABLISHED_IKE_SA, },
 	.to = &state_v2_ESTABLISHED_IKE_SA,
 	.exchange   = ISAKMP_v2_CREATE_CHILD_SA,
 	.processor  = initiate_v2_CREATE_CHILD_SA_rekey_ike_request,
@@ -1655,7 +1685,6 @@ static const struct v2_transition v2_CREATE_CHILD_SA_rekey_ike_responder_transit
 	 */
 
 	{ .story      = "process rekey IKE SA request (CREATE_CHILD_SA)",
-	  .from = { &state_v2_ESTABLISHED_IKE_SA, },
 	  .to = &state_v2_ESTABLISHED_IKE_SA,
 	  .flags = { .release_whack = true, },
 	  .exchange   = ISAKMP_v2_CREATE_CHILD_SA,
@@ -1677,7 +1706,8 @@ const struct v2_exchange v2_CREATE_CHILD_SA_rekey_ike_exchange = {
 	.type = ISAKMP_v2_CREATE_CHILD_SA,
 	.subplot = "rekey IKE SA",
 	.secured = true,
-	.initiate = &v2_CREATE_CHILD_SA_rekey_ike_initiate_transition,
+	.initiate.from = { &state_v2_ESTABLISHED_IKE_SA, },
+	.initiate.transition = &v2_CREATE_CHILD_SA_rekey_ike_initiate_transition,
 	.responder = &v2_CREATE_CHILD_SA_rekey_ike_responder_transitions,
 	.response = &v2_CREATE_CHILD_SA_response_transitions,
 };
@@ -1762,7 +1792,7 @@ stf_status process_v2_CREATE_CHILD_SA_rekey_ike_request(struct ike_sa *ike,
 		 * Already logged?
 		 */
 		record_v2N_response(ike->sa.logger, ike, request_md,
-				    v2N_INVALID_SYNTAX, NULL/*no-data*/,
+				    v2N_INVALID_SYNTAX, empty_shunk/*no-data*/,
 				    ENCRYPTED_PAYLOAD);
 		delete_child_sa(&larval_ike);
 		ike->sa.st_v2_msgid_windows.responder.wip_sa = NULL;
@@ -1784,7 +1814,8 @@ stf_status process_v2_CREATE_CHILD_SA_rekey_ike_request(struct ike_sa *ike,
 	if (n != v2N_NOTHING_WRONG) {
 		pexpect(larval_ike->sa.st_sa_role == SA_RESPONDER);
 		record_v2N_response(larval_ike->sa.logger, ike, request_md,
-				    n, NULL, ENCRYPTED_PAYLOAD);
+				    n, empty_shunk,
+				    ENCRYPTED_PAYLOAD);
 		delete_child_sa(&larval_ike);
 		ike->sa.st_v2_msgid_windows.responder.wip_sa = NULL;
 		return v2_notification_fatal(n) ? STF_FATAL : STF_OK; /* IKE */
@@ -1889,24 +1920,24 @@ static stf_status process_v2_CREATE_CHILD_SA_rekey_ike_request_continue_2(struct
 	passert(v2_msg_role(request_md) == MESSAGE_REQUEST); /* i.e., MD!=NULL */
 	passert(larval_ike->sa.st_sa_role == SA_RESPONDER);
 	pexpect(larval_ike->sa.st_state == &state_v2_REKEY_IKE_R0);
-	pexpect(v2_transition_from(larval_ike->sa.st_v2_transition, &state_v2_REKEY_IKE_R0));
-	pexpect(larval_ike->sa.st_v2_transition->to == &state_v2_ESTABLISHED_IKE_SA);
 	dbg("%s() for #%lu %s",
 	     __func__, larval_ike->sa.st_serialno, larval_ike->sa.st_state->name);
 
 	if (larval_ike->sa.st_dh_shared_secret == NULL) {
 		record_v2N_response(ike->sa.logger, ike, request_md,
-				    v2N_INVALID_SYNTAX, NULL,
+				    v2N_INVALID_SYNTAX, empty_shunk,
 				    ENCRYPTED_PAYLOAD);
 		delete_child_sa(&larval_ike);
 		ike->sa.st_v2_msgid_windows.responder.wip_sa = NULL;
 		return STF_FATAL; /* IKE family is doomed */
 	}
 
-	calc_v2_keymat(&larval_ike->sa,
-		       ike->sa.st_skey_d_nss, /* only IKE has SK_d */
-		       ike->sa.st_oakley.ta_prf, /* for IKE/ESP/AH */
-		       &larval_ike->sa.st_ike_rekey_spis);
+	if (!calc_v2_rekey_ike_keymat(ike, larval_ike,
+				      &larval_ike->sa.st_ike_rekey_spis,
+				      HERE)) {
+		/* already logged */
+		return STF_FATAL;
+	}
 
 	if (!record_v2_rekey_ike_message(ike, larval_ike, /*responder*/request_md)) {
 		return STF_INTERNAL_ERROR;
@@ -2039,8 +2070,6 @@ static stf_status process_v2_CREATE_CHILD_SA_rekey_ike_response_continue_1(struc
 	pexpect(larval_ike->sa.st_sa_role == SA_INITIATOR);
 	pexpect(larval_ike->sa.st_sa_type_when_established == IKE_SA);
 	pexpect(larval_ike->sa.st_state == &state_v2_REKEY_IKE_I1);
-	pexpect(v2_transition_from(larval_ike->sa.st_v2_transition, &state_v2_REKEY_IKE_I1));
-	pexpect(larval_ike->sa.st_v2_transition->to == &state_v2_ESTABLISHED_IKE_SA);
 	pexpect(v2_msg_role(response_md) == MESSAGE_RESPONSE); /* i.e., MD!=NULL */
 
 	dbg("%s() for #%lu %s",
@@ -2056,10 +2085,12 @@ static stf_status process_v2_CREATE_CHILD_SA_rekey_ike_response_continue_1(struc
 		return STF_FATAL;
 	}
 
-	calc_v2_keymat(&larval_ike->sa,
-		       ike->sa.st_skey_d_nss, /* only IKE has SK_d */
-		       ike->sa.st_oakley.ta_prf, /* for IKE/ESP/AH */
-		       &larval_ike->sa.st_ike_rekey_spis/* new SPIs */);
+	if (!calc_v2_rekey_ike_keymat(ike, larval_ike,
+				      &larval_ike->sa.st_ike_rekey_spis/* new SPIs */,
+				      HERE)) {
+		/* already logged */
+		return STF_FATAL;
+	}
 
 	pexpect(larval_ike->sa.st_v2_rekey_pred == ike->sa.st_serialno); /*wow!*/
 	ikev2_rekey_expire_predecessor(larval_ike, larval_ike->sa.st_v2_rekey_pred);
@@ -2176,7 +2207,7 @@ stf_status process_v2_CREATE_CHILD_SA_failure_response(struct ike_sa *ike,
 		state_detach(replacing, (*larval_child)->sa.logger);
 	}
 
-	connection_delete_child(larval_child, HERE);
+	connection_teardown_child(larval_child, REASON_DELETED, HERE);
 
 	return status; /* IKE */
 }
