@@ -32,12 +32,12 @@
 #include "constants.h"
 
 #include "defs.h"
+#include "log.h"
 #include "id.h"
 #include "x509.h"
 #include "certs.h"
 #include "connections.h"        /* needs id.h */
 #include "state.h"
-#include "packet.h"
 #include "crypto.h"
 #include "demux.h"
 #include "ikev2.h"
@@ -53,29 +53,28 @@ void ikev2_derive_child_keys(struct ike_sa *ike, struct child_sa *child)
 	chunk_t ikeymat, rkeymat;
 	/* ??? note assumption that AH and ESP cannot be combined */
 	struct ipsec_proto_info *ipi =
-		child->sa.st_esp.present ? &child->sa.st_esp :
-		child->sa.st_ah.present ? &child->sa.st_ah :
+		child->sa.st_esp.protocol == &ip_protocol_esp ? &child->sa.st_esp :
+		child->sa.st_ah.protocol == &ip_protocol_ah ? &child->sa.st_ah :
 		NULL;
 
 	passert(ipi != NULL);	/* ESP or AH must be present */
-	passert(child->sa.st_esp.present != child->sa.st_ah.present);	/* exactly one */
 
 	/*
 	 * Integrity seed (key).  AEAD, for instance has NULL (no)
 	 * separate integrity.
 	 */
-	const struct integ_desc *integ = ipi->attrs.transattrs.ta_integ;
+	const struct integ_desc *integ = ipi->trans_attrs.ta_integ;
 	size_t integ_key_size = (integ != NULL ? integ->integ_keymat_size : 0);
 	/*
 	 * If there is encryption, then ENCKEYLEN contains the
 	 * required number of bits.
 	 */
-	size_t encrypt_key_size = BYTES_FOR_BITS(ipi->attrs.transattrs.enckeylen);
+	size_t encrypt_key_size = BYTES_FOR_BITS(ipi->trans_attrs.enckeylen);
 	/*
 	 * Finally, some encryption algorithms such as AEAD and CTR
 	 * require "salt" as part of the "starting variable".
 	 */
-	const struct encrypt_desc *encrypt = ipi->attrs.transattrs.ta_encrypt;
+	const struct encrypt_desc *encrypt = ipi->trans_attrs.ta_encrypt;
 	size_t encrypt_salt_size = (encrypt != NULL ? encrypt->salt_size : 0);
 
 	size_t keymat_len = integ_key_size + encrypt_key_size + encrypt_salt_size;
@@ -108,8 +107,9 @@ void ikev2_derive_child_keys(struct ike_sa *ike, struct child_sa *child)
 	 */
 	PK11SymKey *shared = NULL;
 	if (child->sa.st_pfs_group != NULL) {
-		DBGF(DBG_CRYPT, "#%lu %s add g^ir to child key %p",
-		     child->sa.st_serialno, child->sa.st_state->name, child->sa.st_dh_shared_secret);
+		ldbgf(DBG_CRYPT, child->sa.logger,
+		      "#%lu %s add g^ir to child key %p",
+		      child->sa.st_serialno, child->sa.st_state->name, child->sa.st_dh_shared_secret);
 		shared = child->sa.st_dh_shared_secret;
 	}
 
@@ -119,22 +119,22 @@ void ikev2_derive_child_keys(struct ike_sa *ike, struct child_sa *child)
 						   child->sa.st_ni,
 						   child->sa.st_nr,
 						   keymat_len * 2,
-						   child->sa.st_logger);
+						   child->sa.logger);
 	PK11SymKey *ikey = key_from_symkey_bytes("initiator to responder key",
 						 keymat, 0, keymat_len,
-						 HERE, child->sa.st_logger);
+						 HERE, child->sa.logger);
 	ikeymat = chunk_from_symkey("initiator to responder keys", ikey,
-				    child->sa.st_logger);
-	release_symkey(__func__, "ikey", &ikey);
+				    child->sa.logger);
+	symkey_delref(child->sa.logger, "ikey", &ikey);
 
 	PK11SymKey *rkey = key_from_symkey_bytes("responder to initiator key",
 						 keymat, keymat_len, keymat_len,
-						 HERE, child->sa.st_logger);
+						 HERE, child->sa.logger);
 	rkeymat = chunk_from_symkey("responder to initiator keys:", rkey,
-				    child->sa.st_logger);
-	release_symkey(__func__, "rkey", &rkey);
+				    child->sa.logger);
+	symkey_delref(child->sa.logger, "rkey", &rkey);
 
-	release_symkey(__func__, "keymat", &keymat);
+	symkey_delref(child->sa.logger, "keymat", &keymat);
 
 	/*
 	 * The initiator stores outgoing initiator-to-responder keymat
