@@ -23,10 +23,12 @@
 #include "log.h"
 #include "demux.h"
 #include "connections.h"
+#include "ikev2_notification.h"
 #include "ikev2_informational.h"
 
 static bool process_v2DELETE_requests(bool *del_ike, struct ike_sa *ike,
 				      struct msg_digest *md, struct pbs_out *pbs);
+
 static emit_v2_INFORMATIONAL_request_payload_fn emit_v2DELETE;
 static ikev2_state_transition_fn initiate_v2_INFORMATIONAL_v2DELETE_request;
 static ikev2_state_transition_fn process_v2_INFORMATIONAL_v2DELETE_request;
@@ -168,7 +170,8 @@ static stf_status process_v2_INFORMATIONAL_v2DELETE_request(struct ike_sa *ike,
 	bool del_ike = false;
 	if (!process_v2DELETE_requests(&del_ike, ike, md, response.pbs)) {
 		record_v2N_response(ike->sa.logger, ike, md,
-				    v2N_INVALID_SYNTAX, NULL, ENCRYPTED_PAYLOAD);
+				    v2N_INVALID_SYNTAX, empty_shunk,
+				    ENCRYPTED_PAYLOAD);
 		/*
 		 * STF_FATAL will send the recorded message
 		 * and then kill the IKE SA.  Should it
@@ -232,7 +235,7 @@ static void llog_v2_success_v2_INFORMATIONAL_v2DELETE_request(struct ike_sa *ike
 void submit_v2_delete_exchange(struct ike_sa *ike, struct child_sa *child)
 {
 	const struct v2_exchange *exchange = &v2_INFORMATIONAL_v2DELETE_exchange;
-	pexpect(exchange->initiate->exchange == ISAKMP_v2_INFORMATIONAL);
+	pexpect(exchange->initiate.transition->exchange == ISAKMP_v2_INFORMATIONAL);
 	if (child == NULL) {
 		/*
 		 * IKE SA is no longer viable - reviving Child SA's
@@ -413,7 +416,7 @@ bool process_v2DELETE_requests(bool *del_ike, struct ike_sa *ike, struct msg_dig
 						"too many SPIs in Delete Notification payload; ignoring outbound SPI "PRI_IPSEC_SPI,
 						pri_ipsec_spi(outbound_spi));
 				}
-				connection_delete_child(&child, HERE);
+				connection_teardown_child(&child, REASON_DELETED, HERE);
 
 			} /* for each spi */
 
@@ -501,7 +504,7 @@ static stf_status process_v2_INFORMATIONAL_v2DELETE_response(struct ike_sa *ike,
 	 */
 	PEXPECT(ike->sa.logger, &ike->sa != &dead_child->sa);
 	PEXPECT(ike->sa.logger, ike->sa.st_serialno == dead_child->sa.st_clonedfrom);
-	connection_delete_child(&dead_child, HERE);
+	connection_teardown_child(&dead_child, REASON_DELETED, HERE);
 
 	return STF_OK;
 }
@@ -551,7 +554,6 @@ void record_n_send_n_log_v2_delete(struct ike_sa *ike, where_t where)
 
 static const struct v2_transition v2_INFORMATIONAL_v2DELETE_initiate_transition = {
 	.story = "initiate Informational Delete IKE or Child SA",
-	.from = { &state_v2_ESTABLISHED_IKE_SA, },
 	.to = &state_v2_ESTABLISHED_IKE_SA,
 	.exchange = ISAKMP_v2_INFORMATIONAL,
 	.processor = initiate_v2_INFORMATIONAL_v2DELETE_request,
@@ -561,7 +563,6 @@ static const struct v2_transition v2_INFORMATIONAL_v2DELETE_initiate_transition
 
 static const struct v2_transition v2_INFORMATIONAL_v2DELETE_responder_transition[] = {
 	{ .story      = "process Informational Delete IKE or Child SA request",
-	  .from = { &state_v2_ESTABLISHED_IKE_SA, },
 	  .to = &state_v2_ESTABLISHED_IKE_SA,
 	  .exchange   = ISAKMP_v2_INFORMATIONAL,
 	  .recv_role  = MESSAGE_REQUEST,
@@ -579,7 +580,6 @@ static const struct v2_transitions v2_INFORMATIONAL_v2DELETE_responder_transitio
 static const struct v2_transition v2_INFORMATIONAL_v2DELETE_response_transition[] = {
 
 	{ .story      = "process Informational Delete IKE or Child SA response",
-	  .from = { &state_v2_ESTABLISHED_IKE_SA, },
 	  .to = &state_v2_ESTABLISHED_IKE_SA,
 	  .exchange   = ISAKMP_v2_INFORMATIONAL,
 	  .recv_role  = MESSAGE_RESPONSE,
@@ -600,7 +600,8 @@ const struct v2_exchange v2_INFORMATIONAL_v2DELETE_exchange = {
 	.type = ISAKMP_v2_INFORMATIONAL,
 	.subplot = "delete IKE or Child SA",
 	.secured = true,
-	.initiate = &v2_INFORMATIONAL_v2DELETE_initiate_transition,
+	.initiate.from = { &state_v2_ESTABLISHED_IKE_SA, },
+	.initiate.transition = &v2_INFORMATIONAL_v2DELETE_initiate_transition,
 	.responder = &v2_INFORMATIONAL_v2DELETE_responder_transitions,
 	.response = &v2_INFORMATIONAL_v2DELETE_response_transitions,
 };
