@@ -52,6 +52,7 @@
 #include "crypt_prf.h"
 #include "ikev2_states.h"
 #include "ikev2_auth.h"
+#include "ikev2_notification.h"
 
 static ikev2_state_transition_fn process_v2_IKE_AUTH_request_EAP_start;
 static ikev2_state_transition_fn process_v2_IKE_AUTH_request_EAP_final;
@@ -251,9 +252,9 @@ static bool start_eap(struct ike_sa *ike, struct pbs_out *pbs)
 	if (!mycert)
 		return false;
 
-	const struct secret_stuff *pks = get_local_private_key(c, &pubkey_type_rsa,
-								    ike->sa.logger);
-	if (!pks) {
+	const struct secret_pubkey_stuff *pks = get_local_private_key(c, &pubkey_type_rsa,
+								      ike->sa.logger);
+	if (pks == NULL) {
 		llog_sa(RC_LOG, ike, "private key for connection not found");
 		return false;
 	}
@@ -282,7 +283,7 @@ static bool start_eap(struct ike_sa *ike, struct pbs_out *pbs)
 	    SSL_OptionSet(pr, SSL_ENABLE_SSL3, PR_FALSE) != SECSuccess ||
 	    SSL_BadCertHook(pr, eaptls_bad_cert_cb, eap) != SECSuccess ||
 	    SSL_HandshakeCallback(pr, eaptls_handshake_cb, eap) != SECSuccess ||
-	    SSL_ConfigServerCert(pr, mycert->nss_cert, pks->u.pubkey.private_key, 0, 0) != SECSuccess) {
+	    SSL_ConfigServerCert(pr, mycert->nss_cert, pks->private_key, 0, 0) != SECSuccess) {
 		llog_nss_error(RC_LOG, logger, "Failed to start configure TLS options");
 		return false;
 	}
@@ -477,7 +478,7 @@ stf_status process_v2_IKE_AUTH_request_EAP_start(struct ike_sa *ike,
 auth_fail:
 	pstat_sa_failed(&ike->sa, REASON_AUTH_FAILED);
 	record_v2N_response(ike->sa.logger, ike, md,
-			    v2N_AUTHENTICATION_FAILED, NULL/*no-data*/,
+			    v2N_AUTHENTICATION_FAILED, empty_shunk/*no-data*/,
 			    ENCRYPTED_PAYLOAD);
 	return STF_FATAL;
 }
@@ -509,7 +510,7 @@ static stf_status process_v2_IKE_AUTH_request_EAP_start_signature_continue(struc
 		}
 	}
 
-	if (ike->sa.st_ppk_used) {
+	if (ike->sa.st_ppk_ike_auth_used) {
 		if (!emit_v2N(v2N_PPK_IDENTITY, response.pbs))
 			return STF_INTERNAL_ERROR;
 	}
@@ -585,7 +586,7 @@ static stf_status process_v2_IKE_AUTH_request_EAP_start_signature_continue(struc
 auth_fail:
 	pstat_sa_failed(&ike->sa, REASON_AUTH_FAILED);
 	record_v2N_response(ike->sa.logger, ike, md,
-			    v2N_AUTHENTICATION_FAILED, NULL/*no-data*/,
+			    v2N_AUTHENTICATION_FAILED, empty_shunk/*no-data*/,
 			    ENCRYPTED_PAYLOAD);
 	return STF_FATAL;
 }
@@ -722,7 +723,7 @@ stf_status process_v2_IKE_AUTH_request_EAP_final(struct ike_sa *ike,
 		pfree_diag(&d);
 		dbg("EAP AUTH failed");
 		record_v2N_response(ike->sa.logger, ike, md,
-				    v2N_AUTHENTICATION_FAILED, NULL/*no data*/,
+				    v2N_AUTHENTICATION_FAILED, empty_shunk/*no data*/,
 				    ENCRYPTED_PAYLOAD);
 		pstat_sa_failed(&ike->sa, REASON_AUTH_FAILED);
 		return STF_FATAL;
@@ -760,8 +761,9 @@ stf_status process_v2_IKE_AUTH_request_EAP_final(struct ike_sa *ike,
 	 * message with a DELETE payload.
 	 */
 	if (send_redirect) {
-		if (!emit_redirect_notification(shunk1(c->config->redirect.to), response.pbs))
+		if (!emit_v2N_REDIRECT(c->config->redirect.to, response.pbs)) {
 			return STF_INTERNAL_ERROR;
+		}
 		ike->sa.st_sent_redirect = true;	/* mark that we have sent REDIRECT in IKE_AUTH */
 	}
 
@@ -787,7 +789,7 @@ stf_status process_v2_IKE_AUTH_request_EAP_final(struct ike_sa *ike,
 		llog(RC_LOG, ike->sa.logger, "%s", str_diag(d));
 		pfree_diag(&d);
 		record_v2N_response(ike->sa.logger, ike, md,
-				    v2N_AUTHENTICATION_FAILED, NULL/*no data*/,
+				    v2N_AUTHENTICATION_FAILED, empty_shunk/*no data*/,
 				    ENCRYPTED_PAYLOAD);
 		return STF_FATAL;
 	}
@@ -839,7 +841,6 @@ stf_status process_v2_IKE_AUTH_request_EAP_final(struct ike_sa *ike,
 static const struct v2_transition v2_IKE_AUTH_EAP_responder_transition[] = {
 
 	{ .story      = "process initial IKE_AUTH(EAP) request",
-	  .from = { &state_v2_IKE_SA_INIT_R, &state_v2_IKE_INTERMEDIATE_R, },
 	  .to = &state_v2_IKE_AUTH_EAP_R,
 	  .exchange   = ISAKMP_v2_IKE_AUTH,
 	  .recv_role  = MESSAGE_REQUEST,
@@ -851,7 +852,6 @@ static const struct v2_transition v2_IKE_AUTH_EAP_responder_transition[] = {
 	  .timeout_event = EVENT_v2_DISCARD, },
 
 	{ .story      = "process continuing IKE_AUTH(EAP) request",
-	  .from = { &state_v2_IKE_AUTH_EAP_R, },
 	  .to = &state_v2_IKE_AUTH_EAP_R,
 	  .exchange   = ISAKMP_v2_IKE_AUTH,
 	  .recv_role  = MESSAGE_REQUEST,
@@ -862,7 +862,6 @@ static const struct v2_transition v2_IKE_AUTH_EAP_responder_transition[] = {
 	  .timeout_event = EVENT_v2_DISCARD, },
 
 	{ .story      = "process final IKE_AUTH(EAP) request",
-	  .from = { &state_v2_IKE_AUTH_EAP_R, },
 	  .to = &state_v2_ESTABLISHED_IKE_SA,
 	  .flags = { .release_whack = true, },
 	  .exchange   = ISAKMP_v2_IKE_AUTH,
