@@ -17,6 +17,7 @@
  * Copyright (C) 2017-2018 Vukasin Karadzic <vukasin.karadzic@gmail.com>
  * Copyright (C) 2017 Mayank Totale <mtotale@gmail.com>
  * Copyright (C) 2020 Yulia Kuzovkova <ukuzovkova@gmail.com>
+ * Copyright (C) 2020 Nupur Agrawal <nupur202000@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -71,6 +72,9 @@
 #include "revival.h"
 #include "ikev2_parent.h"
 #include "ikev2_states.h"
+#include "ikev2_ike_session_resume.h"
+#include "ikev2_notification.h"
+#include "peer_id.h"
 
 static ikev2_state_transition_fn process_v2_IKE_AUTH_request;
 
@@ -91,6 +95,9 @@ static stf_status process_v2_IKE_AUTH_request_ipseckey_continue(struct ike_sa *i
 
 static stf_status process_v2_IKE_AUTH_request_id_tail(struct ike_sa *ike, struct msg_digest *md);
 
+static stf_status process_v2_IKE_AUTH_request_skip_cert_decode(struct ike_sa *ike,
+							       struct msg_digest *md);
+
 static v2_auth_signature_cb process_v2_IKE_AUTH_request_auth_signature_continue; /* type check */
 
 static stf_status initiate_v2_IKE_AUTH_request(struct ike_sa *ike,
@@ -106,17 +113,19 @@ static stf_status initiate_v2_IKE_AUTH_request(struct ike_sa *ike,
 	struct connection *const pc = ike->sa.st_connection;	/* parent connection */
 
 	/*
+	 * Only RFC 8784 PPK mechanism here:
+	 *
 	 * If we and responder are willing to use a PPK, we need to
 	 * generate NO_PPK_AUTH as well as PPK-based AUTH payload.
 	 *
 	 * Stash the no-ppk keys in st_skey_*_no_ppk, and then
 	 * scramble the st_skey_* keys with PPK.
 	 */
-	if (ike->sa.st_v2_ike_ppk_enabled) {
-		chunk_t *ppk_id;
-		const shunk_t ppk = get_connection_ppk_initiator(ike->sa.st_connection, &ppk_id);
+	if (ike->sa.st_v2_ike_ppk == PPK_IKE_AUTH) {
+		const struct secret_ppk_stuff *ppk =
+			get_connection_ppk_and_ppk_id(ike->sa.st_connection);
 
-		if (ppk.ptr != NULL) {
+		if (ppk != NULL) {
 			dbg("found PPK and PPK_ID for our connection");
 
 			pexpect(ike->sa.st_sk_d_no_ppk == NULL);
@@ -131,7 +140,7 @@ static stf_status initiate_v2_IKE_AUTH_request(struct ike_sa *ike,
 			ike->sa.st_sk_pr_no_ppk = symkey_addref(ike->sa.logger, "sk_pr_no_ppk",
 								ike->sa.st_skey_pr_nss);
 
-			ppk_recalculate(ppk, ike->sa.st_oakley.ta_prf,
+			ppk_recalculate(ppk->key, ike->sa.st_oakley.ta_prf,
 					&ike->sa.st_skey_d_nss,
 					&ike->sa.st_skey_pi_nss,
 					&ike->sa.st_skey_pr_nss,
@@ -151,7 +160,7 @@ static stf_status initiate_v2_IKE_AUTH_request(struct ike_sa *ike,
 				 * Identity, so we pretend we didn't
 				 * see USE_PPK.
 				 */
-				ike->sa.st_v2_ike_ppk_enabled = false;
+				ike->sa.st_v2_ike_ppk = PPK_DISABLED;
 			}
 		}
 	}
@@ -190,35 +199,29 @@ stf_status initiate_v2_IKE_AUTH_request_signature_continue(struct ike_sa *ike,
 
 	/* actual data */
 
-	/* decide whether to send CERT payload */
-
-	bool send_cert = ikev2_send_cert_decision(ike);
-	bool send_idr = ((pc->remote->host.id.kind != ID_NULL && pc->remote->host.id.name.len != 0) ||
-				pc->remote->host.id.kind == ID_NULL); /* me tarzan, you jane */
-
-	if (impair.send_no_idr) {
-		llog_sa(RC_LOG, ike, "IMPAIR: omitting IDr payload");
-		send_idr = false;
-	}
-
-	dbg("IDr payload will %sbe sent", send_idr ? "" : "NOT ");
-
-	/* send out the IDi payload */
+	/* send out the IDi payload (always) */
 
 	{
 		struct pbs_out i_id_pbs;
-		if (!out_struct(&ike->sa.st_v2_id_payload.header,
-				&ikev2_id_i_desc,
-				request.pbs,
-				&i_id_pbs) ||
-		    !out_hunk(ike->sa.st_v2_id_payload.data, &i_id_pbs, "my identity"))
+		if (!pbs_out_struct(request.pbs, &ikev2_id_i_desc,
+				    &ike->sa.st_v2_id_payload.header,
+				    sizeof(ike->sa.st_v2_id_payload.header),
+				    &i_id_pbs)) {
 			return STF_INTERNAL_ERROR;
-		close_output_pbs(&i_id_pbs);
+		}
+		if (!pbs_out_hunk(&i_id_pbs, ike->sa.st_v2_id_payload.data, "my identity")) {
+			return STF_INTERNAL_ERROR;
+		}
+		if (!close_pbs_out(&i_id_pbs)) {
+			return STF_INTERNAL_ERROR;
+		}
 	}
 
 	/* send [CERT,] payload RFC 4306 3.6, 1.2) */
 
-	if (send_cert) {
+	if (ike->sa.st_v2_resume_session != NULL) {
+		ldbg(ike->sa.logger, "resuming, never sending CERT payload");
+	} else if (ikev2_send_cert_decision(ike)) {
 		stf_status certstat = emit_v2CERT(ike->sa.st_connection, request.pbs);
 		if (certstat != STF_OK)
 			return certstat;
@@ -226,17 +229,25 @@ stf_status initiate_v2_IKE_AUTH_request_signature_continue(struct ike_sa *ike,
 
 	/* send CERTREQ */
 
-	if (need_v2CERTREQ_in_IKE_AUTH_request(ike)) {
-		if (DBGP(DBG_BASE)) {
-			dn_buf buf;
-			DBG_log("Sending [CERTREQ] of %s",
-				str_dn(ASN1(ike->sa.st_connection->remote->host.config->ca), &buf));
-		}
+	if (ike->sa.st_v2_resume_session != NULL) {
+		ldbg(ike->sa.logger, "resuming, never sending CERTREQ payload");
+	} else if (need_v2CERTREQ_in_IKE_AUTH_request(ike)) {
+		dn_buf buf;
+		ldbg(ike->sa.logger, "sending [CERTREQ] of %s",
+		     str_dn(ASN1(ike->sa.st_connection->remote->host.config->ca), &buf));
 		emit_v2CERTREQ(ike, request.pbs);
 	}
 
 	/* you Tarzan, me Jane support */
+
+	/* decide whether to send CERT payload */
+
+	bool send_idr = ((pc->remote->host.id.kind != ID_NULL &&
+			  pc->remote->host.id.name.len != 0) ||
+			 pc->remote->host.id.kind == ID_NULL); /* me tarzan, you jane */
+
 	if (send_idr) {
+		ldbg(ike->sa.logger, "sending IDr");
 		switch (pc->remote->host.id.kind) {
 		case ID_DER_ASN1_DN:
 		case ID_FQDN:
@@ -249,19 +260,24 @@ stf_status initiate_v2_IKE_AUTH_request_signature_continue(struct ike_sa *ike,
 				build_v2_id_payload(&pc->remote->host, &id_b,
 						    "their IDr", ike->sa.logger);
 			struct pbs_out r_id_pbs;
-			if (!out_struct(&r_id, &ikev2_id_r_desc, request.pbs,
-				&r_id_pbs) ||
-			    !out_hunk(id_b, &r_id_pbs, "their IDr"))
+			if (!pbs_out_struct(request.pbs, &ikev2_id_r_desc,
+					    &r_id, sizeof(r_id),
+					    &r_id_pbs)) {
 				return STF_INTERNAL_ERROR;
-
-			close_output_pbs(&r_id_pbs);
+			}
+			if (!pbs_out_hunk(&r_id_pbs, id_b, "their IDr")) {
+				return STF_INTERNAL_ERROR;
+			}
+			if (!close_pbs_out(&r_id_pbs)) {
+				return STF_INTERNAL_ERROR;
+			}
 			break;
 		}
 		default:
 		{
 			esb_buf b;
-			dbg("Not sending IDr payload for remote ID type %s",
-			    str_enum(&ike_id_type_names, pc->remote->host.id.kind, &b));
+			ldbg(ike->sa.logger, "not sending IDr payload for remote ID type %s",
+			     str_enum(&ike_id_type_names, pc->remote->host.id.kind, &b));
 			break;
 		}
 		}
@@ -288,6 +304,14 @@ stf_status initiate_v2_IKE_AUTH_request_signature_continue(struct ike_sa *ike,
 		}
 	}
 
+	/* Notification payload for ticket request */
+	if (ike->sa.st_connection->config->session_resumption) {
+		llog(RC_LOG, ike->sa.logger, "asking for session resume ticket");
+		if (!emit_v2N(v2N_TICKET_REQUEST, request.pbs)) {
+			return STF_INTERNAL_ERROR;
+		}
+	}
+
 	/*
 	 * Now that the AUTH payload is done(?), create and emit the
 	 * child using the first pending connection (which could be
@@ -355,11 +379,11 @@ stf_status initiate_v2_IKE_AUTH_request_signature_continue(struct ike_sa *ike,
 	 * If we and responder are willing to use a PPK, we need to
 	 * generate NO_PPK_AUTH as well as PPK-based AUTH payload
 	 */
-	if (ike->sa.st_v2_ike_ppk_enabled) {
-		chunk_t *ppk_id;
-		get_connection_ppk_initiator(ike->sa.st_connection, &ppk_id);
+	if (ike->sa.st_v2_ike_ppk == PPK_IKE_AUTH) {
+		const struct secret_ppk_stuff *ppk =
+			get_connection_ppk_and_ppk_id(ike->sa.st_connection);
 		struct ppk_id_payload ppk_id_p = { .type = 0, };
-		create_ppk_id_payload(ppk_id, &ppk_id_p);
+		create_ppk_id_payload(&ppk->id, &ppk_id_p);
 		if (DBGP(DBG_BASE)) {
 			DBG_log("ppk type: %d", (int) ppk_id_p.type);
 			DBG_dump_hunk("ppk_id from payload:", ppk_id_p.ppk_id);
@@ -439,12 +463,11 @@ stf_status initiate_v2_IKE_AUTH_request_signature_continue(struct ike_sa *ike,
 
 static pam_auth_callback_fn ikev2_pam_continue;	/* type assertion */
 
-static stf_status ikev2_pam_continue(struct state *ike_st,
+static stf_status ikev2_pam_continue(struct ike_sa *ike,
 				     struct msg_digest *md,
 				     const char *name UNUSED,
 				     bool success)
 {
-	struct ike_sa *ike = pexpect_ike_sa(ike_st);
 	pexpect(ike->sa.st_sa_role == SA_RESPONDER);
 	pexpect(v2_msg_role(md) == MESSAGE_REQUEST); /* i.e., MD!=NULL */
 	pexpect(ike->sa.st_state == &state_v2_IKE_SA_INIT_R);
@@ -453,7 +476,7 @@ static stf_status ikev2_pam_continue(struct state *ike_st,
 
 	if (!success) {
 		record_v2N_response(ike->sa.logger, ike, md,
-				    v2N_AUTHENTICATION_FAILED, NULL/*no-data*/,
+				    v2N_AUTHENTICATION_FAILED, empty_shunk/*no-data*/,
 				    ENCRYPTED_PAYLOAD);
 		pstat_sa_failed(&ike->sa, REASON_AUTH_FAILED);
 		return STF_FATAL; /* STF_ZOMBIFY */
@@ -484,14 +507,25 @@ stf_status process_v2_IKE_AUTH_request(struct ike_sa *ike,
 	 */
 	llog_msg_digest(RC_LOG, ike->sa.logger, "processing decrypted", md);
 
-	struct payload_digest *cert_payloads = md->chain[ISAKMP_NEXT_v2CERT];
-	if (cert_payloads != NULL) {
-		submit_v2_cert_decode(ike, md, cert_payloads,
-				      process_v2_IKE_AUTH_request_post_cert_decode, HERE);
-		return STF_SUSPEND;
+	if (ike->sa.st_v2_resume_session != NULL) {
+		ldbg(ike->sa.logger, "resuming, skipping cert decode");
+		return process_v2_IKE_AUTH_request_skip_cert_decode(ike, md);
 	}
 
-	dbg("no certs to decode");
+	struct payload_digest *cert_payloads = md->chain[ISAKMP_NEXT_v2CERT];
+	if (cert_payloads == NULL) {
+		ldbg(ike->sa.logger, "skipping cert decode; there are none");
+		return process_v2_IKE_AUTH_request_skip_cert_decode(ike, md);
+	}
+
+	submit_v2_cert_decode(ike, md, cert_payloads,
+			      process_v2_IKE_AUTH_request_post_cert_decode, HERE);
+	return STF_SUSPEND;
+}
+
+stf_status process_v2_IKE_AUTH_request_skip_cert_decode(struct ike_sa *ike,
+							struct msg_digest *md)
+{
 	ike->sa.st_remote_certs.processed = true;
 	ike->sa.st_remote_certs.harmless = true;
 	return process_v2_IKE_AUTH_request_post_cert_decode(&ike->sa, md);
@@ -516,7 +550,132 @@ stf_status process_v2_IKE_AUTH_request_standard_payloads(struct ike_sa *ike, str
 	 * The RFCs do discuss the idea of using this to refine the
 	 * connection.  Since the ID is available, why bother.
 	 */
-	process_v2CERTREQ_payload(ike, md);
+	if (ike->sa.st_v2_resume_session != NULL) {
+		ldbg(ike->sa.logger, "resuming, skipping any CERTREQ payload");
+	} else {
+		process_v2CERTREQ_payload(ike, md);
+	}
+
+	/*
+	 * Convert the proposed connections into something this
+	 * responder might accept.
+	 *
+	 * + DIGITAL_SIGNATURE code seems a bit dodgy, should this be
+	 * looking inside the auth proposal to see what is actually
+	 * required?
+	 *
+	 * + the legacy ECDSA_SHA2* methods also seem to be a bit
+	 * dodgy, shouldn't they also specify the SHA algorithm so
+	 * that can be matched?
+	 */
+
+	if (PBAD(ike->sa.logger, md->chain[ISAKMP_NEXT_v2AUTH] == NULL)) {
+		return STF_FATAL;
+	}
+
+	lset_t proposed_authbys;
+	if (ike->sa.st_v2_resume_session) {
+		enum keyword_auth auth = resume_session_auth(ike->sa.st_v2_resume_session);
+		name_buf rn, an;
+		ldbg(ike->sa.logger, "resuming, ignoring v2AUTH method %s, using %s",
+		     str_enum_short(&ikev2_auth_method_names,
+				    md->chain[ISAKMP_NEXT_v2AUTH]->payload.v2auth.isaa_auth_method, &an),
+		     str_enum_short(&keyword_auth_names, auth, &rn));
+		proposed_authbys = LELEM(auth);
+	} else {
+		proposed_authbys = proposed_v2AUTH(ike, md);
+	}
+
+	if (proposed_authbys == LEMPTY) {
+		/* already logged */
+		pstat_sa_failed(&ike->sa, REASON_AUTH_FAILED);
+		record_v2N_response(ike->sa.logger, ike, md,
+				    v2N_AUTHENTICATION_FAILED, empty_shunk/*no-data*/,
+				    ENCRYPTED_PAYLOAD);
+		return STF_FATAL;
+	}
+
+	/*
+	 * Decode the peer IDs ready for refining the connection.
+	 *
+	 * Conceivably, in a multi-homed scenario, it could also
+	 * switch based on the contents of the CERTREQ.
+	 */
+
+	struct id initiator_id, responder_id;
+	diag_t d = ikev2_responder_decode_v2ID_payloads(ike, md, &initiator_id, &responder_id);
+	if (d != NULL) {
+		llog(RC_LOG, ike->sa.logger, "%s", str_diag(d));
+		pfree_diag(&d);
+		pstat_sa_failed(&ike->sa, REASON_AUTH_FAILED);
+		record_v2N_response(ike->sa.logger, ike, md,
+				    v2N_AUTHENTICATION_FAILED, empty_shunk/*no-data*/,
+				    ENCRYPTED_PAYLOAD);
+		return STF_FATAL;
+	}
+
+	/*
+	 * IKE_SESSION_RESUME 4.3.3.  IKE_AUTH Exchange:
+	 *
+	 *   The IDi value sent in the IKE_AUTH exchange MUST be
+	 *   identical to the value included in the ticket.  A CERT
+	 *   payload MUST NOT be included in this exchange, and
+	 *   therefore a new IDr value cannot be negotiated (since it
+	 *   would not be authenticated).  As a result, the IDr value
+	 *   sent (by the gateway, and optionally by the client) in
+	 *   this exchange MUST also be identical to the value
+	 *   included in the ticket.
+	 */
+
+	if (ike->sa.st_v2_resume_session != NULL) {
+		if (!verify_resume_session_id(ike->sa.st_v2_resume_session,
+					      &initiator_id, &responder_id,
+					      ike->sa.logger)) {
+			/* already logged */
+			pstat_sa_failed(&ike->sa, REASON_AUTH_FAILED);
+			record_v2N_response(ike->sa.logger, ike, md,
+					    v2N_AUTHENTICATION_FAILED, empty_shunk/*no-data*/,
+					    ENCRYPTED_PAYLOAD);
+			return STF_FATAL;
+		}
+	}
+
+	/*
+	 * IS_MOST_REFINED is subtle.
+	 *
+	 * IS_MOST_REFINED: the state's (possibly updated) connection
+	 * is known to be the best there is (best can include the
+	 * current connection).
+	 *
+	 * !IS_MOST_REFINED: is less specific.  For IKEv1, the search
+	 * didn't find a best; for IKEv2 it can additionally mean that
+	 * there was no search because the initiator proposed
+	 * AUTH_NULL.  AUTH_NULL never switches as it is assumed
+	 * that the perfect connection was chosen during IKE_SA_INIT.
+	 *
+	 * Either way, !IS_MOST_REFINED leads to a same_id() and other
+	 * checks.
+	 *
+	 * This may change ike->sa.st_connection!
+	 *
+	 * We might be surprised!  Which is why C is only captured
+	 * _after_ this operation.
+	 */
+       if (!LHAS(proposed_authbys, AUTH_NULL)) {
+	       refine_host_connection_of_state_on_responder(ike, proposed_authbys,
+							    &initiator_id,
+							    &responder_id);
+       }
+
+       d = update_peer_id(ike, &initiator_id, &responder_id);
+       if (d != NULL) {
+		pfree_diag(&d);
+		pstat_sa_failed(&ike->sa, REASON_AUTH_FAILED);
+		record_v2N_response(ike->sa.logger, ike, md,
+				    v2N_AUTHENTICATION_FAILED, empty_shunk/*no-data*/,
+				    ENCRYPTED_PAYLOAD);
+		return STF_FATAL;
+       }
 
 	/*
 	 * This both decodes the initiator's ID and, when necessary,
@@ -526,17 +685,6 @@ stf_status process_v2_IKE_AUTH_request_standard_payloads(struct ike_sa *ike, str
 	 * switch based on the contents of the CERTREQ.
 	 */
 
-	diag_t d = ikev2_responder_decode_initiator_id(ike, md);
-	if (d != NULL) {
-		llog(RC_LOG, ike->sa.logger, "%s", str_diag(d));
-		pfree_diag(&d);
-		pstat_sa_failed(&ike->sa, REASON_AUTH_FAILED);
-		record_v2N_response(ike->sa.logger, ike, md,
-				    v2N_AUTHENTICATION_FAILED, NULL/*no-data*/,
-				    ENCRYPTED_PAYLOAD);
-		return STF_FATAL;
-	}
-
 	const struct connection *c = ike->sa.st_connection;
 	bool found_ppk = false;
 
@@ -545,45 +693,51 @@ stf_status process_v2_IKE_AUTH_request_standard_payloads(struct ike_sa *ike, str
 	 * either related to the IKE SA, or the Child SA. Here we only
 	 * process the ones related to the IKE SA.
 	 */
-	if (md->pd[PD_v2N_PPK_IDENTITY] != NULL) {
-		dbg("received PPK_IDENTITY");
-		struct ppk_id_payload payl;
-		if (!extract_v2N_ppk_identity(&md->pd[PD_v2N_PPK_IDENTITY]->pbs, &payl, ike)) {
-			dbg("failed to extract PPK_ID from PPK_IDENTITY payload. Abort!");
-			return STF_FATAL;
-		}
 
-		const shunk_t ppk = get_connection_ppk_responder(ike->sa.st_connection,
-								 &payl.ppk_id);
-		free_chunk_content(&payl.ppk_id);
-		if (ppk.ptr != NULL) {
-			found_ppk = true;
-		}
+	/* Only RFC 8784 PPK mechanism here: */
+	if (ike->sa.st_v2_ike_ppk == PPK_IKE_AUTH) {
+		if (md->pd[PD_v2N_PPK_IDENTITY] != NULL) {
+			dbg("received PPK_IDENTITY");
+			struct ppk_id_payload payl;
+			if (!extract_v2N_ppk_identity(&md->pd[PD_v2N_PPK_IDENTITY]->pbs, &payl, ike)) {
+				dbg("failed to extract PPK_ID from PPK_IDENTITY payload. Abort!");
+				return STF_FATAL;
+			}
 
-		if (found_ppk && c->config->ppk.allow) {
-			ppk_recalculate(ppk, ike->sa.st_oakley.ta_prf,
-					&ike->sa.st_skey_d_nss,
-					&ike->sa.st_skey_pi_nss,
-					&ike->sa.st_skey_pr_nss,
-					ike->sa.logger);
-			ike->sa.st_ppk_used = true;
-			llog_sa(RC_LOG, ike,
-				"PPK AUTH calculated as responder");
-		} else {
-			llog_sa(RC_LOG, ike,
-				"ignored received PPK_IDENTITY - connection does not require PPK or PPKID not found");
+			const struct secret_ppk_stuff *ppk =
+				get_connection_ppk(ike->sa.st_connection,
+						   /*ppk_id*/&payl.ppk_id,
+						   /*index*/0);
+			free_chunk_content(&payl.ppk_id);
+			if (ppk != NULL) {
+				found_ppk = true;
+			}
+
+			if (found_ppk && c->config->ppk.allow) {
+				ppk_recalculate(ppk->key, ike->sa.st_oakley.ta_prf,
+						&ike->sa.st_skey_d_nss,
+						&ike->sa.st_skey_pi_nss,
+						&ike->sa.st_skey_pr_nss,
+						ike->sa.logger);
+				ike->sa.st_ppk_ike_auth_used = true;
+				llog_sa(RC_LOG, ike,
+					"PPK AUTH calculated as responder");
+			} else {
+				llog_sa(RC_LOG, ike,
+					"ignored received PPK_IDENTITY - connection does not require PPK or PPKID not found");
+			}
 		}
-	}
-	if (md->pd[PD_v2N_NO_PPK_AUTH] != NULL) {
-		dbg("received NO_PPK_AUTH");
-		if (c->config->ppk.insist) {
-			dbg("Ignored NO_PPK_AUTH data - connection insists on PPK");
-		} else {
-			struct pbs_in pbs = md->pd[PD_v2N_NO_PPK_AUTH]->pbs;
-			/* zero length doesn't matter? */
-			shunk_t no_ppk_auth = pbs_in_left(&pbs);
-			replace_chunk(&ike->sa.st_no_ppk_auth,
-				      no_ppk_auth, "NO_PPK_AUTH extract");
+		if (md->pd[PD_v2N_NO_PPK_AUTH] != NULL) {
+			dbg("received NO_PPK_AUTH");
+			if (c->config->ppk.insist) {
+				dbg("Ignored NO_PPK_AUTH data - connection insists on PPK");
+			} else {
+				struct pbs_in pbs = md->pd[PD_v2N_NO_PPK_AUTH]->pbs;
+				/* zero length doesn't matter? */
+				shunk_t no_ppk_auth = pbs_in_left(&pbs);
+				replace_chunk(&ike->sa.st_no_ppk_auth,
+					no_ppk_auth, "NO_PPK_AUTH extract");
+			}
 		}
 	}
 
@@ -604,19 +758,24 @@ stf_status process_v2_IKE_AUTH_request_standard_payloads(struct ike_sa *ike, str
 	ike->sa.st_ike_seen_v2n_initial_contact = md->pd[PD_v2N_INITIAL_CONTACT] != NULL;
 
 	/*
+	 * Only RFC 8784 PPK mechanism here:
+	 *
 	 * If we found proper PPK ID and policy allows PPK, use that.
 	 * Otherwise use NO_PPK_AUTH
 	 */
-	if (found_ppk && c->config->ppk.allow) {
-		free_chunk_content(&ike->sa.st_no_ppk_auth);
-	}
+	if (ike->sa.st_v2_ike_ppk == PPK_IKE_AUTH) {
+		if (found_ppk && c->config->ppk.allow) {
+			free_chunk_content(&ike->sa.st_no_ppk_auth);
+		}
 
-	if (!found_ppk && c->config->ppk.insist) {
-		llog_sa(RC_LOG, ike, "Requested PPK_ID not found and connection requires a valid PPK");
-		record_v2N_response(ike->sa.logger, ike, md,
-				    v2N_AUTHENTICATION_FAILED, NULL/*no data*/,
-				    ENCRYPTED_PAYLOAD);
-		return STF_FATAL;
+		if (!found_ppk && c->config->ppk.insist) {
+			llog_sa(RC_LOG, ike,
+				"Requested PPK_ID not found and connection requires a valid PPK");
+			record_v2N_response(ike->sa.logger, ike, md,
+					    v2N_AUTHENTICATION_FAILED, empty_shunk/*no data*/,
+					    ENCRYPTED_PAYLOAD);
+			return STF_FATAL;
+		}
 	}
 
 	return STF_OK;
@@ -655,7 +814,7 @@ stf_status process_v2_IKE_AUTH_request_ipseckey_continue(struct ike_sa *ike,
 	if (err) {
 		/* already logged?! */
 		record_v2N_response(ike->sa.logger, ike, md,
-				    v2N_AUTHENTICATION_FAILED, NULL/*no-data*/,
+				    v2N_AUTHENTICATION_FAILED, empty_shunk/*no-data*/,
 				    ENCRYPTED_PAYLOAD);
 		return STF_FATAL;
 	}
@@ -669,13 +828,15 @@ stf_status process_v2_IKE_AUTH_request_id_tail(struct ike_sa *ike, struct msg_di
 
 	/* process AUTH payload */
 
-	enum keyword_auth remote_auth = ike->sa.st_connection->remote->host.config->auth;
-	struct authby remote_authby = ike->sa.st_connection->remote->host.config->authby;
-	passert(remote_auth != AUTH_NEVER && remote_auth != AUTH_UNSET);
-	bool remote_can_authby_null = remote_authby.null;
-	bool remote_can_authby_digsig = authby_has_digsig(remote_authby);
+	struct connection *c = ike->sa.st_connection;
+	enum keyword_auth initiator_auth = (ike->sa.st_v2_resume_session != NULL ? AUTH_PSK :
+					    c->remote->host.config->auth);
+	struct authby initiator_authby = c->remote->host.config->authby;
+	passert(initiator_auth != AUTH_NEVER && initiator_auth != AUTH_UNSET);
+	bool remote_can_authby_null = initiator_authby.null;
+	bool remote_can_authby_digsig = authby_has_digsig(initiator_authby);
 
-	if (!ike->sa.st_ppk_used && ike->sa.st_no_ppk_auth.ptr != NULL) {
+	if (!ike->sa.st_ppk_ike_auth_used && ike->sa.st_no_ppk_auth.ptr != NULL) {
 		/*
 		 * we didn't recalculate keys with PPK, but we found NO_PPK_AUTH
 		 * (meaning that initiator did use PPK) so we try to verify NO_PPK_AUTH.
@@ -692,13 +853,14 @@ stf_status process_v2_IKE_AUTH_request_id_tail(struct ike_sa *ike, struct msg_di
 			pbs_in_from_shunk(HUNK_AS_SHUNK(ike->sa.st_no_ppk_auth),
 					  "struct pbs_in for verifying NO_PPK_AUTH");
 		diag_t d = verify_v2AUTH_and_log(md->chain[ISAKMP_NEXT_v2AUTH]->payload.v2auth.isaa_auth_method,
-						 ike, &idhash_in, &pbs_no_ppk_auth, remote_auth);
+						 ike, &idhash_in, &pbs_no_ppk_auth,
+						 initiator_auth);
 		if (d != NULL) {
 			llog(RC_LOG, ike->sa.logger, "%s", str_diag(d));
 			pfree_diag(&d);
 			dbg("no PPK auth failed");
 			record_v2N_response(ike->sa.logger, ike, md,
-					    v2N_AUTHENTICATION_FAILED, NULL/*no data*/,
+					    v2N_AUTHENTICATION_FAILED, empty_shunk/*no data*/,
 					    ENCRYPTED_PAYLOAD);
 			pstat_sa_failed(&ike->sa, REASON_AUTH_FAILED);
 			return STF_FATAL;
@@ -724,7 +886,7 @@ stf_status process_v2_IKE_AUTH_request_id_tail(struct ike_sa *ike, struct msg_di
 			pfree_diag(&d);
 			dbg("NULL_auth from Notify Payload failed");
 			record_v2N_response(ike->sa.logger, ike, md,
-					    v2N_AUTHENTICATION_FAILED, NULL/*no data*/,
+					    v2N_AUTHENTICATION_FAILED, empty_shunk/*no data*/,
 					    ENCRYPTED_PAYLOAD);
 			pstat_sa_failed(&ike->sa, REASON_AUTH_FAILED);
 			return STF_FATAL;
@@ -733,14 +895,15 @@ stf_status process_v2_IKE_AUTH_request_id_tail(struct ike_sa *ike, struct msg_di
 	} else {
 		dbg("responder verifying AUTH payload");
 		diag_t d = verify_v2AUTH_and_log(md->chain[ISAKMP_NEXT_v2AUTH]->payload.v2auth.isaa_auth_method,
-						 ike, &idhash_in, &md->chain[ISAKMP_NEXT_v2AUTH]->pbs,
-						 remote_auth);
+						 ike, &idhash_in,
+						 &md->chain[ISAKMP_NEXT_v2AUTH]->pbs,
+						 initiator_auth);
 		if (d != NULL) {
 			llog(RC_LOG, ike->sa.logger, "%s", str_diag(d));
 			pfree_diag(&d);
 			dbg("I2 Auth Payload failed");
 			record_v2N_response(ike->sa.logger, ike, md,
-					    v2N_AUTHENTICATION_FAILED, NULL/*no data*/,
+					    v2N_AUTHENTICATION_FAILED, empty_shunk/*no data*/,
 					    ENCRYPTED_PAYLOAD);
 			pstat_sa_failed(&ike->sa, REASON_AUTH_FAILED);
 			return STF_FATAL;
@@ -762,7 +925,7 @@ stf_status process_v2_IKE_AUTH_request_id_tail(struct ike_sa *ike, struct msg_di
 		llog_sa(RC_LOG, ike,
 			"IKEv2: [XAUTH]PAM method requested to authorize '%s'",
 			thatid);
-		if (!pam_auth_fork_request(&ike->sa, md, thatid, "password",
+		if (!pam_auth_fork_request(ike, md, thatid, "password",
 					   "IKEv2", ikev2_pam_continue)) {
 			return STF_FATAL;
 		}
@@ -785,7 +948,7 @@ static stf_status process_v2_IKE_AUTH_request_tail(struct state *ike_st,
 		 * AUTH payload succeed
 		 */
 		record_v2N_response(ike->sa.logger, ike, md,
-				    v2N_AUTHENTICATION_FAILED, NULL/*no data*/,
+				    v2N_AUTHENTICATION_FAILED, empty_shunk/*no data*/,
 				    ENCRYPTED_PAYLOAD);
 		return STF_FATAL;
 	}
@@ -895,7 +1058,6 @@ static stf_status process_v2_IKE_AUTH_request_auth_signature_continue(struct ike
 	}
 
 	/* decide to send CERT payload before we generate IDr */
-	bool send_cert = ikev2_send_cert_decision(ike);
 
 	/* send any NOTIFY payloads */
 	if (ike->sa.st_v2_mobike.enabled) {
@@ -903,7 +1065,7 @@ static stf_status process_v2_IKE_AUTH_request_auth_signature_continue(struct ike
 			return STF_INTERNAL_ERROR;
 	}
 
-	if (ike->sa.st_ppk_used) {
+	if (ike->sa.st_ppk_ike_auth_used) {
 		if (!emit_v2N(v2N_PPK_IDENTITY, response.pbs))
 			return STF_INTERNAL_ERROR;
 	}
@@ -921,11 +1083,28 @@ static stf_status process_v2_IKE_AUTH_request_auth_signature_continue(struct ike
 	 * message with a DELETE payload.
 	 */
 	if (send_redirect) {
-		if (!emit_redirect_notification(shunk1(c->config->redirect.to), response.pbs))
+		if (!emit_v2N_REDIRECT(c->config->redirect.to, response.pbs)) {
 			return STF_INTERNAL_ERROR;
+		}
 		ike->sa.st_sent_redirect = true;	/* mark that we have sent REDIRECT in IKE_AUTH */
 	}
 
+	/*
+	 * Ticket request is in IKE_AUTH, not IKE_SA_INIT, so no need
+	 * to store it in the state.
+	 */
+	if (md->pd[PD_v2N_TICKET_REQUEST] != NULL) {
+		if (c->config->session_resumption) {
+			if (!emit_v2N_TICKET_LT_OPAQUE(ike, response.pbs)) {
+				return STF_INTERNAL_ERROR;
+			}
+		} else {
+			if (!emit_v2N(v2N_TICKET_NACK, response.pbs)) {
+				return STF_INTERNAL_ERROR;
+			}
+		}
+	}
+
 	/* send out the IDr payload */
 	{
 		struct pbs_out r_id_pbs;
@@ -943,7 +1122,9 @@ static stf_status process_v2_IKE_AUTH_request_auth_signature_continue(struct ike
 	 * upon which our received I2 CERTREQ is ignored,
 	 * but ultimately should go into the CERT decision
 	 */
-	if (send_cert) {
+	if (ike->sa.st_v2_resume_session != NULL) {
+		ldbg(ike->sa.logger, "resuming, never sending CERT payload");
+	} else if (ikev2_send_cert_decision(ike)) {
 		stf_status certstat = emit_v2CERT(ike->sa.st_connection, response.pbs);
 		if (certstat != STF_OK)
 			return certstat;
@@ -1034,59 +1215,59 @@ static stf_status process_v2_IKE_AUTH_response_post_cert_decode(struct state *ik
 	}
 
 	struct connection *c = ike->sa.st_connection;
-	enum keyword_auth that_authby = c->remote->host.config->auth;
+	enum keyword_auth responder_auth = (ike->sa.st_v2_resume_session != NULL ? AUTH_PSK :
+					    c->remote->host.config->auth);
 
-	passert(that_authby != AUTH_NEVER && that_authby != AUTH_UNSET);
+	passert(responder_auth != AUTH_NEVER && responder_auth != AUTH_UNSET);
 
-	if (md->pd[PD_v2N_PPK_IDENTITY] != NULL) {
-		if (!c->config->ppk.allow) {
-			llog_sa(RC_LOG, ike, "received PPK_IDENTITY but connection does not allow PPK");
-			return STF_FATAL;
+	if (ike->sa.st_v2_ike_ppk == PPK_IKE_AUTH) {
+		if (md->pd[PD_v2N_PPK_IDENTITY] != NULL) {
+			if (!c->config->ppk.allow) {
+				llog_sa(RC_LOG, ike, "received PPK_IDENTITY but connection does not allow PPK");
+				return STF_FATAL;
+			}
+		} else {
+			if (c->config->ppk.insist) {
+				llog_sa(RC_LOG, ike,
+					"failed to receive PPK confirmation and connection has ppk=insist");
+				dbg("should be initiating a notify that kills the state");
+				pstat_sa_failed(&ike->sa, REASON_AUTH_FAILED);
+				return STF_FATAL;
+			}
 		}
-	} else {
-		if (c->config->ppk.insist) {
-			llog_sa(RC_LOG, ike,
-				"failed to receive PPK confirmation and connection has ppk=insist");
-			dbg("should be initiating a notify that kills the state");
-			pstat_sa_failed(&ike->sa, REASON_AUTH_FAILED);
-			return STF_FATAL;
+
+		/*
+		* If we sent USE_PPK and we did not receive a PPK_IDENTITY,
+		* it means the responder failed to find our PPK ID, but
+		* allowed the connection to continue without PPK by using our
+		* NO_PPK_AUTH payload. We should revert our key material to
+		* NO_PPK versions.
+		*/
+		if (md->pd[PD_v2N_PPK_IDENTITY] == NULL && c->config->ppk.allow) {
+			/* discard the PPK based calculations */
+
+			llog_sa(RC_LOG, ike, "peer wants to continue without PPK - switching to NO_PPK");
+
+			symkey_delref(ike->sa.logger, "st_skey_d_nss",  &ike->sa.st_skey_d_nss);
+			ike->sa.st_skey_d_nss = symkey_addref(ike->sa.logger, "used sk_d from no ppk", ike->sa.st_sk_d_no_ppk);
+
+			symkey_delref(ike->sa.logger, "st_skey_pi_nss", &ike->sa.st_skey_pi_nss);
+			ike->sa.st_skey_pi_nss = symkey_addref(ike->sa.logger, "used sk_pi from no ppk", ike->sa.st_sk_pi_no_ppk);
+
+			symkey_delref(ike->sa.logger, "st_skey_pr_nss", &ike->sa.st_skey_pr_nss);
+			ike->sa.st_skey_pr_nss = symkey_addref(ike->sa.logger, "used sk_pr from no ppk", ike->sa.st_sk_pr_no_ppk);
 		}
 	}
 
-	/*
-	 * If we sent USE_PPK and we did not receive a PPK_IDENTITY,
-	 * it means the responder failed to find our PPK ID, but
-	 * allowed the connection to continue without PPK by using our
-	 * NO_PPK_AUTH payload. We should revert our key material to
-	 * NO_PPK versions.
-	 */
-	if (ike->sa.st_v2_ike_ppk_enabled &&
-	    md->pd[PD_v2N_PPK_IDENTITY] == NULL &&
-	    c->config->ppk.allow) {
-		/* discard the PPK based calculations */
-
-		llog_sa(RC_LOG, ike, "peer wants to continue without PPK - switching to NO_PPK");
-
-		symkey_delref(ike->sa.logger, "st_skey_d_nss",  &ike->sa.st_skey_d_nss);
-		ike->sa.st_skey_d_nss = symkey_addref(ike->sa.logger, "used sk_d from no ppk",
-						      ike->sa.st_sk_d_no_ppk);
-
-		symkey_delref(ike->sa.logger, "st_skey_pi_nss", &ike->sa.st_skey_pi_nss);
-		ike->sa.st_skey_pi_nss = symkey_addref(ike->sa.logger, "used sk_pi from no ppk",
-						       ike->sa.st_sk_pi_no_ppk);
-
-		symkey_delref(ike->sa.logger, "st_skey_pr_nss", &ike->sa.st_skey_pr_nss);
-		ike->sa.st_skey_pr_nss = symkey_addref(ike->sa.logger, "used sk_pr from no ppk",
-						       ike->sa.st_sk_pr_no_ppk);
-	}
-
 	struct crypt_mac idhash_in = v2_remote_id_hash(ike, "idhash auth R2", md);
 
 	/* process AUTH payload */
 
 	dbg("initiator verifying AUTH payload");
 	d = verify_v2AUTH_and_log(md->chain[ISAKMP_NEXT_v2AUTH]->payload.v2auth.isaa_auth_method,
-				  ike, &idhash_in, &md->chain[ISAKMP_NEXT_v2AUTH]->pbs, that_authby);
+				  ike, &idhash_in,
+				  &md->chain[ISAKMP_NEXT_v2AUTH]->pbs,
+				  responder_auth);
 	if (d != NULL) {
 		llog(RC_LOG, ike->sa.logger, "%s", str_diag(d));
 		pfree_diag(&d);
@@ -1120,6 +1301,20 @@ static stf_status process_v2_IKE_AUTH_response_post_cert_decode(struct state *ik
 		return redirect_status;
 	}
 
+	if (md->pd[PD_v2N_TICKET_NACK] != NULL) {
+		llog(RC_LOG, ike->sa.logger, "received v2N_TICKET_NACK");
+	}
+
+	if (md->pd[PD_v2N_TICKET_ACK] != NULL) {
+		llog(RC_LOG, ike->sa.logger, "received v2N_TICKET_ACK");
+	}
+
+	if (md->pd[PD_v2N_TICKET_LT_OPAQUE] != NULL) {
+		if (!process_v2N_TICKET_LT_OPAQUE(ike, md->pd[PD_v2N_TICKET_LT_OPAQUE])) {
+			return STF_FATAL;
+		}
+	}
+
 	ike->sa.st_v2_mobike.enabled =
 		accept_v2_notification(v2N_MOBIKE_SUPPORTED, ike->sa.logger, md, c->config->mobike);
 
@@ -1315,7 +1510,6 @@ static stf_status process_v2_IKE_AUTH_failure_response(struct ike_sa *ike,
 
 static const struct v2_transition v2_IKE_AUTH_initiate_transition = {
 	.story      = "initiating IKE_AUTH",
-	.from = { &state_v2_IKE_SA_INIT_IR, &state_v2_IKE_INTERMEDIATE_IR, },
 	.to = &state_v2_IKE_AUTH_I,
 	.exchange   = ISAKMP_v2_IKE_AUTH,
 	.processor  = initiate_v2_IKE_AUTH_request,
@@ -1326,20 +1520,6 @@ static const struct v2_transition v2_IKE_AUTH_initiate_transition = {
 static const struct v2_transition v2_IKE_AUTH_responder_transition[] = {
 
 	{ .story      = "Responder: process IKE_AUTH request",
-	  .from = { &state_v2_IKE_SA_INIT_R, },
-	  .to = &state_v2_ESTABLISHED_IKE_SA,
-	  .flags = { .release_whack = true, },
-	  .exchange   = ISAKMP_v2_IKE_AUTH,
-	  .recv_role  = MESSAGE_REQUEST,
-	  .message_payloads.required = v2P(SK),
-	  .encrypted_payloads.required = v2P(IDi) | v2P(AUTH),
-	  .encrypted_payloads.optional = v2P(CERT) | v2P(CERTREQ) | v2P(IDr) | v2P(CP) | v2P(SA) | v2P(TSi) | v2P(TSr),
-	  .processor  = process_v2_IKE_AUTH_request,
-	  .llog_success = ldbg_v2_success,
-	  .timeout_event = EVENT_v2_REPLACE, },
-
-	{ .story      = "processing IKE_AUTH request",
-	  .from = { &state_v2_IKE_INTERMEDIATE_R, },
 	  .to = &state_v2_ESTABLISHED_IKE_SA,
 	  .flags = { .release_whack = true, },
 	  .exchange   = ISAKMP_v2_IKE_AUTH,
@@ -1366,7 +1546,6 @@ static const struct v2_transition v2_IKE_AUTH_response_transition[] = {
 	 */
 
 	{ .story      = "Initiator: process IKE_AUTH response",
-	  .from = { &state_v2_IKE_AUTH_I, },
 	  .to = &state_v2_ESTABLISHED_IKE_SA,
 	  .flags = { .release_whack = true, },
 	  .exchange   = ISAKMP_v2_IKE_AUTH,
@@ -1380,7 +1559,6 @@ static const struct v2_transition v2_IKE_AUTH_response_transition[] = {
 	},
 
 	{ .story      = "Initiator: processing IKE_AUTH failure response",
-	  .from = { &state_v2_IKE_AUTH_I, },
 	  .to = &state_v2_IKE_AUTH_I,
 	  .exchange   = ISAKMP_v2_IKE_AUTH,
 	  .recv_role  = MESSAGE_RESPONSE,
@@ -1393,4 +1571,7 @@ static const struct v2_transition v2_IKE_AUTH_response_transition[] = {
 };
 
 V2_EXCHANGE(IKE_AUTH, "authenticate IKE SA", "",
-	    CAT_OPEN_IKE_SA, CAT_ESTABLISHED_IKE_SA, /*secured*/true);
+	    CAT_OPEN_IKE_SA, CAT_ESTABLISHED_IKE_SA, /*secured*/true,
+	    &state_v2_IKE_SA_INIT_IR,
+	    &state_v2_IKE_INTERMEDIATE_IR,
+	    &state_v2_IKE_SESSION_RESUME_IR);
