@@ -19,10 +19,12 @@
 
 #include "state.h"
 #include "demux.h"
+#include "keys.h"
 #include "crypt_dh.h"
 #include "ikev2.h"
 #include "ikev2_send.h"
 #include "ikev2_message.h"
+#include "ikev2_ppk.h"
 #include "ikev2_ike_intermediate.h"
 #include "crypt_symkey.h"
 #include "log.h"
@@ -34,6 +36,10 @@
 #include "crypt_prf.h"
 #include "ikev2_states.h"
 #include "ikev2_eap.h"
+#include "secrets.h"
+#include "crypt_cipher.h"
+#include "ikev2_prf.h"
+#include "ikev2_notification.h"
 
 static dh_shared_secret_cb process_v2_IKE_INTERMEDIATE_response_continue;	/* type assertion */
 static ikev2_state_transition_fn process_v2_IKE_INTERMEDIATE_request;	/* type assertion */
@@ -190,6 +196,45 @@ static void compute_intermediate_mac(struct ike_sa *ike,
 	*int_auth_ir = clone_hunk(mac, "IntAuth");
 }
 
+/*
+ * Calculate PPK Confirmation = prf(PPK, Ni | Nr | SPIi | SPIr). It is used in
+ * draft-ietf-ipsecme-ikev2-qr-alt-04 and only in IKE_INTERMEDIATE exchange.
+ * It is called both by initiator and by the responder.
+ */
+static chunk_t calc_ppk_confirmation(const struct prf_desc *prf_desc,
+				     const shunk_t *ppk,
+				     const chunk_t Ni, const chunk_t Nr,
+				     const ike_spis_t *ike_spis,
+				     struct logger *logger)
+{
+	dbg("calculating PPK Confirmation for PPK_IDENTITY_KEY Notify");
+	PK11SymKey *ppk_key = symkey_from_hunk("PPK Keying material", *ppk, logger);
+
+	/* prf(PPK, ... */
+	struct crypt_prf *prf = crypt_prf_init_symkey("prf(PPK,)",
+						      prf_desc,
+						      "PPK", ppk_key,
+						      logger);
+
+	crypt_prf_update_hunk(prf, "Ni", Ni);
+	crypt_prf_update_hunk(prf, "Nr", Nr);
+	crypt_prf_update_hunk(prf, "SPIi", THING_AS_SHUNK(ike_spis->initiator));
+	crypt_prf_update_hunk(prf, "SPIr", THING_AS_SHUNK(ike_spis->responder));
+	struct crypt_mac ppk_confirmation = crypt_prf_final_mac(&prf, NULL/*no-truncation*/);
+
+	if (DBGP(DBG_CRYPT)) {
+		DBG_dump("prf(PPK, Ni | Nr | SPIi | SPIr) (full PPK confirmation)", ppk_confirmation.ptr,
+			prf_desc->prf_output_size);
+	}
+
+	symkey_delref(logger, "PPK Keying material", &ppk_key);
+
+	/* NOTE: caller should free this */
+	chunk_t ret = clone_bytes_as_chunk(ppk_confirmation.ptr, PPK_CONFIRMATION_LEN, "PPK Confirmation data");
+
+	return ret;
+}
+
 static stf_status initiate_v2_IKE_INTERMEDIATE_request(struct ike_sa *ike,
 						       struct child_sa *null_child,
 						       struct msg_digest *null_md)
@@ -211,7 +256,89 @@ static stf_status initiate_v2_IKE_INTERMEDIATE_request(struct ike_sa *ike,
 		return STF_INTERNAL_ERROR;
 	}
 
-	/* message is empty! */
+	struct connection *const c = ike->sa.st_connection;
+	struct shunks *ppk_ids_shunks = c->config->ppk_ids_shunks;
+	chunk_t ppk_id;
+	bool found_one = false;
+
+	if (ike->sa.st_v2_ike_ppk == PPK_IKE_INTERMEDIATE) {
+		if (ppk_ids_shunks == NULL) {
+			/* find any matching PPK and PPK_ID */
+			const struct secret_ppk_stuff *ppk =
+				get_connection_ppk_and_ppk_id(c);
+			ppk_id = ppk->id;
+			if (ppk != NULL) {
+				found_one = true;
+				chunk_t ppk_confirmation =
+					calc_ppk_confirmation(ike->sa.st_oakley.ta_prf,
+							      &ppk->key,
+							      ike->sa.st_ni, ike->sa.st_nr,
+							      &ike->sa.st_ike_spis,
+							      ike->sa.logger);
+				struct ppk_id_payload payl = { .type = 0, };
+				create_ppk_id_payload(&ppk_id, &payl);
+				if (DBGP(DBG_BASE)) {
+					DBG_log("ppk type: %d", (int) payl.type);
+					DBG_dump_hunk("ppk_id from payload:", payl.ppk_id);
+				}
+
+				struct pbs_out ppks;
+				if (!open_v2N_output_pbs(request.pbs, v2N_PPK_IDENTITY_KEY, &ppks)) {
+					return STF_INTERNAL_ERROR;
+				}
+				if (!emit_unified_ppk_id(&payl, &ppks)) {
+					return STF_INTERNAL_ERROR;
+				}
+				if (!pbs_out_hunk(&ppks, ppk_confirmation, "PPK Confirmation")) {
+					return STF_INTERNAL_ERROR;
+				}
+				close_output_pbs(&ppks);
+				free_chunk_content(&ppk_confirmation);
+			}
+		} else {
+			for (unsigned i = 0; i < ppk_ids_shunks->len; i++) {
+				const struct secret_ppk_stuff *ppk =
+					get_connection_ppk(c, /*ppk_id*/NULL,
+							   /*index*/i);
+				if (ppk != NULL) {
+					found_one = true;
+					chunk_t ppk_confirmation =
+						calc_ppk_confirmation(ike->sa.st_oakley.ta_prf,
+								      &ppk->key,
+								      ike->sa.st_ni, ike->sa.st_nr,
+								      &ike->sa.st_ike_spis,
+								      ike->sa.logger);
+					ppk_id = chunk2((void *) ppk_ids_shunks->list[i].ptr,
+						                 ppk_ids_shunks->list[i].len);
+					struct ppk_id_payload payl = { .type = 0, };
+					create_ppk_id_payload(&ppk_id, &payl);
+					struct pbs_out ppks;
+					if (!open_v2N_output_pbs(request.pbs, v2N_PPK_IDENTITY_KEY, &ppks)) {
+						return STF_INTERNAL_ERROR;
+					}
+					if (!emit_unified_ppk_id(&payl, &ppks)) {
+						return STF_INTERNAL_ERROR;
+					}
+					if (!pbs_out_hunk(&ppks, ppk_confirmation, "PPK Confirmation")) {
+						return STF_INTERNAL_ERROR;
+					}
+					close_output_pbs(&ppks);
+					free_chunk_content(&ppk_confirmation);
+				}
+			}
+		}
+
+		if (!found_one) {
+			if (c->config->ppk.insist) {
+				llog_sa(RC_LOG, ike,
+					"connection requires PPK, but we didn't find one");
+				return STF_FATAL;
+			} else {
+				llog_sa(RC_LOG, ike,
+					"failed to find PPK and PPK_ID, continuing without PPK");
+			}
+		}
+	}
 
 	if (!close_v2_message(&request)) {
 		return STF_INTERNAL_ERROR;
@@ -237,6 +364,53 @@ static stf_status initiate_v2_IKE_INTERMEDIATE_request(struct ike_sa *ike,
 	return STF_OK;
 }
 
+static bool recalc_v2_ppk_interm_keymat(struct ike_sa *ike,
+					shunk_t ppk,
+					const ike_spis_t *new_ike_spis,
+					where_t where)
+{
+	struct logger *logger = ike->sa.logger;
+	const struct prf_desc *prf = ike->sa.st_oakley.ta_prf;
+
+	ldbg(logger, "%s() calculating skeyseed using prf %s",
+	     __func__, prf->common.fqn);
+
+	/*
+	 * We need old_skey_d to recalculate SKEYSEED'.
+	 */
+
+	PK11SymKey *skeyseed =
+		ikev2_ike_sa_ppk_interm_skeyseed(prf,
+						 /*old*/ike->sa.st_skey_d_nss,
+						 ppk, logger);
+	if (skeyseed == NULL) {
+		llog_pexpect(logger, where, "ppk SKEYSEED failed");
+		return false;
+	}
+
+	/* release old keys, salts and cipher contexts */
+
+	symkey_delref(logger, "SK_d", &ike->sa.st_skey_d_nss);
+	symkey_delref(logger, "SK_ai", &ike->sa.st_skey_ai_nss);
+	symkey_delref(logger, "SK_ar", &ike->sa.st_skey_ar_nss);
+	symkey_delref(logger, "SK_ei", &ike->sa.st_skey_ei_nss);
+	symkey_delref(logger, "SK_er", &ike->sa.st_skey_er_nss);
+	symkey_delref(logger, "SK_pi", &ike->sa.st_skey_pi_nss);
+	symkey_delref(logger, "SK_pr", &ike->sa.st_skey_pr_nss);
+	free_chunk_content(&ike->sa.st_skey_chunk_SK_pi);
+	free_chunk_content(&ike->sa.st_skey_chunk_SK_pr);
+	free_chunk_content(&ike->sa.st_skey_initiator_salt);
+	free_chunk_content(&ike->sa.st_skey_responder_salt);
+	cipher_context_destroy(&ike->sa.st_ike_encrypt_cipher_context, logger);
+	cipher_context_destroy(&ike->sa.st_ike_decrypt_cipher_context, logger);
+
+	/* now we have to generate the keys for everything */
+
+	calc_v2_ike_keymat(&ike->sa, skeyseed, new_ike_spis);
+	symkey_delref(logger, "skeyseed", &skeyseed);
+	return true;
+}
+
 stf_status process_v2_IKE_INTERMEDIATE_request(struct ike_sa *ike,
 					       struct child_sa *unused_child UNUSED,
 					       struct msg_digest *md)
@@ -291,7 +465,65 @@ stf_status process_v2_IKE_INTERMEDIATE_request(struct ike_sa *ike,
 		return STF_INTERNAL_ERROR;
 	}
 
-	/* empty message */
+	const struct payload_digest *ppk_id_key_payls = md->pd[PD_v2N_PPK_IDENTITY_KEY];
+	bool found_one = false;		/* found one matching PPK? */
+	const struct secret_ppk_stuff *ppk = NULL;
+
+	while (ppk_id_key_payls != NULL && !found_one && ike->sa.st_v2_ike_ppk == PPK_IKE_INTERMEDIATE) {
+		dbg("received PPK_IDENTITY_KEY");
+		struct ppk_id_key_payload payl;
+		if (!extract_v2N_ppk_id_key(&ppk_id_key_payls->pbs, &payl, ike)) {
+			dbg("failed to extract PPK_ID from PPK_IDENTITY payload. Abort!");
+			return STF_FATAL;
+		}
+
+		ppk = get_connection_ppk(ike->sa.st_connection,
+		                         /*ppk_id*/&payl.ppk_id_payl.ppk_id,
+					 /*index*/0);
+
+		if (ppk != NULL) {
+			chunk_t ppk_confirmation =
+				calc_ppk_confirmation(ike->sa.st_oakley.ta_prf,
+						      &ppk->key,
+						      ike->sa.st_ni, ike->sa.st_nr,
+						      &ike->sa.st_ike_spis,
+						      ike->sa.logger);
+			if (hunk_eq(ppk_confirmation, payl.ppk_confirmation)) {
+				dbg("found matching PPK, send PPK_IDENTITY back");
+				found_one = true;
+				/* we have a match, send PPK_IDENTITY back */
+				struct ppk_id_payload ppk_id_p = { .type = 0, }; 
+				create_ppk_id_payload(&payl.ppk_id_payl.ppk_id, &ppk_id_p);
+
+				struct pbs_out ppks;
+				if (!open_v2N_output_pbs(response.pbs, v2N_PPK_IDENTITY, &ppks)) {
+					return STF_INTERNAL_ERROR;
+				}
+				if (!emit_unified_ppk_id(&ppk_id_p, &ppks)) {
+					return STF_INTERNAL_ERROR;
+				}
+				close_output_pbs(&ppks);
+			}
+			free_chunk_content(&ppk_confirmation);
+		}
+		free_chunk_content(&payl.ppk_id_payl.ppk_id);
+		free_chunk_content(&payl.ppk_confirmation);
+		ppk_id_key_payls = ppk_id_key_payls->next;
+	}
+
+	if (md->pd[PD_v2N_PPK_IDENTITY_KEY] == NULL || !found_one) {
+		if (ike->sa.st_connection->config->ppk.insist) {
+			llog_sa(RC_LOG, ike, "No matching (PPK_ID, PPK) found and connection requires \
+					      a valid PPK. Abort!");
+			record_v2N_response(ike->sa.logger, ike, md,
+					v2N_AUTHENTICATION_FAILED, empty_shunk/*no data*/,
+					ENCRYPTED_PAYLOAD);
+			return STF_FATAL;
+		} else {
+			llog_sa(RC_LOG, ike,
+				"failed to find a matching PPK, continuing without PPK");
+		}
+	}
 
 	if (!close_v2_message(&response)) {
 		return STF_INTERNAL_ERROR;
@@ -314,6 +546,14 @@ stf_status process_v2_IKE_INTERMEDIATE_request(struct ike_sa *ike,
 
 	record_v2_message(&response.message, response.story, response.outgoing_fragments);
 
+	if (found_one) {
+		recalc_v2_ppk_interm_keymat(ike, ppk->key,
+					    &ike->sa.st_ike_spis,
+					    HERE);
+		llog(RC_LOG, ike->sa.logger,
+		     "PPK used in IKE_INTERMEDIATE as responder");
+	}
+
 	return STF_OK;
 }
 
@@ -414,6 +654,34 @@ stf_status process_v2_IKE_INTERMEDIATE_response_continue(struct state *st, struc
 	 * XXX: does the keymat need to be re-computed here?
 	 */
 
+	if (ike->sa.st_v2_ike_ppk == PPK_IKE_INTERMEDIATE && md->pd[PD_v2N_PPK_IDENTITY] != NULL) {
+		struct ppk_id_payload payl;
+		if (!extract_v2N_ppk_identity(&md->pd[PD_v2N_PPK_IDENTITY]->pbs, &payl, ike)) {
+			dbg("failed to extract PPK_ID from PPK_IDENTITY payload. Abort!");
+			return STF_FATAL;
+		}
+		const struct secret_ppk_stuff *ppk =
+			get_connection_ppk(ike->sa.st_connection,
+					   /*ppk_id*/&payl.ppk_id,
+					   /*index*/0);
+		free_chunk_content(&payl.ppk_id);
+
+		recalc_v2_ppk_interm_keymat(ike, ppk->key,
+					    &ike->sa.st_ike_spis,
+					    HERE);
+		llog(RC_LOG, ike->sa.logger,
+		     "PPK used in IKE_INTERMEDIATE as initiator");
+	}
+	if (md->pd[PD_v2N_PPK_IDENTITY] == NULL) {
+		if (ike->sa.st_connection->config->ppk.insist) {
+			llog_sa(RC_LOG, ike, "N(PPK_IDENTITY) not received and connection \
+					      insists on PPK. Abort!");
+			return STF_FATAL;
+		} else {
+			llog_sa(RC_LOG, ike,
+				"N(PPK_IDENTITY) not received, continuing without PPK");
+		}
+	}
 	/*
 	 * We've done one intermediate exchange round, now proceed to
 	 * IKE AUTH.
@@ -431,7 +699,6 @@ stf_status process_v2_IKE_INTERMEDIATE_response_continue(struct state *st, struc
 
 static const struct v2_transition v2_IKE_INTERMEDIATE_initiate_transition = {
 	.story      = "initiating IKE_INTERMEDIATE",
-	.from = { &state_v2_IKE_SA_INIT_IR, &state_v2_IKE_INTERMEDIATE_IR, },
 	.to = &state_v2_IKE_INTERMEDIATE_I,
 	.exchange   = ISAKMP_v2_IKE_INTERMEDIATE,
 	.processor  = initiate_v2_IKE_INTERMEDIATE_request,
@@ -442,19 +709,6 @@ static const struct v2_transition v2_IKE_INTERMEDIATE_initiate_transition = {
 static const struct v2_transition v2_IKE_INTERMEDIATE_responder_transition[] = {
 
 	{ .story      = "Responder: process IKE_INTERMEDIATE request",
-	  .from = { &state_v2_IKE_SA_INIT_R, },
-	  .to = &state_v2_IKE_INTERMEDIATE_R,
-	  .exchange   = ISAKMP_v2_IKE_INTERMEDIATE,
-	  .recv_role  = MESSAGE_REQUEST,
-	  .message_payloads.required = v2P(SK),
-	  .encrypted_payloads.required = LEMPTY,
-	  .encrypted_payloads.optional = LEMPTY,
-	  .processor  = process_v2_IKE_INTERMEDIATE_request,
-	  .llog_success = llog_v2_success_exchange_processed,
-	  .timeout_event = EVENT_v2_DISCARD, },
-
-	{ .story      = "processing IKE_INTERMEDIATE request",
-	  .from = { &state_v2_IKE_INTERMEDIATE_R, },
 	  .to = &state_v2_IKE_INTERMEDIATE_R,
 	  .exchange   = ISAKMP_v2_IKE_INTERMEDIATE,
 	  .recv_role  = MESSAGE_REQUEST,
@@ -469,7 +723,6 @@ static const struct v2_transition v2_IKE_INTERMEDIATE_responder_transition[] = {
 
 static const struct v2_transition v2_IKE_INTERMEDIATE_response_transition[] = {
 	{ .story      = "processing IKE_INTERMEDIATE response",
-	  .from = { &state_v2_IKE_INTERMEDIATE_I, },
 	  .to = &state_v2_IKE_INTERMEDIATE_IR,
 	  .exchange   = ISAKMP_v2_IKE_INTERMEDIATE,
 	  .recv_role  = MESSAGE_RESPONSE,
@@ -487,4 +740,5 @@ V2_STATE(IKE_INTERMEDIATE_R,
 
 V2_EXCHANGE(IKE_INTERMEDIATE, "key IKE SA",
 	    ", initiating IKE_INTERMEDIATE or IKE_AUTH",
-	    CAT_OPEN_IKE_SA, CAT_OPEN_IKE_SA, /*secured*/true);
+	    CAT_OPEN_IKE_SA, CAT_OPEN_IKE_SA, /*secured*/true,
+	    &state_v2_IKE_SA_INIT_IR, &state_v2_IKE_INTERMEDIATE_IR);
