@@ -28,49 +28,50 @@
  */
 
 #include "defs.h"
+
+#include "ikev2_ike_sa_init.h"
+
 #include "demux.h"
 #include "log.h"
 #include "state.h"
-#include "ikev2_send.h"
-#include "ikev2_redirect.h"
-#include "ikev2_host_pair.h"
+#include "crypt_dh.h"
+#include "crypt_ke.h"
+#include "ike_alg.h"
+#include "ike_alg_integ.h"
+#include "connections.h"
 #include "ikev2_states.h"
+#include "instantiate.h"
+#include "pending.h"
+#include "ikev2_parent.h"
+#include "ikev2_ipseckey.h"
+#include "ikev2_proposals.h"
+#include "ikev2_redirect.h"
 #include "ikev2_vendorid.h"
-#include "ikev2_cookie.h"
-#include "ikev2.h"
-#include "ikev2_ike_sa_init.h"
+#include "ikev2_certreq.h"
+#include "pluto_stats.h"
+#include "ikev2_nat.h"
+#include "nat_traversal.h"
 #include "ikev2_ike_intermediate.h"
 #include "ikev2_ike_auth.h"
-#include "connections.h"
-#include "crypt_ke.h"
-#include "nat_traversal.h"
-#include "ikev2_nat.h"
+#include "ikev2_cookie.h"
+#include "ikev2_eap.h"
 #include "unpack.h"
 #include "ikev2_message.h"
-#include "crypt_dh.h"
 #include "ikev2_send.h"
-#include "pluto_x509.h"
-#include "ikev2_cert.h"
-#include "iface.h"
-#include "pending.h"
-#include "ikev2_ipseckey.h"
-#include "pluto_stats.h"
-#include "ikev2_proposals.h"
-#include "ikev2_certreq.h"
-#include "kernel.h"		/* for orphan_holdpass() */
-#include "instantiate.h"	/* for sec_label_instantiate() */
-#include "routing.h"
-#include "ikev2_replace.h"
-#include "revival.h"
-#include "ike_alg_integ.h"	/* for ike_alg_integ_none */
-#include "ikev2_parent.h"
-#include "ikev2_eap.h"
+#include "packet.h"
+#include "passert.h"
+#include "state.h"
+#include "ikev2.h"		/* for ikev2_state_transition_fn */
+#include "ikev2_prf.h"
+#include "crypt_symkey.h"
+#include "ikev2_notification.h"
 
 static ke_and_nonce_cb initiate_v2_IKE_SA_INIT_request_continue;	/* type assertion */
 static dh_shared_secret_cb process_v2_IKE_SA_INIT_response_continue;	/* type assertion */
 static ke_and_nonce_cb process_v2_IKE_SA_INIT_request_continue;		/* forward decl and type assertion */
-static ikev2_state_transition_fn process_v2_IKE_SA_INIT_response_v2N_INVALID_KE_PAYLOAD;
 static ikev2_state_transition_fn process_v2_IKE_SA_INIT_request;
+static ikev2_state_transition_fn process_v2_IKE_SA_INIT_response;
+static ikev2_state_transition_fn process_v2_IKE_SA_INIT_response_v2N_INVALID_KE_PAYLOAD;
 
 static void llog_process_v2_IKE_SA_INIT_request_success(struct ike_sa *ike);
 static void llog_process_v2_IKE_SA_INIT_response_success(struct ike_sa *ike);
@@ -138,488 +139,30 @@ void llog_process_v2_IKE_SA_INIT_response_success(struct ike_sa *ike)
 	}
 }
 
-static void record_first_v2_packet(struct ike_sa *ike, struct msg_digest *md,
-				   where_t where)
+bool calc_v2_new_ike_keymat(struct ike_sa *ike,
+			    const ike_spis_t *new_ike_spis,
+			    where_t where)
 {
-	/*
-	 * Record first packet for later checking of signature.
-	 *
-	 * XXX:
-	 *
-	 * Should this code use pbs_in_all() which uses
-	 * [.start...roof)?  The original code used:
-	 *
-	 * 	clonetochunk(st->st_firstpacket_peer, md->message_pbs.start,
-	 *		     md->message_pbs(.cur-start),
-	 *		     "saved first received packet");
-	 *
-	 * and pbs_in_to_cursor() both use (.cur-.start).
-	 *
-	 * Suspect it doesn't matter as the code initializing
-	 * .message_pbs forces .roof==.cur - look for the comment
-	 * "trim padding (not actually legit)".
-	 */
-	PEXPECT(ike->sa.logger, md->message_pbs.cur == md->message_pbs.roof);
-	replace_chunk(&ike->sa.st_firstpacket_peer,
-		      pbs_in_to_cursor(&md->message_pbs),
-		      where->func);
+	struct logger *logger = ike->sa.logger;
+	PK11SymKey *shared = ike->sa.st_dh_shared_secret;
+	const struct prf_desc *prf = ike->sa.st_oakley.ta_prf;
+
+	ldbg(logger, "%s() calculating skeyseed using prf=%s",
+	     __func__, prf->common.fqn);
+
+	/* generate SKEYSEED from key=(Ni|Nr), hash of shared */
+	PK11SymKey *skeyseed = ikev2_ike_sa_skeyseed(prf, ike->sa.st_ni, ike->sa.st_nr,
+						     shared, logger);
+	if (skeyseed == NULL) {
+		llog_pexpect(logger, where, "rekey SKEYSEED failed");
+		return false;
+	}
+
+	calc_v2_ike_keymat(&ike->sa, skeyseed, new_ike_spis);
+	symkey_delref(logger, "skeyseed", &skeyseed);
+	return true;
 }
 
-void process_v2_IKE_SA_INIT(struct msg_digest *md)
-{
-	/*
-	 * The message ID of the initial exchange is always
-	 * zero.
-	 */
-	if (md->hdr.isa_msgid != 0) {
-		llog_md(md, "IKE_SA_INIT message has non-zero message ID; dropping packet");
-		return;
-	}
-	/*
-	 * Now try to find the state
-	 */
-	switch (v2_msg_role(md)) {
-
-	case MESSAGE_REQUEST:
-	{
-
-		/*
-		 * 3.1.  The IKE Header (Flags)
-		 *
-		 * * I (Initiator) - This bit MUST be set in messages
-		 *   sent by the original initiator of the IKE SA and
-		 *   MUST be cleared in messages sent by the original
-		 *   responder.  It is used by the recipient to
-		 *   determine which eight octets of the SPI were
-		 *   generated by the recipient.  This bit changes to
-		 *   reflect who initiated the last rekey of the IKE
-		 *   SA.
-		 *
-		 * i.e., in the request, I must be set
-		 */
-		if (!(md->hdr.isa_flags & ISAKMP_FLAGS_v2_IKE_I)) {
-			llog_md(md, "IKE_SA_INIT request has I (IKE Initiator) flag clear; dropping packet");
-			return;
-		}
-
-		/*
-		 * 3.1.  The IKE Header (IKE SA Initiator SPI)
-		 *
-		 * o Initiator's SPI (8 octets) - A value chosen by
-		 *   the initiator to identify a unique IKE Security
-		 *   Association.  This value MUST NOT be zero.
-		 *
-		 * (it isn't obvious why this rule is needed;
-		 * exchanges still work)
-		 */
-		if (ike_spi_is_zero(&md->hdr.isa_ike_initiator_spi)) {
-			llog_md(md, "IKE_SA_INIT request has zero IKE SA Initiator SPI; dropping packet");
-			return;
-		}
-
-		/*
-		 * 3.1.  The IKE Header (IKE SA Responder SPI)
-		 *
-		 * o Responder's SPI (8 octets) - A value chosen by
-		 *   the responder to identify a unique IKE Security
-		 *   Association.  This value MUST be zero in the
-		 *   first message of an IKE initial exchange
-		 *   (including repeats of that message including a
-		 *   cookie).
-		 *
-		 * (since this is the very first message, the
-		 * initiator can't know the responder's SPI).
-		 */
-		if (!ike_spi_is_zero(&md->hdr.isa_ike_responder_spi)) {
-			llog_md(md, "IKE_SA_INIT request has non-zero IKE SA Responder SPI; dropping packet");
-			return;
-		}
-
-		/*
-		 * Look for a pre-existing IKE SA responder state
-		 * using just the SPIi (SPIr in the message is zero so
-		 * can't be used).
-		 *
-		 * XXX: RFC 7296 says this isn't sufficient:
-		 *
-		 *   2.1.  Use of Retransmission Timers
-		 *
-		 *   Retransmissions of the IKE_SA_INIT request
-		 *   require some special handling.  When a responder
-		 *   receives an IKE_SA_INIT request, it has to
-		 *   determine whether the packet is a retransmission
-		 *   belonging to an existing "half-open" IKE SA (in
-		 *   which case the responder retransmits the same
-		 *   response), or a new request (in which case the
-		 *   responder creates a new IKE SA and sends a fresh
-		 *   response), or it belongs to an existing IKE SA
-		 *   where the IKE_AUTH request has been already
-		 *   received (in which case the responder ignores
-		 *   it).
-		 *
-		 *   It is not sufficient to use the initiator's SPI
-		 *   and/or IP address to differentiate between these
-		 *   three cases because two different peers behind a
-		 *   single NAT could choose the same initiator SPI.
-		 *   Instead, a robust responder will do the IKE SA
-		 *   lookup using the whole packet, its hash, or the
-		 *   Ni payload.
-		 *
-		 * But realistically, either there's an IOT device
-		 * sending out a hardwired SPIi, or there is a clash
-		 * and a retry will generate a new conflicting SPIi.
-		 *
-		 * If the lookup succeeds then there are several
-		 * possibilities:
-		 *
-		 * State has Message ID == 0:
-		 *
-		 * Either it really is a duplicate; or it's a second
-		 * (fake?) initiator sending the same SPIi at exactly
-		 * the same time as the first (wow, what are the odds,
-		 * it must be our lucky day!).
-		 *
-		 * Either way, the duplicate code needs to compare
-		 * packets and decide if a retransmit or drop is
-		 * required.  If the second initiator is real, then it
-		 * will timeout and then retry with a new SPIi.
-		 *
-		 * State has Message ID > 0:
-		 *
-		 * Either it is an old duplicate; or, again, it's a
-		 * second initiator sending the same SPIi only slightly
-		 * later (again, what are the odds!).
-		 *
-		 * Several choices: let the duplicate code drop the
-		 * packet, which is correct for an old duplicate
-		 * message; or ignore the existing state and create a
-		 * new one, which is good for the second initiator but
-		 * not so good for an old duplicate.  Given an old
-		 * duplicate is far more likely, handle that cleenly -
-		 * let the duplicate code drop the packet.
-		 */
-		struct ike_sa *old = find_v2_ike_sa_by_initiator_spi(&md->hdr.isa_ike_initiator_spi,
-								     SA_RESPONDER);
-		if (old != NULL) {
-			intmax_t msgid = md->hdr.isa_msgid;
-			pexpect(msgid == 0); /* per above */
-			/* XXX: keep test results happy */
-			if (md->fake_clone) {
-				llog_sa(RC_LOG, old, "IMPAIR: processing a fake (cloned) message");
-			}
-
-			if (old->sa.st_state != &state_v2_IKE_SA_INIT_R) {
-				/*
-				 * For a duplicate, the IKE SA can't
-				 * have advanced beyond IKE_SA_INIT.
-				 */
-				llog_sa(RC_LOG, old,
-					"received old IKE_SA_INIT request; packet dropped");
-				return;
-			}
-
-			/*
-			 * The IKE SA hasn't yet started processing
-			 * IKE AUTH (or IKE_INTERMEDIATE).  However it
-			 * may be accumulating fragments or running
-			 * background crypto in preparation.
-			 *
-			 * Ignore that.  Until the fragments have been
-			 * re-assembled and verified they can't be
-			 * trusted.
-			 */
-
-			PEXPECT(old->sa.logger, old->sa.st_v2_msgid_windows.responder.recv == 0);
-			PEXPECT(old->sa.logger, old->sa.st_v2_msgid_windows.responder.sent == 0);
-			if (old->sa.st_v2_msgid_windows.responder.wip != -1) {
-				/*
-				 * Started processing (accumulating)
-				 * the next packet.  No sense in
-				 * replying to an older one.
-				 */
-				PEXPECT(old->sa.logger, old->sa.st_v2_msgid_windows.responder.wip == 1);
-				llog_sa(RC_LOG, old,
-					"received IKE_SA_INIT request from previous exchange; packet dropped");
-			}
-
-			if (hunk_eq(old->sa.st_firstpacket_peer, pbs_in_all(&md->message_pbs))) {
-				/*
-				 * Clearly a duplicate.
-				 *
-				 * XXX: Log message matches
-				 * is_duplicate_request() - keep test
-				 * results happy.
-				 */
-				llog_sa(RC_LOG, old,
-					"received duplicate IKE_SA_INIT request; retransmitting response");
-				send_recorded_v2_message(old, "IKE_SA_INIT responder retransmit",
-							 old->sa.st_v2_msgid_windows.responder.outgoing_fragments);
-				return;
-			}
-
-			/*
-			 * Is this a second IKE_SA_INIT request using
-			 * the same SPIi as the existing IKE SA?  Wow!
-			 * But lets not go there.
-			 *
-			 * XXX: Log message matches
-			 * is_duplicate_request() - keep test results
-			 * happy.
-			 */
-			llog_sa(RC_LOG, old, "received too old IKE_SA_INIT retransmit");
-			return;
-		}
-
-		if (drop_new_exchanges()) {
-			/* only log for debug to prevent disk filling up */
-			dbg("pluto is overloaded with half-open IKE SAs; dropping new exchange");
-			return;
-		}
-
-		/*
-		 * Always check for cookies!
-		 *
-		 * XXX: why?
-		 *
-		 * Because the v2N_COOKIE payload is first, parsing
-		 * and verifying it should be relatively quick and
-		 * cheap.  Right?
-		 *
-		 * No.  The equation uses v2Ni forcing the entire
-		 * payload to be parsed.
-		 *
-		 * The error notification is probably INVALID_SYNTAX,
-		 * but could be v2N_UNSUPPORTED_CRITICAL_PAYLOAD.
-		 */
-		pexpect(!md->message_payloads.parsed);
-		md->message_payloads = ikev2_decode_payloads(md->logger, md,
-							     &md->message_pbs,
-							     md->hdr.isa_np);
-		if (md->message_payloads.n != v2N_NOTHING_WRONG) {
-			if (require_ddos_cookies()) {
-				dbg("DDOS so not responding to invalid packet");
-			} else {
-				shunk_t data = shunk2(md->message_payloads.data,
-						      md->message_payloads.data_size);
-				send_v2N_response_from_md(md, md->message_payloads.n,
-							  &data);
-			}
-			return;
-		}
-
-		/*
-		 * Do I want a cookie?
-		 */
-		if (v2_rejected_initiator_cookie(md, require_ddos_cookies())) {
-			dbg("pluto is overloaded and demanding cookies; dropping new exchange");
-			return;
-		}
-
-		/*
-		 * Check for v2N_REDIRECT_SUPPORTED /
-		 * v2N_REDIRECTED_FROM notification.  If redirection
-		 * is a MUST, try to respond with v2N_REDIRECT and
-		 * don't continue further.  Otherwise continue as
-		 * usual.
-		 *
-		 * The function below will do everything (and log the
-		 * result).
-		 */
-		if (redirect_global(md)) {
-			return;
-		}
-
-		/*
-		 * Check if we would drop the packet based on VID
-		 * before we create a state. Move this to
-		 * ikev2_oppo.c: drop_oppo_requests()?
-		 */
-		for (struct payload_digest *p = md->chain[ISAKMP_NEXT_v2V]; p != NULL; p = p->next) {
-			if (vid_is_oppo((char *)p->pbs.cur, pbs_left(&p->pbs))) {
-				if (pluto_drop_oppo_null) {
-					dbg("Dropped IKE request for Opportunistic IPsec by global policy");
-					return;
-				}
-				dbg("Processing IKE request for Opportunistic IPsec");
-				break;
-			}
-		}
-
-		/*
-		 * Does the message match the (only) expected
-		 * transition?
-		 */
-		const struct v2_transition *transition =
-			find_v2_unsecured_transition(md->logger,
-						     v2_IKE_SA_INIT_exchange.responder,
-						     md);
-		if (transition == NULL) {
-			/* already logged */
-			send_v2N_response_from_md(md, v2N_INVALID_SYNTAX, NULL);
-			return;
-		}
-
-		/*
-		 * Is there a connection that matches the message?
-		 */
-		bool send_reject_response = true;
-		struct connection *c = find_v2_host_pair_connection(md, &send_reject_response);
-		if (c == NULL) {
-			endpoint_buf b;
-			enum_buf xb;
-			llog(RC_LOG, md->logger,
-			     "%s message received on %s but no suitable connection found with IKEv2 policy",
-			     str_enum(&ikev2_exchange_names, md->hdr.isa_xchg, &xb),
-			     str_endpoint(&md->iface->local_endpoint, &b));
-			if (send_reject_response) {
-				/*
-				 * NO_PROPOSAL_CHOSEN is used when the
-				 * list of proposals is empty, like
-				 * when we did not find any connection
-				 * to use.
-				 *
-				 * INVALID_SYNTAX is for errors that a
-				 * configuration change could not fix.
-				 *
-				 * This call will log that the message
-				 * was sent.  Should its message be
-				 * merged with the above?
-				 */
-				send_v2N_response_from_md(md, v2N_NO_PROPOSAL_CHOSEN, NULL);
-			}
-			return;
-		}
-
-		/*
-		 * We've committed to creating a state and,
-		 * presumably, dedicating real resources to the
-		 * connection.
-		 */
-		struct ike_sa *ike = new_v2_ike_sa_responder(c, &state_v2_IKE_SA_INIT_R0, md);
-
-		statetime_t start = statetime_backdate(&ike->sa, &md->md_inception);
-		/* XXX: keep test results happy */
-		if (md->fake_clone) {
-			llog_sa(RC_LOG, ike, "IMPAIR: processing a fake (cloned) message");
-		}
-		v2_dispatch(ike, md, transition);
-		statetime_stop(&start, "%s()", __func__);
-		connection_delref(&c, md->logger);
-		return;
-	}
-
-	case MESSAGE_RESPONSE:
-	{
-		/*
-		 * 3.1.  The IKE Header (Flags)
-		 *
-		 * * I (Initiator) - This bit MUST be set in messages
-		 *   sent by the original initiator of the IKE SA and
-		 *   MUST be cleared in messages sent by the original
-		 *   responder.  It is used by the recipient to
-		 *   determine which eight octets of the SPI were
-		 *   generated by the recipient.  This bit changes to
-		 *   reflect who initiated the last rekey of the IKE
-		 *   SA.
-		 *
-		 * i.e., in the response I must be clear
-		 */
-		if (md->hdr.isa_flags & ISAKMP_FLAGS_v2_IKE_I) {
-			llog_md(md, "IKE_SA_INIT response has I (IKE Initiator) flag set; dropping packet");
-			return;
-		}
-
-		/*
-		 * 2.6.  IKE SA SPIs and Cookies:
-		 *
-		 *   When the IKE_SA_INIT exchange does not result in
-		 *   the creation of an IKE SA due to
-		 *   INVALID_KE_PAYLOAD, NO_PROPOSAL_CHOSEN, or
-		 *   COOKIE, the responder's SPI will be zero also in
-		 *   the response message.  However, if the responder
-		 *   sends a non-zero responder SPI, the initiator
-		 *   should not reject the response for only that
-		 *   reason.
-		 *
-		 * i.e., can't check response for non-zero SPIr.
-		 *
-		 * Look for a pre-existing IKE SA responder state
-		 * using just the SPIi (SPIr in the message isn't
-		 * known so can't be used).
-		 *
-		 * An IKE_SA_INIT error notification response
-		 * (INVALID_KE, COOKIE) should contain a zero SPIr (it
-		 * must be ignored).
-		 *
-		 * An IKE_SA_INIT success response will contain an as
-		 * yet unknown but non-zero SPIr so looking for it
-		 * won't work.
-		 */
-		struct ike_sa *ike = find_v2_ike_sa_by_initiator_spi(&md->hdr.isa_ike_initiator_spi,
-								     SA_INITIATOR);
-		if (ike == NULL) {
-			/*
-			 * There should be a state matching the
-			 * original initiator's IKE SPIs.  Since there
-			 * isn't someone's playing games.  Drop the
-			 * packet.
-			 */
-			llog_md(md, "dropping IKE_SA_INIT response no matching IKE ISA");
-			return;
-		}
-
-		/*
-		 * While the IKE SA is initiating, .sent==-1,
-		 * .recv==-1, and .wip==0.  Once the message is sent,
-		 * .sent==0, .recv==-1, and .wip==-1.
-		 */
-		if (ike->sa.st_state->kind != STATE_V2_IKE_SA_INIT_I ||
-		    ike->sa.st_v2_msgid_windows.initiator.sent != 0 ||
-		    ike->sa.st_v2_msgid_windows.initiator.recv != -1 ||
-		    ike->sa.st_v2_msgid_windows.initiator.wip != -1 ||
-		    ike->sa.st_v2_msgid_windows.initiator.exchange == NULL) {
-			/*
-			 * This doesn't seem right; drop the
-			 * packet.
-			 *
-			 * Log against MD as source may not match IKE.
-			 */
-			llog_md(md, "dropping IKE_SA_INIT response as not expected by IKE SA #%lu",
-				ike->sa.st_serialno);
-			return;
-		}
-
-		dbg("unpacking clear payloads");
-		md->message_payloads = ikev2_decode_payloads(ike->sa.logger, md,
-							     &md->message_pbs,
-							     md->hdr.isa_np);
-		if (md->message_payloads.n != v2N_NOTHING_WRONG) {
-			/* already logged */
-			return;
-		}
-
-		/* transition? */
-		const struct v2_exchange *exchange = ike->sa.st_v2_msgid_windows.initiator.exchange;
-		const struct v2_transition *transition =
-			find_v2_unsecured_transition(ike->sa.logger, exchange->response, md);
-		if (transition == NULL) {
-			/* already logged */
-			return;
-		}
-
-		statetime_t start = statetime_backdate(&ike->sa, &md->md_inception);
-		v2_dispatch(ike, md, transition);
-		statetime_stop(&start, "%s()", __func__);
-		return;
-	}
-
-	default:
-		bad_case(v2_msg_role(md));
-	}
-
-}
-
-
 /*
  *
  ***************************************************************
@@ -641,13 +184,6 @@ struct ike_sa *initiate_v2_IKE_SA_INIT_request(struct connection *c,
 					       shunk_t sec_label,
 					       bool detach_whack)
 {
-	if (drop_new_exchanges()) {
-		/* Only drop outgoing opportunistic connections */
-		if (is_opportunistic(c)) {
-			return NULL;
-		}
-	}
-
 	struct ike_sa *ike = new_v2_ike_sa_initiator(c);
 	if (ike == NULL) {
 		return NULL;
@@ -668,18 +204,22 @@ struct ike_sa *initiate_v2_IKE_SA_INIT_request(struct connection *c,
 		 * sec-label for the child.  Assume this is a forced
 		 * up aka childless IKE SA.
 		 */
-		PEXPECT(c->logger, is_labeled_parent(c));
-		ldbg(c->logger,
+		PEXPECT(ike->sa.logger, is_labeled_parent(c));
+		ldbg(ike->sa.logger,
 		     "labeled parent connection with sec_label="PRI_SHUNK" but no child sec_label; assuming childless",
 		     pri_shunk(c->config->sec_label));
+	} else if (!has_child_policy(policy)) {
+		/*
+		 * When replacing (re-authenticating) the IKE (ISAKMP)
+		 * SA, policy=LEMPTY stopping the code adding a
+		 * pending child - the Child will be lurking on the
+		 * replace queue!?!
+		 */
+		ldbg(ike->sa.logger,
+		     "omitting CHILD SA payloads from the IKE_AUTH request as no child policy");
 	} else if (impair.omit_v2_ike_auth_child) {
 		llog_sa(RC_LOG, ike, "IMPAIR: omitting CHILD SA payloads from the IKE_AUTH request");
-	} else if (has_child_policy(policy)) {
-		/*
-		 * When replacing the IKE (ISAKMP) SA, policy=LEMPTY
-		 * so that a Child SA isn't also initiated and this
-		 * code is skipped.
-		 */
+	} else {
 		struct connection *cc;
 		if (is_labeled(c)) {
 			PEXPECT(ike->sa.logger, is_labeled_parent(c));
@@ -919,10 +459,14 @@ bool record_v2_IKE_SA_INIT_request(struct ike_sa *ike)
 			return false;
 	}
 
-	/* Send INTERMEDIATE_EXCHANGE_SUPPORTED Notify payload */
+	/* Send INTERMEDIATE_EXCHANGE_SUPPORTED (and maybe USE_PPK_INT) Notify payload */
 	if (c->config->intermediate) {
 		if (!emit_v2N(v2N_INTERMEDIATE_EXCHANGE_SUPPORTED, request.pbs))
 			return STF_INTERNAL_ERROR;
+		if (c->config->ppk.allow) {
+			if (!emit_v2N(v2N_USE_PPK_INT, request.pbs))
+				return false;
+		}
 	}
 
 	/* first check if this IKE_SA_INIT came from redirect
@@ -933,9 +477,9 @@ bool record_v2_IKE_SA_INIT_request(struct ike_sa *ike)
 	 *   and send v2N_REDIRECT_SUPPORTED if we do
 	 */
 	if (address_is_specified(c->redirect.ip)) {
-		if (!emit_redirected_from_notification(&c->redirect.old_gw_address,
-						       request.pbs))
+		if (!emit_v2N_REDIRECTED_FROM(&c->redirect.old_gw_address, request.pbs)) {
 			return false;
+		}
 	} else if (c->config->redirect.accept) {
 		if (!emit_v2N(v2N_REDIRECT_SUPPORTED, request.pbs))
 			return false;
@@ -1032,7 +576,7 @@ stf_status process_v2_IKE_SA_INIT_request(struct ike_sa *ike,
 	/* set up new state */
 	update_ike_endpoints(ike, md);
 	passert(ike->sa.st_ike_version == IKEv2);
-	passert(ike->sa.st_state == &state_v2_IKE_SA_INIT_R0);
+	passert(ike->sa.st_state == &state_v2_UNSECURED_R);
 	passert(ike->sa.st_sa_role == SA_RESPONDER);
 
 	/* Vendor ID processing */
@@ -1057,7 +601,8 @@ stf_status process_v2_IKE_SA_INIT_request(struct ike_sa *ike,
 	if (n != v2N_NOTHING_WRONG) {
 		pexpect(ike->sa.st_sa_role == SA_RESPONDER);
 		record_v2N_response(ike->sa.logger, ike, md,
-				    n, NULL, UNENCRYPTED_PAYLOAD);
+				    n, empty_shunk,
+				    UNENCRYPTED_PAYLOAD);
 		/*
 		 * STF_FATAL will send the recorded message and then
 		 * kill the IKE SA.  Should it instead zombify the IKE
@@ -1103,13 +648,30 @@ stf_status process_v2_IKE_SA_INIT_request(struct ike_sa *ike,
 		accept_v2_notification(v2N_IKEV2_FRAGMENTATION_SUPPORTED,
 				       ike->sa.logger, md, c->config->ike_frag.allow);
 
-	ike->sa.st_v2_ike_ppk_enabled =
-		accept_v2_notification(v2N_USE_PPK,
-				       ike->sa.logger, md, c->config->ppk.allow);
-	if (c->config->ppk.insist && !ike->sa.st_v2_ike_ppk_enabled) {
+	ike->sa.st_v2_ike_intermediate.enabled =
+		accept_v2_notification(v2N_INTERMEDIATE_EXCHANGE_SUPPORTED,
+				       ike->sa.logger, md, c->config->intermediate);
+
+	bool ppk_ike_auth_enabled =
+		accept_v2_notification(v2N_USE_PPK, ike->sa.logger,
+				       md, c->config->ppk.allow);
+	bool ppk_ike_intermediate_enabled =
+		accept_v2_notification(v2N_USE_PPK_INT, ike->sa.logger,
+				       md, c->config->ppk.allow);
+
+	/* PPK (RFC8784 + draft-ietf-ipsecme-ikev2-qr-alt-04) logic */
+	if (ike->sa.st_v2_ike_intermediate.enabled && ppk_ike_intermediate_enabled) {
+		ike->sa.st_v2_ike_ppk = PPK_IKE_INTERMEDIATE;
+	} else if (ppk_ike_auth_enabled) {
+		ike->sa.st_v2_ike_ppk = PPK_IKE_AUTH;
+	} else {
+		ike->sa.st_v2_ike_ppk = PPK_DISABLED;
+	}
+
+	if (c->config->ppk.insist && ike->sa.st_v2_ike_ppk == PPK_DISABLED) {
 		record_v2N_response(ike->sa.logger, ike, md,
-				    v2N_NO_PROPOSAL_CHOSEN,
-				    NULL, UNENCRYPTED_PAYLOAD);
+				    v2N_NO_PROPOSAL_CHOSEN, empty_shunk,
+				    UNENCRYPTED_PAYLOAD);
 		llog_sa(RC_LOG, ike,
 			"connection has ppk=insist but peer does not support PPK");
 		return STF_FATAL;
@@ -1142,7 +704,8 @@ stf_status process_v2_IKE_SA_INIT_request(struct ike_sa *ike,
 	if (md->pd[PD_v2N_SIGNATURE_HASH_ALGORITHMS] != NULL) {
 		if (!negotiate_hash_algo_from_notification(&md->pd[PD_v2N_SIGNATURE_HASH_ALGORITHMS]->pbs, ike)) {
 			record_v2N_response(ike->sa.logger, ike, md,
-					    v2N_INVALID_SYNTAX, NULL, UNENCRYPTED_PAYLOAD);
+					    v2N_INVALID_SYNTAX, empty_shunk,
+					    UNENCRYPTED_PAYLOAD);
 			/*
 			 * STF_FATAL will send the recorded
 			 * message and then kill the IKE SA.
@@ -1163,15 +726,15 @@ stf_status process_v2_IKE_SA_INIT_request(struct ike_sa *ike,
 	return STF_SUSPEND;
 }
 
-static stf_status process_v2_IKE_SA_INIT_request_continue(struct state *ike_st,
-							  struct msg_digest *md,
-							  struct dh_local_secret *local_secret,
-							  chunk_t *nonce)
+stf_status process_v2_IKE_SA_INIT_request_continue(struct state *ike_st,
+						   struct msg_digest *md,
+						   struct dh_local_secret *local_secret,
+						   chunk_t *nonce)
 {
 	struct ike_sa *ike = pexpect_ike_sa(ike_st);
 	pexpect(ike->sa.st_sa_role == SA_RESPONDER);
 	pexpect(v2_msg_role(md) == MESSAGE_REQUEST); /* i.e., MD!=NULL */
-	pexpect(ike->sa.st_state == &state_v2_IKE_SA_INIT_R0);
+	pexpect(ike->sa.st_state == &state_v2_UNSECURED_R);
 	dbg("%s() for #%lu %s: calculated ke+nonce, sending R1",
 	    __func__, ike->sa.st_serialno, ike->sa.st_state->name);
 
@@ -1215,7 +778,7 @@ static stf_status process_v2_IKE_SA_INIT_request_continue(struct state *ike_st,
 		 * family, hence FATAL.
 		 */
 		record_v2N_response(ike->sa.logger, ike, md,
-				    v2N_INVALID_SYNTAX, NULL/*no-data*/,
+				    v2N_INVALID_SYNTAX, empty_shunk/*no-data*/,
 				    UNENCRYPTED_PAYLOAD);
 		return STF_FATAL;
 	}
@@ -1261,22 +824,22 @@ static stf_status process_v2_IKE_SA_INIT_request_continue(struct state *ike_st,
 			return STF_INTERNAL_ERROR;
 	}
 
-	/* Send USE_PPK Notify payload */
-	if (ike->sa.st_v2_ike_ppk_enabled) {
-		if (!emit_v2N(v2N_USE_PPK, response.pbs))
-			return STF_INTERNAL_ERROR;
-	 }
-
 	/* Send INTERMEDIATE_EXCHANGE_SUPPORTED Notify payload */
-	ike->sa.st_v2_ike_intermediate.enabled =
-		accept_v2_notification(v2N_INTERMEDIATE_EXCHANGE_SUPPORTED,
-				       ike->sa.logger, md, c->config->intermediate);
 	if (ike->sa.st_v2_ike_intermediate.enabled) {
 		if (!emit_v2N(v2N_INTERMEDIATE_EXCHANGE_SUPPORTED, response.pbs)) {
 			return STF_INTERNAL_ERROR;
 		}
 	}
 
+	/* Send USE_PPK / USE_PPK_INT Notify payload */
+	if (ike->sa.st_v2_ike_ppk == PPK_IKE_AUTH) {
+		if (!emit_v2N(v2N_USE_PPK, response.pbs))
+			return STF_INTERNAL_ERROR;
+	} else if (ike->sa.st_v2_ike_ppk == PPK_IKE_INTERMEDIATE) {
+		if (!emit_v2N(v2N_USE_PPK_INT, response.pbs))
+			return STF_INTERNAL_ERROR;
+	}
+
 	/*
 	 * Send the responder's SIGNATURE_HASH_ALGORITHMS notification
 	 * unconditionally:
@@ -1463,9 +1026,9 @@ stf_status process_v2_IKE_SA_INIT_response_v2N_INVALID_KE_PAYLOAD(struct ike_sa
  * IKE_INTERMEDIATE paths.
  */
 
-static stf_status process_v2_IKE_SA_INIT_response(struct ike_sa *ike,
-						  struct child_sa *unused_child UNUSED,
-						  struct msg_digest *md)
+stf_status process_v2_IKE_SA_INIT_response(struct ike_sa *ike,
+					   struct child_sa *unused_child UNUSED,
+					   struct msg_digest *md)
 {
 	v2_notification_t n;
 	struct connection *c = ike->sa.st_connection;
@@ -1516,10 +1079,34 @@ static stf_status process_v2_IKE_SA_INIT_response(struct ike_sa *ike,
 		accept_v2_notification(v2N_IKEV2_FRAGMENTATION_SUPPORTED,
 				       ike->sa.logger, md, c->config->ike_frag.allow);
 
-	ike->sa.st_v2_ike_ppk_enabled =
-		accept_v2_notification(v2N_USE_PPK,
-				       ike->sa.logger, md, c->config->ppk.allow);
-	if (c->config->ppk.insist && !ike->sa.st_v2_ike_ppk_enabled) {
+	/*
+	 * If we see the intermediate AND we are configured to use
+	 * intermediate.
+	 *
+	 * For now, do only one Intermediate Exchange round and
+	 * proceed with IKE_AUTH.
+	 */
+	ike->sa.st_v2_ike_intermediate.enabled =
+		accept_v2_notification(v2N_INTERMEDIATE_EXCHANGE_SUPPORTED,
+				       ike->sa.logger, md, c->config->intermediate);
+
+	bool ppk_ike_auth_enabled =
+		accept_v2_notification(v2N_USE_PPK, ike->sa.logger,
+				       md, c->config->ppk.allow);
+	bool ppk_ike_intermediate_enabled =
+		accept_v2_notification(v2N_USE_PPK_INT, ike->sa.logger,
+				       md, c->config->ppk.allow);
+
+	/* PPK (RFC8784 + draft-ietf-ipsecme-ikev2-qr-alt-04) logic */
+	if (ike->sa.st_v2_ike_intermediate.enabled && ppk_ike_intermediate_enabled) {
+		ike->sa.st_v2_ike_ppk = PPK_IKE_INTERMEDIATE;
+	} else if (ppk_ike_auth_enabled) {
+		ike->sa.st_v2_ike_ppk = PPK_IKE_AUTH;
+	} else {
+		ike->sa.st_v2_ike_ppk = PPK_DISABLED;
+	}
+
+	if (c->config->ppk.insist && ike->sa.st_v2_ike_ppk == PPK_DISABLED) {
 		llog_sa(RC_LOG, ike,
 			"connection has ppk=insist but peer does not support PPK");
 		return STF_FATAL;
@@ -1640,30 +1227,8 @@ static stf_status process_v2_IKE_SA_INIT_response(struct ike_sa *ike,
 
 	/*
 	 * Initiate the calculation of g^xy.
-	 *
-	 * Form and pass in the full SPI[ir] that will eventually be
-	 * used by this IKE SA.  Only once DH has been computed and
-	 * the SA is secure (but not authenticated) should the state's
-	 * IKE SPIr be updated.
 	 */
 
-	pexpect(ike_spi_is_zero(&ike->sa.st_ike_spis.responder));
-	ike->sa.st_ike_rekey_spis = (ike_spis_t) {
-		.initiator = ike->sa.st_ike_spis.initiator,
-		.responder = md->hdr.isa_ike_responder_spi,
-	};
-
-	/*
-	 * If we see the intermediate AND we are configured to use
-	 * intermediate.
-	 *
-	 * For now, do only one Intermediate Exchange round and
-	 * proceed with IKE_AUTH.
-	 */
-	ike->sa.st_v2_ike_intermediate.enabled =
-		accept_v2_notification(v2N_INTERMEDIATE_EXCHANGE_SUPPORTED,
-				       ike->sa.logger, md, c->config->intermediate);
-
 	submit_dh_shared_secret(/*callback*/&ike->sa, /*task*/&ike->sa, md,
 				ike->sa.st_gr/*initiator needs responder KE*/,
 				process_v2_IKE_SA_INIT_response_continue, HERE);
@@ -1689,10 +1254,23 @@ stf_status process_v2_IKE_SA_INIT_response_continue(struct state *ike_sa,
 		return STF_FATAL;
 	}
 
-	calc_v2_keymat(&ike->sa,
-		       NULL /* no old keymat; not a rekey */,
-		       NULL /* no old prf; not a rekey */,
-		       &ike->sa.st_ike_rekey_spis);
+	/*
+	 * Form and pass in the full SPI[ir] that will eventually be
+	 * used by this IKE SA.  Only once DH has been computed and
+	 * the SA is secure (but not authenticated) should the state's
+	 * IKE SPIr be updated.
+	 */
+
+	pexpect(ike_spi_is_zero(&ike->sa.st_ike_spis.responder));
+	ike->sa.st_ike_rekey_spis = (ike_spis_t) {
+		.initiator = ike->sa.st_ike_spis.initiator,
+		.responder = md->hdr.isa_ike_responder_spi,
+	};
+
+	if (!calc_v2_new_ike_keymat(ike, &ike->sa.st_ike_rekey_spis, HERE)) {
+		/* already logged */
+		return STF_FATAL;
+	}
 
 	/*
 	 * All systems are go.
@@ -1736,7 +1314,6 @@ static const struct v2_transition v2_IKE_SA_INIT_initiate_transition = {
 	 * HDR, SAi1, KEi, Ni -->
 	 */
 	.story      = "initiating IKE_SA_INIT",
-	.from = { &state_v2_IKE_SA_INIT_I0, },
 	.to = &state_v2_IKE_SA_INIT_I,
 	.exchange   = ISAKMP_v2_IKE_SA_INIT,
 	.processor  = NULL, /* XXX: should be set */
@@ -1750,7 +1327,6 @@ static const struct v2_transition v2_IKE_SA_INIT_responder_transition[] = {
 	 * HDR, SAr1, KEr, Nr, [CERTREQ] -->
 	 */
 	{ .story      = "Respond to IKE_SA_INIT",
-	  .from = { &state_v2_IKE_SA_INIT_R0, },
 	  .to = &state_v2_IKE_SA_INIT_R,
 	  .exchange   = ISAKMP_v2_IKE_SA_INIT,
 	  .recv_role  = MESSAGE_REQUEST,
@@ -1768,7 +1344,6 @@ static const struct v2_transition v2_IKE_SA_INIT_response_transition[] = {
 	 */
 
 	{ .story      = "received anti-DDOS COOKIE response; resending IKE_SA_INIT request with cookie payload added",
-	  .from = { &state_v2_IKE_SA_INIT_I, },
 	  .to = &state_v2_IKE_SA_INIT_I0,
 	  .exchange   = ISAKMP_v2_IKE_SA_INIT,
 	  .recv_role  = MESSAGE_RESPONSE,
@@ -1779,7 +1354,6 @@ static const struct v2_transition v2_IKE_SA_INIT_response_transition[] = {
 	  .timeout_event = EVENT_v2_DISCARD, },
 
 	{ .story      = "received INVALID_KE_PAYLOAD response; resending IKE_SA_INIT with new KE payload",
-	  .from = { &state_v2_IKE_SA_INIT_I, },
 	  .to = &state_v2_IKE_SA_INIT_I0,
 	  .exchange   = ISAKMP_v2_IKE_SA_INIT,
 	  .recv_role  = MESSAGE_RESPONSE,
@@ -1790,13 +1364,12 @@ static const struct v2_transition v2_IKE_SA_INIT_response_transition[] = {
 	  .timeout_event = EVENT_v2_DISCARD, },
 
 	{ .story      = "received REDIRECT response; resending IKE_SA_INIT request to new destination",
-	  .from = { &state_v2_IKE_SA_INIT_I, },
 	  .to = &state_v2_IKE_SA_INIT_I0, /* XXX: never happens STF_SUSPEND */
 	  .exchange   = ISAKMP_v2_IKE_SA_INIT,
 	  .recv_role  = MESSAGE_RESPONSE,
 	  .message_payloads.required = v2P(N),
 	  .message_payloads.notification = v2N_REDIRECT,
-	  .processor  = process_v2_IKE_SA_INIT_v2N_REDIRECT_response,
+	  .processor  = process_v2_IKE_SA_INIT_response_v2N_REDIRECT,
 	  .llog_success = ldbg_v2_success,
 	  .timeout_event = EVENT_v2_DISCARD,
 	},
@@ -1808,7 +1381,6 @@ static const struct v2_transition v2_IKE_SA_INIT_response_transition[] = {
 	 *      TSi, TSr}      -->
 	 */
 	{ .story      = "Initiator: process IKE_SA_INIT reply, initiate IKE_AUTH or IKE_INTERMEDIATE",
-	  .from = { &state_v2_IKE_SA_INIT_I, },
 	  .to = &state_v2_IKE_SA_INIT_IR, /* next exchange does IKE_AUTH | IKE_INTERMEDIATE */
 	  .exchange   = ISAKMP_v2_IKE_SA_INIT,
 	  .recv_role  = MESSAGE_RESPONSE,
@@ -1823,10 +1395,6 @@ static const struct v2_transition v2_IKE_SA_INIT_response_transition[] = {
 
 V2_STATE(IKE_SA_INIT_I0, "waiting for KE to finish", CAT_IGNORE, /*secured*/false);
 
-V2_STATE(IKE_SA_INIT_R0, "processing IKE_SA_INIT request",
-	 CAT_HALF_OPEN_IKE_SA, /*secured*/false,
-	 &v2_IKE_SA_INIT_exchange);
-
 V2_STATE(IKE_SA_INIT_R,
 	 "sent IKE_SA_INIT response, waiting for IKE_INTERMEDIATE or IKE_AUTH request",
 	 CAT_HALF_OPEN_IKE_SA, /*secured*/true,
@@ -1834,4 +1402,5 @@ V2_STATE(IKE_SA_INIT_R,
 
 V2_EXCHANGE(IKE_SA_INIT, "initiate IKE SA",
 	    ", preparing IKE_INTERMEDIATE or IKE_AUTH request",
-	    CAT_HALF_OPEN_IKE_SA, CAT_OPEN_IKE_SA, /*secured*/false);
+	    CAT_HALF_OPEN_IKE_SA, CAT_OPEN_IKE_SA, /*secured*/false,
+	    &state_v2_IKE_SA_INIT_I0);
