@@ -20,7 +20,9 @@ struct state;
 struct ike_sa;
 struct child_sa;
 
-void liveness_check(struct state *st);
+void event_v2_liveness(struct state *st);
 void submit_v2_liveness_exchange(struct ike_sa *ike, so_serial_t);
 
+extern const struct v2_exchange v2_INFORMATIONAL_liveness_exchange;
+
 #endif
