@@ -16,6 +16,7 @@
  * Copyright (C) 2017 Vukasin Karadzic <vukasin.karadzic@gmail.com>
  * Copyright (C) 2017 Mayank Totale <mtotale@gmail.com>
  * Copyright (C) 2020 Yulia Kuzovkova <ukuzovkova@gmail.com>
+ * Copyright (C) 2020 Nupur Agrawal <nupur202000@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -47,6 +48,7 @@
 #include "iface.h"
 #include "ip_protocol.h"
 #include "ikev2_send.h"
+#include "ikev2_notification.h"
 
 /*
  * Determine the IKE version we will use for the IKE packet
@@ -1106,16 +1108,15 @@ static bool record_outbound_fragment(struct logger *logger,
 				     unsigned int number, unsigned int total,
 				     const char *desc)
 {
-	/* make sure HDR is at start of a clean buffer */
-	unsigned char frag_buffer[PMAX(MIN_MAX_UDP_DATA_v4, MIN_MAX_UDP_DATA_v6)];
-	struct pbs_out frag_stream = open_pbs_out("reply frag packet",
-						  frag_buffer, sizeof(frag_buffer),
-						  logger);
+	struct fragment_pbs_out message_fragment;
+	if (!open_fragment_pbs_out("fragment", &message_fragment, logger)) {
+		return false;
+	}
 
 	/* HDR out */
 
 	struct pbs_out body;
-	if (!pbs_out_struct(&frag_stream, &isakmp_hdr_desc, hdr, sizeof(*hdr), &body))
+	if (!pbs_out_struct(&message_fragment.pbs, &isakmp_hdr_desc, hdr, sizeof(*hdr), &body))
 		return false;
 
 	/*
@@ -1185,7 +1186,7 @@ static bool record_outbound_fragment(struct logger *logger,
 	}
 
 	close_output_pbs(&body);
-	close_output_pbs(&frag_stream);
+	close_output_pbs(&message_fragment.pbs);
 
 	if (!encrypt_v2SK_payload(&skf)) {
 		llog(RC_LOG, logger, "error encrypting fragment %u", number);
@@ -1193,7 +1194,7 @@ static bool record_outbound_fragment(struct logger *logger,
 	}
 
 	dbg("recording fragment %u", number);
-	record_v2_outgoing_fragment(&frag_stream, desc, fragp);
+	record_v2_outgoing_fragment(&message_fragment.pbs, desc, fragp);
 	return true;
 }
 
@@ -1219,7 +1220,7 @@ static bool record_outbound_fragments(const struct pbs_out *body,
 	 * function above be left to do the computation on-the-fly?
 	 */
 
-	unsigned int len = endpoint_type(&sk->ike->sa.st_remote_endpoint)->ikev2_max_fragment_size;
+	unsigned int len = endpoint_info(sk->ike->sa.st_remote_endpoint)->ikev2_max_fragment_size;
 
 	/*
 	 * If we are doing NAT, so that the other end doesn't mistake
@@ -1338,7 +1339,7 @@ static stf_status record_v2SK_message(struct pbs_out *msg,
 	/* IPv4 and IPv6 have different fragment sizes */
 	if (sk->ike->sa.st_iface_endpoint->io->protocol == &ip_protocol_udp &&
 	    sk->ike->sa.st_v2_ike_fragmentation_enabled &&
-	    len >= endpoint_type(&sk->ike->sa.st_remote_endpoint)->ikev2_max_fragment_size) {
+	    len >= endpoint_info(sk->ike->sa.st_remote_endpoint)->ikev2_max_fragment_size) {
 		if (!record_outbound_fragments(msg, sk, what, outgoing_fragments)) {
 			dbg("record outbound fragments failed");
 			return STF_INTERNAL_ERROR;
@@ -1397,14 +1398,29 @@ bool open_v2_message(const char *story,
 
 	switch (security) {
 	case ENCRYPTED_PAYLOAD:
-		/* never encrypt an IKE_SA_INIT exchange */
-		if (exchange_type == ISAKMP_v2_IKE_SA_INIT) {
+		/*
+		 * Never encrypt the initial exchange (peer requires
+		 * Nr in response before it can compute keys and
+		 * decrypt it).
+		 */
+		if (exchange_type == ISAKMP_v2_IKE_SA_INIT ||
+		    exchange_type == ISAKMP_v2_IKE_SESSION_RESUME) {
+			name_buf nb;
 			llog_pexpect(message->logger, HERE,
-				     "exchange type IKE_SA_INIT is invalid for encrypted notification");
+				     "exchange type %s is invalid for encrypted notification",
+				     str_enum_short(&ikev2_exchange_names, exchange_type, &nb));
 			return false;
 		}
-		/* check things are at least protected */
-		if (!pexpect(ike != NULL && ike->sa.hidden_variables.st_skeyid_calculated)) {
+		/*
+		 * Encrypting requires an IKE SA for the keys.
+		 */
+		if (PBAD(message->logger, ike == NULL)) {
+			return false;
+		}
+		/*
+		 * Encryption requires the IKE SA to have keys.
+		 */
+		if (!PEXPECT(message->logger, ike->sa.hidden_variables.st_skeyid_calculated)) {
 			return false;
 		}
 		if (!open_body_v2SK_payload(&message->body, ike, logger, &message->sk)) {
@@ -1418,8 +1434,25 @@ bool open_v2_message(const char *story,
 		}
 		break;
 	case UNENCRYPTED_PAYLOAD:
-		/* unsecured payload when secured allowed? */
-		pexpect(ike == NULL || !ike->sa.hidden_variables.st_skeyid_calculated);
+		/*
+		 * Only send the initial exchange unencrypted (peer
+		 * needs unencrypted response containing Nr before it
+		 * can generate keys; all following exchanges should
+		 * be encrypted).
+		 *
+		 * Note that .ST_SKEYID_CALCULATED may be set. With
+		 * IKE_SESSION_RESUME, it is computed before building
+		 * the response (for IKE_SA_INIT it is only computed
+		 * upon the arrival of the IKE_AUTH message).
+		 */
+		if (exchange_type != ISAKMP_v2_IKE_SA_INIT &&
+		    exchange_type != ISAKMP_v2_IKE_SESSION_RESUME) {
+			name_buf nb;
+			llog_pexpect(message->logger, HERE,
+				     "exchange type %s is invalid for encrypted notification",
+				     str_enum_short(&ikev2_exchange_names, exchange_type, &nb));
+			return false;
+		}
 		message->pbs = &message->body;
 		break;
 	}
