@@ -48,6 +48,7 @@
 #include "ikev2.h"
 #include "ikev2_states.h"
 #include "ikev2_informational.h"
+#include "ikev2_notification.h"
 
 static emit_v2_INFORMATIONAL_request_payload_fn add_mobike_payloads; /* type check */
 
@@ -61,7 +62,7 @@ static bool add_mobike_response_payloads(shunk_t cookie2, struct msg_digest *md,
 static bool update_mobike_endpoints(struct ike_sa *ike, const struct msg_digest *md)
 {
 	struct connection *c = ike->sa.st_connection;
-	const struct ip_info *afi = endpoint_type(&md->iface->local_endpoint);
+	const struct ip_info *afi = endpoint_info(md->iface->local_endpoint);
 
 	/*
 	 * AA_201705 is this the right way to find Child SA(s)?
@@ -83,7 +84,7 @@ static bool update_mobike_endpoints(struct ike_sa *ike, const struct msg_digest
 	}
 
 	/* check for all conditions before updating IPsec SA's */
-	if (afi != address_type(&c->remote->host.addr)) {
+	if (afi != address_info(c->remote->host.addr)) {
 		llog_sa(RC_LOG, ike,
 			  "MOBIKE: AF change switching between v4 and v6 not supported");
 		return false;
@@ -391,9 +392,13 @@ void record_newaddr(ip_address *ip, char *a_type)
 	dbg("XFRM RTM_NEWADDR %s %s", str_address(ip, &ip_str), a_type);
 	struct state_filter sf = {
 		.ike_version = IKEv2,
-		.where = HERE,
+		.search = {
+			.order = NEW2OLD,
+			.verbose.logger = &global_logger,
+			.where = HERE,
+		},
 	};
-	while (next_state(NEW2OLD, &sf)) {
+	while (next_state(&sf)) {
 
 		if (!IS_IKE_SA(sf.st)) {
 			continue;
@@ -429,9 +434,13 @@ void record_deladdr(ip_address *ip, char *a_type)
 	dbg("XFRM RTM_DELADDR %s %s", str_address(ip, &ip_str), a_type);
 	struct state_filter sf = {
 		.ike_version = IKEv2,
-		.where = HERE,
+		.search = {
+			.order = NEW2OLD,
+			.verbose.logger = &global_logger,
+			.where = HERE,
+		},
 	};
-	while (next_state(NEW2OLD, &sf)) {
+	while (next_state(&sf)) {
 
 		if (!IS_IKE_SA(sf.st)) {
 			continue;
@@ -501,13 +510,11 @@ static stf_status process_v2_INFORMATIONAL_mobike_response(struct ike_sa *ike,
 
 static const struct v2_transition v2_INFORMATIONAL_mobike_initiate_transition = {
 	.story = "MOBIKE",
-	.from = { &state_v2_ESTABLISHED_IKE_SA, },
 	.to = &state_v2_ESTABLISHED_IKE_SA,
 };
 
 static const struct v2_transition v2_INFORMATIONAL_mobike_response_transition[] = {
 	{ .story      = "Informational Response",
-	  .from = { &state_v2_ESTABLISHED_IKE_SA, },
 	  .to = &state_v2_ESTABLISHED_IKE_SA,
 	  .exchange   = ISAKMP_v2_INFORMATIONAL,
 	  .recv_role  = MESSAGE_RESPONSE,
@@ -526,7 +533,8 @@ const struct v2_exchange v2_INFORMATIONAL_mobike_exchange = {
 	.type = ISAKMP_v2_INFORMATIONAL,
 	.subplot = "MOBIKE probe",
 	.secured = true,
-	.initiate = &v2_INFORMATIONAL_mobike_initiate_transition,
+	.initiate.from = { &state_v2_ESTABLISHED_IKE_SA, },
+	.initiate.transition = &v2_INFORMATIONAL_mobike_initiate_transition,
 	.response = &v2_INFORMATIONAL_mobike_response_transitions,
 };
 
