@@ -24,7 +24,9 @@
 struct state;
 struct ike_sa;
 struct msg_digest;
-struct v2_state_transition;
+struct v2_transition;
+struct v2_transitions;
+struct v2_exchange;
 enum message_role;
 
 /*
@@ -40,21 +42,58 @@ enum message_role;
 struct v2_msgid_window {
 	monotime_t last_sent;  /* sent a message */
 	monotime_t last_recv;  /* received a message */
-	intmax_t sent;
-	intmax_t recv;
-	unsigned recv_frags;	/* number of fragments making up incoming message */
 	/*
-	 * The Message ID for the IKE SA's's in-progress exchange(s).
-	 * If no exchange is in progress then its value is -1.
+	 * .sent: last outbound message sent
+	 * .recv: last inbound message processed
+	 * .wip: message being processed; or -1
 	 *
-	 * The INITIATOR's Message ID is valid from the time the
-	 * request is sent (possibly earlier?) through to when the
-	 * response is received.
+	 * Initiator:
 	 *
-	 * The RESPONDER Message ID is valid for the period that the
-	 * state is processing the request.
+	 * Idle:       .sent:N   .recv:N   .wip:-1   .state=_IRn?
+	 * Initiating: .sent:N   .recv:N   .wip:N+1  .state=_IRn?
+	 * Initiated:  .sent:N+1 .recv:N   .wip:-1   .state=_In+1
+	 *
+	 * Responder:
+	 *
+	 * Idle:       .sent:N   .recv:N   .wip:-1   .state=_Rn?
+	 * Responding: .sent:N   .recv:N   .wip:N+1  .state=_Rn?
+	 * Responded:  .sent:N+1 .recv:N+1 .wip:N+1  .state=_Rn+1
+	 *
+	 * Initiator:
+	 *
+	 * Response:   .sent:N+1 .recv:N   .wip:N+1  .state=_In+1
+	 * (after packet has been assembled)
+	 * Idle    :   .sent:N+1 .recv:N+1 .wip:-1   .state=_IRn+1
+	 *
+	 * XXX: should .wip be set _while_ protected and verified
+	 * fragments are being accumulated.
+	 *
+	 * When retransmitting a response, the RFC says to also check
+	 * that the fragments match.  Hence .recv_frags which
+	 * out-lives the .incomming_fragments field.
 	 */
-	intmax_t wip;
+	intmax_t sent;		/* starts with -1 */
+	intmax_t recv;		/* starts with -1 */
+	unsigned recv_frags;	/* number of fragments in last .recv */
+	intmax_t wip;		/* >=0 when busy */
+	/*
+	 * Fragments:
+	 *
+	 * Incoming fragments are accumulated and then re-assembled
+	 * into a single message.  Once assembled the structure is
+	 * deleted.  Note that this all happens before .recv and
+	 * .recv_frags (used to detect the need to retransmit a
+	 * response) are set.
+	 *
+	 * Outgoing fragments are used when retransmitting.  A todo is
+	 * to delete them after a few minutes.
+	 */
+	struct v2_incoming_fragments *incoming_fragments;
+	struct v2_outgoing_fragment *outgoing_fragments;
+	/*
+	 * The exchange being worked on by the initiator.
+	 */
+	const struct v2_exchange *exchange;
 	/*
 	 * The SA being worked on by the exchange.
 	 *
@@ -63,6 +102,12 @@ struct v2_msgid_window {
 	 * rekeyd by a CREATE_CHILD_SA exchange.
 	 */
 	struct child_sa *wip_sa;
+	/*
+	 * The IKE or Child SA being deleted using a Delete exchange.
+	 * Crossing deletes and larval Child SAs mean that the
+	 * corresponding state may not exist.
+	 */
+	so_serial_t dead_sa;
 };
 
 struct v2_msgid_windows {
@@ -93,9 +138,17 @@ bool v2_msgid_request_pending(struct ike_sa *ike);
  * record 'n' send has been eliminated.
  */
 
-void v2_msgid_start(struct ike_sa *ike, const struct msg_digest *md);
-void v2_msgid_cancel(struct ike_sa *ike, const struct msg_digest *md);
-void v2_msgid_finish(struct ike_sa *ike, const struct msg_digest *md);
+void v2_msgid_start(struct ike_sa *ike, const struct v2_exchange *,
+		    const struct msg_digest *md, where_t where);
+void v2_msgid_cancel(struct ike_sa *ike, const struct msg_digest *md, where_t where);
+void v2_msgid_finish(struct ike_sa *ike, const struct msg_digest *md, where_t where);
+
+/*
+ * This is a hack for code that forces sending an additional request
+ * even though the request window is full.
+ */
+
+void v2_msgid_start_record_n_send(struct ike_sa *ike, const struct v2_exchange *exchange);
 
 /*
  * Handle multiple initiators trying to send simultaneously.
@@ -113,15 +166,18 @@ void v2_msgid_finish(struct ike_sa *ike, const struct msg_digest *md);
  * message id up front, but only when one was available?
  */
 
-void v2_msgid_queue_initiator(struct ike_sa *ike, struct child_sa *child/*optional*/,
-			      const struct v2_state_transition *transition);
+void v2_msgid_queue_exchange(struct ike_sa *ike,
+			     struct child_sa *child/*optional*/,
+			     const struct v2_exchange *exchange);
 
 void v2_msgid_migrate_queue(struct ike_sa *from, struct child_sa *to);
 
 void v2_msgid_schedule_next_initiator(struct ike_sa *ike);
 
 void dbg_v2_msgid(struct ike_sa *ike, const char *msg, ...) PRINTF_LIKE(2);
-void fail_v2_msgid_where(where_t where, struct ike_sa *ike, const char *fmt, ...) PRINTF_LIKE(3);
-#define fail_v2_msgid(IKE, FMT, ...) fail_v2_msgid_where(HERE, IKE, FMT,##__VA_ARGS__)
+void llog_pexpect_v2_msgid_where(where_t where, struct ike_sa *ike, const char *fmt, ...) PRINTF_LIKE(3);
+#define llog_pexpect_v2_msgid(IKE, FMT, ...) llog_pexpect_v2_msgid_where(HERE, IKE, FMT, ##__VA_ARGS__)
+
+struct v2_msgid_window *v2_msgid_window(struct ike_sa *ike, enum message_role message_role);
 
 #endif
