@@ -11,6 +11,7 @@
  * Copyright (C) 2015-2019 Andrew Cagney <cagney@gnu.org>
  * Copyright (C) 2017 Vukasin Karadzic <vukasin.karadzic@gmail.com>
  * Copyright (C) 2020 Yulia Kuzovkova <ukuzovkova@gmail.com>
+ * Copyright (C) 2020 Nupur Agrawal <nupur202000@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -61,16 +62,18 @@ diag_t ikev2_calculate_psk_sighash(enum perspective perspective,
 				   const chunk_t firstpacket,
 				   struct crypt_mac *sighash)
 {
+	struct logger *logger = ike->sa.logger;
 	const struct connection *c = ike->sa.st_connection;
 	*sighash = empty_mac;
 	passert(authby == AUTH_EAPONLY || authby == AUTH_PSK || authby == AUTH_NULL);
 
 	enum_buf pb;
 	enum_buf an;
-	ldbg_sa(ike, "%s() called for %s to %s PSK with authby=%s",
-		__func__, ike->sa.st_state->name,
-		str_enum_short(&perspective_names, perspective, &pb),
-		str_enum(&keyword_auth_names, authby, &an));
+	ldbg(logger, "%s() called for %s to %s PSK with authby=%s resume=%s",
+	     __func__, ike->sa.st_state->name,
+	     str_enum_short(&perspective_names, perspective, &pb),
+	     str_enum(&keyword_auth_names, authby, &an),
+	     bool_str(ike->sa.st_v2_resume_session != NULL));
 
 	/* this is the IKE_AUTH exchange, so a given */
 	passert(ike->sa.hidden_variables.st_skeyid_calculated);
@@ -90,22 +93,47 @@ diag_t ikev2_calculate_psk_sighash(enum perspective perspective,
 	 * Pick nullauth_pss, nonce, and nonce_name suitable for
 	 * (state, verify).
 	 *
-	 * RFC-7619:
+	 * For pre-shared key:
 	 *
-	 * When using the NULL Authentication Method, the content of
-	 * the AUTH payload is computed using the syntax of pre-shared
-	 * secret authentication, described in Section 2.15 of
-	 * [RFC7296].  The values SK_pi and SK_pr are used as shared
-	 * secrets for the content of the AUTH payloads generated by
-	 * the initiator and the responder respectively.
+	 *   For the initiator:
+	 *
+	 *     AUTH = prf( prf(Shared Secret, "Key Pad for IKEv2"),
+	 *                 <InitiatorSignedOctets>)
+	 *   For the responder:
+	 *
+	 *     AUTH = prf( prf(Shared Secret, "Key Pad for IKEv2"),
+	 *                 <ResponderSignedOctets>)
+	 *
+	 * The NULL Authentication Method (RFC-7619):
+	 *
+	 *   When using the NULL Authentication Method, the content of
+	 *   the AUTH payload is computed using the syntax of
+	 *   pre-shared secret authentication, described in Section
+	 *   2.15 of [RFC7296].  The values SK_pi and SK_pr are used
+	 *   as shared secrets for the content of the AUTH payloads
+	 *   generated by the initiator and the responder
+	 *   respectively.
+	 *
+	 * I'm guessing this means:
+	 *
+	 *   prf(prf(SK_px, "Key Pad for IKEv2"), signed octets)
 	 *
 	 * We have SK_pi/SK_pr as PK11SymKey in st_skey_pi_nss and
 	 * st_skey_pr_nss.
+
+	 * Session resume (RFC-5723):
+	 *
+	 *   The value of the AUTH payload is derived in a manner
+	 *   similar to the usage of IKEv2 pre-shared secret
+	 *   authentication:
+	 *
+         *     AUTH = prf(SK_px, <message octets>)
 	 */
 
 	chunk_t nullauth_pss;
 	const chunk_t *nonce;
 	const char *nonce_name;
+	PK11SymKey *SK_px;
 
 	switch (ike->sa.st_sa_role) {
 	case SA_INITIATOR:
@@ -115,12 +143,14 @@ diag_t ikev2_calculate_psk_sighash(enum perspective perspective,
 			nonce_name = "Ni: initiator re-generating hash from responder";
 			nonce = &ike->sa.st_ni;
 			nullauth_pss = ike->sa.st_skey_chunk_SK_pr;
+			SK_px = ike->sa.st_skey_pr_nss;
 			break;
 		case LOCAL_PERSPECTIVE:
 			/* we are initiator sending PSK */
 			nonce_name = "Nr: initiator generating hash for responder";
 			nonce = &ike->sa.st_nr;
 			nullauth_pss = ike->sa.st_skey_chunk_SK_pi;
+			SK_px = ike->sa.st_skey_pi_nss;
 			break;
 		case NO_PERSPECTIVE:
 		default:
@@ -135,12 +165,14 @@ diag_t ikev2_calculate_psk_sighash(enum perspective perspective,
 			nonce_name = "Nr: responder re-generating hash from initiator";
 			nonce = &ike->sa.st_nr;
 			nullauth_pss = ike->sa.st_skey_chunk_SK_pi;
+			SK_px = ike->sa.st_skey_pi_nss;
 			break;
 		case LOCAL_PERSPECTIVE:
 			/* we are responder sending PSK */
 			nonce_name = "Ni: create: responder generating hash for initiator";
 			nonce = &ike->sa.st_ni;
 			nullauth_pss = ike->sa.st_skey_chunk_SK_pr;
+			SK_px = ike->sa.st_skey_pr_nss;
 			break;
 		case NO_PERSPECTIVE:
 		default:
@@ -152,6 +184,21 @@ diag_t ikev2_calculate_psk_sighash(enum perspective perspective,
 		bad_case(ike->sa.st_sa_role);
 	}
 
+	/*
+	 * IKE_SESSION_RESUME has its own idea of how to do crypto.
+	 */
+
+	if (ike->sa.st_v2_resume_session != NULL) {
+		/*
+		 * RFC 5723 6.2:
+		 * AUTH = prf(SK_px, <message octets>)
+		 */
+		(*sighash) = ikev2_psk_resume(ike->sa.st_oakley.ta_prf, SK_px,
+					      firstpacket, logger);
+		PASSERT(logger, sighash->len > 0);
+		return NULL;
+	}
+
 	/* pick pss */
 
 	shunk_t pss;
@@ -170,7 +217,7 @@ diag_t ikev2_calculate_psk_sighash(enum perspective perspective,
 		 * XXX: same PSK used for both local and remote end,
 		 * so peer doesn't apply?
 		 */
-		const chunk_t *psk = get_connection_psk(c);
+		const struct secret_preshared_stuff *psk = get_connection_psk(c);
 		if (psk == NULL) {
 			id_buf idb;
 			return diag("authentication failed: no PSK found for '%s'",
@@ -306,16 +353,20 @@ diag_t verify_v2AUTH_and_log_using_psk(enum keyword_auth authby,
 		     "?"));
 		/* all methods log this string */
 		jam_string(buf, "authenticated peer ");
-		/* what was in the AUTH payload */
-		/* XXX: log prf(prf(hash based on null or secret)) how? */
-		/* now it was authenticated */
-		jam_string(buf, "using authby=");
-		jam_enum(buf, &keyword_auth_names, authby);
-		jam_string(buf, " and ");
-		jam_enum(buf, &ike_id_type_names, ike->sa.st_connection->remote->host.id.kind);
-		jam_string(buf, " '");
-		jam_id_bytes(buf, &ike->sa.st_connection->remote->host.id, jam_raw_bytes);
-		jam_string(buf, "'");
+		if (ike->sa.st_v2_resume_session != NULL) {
+			jam_string(buf, "using authby=session-resume");
+		} else {
+			/* what was in the AUTH payload */
+			/* XXX: log prf(prf(hash based on null or secret)) how? */
+			/* now it was authenticated */
+			jam_string(buf, "using authby=");
+			jam_enum(buf, &keyword_auth_names, authby);
+			jam_string(buf, " and ");
+			jam_enum(buf, &ike_id_type_names, ike->sa.st_connection->remote->host.id.kind);
+			jam_string(buf, " '");
+			jam_id_bytes(buf, &ike->sa.st_connection->remote->host.id, jam_raw_bytes);
+			jam_string(buf, "'");
+		}
 	}
 	return NULL;
 }
