@@ -14,6 +14,7 @@
  * Copyright (C) 2015-2019 Andrew Cagney
  * Copyright (C) 2016-2018 Antony Antony <appu@phenome.org>
  * Copyright (C) 2017 Sahana Prasad <sahana.prasad07@gmail.com>
+ * Copyright (C) 2020 Nupur Agrawal <nupur202000@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -27,8 +28,8 @@
  *
  */
 
-#define ldbg_ft(LOGGER, FORMAT, ...)		\
-	ldbg(LOGGER, "ft: %*s"FORMAT, indent*2, "", ##__VA_ARGS__)
+#define vdbg_ft(FORMAT, ...)			\
+	vdbg("ft: "FORMAT, ##__VA_ARGS__)
 
 #include "defs.h"
 #include "state.h"
@@ -37,19 +38,21 @@
 #include "ikev2.h"
 #include "log.h"
 #include "connections.h"
-#include "ikev2_notify.h"
+#include "ikev2_notification.h"
 #include "ikev2_retransmit.h"
 #include "ikev2_ike_sa_init.h"
 #include "ikev2_ike_auth.h"
 #include "ikev2_ike_sa_init.h"
 #include "ikev2_ike_intermediate.h"
 #include "ikev2_informational.h"
+#include "ikev2_liveness.h"
 #include "ikev2_cookie.h"
 #include "ikev2_redirect.h"
 #include "ikev2_eap.h"
 #include "ikev2_create_child_sa.h"
 #include "ikev2_delete.h"
 #include "log_limiter.h"		/* for payload_errors_log_limiter; */
+#include "verbose.h"
 
 struct ikev2_payload_errors {
 	bool bad;
@@ -59,10 +62,13 @@ struct ikev2_payload_errors {
 	v2_notification_t notification;
 };
 
-static void llog_v2_payload_errors(struct logger *logger, struct msg_digest *md,
+static void llog_v2_payload_errors(struct logger *logger, const struct msg_digest *md,
 				   const struct ikev2_payload_errors *errors);
 
-static struct ikev2_payload_errors ikev2_verify_payloads(struct msg_digest *md,
+static void jam_v2_payload_errors(struct jambuf *buf, const struct msg_digest *md,
+				  const struct ikev2_payload_errors *errors);
+
+static struct ikev2_payload_errors ikev2_verify_payloads(const struct msg_digest *md,
 							 const struct payload_summary *summary,
 							 const struct ikev2_expected_payloads *payloads);
 
@@ -80,6 +86,18 @@ static struct ikev2_payload_errors ikev2_verify_payloads(struct msg_digest *md,
 		##__VA_ARGS__,						\
 	}
 
+static void jam_expected_payloads(struct jambuf *buf,
+				  const struct ikev2_expected_payloads *payloads)
+{
+	jam_string(buf, "(");
+	jam_lset_short(buf, &ikev2_payload_names, "+",
+		       payloads->required);
+	jam_string(buf, ")[");
+	jam_lset_short(buf, &ikev2_payload_names, "_",
+		       payloads->optional);
+	jam_string(buf, "]");
+}
+
 /*
  * From RFC 5996 syntax: [optional] and {encrypted}
  *
@@ -149,7 +167,6 @@ static struct ikev2_payload_errors ikev2_verify_payloads(struct msg_digest *md,
 
 static const struct v2_transition v2_REKEY_IKE_I0_transition = {
 	.story      = "initiate rekey IKE_SA (CREATE_CHILD_SA)",
-	.from = { &state_v2_REKEY_IKE_I0, },
 	.to = &state_v2_REKEY_IKE_I1,
 	.exchange   = ISAKMP_v2_CREATE_CHILD_SA,
 };
@@ -158,7 +175,6 @@ V2_CHILD(REKEY_IKE_I0, "STATE_V2_REKEY_IKE_I0", CAT_IGNORE);
 
 static const struct v2_transition v2_REKEY_IKE_R0_transition = {
 	.story      = "process rekey IKE SA request (CREATE_CHILD_SA)",
-	.from = { &state_v2_REKEY_IKE_R0, },
 	.to = &state_v2_ESTABLISHED_IKE_SA,
 	.exchange   = ISAKMP_v2_CREATE_CHILD_SA,
 };
@@ -167,7 +183,6 @@ V2_CHILD(REKEY_IKE_R0, "STATE_V2_REKEY_IKE_R0", CAT_OPEN_IKE_SA);
 
 static const struct v2_transition v2_REKEY_IKE_I1_transition = {
 	.story      = "process rekey IKE SA response (CREATE_CHILD_SA)",
-	.from = { &state_v2_REKEY_IKE_I1, },
 	.to = &state_v2_ESTABLISHED_IKE_SA,
 	.exchange   = ISAKMP_v2_CREATE_CHILD_SA,
 };
@@ -180,7 +195,6 @@ V2_CHILD(REKEY_IKE_I1, "sent CREATE_CHILD_SA request to rekey IKE SA", CAT_OPEN_
 
 static const struct v2_transition v2_REKEY_CHILD_I0_transition = {
 	.story      = "initiate rekey Child SA (CREATE_CHILD_SA)",
-	.from = { &state_v2_REKEY_CHILD_I0, },
 	.to = &state_v2_REKEY_CHILD_I1,
 	.exchange   = ISAKMP_v2_CREATE_CHILD_SA,
 };
@@ -189,7 +203,6 @@ V2_CHILD(REKEY_CHILD_I0, "STATE_V2_REKEY_CHILD_I0", CAT_IGNORE);
 
 static const struct v2_transition v2_REKEY_CHILD_R0_transition = {
 	.story      = "process rekey Child SA request (CREATE_CHILD_SA)",
-	.from = { &state_v2_REKEY_CHILD_R0, },
 	.to = &state_v2_ESTABLISHED_CHILD_SA,
 	.exchange   = ISAKMP_v2_CREATE_CHILD_SA,
 };
@@ -198,7 +211,6 @@ V2_CHILD(REKEY_CHILD_R0, "STATE_V2_REKEY_CHILD_R0", CAT_OPEN_CHILD_SA);
 
 static const struct v2_transition v2_REKEY_CHILD_I1_transition = {
 	.story      = "process rekey Child SA response (CREATE_CHILD_SA)",
-	.from = { &state_v2_REKEY_CHILD_I1, },
 	.to = &state_v2_ESTABLISHED_CHILD_SA,
 	.flags = { .release_whack = true, },
 	.exchange   = ISAKMP_v2_CREATE_CHILD_SA,
@@ -212,7 +224,6 @@ V2_CHILD(REKEY_CHILD_I1, "sent CREATE_CHILD_SA request to rekey IPsec SA", CAT_O
 
 static const struct v2_transition v2_NEW_CHILD_I0_transition = {
 	.story      = "initiate create Child SA (CREATE_CHILD_SA)",
-	.from = { &state_v2_NEW_CHILD_I0, },
 	.to = &state_v2_NEW_CHILD_I1,
 	.exchange   = ISAKMP_v2_CREATE_CHILD_SA,
 };
@@ -221,7 +232,6 @@ V2_CHILD(NEW_CHILD_I0, "STATE_V2_NEW_CHILD_I0", CAT_IGNORE);
 
 static const struct v2_transition v2_NEW_CHILD_R0_transition = {
 	.story      = "process create Child SA request (CREATE_CHILD_SA)",
-	.from = { &state_v2_NEW_CHILD_R0, },
 	.to = &state_v2_ESTABLISHED_CHILD_SA,
 	.exchange   = ISAKMP_v2_CREATE_CHILD_SA,
 };
@@ -231,7 +241,6 @@ V2_CHILD(NEW_CHILD_R0, "STATE_V2_NEW_CHILD_R0",
 
 static const struct v2_transition v2_NEW_CHILD_I1_transition = {
 	.story      = "process create Child SA response (CREATE_CHILD_SA)",
-	.from = { &state_v2_NEW_CHILD_I1, },
 	.to = &state_v2_ESTABLISHED_CHILD_SA,
 	.exchange   = ISAKMP_v2_CREATE_CHILD_SA,
 };
@@ -245,10 +254,18 @@ V2_CHILD(NEW_CHILD_I1, "sent CREATE_CHILD_SA request for new IPsec SA",
 
 V2_STATE(ESTABLISHED_IKE_SA, "established IKE SA",
 	 CAT_ESTABLISHED_IKE_SA, /*secured*/true,
-	 /* informational */
+	 /*
+	  * Informational.  Order is important.
+	  *
+	  * The liveness probe, which strictly matches an empty
+	  * message must be before the generic informational exchange.
+	  * Otherwise the generic exchange, which can accept an empty
+	  * message, would do the processing.
+	  */
 	 &v2_INFORMATIONAL_v2DELETE_exchange,
 	 &v2_INFORMATIONAL_v2N_REDIRECT_exchange,
-	 &v2_INFORMATIONAL_exchange,
+	 &v2_INFORMATIONAL_liveness_exchange,
+	 &v2_INFORMATIONAL_exchange, /* last; matches mobike! */
 	 /*
 	  * Create/Rekey IKE/Child SAs.  Danger: order is important.
 	  */
@@ -265,11 +282,19 @@ V2_STATE(ZOMBIE, "deleted state", CAT_ESTABLISHED_IKE_SA, /*secured*/true);
 
 static const struct finite_state *v2_states[] = {
 #define S(KIND, ...) [STATE_V2_##KIND - STATE_IKEv2_FLOOR] = &state_v2_##KIND
+	S(UNSECURED_R),
+	/* includes larval states */
 	S(IKE_SA_INIT_I0),
 	S(IKE_SA_INIT_I),
-	S(IKE_SA_INIT_R0),
 	S(IKE_SA_INIT_R),
 	S(IKE_SA_INIT_IR),
+	/* includes larval states */
+	S(IKE_SESSION_RESUME_I0),
+	S(IKE_SESSION_RESUME_I),
+	S(IKE_SESSION_RESUME_R0),
+	S(IKE_SESSION_RESUME_R),
+	S(IKE_SESSION_RESUME_IR),
+	/* comes after above */
 	S(IKE_INTERMEDIATE_I),
 	S(IKE_INTERMEDIATE_R),
 	S(IKE_INTERMEDIATE_IR),
@@ -322,7 +347,7 @@ static const struct finite_state *v2_states[] = {
 static const lset_t everywhere_payloads = v2P(N) | v2P(V);	/* can appear in any packet */
 static const lset_t repeatable_payloads = v2P(N) | v2P(D) | v2P(CP) | v2P(V) | v2P(CERT) | v2P(CERTREQ);	/* if one can appear, many can appear */
 
-struct ikev2_payload_errors ikev2_verify_payloads(struct msg_digest *md,
+struct ikev2_payload_errors ikev2_verify_payloads(const struct msg_digest *md,
 						  const struct payload_summary *summary,
 						  const struct ikev2_expected_payloads *payloads)
 {
@@ -336,14 +361,20 @@ struct ikev2_payload_errors ikev2_verify_payloads(struct msg_digest *md,
 		seen |= v2P(SK);
 	}
 
+	/*
+	 * LIVENESS really does want to only match an empty message
+	 * and, hence, does not want everywhere_payloads in the
+	 * responder.
+	 */
+	lset_t opt_payloads = (payloads->exact_match ? payloads->optional :
+			       payloads->optional | everywhere_payloads);
 	lset_t req_payloads = payloads->required;
-	lset_t opt_payloads = payloads->optional;
 
 	struct ikev2_payload_errors errors = {
 		.bad = false,
 		.excessive = summary->repeated & ~repeatable_payloads,
 		.missing = req_payloads & ~seen,
-		.unexpected = seen & ~req_payloads & ~opt_payloads & ~everywhere_payloads,
+		.unexpected = seen & ~req_payloads & ~opt_payloads,
 	};
 
 	if ((errors.excessive | errors.missing | errors.unexpected) != LEMPTY) {
@@ -361,41 +392,50 @@ struct ikev2_payload_errors ikev2_verify_payloads(struct msg_digest *md,
 	return errors;
 }
 
-static const struct v2_transition *find_v2_transition(struct logger *logger, unsigned indent,
+static const struct v2_transition *find_v2_transition(struct verbose verbose,
+						      const struct msg_digest *md,
 						      const struct v2_transitions *transitions,
-						      struct msg_digest *md,
 						      struct ikev2_payload_errors *message_payload_status,
 						      struct ikev2_payload_errors *encrypted_payload_status)
 {
+	const unsigned level = verbose.level;
+	enum message_role role = v2_msg_role(md);
+
 	FOR_EACH_ITEM(transition, transitions) {
 
-		ldbg_ft(logger, "trying %s ...", transition->story);
+		verbose.level = level;
+		vdbg_ft("trying %s ...", transition->story);
+		verbose.level++;
 
 		/* message type? */
 		if (transition->exchange != md->hdr.isa_xchg) {
 			enum_buf xb;
-			ldbg_ft(logger, "  exchange type does not match %s",
+			vdbg_ft("exchange type does not match %s",
 				str_enum_short(&ikev2_exchange_names, transition->exchange, &xb));
 			continue;
 		}
 
 		/* role? */
-		if (transition->recv_role != v2_msg_role(md)) {
+		if (transition->recv_role != role) {
 			enum_buf rb;
-			ldbg_ft(logger, "  message role does not match %s",
+			vdbg_ft("message role does not match %s",
 				str_enum_short(&message_role_names, transition->recv_role, &rb));
 			continue;
 		}
 
 		/* message payloads */
-		if (!PEXPECT(logger, md->message_payloads.parsed)) {
+		if (!vexpect(md->message_payloads.parsed)) {
 			return NULL;
 		}
 		struct ikev2_payload_errors message_payload_errors
 			= ikev2_verify_payloads(md, &md->message_payloads,
 						&transition->message_payloads);
 		if (message_payload_errors.bad) {
-			ldbg_ft(logger, "  message payloads do not match");
+			LDBGP_JAMBUF(DBG_BASE, verbose.logger, buf) {
+				jam(buf, PRI_VERBOSE, pri_verbose);
+				jam(buf, "message payloads do not match ");
+				jam_expected_payloads(buf, &transition->message_payloads);
+			}
 			/* save error for last pattern!?! */
 			*message_payload_status = message_payload_errors;
 			continue;
@@ -407,8 +447,12 @@ static const struct v2_transition *find_v2_transition(struct logger *logger, uns
 		 * so checking is complete and things have matched.
 		 */
 		if (encrypted_payload_status == NULL) {
-			PEXPECT(logger, (transition->message_payloads.required & v2P(SK)) == LEMPTY);
-			ldbg_ft(logger, "  unsecured message matched");
+			vexpect((transition->message_payloads.required & v2P(SK)) == LEMPTY);
+			LDBGP_JAMBUF(DBG_BASE, verbose.logger, buf) {
+				jam(buf, PRI_VERBOSE, pri_verbose);
+				jam(buf, "unsecured message matched ");
+				jam_expected_payloads(buf, &transition->message_payloads);
+			}
 			return transition;
 		}
 
@@ -416,10 +460,10 @@ static const struct v2_transition *find_v2_transition(struct logger *logger, uns
 		 * Since SK{} payloads are expected, the caller should
 		 * have parsed them.
 		 */
-		if (!PEXPECT(logger, (transition->message_payloads.required & v2P(SK)) != LEMPTY)) {
+		if (!vexpect((transition->message_payloads.required & v2P(SK)) != LEMPTY)) {
 			continue;
 		}
-		if (!PEXPECT(logger, md->encrypted_payloads.parsed)) {
+		if (!vexpect(md->encrypted_payloads.parsed)) {
 			return NULL;
 		}
 
@@ -427,35 +471,83 @@ static const struct v2_transition *find_v2_transition(struct logger *logger, uns
 			= ikev2_verify_payloads(md, &md->encrypted_payloads,
 						&transition->encrypted_payloads);
 		if (encrypted_payload_errors.bad) {
-			ldbg_ft(logger, "  secured payloads do not match");
+			LDBGP_JAMBUF(DBG_BASE, verbose.logger, buf) {
+				jam(buf, PRI_VERBOSE, pri_verbose);
+				jam_string(buf, "secured payloads do not match ");
+				jam_expected_payloads(buf, &transition->encrypted_payloads);
+			}
 			/* save error for last pattern!?! */
 			*encrypted_payload_status = encrypted_payload_errors;
 			continue;
 		}
 
-		ldbg_ft(logger, "  secured message matched");
+		LDBGP_JAMBUF(DBG_BASE, verbose.logger, buf) {
+			jam(buf, PRI_VERBOSE, pri_verbose);
+			jam_string(buf, "secured message matched ");
+			jam_expected_payloads(buf, &transition->message_payloads);
+			jam_string(buf, " ");
+			jam_expected_payloads(buf, &transition->encrypted_payloads);
+		}
 		return transition;
 	}
 
 	return NULL;
 }
 
+static const struct v2_transition *find_v2_exchange_transition(struct verbose verbose,
+							       const struct msg_digest *md,
+							       const struct v2_exchanges *exchanges,
+							       struct ikev2_payload_errors *message_payload_status,
+							       struct ikev2_payload_errors *encrypted_payload_status)
+{
+	const unsigned level = verbose.level;
+	/*
+	 * Does the message match one of the responder state's
+	 * exchanges?
+	 *
+	 * For instance, the IKE_SA_INIT responder state accepts a
+	 * request for the IKE_AUTH and IKE_INTERMEDIATE exchanges.
+	 * With a matching exchange, look for a matching transition.
+	 */
+	FOR_EACH_ITEM(exchangep, exchanges) {
+		const struct v2_exchange *exchange = (*exchangep);
+
+		verbose.level = level;
+		vdbg("trying exchange %s ...", exchange->subplot);
+		verbose.level++;
+
+		if (exchange->type != md->hdr.isa_xchg) {
+			vdbg_ft("wrong exchange type");
+			continue;
+		}
+		const struct v2_transition *t =
+			find_v2_transition(verbose, md,
+					   exchange->responder,
+					   message_payload_status,
+					   encrypted_payload_status);
+		if (t != NULL) {
+			return t;
+		}
+	}
+	return NULL;
+}
+
 const struct v2_transition *find_v2_secured_transition(struct ike_sa *ike,
-						       struct msg_digest *md,
+						       const struct msg_digest *md,
 						       bool *secured_payload_failed)
 {
+	enum message_role role = v2_msg_role(md);
+
 	enum_buf xb, rb;
-	unsigned indent = 0;
-	ldbg_ft(ike->sa.logger, "looking for secured transition matching exchange %s %s ...",
-		str_enum_short(&ikev2_exchange_names, md->hdr.isa_xchg, &xb),
-		str_enum_short(&message_role_names, v2_msg_role(md), &rb));
-	indent = 1;
-	PASSERT(ike->sa.logger, secured_payload_failed != NULL);
+	VERBOSE_DBGP(DBG_BASE, ike->sa.logger,
+		     "looking for secured transition matching exchange %s %s ...",
+		     str_enum_short(&ikev2_exchange_names, md->hdr.isa_xchg, &xb),
+		     str_enum_short(&message_role_names, role, &rb));
+	vassert(secured_payload_failed != NULL);
 
 	struct ikev2_payload_errors message_payload_status = { .bad = false };
 	struct ikev2_payload_errors encrypted_payload_status = { .bad = false };
 
-	enum message_role role = v2_msg_role(md);
 	switch (role) {
 	default:
 	case NO_MESSAGE:
@@ -463,42 +555,24 @@ const struct v2_transition *find_v2_secured_transition(struct ike_sa *ike,
 		break;
 	case MESSAGE_REQUEST:
 	{
-		/*
-		 * Does the message match one of the responder state's
-		 * exchanges?
-		 *
-		 * For instance, the IKE_SA_INIT responder state
-		 * accepts a request for the IKE_AUTH and
-		 * IKE_INTERMEDIATE exchanges.  With a matching
-		 * exchange, look for a matching transition.
-		 */
-		FOR_EACH_ITEM(exchangep, ike->sa.st_state->v2.ike_exchanges) {
-			const struct v2_exchange *exchange = (*exchangep);
-			ldbg_ft(ike->sa.logger, "trying exchange %s ...", exchange->subplot);
-			if (exchange->type != md->hdr.isa_xchg) {
-				ldbg_ft(ike->sa.logger, "  wrong exchange type");
-				continue;
-			}
-			const struct v2_transition *t =
-				find_v2_transition(ike->sa.logger, indent+1,
-						   exchange->responder,
-						   md, &message_payload_status,
-						   &encrypted_payload_status);
-			if (t != NULL) {
-				return t;
-			}
+		const struct v2_transition *t =
+			find_v2_exchange_transition(verbose, md,
+						    ike->sa.st_state->v2.ike_exchanges,
+						    &message_payload_status,
+						    &encrypted_payload_status);
+		if (t != NULL) {
+			return t;
 		}
 		break;
 	}
 	case MESSAGE_RESPONSE:
 	{
 		const struct v2_exchange *exchange = ike->sa.st_v2_msgid_windows.initiator.exchange;
-		PASSERT(ike->sa.logger, exchange != NULL);
-		ldbg_ft(ike->sa.logger, "trying outstanding exchange %s", exchange->subplot);
+		vassert(exchange != NULL);
 		const struct v2_transition *t =
-			find_v2_transition(ike->sa.logger, indent+1,
+			find_v2_transition(verbose, md,
 					   exchange->response,
-					   md, &message_payload_status,
+					   &message_payload_status,
 					   &encrypted_payload_status);
 		if (t != NULL) {
 			return t;
@@ -539,48 +613,84 @@ const struct v2_transition *find_v2_secured_transition(struct ike_sa *ike,
 	return NULL;
 }
 
-const struct v2_transition *find_v2_unsecured_transition(struct logger *logger,
-							 const struct v2_transitions *transitions,
-							 struct msg_digest *md)
+diag_t find_v2_unsecured_request_transition(struct logger *logger,
+					    const struct finite_state *state,
+					    const struct msg_digest *md,
+					    const struct v2_transition **transition)
 {
-	unsigned indent = 0;
+	enum message_role role = v2_msg_role(md);
+
 	enum_buf xb, rb;
-	ldbg_ft(logger, "looking for an unsecured transition matching exchange %s %s ...",
-		str_enum_short(&ikev2_exchange_names, md->hdr.isa_xchg, &xb),
-		str_enum_short(&message_role_names, v2_msg_role(md), &rb));
-	indent = 1;
+	VERBOSE_DBGP(DBG_BASE, logger,
+		     "looking for an unsecured transition matching exchange %s %s ...",
+		     str_enum_short(&ikev2_exchange_names, md->hdr.isa_xchg, &xb),
+		     str_enum_short(&message_role_names, role, &rb));
+	vassert(role == MESSAGE_REQUEST);
 
 	struct ikev2_payload_errors message_payload_status = { .bad = false };
-	const struct v2_transition *t = find_v2_transition(logger, indent,
-							   transitions, md,
-							   &message_payload_status, NULL);
-	if (t != NULL) {
-		return t;
+	(*transition) = find_v2_exchange_transition(verbose, md,
+						    state->v2.ike_exchanges,
+						    &message_payload_status, NULL);
+	if ((*transition) != NULL) {
+		return NULL; /*no-diag*/
 	}
 
 	/*
-	 * A very messed up message - none of the state
-	 * transitions recognized it!.
+	 * A very messed up message - none of the state transitions
+	 * recognized it!.
 	 */
-	if (message_payload_status.bad) {
-		llog_v2_payload_errors(logger, md, &message_payload_status);
+	diag_t d = NULL;
+	JAMBUF(buf) {
+		jam_v2_payload_errors(buf, md, &message_payload_status);
+		d = diag_jambuf(buf);
+	}
+	return d;
+}
+
+diag_t find_v2_unsecured_response_transition(struct ike_sa *ike,
+					     const struct msg_digest *md,
+					     const struct v2_transition **transition)
+{
+	enum_buf xb, rb;
+	enum message_role role = v2_msg_role(md);
+	VERBOSE_DBGP(DBG_BASE, ike->sa.logger,
+		     "looking for an unsecured transition matching exchange %s %s ...",
+		     str_enum_short(&ikev2_exchange_names, md->hdr.isa_xchg, &xb),
+		     str_enum_short(&message_role_names, role, &rb));
+
+	vassert(role == MESSAGE_RESPONSE);
+
+	struct ikev2_payload_errors message_payload_status = { .bad = false };
+
+	const struct v2_exchange *exchange = ike->sa.st_v2_msgid_windows.initiator.exchange;
+	vassert(exchange != NULL);
+	(*transition) = find_v2_transition(verbose, md, exchange->response,
+					   &message_payload_status, NULL);
+	if (*transition != NULL) {
 		return NULL;
 	}
 
-	llog(RC_LOG, logger, "no useful state microcode entry found for incoming unsecured packet");
-	return NULL;
+	/*
+	 * A very messed up message - none of the state transitions
+	 * recognized it!
+	 */
+	diag_t d = NULL;
+	JAMBUF(buf) {
+		jam_v2_payload_errors(buf, md, &message_payload_status);
+		d = diag_jambuf(buf);
+	}
+	return d;
 }
 
 bool is_plausible_secured_v2_exchange(struct ike_sa *ike, struct msg_digest *md)
 {
 	enum message_role role = v2_msg_role(md);
-	unsigned indent = 0;
 
 	enum_buf xb, rb;
-	ldbg_ft(ike->sa.logger, "looking for plausible secured exchange matching %s %s ...",
-		str_enum_short(&ikev2_exchange_names, md->hdr.isa_xchg, &xb),
-		str_enum_short(&message_role_names, v2_msg_role(md), &rb));
-	indent = 1;
+	VERBOSE_DBGP(DBG_BASE, ike->sa.logger,
+		     "looking for plausible secured exchange matching %s %s ...",
+		     str_enum_short(&ikev2_exchange_names, md->hdr.isa_xchg, &xb),
+		     str_enum_short(&message_role_names, role, &rb));
 
 	/*
 	 * See if the decrypted message payloads include the secured
@@ -588,8 +698,8 @@ bool is_plausible_secured_v2_exchange(struct ike_sa *ike, struct msg_digest *md)
 	 *
 	 * At this point, only the message payloads have been parsed.
 	 */
-	PASSERT(ike->sa.logger, md->message_payloads.parsed);
-	PEXPECT(ike->sa.logger, !md->encrypted_payloads.parsed);
+	vassert(md->message_payloads.parsed);
+	vassert(!md->encrypted_payloads.parsed);
 	if ((md->message_payloads.present & (v2P(SK) | v2P(SKF))) == LEMPTY) {
 		llog(RC_LOG, ike->sa.logger, "missing SK or SKF payload; message dropped");
 		return false;
@@ -615,8 +725,8 @@ bool is_plausible_secured_v2_exchange(struct ike_sa *ike, struct msg_digest *md)
 			     str_enum_short(&ikev2_exchange_names, md->hdr.isa_xchg, &xb));
 			return false;
 		}
-		ldbg_ft(ike->sa.logger, "plausible; exchange type matches responder %s exchange",
-		     exchange->subplot);
+		vdbg_ft("plausible; exchange type matches responder %s exchange",
+			exchange->subplot);
 		break;
 	case MESSAGE_RESPONSE:
 		exchange = ike->sa.st_v2_msgid_windows.initiator.exchange;
@@ -631,7 +741,7 @@ bool is_plausible_secured_v2_exchange(struct ike_sa *ike, struct msg_digest *md)
 			     exchange->subplot);
 			return false;
 		}
-		ldbg_ft(ike->sa.logger, "plausible; exchange type matches outstanding %s exchange",
+		vdbg_ft("plausible; exchange type matches outstanding %s exchange",
 		     exchange->subplot);
 		break;
 	}
@@ -656,8 +766,9 @@ bool is_plausible_secured_v2_exchange(struct ike_sa *ike, struct msg_digest *md)
  * report problems - but less so when OE
  */
 
-void llog_v2_payload_errors(struct logger *logger, struct msg_digest *md,
-			   const struct ikev2_payload_errors *errors)
+void llog_v2_payload_errors(struct logger *logger,
+			    const struct msg_digest *md,
+			    const struct ikev2_payload_errors *errors)
 {
 	lset_t rc_flags = log_limiter_rc_flags(logger, PAYLOAD_ERRORS_LOG_LIMITER);
 	if (rc_flags == LEMPTY) {
@@ -665,71 +776,83 @@ void llog_v2_payload_errors(struct logger *logger, struct msg_digest *md,
 	}
 
 	LLOG_JAMBUF(rc_flags, logger, buf) {
-		const enum ikev2_exchange ix = md->hdr.isa_xchg;
-		jam_string(buf, "unexpected ");
-		jam_enum_short(buf, &ikev2_exchange_names, ix);
-		jam(buf, " message");
-		/* we want to print and log the first notify payload */
-		struct payload_digest *ntfy = md->chain[ISAKMP_NEXT_v2N];
-		if (ntfy != NULL) {
-			jam(buf, " containing ");
-			jam_enum_short(buf, &v2_notification_names,
-				       ntfy->payload.v2n.isan_type);
-			if (ntfy->next != NULL) {
-				jam(buf, "...");
-			}
-			jam(buf, " notification");
+		jam_v2_payload_errors(buf, md, errors);
+	}
+}
+
+void jam_v2_payload_errors(struct jambuf *buf, const struct msg_digest *md,
+			   const struct ikev2_payload_errors *errors)
+{
+	/*
+	 * Ignore .bad; who is this function to judge that the message
+	 * is at fault (invalid or unexpected) when, in truth, the
+	 * cause is a gap in the state machine.
+	 */
+	const enum ikev2_exchange ix = md->hdr.isa_xchg;
+	jam_enum_short(buf, &ikev2_exchange_names, ix);
+	jam_string(buf, " ");
+	jam_enum_human(buf, &message_role_names, v2_msg_role(md));
+	/* we want to print and log the first notify payload */
+	struct payload_digest *ntfy = md->chain[ISAKMP_NEXT_v2N];
+	if (ntfy != NULL) {
+		jam_string(buf, " containing ");
+		jam_enum_short(buf, &v2_notification_names,
+			       ntfy->payload.v2n.isan_type);
+		if (ntfy->next != NULL) {
+			jam_string(buf, "...");
 		}
-		if (md->message_payloads.parsed) {
-			jam(buf, "; message payloads: ");
+		jam_string(buf, " notification");
+	}
+	jam(buf, " (Message ID %u", md->hdr.isa_msgid);
+	if (md->message_payloads.parsed) {
+		jam_string(buf, ";");
+		if (md->message_payloads.present == LEMPTY) {
+			jam_string(buf, " no payloads");
+		} else {
+			jam_string(buf, " message payloads ");
 			jam_lset_short(buf, &ikev2_payload_names, ",",
 				       md->message_payloads.present);
 		}
-		if (md->encrypted_payloads.parsed) {
-			jam(buf, "; encrypted payloads: ");
+		if (md->encrypted_payloads.present != LEMPTY) {
+			jam_string(buf, ", encrypted payloads ");
 			jam_lset_short(buf, &ikev2_payload_names, ",",
 				       md->encrypted_payloads.present);
 		}
 		if (errors->missing != LEMPTY) {
-			jam(buf, "; missing payloads: ");
+			jam_string(buf, ", missing ");
 			jam_lset_short(buf, &ikev2_payload_names, ",",
 				       errors->missing);
 		}
 		if (errors->unexpected != LEMPTY) {
-			jam(buf, "; unexpected payloads: ");
+			jam_string(buf, ", unexpected ");
 			jam_lset_short(buf, &ikev2_payload_names, ",",
 				       errors->unexpected);
 		}
 		if (errors->excessive != LEMPTY) {
-			jam(buf, "; excessive payloads: ");
+			jam_string(buf, ", excessive ");
 			jam_lset_short(buf, &ikev2_payload_names, ",",
 				       errors->excessive);
 		}
 		if (errors->notification != v2N_NOTHING_WRONG) {
-			jam(buf, "; missing notification ");
+			jam_string(buf, ", no ");
 			jam_enum_short(buf, &v2_notification_names,
 				       errors->notification);
+			jam_string(buf, " notification");
 		}
 	}
+	jam_string(buf, ")");
 }
 
-static void ldbg_transition(struct logger *logger, const char *indent,
+static void vdbg_transition(struct verbose verbose,
 			    const struct v2_transition *t)
 {
 	if (DBGP(DBG_BASE)) {
 
-		LLOG_JAMBUF(DEBUG_STREAM, logger, buf) {
-			jam_string(buf, indent);
-			jam_string(buf, "-> ");
-			jam_string(buf, (t->to == NULL ? "<NULL>" :
-					 t->to->short_name));
+		LLOG_JAMBUF(DEBUG_STREAM, verbose.logger, buf) {
+			jam(buf, PRI_VERBOSE, pri_verbose);
+			jam_string(buf, "->");
+			jam_enum_short(buf, &ikev2_exchange_names, t->exchange);
 			jam_string(buf, "; ");
-			jam_enum_short(buf, &event_type_names, t->timeout_event);
-		}
-
-		LLOG_JAMBUF(DEBUG_STREAM, logger, buf) {
-			jam_string(buf, indent);
-			jam_string(buf, "   ");
 			switch (t->recv_role) {
 			case NO_MESSAGE:
 				/* reverse polarity */
@@ -744,8 +867,6 @@ static void ldbg_transition(struct logger *logger, const char *indent,
 			default:
 				bad_case(t->recv_role);
 			}
-			jam_string(buf, ": ");
-			jam_enum_short(buf, &ikev2_exchange_names, t->exchange);
 			jam_string(buf, "; ");
 			jam_string(buf, "payloads: ");
 			FOR_EACH_THING(payloads, &t->message_payloads, &t->encrypted_payloads) {
@@ -776,74 +897,111 @@ static void ldbg_transition(struct logger *logger, const char *indent,
 					jam(buf, "}");
 				}
 			}
+
 		}
 
-		DBG_log("%s   story: %s", indent, t->story);
+		verbose.level++;
+
+		LDBG_log(verbose.logger, PRI_VERBOSE"%s", pri_verbose, t->story);
+
+		LLOG_JAMBUF(DEBUG_STREAM, verbose.logger, buf) {
+			jam(buf, PRI_VERBOSE, pri_verbose);
+			jam_string(buf, "->");
+			jam_string(buf, (t->to == NULL ? "<NULL>" :
+					 t->to->short_name));
+			jam_string(buf, "; ");
+			jam_enum_short(buf, &event_type_names, t->timeout_event);
+		}
+
+
 	}
 }
 
-static void validate_state_child_transition(struct logger *logger,
-					    const struct finite_state *from,
+static void validate_state_child_transition(struct verbose verbose,
 					    const struct v2_transition *t)
 {
-	bool found_from = false;
-	FOR_EACH_ELEMENT(f, t->from) {
-		if (*f == from) {
-			found_from = true;
-		}
-	}
-	passert(found_from);
-
 	const struct finite_state *to = t->to;
-	passert(to != NULL);
-	passert(to->kind >= STATE_IKEv2_FLOOR);
-	passert(to->kind < STATE_IKEv2_ROOF);
-	passert(to->ike_version == IKEv2);
-	ldbg_transition(logger, "     ", t);
+	vassert(to != NULL);
+	vassert(to->kind >= STATE_IKEv2_FLOOR);
+	vassert(to->kind < STATE_IKEv2_ROOF);
+	vassert(to->ike_version == IKEv2);
+	vdbg_transition(verbose, t);
 	FOR_EACH_THING(payloads, &t->message_payloads, &t->encrypted_payloads) {
-		passert(payloads->notification == 0);
-		passert(payloads->required == LEMPTY);
-		passert(payloads->optional == LEMPTY);
+		vassert(payloads->notification == 0);
+		vassert(payloads->required == LEMPTY);
+		vassert(payloads->optional == LEMPTY);
 	}
-	passert(t->exchange != 0);
-	passert(t->recv_role == 0);
-	passert(t->processor == NULL);
-	passert(t->llog_success == NULL);
+	vassert(t->exchange != 0);
+	vassert(t->recv_role == 0);
+	vassert(t->processor == NULL);
+	vassert(t->llog_success == NULL);
 }
 
-static void validate_state_exchange(struct logger *logger,
+static void validate_state_exchange_transition(struct verbose verbose,
+					       const struct v2_transition *transition,
+					       enum message_role recv_role,
+					       const struct v2_exchange *exchange)
+{
+	vdbg_transition(verbose, transition);
+	vassert(transition->llog_success != NULL);
+	vassert(transition->recv_role == recv_role);
+	vassert(transition->exchange == exchange->type);
+}
+
+static void validate_state_exchange(struct verbose verbose,
 				    const struct finite_state *from,
 				    const struct v2_exchange *exchange)
 {
 	enum_buf ixb;
-	ldbg(logger, "     => %s (%s); secured: %s",
+	vdbg("=>%s (%s); secured: %s",
 	     str_enum_short(&ikev2_exchange_names, exchange->type, &ixb),
 	     (exchange->subplot == NULL ? "<subplot>" : exchange->subplot),
 	     bool_str(exchange->secured));
+	const unsigned level = ++verbose.level;
 
-	if (exchange->initiate != NULL) {
-		ldbg(logger, "        => initiator");
-		ldbg_transition(logger, "           ", exchange->initiate);
+	verbose.level = level;
+	if (verbose.rc_flags != 0) {
+		LLOG_JAMBUF(verbose.rc_flags, verbose.logger, buf) {
+			jam(buf, PRI_VERBOSE, pri_verbose);
+			jam_string(buf, "from:");
+			FOR_EACH_ELEMENT(f, exchange->initiate.from) {
+				if ((*f) == NULL) {
+					break;
+				}
+				jam_string(buf, " ");
+				jam_string(buf, (*f)->short_name);
+			}
+		}
 	}
 
+	verbose.level = level;
+	if (exchange->initiate.transition != NULL) {
+		vdbg("initiator:");
+		verbose.level++;
+		validate_state_exchange_transition(verbose, exchange->initiate.transition, NO_MESSAGE, exchange);
+	}
+
+	verbose.level = level;
 	if (exchange->responder != NULL) {
-		ldbg(logger, "        => responder");
+		vdbg("responder:");
+		verbose.level++;
 		FOR_EACH_ITEM(t, exchange->responder) {
-			ldbg_transition(logger, "           ", t);
-			PASSERT(logger, t->exchange == exchange->type);
-			PASSERT(logger, t->recv_role == MESSAGE_REQUEST);
+			validate_state_exchange_transition(verbose, t, MESSAGE_REQUEST, exchange);
 		}
 	}
 
+	verbose.level = level;
 	if (exchange->response != NULL) {
-		ldbg(logger, "        => response");
+		vdbg("response:");
+		verbose.level++;
 		FOR_EACH_ITEM(t, exchange->response) {
-			ldbg_transition(logger, "           ", t);
+			validate_state_exchange_transition(verbose, t, MESSAGE_RESPONSE, exchange);
 		}
 	}
 
-	PASSERT(logger, exchange->subplot != NULL);
-	PASSERT(logger, from->v2.secured == exchange->secured);
+	verbose.level = level;
+	vassert(exchange->subplot != NULL);
+	vassert(from->v2.secured == exchange->secured);
 
 	/* does the exchange appear in the state's transitions? */
 	bool found_transition = false;
@@ -853,37 +1011,44 @@ static void validate_state_exchange(struct logger *logger,
 			break;
 		}
 	}
-	PASSERT(logger, found_transition);
+	vassert(found_transition);
 }
 
-static void validate_state(struct logger *logger, const struct finite_state *from)
+static void validate_state(struct verbose verbose, const struct finite_state *from)
 {
 	if (DBGP(DBG_BASE)) {
-		LLOG_JAMBUF(DEBUG_STREAM, logger, buf) {
-			jam(buf, "  ");
+		LLOG_JAMBUF(DEBUG_STREAM, verbose.logger, buf) {
+			jam(buf, PRI_VERBOSE, pri_verbose);
 			jam_finite_state(buf, from);
 		}
 	}
+	const unsigned level = ++verbose.level;
 
 	/*
 	 * Validate transitions XOR exchanges.  Can have at most one.
 	 */
 
-	PASSERT(logger, ((from->v2.child_transition == NULL) ||
-			 (from->v2.ike_exchanges == NULL)));
+	vassert((from->v2.child_transition == NULL) ||
+		(from->v2.ike_exchanges == NULL));
 
 	if (from->v2.child_transition != NULL) {
-		validate_state_child_transition(logger, from, from->v2.child_transition);
+		verbose.level = level;
+		vdbg("child transition:");
+		verbose.level++;
+		validate_state_child_transition(verbose, from->v2.child_transition);
 	}
 
+	verbose.level = level;
+	vdbg("exchanges:");
+	verbose.level++;
 	FOR_EACH_ITEM(exchange, from->v2.ike_exchanges) {
-		validate_state_exchange(logger, from, *exchange);
+		validate_state_exchange(verbose, from, *exchange);
 	}
 }
 
 void init_ikev2_states(struct logger *logger)
 {
-	ldbg(logger, "checking IKEv2 state table");
+	VERBOSE_DBGP(DBG_BASE, logger, "checking IKEv2 state table");
 	/* XXX: debug this using <<--selftest --debug-all --stderrlog>> */
 
 	/*
@@ -900,11 +1065,11 @@ void init_ikev2_states(struct logger *logger)
 		/* fill in using static struct */
 		const struct finite_state *fs = v2_states[kind - STATE_IKEv2_FLOOR];
 		if (fs == NULL) {
-			llog_passert(logger, HERE, "entry %d is NULL", kind);
+			llog_passert(verbose.logger, HERE, "entry %d is NULL", kind);
 		}
-		passert(fs->kind == kind);
-		passert(fs->ike_version == IKEv2);
-		passert(finite_states[kind] == NULL);
+		vassert(fs->kind == kind);
+		vassert(fs->ike_version == IKEv2);
+		vassert(finite_states[kind] == NULL);
 		finite_states[kind] = fs;
 	}
 
@@ -920,8 +1085,7 @@ void init_ikev2_states(struct logger *logger)
 	for (enum state_kind kind = STATE_IKEv2_FLOOR; kind < STATE_IKEv2_ROOF; kind++) {
 		/* fill in using static struct */
 		const struct finite_state *from = finite_states[kind];
-		validate_state(logger, from);
-
+		validate_state(verbose, from);
 	}
 
 }
