@@ -1,6 +1,6 @@
 /* impair messages recv/sent, for libreswan
  *
- * Copyright (C) 2020  Andrew Cagney
+ * Copyright (C) 2020,2023  Andrew Cagney
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -20,7 +20,7 @@
  * Impair a message identified using the message's direction's
  * sequence number:
  *
- * - incoming and outgoing message streams are numbered separately
+ * - inbound and outbound message streams are numbered separately
  *
  * - only unique messages (for each stream) are assigned a new number
  *
@@ -29,11 +29,11 @@
  *
  * - once an impair has been matched and actioned it is discarded
  *
- *   i.e., drop-inomming:2 will only match and drop the first incoming
+ *   i.e., drop-inomming:2 will only match and drop the first inbound
  *   message #2
  *
  * For instance, here's an exchange with the INITIATOR impaired to
- * drop the second incoming message:
+ * drop the second inbound message:
  *
  *     initiator                             responder
  *       out#1     IKE_SA_INIT req ->           in#1
@@ -49,16 +49,23 @@
 #include <stdbool.h>
 
 #include "shunk.h"
+#include "ip_endpoint.h"
 
 struct logger;
 struct msg_digest;
 enum impair_action;
+struct iface_device;
 
-void add_message_impairment(unsigned nr, enum impair_action action, struct logger *logger);
+void add_message_impairment(enum impair_action impair_action,
+			    enum impair_message_direction impair_direction,
+			    bool whack_enable, unsigned whack_value,
+			    struct logger *logger);
 
-bool impair_incoming_message(struct msg_digest *md);
-bool impair_outgoing_message(shunk_t message, struct logger *logger);
+bool impair_outbound(const struct iface_endpoint *interface, shunk_t message,
+		     const ip_endpoint *endpoint, struct logger *logger);
 
-void free_impair_message(struct logger *logger);
+bool impair_inbound(struct msg_digest *md);
+
+void shutdown_impair_message(struct logger *logger);
 
 #endif /* _DEMUX_H */
