@@ -13,6 +13,7 @@
  * Copyright (C) 2012-2019 Paul Wouters <pwouters@redhat.com>
  * Copyright (C) 2013 Antony Antony <antony@phenome.org>
  * Copyright (C) 2019 Andrew Cagney <cagney@gnu.org>
+ * Copyright (C) 2020 Nupur Agrawal <nupur202000@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -41,6 +42,7 @@
 #include "ikev2_create_child_sa.h"	/* for initiate_v2_CREATE_CHILD_SA_create_child() */
 #include "instantiate.h"
 #include "terminate.h"
+#include "ikev2_ike_session_resume.h"
 
 static bool initiate_connection_1_basics(struct connection *c,
 					 const char *remote_host,
@@ -63,7 +65,9 @@ bool initiate_connection(struct connection *c,
 			 bool background,
 			 struct logger *logger)
 {
-	ldbg_connection(c, HERE, "initiate: remote_host=%s",
+	VERBOSE_DBGP(DBG_BASE, c->logger, "%s()", __func__);
+	vdbg_connection(c, verbose, HERE,
+			"initiate: remote_host=%s",
 			(remote_host == NULL ? "<null> (using host from connection)" : remote_host));
 	connection_attach(c, logger);
 	bool ok = initiate_connection_1_basics(c, remote_host, background);
@@ -189,12 +193,12 @@ static bool initiate_connection_2_address(struct connection *c,
 		 */
 
 		if (c->config->dnshostname != NULL) {
-			if (c->config->ikev2_allow_narrowing) {
+			if (c->config->narrowing) {
 				esb_buf b;
 				llog(RC_NOPEERIP, c->logger,
 				     "cannot initiate connection without resolved dynamic peer IP address, will keep retrying (kind=%s, narrowing=%s)",
 				     str_enum(&connection_kind_names, c->local->kind, &b),
-				     bool_str(c->config->ikev2_allow_narrowing));
+				     bool_str(c->config->narrowing));
 			} else {
 				esb_buf b;
 				llog(RC_NOPEERIP, c->logger,
@@ -205,12 +209,12 @@ static bool initiate_connection_2_address(struct connection *c,
 			return true;
 		}
 
-		if (c->config->ikev2_allow_narrowing) {
+		if (c->config->narrowing) {
 			esb_buf b;
 			llog(RC_NOPEERIP, c->logger,
 			     "cannot initiate connection without knowing peer IP address (kind=%s narrowing=%s)",
 			     str_enum(&connection_kind_names, c->local->kind, &b),
-			     bool_str(c->config->ikev2_allow_narrowing));
+			     bool_str(c->config->narrowing));
 		} else {
 			esb_buf b;
 			llog(RC_NOPEERIP, c->logger,
@@ -255,7 +259,7 @@ static bool initiate_connection_3_template(struct connection *c,
 
 	if (is_template(c) &&
 	    c->config->ike_version == IKEv2 &&
-	    c->config->ikev2_allow_narrowing) {
+	    c->config->narrowing) {
 		struct connection *d = spd_instantiate(c, c->remote->host.addr, HERE);
 		connection_attach(d, c->logger);
 		/*
@@ -313,10 +317,12 @@ void initiate(struct connection *c,
 	      enum initiated_by initiated_by,
 	      where_t where)
 {
+	VERBOSE_DBGP(DBG_BASE, c->logger, "%s()", __func__);
 	enum_buf ifnb;
 	child_policy_buf pb;
 	enum_buf epb;
-	ldbg_connection(c, where, "%s() by %s policy=%s proto=%s sec_label="PRI_SHUNK,
+	vdbg_connection(c, verbose, where,
+			"%s() by %s policy=%s proto=%s sec_label="PRI_SHUNK,
 			__func__,
 			str_enum_short(&initiated_by_names, initiated_by, &ifnb),
 			str_child_policy(policy, &pb),
@@ -343,6 +349,19 @@ void initiate(struct connection *c,
 	 */
 
 	if (ike == NULL) {
+
+		/*
+		 * When overloaded constrain things to
+		 * non-opportunistic connections
+		 */
+		if (is_opportunistic(c)) {
+			err_t reason = drop_new_exchanges(logger);
+			if (reason != NULL) {
+				llog(RC_LOG, logger, "dropping new Opportunistic exchange, %s", reason);
+				return;
+			}
+		}
+
 		switch (c->config->ike_version) {
 #ifdef USE_IKEv1
 		case IKEv1:
@@ -356,9 +375,18 @@ void initiate(struct connection *c,
 			break;
 #endif
 		case IKEv2:
-			ike = initiate_v2_IKE_SA_INIT_request(c, NULL, policy,
-							      inception, sec_label,
-							      detach_whack);
+			if (c->session != NULL) {
+				/* if session is expired, this
+				 * redirects to IKE_SA_INIT */
+				ike = initiate_v2_IKE_SESSION_RESUME_request(c, policy,
+									     inception,
+									     sec_label,
+									     detach_whack);
+			} else {
+				ike = initiate_v2_IKE_SA_INIT_request(c, NULL, policy,
+								      inception, sec_label,
+								      detach_whack);
+			}
 			break;
 		}
 		if (ike == NULL) {
