@@ -17,9 +17,29 @@
 #ifndef INITIATE_H
 #define INITIATE_H
 
-bool initiate_connection(struct connection *c, struct fd *whackfd,
-			 const char *remote_host);
-void initiate_connections_by_name(const char *name, struct fd *whackfd,
-				  const char *remote_host);
+#include <stdbool.h>
+
+#include "lset.h"
+#include "shunk.h"
+#include "ip_packet.h"
+#include "pluto_timing.h"	/* for threadtime_t */
+#include "initiated_by.h"
+
+struct logger;
+struct kernel_acquire;
+struct connection;
+struct child_policy;
+
+bool initiate_connection(struct connection *c, const char *remote_host,
+			 bool background, struct logger *logger);
+
+void initiate(struct connection *c,
+	      const struct child_policy *policy,
+	      so_serial_t replacing,
+	      const threadtime_t *inception,
+	      shunk_t sec_label,
+	      bool background, struct logger *logger,
+	      enum initiated_by initiated_by,
+	      where_t where);
 
 #endif
