@@ -45,6 +45,7 @@
 #include "ipsec_interface.h"
 #include "virtual_ip.h"
 #include "kernel.h"
+#include "verbose.h"
 
 #define MINIMUM_IPSEC_SA_RANDOM_MARK 65536
 static uint32_t global_marks = MINIMUM_IPSEC_SA_RANDOM_MARK;
@@ -77,17 +78,6 @@ static struct connection *duplicate_connection(const char *name, struct connecti
 
 	c->iface = iface_addref(t->iface);
 
-	FOR_EACH_THING(end, LEFT_END, RIGHT_END) {
-		/*
-		 * Need to propagate template's .has_client to
-		 * instance.  Should selector code setting up SPDs
-		 * instead handle this.
-		 */
-		bool has_client = t->end[end].child.has_client;
-		set_end_child_has_client(c, end, has_client);
-		PEXPECT(t->logger, (has_client == (t->end[end].child.config->selectors.len > 0)));
-	}
-
 	c->local->host.id = clone_id(&t->local->host.id, "unshare local connection id");
 	c->remote->host.id = clone_id((peer_id != NULL ? peer_id : &t->remote->host.id),
 				      "unshare remote connection id");
@@ -130,8 +120,10 @@ struct connection *group_instantiate(struct connection *group,
 				     ip_port remote_port,
 				     where_t where)
 {
+	VERBOSE_DBGP(DBG_BASE, group->logger, "%s() ...", __func__);
 	subnet_buf rsb;
-	ldbg_connection(group, where, "%s: "PRI_HPORT" %s -> [%s]:"PRI_HPORT,
+	vdbg_connection(group, verbose, where,
+			"%s: "PRI_HPORT" %s -> [%s]:"PRI_HPORT,
 			__func__,
 			pri_hport(local_port),
 			protocol->name,
@@ -183,6 +175,14 @@ struct connection *group_instantiate(struct connection *group,
 	 */
 	t->next_instance_serial = 1;
 
+#define set_end_selector(END, SELECTOR, LOGGER)				\
+	{								\
+		PASSERT(LOGGER, (END)->child.selectors.proposed.list == NULL); \
+		PASSERT(LOGGER, (END)->child.selectors.proposed.len == 0); \
+		append_end_selector(END, selector_info(SELECTOR),	\
+				    SELECTOR, LOGGER, HERE);		\
+	}
+
 	/*
 	 * For the remote end, just use what ever the group specified
 	 * (i.e., ignore protoport=).
@@ -237,23 +237,17 @@ struct connection *group_instantiate(struct connection *group,
 	 * inherited resulting in instance+group aka GROUPINSTANCE
 	 * also. */
 	t->local->kind = t->remote->kind = CK_TEMPLATE;
-	t->child.reqid = (t->config->sa_reqid == 0 ? gen_reqid() :
-			  t->config->sa_reqid);
-	ldbg(t->logger,
-	     "%s t.child.reqid="PRI_REQID" because group->sa_reqid="PRI_REQID" (%s)",
-	     t->name,
-	     pri_reqid(t->child.reqid),
-	     pri_reqid(t->config->sa_reqid),
-	     (t->config->sa_reqid == 0 ? "generate" : "use"));
+	t->child.reqid = child_reqid(t->config, t->logger);
 
 	PEXPECT(t->logger, oriented(t));
 	connection_db_add(t);
 
 	/* fill in the SPDs */
-	add_connection_spds(t, address_info(t->local->host.addr));
+	add_connection_spds(t);
 
 	connection_buf gb;
-	ldbg_connection(t, HERE, "%s: from "PRI_CONNECTION,
+	vdbg_connection(t, verbose, HERE,
+			"%s: from "PRI_CONNECTION,
 			__func__, pri_connection(group, &gb));
 	return t;
 }
@@ -277,26 +271,30 @@ static struct connection *instantiate(struct connection *t,
 				      const ip_address remote_addr,
 				      const struct id *peer_id,
 				      shunk_t sec_label, /* for ldbg() message only */
-				      const char *func, where_t where)
+				      const char *func,
+				      struct verbose verbose,
+				      where_t where)
 {
 	address_buf ab;
 	id_buf idb;
 	enum_buf kb;
-	ldbg_connection(t, where, "%s: remote=%s id=%s kind=%s sec_label="PRI_SHUNK,
+	vdbg_connection(t, verbose, where,
+			"%s: remote=%s id=%s kind=%s sec_label="PRI_SHUNK,
 			func, str_address(&remote_addr, &ab),
 			str_id(peer_id, &idb),
 			str_enum_short(&connection_kind_names, t->local->kind, &kb),
 			pri_shunk(sec_label));
 
-	PASSERT(t->logger, address_is_specified(remote_addr)); /* always */
-	PASSERT(t->logger, (is_template(t) ||
+	vassert(address_is_specified(remote_addr)); /* always */
+	vassert((is_template(t) ||
 			    is_labeled_template(t) ||
 			    is_labeled_parent(t)));
 
 	if (peer_id != NULL) {
+		struct verbose verbose = { .logger = t->logger, };
 		int wildcards;	/* value ignored */
 		passert(t->remote->host.id.kind == ID_FROMCERT ||
-			match_id("", peer_id, &t->remote->host.id, &wildcards));
+			match_id(peer_id, &t->remote->host.id, &wildcards, verbose));
 	}
 
 	struct connection *d = duplicate_connection(t->name, t, peer_id, where);
@@ -317,13 +315,7 @@ static struct connection *instantiate(struct connection *t,
 						remote_addr, HERE); /* from whack initiate */
 	}
 
-	d->child.reqid = (t->config->sa_reqid == 0 ? gen_reqid() : t->config->sa_reqid);
-	pdbg(d->logger,
-	     "%s .child.reqid="PRI_REQID" because t.config.sa_requid="PRI_REQID" (%s)",
-	     d->name,
-	     pri_reqid(d->child.reqid),
-	     pri_reqid(t->config->sa_reqid),
-	     (t->config->sa_reqid == 0 ? "generate" : "use"));
+	d->child.reqid = child_reqid(t->config, d->logger);
 
 	/*
 	 * assumption: orientation is the same as c's - while the
@@ -347,109 +339,6 @@ static struct connection *instantiate(struct connection *t,
 	return d;
 }
 
-/*
- * XXX: unlike update_subnet_selectors() this must set each selector
- * to something valid?  For instance, of the end has addresspool, ask
- * for the entire address range.
- */
-
-static void update_selectors(struct connection *d)
-{
-	FOR_EACH_ELEMENT(end, d->end) {
-		const char *leftright = end->config->leftright;
-		PASSERT(d->logger, end->child.selectors.proposed.list == NULL);
-		PASSERT(d->logger, end->child.selectors.proposed.len == 0);
-		if (end->child.config->selectors.len > 0) {
-			ldbg(d->logger,
-			     "%s() %s selectors from %d child.selectors",
-			     __func__, leftright, end->child.config->selectors.len);
-			end->child.selectors.proposed = end->child.config->selectors;
-		} else if (end->host.config->pool_ranges.len > 0) {
-			/*
-			 * Make space for the selectors that will be
-			 * assigned from the addresspool.
-			 */
-			ldbg(d->logger,
-			     "%s() %s selectors formed from address pool",
-			     __func__, leftright);
-			FOR_EACH_ELEMENT(afi, ip_families) {
-				if (end->host.config->pool_ranges.ip[afi->ip_index].len > 0) {
-					append_end_selector(end, afi, afi->selector.all,
-							    d->logger, HERE);
-				}
-			}
-		} else {
-			ldbg(d->logger,
-			     "%s() %s.child selector formed from host address+protoport",
-			     __func__, leftright);
-			/*
-			 * Default the end's child selector (client) to a
-			 * subnet containing only the end's host address.
-			 */
-			ip_selector selector =
-				selector_from_address_protoport(end->host.addr,
-								end->child.config->protoport);
-			append_end_selector(end, selector_info(selector), selector,
-					    d->logger, HERE);
-		}
-	}
-}
-
-/*
- * XXX: unlike update_selectors(), this code has to handle the remote
- * subnet
- */
-static void update_refined_selectors(struct connection *d,
-				     const ip_selector *remote_subnet)
-{
-	FOR_EACH_ELEMENT(end, d->end) {
-		const char *leftright = end->config->leftright;
-		if (end->child.config->selectors.len > 0) {
-			ldbg(d->logger,
-			     "%s.child has %d configured selectors",
-			     leftright, end->child.config->selectors.len);
-			end->child.selectors.proposed = end->child.config->selectors;
-		} else if (remote_subnet != NULL &&
-			   &end->child == &d->remote->child &&
-			   d->remote->config->child.virt != NULL) {
-			PASSERT(d->logger, &end->host == &d->remote->host);
-			set_end_selector(end, *remote_subnet, d->logger);
-			if (selector_eq_address(*remote_subnet, d->remote->host.addr)) {
-				ldbg(d->logger,
-				     "forcing remote %s.spd.has_client=false",
-				     d->spd->remote->config->leftright);
-				set_child_has_client(d, remote, false);
-			}
-		} else if (end->host.config->pool_ranges.len > 0) {
-			/*
-			 * Make space for the selectors that will be
-			 * assigned from the addresspool.
-			 */
-			ldbg(d->logger,
-			     "%s() %s selectors formed from address pool",
-			     __func__, leftright);
-			FOR_EACH_ELEMENT(afi, ip_families) {
-				if (end->host.config->pool_ranges.ip[afi->ip_index].len > 0) {
-					append_end_selector(end, afi, afi->selector.all,
-							    d->logger, HERE);
-				}
-			}
-		} else {
-			ldbg(d->logger,
-			     "%s() %s selector formed from host",
-			     __func__, leftright);
-			/*
-			 * Default the end's child selector (client) to a
-			 * subnet containing only the end's host address.
-			 */
-			ip_selector selector =
-				selector_from_address_protoport(end->host.addr,
-								end->child.config->protoport);
-			set_end_selector(end, selector, d->logger);
-		}
-	}
-}
-
 /*
  * In addition to instantiate() also clone the SPD entries.
  *
@@ -462,20 +351,23 @@ struct connection *spd_instantiate(struct connection *t,
 				   const ip_address remote_addr,
 				   where_t where)
 {
-	PASSERT(t->logger, !is_labeled(t));
+	VERBOSE_DBGP(DBG_BASE, t->logger, "%s() ...", __func__);
+	vassert(!is_labeled(t));
 
 	struct connection *d = instantiate(t, remote_addr, /*peer-id*/NULL,
-					   empty_shunk, __func__, where);
+					   empty_shunk, __func__,
+					   verbose, where);
 
-	update_selectors(d);
-	add_connection_spds(d, address_info(d->local->host.addr));
+	add_proposals(d, NULL/*afi-isn't-needed*/, verbose);
+	add_connection_spds(d);
 
 	/* leave breadcrumb */
 	pexpect(d->negotiating_child_sa == SOS_NOBODY);
 	pexpect(d->routing.state == RT_UNROUTED);
 
 	connection_buf tb;
-	ldbg_connection(d, where, "%s: from "PRI_CONNECTION,
+	vdbg_connection(d, verbose, where,
+			"%s: from "PRI_CONNECTION,
 			__func__, pri_connection(t, &tb));
 
 	return d;
@@ -490,19 +382,22 @@ struct connection *labeled_template_instantiate(struct connection *t,
 						const ip_address remote_address,
 						where_t where)
 {
-	PASSERT(t->logger, is_labeled_template(t));
+	VERBOSE_DBGP(DBG_BASE, t->logger, "%s() ...", __func__);
+	vassert(is_labeled_template(t));
 
 	struct connection *p = instantiate(t, remote_address, /*peer-id*/NULL,
-					   empty_shunk, __func__, where);
+					   empty_shunk, __func__,
+					   verbose, where);
 
-	update_selectors(p);
-	add_connection_spds(p, address_info(p->local->host.addr));
+	add_proposals(p, NULL/*afi-isn't-needed*/, verbose);
+	add_connection_spds(p);
 
 	pexpect(p->negotiating_child_sa == SOS_NOBODY);
 	pexpect(p->routing.state == RT_UNROUTED);
 
 	connection_buf tb;
-	ldbg_connection(p, where, "%s: from "PRI_CONNECTION,
+	vdbg_connection(p, verbose, where,
+			"%s: from "PRI_CONNECTION,
 			__func__, pri_connection(t, &tb));
 
 	return p;
@@ -518,11 +413,13 @@ struct connection *labeled_parent_instantiate(struct ike_sa *ike,
 					      where_t where)
 {
 	struct connection *p = ike->sa.st_connection;
-	PASSERT(p->logger, is_labeled_parent(p));
+	VERBOSE_DBGP(DBG_BASE, p->logger, "%s() ...", __func__);
+	vassert(is_labeled_parent(p));
 
 	ip_address remote_addr = endpoint_address(ike->sa.st_remote_endpoint);
 	struct connection *c = instantiate(p, remote_addr, /*peer-id*/NULL,
-					   sec_label, __func__, where);
+					   sec_label, __func__,
+					   verbose, where);
 
 	/*
 	 * Install the sec_label from either an acquire or child
@@ -531,14 +428,15 @@ struct connection *labeled_parent_instantiate(struct ike_sa *ike,
 	PASSERT(c->logger, c->child.sec_label.ptr == NULL);
 	c->child.sec_label = clone_hunk(sec_label, __func__);
 
-	update_selectors(c);
-	add_connection_spds(c, address_info(c->local->host.addr));
+	add_proposals(c, NULL/*afi-isn't-needed*/, verbose);
+	add_connection_spds(c);
 
 	pexpect(c->negotiating_child_sa == SOS_NOBODY);
 	pexpect(c->routing.state == RT_UNROUTED);
 
 	connection_buf tb;
-	ldbg_connection(c, where, "%s: from "PRI_CONNECTION,
+	vdbg_connection(c, verbose, where,
+			"%s: from "PRI_CONNECTION,
 			__func__, pri_connection(p, &tb));
 
 	return c;
@@ -548,55 +446,218 @@ struct connection *rw_responder_instantiate(struct connection *t,
 					    const ip_address peer_addr,
 					    where_t where)
 {
-	PASSERT(t->logger, !is_opportunistic(t));
-	PASSERT(t->logger, !is_labeled(t));
+	VERBOSE_DBGP(DBG_BASE, t->logger, "%s() ...", __func__);
+	vassert(!is_opportunistic(t));
+	vassert(!is_labeled(t));
 
-	struct connection *d = instantiate(t, peer_addr, /*TBD peer_id*/NULL,
-					   empty_shunk, __func__, where);
+	struct connection *d = instantiate(t, peer_addr,
+					   /*TBD-peer_id*/NULL,
+					   empty_shunk, __func__,
+					   verbose, where);
 
-	update_selectors(d);
-	add_connection_spds(d, address_info(d->local->host.addr));
+	add_proposals(d, NULL/*afi-isn't-needed*/, verbose);
+	add_connection_spds(d);
 
 	connection_buf tb;
-	ldbg_connection(d, where, "%s: from "PRI_CONNECTION,
+	vdbg_connection(d, verbose, where,
+			"%s: from "PRI_CONNECTION,
 			__func__, pri_connection(t, &tb));
 	return d;
 }
 
-struct connection *rw_responder_refined_instantiate(struct connection *t,
-						    const ip_address remote_addr,
-						    const ip_selector *remote_subnet,
-						    const struct id *remote_id,
-						    where_t where)
+struct connection *rw_responder_id_instantiate(struct connection *t,
+					       const ip_address remote_addr,
+					       const struct id *remote_id,
+					       where_t where)
 {
-	PASSERT(t->logger, !is_opportunistic(t));
-	PASSERT(t->logger, !is_labeled(t));
+	VERBOSE_DBGP(DBG_BASE, t->logger, "%s() ...", __func__);
+	vassert(!is_opportunistic(t));
+	vassert(!is_labeled(t));
+	vassert(remote_id != NULL);
 
 	/*
 	 * XXX: this function is never called when there are
 	 * sec_labels?
 	 */
 	struct connection *d = instantiate(t, remote_addr, remote_id,
-					   empty_shunk, __func__, where);
+					   empty_shunk, __func__,
+					   verbose, where);
 
-	update_refined_selectors(d, remote_subnet);
-	add_connection_spds(d, address_info(d->local->host.addr));
+	/* real selectors are still unknown */
+	add_proposals(d, NULL/*afi-isn't-needed*/, verbose);
+	add_connection_spds(d);
 
 	connection_buf tb;
-	ldbg_connection(d, where, "%s: from "PRI_CONNECTION,
+	vdbg_connection(d, verbose, where,
+			"%s: from "PRI_CONNECTION,
 			__func__, pri_connection(t, &tb));
 	return d;
 
 }
 
+/*
+ * XXX: unlike update_selectors(), this code has to handle the remote
+ * subnet.
+ */
+
+static bool update_v1_quick_n_dirty_selectors(struct connection *d,
+					      const ip_selector remote_subnet,
+					      struct verbose verbose)
+{
+	selector_buf sb; /*handles NULL*/
+	vdbg("%s() %s ...", __func__, str_selector(&remote_subnet, &sb));
+	verbose.level++;
+
+	/*
+	 * Need to fill in selectors for both left and right.
+	 */
+	FOR_EACH_ELEMENT(end, d->end) {
+		const char *leftright = end->config->leftright;
+
+		vassert(end->child.selectors.proposed.list == NULL);
+		vassert(end->child.selectors.proposed.len == 0);
+		vexpect(end->child.has_client == false);
+
+		/* {left,right}subnet=... */
+		if (end->child.config->selectors.len > 0) {
+			vdbg("%s selectors from %d child.selectors",
+			     leftright, end->child.config->selectors.len);
+			end->child.selectors.proposed = end->child.config->selectors;
+			/* see also clone_connection */
+			set_end_child_has_client(d, end->config->index, true);
+			continue;
+		}
+
+		/* remote is virtual-ip */
+		if (&end->child == &d->remote->child &&
+		    d->remote->config->child.virt != NULL) {
+			vdbg("%s.child is virtual", leftright);
+			PASSERT(d->logger, &end->host == &d->remote->host);
+			set_end_selector(end, remote_subnet, d->logger);
+			if (selector_eq_address(remote_subnet, d->remote->host.addr)) {
+				ldbg(d->logger,
+				     "forcing remote %s.spd.has_client=false",
+				     d->spd->remote->config->leftright);
+			}
+			continue;
+		}
+
+		/* {left,right}addresspool= */
+		if (end->child.config->addresspools.len > 0) {
+			/*
+			 * Set the selectors to the pool range:
+			 *
+			 * IKEv2: addresspool implies narrowing so
+			 * peer sending ::/0 will be allowed to narrow
+			 * down to the addresspool range.
+			 *
+			 * IKEv1: peer is expected to send the lease
+			 * it obtained earlier (either during
+			 * MODE_CFG, or hard-wired in the config
+			 * file).
+			 */
+			FOR_EACH_ITEM(range, &end->child.config->addresspools) {
+				ip_selector selector = selector_from_range((*range));
+				selector_buf sb;
+				vdbg("%s selector formed from address pool %s",
+				     leftright, str_selector(&selector, &sb));
+				append_end_selector(end, selector_info(selector), selector, verbose.logger, HERE);
+			}
+			continue;
+		}
+
+		/* {left,right}= */
+		if (address_is_specified(end->host.addr)) {
+			/*
+			 * Default the end's child selector (client)
+			 * to a subnet containing only the end's host
+			 * address.
+			 *
+			 * If the other end has multiple child
+			 * selectors then the combination becomes a
+			 * list.
+			 */
+			address_buf ab;
+			protoport_buf pb;
+			vdbg("%s selector proposals from host address+protoport %s %s",
+			     leftright,
+			     str_address(&end->host.addr, &ab),
+			     str_protoport(&end->child.config->protoport, &pb));
+			ip_selector selector =
+				selector_from_address_protoport(end->host.addr,
+								end->child.config->protoport);
+			append_end_selector(end, selector_info(selector), selector, verbose.logger, HERE);
+			continue;
+		}
+
+#if 0
+		/*
+		 * to-be-determined from the host (for instance,
+		 * waiting on DNS) or the opportunistic group (needs
+		 * to be expanded).
+		 *
+		 * Make space regardless so that loops have something
+		 * to iterate over.
+		 */
+		if (vbad(host_afi == NULL)) {
+			return;
+		}
+
+		vexpect(is_permanent(c) || is_group(c) || is_template(c));
+		vdbg("%s selector proposals from unset host family %s",
+		     leftright, host_afi->ip_name);
+		append_end_selector(end, host_afi, unset_selector, verbose.logger, HERE);
+#else
+		llog_pexpect(verbose.logger, HERE, "no address");
+		return false;
+#endif
+	}
+
+	return true;
+}
+
+struct connection *rw_responder_v1_quick_n_dirty_instantiate(struct connection *t,
+							     const ip_address remote_addr,
+							     const ip_selector remote_subnet,
+							     const struct id *remote_id,
+							     struct verbose verbose,
+							     where_t where)
+{
+	vdbg("%s() ...", __func__);
+	verbose.level++;
+
+	vassert(!is_opportunistic(t));
+	vassert(!is_labeled(t));
+	vassert(remote_id != NULL);
+
+	/*
+	 * XXX: this function is never called when there are
+	 * sec_labels?
+	 */
+	struct connection *d = instantiate(t, remote_addr, remote_id,
+					   empty_shunk, __func__,
+					   verbose, where);
+
+	update_v1_quick_n_dirty_selectors(d, remote_subnet, verbose);
+	add_connection_spds(d);
+
+	connection_buf tb;
+	vdbg_connection(d, verbose, where,
+			"%s: from "PRI_CONNECTION,
+			__func__, pri_connection(t, &tb));
+	return d;
+}
+
 static struct connection *oppo_instantiate(struct connection *t,
 					   const ip_address remote_address,
-					   const char *func, where_t where)
+					   const char *func,
+					   struct verbose verbose,
+					   where_t where)
 {
-	PASSERT(t->logger, is_template(t));
-	PASSERT(t->logger, oriented(t)); /* else won't instantiate */
-	PASSERT(t->logger, t->local->child.selectors.proposed.len == 1);
-	PASSERT(t->logger, t->remote->child.selectors.proposed.len == 1);
+	vassert(is_template(t));
+	vassert(oriented(t)); /* else won't instantiate */
+	vassert(t->local->child.selectors.proposed.len == 1);
+	vassert(t->remote->child.selectors.proposed.len == 1);
 
 	/*
 	 * Instance inherits remote ID of child; exception being when
@@ -604,7 +665,8 @@ static struct connection *oppo_instantiate(struct connection *t,
 	 */
 
 	struct connection *d = instantiate(t, remote_address, /*peer_id*/NULL,
-					   empty_shunk, func, where);
+					   empty_shunk, func,
+					   verbose, where);
 
 	PASSERT(d->logger, is_instance(d));
 	PASSERT(d->logger, oriented(d)); /* else won't instantiate */
@@ -631,10 +693,11 @@ static struct connection *oppo_instantiate(struct connection *t,
 	set_end_selector(d->remote, remote_selector, d->logger);
 
 	PEXPECT(d->logger, oriented(d));
-	add_connection_spds(d, address_info(d->local->host.addr));
+	add_connection_spds(d);
 
 	connection_buf tb;
-	ldbg_connection(d, where, "%s: from "PRI_CONNECTION,
+	vdbg_connection(d, verbose, where,
+			"%s: from "PRI_CONNECTION,
 			func, pri_connection(t, &tb));
 	return d;
 }
@@ -651,10 +714,11 @@ struct connection *oppo_responder_instantiate(struct connection *t,
 	 * it falls within the selector's range (can't match port as
 	 * not yet known).
 	 */
-	PASSERT(t->logger, t->remote->child.selectors.proposed.len == 1);
+	VERBOSE_DBGP(DBG_BASE, t->logger, "%s() ...", __func__);
+	vassert(t->remote->child.selectors.proposed.len == 1);
 	ip_selector remote_template = t->remote->child.selectors.proposed.list[0];
-	PASSERT(t->logger, address_in_selector_range(remote_address, remote_template));
-	return oppo_instantiate(t, remote_address, __func__, where);
+	vassert(address_in_selector_range(remote_address, remote_template));
+	return oppo_instantiate(t, remote_address, __func__, verbose, where);
 }
 
 struct connection *oppo_initiator_instantiate(struct connection *t,
@@ -668,12 +732,13 @@ struct connection *oppo_initiator_instantiate(struct connection *t,
 	 * endpoint that needs to be negotiated.  Hence this endpoint
 	 * must be fully within the template's selector).
 	 */
-	PASSERT(t->logger, t->remote->child.selectors.proposed.len == 1);
+	VERBOSE_DBGP(DBG_BASE, t->logger, "%s() ...", __func__);
+	vassert(t->remote->child.selectors.proposed.len == 1);
 	ip_selector remote_template = t->remote->child.selectors.proposed.list[0];
 	ip_endpoint remote_endpoint = packet_dst_endpoint(packet);
-	PASSERT(t->logger, endpoint_in_selector(remote_endpoint, remote_template));
+	vassert(endpoint_in_selector(remote_endpoint, remote_template));
 	ip_address local_address = packet_src_address(packet);
 	PEXPECT(t->logger, address_eq_address(local_address, t->local->host.addr));
 	ip_address remote_address = endpoint_address(remote_endpoint);
-	return oppo_instantiate(t, remote_address, __func__, where);
+	return oppo_instantiate(t, remote_address, __func__, verbose, where);
 }
