@@ -27,6 +27,8 @@
 #include "verbose.h"
 #include "iface.h"
 
+static enum yn_options ipsec_interface_managed = YN_YES;
+
 static ip_cidr get_connection_ipsec_interface_cidr(const struct connection *c,
 						   struct verbose verbose);
 
@@ -49,6 +51,10 @@ static unsigned unmap_id(ipsec_interface_id_t ipsec_if_id)
 
 size_t jam_ipsec_interface_id(struct jambuf *buf, ipsec_interface_id_t ipsec_if_id)
 {
+	if (ipsec_interface_managed == YN_NO) {
+		return jam(buf, "%s[%u]", kernel_ops->ipsec_interface->name, ipsec_if_id);
+	}
+
 	/* Map the IPSEC_IF_ID back to the name's number, when
 	 * needed */
 	unsigned id = unmap_id(ipsec_if_id);
@@ -370,6 +376,11 @@ bool add_kernel_ipsec_interface_address(struct connection *c,
 		return true;
 	}
 
+	if (ipsec_interface_managed != YN_YES) {
+		vdbg("skipped; ipsec-interface is unmanaged");
+		return true;
+	}
+
 	if (vbad(c->iface == NULL) ||
 	    vbad(c->iface->real_device_name == NULL)) {
 		return false;
@@ -394,6 +405,12 @@ void del_kernel_ipsec_interface_address(struct connection *c,
 		return;
 	}
 
+	if (ipsec_interface_managed != YN_YES) {
+		vdbg("skipped; ipsec-interface is unmanaged");
+		vexpect(c->ipsec_interface_address == NULL);
+		return;
+	}
+
 	if (c->ipsec_interface_address == NULL) {
 		vdbg("skipped; no ipsec-interface-address to delete");
 		return;
@@ -419,7 +436,8 @@ static struct ipsec_interface *find_ipsec_interface_by_id(ipsec_interface_id_t i
 	return ret;
 }
 
-static struct ipsec_interface *alloc_ipsec_interface(ipsec_interface_id_t ipsec_if_id)
+static struct ipsec_interface *alloc_ipsec_interface(ipsec_interface_id_t ipsec_if_id,
+						     const struct iface_device *iface)
 {
 	/*
 	 * Create a new ref-counted ipsec_interface, it is not added
@@ -434,6 +452,7 @@ static struct ipsec_interface *alloc_ipsec_interface(ipsec_interface_id_t ipsec_
 			 kernel_ops->ipsec_interface->name,
 			 unmap_id(ipsec_if_id));
 	passert(l < IFNAMSIZ);
+	jam_str(p->physical, sizeof(p->physical), iface->real_device_name);
 	/* add to known interfaces */
 	p->next = ipsec_interfaces;
 	ipsec_interfaces = p;
@@ -500,8 +519,8 @@ void ipsec_interface_delref(struct ipsec_interface **ipsec_if,
 		     ipsec_interface, ipsec_interface->name, ipsec_interface->if_id);
 }
 
-diag_t parse_ipsec_interface(struct config *config,
-			     const char *ipsec_interface,
+diag_t parse_ipsec_interface(const char *ipsec_interface,
+			     struct ipsec_interface_config *config,
 			     struct logger *logger)
 {
 	VERBOSE_DBGP(DBG_BASE, logger, "adding %s to config", ipsec_interface);
@@ -518,20 +537,10 @@ diag_t parse_ipsec_interface(struct config *config,
 		return NULL;
 	}
 
-	/*
-	 * Note: check for kernel support after YN check; this way
-	 * ipsec-interface=no is silently ignored.
-	 */
-	if (kernel_ops->ipsec_interface == NULL) {
-		return diag("ipsec-interface is not implemented by %s",
-			    kernel_ops->interface_name);
-	}
-
 	/* something other than ipsec-interface=no, check support */
-	err_t err = kernel_ipsec_interface_supported(verbose);
-	if (err != NULL) {
-		return diag("ipsec-interface=%s not supported: %s",
-			    ipsec_interface, err);
+	if (ipsec_interface_managed == YN_UNSET) {
+		return diag("ipsec-interface=%s is not supported",
+			    ipsec_interface);
 	}
 
 	ipsec_interface_id_t if_id;
@@ -564,8 +573,8 @@ diag_t parse_ipsec_interface(struct config *config,
 	vdbg("ipsec-interface=%s parsed to %s",
 	     ipsec_interface, str_ipsec_interface_id(if_id, &ib));
 
-	config->ipsec_interface.enabled = true;
-	config->ipsec_interface.id = if_id;
+	config->enabled = true;
+	config->id = if_id;
 
 	return NULL;
 }
@@ -594,6 +603,16 @@ bool add_ipsec_interface(struct connection *c,
 		find_ipsec_interface_by_id(c->config->ipsec_interface.id);
 	if (ipsec_if != NULL) {
 		c->ipsec_interface = ipsec_interface_addref(ipsec_if, c->logger, HERE);
+		ipsec_interface_buf ib;
+		vdbg("reusing ipsec-interface %s", str_ipsec_interface(c->ipsec_interface, &ib));
+		return true;
+	}
+
+	if (ipsec_interface_managed == YN_NO) {
+		c->ipsec_interface = alloc_ipsec_interface(c->config->ipsec_interface.id, iface);
+		ipsec_interface_buf ib;
+		vdbg("using un-managed ipsec-interface %s", str_ipsec_interface(c->ipsec_interface, &ib));
+		return true;
 		return true;
 	}
 
@@ -612,9 +631,7 @@ bool add_ipsec_interface(struct connection *c,
 						iface, verbose)) {
 			return false;
 		}
-		c->ipsec_interface = alloc_ipsec_interface(c->config->ipsec_interface.id);
-		jam_str(c->ipsec_interface->physical, sizeof(c->ipsec_interface->physical),
-			iface->real_device_name);
+		c->ipsec_interface = alloc_ipsec_interface(c->config->ipsec_interface.id, iface);
 		c->ipsec_interface->pluto_added = true;
 		ipsec_interface_buf ib;
 		vlog("created ipsec-interface %s",
@@ -646,16 +663,62 @@ bool add_ipsec_interface(struct connection *c,
 		return false;
 	}
 
-	c->ipsec_interface = alloc_ipsec_interface(c->config->ipsec_interface.id);
-	jam_str(c->ipsec_interface->physical, sizeof(c->ipsec_interface->physical),
-		iface->real_device_name);
+	c->ipsec_interface = alloc_ipsec_interface(c->config->ipsec_interface.id, iface);
 	ipsec_interface_buf ib;
-	vdbg("using ipsec-interface %s", str_ipsec_interface(c->ipsec_interface, &ib));
+	vdbg("using managed ipsec-interface %s", str_ipsec_interface(c->ipsec_interface, &ib));
 	return true;
 }
 
-void check_stale_ipsec_interfaces(struct logger *logger)
+reqid_t ipsec_interface_reqid(ipsec_interface_id_t if_id, struct logger *logger)
 {
-	VERBOSE_DBGP(DBG_BASE, logger, "...");
-	kernel_ipsec_interface_check_stale(verbose);
+	VERBOSE_DBGP(DBG_BASE, logger, "%s:%s() if_id=%d",
+		     kernel_ops->ipsec_interface->name, __func__,
+		     if_id);
+	if (kernel_ops->ipsec_interface->reqid != NULL) {
+		return kernel_ops->ipsec_interface->reqid(if_id, verbose);
+	}
+	return 0;
+}
+
+void config_ipsec_interface(enum yn_options managed, struct logger *logger)
+{
+	name_buf nb;
+	VERBOSE_DBGP(DBG_BASE, logger, "%s:%s() managed=%s",
+		     (kernel_ops->ipsec_interface == NULL ? "?!?" :
+		      kernel_ops->ipsec_interface->name),
+		     __func__,
+		     str_sparse(&yn_option_names, managed, &nb));
+	switch (managed) {
+	case YN_UNSET:
+		return;
+	case YN_YES:
+	case YN_NO:
+		ipsec_interface_managed = managed;
+		return;
+	}
+	bad_case(managed);
+}
+
+enum yn_options init_ipsec_interface(struct logger *logger)
+{
+	name_buf nb;
+	VERBOSE_DBGP(DBG_BASE, logger, "%s() managed=%s",
+		     __func__, str_sparse(&yn_option_names, ipsec_interface_managed, &nb));
+
+	if (kernel_ops->ipsec_interface == NULL) {
+		ipsec_interface_managed = YN_UNSET;
+		vlog("ipsec-interface is not supported");
+		return YN_UNSET;
+	}
+
+	if (ipsec_interface_managed == YN_YES) {
+		err_t err = kernel_ops->ipsec_interface->init(verbose);
+		if (err != NULL) {
+			ipsec_interface_managed = YN_UNSET;
+			vlog("ipsec-interface is not working: %s", err);
+			return YN_UNSET;
+		}
+	}
+
+	return ipsec_interface_managed;
 }
