@@ -157,11 +157,19 @@ spd_priority_t spd_priority(const struct spd *spd)
 	 * set based on the host_addr parameters.
 	 *
 	 * A longer prefix wins over a shorter prefix, hence the
-	 * reversal.  Value needs to fit 0-128, hence 8 bits.
+	 * reversal.
+	 *
+	 * Note: Value needs to fit 0-128, hence 8 bits wide.
+	 *
+	 * Note: should CLIENT be non-CIDR, prefix_len() returns -1
+	 * and the below scores 129 - very poor.  Hopefully client
+	 * isn't CIDR.
 	 */
-	unsigned srcw = 128 - spd->local->client.maskbits;
+	int srcp = selector_prefix_len(spd->local->client);
+	int dstp = selector_prefix_len(spd->remote->client);
+	unsigned srcw = 128 - srcp;
 	prio = (prio << 8) | srcw;
-	unsigned dstw = 128 - spd->remote->client.maskbits;
+	unsigned dstw = 128 - dstp;
 	prio = (prio << 8) | dstw;
 
 	/*
@@ -281,7 +289,7 @@ static void jam_bare_shunt(struct jambuf *buf, const struct bare_shunt *bs)
 	jam(buf, " => ");
 	jam_enum_short(buf, &shunt_policy_names, bs->shunt_policy);
 	jam(buf, "    %s", bs->why);
-	if (bs->restore_serialno != UNSET_CO_SERIAL) {
+	if (bs->restore_serialno != COS_NOBODY) {
 		jam(buf, " "PRI_CO, pri_co(bs->restore_serialno));
 	}
 }
@@ -755,7 +763,7 @@ struct spd_owner spd_owner(const struct spd *c_spd,
  * XXX: can this and/or route_owner() be merged?
  */
 
-static void clear_connection_spd_conflicts(struct connection *c)
+void clear_connection_spd_conflicts(struct connection *c)
 {
 	FOR_EACH_ITEM(spd, &c->child.spds) {
 		zero(&spd->wip);
@@ -781,11 +789,11 @@ static void llog_spd_conflict(struct logger *logger, const struct spd *spd,
 	}
 }
 
-static bool get_connection_spd_conflict(const struct spd *spd,
-					const enum routing new_routing,
-					struct spd_owner *owner,
-					struct bare_shunt ***bare_shunt,
-					struct logger *logger)
+bool get_connection_spd_conflict(const struct spd *spd,
+				 const enum routing new_routing,
+				 struct spd_owner *owner,
+				 struct bare_shunt ***bare_shunt,
+				 struct logger *logger)
 {
 	*owner = (struct spd_owner) {0};
 	*bare_shunt = NULL;
@@ -819,9 +827,9 @@ static bool get_connection_spd_conflict(const struct spd *spd,
 	return true;
 }
 
-static void revert_kernel_policy(struct spd *spd,
-				 struct child_sa *child/*could be NULL*/,
-				 struct logger *logger)
+void revert_kernel_policy(struct spd *spd,
+			  struct child_sa *child/*could be NULL*/,
+			  struct logger *logger)
 {
 	struct connection *c = spd->connection;
 	PEXPECT(logger, child == NULL || child->sa.st_connection == c);
@@ -870,12 +878,14 @@ static void revert_kernel_policy(struct spd *spd,
 		/* go back to old routing */
 		struct spd_owner owner = spd_owner(spd, c->routing.state,
 						   logger, HERE);
-		delete_spd_kernel_policy(spd, &owner, DIRECTION_OUTBOUND,
-					 EXPECT_KERNEL_POLICY_OK,
+		delete_spd_kernel_policy(spd, &owner,
+					 DIRECTION_OUTBOUND,
+					 KERNEL_POLICY_PRESENT,
 					 c->logger, HERE,
 					 "deleting failed policy");
-		delete_spd_kernel_policy(spd, &owner, DIRECTION_INBOUND,
-					 EXPECT_KERNEL_POLICY_OK,
+		delete_spd_kernel_policy(spd, &owner,
+					 DIRECTION_INBOUND,
+					 KERNEL_POLICY_PRESENT,
 					 c->logger, HERE,
 					 "deleting failed policy");
 		return;
@@ -957,7 +967,7 @@ bool unrouted_to_routed(struct connection *c, enum routing new_routing, where_t
 		if (spd->wip.conflicting.shunt != NULL &&
 		    PEXPECT(c->logger, !kernel_ops->overlap_supported)) {
 			delete_bare_shunt_kernel_policy(*spd->wip.conflicting.shunt,
-							EXPECT_KERNEL_POLICY_OK,
+							KERNEL_POLICY_PRESENT,
 							c->logger, where);
 			/* if everything succeeds, delete below */
 		}
@@ -974,7 +984,7 @@ bool unrouted_to_routed(struct connection *c, enum routing new_routing, where_t
 		if (spd->wip.conflicting.shunt != NULL &&
 		    PBAD(c->logger, kernel_ops->overlap_supported)) {
 			delete_bare_shunt_kernel_policy(*spd->wip.conflicting.shunt,
-							EXPECT_KERNEL_POLICY_OK,
+							KERNEL_POLICY_PRESENT,
 							c->logger, where);
 			/* if everything succeeds, delete below */
 		}
@@ -1095,14 +1105,14 @@ void show_shunt_status(struct show *s)
 	for (const struct bare_shunt *bs = bare_shunts; bs != NULL; bs = bs->next) {
 		/* Print interesting fields.  Ignore count and last_active. */
 		SHOW_JAMBUF(s, buf) {
-			jam_selector_subnet_port(buf, &(bs)->our_client);
+			jam_selector_range_port(buf, &(bs)->our_client);
 			jam(buf, " -%d-> ", bs->transport_proto->ipproto);
-			jam_selector_subnet_port(buf, &(bs)->peer_client);
+			jam_selector_range_port(buf, &(bs)->peer_client);
 			jam_string(buf, " => ");
 			jam_enum(buf, &shunt_policy_percent_names, bs->shunt_policy);
 			jam_string(buf, "    ");
 			jam_string(buf, bs->why);
-			if (bs->restore_serialno != UNSET_CO_SERIAL) {
+			if (bs->restore_serialno != COS_NOBODY) {
 				jam_string(buf, " ");
 				jam_co(buf, bs->restore_serialno);
 			}
@@ -1154,9 +1164,9 @@ static void delete_bare_shunt_kernel_policy(const struct bare_shunt *bsp,
  * address to single address.
  */
 
-static void clear_narrow_holds(const ip_selector *src_client,
-			       const ip_selector *dst_client,
-			       struct logger *logger)
+void clear_narrow_holds(const ip_selector *src_client,
+			const ip_selector *dst_client,
+			struct logger *logger)
 {
 	const struct ip_protocol *transport_proto = protocol_from_ipproto(src_client->ipproto);
 	struct bare_shunt **bspp = &bare_shunts;
@@ -1169,8 +1179,7 @@ static void clear_narrow_holds(const ip_selector *src_client,
 		    transport_proto == bsp->transport_proto &&
 		    selector_in_selector(bsp->our_client, *src_client) &&
 		    selector_in_selector(bsp->peer_client, *dst_client)) {
-			delete_bare_shunt_kernel_policy(bsp,
-							EXPECT_KERNEL_POLICY_OK,
+			delete_bare_shunt_kernel_policy(bsp, KERNEL_POLICY_PRESENT,
 							logger, HERE);
 			free_bare_shunt(bspp);
 		} else {
@@ -1260,12 +1269,17 @@ static bool setup_half_kernel_state(struct child_sa *child, enum direction direc
 		.dst.route = route.dst.route,
 		.direction = direction,
 		.mode = route.mode,
+		.iptfs = (child->sa.st_seen_and_use_iptfs ? &c->config->child_sa.iptfs : NULL),
 		.sa_lifetime = c->config->sa_ipsec_max_lifetime,
 		.sa_max_soft_bytes = sa_ipsec_soft_bytes,
 		.sa_max_soft_packets = sa_ipsec_soft_packets,
 		.sa_ipsec_max_bytes = c->config->sa_ipsec_max_bytes,
 		.sa_ipsec_max_packets = c->config->sa_ipsec_max_packets,
 		.sec_label = c->child.sec_label /* assume connection outlive their kernel_sa's */,
+		.ipsec_interface = c->ipsec_interface,
+		.sa_mark_out = (c->ipsec_interface == NULL ? NULL :
+				(c->sa_marks.out.val == 0 && c->sa_marks.out.mask == 0) ? NULL :
+				&c->sa_marks.out),
 	};
 
 	address_buf sab, dab;
@@ -1299,11 +1313,6 @@ static bool setup_half_kernel_state(struct child_sa *child, enum direction direc
 					    &ip_protocol_ipcomp,
 					    ipcomp_spi, &text_ipcomp);
 
-		if (c->ipsec_interface != NULL) {
-			said_next->ipsec_interface = c->ipsec_interface;
-			said_next->sa_mark_out = &c->sa_marks.out;
-		}
-
 		if (!kernel_ops_add_sa(said_next, replace, child->sa.logger)) {
 			llog_sa(RC_LOG, child, "add_sa ipcomp failed");
 			goto fail;
@@ -1408,19 +1417,17 @@ static bool setup_half_kernel_state(struct child_sa *child, enum direction direc
 		*said_next = said_boilerplate;
 		said_next->spi = esp_spi;
 		said_next->proto = &ip_protocol_esp;
+
+		/*
+		 * Note: Linux kernels put various restrictions and
+		 * expectations replay-window size (for instance,
+		 * cleared on outbound SA).  The kernel backend gets
+		 * to handle this.
+		 */
 		said_next->replay_window = c->config->child_sa.replay_window;
 		ldbg(child->sa.logger, "kernel: setting IPsec SA replay-window to %ju",
 		     c->config->child_sa.replay_window);
 
-		if (c->ipsec_interface != NULL) {
-			said_next->ipsec_interface = c->ipsec_interface;
-			if (c->sa_marks.out.val != 0 ||
-			    c->sa_marks.out.mask != 0) {
-				/* manually configured mark-out=mark/mask */
-				said_next->sa_mark_out = &c->sa_marks.out;
-			}
-		}
-
 		if (direction == DIRECTION_OUTBOUND &&
 		    c->config->child_sa.tfcpad != 0 &&
 		    !child->sa.st_seen_no_tfc) {
@@ -1429,15 +1436,30 @@ static bool setup_half_kernel_state(struct child_sa *child, enum direction direc
 			said_next->tfcpad = c->config->child_sa.tfcpad;
 		}
 
-		if (c->config->decap_dscp) {
+		/* IPTFS kernel options */
+		if (child->sa.st_seen_and_use_iptfs) {
+			said_next->iptfs = &c->config->child_sa.iptfs;
+			deltatime_buf dtb, idb;
+			ldbg(child->sa.logger,
+			     "kernel: IPTFS fragmentation=%s max-queue-size=%ju, packet-size=%ju %s, drop-time=%s, init-delay=%s, reorder-window=%ju",
+			     bool_str(said_next->iptfs->fragmentation),
+			     said_next->iptfs->max_queue_size,
+			     said_next->iptfs->packet_size,
+			     (said_next->iptfs->packet_size > 0 ? "bytes" : "PMTU"),
+			     str_deltatime(said_next->iptfs->drop_time, &dtb),
+			     str_deltatime(said_next->iptfs->init_delay, &idb),
+			     said_next->iptfs->reorder_window);
+		}
+
+		if (c->config->decap_dscp && direction == DIRECTION_INBOUND) {
 			ldbg(child->sa.logger, "kernel: Enabling Decap ToS/DSCP bits");
 			said_next->decap_dscp = true;
 		}
-		if (!c->config->encap_dscp) {
+		if (!c->config->encap_dscp && direction == DIRECTION_OUTBOUND) {
 			ldbg(child->sa.logger, "kernel: Disabling Encap ToS/DSCP bits");
 			said_next->encap_dscp = false;
 		}
-		if (c->config->nopmtudisc) {
+		if (c->config->nopmtudisc && direction == DIRECTION_OUTBOUND) {
 			ldbg(child->sa.logger, "kernel: Disabling Path MTU Discovery");
 			said_next->nopmtudisc = true;
 		}
@@ -1545,6 +1567,12 @@ static bool setup_half_kernel_state(struct child_sa *child, enum direction direc
 					    &ip_protocol_ah,
 					    ah_spi, &text_ah);
 
+		/*
+		 * Note: Linux kernels put various restrictions and
+		 * expectations replay-window size (for instance,
+		 * cleared on outbound SA).  The kernel backend gets
+		 * to handle this.
+		 */
 		said_next->replay_window = c->config->child_sa.replay_window;
 		ldbg(child->sa.logger, "kernel: setting IPsec SA replay-window to %ju",
 		     c->config->child_sa.replay_window);
@@ -1607,49 +1635,6 @@ fail:
 	return false;
 }
 
-static bool install_inbound_ipsec_kernel_policies(struct child_sa *child)
-{
-	const struct connection *c = child->sa.st_connection;
-	struct logger *logger = child->sa.logger;
-	/*
-	 * Add an inbound eroute to enforce an arrival check.
-	 *
-	 * If inbound,
-	 * ??? and some more mysterious conditions,
-	 * Note reversed ends.
-	 * Not much to be done on failure.
-	 */
-
-	if (is_labeled_child(c)) {
-		pdbg(logger, "kernel: %s() skipping as IKEv2 config.sec_label="PRI_SHUNK,
-		     __func__, pri_shunk(c->config->sec_label));
-		return true;
-	}
-
-	FOR_EACH_ITEM(spd, &c->child.spds) {
-		selector_buf sb, db;
-		enum_buf eb;
-		pdbg(logger, "kernel: %s() installing SPD for %s=>%s %s",
-		     __func__,
-		     /* inbound */
-		     str_selector(&spd->remote->client, &sb),
-		     str_selector(&spd->local->client, &db),
-		     str_enum_short(&routing_names, c->routing.state, &eb));
-
-		if (!install_inbound_ipsec_kernel_policy(child, spd, HERE)) {
-		    log_state(RC_LOG, &child->sa, "Installing IPsec SA failed - check logs or dmesg");
-			return false;
-		}
-	}
-
-	if (impair.install_ipsec_sa_inbound_policy) {
-		llog(RC_LOG, logger, "IMPAIR: kernel: install_ipsec_sa_inbound_policy in %s()", __func__);
-		return false;
-	}
-
-	return true;
-}
-
 /*
  * XXX: Two cases:
  *
@@ -1779,165 +1764,6 @@ static bool uninstall_kernel_state(struct child_sa *child, enum direction direct
 	return result;
 }
 
-static bool install_outbound_ipsec_kernel_policies(struct child_sa *child,
-						   enum routing new_routing,
-						   struct do_updown updown)
-{
-	struct logger *logger = child->sa.logger;
-	struct connection *c = child->sa.st_connection;
-
-	if (is_labeled_child(c)) {
-		pdbg(logger, "kernel: %s() skipping as IKEv2 config.sec_label="PRI_SHUNK,
-		     __func__, pri_shunk(c->config->sec_label));
-		return true;
-	}
-
-	/* clear the deck */
-	clear_connection_spd_conflicts(c);
-
-	bool ok = true;	/* sticky: once false, stays false */
-
-	/*
-	 * Install the IPsec kernel policies.
-	 */
-
-	FOR_EACH_ITEM(spd, &c->child.spds) {
-
-		selector_buf sb, db;
-		enum_buf eb;
-		pdbg(logger,
-		     "kernel: %s() installing SPD for %s=>%s %s route=%s up=%s",
-		     __func__,
-		     /* outbound */
-		     str_selector(&spd->local->client, &db),
-		     str_selector(&spd->remote->client, &sb),
-		     str_enum_short(&routing_names, c->routing.state, &eb),
-		     bool_str(updown.route),
-		     bool_str(updown.up));
-
-		struct spd_owner owner;
-		ok = get_connection_spd_conflict(spd, new_routing, &owner,
-						 &spd->wip.conflicting.shunt,
-						 c->logger);
-		if (!ok) {
-			break;
-		}
-
-		spd->wip.ok = true;
-
-		if (is_v1_cisco_split(spd, HERE)) {
-			/* XXX: why is CISCO skipped? */
-			continue;
-		}
-
-		PEXPECT(logger, spd->wip.ok);
-		enum kernel_policy_op op =
-			(spd->wip.conflicting.shunt != NULL ? KERNEL_POLICY_OP_REPLACE :
-			 KERNEL_POLICY_OP_ADD);
-		if (spd->block) {
-			llog(RC_LOG, logger, "state spd requires a block (and no CAT?)");
-			ok = spd->wip.installed.kernel_policy =
-				add_spd_kernel_policy(spd, op, DIRECTION_OUTBOUND,
-						      SHUNT_KIND_BLOCK,
-						      logger, HERE,
-						      "install IPsec block policy");
-		} else {
-			ok = spd->wip.installed.kernel_policy =
-				install_outbound_ipsec_kernel_policy(child, spd, op, HERE);
-		}
-		if (!ok) {
-			break;
-		}
-
-		/*
-		 * Do we have to make a mess of the routing?
-		 *
-		 * Probably.  This code path needs a re-think.
-		 */
-
-		PEXPECT(logger, spd->wip.ok);
-		if ((updown.route || updown.up) && owner.bare_route == NULL) {
-			/* a new route: no deletion required, but preparation is */
-			if (!do_updown(UPDOWN_PREPARE, c, spd, child, logger))
-				ldbg(logger, "kernel: prepare command returned an error");
-		} else {
-			ldbg(logger, "kernel: %s() skipping updown-prepare", __func__);
-		}
-
-		PEXPECT(logger, spd->wip.ok);
-		if (updown.route && owner.bare_route == NULL) {
-			/* a new route: no deletion required, but preparation is */
-			ok = spd->wip.installed.route =
-				do_updown(UPDOWN_ROUTE, c, spd, child, logger);
-		} else {
-			ldbg(logger, "kernel: %s() skipping updown-route as non-bare", __func__);
-		}
-		if (!ok) {
-			break;
-		}
-
-		/*
-		 * Do we have to notify the firewall?
-		 *
-		 * Yes if this is the first time that the tunnel is
-		 * established (rekeys do not need to re-UP).
-		 *
-		 * Yes, if we are installing a tunnel eroute and the
-		 * firewall wasn't notified for a previous tunnel with
-		 * the same clients.  Any Previous tunnel would have
-		 * to be for our connection, so the actual test is
-		 * simple.
-		 */
-
-		if (updown.up) {
-			PEXPECT(logger, spd->wip.ok);
-			ok = spd->wip.installed.up =
-				do_updown(UPDOWN_UP, c, spd, child, logger);
-		}
-
-		if (!ok) {
-			break;
-		}
-	}
-
-	if (impair.install_ipsec_sa_outbound_policy) {
-		llog(RC_LOG, logger, "IMPAIR: kernel: install_ipsec_sa_outbound_policy in %s()", __func__);
-		return false;
-	}
-
-	/*
-	 * Finally clean up.
-	 */
-
-	if (!ok) {
-		FOR_EACH_ITEM(spd, &c->child.spds) {
-			/* go back to old routing */
-			struct spd_owner owner = spd_owner(spd, c->routing.state,
-							   logger, HERE);
-			delete_cat_kernel_policies(spd, &owner, child->sa.logger, HERE);
-			revert_kernel_policy(spd, child, logger);
-		}
-		return false;
-	}
-
-	FOR_EACH_ITEM(spd, &c->child.spds) {
-
-		if (is_v1_cisco_split(spd, HERE)) {
-			continue;
-		}
-
-		PEXPECT(logger, spd->wip.ok);
-		struct bare_shunt **bspp = spd->wip.conflicting.shunt;
-		if (bspp != NULL) {
-			free_bare_shunt(bspp);
-		}
-		/* clear host shunts that clash with freshly installed route */
-		clear_narrow_holds(&spd->local->client, &spd->remote->client, logger);
-	}
-
-	return true;
-}
-
 static bool connection_has_policy_conflicts(const struct connection *c,
 					    enum routing new_routing,
 					    struct logger *logger, where_t where)
@@ -2175,7 +2001,7 @@ bool was_eroute_idle(struct child_sa *child, deltatime_t since_when)
 		/* snafu; assume idle!?! */
 		return true;
 	}
-	deltatime_t idle_time = realtimediff(realnow(), first_proto_info->inbound.last_used);
+	deltatime_t idle_time = realtime_diff(realnow(), first_proto_info->inbound.last_used);
 	return deltatime_cmp(idle_time, >=, since_when);
 }
 
@@ -2196,7 +2022,7 @@ static void set_sa_info(struct ipsec_proto_info *p2, uint64_t bytes,
 			p2->inbound.last_used = now;
 		}
 		if (ago != NULL) {
-			*ago = realtimediff(now, p2->inbound.last_used);
+			*ago = realtime_diff(now, p2->inbound.last_used);
 		}
 	} else {
 		if (bytes > p2->outbound.bytes) {
@@ -2204,7 +2030,7 @@ static void set_sa_info(struct ipsec_proto_info *p2, uint64_t bytes,
 			p2->outbound.last_used = now;
 		}
 		if (ago != NULL)
-			*ago = realtimediff(now, p2->outbound.last_used);
+			*ago = realtime_diff(now, p2->outbound.last_used);
 	}
 }
 
@@ -2370,7 +2196,7 @@ void orphan_holdpass(struct connection *c,
 		    selector_eq_selector(dst, t->remote->child.selectors.proposed.list[0])) {
 			restore = t->serialno;
 		} else {
-			restore = UNSET_CO_SERIAL;
+			restore = COS_NOBODY;
 		}
 
 		/*
@@ -2398,11 +2224,11 @@ static void expire_bare_shunts(struct logger *logger)
 	dbg("kernel: checking for aged bare shunts from shunt table to expire");
 	for (struct bare_shunt **bspp = &bare_shunts; *bspp != NULL; ) {
 		struct bare_shunt *bsp = *bspp;
-		time_t age = deltasecs(monotimediff(mononow(), bsp->last_activity));
+		time_t age = deltasecs(monotime_diff(mononow(), bsp->last_activity));
 
 		if (age > deltasecs(pluto_shunt_lifetime)) {
 			ldbg_bare_shunt(logger, "expiring old", bsp);
-			if (co_serial_is_set(bsp->restore_serialno)) {
+			if (bsp->restore_serialno != COS_NOBODY) {
 				/*
 				 * Time to restore the connection's
 				 * shunt.  Presumably the bare shunt
@@ -2421,8 +2247,7 @@ static void expire_bare_shunts(struct logger *logger)
 					}
 				}
 			} else {
-				delete_bare_shunt_kernel_policy(bsp,
-								EXPECT_KERNEL_POLICY_OK,
+				delete_bare_shunt_kernel_policy(bsp, KERNEL_POLICY_PRESENT,
 								logger, HERE);
 			}
 			free_bare_shunt(bspp);
@@ -2438,8 +2263,7 @@ static void delete_bare_shunt_kernel_policies(struct logger *logger)
 	dbg("kernel: emptying bare shunt table");
 	while (bare_shunts != NULL) { /* nothing left */
 		const struct bare_shunt *bsp = bare_shunts;
-		delete_bare_shunt_kernel_policy(bsp,
-						EXPECT_KERNEL_POLICY_OK,
+		delete_bare_shunt_kernel_policy(bsp, KERNEL_POLICY_PRESENT,
 						logger, HERE);
 		free_bare_shunt(&bare_shunts); /* also updates BARE_SHUNTS */
 	}
@@ -2578,7 +2402,8 @@ const struct kernel_ops *kernel_ops = NULL/*kernel_stacks[0]*/;
 
 static bool kernel_initialized = false;
 
-deltatime_t bare_shunt_interval = DELTATIME_INIT(SHUNT_SCAN_INTERVAL);
+deltatime_t bare_shunt_interval;
+deltatime_t pluto_shunt_lifetime;
 
 static global_timer_cb kernel_scan_shunts;
 
