@@ -69,6 +69,38 @@ enum direction {
 
 extern const struct enum_names direction_names;
 
+enum directions {
+	DIRECTIONS_INBOUND = DIRECTION_INBOUND,
+	DIRECTIONS_OUTBOUND = DIRECTION_OUTBOUND,
+	DIRECTIONS_INBOUND_AND_OUTBOUND = DIRECTION_INBOUND|DIRECTION_OUTBOUND,
+};
+
+/*
+ * What to do when the kernel policy operation returns ENOENT?
+ */
+
+enum expect_kernel_policy {
+	/* Kernel policy can return either ENOENT or 0. */
+	KERNEL_POLICY_PRESENT_OR_MISSING,
+	KERNEL_POLICY_PRESENT,
+	KERNEL_POLICY_MISSING,
+};
+
+#define expect_kernel_policy(DIRECTIONS, DIRECTION)	\
+	(DIRECTIONS & DIRECTION ? KERNEL_POLICY_PRESENT : KERNEL_POLICY_MISSING)
+
+#define expect_kernel_policy_name(E)					\
+	({								\
+		enum expect_kernel_policy e_ = E;			\
+		const char *n_ = "?";					\
+		switch (e_) {						\
+		case KERNEL_POLICY_PRESENT_OR_MISSING: n_ = "PRESENT or MISSING"; break; \
+		case KERNEL_POLICY_MISSING: n_ = "MISSING"; break;	\
+		case KERNEL_POLICY_PRESENT: n_ = "PRESENT"; break;	\
+		}							\
+		n_;							\
+	})
+
 enum kernel_offload_type {
 	KERNEL_OFFLOAD_NONE,
 	KERNEL_OFFLOAD_CRYPTO,
@@ -80,7 +112,6 @@ struct nic_offload {
 	enum kernel_offload_type type;
 };
 
-
 /*
  * The CHILD (IPsec, kernel) SA has two IP ends.
  */
@@ -168,43 +199,9 @@ struct kernel_state {
 	bool nopmtudisc;
 	uint32_t tfcpad;
 
+	const struct config_iptfs *iptfs;	/* non-NULL when enabled */
 };
 
-/*
- * What to do when there's a policy op returns the ENOENT response?
- *
- * The old test, which looked like this:
- *
- *	bool enoent_ok = (op == KP_DELETE_INBOUND ||
- *			  (op == KP_DELETE_OUTBOUND && ntohl(cur_spi) == SPI_HOLD));
- *
- * but was too forgiving.  It hid bugs such as trying to delete the
- * wrong policy.
- */
-
-enum expect_kernel_policy {
-	/* Kernel policy can return either ENOENT or 0. */
-	IGNORE_KERNEL_POLICY_MISSING,
-#if 0
-	REPORT_KERNEL_POLICY_PRESENT,
-#endif
-	EXPECT_NO_INBOUND,
-	/* op can only return 0 */
-	EXPECT_KERNEL_POLICY_OK,
-};
-
-#define expect_kernel_policy_name(E)					\
-	({								\
-		enum expect_kernel_policy e_ = E;			\
-		const char *n_ = "?";					\
-		switch (e_) {						\
-		case IGNORE_KERNEL_POLICY_MISSING: n_ = "IGNORE_KERNEL_POLICY_MISSING"; break; \
-		case EXPECT_NO_INBOUND: n_ = "EXPECT_NO_INBOUND"; break; \
-		case EXPECT_KERNEL_POLICY_OK: n_ = "REPORT_NO_INBOUND"; break; \
-		}							\
-		n_;							\
-	})
-
 struct kernel_ops {
 	/*
 	 * The names used to identify the interface.
@@ -306,6 +303,10 @@ struct kernel_ops {
 	 */
 	err_t (*migrate_ipsec_sa_is_enabled)(struct logger *);
 	bool (*migrate_ipsec_sa)(struct child_sa *child);
+	err_t (*iptfs_ipsec_sa_is_enabled)(struct logger *);
+	bool (*iptfs_ipsec_sa)(struct child_sa *child);
+	err_t (*directional_ipsec_sa_is_enabled)(struct logger *);
+	bool (*directional_ipsec_sa)(struct child_sa *child);
 	bool (*poke_ipsec_policy_hole)(int fd, const struct ip_info *afi, struct logger *logger);
 	bool (*detect_nic_offload)(const char *name, struct logger *logger);
 	bool (*poke_ipsec_offload_policy_hole)(struct nic_offload *nic_offload, struct logger *logger);
@@ -337,15 +338,20 @@ extern const struct kernel_ops *const kernel_stacks[];
  * which %holds are news and which others should expire.
  */
 
-//#define SHUNT_SCAN_INTERVAL     (2 * secs_per_minute)   /* time between scans of eroutes */
-#define SHUNT_SCAN_INTERVAL     (2 * 10)   /* time between scans of eroutes */
+#define SHUNT_SCAN_INTERVAL_SECONDS (2*10)
+#define SHUNT_SCAN_INTERVAL     deltatime(SHUNT_SCAN_INTERVAL_SECONDS)   /* time between scans of eroutes */
 
 /* SHUNT_PATIENCE only has resolution down to a multiple of the sample rate,
  * SHUNT_SCAN_INTERVAL.
  * By making SHUNT_PATIENCE an odd multiple of half of SHUNT_SCAN_INTERVAL,
  * we minimize the effects of jitter.
  */
-#define SHUNT_PATIENCE  (SHUNT_SCAN_INTERVAL * 15 / 2)  /* inactivity timeout */
+#define SHUNT_PATIENCE  deltatime(SHUNT_SCAN_INTERVAL_SECONDS * 15 / 2)  /* inactivity timeout */
+
+#define PLUTO_SHUNT_LIFE_DURATION_DEFAULT deltatime(15 * secs_per_minute)
+
+extern deltatime_t bare_shunt_interval;
+extern deltatime_t pluto_shunt_lifetime;
 
 extern void show_shunt_status(struct show *);
 extern unsigned shunt_count(void);
@@ -401,8 +407,6 @@ bool kernel_ops_migrate_ipsec_sa(struct child_sa *child);
 extern void show_kernel_interface(struct show *s);
 void shutdown_kernel(struct logger *logger);
 
-extern deltatime_t bare_shunt_interval;
-
 extern bool kernel_ops_detect_nic_offload(const char *name, struct logger *logger);
 extern void handle_sa_expire(ipsec_spi_t spi, uint8_t protoid, ip_address dst,
 			     bool hard, uint64_t bytes, uint64_t packets, uint64_t add_time,
@@ -433,4 +437,17 @@ void setup_esp_nic_offload(struct nic_offload *nic_offload,
 struct spd_owner spd_owner(const struct spd *spd, enum routing new_routing,
 			   struct logger *logger, where_t where);
 
+void clear_connection_spd_conflicts(struct connection *c);
+bool get_connection_spd_conflict(const struct spd *spd,
+				 const enum routing new_routing,
+				 struct spd_owner *owner,
+				 struct bare_shunt ***bare_shunt,
+				 struct logger *logger);
+void clear_narrow_holds(const ip_selector *src_client,
+			const ip_selector *dst_client,
+			struct logger *logger);
+void revert_kernel_policy(struct spd *spd,
+			  struct child_sa *child/*could be NULL*/,
+			  struct logger *logger);
+
 #endif
