@@ -21,14 +21,14 @@
 
 struct logger;
 
-struct raw_iface {
+struct kernel_iface {
 	ip_address addr;
-	struct raw_iface *next;
-	char name[1]; /* MUST BE LAST; overalloc hack */
+	struct kernel_iface *next;
+	char name[]; /* MUST BE LAST; overalloc hack */
 };
 
-extern struct raw_iface *find_raw_ifaces(const struct ip_info *afi, struct logger *logger);
-extern struct raw_iface *find_raw_ifaces4(struct logger *logger);
-extern struct raw_iface *find_raw_ifaces6(struct logger *logger);
+extern struct kernel_iface *find_kernel_ifaces(const struct ip_info *afi, struct logger *logger);
+extern struct kernel_iface *find_kernel_ifaces4(struct logger *logger);
+extern struct kernel_iface *find_kernel_ifaces6(struct logger *logger);
 
 #endif
