@@ -362,7 +362,7 @@ static struct sadb_address *put_sadb_selector(struct outbuf *msg,
 				  protocol != NULL ? protocol->ipproto :
 				  pexpect(0));
 	ip_address prefix = selector_prefix(selector);
-	unsigned prefix_len = selector_prefix_bits(selector);
+	unsigned prefix_len = selector_prefix_len(selector);
 	struct sadb_address *address =
 		put_sadb_ext(msg, sadb_address, srcdst_exttype,
 			     .sadb_address_proto = proto,
@@ -378,8 +378,8 @@ static struct sadb_address *put_sadb_address(struct outbuf *msg,
 					     const ip_address addr,
 					     struct verbose verbose)
 {
-	const struct ip_info *afi = address_info(addr);
 #ifdef sadb_address_prefixlen /* NetBSD FreeBSD */
+	const struct ip_info *afi = address_info(addr);
 	struct sadb_address *address =
 		put_sadb_ext(msg, sadb_address, srcdst_exttype,
 			     .sadb_address_proto = IPSEC_PROTO_ANY/*255*/,
@@ -563,6 +563,10 @@ static bool register_alg(const struct sadb_msg *b,
 		const struct ike_alg *ike_alg = ike_alg_by_sadb_alg_id(type, alg->sadb_alg_id);
 		if (ike_alg != NULL) {
 			kernel_alg_add(ike_alg);
+		} else {
+			vdbg("%s algorithm %d not recognized",
+			     ike_alg_type_name(type),
+			     alg->sadb_alg_id);
 		}
 	}
 	return true;
@@ -692,7 +696,7 @@ static ipsec_spi_t pfkeyv2_get_ipsec_spi(ipsec_spi_t avoid UNUSED,
 					 const char *story UNUSED,	/* often SAID string */
 					 struct logger *logger)
 {
-	VERBOSE_DBGP(DBG_BASE, logger, "...");
+	VERBOSE_DBGP(DBG_BASE, logger, "reqid=%d ...", reqid);
 
 	/* GETSPI */
 	/* send: <base, address, SPI range> */
@@ -1531,7 +1535,7 @@ static bool kernel_pfkeyv2_policy_del(enum direction direction,
 				      const shunk_t sec_label UNUSED,
 				      struct logger *logger, const char *func)
 {
-	VERBOSE_DBGP(DBG_BASE, logger, "%s ...", func);
+	VERBOSE_DBGP(DBG_BASE, logger, "%s policy_id=%u ...", func, policy_id);
 	if (ipsec_interface != NULL) {
 		ipsec_interface_buf ifb;
 		vlog("BSD doesn't add kernel-policy to an ipsec-interface (%s)",
@@ -1609,11 +1613,11 @@ static bool kernel_pfkeyv2_policy_del(enum direction direction,
 	struct inbuf resp;
 	if (!msg_sendrecv(&req, &resp, verbose)) {
 		switch (expect_kernel_policy) {
-		case IGNORE_KERNEL_POLICY_MISSING:
-		case EXPECT_NO_INBOUND:
+		case KERNEL_POLICY_PRESENT_OR_MISSING:
+		case KERNEL_POLICY_MISSING:
 			ldbg(logger, "%s()   ignoring pfkey error", func);
 			break;
-		case EXPECT_KERNEL_POLICY_OK:
+		case KERNEL_POLICY_PRESENT:
 			llog_pexpect(logger, HERE, "%s()   receiving", func);
 			return false;
 		}
@@ -1709,7 +1713,7 @@ static void parse_sadb_acquire(const struct sadb_msg *msg,
 	}
 
 	ip_packet packet = packet_from_raw(HERE,
-					   address_type(&src_address),
+					   address_info(src_address),
 					   &src_address.bytes,
 					   &dst_address.bytes,
 					   &ip_protocol_all,
@@ -1774,6 +1778,24 @@ static void kernel_pfkeyv2_shutdown(struct logger *logger)
 	ldbg(logger, "%s() called; nothing to do", __func__);
 }
 
+static err_t kernel_directional_ipsec_is_enabled(struct logger *logger)
+{
+	ldbg(logger, "%s() called; nothing to do", __func__);
+	return "not supported";
+}
+
+static err_t kernel_iptfs_ipsec_sa_is_enabled(struct logger *logger)
+{
+	ldbg(logger, "%s() called; nothing to do", __func__);
+	return "not supported";
+}
+
+static err_t kernel_migrate_ipsec_sa_is_enabled(struct logger *logger)
+{
+	ldbg(logger, "%s() called; nothing to do", __func__);
+	return "not supported";
+}
+
 static const char *pfkeyv2_protostack_names[] = {
 	"pfkeyv2",
 	"pfkey",
@@ -1814,4 +1836,7 @@ const struct kernel_ops pfkeyv2_kernel_ops = {
 	.policy_del = kernel_pfkeyv2_policy_del,
 	.policy_add = kernel_pfkeyv2_policy_add,
 	.ipsec_interface = &kernel_ipsec_interface_ifconfig,
+	.directional_ipsec_sa_is_enabled = kernel_directional_ipsec_is_enabled,
+	.iptfs_ipsec_sa_is_enabled = kernel_iptfs_ipsec_sa_is_enabled,
+	.migrate_ipsec_sa_is_enabled = kernel_migrate_ipsec_sa_is_enabled,
 };
