@@ -136,6 +136,7 @@ struct kernel_policy {
 	 * mode.
 	 */
 	enum kernel_mode mode;
+	bool iptfs;
 	unsigned nr_rules;
 	struct kernel_policy_rule rule[3/*IPCOMP+{ESP,AH}+PADDING*/];
 	struct nic_offload nic_offload;
@@ -212,7 +213,7 @@ bool delete_spd_kernel_policy(const struct spd *spd,
 
 void delete_spd_kernel_policies(struct spd *spd,
 				const struct spd_owner *owner,
-				enum expect_kernel_policy inbound_policy_expectation,
+				enum directions directions,
 				struct logger *logger,
 				where_t where,
 				const char *story);
@@ -239,11 +240,6 @@ void delete_cat_kernel_policies(const struct spd *spd,
 				struct logger *logger,
 				where_t where);
 
-bool install_inbound_ipsec_kernel_policy(struct child_sa *child, struct spd *spd,
-					 where_t where);
-bool install_outbound_ipsec_kernel_policy(struct child_sa *child, struct spd *spd,
-					  enum kernel_policy_op op, where_t where);
-
 void replace_ipsec_with_bare_kernel_policy(struct child_sa *child,
 					   struct connection *c,
 					   struct spd *spd,
@@ -258,4 +254,9 @@ bool install_bare_kernel_policy(ip_selector src, ip_selector dst,
 				const struct nic_offload *nic_offload,
 				struct logger *logger, where_t where);
 
+bool install_inbound_ipsec_kernel_policies(struct child_sa *child);
+bool install_outbound_ipsec_kernel_policies(struct child_sa *child,
+					    enum routing new_routing,
+					    struct do_updown updown);
+
 #endif
