@@ -81,14 +81,19 @@ void free_preshared_secrets(struct logger *logger)
 	lsw_free_preshared_secrets(&pluto_secrets, logger);
 }
 
+struct secret_context {
+	struct show *s;
+};
+
 static int print_secrets(struct secret *secret,
-			 struct secret_stuff *pks UNUSED,
-			 void *uservoid)
+			 enum secret_kind secret_kind,
+			 unsigned secret_line,
+			 struct secret_context *context)
 {
-	struct show *s = uservoid;
+	struct show *s = context->s;
 
 	const char *kind;
-	switch (pks->kind) {
+	switch (secret_kind) {
 	case SECRET_PSK:
 		kind = "PSK";
 		break;
@@ -109,7 +114,7 @@ static int print_secrets(struct secret *secret,
 
 	int indent = 0;
 	SHOW_JAMBUF(s, buf) {
-		indent = jam(buf, "%5d:", pks->line);
+		indent = jam(buf, "%5d:", secret_line);
 		jam(buf, " %s ", kind);
 		if (ids == NULL) {
 			jam(buf, "%%any");
@@ -142,9 +147,12 @@ void list_psks(struct show *s)
 	const struct lsw_conf_options *oco = lsw_init_options();
 	show_blank(s);
 	show(s, "List of Pre-shared secrets (from %s)",
-		     oco->secretsfile);
+	     oco->secretsfile);
 	show_blank(s);
-	foreach_secret(pluto_secrets, print_secrets, s);
+	struct secret_context context = {
+		.s = s,
+	};
+	foreach_secret(pluto_secrets, print_secrets, &context);
 }
 
 /*
@@ -228,8 +236,10 @@ static bool try_all_keys(enum cert_origin cert_origin,
 			continue;
 		}
 
+		struct verbose verbose = { .logger = &global_logger, };
 		int wildcards; /* value ignored */
-		if (!match_id("  ", &key->id, &s->remote->id, &wildcards)) {
+		if (!match_id(&key->id, &s->remote->id,
+			      &wildcards, verbose)) {
 			id_buf printkid;
 			dbg("  skipping '%s' with wrong ID",
 			    str_id(&key->id, &printkid));
@@ -237,7 +247,8 @@ static bool try_all_keys(enum cert_origin cert_origin,
 		}
 
 		int pl;	/* value ignored */
-		if (!trusted_ca(key->issuer, ASN1(s->remote->config->ca), &pl)) {
+		if (!trusted_ca(key->issuer, ASN1(s->remote->config->ca),
+				&pl, verbose)) {
 			id_buf printkid;
 			dn_buf buf;
 			dbg("  skipping '%s' with untrusted CA '%s'",
@@ -471,8 +482,8 @@ static struct secret *lsw_get_secret(const struct connection *c,
 		pexpect(address_is_specified(c->local->host.addr));
 		/* roadwarrior: replace that with %ANYADDR */
 		rw_id = (struct id) {
-			.kind = address_type(&c->local->host.addr)->id_ip_addr,
-			.ip_addr = address_type(&c->local->host.addr)->address.unspec,
+			.kind = address_info(c->local->host.addr)->id_ip_addr,
+			.ip_addr = address_info(c->local->host.addr)->address.unspec,
 		};
 		id_buf old_buf, new_buf;
 		dbg("%s() switching remote roadwarrier ID from %s to %s (%%ANYADDR)",
@@ -495,10 +506,8 @@ static struct secret *lsw_get_secret(const struct connection *c,
 /*
  * find the struct secret associated with an XAUTH username.
  */
-struct secret *lsw_get_xauthsecret(char *xauthname)
+const struct secret_preshared_stuff *xauth_secret_by_xauthname(char *xauthname)
 {
-	struct secret *best = NULL;
-
 	dbg("started looking for xauth secret for %s", xauthname);
 
 	struct id xa_id = {
@@ -509,11 +518,14 @@ struct secret *lsw_get_xauthsecret(char *xauthname)
 		}
 	};
 
-	best = lsw_find_secret_by_id(pluto_secrets,
-				     SECRET_XAUTH,
-				     &xa_id, NULL, true);
+	struct secret *best = lsw_find_secret_by_id(pluto_secrets,
+						    SECRET_XAUTH,
+						    &xa_id, NULL, true);
+	if (best == NULL) {
+		return NULL;
+	}
 
-	return best;
+	return secret_preshared_stuff(best);
 }
 
 /*
@@ -523,7 +535,7 @@ struct secret *lsw_get_xauthsecret(char *xauthname)
  * Note2: this seems to be called for connections using RSA too?
  */
 
-const chunk_t *get_connection_psk(const struct connection *c)
+const struct secret_preshared_stuff *get_connection_psk(const struct connection *c)
 {
 	struct secret *s = lsw_get_secret(c, SECRET_PSK, false);
 	if (s == NULL) {
@@ -531,7 +543,7 @@ const chunk_t *get_connection_psk(const struct connection *c)
 		return NULL;
 	}
 
-	const chunk_t *psk = &get_secret_stuff(s)->u.preshared_secret;
+	const struct secret_preshared_stuff *psk = secret_preshared_stuff(s);
 	if (DBGP(DBG_CRYPT)) {
 		DBG_dump_hunk("PreShared Key", *psk);
 	}
@@ -539,29 +551,10 @@ const chunk_t *get_connection_psk(const struct connection *c)
 }
 
 /*
- * Find PPK, by its id (PPK_ID).
+ * Get connection PPK and store corresponding ppk_id in *ppk_id.
  */
-static shunk_t get_ppk_by_id(const chunk_t *ppk_id)
-{
-	struct secret *s = lsw_get_ppk_by_id(pluto_secrets, *ppk_id);
 
-	if (s != NULL) {
-		const struct secret_stuff *pks = get_secret_stuff(s);
-		if (DBGP(DBG_CRYPT)) {
-			DBG_dump_hunk("Found PPK:", pks->ppk);
-			DBG_dump_hunk("with PPK_ID:", *ppk_id);
-		}
-		return HUNK_AS_SHUNK(pks->ppk);
-	}
-	dbg("No PPK found with given PPK_ID");
-	return null_shunk;
-}
-
-/*
- * Get connection PPK and store ppk_id in *ppk_id. If ppk-ids conn option
- * was set, one of the listed PPK_IDs needs to be found in .secrets
- */
-shunk_t get_connection_ppk_initiator(const struct connection *c, chunk_t **ppk_id)
+const struct secret_ppk_stuff *get_connection_ppk_and_ppk_id(const struct connection *c)
 {
 	struct shunks *ppk_ids_shunks = c->config->ppk_ids_shunks;
 
@@ -572,22 +565,20 @@ shunk_t get_connection_ppk_initiator(const struct connection *c, chunk_t **ppk_i
 		 */
 		ldbg(c->logger, "ppk-ids conn option not specified, look for first matching secret");
 		struct secret *s = lsw_get_secret(c, SECRET_PPK, false);
-
 		if (s == NULL) {
-			*ppk_id = NULL;
-			return null_shunk;
+			return NULL;
 		}
 
-		struct secret_stuff *pks = get_secret_stuff(s);
-		*ppk_id = &pks->ppk_id;
+		const struct secret_ppk_stuff *ppk = secret_ppk_stuff(s);
 		if (DBGP(DBG_CRYPT)) {
-			DBG_log("Found PPK");
-			DBG_dump_hunk("PPK_ID:", **ppk_id);
-			DBG_dump_hunk("PPK:", pks->ppk);
+			DBG_log("found PPK");
+			DBG_dump_hunk("PPK_ID:", ppk->id);
+			DBG_dump_hunk("PPK:", ppk->key);
 		}
-		return HUNK_AS_SHUNK(pks->ppk);
+		return ppk;
 	} else {
-		ldbg(c->logger, "ppk-ids conn option specified, iterate through list: %s",
+		ldbg(c->logger,
+		     "ppk-ids conn option specified, find any matching PPK_ID in list: %s",
 		     c->config->ppk_ids);
 		/*
 		 * iterate through PPK_ID (ppk-ids=) list and try to find
@@ -595,57 +586,56 @@ shunk_t get_connection_ppk_initiator(const struct connection *c, chunk_t **ppk_i
 		 * list.
 		 */
 		FOR_EACH_ITEM(ppk_id_shunk, ppk_ids_shunks) {
-			/* XXX cast away the const qualifier from shunk_t pointer... */
-			chunk_t chunk = chunk2((void *) ppk_id_shunk->ptr, ppk_id_shunk->len);
-
 			ldbg(c->logger, "try to find PPK with PPK_ID:");
 			ldbg_hunk(c->logger, *ppk_id_shunk);
 
-			const shunk_t ppk = get_ppk_by_id(&chunk);
-
-			if (ppk.ptr != NULL) {
-				*ppk_id = &chunk;
+			const struct secret_ppk_stuff *ppk =
+				secret_ppk_stuff_by_id(pluto_secrets, *ppk_id_shunk);
+			if (ppk != NULL) {
 				return ppk;
 			}
 		}
 	}
-	return null_shunk;
+	return NULL;
 }
 
 /*
- * Get connection PPK with PPK_ID ppk_id. If ppk-ids conn option was set,
- * ppk_id (that was received) needs to be in it. Used by responder.
+ * Get connection PPK, in one of the two ways:
+ * - With specified PPK_ID ppk_id (if ppk_id is not NULL).
+ * or
+ * - With a PPK_ID that is at place 'index' in the ppk-ids=
+ * conn option list.
  */
-shunk_t get_connection_ppk_responder(const struct connection *c, chunk_t *ppk_id)
+const struct secret_ppk_stuff *get_connection_ppk(const struct connection *c,
+						  chunk_t *ppk_id,
+						  unsigned int index)
 {
 	struct shunks *ppk_ids_shunks = c->config->ppk_ids_shunks;
 
-	if (ppk_ids_shunks == NULL) {
+	if (ppk_id != NULL) {
 		/* try to find PPK with PPK_ID ppk_id */
-		ldbg(c->logger, "ppk-ids conn option not specified, look for PPK with ID:");
+		ldbg(c->logger, "looking for PPK with ID:");
 		ldbg_hunk(c->logger, *ppk_id);
-		return get_ppk_by_id(ppk_id);
+		return secret_ppk_stuff_by_id(pluto_secrets, HUNK_AS_SHUNK(*ppk_id));
 	} else {
-		ldbg(c->logger, "ppk-ids conn option specified, iterate through list: %s",
-		     c->config->ppk_ids);
-		/*
-		 * iterate through PPK_ID (ppk-ids=) list and try to match
-		 * ppk_id with one entry from PPK_ID list.
-		 */
-		FOR_EACH_ITEM(ppk_id_shunk, ppk_ids_shunks) {
-			/* XXX cast away the const qualifier from shunk_t pointer... */
-			chunk_t ppk_id_chunk = chunk2((void *) ppk_id_shunk->ptr, ppk_id_shunk->len);
+		passert(index < ppk_ids_shunks->len);
 
-			ldbg(c->logger, "checking if received PPK_ID is equal to:");
-			ldbg_hunk(c->logger, *ppk_id_shunk);
+		ldbg(c->logger, "looking for PPK with PPK ID in list %s at place: %u",
+		     c->config->ppk_ids, index);
 
-			if (hunk_eq(*ppk_id, *ppk_id_shunk)) {
-				ldbg(c->logger, "match! try to find PPK with that PPK_ID");
-				return get_ppk_by_id(&ppk_id_chunk);
-			}
+		/* XXX cast away the const qualifier from shunk_t pointer... */
+		shunk_t id = ppk_ids_shunks->list[index];
+
+		ldbg(c->logger, "try to find PPK with PPK_ID:");
+		ldbg_hunk(c->logger, id);
+
+		const struct secret_ppk_stuff *ppk =
+			secret_ppk_stuff_by_id(pluto_secrets, id);
+		if (ppk != NULL) {
+			return ppk;
 		}
 	}
-	return null_shunk;
+	return NULL;
 }
 
 /*
@@ -653,9 +643,9 @@ shunk_t get_connection_ppk_responder(const struct connection *c, chunk_t *ppk_id
  * indicated by a NULL pointer.
  */
 
-const struct secret_stuff *get_local_private_key(const struct connection *c,
-						      const struct pubkey_type *type,
-						      struct logger *logger)
+struct secret_pubkey_stuff *get_local_private_key(const struct connection *c,
+						  const struct pubkey_type *type,
+						  struct logger *logger)
 {
 	/* is there a certificate assigned to this connection? */
 	if (c->local->host.config->cert.nss_cert != NULL) {
@@ -668,7 +658,7 @@ const struct secret_stuff *get_local_private_key(const struct connection *c,
 		    str_id(&c->remote->host.id, &that_buf),
 		    type->name);
 
-		const struct secret_stuff *pks = NULL;
+		struct secret_pubkey_stuff *pks = NULL;
 		bool load_needed;
 		err_t err = find_or_load_private_key_by_cert(&pluto_secrets,
 							     &c->local->host.config->cert,
@@ -696,8 +686,7 @@ const struct secret_stuff *get_local_private_key(const struct connection *c,
 		 * If we don't find the right keytype (RSA, ECDSA,
 		 * etc) then best will end up as NULL
 		 */
-		pexpect(pks->kind == type->private_key_kind);
-		pexpect(pks->u.pubkey.content.type == type);
+		pexpect(pks->content.type == type);
 		dbg("connection %s's %s private key found in NSS DB using cert",
 		    c->name, type->name);
 		return pks;
@@ -713,7 +702,7 @@ const struct secret_stuff *get_local_private_key(const struct connection *c,
 		    str_id(&c->remote->host.id, &that_buf),
 		    type->name);
 
-		const struct secret_stuff *pks;
+		struct secret_pubkey_stuff *pks;
 		bool load_needed;
 		err_t err = find_or_load_private_key_by_ckaid(&pluto_secrets, c->local->host.config->ckaid,
 							      &pks, &load_needed, logger);
@@ -744,8 +733,7 @@ const struct secret_stuff *get_local_private_key(const struct connection *c,
 		 * If we don't find the right keytype (RSA, ECDSA,
 		 * etc) then best will end up as NULL
 		 */
-		pexpect(pks->kind == type->private_key_kind);
-		pexpect(pks->u.pubkey.content.type == type);
+		pexpect(pks->content.type == type);
 		dbg("connection %s's %s private key found in NSS DB using CKAID",
 		    c->name, type->name);
 		return pks;
@@ -760,11 +748,10 @@ const struct secret_stuff *get_local_private_key(const struct connection *c,
 		return NULL;
 	}
 
-	const struct secret_stuff *pks = get_secret_stuff(s);
+	struct secret_pubkey_stuff *pks = secret_pubkey_stuff(s);
 	passert(pks != NULL);
 
-	pexpect(pks->kind == type->private_key_kind);
-	pexpect(pks->u.pubkey.content.type == type);
+	pexpect(pks->content.type == type);
 	dbg("connection %s's %s private key found",
 	    c->name, type->name);
 	return pks;
@@ -807,7 +794,7 @@ static const char *check_expiry(realtime_t expiration_date, time_t warning_inter
 	if (is_realtime_epoch(expiration_date))
 		return "ok (expires never)";
 
-	time_t time_left = deltasecs(realtimediff(expiration_date, realnow()));
+	time_t time_left = deltasecs(realtime_diff(expiration_date, realnow()));
 
 	if (time_left < 0)
 		return "fatal (expired)";
@@ -896,7 +883,7 @@ void show_pubkeys(struct show *s, bool utc, enum keys_to_show keys_to_show)
 err_t preload_private_key_by_cert(const struct cert *cert, bool *load_needed, struct logger *logger)
 {
 	threadtime_t start = threadtime_start();
-	const struct secret_stuff *pks;
+	struct secret_pubkey_stuff *pks;
 	err_t err = find_or_load_private_key_by_cert(&pluto_secrets, cert,
 						     &pks, load_needed, logger);
 	threadtime_stop(&start, SOS_NOBODY, "%s() loading private key %s", __func__,
@@ -907,7 +894,7 @@ err_t preload_private_key_by_cert(const struct cert *cert, bool *load_needed, st
 err_t preload_private_key_by_ckaid(const ckaid_t *ckaid, bool *load_needed, struct logger *logger)
 {
 	threadtime_t start = threadtime_start();
-	const struct secret_stuff *pks;
+	struct secret_pubkey_stuff *pks;
 	err_t err = find_or_load_private_key_by_ckaid(&pluto_secrets, ckaid,
 						      &pks, load_needed, logger);
 	threadtime_stop(&start, SOS_NOBODY, "%s() loading private key using CKAID", __func__);
