@@ -37,15 +37,14 @@ struct pubkey_content;
 struct pubkey;
 struct pubkey_type;
 struct crypt_mac;
-struct secret_stuff;
 struct hash_desc;
 struct show;
 struct ike_sa;
 struct pubkey_signer;
 
-const struct secret_stuff *get_local_private_key(const struct connection *c,
-						      const struct pubkey_type *type,
-						      struct logger *logger);
+struct secret_pubkey_stuff *get_local_private_key(const struct connection *c,
+						  const struct pubkey_type *type,
+						  struct logger *logger);
 
 extern bool has_private_key(cert_t cert);
 extern void list_psks(struct show *s);
@@ -53,9 +52,11 @@ extern void list_psks(struct show *s);
 enum keys_to_show { SHOW_ALL_KEYS = 1, SHOW_EXPIRED_KEYS, };
 extern void show_pubkeys(struct show *s, bool utc, enum keys_to_show keys_to_show);
 
-extern const chunk_t *get_connection_psk(const struct connection *c);
-extern shunk_t get_connection_ppk_initiator(const struct connection *c, chunk_t **ppk_id);
-extern shunk_t get_connection_ppk_responder(const struct connection *c, chunk_t *ppk_id);
+const struct secret_preshared_stuff *get_connection_psk(const struct connection *c);
+const struct secret_ppk_stuff *get_connection_ppk_and_ppk_id(const struct connection *c);
+const struct secret_ppk_stuff *get_connection_ppk(const struct connection *c,
+						  chunk_t *ppk_id,
+						  unsigned int index);
 
 extern void load_preshared_secrets(struct logger *logger);
 extern void free_preshared_secrets(struct logger *logger);
@@ -63,7 +64,7 @@ extern void free_remembered_public_keys(void);
 err_t preload_private_key_by_cert(const struct cert *cert, bool *load_needed, struct logger *logger);
 err_t preload_private_key_by_ckaid(const ckaid_t *ckaid, bool *load_needed, struct logger *logger);
 
-extern struct secret *lsw_get_xauthsecret(char *xauthname);
+extern const struct secret_preshared_stuff  *xauth_secret_by_xauthname(char *xauthname);
 
 /* keys from ipsec.conf */
 extern struct pubkey_list *pluto_pubkeys;
