@@ -157,7 +157,8 @@ enum {
 #define XFRM_MODE_ROUTEOPTIMIZATION 2
 #define XFRM_MODE_IN_TRIGGER 3
 #define XFRM_MODE_BEET 4
-#define XFRM_MODE_MAX 5
+#define XFRM_MODE_IPTFS 5
+#define XFRM_MODE_MAX 6
 
 /* Netlink configuration messages.  */
 enum {
@@ -321,6 +322,12 @@ enum xfrm_attr_type_t {
 	XFRMA_MTIMER_THRESH,	/* __u32 in seconds for input SA */
 	XFRMA_SA_DIR,		/* __u8 */
 	XFRMA_NAT_KEEPALIVE_INTERVAL,   /* __u32 in seconds for NAT keepalive */
+        XFRMA_IPTFS_DROP_TIME,  /* __u32 in: usec to wait for next seq */
+        XFRMA_IPTFS_REORDER_WINDOW, /* __u16 in: reorder window size (pkts) */
+        XFRMA_IPTFS_DONT_FRAG,  /* out: don't use fragmentation */
+        XFRMA_IPTFS_INIT_DELAY, /* __u32 out: initial packet wait delay (usec) */
+        XFRMA_IPTFS_MAX_QSIZE,  /* __u32 out: max ingress queue size (octets) */
+        XFRMA_IPTFS_PKT_SIZE,   /* __u32 out: size of outer packet, 0 for PMTU */
 	__XFRMA_MAX
 
 #define XFRMA_OUTPUT_MARK XFRMA_SET_MARK	/* Compatibility */
