@@ -39,7 +39,7 @@
 #include "sysdep.h"
 #include "constants.h"
 #include "lswconf.h"
-#include "lswfips.h"
+#include "fips_mode.h"
 
 #include "defs.h"
 #include "log.h"
@@ -81,7 +81,7 @@ void linux_audit_init(int do_audit, struct logger *logger)
 	if (audit_fd < 0) {
 		if (errno == EINVAL || errno == EPROTONOSUPPORT ||
 			errno == EAFNOSUPPORT) {
-			llog(RC_LOG_SERIOUS, logger,
+			llog(RC_LOG, logger,
 				    "Warning: kernel has no audit support");
 			close(audit_fd);
 			log_to_audit = false;
@@ -169,7 +169,7 @@ void linux_audit_conn(const struct state *st, enum linux_audit_kind op)
 			 * XXX: is this reliable; it's the intent not
 			 * result.  Is local correct?
 			 */
-			struct authby authby = c->local->config->host.authby;
+			struct authby authby = c->local->host.config->authby;
 			jam_string(&buf, ((authby.psk) ? "PRESHARED_KEY" :
 					  (authby.rsasig) ? "RSA_SIG" :
 					  (authby.rsasig_v1_5) ? "RSA_SIG" :
@@ -204,7 +204,7 @@ void linux_audit_conn(const struct state *st, enum linux_audit_kind op)
 			 */
 			jam(&buf, "%s_%zu",
 			    st->st_oakley.ta_integ->integ_ike_audit_name,
-			    st->st_oakley.ta_integ->integ_output_size * BITS_PER_BYTE);
+			    st->st_oakley.ta_integ->integ_output_size * BITS_IN_BYTE);
 		} else {
 			/*
 			 * XXX: dead code path?  Integ is never NULL?
@@ -231,12 +231,13 @@ void linux_audit_conn(const struct state *st, enum linux_audit_kind op)
 	case LAK_CHILD_FAIL:
 	{
 		/* head */
-		jam(&buf, "op=%s %s connstate=%lu, satype=%s samode=%s",
+		jam(&buf, "op=%s %s connstate=%lu, satype=%s",
 		    op == LAK_CHILD_DESTROY ? "destroy" : "start", /* fail uses op=start */
 		    conn_encode,
 		    st->st_serialno,
-		    st->st_esp.present ? "ipsec-esp" : st->st_ah.present ? "ipsec-ah" : "ipsec-policy",
-		    c->policy & POLICY_TUNNEL ? "tunnel" : "transport");
+		    st->st_esp.protocol == &ip_protocol_esp ? "ipsec-esp" : st->st_ah.protocol == &ip_protocol_ah ? "ipsec-ah" : "ipsec-policy");
+		jam_string(&buf, " samode=");
+		jam_enum_short(&buf, &encap_mode_story, c->config->child_sa.encap_mode);
 
 		/*
 		 * XXX: Instead of IKEv1_ESP_ID, this should use
@@ -250,15 +251,15 @@ void linux_audit_conn(const struct state *st, enum linux_audit_kind op)
 		const struct integ_desc *integ;
 		unsigned enckeylen;
 
-		if (st->st_esp.present) {
+		if (st->st_esp.protocol == &ip_protocol_esp) {
 			pi = &st->st_esp;
-			encrypt = st->st_esp.attrs.transattrs.ta_encrypt;
-			integ = st->st_esp.attrs.transattrs.ta_integ;
-			enckeylen = st->st_esp.attrs.transattrs.enckeylen;
-		} else if (st->st_ah.present) {
+			encrypt = st->st_esp.trans_attrs.ta_encrypt;
+			integ = st->st_esp.trans_attrs.ta_integ;
+			enckeylen = st->st_esp.trans_attrs.enckeylen;
+		} else if (st->st_ah.protocol == &ip_protocol_ah) {
 			pi = &st->st_ah;
 			encrypt = NULL;
-			integ = st->st_ah.attrs.transattrs.ta_integ;
+			integ = st->st_ah.trans_attrs.ta_integ;
 			enckeylen = 0;
 		} else {
 			pi = &st->st_esp;	/* hack: will yield zero SPIs, I think */
@@ -288,8 +289,9 @@ void linux_audit_conn(const struct state *st, enum linux_audit_kind op)
 	}
 	free(conn_encode); /* allocated by audit_encode_nv_string() */
 
+	/* note laddr_buf and laddr at same scope */
 	address_buf laddr_buf;
-	const char *laddr = ipstr(&c->local->host.addr, &laddr_buf);
+	const char *laddr = str_address(&c->local->host.addr, &laddr_buf);
 
 	jam(&buf, " raddr=");
 	jam_address(&buf, &c->remote->host.addr);
@@ -298,7 +300,7 @@ void linux_audit_conn(const struct state *st, enum linux_audit_kind op)
 			AUDIT_CRYPTO_IPSEC_SA : AUDIT_CRYPTO_IKE_SA,
 			audit_str, laddr,
 		    (op == LAK_PARENT_FAIL || op == LAK_CHILD_FAIL) ? AUDIT_RESULT_FAIL : AUDIT_RESULT_OK,
-		    st->st_logger);
+		    st->logger);
 }
 #if __GNUC__ >= 7
 #pragma GCC diagnostic pop
