@@ -26,6 +26,7 @@
 #include "ip_endpoint.h"
 #include "monotime.h"
 
+struct ike_sa;
 struct state;
 struct connection;
 struct msg_digest;
@@ -55,7 +56,7 @@ void show_log(struct show *s);
 extern bool log_to_audit;
 extern char *pluto_stats_binary;
 
-extern bool whack_prompt_for(struct state *st,
+extern bool whack_prompt_for(struct ike_sa *ike,
 			     const char *prompt,
 			     bool echo,
 			     char *ansbuf,
@@ -137,10 +138,6 @@ void log_state(lset_t rc_flags, const struct state *st,
 #define PRI_STATE PRI_CONNECTION" "PRI_SO
 #define pri_state(ST, B) pri_connection((ST)->st_connection, B), pri_so((ST)->st_serialno)
 
-#define sa_buf state_buf
-#define PRI_SA PRI_STATE
-#define pri_sa(SA, B) pri_connection((SA)->sa.st_connection, B), pri_so((SA)->sa.st_serialno)
-
 size_t jam_state(struct jambuf *buf, const struct state *st);
 
 extern void show_setup_plutomain(struct show *s);
