@@ -20,26 +20,18 @@
 #include "demux.h"      /* needs packet.h */
 #include "iface.h"
 
-static void free_md(void *obj, where_t where)
-{
-	struct msg_digest *md = obj;
-	free_chunk_content(&md->raw_packet);
-	free_logger(&md->md_logger, where);
-	iface_endpoint_delref_where(&md->iface, where);
-	pfree(md);
-}
-
 struct msg_digest *alloc_md(struct iface_endpoint *ifp,
 			    const ip_endpoint *sender,
 			    const uint8_t *packet, size_t packet_len,
 			    where_t where)
 {
-	struct msg_digest *md = refcnt_overalloc(struct msg_digest, packet_len, free_md, where);
+	struct msg_digest *md = refcnt_overalloc(struct msg_digest, packet_len, where);
 	md->iface = iface_endpoint_addref_where(ifp, where);
 	md->sender = *sender;
-	md->md_logger = alloc_logger(md, &logger_message_vec, where);
+	md->logger = alloc_logger(md, &logger_message_vec,
+				     /*debugging*/LEMPTY, where);
 	void *buffer = md + 1;
-	init_pbs(&md->packet_pbs, buffer, packet_len, "packet");
+	md->packet_pbs = pbs_in_from_shunk(shunk2(buffer, packet_len), "packet");
 	if (packet != NULL) {
 		memcpy(buffer, packet, packet_len);
 	}
@@ -48,9 +40,9 @@ struct msg_digest *alloc_md(struct iface_endpoint *ifp,
 
 struct msg_digest *clone_raw_md(struct msg_digest *md, where_t where)
 {
-	size_t packet_len = pbs_room(&md->packet_pbs);
+	shunk_t packet = pbs_in_all(&md->packet_pbs);
 	struct msg_digest *clone = alloc_md(md->iface, &md->sender,
-					    md->packet_pbs.start, packet_len,
+					    packet.ptr, packet.len,
 					    where);
 	clone->fake_clone = true;
 	clone->md_inception = threadtime_start();
@@ -64,5 +56,13 @@ struct msg_digest *md_addref_where(struct msg_digest *md, where_t where)
 
 void md_delref_where(struct msg_digest **mdp, where_t where)
 {
-	delref_where(mdp, where);
+	/* need non-NULL so .logger is available */
+	const struct logger *logger = (*mdp != NULL ? (*mdp)->logger : &global_logger);
+	struct msg_digest *md = delref_where(mdp, logger, where);
+	if (md != NULL) {
+		free_chunk_content(&md->raw_packet);
+		free_logger(&md->logger, where);
+		iface_endpoint_delref_where(&md->iface, where);
+		pfree(md);
+	}
 }
