@@ -31,7 +31,7 @@
 #include "constants.h"
 #include "x509.h"
 #include "nss_cert_verify.h"
-#include "lswfips.h" /* for libreswan_fipsmode() */
+#include "fips_mode.h" /* for is_fips_mode() */
 #include "certs.h"
 #include <secder.h>
 #include <secerr.h>
@@ -44,6 +44,9 @@
 #include "root_certs.h"
 #include "ip_info.h"
 #include "log.h"
+#include "log_limiter.h"
+
+bool groundhogday;
 
 static bool crl_is_current(CERTSignedCrl *crl)
 {
@@ -110,7 +113,7 @@ static void log_bad_cert(struct logger *logger, const char *prefix,
 		last_sn = node->cert->subjectName;
 		last_error = node->error;
 		/* ??? we ignore node->depth and node->arg */
-		llog_nss_error_code(RC_LOG_SERIOUS, logger, node->error,
+		llog_nss_error_code(RC_LOG, logger, node->error,
 				    "%s: %s certificate %s invalid",
 				    prefix, usage, node->cert->subjectName);
 	}
@@ -165,8 +168,9 @@ static void set_rev_params(CERTRevocationFlags *rev,
 			      (err) == SEC_ERROR_INADEQUATE_KEY_USAGE)
 
 static bool verify_end_cert(struct logger *logger,
-			    const struct root_certs *root_certs,
+			    const CERTCertList *trustcl,
 			    const struct rev_opts *rev_opts,
+			    PRTime groundhogtime,
 			    CERTCertificate *end_cert)
 {
 	CERTRevocationFlags rev;
@@ -178,6 +182,8 @@ static bool verify_end_cert(struct logger *logger,
 	set_rev_per_meth(&rev, revFlagsLeaf, revFlagsChain);
 	set_rev_params(&rev, rev_opts);
 
+	ldbg(logger, "groundhogtime is %ju", (uintmax_t)groundhogtime);
+
 	CERTValInParam cvin[] = {
 		{
 			.type = cert_pi_revocationFlags,
@@ -189,12 +195,16 @@ static bool verify_end_cert(struct logger *logger,
 		},
 		{
 			.type = cert_pi_trustAnchors,
-			.value = { .pointer = { .chain = root_certs->trustcl, } }
+			.value = { .pointer = { .chain = trustcl, } }
 		},
 		{
 			.type = cert_pi_useOnlyTrustAnchors,
 			.value = { .scalar = { .b = PR_TRUE } }
 		},
+		{
+			.type = cert_pi_date,
+			.value.scalar.time = groundhogtime,
+		},
 		{
 			.type = cert_pi_end
 		}
@@ -216,7 +226,6 @@ static bool verify_end_cert(struct logger *logger,
 	if (DBGP(DBG_BASE)) {
 		DBG_log("%s verifying %s using:", __func__, end_cert->subjectName);
 		unsigned nr = 0;
-		CERTCertList *trustcl = root_certs->trustcl;
 		for (CERTCertListNode *node = CERT_LIST_HEAD(trustcl);
 		     !CERT_LIST_END(node, trustcl);
 		     node = CERT_LIST_NEXT(node)) {
@@ -366,13 +375,20 @@ static void add_decoded_cert(CERTCertDBHandle *handle,
 							PR_TRUE /* copyDER */);
 	if (cert == NULL) {
 		/*
-		 * XXX: need to log something here.  When the certs
-		 * are rejected, pluto stumbles on, eventually
-		 * rejecting the connection for some seamingly
-		 * unrelated reason.
+		 * XXX: need to log something here.
+		 *
+		 * When the certificate payload is rejected pluto
+		 * stumbles on, only to eventually reject the peer's
+		 * auth for some for some seamingly unrelated reason.
 		 */
-		llog_nss_error(RC_LOG_SERIOUS, logger,
-			       "NSS: decoding certificate using CERT_NewTempCertificate() failed");
+		llog_nss_error(RC_LOG, logger,
+			       "NSS: decoding certificate payload using CERT_NewTempCertificate() failed");
+		if (PR_GetError() == SEC_ERROR_REUSED_ISSUER_AND_SERIAL) {
+			lset_t rc_flags = log_limiter_rc_flags(logger, CERTIFICATE_LOG_LIMITER);
+			if (rc_flags != LEMPTY) {
+				llog_pem_bytes(rc_flags, logger, "CERTIFICATE", der_cert.data, der_cert.len);
+			}
+		}
 		return;
 	}
 	dbg("decoded cert: %s", cert->subjectName);
@@ -384,10 +400,10 @@ static void add_decoded_cert(CERTCertDBHandle *handle,
 	 *
 	 * See also RSA_secret_sane() and ECDSA_secret_sane()
 	 */
-	if (libreswan_fipsmode()) {
+	if (is_fips_mode()) {
 		SECKEYPublicKey *pk = CERT_ExtractPublicKey(cert);
 		passert(pk != NULL);
-		unsigned key_bit_size = pk->u.rsa.modulus.len * BITS_PER_BYTE;
+		unsigned key_bit_size = pk->u.rsa.modulus.len * BITS_IN_BYTE;
 		if (pk->keyType == rsaKey && key_bit_size < FIPS_MIN_RSA_KEY_SIZE) {
 			llog(RC_LOG, logger,
 			     "FIPS: rejecting peer cert with key size %u under %u: %s",
@@ -426,29 +442,30 @@ static struct certs *decode_cert_payloads(CERTCertDBHandle *handle,
 	dbg("checking for known CERT payloads");
 	for (struct payload_digest *p = cert_payloads; p != NULL; p = p->next) {
 		enum ike_cert_type cert_type;
-		const char *cert_name;
+		const struct enum_names *cert_names;
 		switch (ike_version) {
 		case IKEv2:
 			cert_type = p->payload.v2cert.isac_enc;
-			cert_name = enum_name_short(&ikev2_cert_type_names, cert_type);
+			cert_names = &ikev2_cert_type_names;
 			break;
 		case IKEv1:
 			cert_type = p->payload.cert.isacert_type;
-			cert_name = enum_name_short(&ike_cert_type_names, cert_type);
+			cert_names = &ike_cert_type_names;
 			break;
 		default:
 			bad_case(ike_version);
 		}
-		if (cert_name == NULL) {
-			llog(RC_LOG_SERIOUS, logger,
+		enum_buf cert_name;
+		if (!enum_name_short(cert_names, cert_type, &cert_name)) {
+			llog(RC_LOG, logger,
 				    "ignoring certificate with unknown type %d",
 				    cert_type);
 			continue;
 		}
 
-		dbg("saving certificate of type '%s'", cert_name);
+		ldbg(logger, "saving certificate of type '%s'", cert_name.buf);
 		/* convert remaining buffer to something nss likes */
-		shunk_t payload_hunk = pbs_in_left_as_shunk(&p->pbs);
+		shunk_t payload_hunk = pbs_in_left(&p->pbs);
 		/* NSS doesn't do const */
 		SECItem payload = {
 			.type = siDERCertBuffer,
@@ -465,12 +482,12 @@ static struct certs *decode_cert_payloads(CERTCertDBHandle *handle,
 			SEC_PKCS7ContentInfo *contents = SEC_PKCS7DecodeItem(&payload, NULL, NULL, NULL, NULL,
 									     NULL, NULL, NULL);
 			if (contents == NULL) {
-				llog(RC_LOG_SERIOUS, logger,
+				llog(RC_LOG, logger,
 					    "Wrapped PKCS7 certificate payload could not be decoded");
 				continue;
 			}
 			if (!SEC_PKCS7ContainsCertsOrCrls(contents)) {
-				llog(RC_LOG_SERIOUS, logger,
+				llog(RC_LOG, logger,
 					    "Wrapped PKCS7 certificate payload did not contain any certificates");
 				SEC_PKCS7DestroyContentInfo(contents);
 				continue;
@@ -483,8 +500,8 @@ static struct certs *decode_cert_payloads(CERTCertDBHandle *handle,
 			break;
 		}
 		default:
-			llog(RC_LOG_SERIOUS, logger,
-				    "ignoring %s certificate payload", cert_name);
+			llog(RC_LOG, logger,
+			     "ignoring %s certificate payload", cert_name.buf);
 			break;
 		}
 	}
@@ -507,6 +524,7 @@ struct verified_certs find_and_verify_certs(struct logger *logger,
 		.cert_chain = NULL,
 		.crl_update_needed = false,
 		.harmless = true,
+		.groundhog = false,
 	};
 
 	if (!pexpect(cert_payloads != NULL)) {
@@ -581,14 +599,53 @@ struct verified_certs find_and_verify_certs(struct logger *logger,
 	}
 
 	logtime_t verify_time = logtime_start(logger);
-	bool end_ok = verify_end_cert(logger, root_certs, rev_opts, end_cert);
+	bool end_ok = verify_end_cert(logger, root_certs->trustcl, rev_opts,
+				      0, end_cert);
+	if (!end_ok && groundhogday) {
+		/*
+		 * Go through the CA certs retrying any with an
+		 * expired time.
+		 */
+		PRTime prnow = PR_Now();
+		for (CERTCertListNode *node = CERT_LIST_HEAD(root_certs->trustcl);
+		     !CERT_LIST_END(node, root_certs->trustcl);
+		     node = CERT_LIST_NEXT(node)) {
+			PRTime not_before, not_after;
+			PRTime groundhogtime = 0;
+			if (CERT_GetCertTimes(node->cert, &not_before, &not_after) != SECSuccess) {
+				continue;
+			}
+			if (LL_CMP(not_after, <, prnow)) {
+				groundhogtime = not_after;
+			} else if (LL_CMP(not_before, >, prnow)) {
+				groundhogtime = not_before;
+			} else {
+				continue;
+			}
+			ldbg(logger, "  retrying groundhog CA: %s", node->cert->subjectName);
+			CERTCertList ground_certs = {
+				.list = PR_INIT_STATIC_CLIST(&ground_certs.list),
+			};
+			CERTCertListNode ground_cert = {
+				.cert = node->cert,
+			};
+			PR_INSERT_LINK(&ground_cert.links, &ground_certs.list);
+
+			if (verify_end_cert(logger, &ground_certs, rev_opts,
+					    groundhogtime, end_cert)) {
+				result.groundhog = true;
+				end_ok = true;
+				break;
+			}
+		}
+	}
 	logtime_stop(&verify_time, "%s() calling verify_end_cert()", __func__);
 	if (!end_ok) {
 		/*
 		 * XXX: preserve verify_end_cert()'s behaviour? only
 		 * send this to the file
 		 */
-		llog(RC_LOG|LOG_STREAM, logger, "NSS: end certificate invalid");
+		llog(LOG_STREAM/*not-whack*/, logger, "NSS: end certificate invalid");
 		release_certs(&result.cert_chain);
 		result.harmless = false;
 		return result;
@@ -601,7 +658,9 @@ struct verified_certs find_and_verify_certs(struct logger *logger,
 	return result;
 }
 
-diag_t cert_verify_subject_alt_name(const CERTCertificate *cert, const struct id *id)
+diag_t cert_verify_subject_alt_name(const char *who,
+				    const CERTCertificate *cert,
+				    const struct id *id)
 {
 	/*
 	 * Get a handle on the certificate's subject alt name.
@@ -611,8 +670,9 @@ diag_t cert_verify_subject_alt_name(const CERTCertificate *cert, const struct id
 					      &subAltName);
 	if (rv != SECSuccess) {
 		id_buf idb;
-		return diag("peer certificate contains no subjectAltName extension to match %s '%s'",
-			    enum_name(&ike_id_type_names, id->kind),
+		enum_buf kb;
+		return diag("%s certificate contains no subjectAltName extension to match %s '%s'",
+			    who, str_enum(&ike_id_type_names, id->kind, &kb),
 			    str_id(id, &idb));
 	}
 
@@ -626,8 +686,9 @@ diag_t cert_verify_subject_alt_name(const CERTCertificate *cert, const struct id
 	if (nameList == NULL) {
 		PORT_FreeArena(arena, PR_FALSE);
 		id_buf idb;
-		return diag("peer certificate subjectAltName extension failed to decode while looking for %s '%s'",
-			    enum_name(&ike_id_type_names, id->kind),
+		enum_buf kb;
+		return diag("%s certificate subjectAltName extension failed to decode while looking for %s '%s'",
+			    who, str_enum(&ike_id_type_names, id->kind, &kb),
 			    str_id(id, &idb));
 	}
 
@@ -711,7 +772,7 @@ diag_t cert_verify_subject_alt_name(const CERTCertificate *cert, const struct id
 			}
 
 			if (c_len == strlen(n_ptr) && strncaseeq(n_ptr, c_ptr, c_len)) {
-				LSWDBGP(DBG_BASE, buf) {
+				LDBGP_JAMBUF(DBG_BASE, &global_logger, buf) {
 					jam(buf, "peer certificate subjectAltname '%s' matched '", ascii_id),
 					jam_sanitized_bytes(buf, current->name.other.data,
 							    current->name.other.len);
@@ -735,8 +796,8 @@ diag_t cert_verify_subject_alt_name(const CERTCertificate *cert, const struct id
 			if (hunk_memeq(as, current->name.other.data,
 				       current->name.other.len)) {
 				address_buf b;
-				dbg("peer certificate subjectAltname matches address %s",
-				    str_address(&myip, &b));
+				dbg("%s certificate subjectAltname matches address %s",
+				    who, str_address(&myip, &b));
 				PORT_FreeArena(arena, PR_FALSE);
 				return NULL;
 			}
@@ -757,8 +818,9 @@ diag_t cert_verify_subject_alt_name(const CERTCertificate *cert, const struct id
 	 */
 	PORT_FreeArena(arena, PR_FALSE);
 	esb_buf esb;
-	return diag("peer certificate subjectAltName extension does not match %s '%s'",
-		    enum_show(&ike_id_type_names, id->kind, &esb), ascii_id);
+	return diag("%s certificate subjectAltName extension does not match %s '%s'",
+		    who, str_enum(&ike_id_type_names, id->kind, &esb),
+		    ascii_id);
 }
 
 SECItem *nss_pkcs7_blob(const struct cert *cert, bool send_full_chain)
