@@ -46,6 +46,7 @@ struct verified_certs {
 	struct pubkey_list *pubkey_db;
 	bool crl_update_needed;
 	bool harmless;
+	bool groundhog;
 };
 
 struct verified_certs find_and_verify_certs(struct logger *log,
@@ -55,9 +56,11 @@ struct verified_certs find_and_verify_certs(struct logger *log,
 					    struct root_certs *root_cert,
 					    const struct id *keyid);
 
-extern bool cert_VerifySubjectAltName(const CERTCertificate *cert,
-				      const struct id *id, struct logger *logger);
+extern diag_t cert_verify_subject_alt_name(const char *who, const CERTCertificate *cert,
+					   const struct id *id);
 
-extern SECItem *nss_pkcs7_blob(CERTCertificate *cert, bool send_full_chain);
+extern SECItem *nss_pkcs7_blob(const struct cert *cert, bool send_full_chain);
+
+extern bool groundhogday;
 
 #endif /* NSS_CERT_VFY_H */
