@@ -27,9 +27,9 @@
 
 /* note: returning a diag is fatal! */
 diag_t init_nss_ocsp(const char *responder_url, const char *trust_cert_name,
-		     int timeout, bool strict, int cache_size,
-		     int cache_min, int cache_max, bool ocsp_post,
-		     struct logger *logger)
+		     deltatime_t timeout, bool strict, int cache_size,
+		     deltatime_t cache_min, deltatime_t cache_max,
+		     bool ocsp_post, struct logger *logger)
 {
 	SECStatus rv;
 
@@ -64,11 +64,11 @@ diag_t init_nss_ocsp(const char *responder_url, const char *trust_cert_name,
 			if (rv != SECSuccess) {
 				int err = PORT_GetError();
 				if (err == SEC_ERROR_OCSP_RESPONDER_CERT_INVALID) {
-					llog(RC_LOG_SERIOUS, logger,
+					llog(RC_LOG, logger,
 						    "responder certificate %s is invalid. please verify its keyUsage extensions for OCSP",
 						    trust_cert_name);
 				} else {
-					llog_nss_error(RC_LOG_SERIOUS, logger,
+					llog_nss_error(RC_LOG, logger,
 						       "error enabling OCSP default responder");
 				}
 			}
@@ -85,11 +85,11 @@ diag_t init_nss_ocsp(const char *responder_url, const char *trust_cert_name,
 		}
 	}
 
-	if (timeout != 0) {
-		dbg("OCSP timeout of %d seconds", timeout);
-		if (CERT_SetOCSPTimeout(timeout) != SECSuccess) {
+	if (deltatime_cmp(timeout, >, deltatime_zero)) {
+		dbg("OCSP timeout of %ju seconds", deltasecs(timeout));
+		if (CERT_SetOCSPTimeout(deltasecs(timeout)) != SECSuccess) {
 			/* don't shoot pluto over this */
-			llog_nss_error(RC_LOG_SERIOUS, logger, "error setting OCSP timeout");
+			llog_nss_error(RC_LOG, logger, "error setting OCSP timeout to %ju", deltasecs(timeout));
 		}
 	}
 
@@ -112,23 +112,27 @@ diag_t init_nss_ocsp(const char *responder_url, const char *trust_cert_name,
 
 	if (rv != SECSuccess) {
 		/* don't shoot pluto over this */
-		llog_nss_error(RC_LOG_SERIOUS, logger, "error enabling OCSP POST method");
+		llog_nss_error(RC_LOG, logger, "error enabling OCSP POST method");
 	}
 
 	/*
-	 * NSS uses 0 for unlimited and -1 for disabled. We use 0 for disabled
-	 * and just a large number for a large cache
+	 * NSS uses 0 for unlimited and -1 for disabled. We use 0 for
+	 * disabled and just a large number for a large cache.
 	 */
-	if (cache_max == 0) {
-		cache_max = -1;
+	int nss_max = deltasecs(cache_max);
+	if (nss_max == 0) {
+		nss_max = -1;
 	}
 
-	rv = CERT_OCSPCacheSettings(cache_size, cache_min, cache_max);
+	rv = CERT_OCSPCacheSettings(cache_size, deltasecs(cache_min), nss_max);
 	if (rv != SECSuccess) {
 		/* don't shoot pluto over this */
-		llog_nss_error(RC_LOG_SERIOUS, logger,
-			       "error setting OCSP cache parameters (size=%d, min=%d, max=%d)",
-			       cache_size, cache_min, cache_max);
+		deltatime_buf minb, maxb;
+		llog_nss_error(RC_LOG, logger,
+			       "error setting OCSP cache parameters (size=%d, min=%s, max=%s)",
+			       cache_size,
+			       str_deltatime(cache_min, &minb),
+			       str_deltatime(cache_max, &maxb));
 	}
 
 	return NULL;
