@@ -17,13 +17,14 @@
 #define _NSS_OCSP_H
 
 #include "diag.h"
+#include "deltatime.h"
 
 struct logger;
 
 
 extern diag_t init_nss_ocsp(const char *responder_url, const char *trust_cert_name,
-			    int timeout, bool strict, int cache_size,
-			    int cache_min, int cache_max, bool ocsp_post,
-			    struct logger *logger);
+			    deltatime_t timeout, bool strict, int cache_size,
+			    deltatime_t cache_min, deltatime_t cache_max,
+			    bool ocsp_post, struct logger *logger);
 
 #endif /* _NSS_OCSP_H */
