@@ -9,6 +9,7 @@
  * Copyright (C) 2012-2013 Paul Wouters <pwouters@redhat.com>
  * Copyright (C) 2013 Wolfgang Nothdurft <wolfgang@linogate.de>
  * Copyright (C) 2018-2019 Andrew Cagney
+ * Copyright (C) 2020 Nupur Agrawal <nupur202000@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -253,21 +254,35 @@ diag_t pbs_in_shunk(struct pbs_in *pbs, size_t len, shunk_t *shunk,
  */
 extern struct pbs_out open_pbs_out(const char *name, uint8_t *buffer,
 				   size_t sizeof_buffer, struct logger *logger);
-extern void close_output_pbs(struct pbs_out *pbs);
 
-struct small_pbs_out {
+bool close_pbs_out(struct pbs_out *pbs);
+#define close_output_pbs close_pbs_out
+
+/*
+ * For sending packets (such as notifications) that won't fragment.
+ *
+ * MIN_MAX_UDP_DATA_v[46] is the largest a UDP packet can be before it
+ * can be fragmented in transit.  i.e., UDP messages smaller than that
+ * don't fragment..
+ *
+ * Filling this buffer is sure sign of code that needs to fragment the
+ * message (IKEv2 code uses a code path that always fragments when
+ * need; IKEv1 does not).
+ */
+
+struct fragment_pbs_out {
 	struct pbs_out pbs;
-	uint8_t buffer[1024];
+	uint8_t buffer[PMAX(MIN_MAX_UDP_DATA_v4, MIN_MAX_UDP_DATA_v6)];
 };
 
-void open_small_pbs_out(const char *name, struct small_pbs_out *pbs, struct logger *logger);
+bool open_fragment_pbs_out(const char *name, struct fragment_pbs_out *pbs, struct logger *logger);
 
 struct large_pbs_out {
 	struct pbs_out pbs;
 	uint8_t buffer[MAX_OUTPUT_UDP_SIZE];
 };
 
-void open_large_pbs_out(const char *name, struct large_pbs_out *pbs, struct logger *logger);
+bool open_large_pbs_out(const char *name, struct large_pbs_out *pbs, struct logger *logger);
 
 /*
  * Map/clone the current contents (i.e., everything written so far)
@@ -1222,7 +1237,6 @@ extern struct_desc ikev2_skf_desc;
 extern struct_desc ikev2_vendor_id_desc;
 
 
-
 /* union of all payloads */
 
 union payload {
@@ -1273,6 +1287,11 @@ struct ikev2_notify_ipcomp_data {
 };
 extern struct_desc ikev2notify_ipcomp_data_desc;
 
+struct ikev2_ticket_lifetime {
+    uint32_t sr_lifetime;
+};
+extern struct_desc ikev2_ticket_lifetime_desc;
+
 extern struct_desc sec_ctx_desc;
 
 /*
