@@ -45,6 +45,7 @@
 #include "pluto_x509.h"
 #include "instantiate.h"
 #include "orient.h"		/* for oriented()! */
+#include "ip_info.h"
 
 /*
  * This is to support certificates with SAN using wildcard, eg SAN
@@ -101,18 +102,23 @@ static bool idr_wildmatch(const struct host_end *this, const struct id *idr, str
 }
 
 /*
- * Extracts the peer's ca from the chained list of public keys.
+ * Given ID, find and return the CA that issued the pubkey in the
+ * PUBKEY_DB.
+ *
+ * XXX: why does this only look at pubkey_type_rsa?
  */
-static asn1_t get_peer_ca(struct pubkey_list *const *pubkey_db,
-			   const struct id *peer_id)
+static asn1_t get_ca(struct pubkey_list *const *pubkey_db,
+		     const struct id *id)
 {
 	struct pubkey_list *p;
 
 	for (p = *pubkey_db; p != NULL; p = p->next) {
 		struct pubkey *key = p->key;
-		if (key->content.type == &pubkey_type_rsa && same_id(peer_id, &key->id))
+		if (key->content.type == &pubkey_type_rsa && same_id(id, &key->id)) {
 			return key->issuer;
+		}
 	}
+
 	return null_shunk;
 }
 
@@ -170,25 +176,22 @@ static asn1_t get_peer_ca(struct pubkey_list *const *pubkey_db,
  * as nothing already used is changed.
  */
 
-#define dbg_rhc(FORMAT, ...) pdbg(ike->sa.logger, "rhc:%*s "FORMAT, indent*2, "", ##__VA_ARGS__)
-
 struct score {
-	bool matching_peer_id;
-	int our_pathlen;
-	int peer_pathlen;
+	bool initiator_id_matched;
+	int v1_requested_ca_pathlen;
+	int initiator_ca_pathlen;
 	int wildcards;
 	struct connection *connection;
 };
 
-static bool score_host_connection(unsigned indent,
-				  const struct ike_sa *ike,
+static bool score_host_connection(const struct ike_sa *ike,
 				  lset_t proposed_authbys,
-				  const struct id *peer_id,
-				  const struct id *tarzan_id,
-				  asn1_t peer_ca,
-				  struct score *score)
+				  const struct id *initiator_id,/*IDi*/
+				  const struct id *responder_id,/*IDr*/
+				  asn1_t initiator_ca,
+				  struct score *score,
+				  struct verbose verbose)
 {
-	const generalName_t *requested_ca = ike->sa.st_v1_requested_ca;
 	struct connection *c = ike->sa.st_connection;
 	struct connection *d = score->connection;
 	PEXPECT(ike->sa.logger, oriented(d));
@@ -208,18 +211,27 @@ static bool score_host_connection(unsigned indent,
 
 	if (c != d && is_instance(d) && d->remote->host.id.kind == ID_NULL) {
 		connection_buf cb;
-		dbg_rhc("skipping ID_NULL instance "PRI_CONNECTION"",
-			pri_connection(d, &cb));
+		vdbg("skipping ID_NULL instance "PRI_CONNECTION"",
+		     pri_connection(d, &cb));
 		return false;
 	}
 
 	if (ike->sa.st_remote_certs.groundhog && !d->remote->config->host.groundhog) {
 		connection_buf cb;
-		dbg_rhc("skipping non-groundhog instance "PRI_CONNECTION"",
-			pri_connection(d, &cb));
+		vdbg("skipping non-groundhog instance "PRI_CONNECTION"",
+		     pri_connection(d, &cb));
 		return false;
 	}
 
+	if (ike->sa.st_v2_resume_session != NULL) {
+		if (!d->config->session_resumption) {
+			connection_buf cb;
+			vdbg("skipping non-IKE_SESSION_RESUME connection "PRI_CONNECTION"",
+			     pri_connection(d, &cb));
+			return false;
+		}
+	}
+
 	/*
 	 * An Opportunistic connection is never better.
 	 *
@@ -229,8 +241,8 @@ static bool score_host_connection(unsigned indent,
 
 	if (c != d && is_opportunistic(d)) {
 		connection_buf cb;
-		dbg_rhc("skipping opportunistic connection "PRI_CONNECTION"",
-			pri_connection(d, &cb));
+		vdbg("skipping opportunistic connection "PRI_CONNECTION"",
+		     pri_connection(d, &cb));
 		return false;
 	}
 
@@ -241,8 +253,8 @@ static bool score_host_connection(unsigned indent,
 
 	if (is_labeled_child(d)) {
 		connection_buf cb;
-		dbg_rhc("skipping labeled child "PRI_CONNECTION,
-			pri_connection(d, &cb));
+		vdbg("skipping labeled child "PRI_CONNECTION,
+		     pri_connection(d, &cb));
 		return false;
 	}
 
@@ -252,8 +264,8 @@ static bool score_host_connection(unsigned indent,
 
 	if (is_group(d)) {
 		connection_buf cb;
-		dbg_rhc("skipping group template connection "PRI_CONNECTION,
-			pri_connection(d, &cb));
+		vdbg("skipping group template connection "PRI_CONNECTION,
+		     pri_connection(d, &cb));
 		return false;
 	}
 
@@ -266,39 +278,39 @@ static bool score_host_connection(unsigned indent,
 
 	if (d->local->host.config->xauth.server != c->local->host.config->xauth.server) {
 		/* Disallow IKEv2 CP or IKEv1 XAUTH mismatch */
-		dbg_rhc("skipping because mismatched xauth_server");
+		vdbg("skipping because mismatched xauth_server");
 		return false;
 	}
 
 	if (d->local->host.config->xauth.client != c->local->host.config->xauth.client) {
 		/* Disallow IKEv2 CP or IKEv1 XAUTH mismatch */
-		dbg_rhc("skipping because mismatched xauth_client");
+		vdbg("skipping because mismatched xauth_client");
 		return false;
 	}
 
 	/*
-	 * 'You Tarzan, me Jane' check based on
-	 * received IDr (remember, this is the
-	 * responder).
+	 * 'You Tarzan, me Jane' check based on received IDr
+	 * (remember, this is the responder being told our ID by the
+	 * initiator).
 	 */
-	if (tarzan_id != NULL) {
+	if (responder_id != NULL && responder_id->kind != ID_NONE) {
 		id_buf tzb;
 		esb_buf tzesb;
-		dbg_rhc("peer expects us to be %s (%s) according to its IDr payload",
-			str_id(tarzan_id, &tzb),
-			str_enum(&ike_id_type_names, tarzan_id->kind, &tzesb));
+		vdbg("peer expects us to be %s (%s) according to its IDr (tarzan) payload",
+		     str_id(responder_id, &tzb),
+		     str_enum(&ike_id_type_names, responder_id->kind, &tzesb));
 		id_buf usb;
 		esb_buf usesb;
-		dbg_rhc("this connection's local id is %s (%s)",
-			str_id(&d->local->host.id, &usb),
-			str_enum(&ike_id_type_names, d->local->host.id.kind, &usesb));
+		vdbg("this connection's local id is %s (%s)",
+		     str_id(&d->local->host.id, &usb),
+		     str_enum(&ike_id_type_names, d->local->host.id.kind, &usesb));
 		/* ??? pexpect(d->spd->spd_next == NULL); */
-		if (!idr_wildmatch(&d->local->host, tarzan_id, ike->sa.logger)) {
-			dbg_rhc("skipping because peer IDr payload does not match our expected ID");
+		if (!idr_wildmatch(&d->local->host, responder_id, ike->sa.logger)) {
+			vdbg("skipping because peer IDr (tarzan) payload does not match our expected ID");
 			return false;
 		}
 	} else {
-		dbg_rhc("no IDr payload received from peer");
+		vdbg("no IDr (tarzan) payload received from peer, skipping check");
 	}
 
 	/*
@@ -308,24 +320,24 @@ static bool score_host_connection(unsigned indent,
 	switch (ike->sa.st_ike_version) {
 	case IKEv1:
 		if (d->config->aggressive) {
-			dbg_rhc("skipping because AGGRESSIVE isn't right");
+			vdbg("skipping because AGGRESSIVE isn't right");
 			return false;	/* differ about aggressive mode */
 		}
 		if (LHAS(proposed_authbys, AUTH_PSK)) {
 			if (!(d->remote->host.config->auth == AUTH_PSK)) {
 				/* there needs to be a key */
-				dbg_rhc("skipping because no PSK in POLICY");
+				vdbg("skipping because no PSK in POLICY");
 				return false;
 			}
 			if (get_connection_psk(d) == NULL) {
 				/* there needs to be a key */
-				dbg_rhc("skipping because PSK and no secret");
+				vdbg("skipping because PSK and no secret");
 				return false; /* no secret */
 			}
 		}
 		if (LHAS(proposed_authbys, AUTH_RSASIG)) {
 			if (!(d->remote->host.config->auth == AUTH_RSASIG)) {
-				dbg_rhc("skipping because not RSASIG in POLICY");
+				vdbg("skipping because not RSASIG in POLICY");
 				return false;	/* no key */
 			}
 			if (get_local_private_key(d, &pubkey_type_rsa,
@@ -334,7 +346,7 @@ static bool score_host_connection(unsigned indent,
 				 * We must at least be able to find
 				 * our private key.
 				 */
-				dbg_rhc("skipping because RSASIG and no private key");
+				vdbg("skipping because RSASIG and no private key");
 				return false;	/* no key */
 			}
 		}
@@ -354,7 +366,7 @@ static bool score_host_connection(unsigned indent,
 		 * ECDSA?
 		 */
 		if (!LHAS(proposed_authbys, d->remote->host.config->auth)) {
-			dbg_rhc("skipping because mismatched authby");
+			vdbg("skipping because mismatched authby");
 			return false;
 		}
 		/* check that the chosen one has a key */
@@ -368,33 +380,33 @@ static bool score_host_connection(unsigned indent,
 			if (get_connection_psk(d) == NULL) {
 				/* need a key */
 #if 0
-				dbg_rhc("skipping because PSK and no secret");
+				vdbg("skipping because PSK and no secret");
 				return false; /* no secret */
 #else
-				dbg_rhc("has no PSK; why?");
+				vdbg("has no PSK; why?");
 			}
 #endif
 			break;
 		case AUTH_RSASIG:
 			if (get_local_private_key(d, &pubkey_type_rsa,
 						  ike->sa.logger) == NULL) {
-				dbg_rhc("skipping because RSASIG and no private key");
+				vdbg("skipping because RSASIG and no private key");
 				return false;	/* no key */
 			}
 			break;
 		case AUTH_ECDSA:
 			if (get_local_private_key(d, &pubkey_type_ecdsa,
 						  ike->sa.logger) == NULL) {
-				dbg_rhc("skipping because ECDSA and no private key");
+				vdbg("skipping because ECDSA and no private key");
 				return false;	/* no key */
 			}
 			break;
 		default:
 		{
 			lset_buf eb;
-			dbg_rhc("%s so no authby checks performed",
-				str_lset_short(&keyword_auth_names, "+",
-					       proposed_authbys, &eb));
+			vdbg("%s so no authby checks performed",
+			     str_lset_short(&keyword_auth_names, "+",
+					    proposed_authbys, &eb));
 			break;
 		}
 		}
@@ -410,55 +422,61 @@ static bool score_host_connection(unsigned indent,
 	 * were used.
 	 */
 
-	score->matching_peer_id = match_id("rhc:       ", peer_id,
-					   &d->remote->host.id,
-					   &score->wildcards);
+	score->initiator_id_matched = match_id(initiator_id,
+					       &d->remote->host.id,
+					       &score->wildcards,
+					       verbose);
 
 	/*
-	 * Check if peer_id matches, exactly or after
+	 * Check if initiator_id matches, exactly or after
 	 * instantiation.
 	 *
 	 * Check for the match but also check to see
 	 * if it's the %fromcert + peer id match
 	 * result. - matt
 	 */
-	if (!score->matching_peer_id) {
+	if (!score->initiator_id_matched) {
 		/* must be checking certs */
 		if (d->remote->host.id.kind != ID_FROMCERT) {
-			dbg_rhc("skipping because peer_id does not match and that.id.kind is not a cert");
+			vdbg("skipping because initiator_id does not match and that.id.kind is not a cert");
 			return false;
 		}
 	}
 
+	/*
+	 * IKEv2 doesn't have v1_requested_ca so can be ignored.
+	 *
+	 * match_v1_requested_ca() succeeds when there's no
+	 * .st_v1_requested_ca to match.
+	 */
+	score->v1_requested_ca_pathlen = 0;
+	if (ike->sa.st_ike_version == IKEv1) {
+		if (!match_v1_requested_ca(ike, d->local->host.config->ca,
+					   &score->v1_requested_ca_pathlen,
+					   verbose)) {
+			vdbg("skipping because match_v1_requested_ca() failed");
+			return false;
+		}
+
+		vdbg("v1_requested_ca_pathlen=%d", score->v1_requested_ca_pathlen);
+	}
+
 	/*
 	 * XXX: When there are no certificates at all
-	 * (PEER_CA and THAT.CA are NULL; REQUESTED_CA
+	 * (INITIATOR_CA and THAT.CA are NULL; REQUESTED_CA
 	 * is NULL), these lookups return TRUE and
 	 * *_pathlen==0 - a perfect match.
 	 */
-	bool matching_peer_ca = trusted_ca(peer_ca,
-					   ASN1(d->remote->host.config->ca),
-					   &score->peer_pathlen);
-	bool matching_requested_ca = match_requested_ca(requested_ca,
-							d->local->host.config->ca,
-							&score->our_pathlen);
-	dbg_rhc("matching_peer_ca=%s(%d)/matching_request_ca=%s(%d))",
-		bool_str(matching_peer_ca), score->peer_pathlen,
-		bool_str(matching_requested_ca), score->our_pathlen);
-	indent++;
-
-	/*
-	 * Both matching_peer_ca and
-	 * matching_requested_ca are required.
-	 *
-	 * XXX: Remember, when there are no
-	 * certificates, both are forced to TRUE.
-	 */
-	if (!matching_peer_ca || !matching_requested_ca) {
-		dbg_rhc("skipping because !matching_peer_ca || !matching_requested_ca");
+	if (!trusted_ca(initiator_ca,
+			ASN1(d->remote->host.config->ca),
+			&score->initiator_ca_pathlen,
+			verbose)) {
+		vdbg("skipping because trusted_ca() failed");
 		return false;
 	}
 
+	vdbg("initiator_ca_pathlen=%d", score->initiator_ca_pathlen);
+
 	/*
 	 * Paul: We need to check all the other relevant policy bits,
 	 * like compression, pfs, etc.
@@ -479,10 +497,10 @@ static bool score_host_connection(unsigned indent,
 static bool exact_id_match(struct score score)
 {
 	return (score.connection != NULL &&
-		score.matching_peer_id &&
+		score.initiator_id_matched &&
 		score.wildcards == 0 &&
-		score.peer_pathlen == 0 &&
-		score.our_pathlen == 0 &&
+		score.initiator_ca_pathlen == 0 &&
+		score.v1_requested_ca_pathlen == 0 &&
 		(is_permanent(score.connection) ||
 		 is_instance(score.connection)));
 }
@@ -504,18 +522,18 @@ static bool better_score(struct score best, struct score score, struct logger *l
 	/*
 	 * ??? the logic involving *_pathlen looks wrong.
 	 *
-	 * ??? which matters more peer_pathlen or our_pathlen
+	 * ??? which matters more initiator_ca_pathlen or v1_requested_ca_pathlen
 	 * minimization?
 	 *
 	 * XXX: presumably peer as we're more worried about
 	 * authenticating the peer using the best match?
 	 */
-	if (score.peer_pathlen != best.peer_pathlen) {
-		return (score.peer_pathlen < best.peer_pathlen);
+	if (score.initiator_ca_pathlen != best.initiator_ca_pathlen) {
+		return (score.initiator_ca_pathlen < best.initiator_ca_pathlen);
 	}
 
-	if (score.our_pathlen != best.our_pathlen) {
-		return (score.our_pathlen < best.our_pathlen);
+	if (score.v1_requested_ca_pathlen != best.v1_requested_ca_pathlen) {
+		return (score.v1_requested_ca_pathlen < best.v1_requested_ca_pathlen);
 	}
 
 	/*
@@ -536,13 +554,12 @@ static bool better_score(struct score best, struct score score, struct logger *l
 	return false;
 }
 
-static struct connection *refine_host_connection_on_responder(int indent,
-							      const struct ike_sa *ike,
+static struct connection *refine_host_connection_on_responder(const struct ike_sa *ike,
 							      lset_t proposed_authbys,
-							      const struct id *peer_id,
-							      const struct id *tarzan_id)
+							      const struct id *initiator_id,
+							      const struct id *responder_id,
+							      struct verbose verbose)
 {
-
 	struct connection *c = ike->sa.st_connection;
 
 	PASSERT(ike->sa.logger, !LHAS(proposed_authbys, AUTH_NEVER));
@@ -560,9 +577,9 @@ static struct connection *refine_host_connection_on_responder(int indent,
 	 * Find the PEER's CA, check the per-state DB first.
 	 */
 	PEXPECT(ike->sa.logger, ike->sa.st_remote_certs.processed);
-	asn1_t peer_ca = get_peer_ca(&ike->sa.st_remote_certs.pubkey_db, peer_id);
-	if (hunk_isempty(peer_ca)) {
-		peer_ca = get_peer_ca(&pluto_pubkeys, peer_id);
+	asn1_t initiator_ca = get_ca(&ike->sa.st_remote_certs.pubkey_db, initiator_id);
+	if (hunk_isempty(initiator_ca)) {
+		initiator_ca = get_ca(&pluto_pubkeys, initiator_id);
 	}
 
 	/*
@@ -581,46 +598,54 @@ static struct connection *refine_host_connection_on_responder(int indent,
 	 *
 	 * We need to match:
 	 *
-	 * - peer_id (slightly complicated by instantiation)
+	 * - initiator_id (slightly complicated by instantiation)
 	 * - if PSK auth, the key must not change (we used it to decode message)
 	 * - policy-as-used must be acceptable to new connection
 	 * - if initiator, also:
 	 *   + our ID must not change (we sent it in previous message)
 	 *   + our RSA key must not change (we used in in previous message)
-	 *
-	 * PASS 0: Score the existing connection to kick-start the
-	 * search and check for an instant winner
-	 *
-	 * PASS 1: Match anything with the exact same SRC->DST. This
-	 * list contains instantiated templates and oriented permanent
-	 * connections.
-	 *
-	 * PASS 2: Match matching SRC->%any.  This list contains
-	 * oriented template connections (since the remote address is
-	 * %any).
 	 */
 
 	struct score best = {0};
 
+	/*
+	 * PASS 0: Score the existing connection to kick-start the
+	 * search and check for an instant winner
+	 */
+
 	struct score c_score = { .connection = c, };
-	if (score_host_connection(indent, ike,
-				  proposed_authbys,
-				  peer_id, tarzan_id, peer_ca,
-				  &c_score)) {
+	if (score_host_connection(ike, proposed_authbys,
+				  initiator_id, responder_id,
+				  initiator_ca,
+				  &c_score, verbose)) {
 		best = c_score;
 		if (exact_id_match(best)) {
-			dbg_rhc("returning initial connection because exact (peer) ID match");
+			vdbg("returning initial connection because exact (peer) ID match");
 			return best.connection;
 		}
 	}
 
+	/*
+	 * PASS 1: Match anything with the exact same
+	 * initiator/remote==SRC->responder/local==DST
+	 *
+	 * This list contains instantiated templates and oriented
+	 * permanent connections.
+	 *
+	 * PASS 2: Match matching with a wildcard remote
+	 * initiator/remote==%ANY->responder/local==DST
+	 *
+	 * This list contains oriented template connections (since the
+	 * remote address is %any).
+	 */
+
 	ip_address local = c->iface->local_address;
 	FOR_EACH_THING(remote, endpoint_address(ike->sa.st_remote_endpoint), unset_address) {
 
-		indent = 1;
+		verbose.level = 1;
 		address_buf lb, rb;
-		dbg_rhc("trying connections matching %s->%s",
-			str_address(&local, &lb), str_address(&remote, &rb));
+		vdbg("trying connections matching %s->%s",
+		     str_address(&local, &lb), str_address(&remote, &rb));
 
 		struct connection_filter hpf = {
 			.host_pair = {
@@ -630,10 +655,11 @@ static struct connection *refine_host_connection_on_responder(int indent,
 			.ike_version = c->config->ike_version,
 			.search = {
 				.order = OLD2NEW,
-				.logger = ike->sa.logger,
+				.verbose.logger = ike->sa.logger,
 				.where = HERE,
 			},
 		};
+
 		while (next_connection(&hpf)) {
 			struct connection *d = hpf.c;
 			if (c == d) {
@@ -642,17 +668,17 @@ static struct connection *refine_host_connection_on_responder(int indent,
 			}
 
 			connection_buf b2;
-			indent = 2;
-			dbg_rhc("checking "PRI_CONNECTION, pri_connection(d, &b2));
-			indent++;
+			verbose.level = 2;
+			vdbg("checking "PRI_CONNECTION, pri_connection(d, &b2));
+			verbose.level++;
 
 			struct score score = {
 				.connection = d,
 			};
-			if (!score_host_connection(indent, ike,
-						   proposed_authbys,
-						   peer_id, tarzan_id, peer_ca,
-						   &score)) {
+			if (!score_host_connection(ike, proposed_authbys,
+						   initiator_id, responder_id,
+						   initiator_ca,
+						   &score, verbose)) {
 				continue;
 			}
 
@@ -665,8 +691,8 @@ static struct connection *refine_host_connection_on_responder(int indent,
 			 */
 			if (exact_id_match(score)) {
 				connection_buf dcb;
-				dbg_rhc("returning "PRI_CONNECTION" because exact (peer) ID match",
-					pri_connection(d, &dcb));
+				vdbg("returning "PRI_CONNECTION" because exact (peer) ID match",
+				     pri_connection(d, &dcb));
 				return d;
 			}
 
@@ -676,14 +702,16 @@ static struct connection *refine_host_connection_on_responder(int indent,
 			 * exact match doesn't come along.
 			 *
 			 * ??? the logic involving *_pathlen looks wrong.
-			 * ??? which matters more peer_pathlen or our_pathlen minimization?
+			 *
+			 * ??? which matters more initiator_ca_pathlen
+			 * or v1_requested_ca_pathlen minimization?
 			 */
 			if (better_score(best, score, ike->sa.logger)) {
 				connection_buf cib;
-				dbg_rhc("picking new best "PRI_CONNECTION" (wild=%d, peer_pathlen=%d/our=%d)",
-					pri_connection(d, &cib),
-					score.wildcards, score.peer_pathlen,
-					score.our_pathlen);
+				vdbg("picking new best "PRI_CONNECTION" (wild=%d, initiator_ca_pathlen=%d/our=%d)",
+				     pri_connection(d, &cib),
+				     score.wildcards, score.initiator_ca_pathlen,
+				     score.v1_requested_ca_pathlen);
 				best = score;
 			}
 		}
@@ -693,27 +721,29 @@ static struct connection *refine_host_connection_on_responder(int indent,
 
 bool refine_host_connection_of_state_on_responder(struct ike_sa *ike,
 						  lset_t proposed_authbys,
-						  const struct id *peer_id,
-						  const struct id *tarzan_id)
+						  const struct id *initiator_id,
+						  const struct id *responder_id)
 {
-	int indent = 0;
 	connection_buf cib;
-	dbg_rhc("looking for an %s connection more refined than "PRI_CONNECTION"",
-		ike->sa.st_connection->config->ike_info->version_name,
-	    pri_connection(ike->sa.st_connection, &cib));
-	indent = 1;
+	VERBOSE_DBGP(DBG_BASE, ike->sa.logger,
+		     "looking for an %s connection more refined than "PRI_CONNECTION"",
+		     ike->sa.st_connection->config->ike_info->version_name,
+		     pri_connection(ike->sa.st_connection, &cib));
+	verbose.prefix = "rhc:";
+	verbose.level = 1;
 
-	struct connection *r = refine_host_connection_on_responder(indent, ike,
+	struct connection *r = refine_host_connection_on_responder(ike,
 								   proposed_authbys,
-								   peer_id, tarzan_id);
+								   initiator_id, responder_id,
+								   verbose);
 	if (r == NULL) {
-		dbg_rhc("returning FALSE because nothing is sufficiently refined");
+		vdbg("returning FALSE because nothing is sufficiently refined");
 		return false;
 	}
 
 	connection_buf bfb;
-	dbg_rhc("returning TRUE as "PRI_CONNECTION" is most refined",
-		pri_connection(r, &bfb));
+	vdbg("returning TRUE as "PRI_CONNECTION" is most refined",
+	     pri_connection(r, &bfb));
 
 	if (r != ike->sa.st_connection) {
 		/*
@@ -746,9 +776,8 @@ bool refine_host_connection_of_state_on_responder(struct ike_sa *ike,
 			 * ID.
 			 */
 			pexpect(is_template(r));
-			r = rw_responder_refined_instantiate(r, ike->sa.st_connection->remote->host.addr,
-							    NULL/*not-yet-known*/,
-							    peer_id, HERE);
+			r = rw_responder_id_instantiate(r, ike->sa.st_connection->remote->host.addr,
+							initiator_id, HERE);
 		} else {
 			r = connection_addref(r, ike->sa.logger);
 		}
@@ -760,8 +789,8 @@ bool refine_host_connection_of_state_on_responder(struct ike_sa *ike,
 	}
 
 	connection_buf bcb;
-	dbg_rhc("most refined is "PRI_CONNECTION,
-		pri_connection(ike->sa.st_connection, &bcb));
+	vdbg("most refined is "PRI_CONNECTION,
+	     pri_connection(ike->sa.st_connection, &bcb));
 	return true;
 }
 
@@ -799,6 +828,13 @@ diag_t update_peer_id_certs(struct ike_sa *ike)
        return NULL;
 }
 
+/*
+ * IKEv2: PEER_ID could be either IDi or IDr, when non-NULL TARZAN_ID
+ * is IDr on responder.
+ *
+ * IKEv1: PEER_ID is the peer id.
+ */
+
 diag_t update_peer_id(struct ike_sa *ike, const struct id *peer_id, const struct id *tarzan_id)
 {
 	if (ike->sa.st_remote_certs.verified != NULL) {
@@ -841,3 +877,143 @@ diag_t update_peer_id(struct ike_sa *ike, const struct id *peer_id, const struct
 
 	return NULL;
 }
+
+bool compare_connection_id(const struct connection *c,
+			   const struct connection *d,
+			   struct connection_id_score *score,
+			   struct verbose verbose)
+{
+	if (c->config->connalias != NULL &&
+	    d->config->connalias != NULL &&
+	    streq(c->config->connalias, d->config->connalias)) {
+		/*
+		 * conns created as aliases from the same
+		 * source have identical ID/CA.
+		 */
+		vdbg("connalias %s match, skipping ID check",
+		     c->config->connalias);
+		return true;
+	}
+
+	/* local */
+	if (!same_id(&c->local->host.id, &d->local->host.id)) {
+		id_buf cb, db;
+		vdbg("skipping connection, local ID %s needs to be the same as %s",
+		     str_id(&d->local->host.id, &db),
+		     str_id(&c->local->host.id, &cb));
+		return false;
+	}
+
+	/* remote */
+	if (!match_id(&c->remote->host.id, &d->remote->host.id,
+		      &score->wildcards, verbose)) {
+		id_buf cb, db;
+		vdbg("skipping connection, remote ID %s needs to match %s",
+		     str_id(&d->remote->host.id, &db),
+		     str_id(&c->remote->host.id, &cb));
+		return false;
+	}
+
+	/* remote */
+	if (!trusted_ca(ASN1(c->remote->host.config->ca),
+			ASN1(d->remote->host.config->ca),
+			&score->pathlen, verbose)) {
+		vdbg("skipping connection, remote CA 'B' needs to trust CA 'A'");
+		return false;
+	}
+
+	return true;
+}
+
+/*
+ * Decode the ID payload.
+ *
+ * IKEv2 uses this to decode both IDi and IDr.
+ *
+ * IKEv1 Phase 1 (main_inI3_outR3 and main_inR3) Clears *peer to avoid
+ * surprises.
+ *
+ * Note: what we discover may oblige Pluto to switch connections.  We
+ * must be called before SIG or HASH are decoded since we may change
+ * the peer's RSA key or ID.
+ */
+
+diag_t unpack_id(enum ike_id_type kind, struct id *peer, const struct pbs_in *id_pbs)
+{
+	struct pbs_in in_pbs = *id_pbs; /* local copy */
+	shunk_t name = pbs_in_left(&in_pbs);
+
+	*peer = (struct id) {.kind = kind };	/* clears everything */
+
+	switch (kind) {
+
+	/* ident types mostly match between IKEv1 and IKEv2 */
+	case ID_IPV4_ADDR:
+		/* failure mode for initaddr is probably inappropriate address length */
+		return pbs_in_address(&in_pbs, &peer->ip_addr, &ipv4_info, "peer ID");
+
+	case ID_IPV6_ADDR:
+		/* failure mode for initaddr is probably inappropriate address length */
+		return pbs_in_address(&in_pbs, &peer->ip_addr, &ipv6_info, "peer ID");
+
+	/* seems odd to continue as ID_FQDN? */
+	case ID_USER_FQDN:
+#if 0
+		if (memchr(name.ptr, '@', name.len) == NULL) {
+			llog(RC_LOG, logger,
+				    "peer's ID_USER_FQDN contains no @: %.*s",
+				    (int) left, id_pbs->cur);
+			/* return false; */
+		}
+#endif
+		if (memchr(name.ptr, '\0', name.len) != NULL) {
+			esb_buf b;
+			return diag("Phase 1 (Parent)ID Payload of type %s contains a NUL",
+				    str_enum(&ike_id_type_names, kind, &b));
+		}
+		/* ??? ought to do some more sanity check, but what? */
+		peer->name = name;
+		break;
+
+	case ID_FQDN:
+		if (memchr(name.ptr, '\0', name.len) != NULL) {
+			esb_buf b;
+			return diag("Phase 1 (Parent)ID Payload of type %s contains a NUL",
+				    str_enum(&ike_id_type_names, kind, &b));
+		}
+		/* ??? ought to do some more sanity check, but what? */
+		peer->name = name;
+		break;
+
+	case ID_KEY_ID:
+		peer->name = name;
+		if (DBGP(DBG_BASE)) {
+			DBG_dump_hunk("KEY ID:", peer->name);
+		}
+		break;
+
+	case ID_DER_ASN1_DN:
+		peer->name = name;
+		if (DBGP(DBG_BASE)) {
+		    DBG_dump_hunk("DER ASN1 DN:", peer->name);
+		}
+		break;
+
+	case ID_NULL:
+		if (name.len != 0) {
+			if (DBGP(DBG_BASE)) {
+				DBG_dump_hunk("unauthenticated NULL ID:", name);
+			}
+		}
+		break;
+
+	default:
+	{
+		esb_buf b;
+		return diag("Unsupported identity type (%s) in Phase 1 (Parent) ID Payload",
+			    str_enum(&ike_id_type_names, kind, &b));
+	}
+	}
+
+	return NULL;
+}
