@@ -37,8 +37,8 @@ static struct timespec thread_clock(void)
 static struct timespec wall_clock(void)
 {
 	struct timespec now;
-	/* assume never suspended; CLOCK_BOOTTIME is linux specific */
-	int e = clock_gettime(CLOCK_MONOTONIC, &now);
+	/* use the same clockid as monotime */
+	int e = clock_gettime(monotime_clockid(), &now);
 	passert(e == 0);
 	return now;
 }
@@ -75,7 +75,7 @@ void threadtime_stop(const threadtime_t *start, long serialno, const char *fmt,
 {
 	if (DBGP(DBG_CPU_USAGE)) {
 		struct cpu_usage usage = threadtime_sub(threadtime_start(), *start);
-		LSWLOG_DEBUG(buf) {
+		LLOG_JAMBUF(DEBUG_STREAM, &global_logger, buf) {
 			if (serialno > 0) {
 				/* on thread so in background */
 				jam(buf, "(#%lu) ", serialno);
@@ -104,7 +104,7 @@ struct cpu_usage logtime_stop(const logtime_t *start, const char *fmt, ...)
 {
 	struct cpu_usage usage = threadtime_sub(threadtime_start(), start->time);
 	if (DBGP(DBG_CPU_USAGE)) {
-		LSWLOG_DEBUG(buf) {
+		LLOG_JAMBUF(DEBUG_STREAM, &global_logger, buf) {
 			/* update is indented by 2 indents */
 			for (int i = 0; i < start->level; i++) {
 				jam_string(buf, INDENT INDENT);
@@ -136,7 +136,7 @@ static void DBG_missing(const statetime_t *start, threadtime_t now,
 	 */
 	struct cpu_usage missing = threadtime_sub(now, last_log);
 	if (missing.thread_seconds > MISSING_FUDGE) {
-		LSWLOG_DEBUG(buf) {
+		LLOG_JAMBUF(DEBUG_STREAM, &global_logger, buf) {
 			for (int i = 0; i < start->level; i++) {
 				jam_string(buf, INDENT INDENT);
 			}
@@ -230,7 +230,7 @@ void statetime_stop(const statetime_t *start, const char *fmt, ...)
 	pexpect(start->level == 0 || DBGP(DBG_CPU_USAGE));
 
 	/* state disappeared? */
-	struct state *st = state_with_serialno(start->so);
+	struct state *st = state_by_serialno(start->so);
 	if (st == NULL) {
 		dbg("in %s() and could not find #%lu", __func__, start->so);
 		return;
@@ -250,7 +250,7 @@ void statetime_stop(const statetime_t *start, const char *fmt, ...)
 	/* time since start */
 	struct cpu_usage usage = threadtime_sub(stop_time, start->time);
 	if (DBGP(DBG_CPU_USAGE)) {
-		LSWLOG_DEBUG(buf) {
+		LLOG_JAMBUF(DEBUG_STREAM, &global_logger, buf) {
 			/* update is indented by 2 indents */
 			for (int i = 0; i < start->level; i++) {
 				jam_string(buf, INDENT INDENT);
@@ -275,3 +275,14 @@ void statetime_stop(const statetime_t *start, const char *fmt, ...)
 		cpu_usage_add(st->st_timing.main_usage, usage);
 	}
 }
+
+monotime_t monotime_from_threadtime(const threadtime_t time)
+{
+	monotime_t m = {
+		.mt = {
+			.tv_sec = time.wall_clock.tv_sec,
+			.tv_usec = time.wall_clock.tv_nsec / 1000,
+		}
+	};
+	return m;
+}
