@@ -46,8 +46,9 @@ extern generalName_t *collect_rw_ca_candidates(ip_address local_address,
 extern void load_authcerts(const char *type, const char *path,
 			   uint8_t auth_flags);
 
-extern bool match_requested_ca(const generalName_t *requested_ca,
-			       chunk_t our_ca, int *our_pathlen);
+extern bool match_v1_requested_ca(const struct ike_sa *ike,
+				  chunk_t our_ca, int *our_pathlen,
+				  struct verbose verbose);
 
 extern int get_auth_chain(chunk_t *out_chain, int chain_max,
 			  const struct cert *end_cert, bool full_chain);
@@ -57,6 +58,6 @@ extern void free_auth_chain(chunk_t *chain, int chain_len);
 bool find_crl_fetch_dn(chunk_t *issuer_dn, struct connection *c);
 #endif
 
-bool remote_has_preloaded_pubkey(const struct state *st);
+bool remote_has_preloaded_pubkey(const struct ike_sa *ike);
 
 #endif /* _PLUTO_X509_H */
