@@ -11,6 +11,7 @@
  * Copyright (C) 2014-2020 Paul Wouters <pwouters@redhat.com>
  * Copyright (C) 2014-2017 Antony Antony <antony@phenome.org>
  * Copyright (C) 2019-2023 Andrew Cagney <cagney@gnu.org>
+ * Copyright (C) 2020 Nupur Agrawal <nupur202000@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -57,7 +58,7 @@
 
 #include "whack_add.h"
 #include "whack_briefconnectionstatus.h"
-#include "whack_connection.h"
+#include "visit_connection.h"
 #include "whack_connectionstatus.h"
 #include "whack_crash.h"
 #include "whack_debug.h"
@@ -75,6 +76,7 @@
 #include "whack_trafficstatus.h"
 #include "whack_unroute.h"
 #include "whack_showstates.h"
+#include "whack_suspend.h"
 
 static void whack_rereadsecrets(struct show *s)
 {
@@ -110,13 +112,6 @@ static void do_whacklisten(struct logger *logger)
 	listening = true;
 	find_ifaces(true /* remove dead interfaces */, logger);
 
-	/* only do on first listen !?! */
-	static bool checked_stale_ipsec_interfaces = false;
-	if (!checked_stale_ipsec_interfaces) {
-		checked_stale_ipsec_interfaces = true;
-		check_stale_ipsec_interfaces(logger);
-	}
-
 	load_preshared_secrets(logger);
 	load_groups(logger);
 #ifdef USE_SYSTEMD_WATCHDOG
@@ -256,7 +251,7 @@ static void key_add_request(const struct whack_message *msg, struct logger *logg
 	free_id_content(&keyid);
 }
 
-static void dbg_whack(struct show *s, const char *fmt, ...) PRINTF_LIKE(2);
+PRINTF_LIKE(2)
 static void dbg_whack(struct show *s, const char *fmt, ...)
 {
 	if (DBGP(DBG_BASE)) {
@@ -276,7 +271,9 @@ static void dbg_whack(struct show *s, const char *fmt, ...)
  *
  */
 
-static void dispatch(const struct whack_message *const m, struct show *s,
+PRINTF_LIKE(5)
+static void dispatch(const struct whack_message *const m,
+		     struct show *s,
 		     void (*op)(const struct whack_message *const m, struct show *s),
 		     const char *name,
 		     const char *fmt, ...)
@@ -522,13 +519,11 @@ static void whack_process(const struct whack_message *const m, struct show *s)
 		dbg_whack(s, "purgeocsp: stop:");
 	}
 
-#if defined(LIBCURL) || defined(LIBLDAP)
 	if (m->whack_fetchcrls) {
 		dbg_whack(s, "fetchcrls: start:");
 		whack_fetchcrls(s);
 		dbg_whack(s, "fetchcrls: stop:");
 	}
-#endif
 
 	if (m->whack_rereadcerts) {
 		dbg_whack(s, "rereadcerts: start:");
@@ -601,6 +596,12 @@ static void whack_process(const struct whack_message *const m, struct show *s)
 			  bool_str(m->whack_async));
 	}
 
+	if (m->whack_suspend) {
+		dbg_whack(s, "suspend: start:");
+		whack_suspend(m, s);
+		dbg_whack(s, "suspend: stop:");
+	}
+
 	if (m->whack_oppo_initiate) {
 		dbg_whack(s, "oppo_initiate: start:");
 		whack_oppo_initiate(m, s);
