@@ -18,14 +18,8 @@
 #ifndef RCV_WHACK_H
 #define RCV_WHACK_H
 
-#include <event2/event.h>	/* for evutil_socket_t */
+struct logger;
 
-#include "lswcdefs.h"
-#include "fd.h"
-
-extern void whack_handle_cb(evutil_socket_t fd,
-		const short event UNUSED, void *arg UNUSED);
-
-extern void whack_process(fd_t whackfd, const struct whack_message *const m);
+extern void whack_handle_cb(int fd, void *arg, struct logger *logger);
 
 #endif
