@@ -191,58 +191,9 @@ static void schedule_revival_event(struct connection *c, struct logger *logger,
 				  (impair.revival ? "revival" : NULL), logger);
 }
 
-bool scheduled_revival(struct connection *c, struct state *st/*can be NULL*/,
-		       const char *subplot, struct logger *logger)
+static bool scheduled_revival(struct connection *c, struct state *st/*can be NULL*/,
+			      const char *subplot, struct logger *logger)
 {
-	if (st != NULL) {
-		/*
-		 * pexpect() ST is the owner.  Routing should never
-		 * call when it isn't.
-		 */
-		if (IS_CHILD_SA(st)) {
-			if (c->negotiating_child_sa != SOS_NOBODY &&
-			    c->negotiating_child_sa != st->st_serialno) {
-				/*
-				 * There's a newer SA playing with the routing.
-				 * Presumably this is an old Child SA that is in the
-				 * process of being rekeyed or replaced.
-				 */
-				llog_pexpect(st->logger, HERE,
-					     "revival: skipping, .negotiating_child_sa "PRI_SO" is not us",
-					     pri_so(c->negotiating_child_sa));
-				return false;
-			}
-
-			if (c->established_child_sa != SOS_NOBODY &&
-			    c->established_child_sa != st->st_serialno) {
-				/* should be covered by above */
-				llog_pexpect(st->logger, HERE,
-					     "revival: skipping, .established_child_sa "PRI_SO" is not us",
-					     pri_so(c->established_child_sa));
-				return false;
-			}
-		}
-
-		if (IS_IKE_SA(st)) {
-			if (c->negotiating_ike_sa != SOS_NOBODY &&
-			    c->negotiating_ike_sa != st->st_serialno) {
-				/* should be covered by above */
-				llog_pexpect(st->logger, HERE,
-					     "revival: skipping, .negotiating_ike_sa "PRI_SO" is is not us",
-					     pri_so(c->negotiating_ike_sa));
-				return false;
-			}
-			if (c->established_ike_sa != SOS_NOBODY &&
-			    c->established_ike_sa != st->st_serialno) {
-				/* should be covered by above */
-				llog_pexpect(st->logger, HERE,
-					     "revival: skipping, .established_ike_sa "PRI_SO" is is not us",
-					     pri_so(c->established_ike_sa));
-				return false;
-			}
-		}
-	}
-
 	if (!revival_plausable(c, logger)) {
 		return false;
 	}
@@ -275,14 +226,63 @@ bool scheduled_revival(struct connection *c, struct state *st/*can be NULL*/,
 
 }
 
+bool scheduled_connection_revival(struct connection *c, const char *subplot)
+{
+	return scheduled_revival(c, NULL, subplot, c->logger);
+}
+
 bool scheduled_child_revival(struct child_sa *child, const char *subplot)
 {
-	return scheduled_revival(child->sa.st_connection, &child->sa, subplot, child->sa.logger);
+	struct connection *c = child->sa.st_connection;
+	if (c->negotiating_child_sa != SOS_NOBODY &&
+	    c->negotiating_child_sa != child->sa.st_serialno) {
+		/*
+		 * There's a newer SA playing with the routing.
+		 * Presumably this is an old Child SA that is in the
+		 * process of being rekeyed or replaced.
+		 */
+		llog_pexpect(child->sa.logger, HERE,
+			     "revival: skipping, .negotiating_child_sa "PRI_SO" is not us",
+			     pri_so(c->negotiating_child_sa));
+		return false;
+	}
+
+	if (c->established_child_sa != SOS_NOBODY &&
+	    c->established_child_sa != child->sa.st_serialno) {
+		/* should be covered by above */
+		llog_pexpect(child->sa.logger, HERE,
+			     "revival: skipping, .established_child_sa "PRI_SO" is not us",
+			     pri_so(c->established_child_sa));
+		return false;
+	}
+	return scheduled_revival(c, &child->sa, subplot, child->sa.logger);
 }
 
 bool scheduled_ike_revival(struct ike_sa *ike, const char *subplot)
 {
-	return scheduled_revival(ike->sa.st_connection, &ike->sa, subplot, ike->sa.logger);
+	struct connection *c = ike->sa.st_connection;
+	if (c->routing_sa != ike->sa.st_serialno) {
+		llog_pexpect(ike->sa.logger, HERE,
+			     "revival: skipping, .routing_sa "PRI_SO" is is not us",
+			     pri_so(c->routing_sa));
+		return false;
+	}
+	if (c->negotiating_ike_sa != SOS_NOBODY &&
+	    c->negotiating_ike_sa != ike->sa.st_serialno) {
+		/*
+		 * For instance, crossing stream establishes IKE SA,
+		 * but some other Child SA, leaving .routing_sa
+		 * hanging.
+		 */
+		ldbg(ike->sa.logger, "revival: .negotiating_ike_sa "PRI_SO" is is not us",
+		     pri_so(c->negotiating_ike_sa));
+	}
+	if (c->established_ike_sa != SOS_NOBODY &&
+	    c->established_ike_sa != ike->sa.st_serialno) {
+		ldbg(ike->sa.logger, "revival: .established_ike_sa "PRI_SO" is is not us",
+		     pri_so(c->established_ike_sa));
+	}
+	return scheduled_revival(c, &ike->sa, subplot, ike->sa.logger);
 }
 
 void revive_connection(struct connection *c, const char *subplot,
