@@ -104,14 +104,27 @@ static bool connection_cannot_die(enum routing_event event,
 				  struct logger *logger,
 				  const struct routing_annex *e)
 {
-	struct state *st = (e->child != NULL && (*e->child) != NULL ? &(*e->child)->sa :
-			    e->ike != NULL && (*e->ike) != NULL ? &(*e->ike)->sa :
-			    NULL);
 	const char *subplot = (event == CONNECTION_TEARDOWN_IKE ? e->story :
 			       event == CONNECTION_TEARDOWN_CHILD ? e->story :
 			       event == CONNECTION_RESCHEDULE ? e->story :
 			       "???");
-	return scheduled_revival(c, st, subplot, logger);
+	if (e->child != NULL) {
+		struct child_sa *child = (*e->child);
+		if (child != NULL) {
+			PEXPECT(logger, child->sa.st_connection == c);
+			return scheduled_child_revival(child, subplot);
+		}
+	}
+
+	if (e->ike != NULL) {
+		struct ike_sa *ike = (*e->ike);
+		if (ike != NULL) {
+			PEXPECT(logger, ike->sa.st_connection == c);
+			return scheduled_ike_revival(ike, subplot);
+		}
+	}
+
+	return scheduled_connection_revival(c, subplot);
 }
 
 static void jam_sa(struct jambuf *buf, struct state *st, const char **sep)
@@ -626,7 +639,7 @@ static bool unrouted_to_routed_ondemand_sec_label(struct connection *c,
 				struct spd_owner owner = spd_owner(c->spd, c->routing.state,
 								   logger, where);
 				delete_spd_kernel_policy(c->spd, &owner, DIRECTION_OUTBOUND,
-							 EXPECT_KERNEL_POLICY_OK,
+							 KERNEL_POLICY_PRESENT,
 							 /*logger*/logger,
 							 where, "security label policy");
 			}
@@ -648,11 +661,13 @@ static bool unrouted_to_routed_ondemand_sec_label(struct connection *c,
 		/* go back to old routing */
 		struct spd_owner owner = spd_owner(c->spd, c->routing.state,
 						   logger, where);
-		delete_spd_kernel_policy(c->spd, &owner, DIRECTION_OUTBOUND,
-					 EXPECT_KERNEL_POLICY_OK,
+		delete_spd_kernel_policy(c->spd, &owner,
+					 DIRECTION_OUTBOUND,
+					 KERNEL_POLICY_PRESENT,
 					 logger, where, "failed security label");
-		delete_spd_kernel_policy(c->spd, &owner, DIRECTION_INBOUND,
-					 EXPECT_KERNEL_POLICY_OK,
+		delete_spd_kernel_policy(c->spd, &owner,
+					 DIRECTION_INBOUND,
+					 KERNEL_POLICY_PRESENT,
 					 logger, where, "failed security label");
 		return false;
 	}
@@ -700,7 +715,7 @@ static void routed_tunnel_to_routed_ondemand(struct child_sa *child,
 		delete_cat_kernel_policies(spd, &owner, logger, where);
 		replace_ipsec_with_bare_kernel_policy(child, c, spd, &owner,
 						      SHUNT_KIND_ONDEMAND,
-						      EXPECT_KERNEL_POLICY_OK,
+						      KERNEL_POLICY_PRESENT,
 						      logger, where);
 	}
 
@@ -729,7 +744,7 @@ static void routed_tunnel_to_routed_failure(struct child_sa *child,
 		delete_cat_kernel_policies(spd, &owner, logger, where);
 		replace_ipsec_with_bare_kernel_policy(child, c, spd, &owner,
 						      SHUNT_KIND_FAILURE,
-						      EXPECT_KERNEL_POLICY_OK,
+						      KERNEL_POLICY_PRESENT,
 						      logger, where);
 	}
 
@@ -737,24 +752,11 @@ static void routed_tunnel_to_routed_failure(struct child_sa *child,
 }
 
 static void routed_kernel_policy_to_unrouted(struct connection *c,
-					     lset_t direction,
+					     enum directions directions,
 					     struct logger *logger,
 					     where_t where,
 					     const char *story)
 {
-	enum expect_kernel_policy inbound_policy_expectation;
-	switch (direction) {
-	case DIRECTION_INBOUND|DIRECTION_OUTBOUND:
-	case DIRECTION_INBOUND:
-		inbound_policy_expectation = EXPECT_KERNEL_POLICY_OK;
-		break;
-	case DIRECTION_OUTBOUND:
-		inbound_policy_expectation = EXPECT_NO_INBOUND;
-		break;
-	default:
-		bad_case(direction);
-	}
-
 	FOR_EACH_ITEM(spd, &c->child.spds) {
 
 		if (is_v1_cisco_split(spd, HERE)) {
@@ -764,8 +766,7 @@ static void routed_kernel_policy_to_unrouted(struct connection *c,
 		struct spd_owner owner = spd_owner(spd, RT_UNROUTED,
 						   logger, where);
 
-		delete_spd_kernel_policies(spd, &owner,
-					   inbound_policy_expectation,
+		delete_spd_kernel_policies(spd, &owner, directions,
 					   logger, where, story);
 		do_updown_unroute_spd(spd, &owner, NULL, logger,
 				      (struct updown_env) {0});
@@ -775,23 +776,10 @@ static void routed_kernel_policy_to_unrouted(struct connection *c,
 }
 
 static void unrouted_kernel_policy_to_unrouted(struct connection *c,
-					       lset_t direction,
+					       enum directions directions,
 					       struct logger *logger, where_t where,
 					       const char *story)
 {
-	enum expect_kernel_policy inbound_policy_expectation;
-	switch (direction) {
-	case DIRECTION_INBOUND|DIRECTION_OUTBOUND:
-	case DIRECTION_INBOUND:
-		inbound_policy_expectation = EXPECT_KERNEL_POLICY_OK;
-		break;
-	case DIRECTION_OUTBOUND:
-		inbound_policy_expectation = EXPECT_NO_INBOUND;
-		break;
-	default:
-		bad_case(direction);
-	}
-
 	FOR_EACH_ITEM(spd, &c->child.spds) {
 
 		if (is_v1_cisco_split(spd, HERE)) {
@@ -801,8 +789,7 @@ static void unrouted_kernel_policy_to_unrouted(struct connection *c,
 		struct spd_owner owner = spd_owner(spd, RT_UNROUTED,
 						   logger, where);
 
-		delete_spd_kernel_policies(spd, &owner,
-					   inbound_policy_expectation,
+		delete_spd_kernel_policies(spd, &owner, directions,
 					   logger, where, story);
 	}
 
@@ -828,7 +815,8 @@ static void routed_tunnel_to_unrouted(struct child_sa *child,
 		struct spd_owner owner = spd_owner(spd, RT_UNROUTED,
 						   logger, where);
 
-		delete_spd_kernel_policies(spd, &owner, EXPECT_KERNEL_POLICY_OK,
+		delete_spd_kernel_policies(spd, &owner,
+					   DIRECTIONS_INBOUND_AND_OUTBOUND,
 					   logger, where, "delete");
 		do_updown_unroute_spd(spd, &owner, child, logger,
 				      (struct updown_env) {0});
@@ -979,7 +967,7 @@ static void unrouted_tunnel_to_routed_ondemand(struct child_sa *child,
 		delete_cat_kernel_policies(spd, &owner, logger, where);
 		replace_ipsec_with_bare_kernel_policy(child, c, spd, &owner,
 						      SHUNT_KIND_ONDEMAND,
-						      EXPECT_KERNEL_POLICY_OK,
+						      KERNEL_POLICY_PRESENT,
 						      logger, where);
 	}
 
@@ -1009,7 +997,7 @@ static void unrouted_tunnel_to_routed_failure(struct child_sa *child,
 		delete_cat_kernel_policies(spd, &owner, logger, where);
 		replace_ipsec_with_bare_kernel_policy(child, c, spd, &owner,
 						      SHUNT_KIND_FAILURE,
-						      EXPECT_KERNEL_POLICY_OK,
+						      KERNEL_POLICY_PRESENT,
 						      logger, where);
 	}
 
@@ -1033,7 +1021,8 @@ static void unrouted_tunnel_to_unrouted(struct connection *c,
 		struct spd_owner owner = spd_owner(spd, RT_UNROUTED,
 						   logger, where);
 
-		delete_spd_kernel_policies(spd, &owner, EXPECT_KERNEL_POLICY_OK,
+		delete_spd_kernel_policies(spd, &owner,
+					   DIRECTIONS_INBOUND_AND_OUTBOUND,
 					   logger, where, story);
 	}
 
@@ -1091,7 +1080,8 @@ static void routed_inbound_negotiation_to_unrouted(struct connection *c,
 		struct spd_owner owner = spd_owner(spd, RT_UNROUTED/*ignored*/,
 						   logger, where);
 
-		delete_spd_kernel_policies(spd, &owner, EXPECT_KERNEL_POLICY_OK,
+		delete_spd_kernel_policies(spd, &owner,
+					   DIRECTIONS_INBOUND_AND_OUTBOUND,
 					   logger, where, story);
 		do_updown_unroute_spd(spd, &owner, child, logger,
 				      (struct updown_env) {0});
@@ -1117,7 +1107,7 @@ static void unrouted_inbound_to_unrouted(struct connection *c,
 						   logger, where);
 
 		delete_spd_kernel_policies(spd, &owner,
-					   EXPECT_KERNEL_POLICY_OK,
+					   DIRECTIONS_INBOUND_AND_OUTBOUND,
 					   logger, where, story);
 	}
 
@@ -1145,12 +1135,12 @@ static void teardown_unrouted_inbound_negotiation(struct connection *c,
 						  const char *story)
 {
 	if (scheduled_child_revival(child, story)) {
-		unrouted_kernel_policy_to_unrouted(c, DIRECTION_INBOUND,
+		unrouted_kernel_policy_to_unrouted(c, DIRECTIONS_INBOUND,
 						   logger, where, story);
 		return;
 	}
 
-	unrouted_kernel_policy_to_unrouted(c, DIRECTION_INBOUND,
+	unrouted_kernel_policy_to_unrouted(c, DIRECTIONS_INBOUND,
 					   logger, where, story);
 }
 
@@ -1189,8 +1179,11 @@ static void teardown_routed_negotiation(struct connection *c,
 
 	/*
 	 * Should this instead install a failure shunt?
+	 *
+	 * A ROUTED_NEGOTIATION doesn't have inbound policy so don't
+	 * expect it.
 	 */
-	routed_kernel_policy_to_unrouted(c, DIRECTION_INBOUND,
+	routed_kernel_policy_to_unrouted(c, DIRECTIONS_INBOUND,
 					 logger, where, "deleting");
 	PEXPECT(logger, c->routing.state == RT_UNROUTED);
 }
@@ -1217,11 +1210,15 @@ static void teardown_child_post_op(const struct routing_annex *e)
 	delete_child_sa(e->child);
 }
 
-static void teardown_child(struct child_sa **child, const char *story, where_t where)
+void connection_teardown_child(struct child_sa **child,
+			       enum terminate_reason reason,
+			       where_t where)
 {
 	struct connection *cc = (*child)->sa.st_connection;
 	struct logger *logger = clone_logger((*child)->sa.logger, HERE); /* must free */
 
+	name_buf rb;
+
 	struct routing_annex annex = {
 		.child = child,
 		.where = where,
@@ -1230,7 +1227,9 @@ static void teardown_child(struct child_sa **child, const char *story, where_t w
 		 */
 		.dispatch_ok = teardown_child_dispatch_ok,
 		.post_op = teardown_child_post_op,
-		.story = story,
+		.story = (reason == REASON_DELETED ? "delete Child SA" : /* logging compat hack */
+			  reason == REASON_TOO_MANY_RETRANSMITS ? "timeout Child SA" : /* logging compat hack */
+			  str_enum(&terminate_reason_names, reason, &rb)),
 	};
 
 	/*
@@ -1242,16 +1241,6 @@ static void teardown_child(struct child_sa **child, const char *story, where_t w
 	free_logger(&logger, HERE);
 }
 
-void connection_delete_child(struct child_sa **child, where_t where)
-{
-	teardown_child(child, "delete Child SA", where);
-}
-
-void connection_timeout_child(struct child_sa **child, where_t where)
-{
-	teardown_child(child, "timeout Child SA", where);
-}
-
 /*
  * If there's an established IKE SA and it isn't this one (i.e., not
  * owner) skip the route change.
@@ -1278,12 +1267,17 @@ static void teardown_ike_post_op(const struct routing_annex *e)
 	delete_ike_sa(e->ike);
 }
 
-static void teardown_ike(struct ike_sa **ike, const char *story, where_t where)
+void connection_teardown_ike(struct ike_sa **ike,
+			     enum terminate_reason reason,
+			     where_t where)
 {
 	struct connection *c = (*ike)->sa.st_connection;
 	struct logger *logger = clone_logger((*ike)->sa.logger, HERE);
+	name_buf rb;
 	struct routing_annex annex = {
-		.story = story,
+		.story = (reason == REASON_DELETED ? "delete IKE SA" : /* logging compat hack */
+			  reason == REASON_TOO_MANY_RETRANSMITS ? "timeout IKE SA" : /* logging compat hack */
+			  str_enum(&terminate_reason_names, reason, &rb)),
 		.ike = ike,
 		.where = where,
 		.dispatch_ok = teardown_ike_dispatch_ok,
@@ -1296,16 +1290,6 @@ static void teardown_ike(struct ike_sa **ike, const char *story, where_t where)
 	free_logger(&logger, HERE);
 }
 
-void connection_delete_ike(struct ike_sa **ike, where_t where)
-{
-	teardown_ike(ike, "delete IKE SA", where);
-}
-
-void connection_timeout_ike(struct ike_sa **ike, where_t where)
-{
-	teardown_ike(ike, "timeout IKE SA", where);
-}
-
 /*
  * Stop reviving children trying to use this IKE SA.
  */
@@ -1535,7 +1519,7 @@ static void set_established_ike(enum routing_event event UNUSED,
 	linux_audit_conn(&ike->sa, LAK_PARENT_START);
 	/* dump new keys */
 	if (DBGP(DBG_PRIVATE)) {
-		DBG_tcpdump_ike_sa_keys(&ike->sa);
+		LDBG_tcpdump_ike_sa_keys(&global_logger, ike);
 	}
 }
 
@@ -1861,7 +1845,7 @@ static bool dispatch_1(enum routing_event event,
 			return true;
 		}
 		/* is this reachable? */
-		routed_kernel_policy_to_unrouted(c, DIRECTION_OUTBOUND,
+		routed_kernel_policy_to_unrouted(c, DIRECTIONS_OUTBOUND,
 						 logger, e->where, "deleting");
 		PEXPECT(logger, c->routing.state == RT_UNROUTED);
 		/* connection lives to fight another day */
@@ -1870,7 +1854,7 @@ static bool dispatch_1(enum routing_event event,
 	case X(TEARDOWN_CHILD, UNROUTED_NEGOTIATION, INSTANCE):
 	case X(TEARDOWN_IKE, UNROUTED_NEGOTIATION, INSTANCE):
 		if (connection_cannot_die(event, c, logger, e)) {
-			unrouted_kernel_policy_to_unrouted(c, DIRECTION_OUTBOUND,
+			unrouted_kernel_policy_to_unrouted(c, DIRECTIONS_OUTBOUND,
 							   logger, e->where, e->story);
 			return true;
 		}
@@ -1886,7 +1870,7 @@ static bool dispatch_1(enum routing_event event,
 			set_routing(c, RT_UNROUTED);
 			return true;
 		}
-		unrouted_kernel_policy_to_unrouted(c, DIRECTION_OUTBOUND,
+		unrouted_kernel_policy_to_unrouted(c, DIRECTIONS_OUTBOUND,
 						   logger, e->where, e->story);
 		return true;
 
@@ -2221,7 +2205,7 @@ static bool dispatch_1(enum routing_event event,
 
 	case X(UNROUTE, ROUTED_FAILURE, INSTANCE):
 	case X(UNROUTE, ROUTED_FAILURE, PERMANENT):
-		routed_kernel_policy_to_unrouted(c, DIRECTION_OUTBOUND,
+		routed_kernel_policy_to_unrouted(c, DIRECTIONS_OUTBOUND,
 						 logger, e->where, "unroute");
 		return true;
 
@@ -2233,7 +2217,7 @@ static bool dispatch_1(enum routing_event event,
 
 	case X(UNROUTE, ROUTED_NEGOTIATION, INSTANCE):
 	case X(UNROUTE, ROUTED_NEGOTIATION, PERMANENT):
-		routed_kernel_policy_to_unrouted(c, DIRECTION_OUTBOUND,
+		routed_kernel_policy_to_unrouted(c, DIRECTIONS_OUTBOUND,
 						 logger, e->where, "unroute");
 		return true;
 
@@ -2250,7 +2234,7 @@ static bool dispatch_1(enum routing_event event,
 		    c->local->kind == CK_PERMANENT) {
 			flush_routed_ondemand_revival(c);
 		}
-		routed_kernel_policy_to_unrouted(c, DIRECTION_OUTBOUND,
+		routed_kernel_policy_to_unrouted(c, DIRECTIONS_OUTBOUND,
 						 logger, e->where, "unroute");
 		return true;
 
@@ -2267,7 +2251,7 @@ static bool dispatch_1(enum routing_event event,
 		return true;
 
 	case X(UNROUTE, UNROUTED_NEGOTIATION, INSTANCE):
-		unrouted_kernel_policy_to_unrouted(c, DIRECTION_OUTBOUND,
+		unrouted_kernel_policy_to_unrouted(c, DIRECTIONS_OUTBOUND,
 						   logger, e->where, "unroute");
 		return true;
 
