@@ -73,9 +73,6 @@ static bool send_shunks(const char *where, bool just_a_keepalive,
 			shunk_t a, shunk_t b,
 			struct logger *logger)
 {
-	/* NOTE: on system with limited stack, buf could be made static */
-	uint8_t buf[MAX_OUTPUT_UDP_SIZE];
-
 	/* Each fragment, if we are doing NATT, needs a non-ESP_Marker prefix.
 	 * natt_bonus is the size of the addition (0 if not needed).
 	 */
@@ -102,7 +99,7 @@ static bool send_shunks(const char *where, bool just_a_keepalive,
 	 *
 	 * Code attempting to call this function with
 	 * hsetportof(port,addr) where addr is invalid also get an
-	 * expecation failed message.
+	 * expectation failed message.
 	 */
 	if (!endpoint_is_specified(remote_endpoint)) {
 		/* not asserting, who knows what nonsense a user can generate */
@@ -123,16 +120,27 @@ static bool send_shunks(const char *where, bool just_a_keepalive,
 				  interface->esp_encapsulation_enabled ?
 				  NON_ESP_MARKER_SIZE : 0;
 
-	const uint8_t *ptr;
 	size_t len = natt_bonus + a.len + b.len;
 
 	if (len > MAX_OUTPUT_UDP_SIZE) {
 		/* XXX: UDP centric? */
-		llog(RC_LOG_SERIOUS, logger,
+		llog(RC_LOG, logger,
 			    "send_ike_msg(): really too big %zu bytes", len);
 		return false;
 	}
 
+	/*
+	 * BUF must be same scope as PACKET.  Use dummy assignment to
+	 * enforce this.
+	 *
+	 * NOTE: on system with limited stack, buf could be made
+	 * static.  XXX: except this is going to be made parallel.
+	 *
+	 * Instead of the copy, should write_packet act more like
+	 * iovec?
+	 */
+	uint8_t buf[MAX_OUTPUT_UDP_SIZE];
+	shunk_t packet = THING_AS_HUNK(buf);
 	if (len != a.len) {
 		/* copying required */
 
@@ -145,27 +153,27 @@ static bool send_shunks(const char *where, bool just_a_keepalive,
 		/* 3. chunk b */
 		memcpy(buf + natt_bonus + a.len, b.ptr, b.len);
 
-		ptr = buf;
+		packet = shunk2(buf, len);
 	} else {
-		ptr = a.ptr;
+		packet = a;
 	}
 
 	if (DBGP(DBG_BASE)) {
 		endpoint_buf lb;
 		endpoint_buf rb;
 		llog(DEBUG_STREAM, logger,
-			    "sending %zu bytes for %s through %s from %s to %s using %s (for #%lu)",
-			    len, where,
-			    interface->ip_dev->id_rname,
-			    str_endpoint(&interface->local_endpoint, &lb),
-			    str_endpoint(&remote_endpoint, &rb),
-			    interface->io->protocol->name,
-			    serialno);
-		DBG_dump(NULL, ptr, len);
+		     "sending %zu bytes for %s through %s from %s to %s using %s (for #%lu)",
+		     packet.len, where,
+		     interface->ip_dev->real_device_name,
+		     str_endpoint(&interface->local_endpoint, &lb),
+		     str_endpoint(&remote_endpoint, &rb),
+		     interface->io->protocol->name,
+		     serialno);
+		llog_dump_hunk(DEBUG_STREAM, logger, packet);
 	}
 
-	if (!impair_outgoing_message(shunk2(ptr, len), logger)) {
-		ssize_t wlen = interface->io->write_packet(interface, ptr, len,
+	if (!impair_outbound(interface, packet, &remote_endpoint, logger)) {
+		ssize_t wlen = interface->io->write_packet(interface, packet,
 							   &remote_endpoint, logger);
 		if (wlen != (ssize_t)len) {
 			if (!just_a_keepalive) {
@@ -173,7 +181,7 @@ static bool send_shunks(const char *where, bool just_a_keepalive,
 				endpoint_buf rb;
 				llog_error(logger, errno,
 					   "send on %s from %s to %s using %s failed in %s",
-					   interface->ip_dev->id_rname,
+					   interface->ip_dev->real_device_name,
 					   str_endpoint(&interface->local_endpoint, &lb),
 					   str_endpoint_sensitive(&remote_endpoint, &rb),
 					   interface->io->protocol->name,
@@ -181,7 +189,7 @@ static bool send_shunks(const char *where, bool just_a_keepalive,
 			}
 			return false;
 		}
-		pstats_ike_out_bytes += len;
+		pstats_ike_bytes.out += len;
 	}
 
 	/*
@@ -198,17 +206,17 @@ static bool send_shunks(const char *where, bool just_a_keepalive,
 		llog(RC_LOG, logger,
 			    "IMPAIR: JACOB 2-2: resending %zu bytes for %s through %s from %s to %s:",
 			    len, where,
-			    interface->ip_dev->id_rname,
+			    interface->ip_dev->real_device_name,
 			    str_endpoint(&interface->local_endpoint, &ib),
 			    str_endpoint(&remote_endpoint, &b));
 
 		ip_sockaddr remote_sa = sockaddr_from_endpoint(remote_endpoint);
-		ssize_t wlen = sendto(interface->fd, ptr, len, 0, &remote_sa.sa.sa, remote_sa.len);
+		ssize_t wlen = sendto(interface->fd, packet.ptr, packet.len, 0, &remote_sa.sa.sa, remote_sa.len);
 		if (wlen != (ssize_t)len) {
 			if (!just_a_keepalive) {
 				llog_error(logger, errno,
 					   "sendto on %s to %s failed in %s",
-					   interface->ip_dev->id_rname,
+					   interface->ip_dev->real_device_name,
 					   str_endpoint(&remote_endpoint, &b),
 					   where);
 			}
@@ -222,30 +230,28 @@ bool send_pbs_out_using_md(struct msg_digest *md, const char *where, struct pbs_
 {
 	return send_shunks(where, false, SOS_NOBODY,
 			   md->iface, md->sender,
-			   same_pbs_out_as_shunk(packet), null_shunk,
-			   md->md_logger);
+			   pbs_out_all(packet), null_shunk,
+			   md->logger);
 }
 
-bool send_chunks_using_state(struct state *st, const char *where,
-			     chunk_t chunk_a, chunk_t chunk_b)
+bool send_shunks_using_state(struct state *st, const char *where,
+			     shunk_t shunk_a, shunk_t shunk_b)
 {
 	return send_shunks(where, false, st->st_serialno,
-			   st->st_interface, st->st_remote_endpoint,
-			   HUNK_AS_SHUNK(chunk_a), HUNK_AS_SHUNK(chunk_b),
-			   st->st_logger);
+			   st->st_iface_endpoint, st->st_remote_endpoint,
+			   shunk_a, shunk_b,
+			   st->logger);
 }
 
-bool send_chunk_using_state(struct state *st, const char *where, chunk_t packet)
+bool send_shunk_using_state(struct state *st, const char *where,
+			    shunk_t shunk)
 {
-	return send_chunks_using_state(st, where, packet, EMPTY_CHUNK);
+	return send_shunks_using_state(st, where, shunk, empty_shunk);
 }
 
 bool send_pbs_out_using_state(struct state *st, const char *where, struct pbs_out *pbs)
 {
-	return send_shunks(where, /*just_a_keepalive?*/false,
-			   st->st_serialno, st->st_interface, st->st_remote_endpoint,
-			   same_pbs_out_as_shunk(pbs), null_shunk,
-			   st->st_logger);
+	return send_shunk_using_state(st, where, pbs_out_all(pbs));
 }
 
 /*
@@ -257,8 +263,8 @@ bool send_keepalive_using_state(struct state *st, const char *where)
 {
 	static unsigned char ka_payload = 0xff;
 
-	return send_shunks(where, true, st->st_serialno, st->st_interface,
+	return send_shunks(where, true, st->st_serialno, st->st_iface_endpoint,
 			   st->st_remote_endpoint,
 			   THING_AS_SHUNK(ka_payload), null_shunk,
-			   st->st_logger);
+			   st->logger);
 }
