@@ -34,9 +34,6 @@ struct show;
  * state back MUST check ST before processing.  Caller sets CUR_STATE
  * so don't play with that.
  *
- * MDP either points at the unsuspended contents of .st_suspended_md,
- * or NULL.  On return, if *MDP is non-NULL, then it will be released.
- *
  * STATUS is the child processes exit code as returned by things like
  * waitpid().
  */
@@ -47,16 +44,20 @@ typedef stf_status server_fork_cb(struct state *st,
 				  struct logger *logger);
 typedef int server_fork_op(void *context, struct logger *logger);
 
-extern int server_fork(const char *name, so_serial_t serialno, server_fork_op *op,
+extern int server_fork(const char *name,
+		       so_serial_t serialno,
+		       struct msg_digest *md,
+		       server_fork_op *op,
 		       server_fork_cb *callback, void *callback_context,
 		       struct logger *logger);
-void server_fork_exec(const char *what, const char *path,
+void server_fork_exec(const char *path,
 		      char *argv[], char *envp[],
 		      server_fork_cb *callback, void *callback_context,
 		      struct logger *logger);
 
 void server_fork_sigchld_handler(struct logger *logger);
-void init_server_fork(void);
+void init_server_fork(struct logger *logger);
+void check_server_fork(struct logger *logger);
 void show_process_status(struct show *s);
 
 #endif /* _SERVER_H */
