@@ -22,12 +22,17 @@
 
 #include <stdbool.h>
 
+#include "chunk.h"
+
 struct verbose;
 struct logger;
 
-bool server_run(const char *verb, const char *verb_suffix, const char *cmd,
-		struct logger *logger);
+bool server_run(const char *verb, const char *verb_suffix,
+		const char *cmd,
+		struct verbose verbose);
 
 bool server_runv(const char *argv[], struct verbose verbose);
 
+chunk_t server_runv_chunk(const char *argv[], struct verbose verbose);
+
 #endif
