@@ -1,4 +1,4 @@
-/* show functions, for libreswan
+/* show (whack-only) output functions, for libreswan
  *
  * Copyright (C) 2020 Andrew Cagney
  *
@@ -16,13 +16,17 @@
 #ifndef SHOW_H
 #define SHOW_H
 
-/*
- * Try to deal with the separator (aka blank line or spacer) problem
- * in show output.
- */
+#include "lswcdefs.h"		/* for PRINTF_LIKE() */
 
 struct show;
 enum rc_type;
+struct logger;
+
+/*
+ * Try to deal with the separator (i.e., don't output duplicate blank
+ * / spacer lines when combining functions that send output to whack)
+ * in show (whack-only) output.
+ */
 
 struct show *alloc_show(struct logger *logger);
 void free_show(struct show **s);
@@ -34,12 +38,12 @@ struct logger *show_logger(struct show *s);
  * a jambuf.
  */
 
-struct jambuf *show_jambuf(struct show *s);
-void jambuf_to_show(struct jambuf *jambuf, struct show *s, enum rc_type rc);
-#define SHOW_JAMBUF(RC, S, BUF)				\
-	for (struct jambuf *BUF = show_jambuf(s);	\
-	     BUF != NULL;				\
-	     jambuf_to_show(BUF, S, RC), BUF = NULL)
+struct jambuf *show_jambuf(struct show *s, enum rc_type rc);
+void show_to_logger(struct show *s);
+#define SHOW_JAMBUF(S, BUF)					\
+	for (struct jambuf *BUF = show_jambuf(S, RC_LOG);	\
+	     BUF != NULL;					\
+	     show_to_logger(S), BUF = NULL)
 
 /*
  * Flag that the next line needs to be preceded by a separator (aka
@@ -48,10 +52,10 @@ void jambuf_to_show(struct jambuf *jambuf, struct show *s, enum rc_type rc);
  * Example 1:
  *
  *    show_separator(s);
- *    show_comment(s, "heading 1");
+ *    show(s, "heading 1");
  *    show_separator(s);
  *    show_separator(s);
- *    show_comment(s, "heading 2");
+ *    show(s, "heading 2");
  *    show_separator(s);
  *
  *    heading 1
@@ -64,7 +68,7 @@ void jambuf_to_show(struct jambuf *jambuf, struct show *s, enum rc_type rc);
  *    show_blank(s);
  *    show_separator(s);
  *    show_blank(s);
- *    show_comment(s, "heading 1");
+ *    show(s, "heading 1");
  *
  * will output:
  *
@@ -79,14 +83,22 @@ void show_blank(struct show *s);
  * If necessary show the separator (aka blank line), and then show the
  * message.  Suppress further separation.
  *
- * "comment" comes from RC_COMMENT, better name?
+ * "comment" comes from RC_LOG, better name?
  */
 
-void show_comment(struct show *s, const char *message, ...) PRINTF_LIKE(2);
+void show(struct show *s, const char *message, ...) PRINTF_LIKE(2);
 
 /*
- * A raw line (no prefix).
+ * Whack only logging.
+ *
+ * None of these functions add a context prefix (such as connection
+ * name).  If that's really really needed then use
+ * log_*(WHACK_STREAM,...) above.
+ *
+ * also requires a valid whackfd.  It should only be used by show
+ * commands.
  */
-void show_raw(struct show *s, const char *message, ...) PRINTF_LIKE(2);
+
+void whack_log(enum rc_type rc, struct show *s, const char *message, ...) PRINTF_LIKE(3);
 
 #endif
