@@ -704,20 +704,24 @@ static void flush_incomplete_children(struct ike_sa *ike)
 {
 	struct state_filter sf = {
 		.clonedfrom = ike->sa.st_serialno,
-		.where = HERE,
+		.search = {
+			.order = OLD2NEW,
+			.verbose.logger = &global_logger,
+			.where = HERE,
+		},
 	};
-	while (next_state(OLD2NEW, &sf)) {
+	while (next_state(&sf)) {
 		struct child_sa *child = pexpect_child_sa(sf.st);
 		switch (child->sa.st_ike_version) {
 		case IKEv1:
 			if (!IS_IPSEC_SA_ESTABLISHED(&child->sa)) {
 				state_attach(&child->sa, ike->sa.logger);
-				connection_delete_child(&child, HERE);
+				connection_teardown_child(&child, REASON_DELETED, HERE);
 			}
 			continue;
 		case IKEv2:
 			state_attach(&child->sa, ike->sa.logger);
-			connection_delete_child(&child, HERE);
+			connection_teardown_child(&child, REASON_DELETED, HERE);
 			continue;
 		}
 		bad_enum(ike->sa.logger, &ike_version_names, child->sa.st_ike_version);
@@ -795,7 +799,7 @@ void llog_sa_delete_n_send(struct ike_sa *ike, struct state *st)
 		jam_string(buf, ")");
 		/* aged NNNs */
 		jam_string(buf, " aged ");
-		jam_deltatime(buf, realtimediff(realnow(), st->st_inception));
+		jam_deltatime(buf, realtime_diff(realnow(), st->st_inception));
 		jam_string(buf, "s");
 		if (ike == NULL) {
 			jam_string(buf, " and NOT sending notification");
@@ -841,9 +845,13 @@ void delete_state(struct state *st)
 	    DBGP(DBG_BASE)) {
 		struct state_filter sf = {
 			.clonedfrom = st->st_serialno,
-			.where = HERE,
+			.search = {
+				.order = OLD2NEW,
+				.verbose.logger = &global_logger,
+				.where = HERE,
+			},
 		};
-		while (next_state(OLD2NEW, &sf)) {
+		while (next_state(&sf)) {
 			state_buf sb;
 			barf((DBGP(DBG_BASE) ? PASSERT_FLAGS : PEXPECT_FLAGS),
 			     st->logger, /*ignore-exit-code*/0, HERE,
@@ -937,7 +945,10 @@ void delete_state(struct state *st)
 
 #ifdef USE_PAM_AUTH
 	if (st->st_pam_auth != NULL) {
-		pam_auth_abort(st, "deleting state");
+		struct ike_sa *ike = pexpect_ike_sa(st);
+		if (ike != NULL) {
+			pam_auth_abort(ike, "deleting state");
+		}
 	}
 #endif
 
@@ -945,6 +956,9 @@ void delete_state(struct state *st)
 	free_chunk_content(&st->st_v2_ike_intermediate.initiator);
 	free_chunk_content(&st->st_v2_ike_intermediate.responder);
 
+	/* session resumption */
+	pfreeany(st->st_v2_resume_session);
+
 	/* if there's an IKEv1 backgroud md, release it */
 	if (st->st_v1_background_md != NULL) {
 		ldbg(st->logger, "releasing IKEv1 MD received during background task");
@@ -1154,7 +1168,7 @@ void delete_state(struct state *st)
 	pfreeany(st->st_seen_cfg_banner);
 
 	free_chunk_content(&st->st_no_ppk_auth);
-	free_chunk_content(&st->st_active_redirect_gw);
+	pfreeany(st->st_active_redirect_gw);
 
 	free_logger(&st->logger, HERE);
 	messup(st);
@@ -1193,54 +1207,30 @@ static struct child_sa *duplicate_state(struct connection *c,
 					   sa_type, sa_role, HERE));
 
 	connection_buf cib;
-	dbg("duplicating state object #%lu "PRI_CONNECTION" as #%lu for %s",
-	    ike->sa.st_serialno, pri_connection(ike->sa.st_connection, &cib),
-	    child->sa.st_serialno, sa_type == CHILD_SA ? "IPSEC SA" : "IKE SA");
+	ldbg(ike->sa.logger, "duplicating state object #%lu "PRI_CONNECTION" as #%lu for %s",
+	     ike->sa.st_serialno, pri_connection(ike->sa.st_connection, &cib),
+	     child->sa.st_serialno, sa_type == CHILD_SA ? "IPSEC SA" : "IKE SA");
 
+	/*
+	 * Some sloppy value inheritance.
+	 *
+	 * A child created during IKE_AUTH uses the IKE SA's PRF,
+	 * while a child created using CREATE_CHILD_SA uses its own
+	 * negotiated PRF.
+	 *
+	 * A child created during IKE_AUTH uses the N[ir] from the IKE
+	 * SA's IKE_SA_INIT exchange, while a child created using
+	 * CREATE_CHILD_SA uses N[ir] from that exchange.
+	 *
+	 * See 2.17. Generating Keying Material for Child SAs
+	 *
+	 * IKEv1 probably does something similar.
+	 *
+	 * Should code instead be copying values explicitly and only
+	 * during IKE_AUTH?
+	 */
 	if (sa_type == CHILD_SA) {
 		child->sa.st_oakley = ike->sa.st_oakley;
-	}
-
-	child->sa.quirks = ike->sa.quirks;
-	child->sa.hidden_variables = ike->sa.hidden_variables;
-	endpoint_buf eb;
-	dbg("#%lu setting local endpoint to %s from #%ld.st_localport "PRI_WHERE,
-	    child->sa.st_serialno,
-	    str_endpoint(&ike->sa.st_iface_endpoint->local_endpoint, &eb),
-	    ike->sa.st_serialno,pri_where(HERE));
-	passert(child->sa.st_ike_version == ike->sa.st_ike_version);
-	child->sa.st_ikev2_anon = ike->sa.st_ikev2_anon;
-	child->sa.st_v1_seen_fragmentation_supported = ike->sa.st_v1_seen_fragmentation_supported;
-	child->sa.st_v2_ike_fragmentation_enabled = ike->sa.st_v2_ike_fragmentation_enabled;
-	child->sa.st_v1_seen_fragments = ike->sa.st_v1_seen_fragments;
-	child->sa.st_v2_ike_ppk_enabled = ike->sa.st_v2_ike_ppk_enabled;
-	child->sa.st_seen_redirect_sup = ike->sa.st_seen_redirect_sup;
-	child->sa.st_sent_redirect = ike->sa.st_sent_redirect;
-
-	/* these were set while we didn't have client state yet */
-	/* we should really split the NOTIFY loop in two cleaner ones */
-	child->sa.st_ipcomp.trans_attrs = ike->sa.st_ipcomp.trans_attrs;
-	child->sa.st_ipcomp.v1_lifetime = ike->sa.st_ipcomp.v1_lifetime;
-	child->sa.st_ipcomp.protocol = ike->sa.st_ipcomp.protocol;
-	child->sa.st_ipcomp.inbound.spi = ike->sa.st_ipcomp.inbound.spi;
-
-	if (sa_type == CHILD_SA) {
-#   define clone_nss_symkey_field(field)				\
-		child->sa.field = symkey_addref(ike->sa.logger, #field, ike->sa.field)
-
-		clone_nss_symkey_field(st_skeyid_nss);
-		clone_nss_symkey_field(st_skey_d_nss); /* aka st_skeyid_d_nss */
-		clone_nss_symkey_field(st_skey_ai_nss); /* aka st_skeyid_a_nss */
-		clone_nss_symkey_field(st_skey_ar_nss);
-		clone_nss_symkey_field(st_skey_ei_nss); /* aka st_skeyid_e_nss */
-		clone_nss_symkey_field(st_skey_er_nss);
-		clone_nss_symkey_field(st_skey_pi_nss);
-		clone_nss_symkey_field(st_skey_pr_nss);
-		clone_nss_symkey_field(st_enc_key_nss);
-
-#   undef clone_nss_symkey_field
-
-		/* v2 duplication of state */
 #   define state_clone_chunk(CHUNK) child->sa.CHUNK = clone_hunk(ike->sa.CHUNK, #CHUNK " in duplicate state")
 		state_clone_chunk(st_ni);
 		state_clone_chunk(st_nr);
@@ -1248,25 +1238,48 @@ static struct child_sa *duplicate_state(struct connection *c,
 	}
 
 	/*
-	 * These are done because we need them in child st when
-	 * do_command() uses them to fill in our format string.
-	 * Maybe similarly to above for chunks, do this for all
-	 * strings on the state?
+	 * Due to IKEv1, these IKE (ISAKMP) SA fields need to be
+	 * copied to the Child SA so that they are still available
+	 * when the IKE (ISAKMP) SA gets deleted.
+	 *
+	 * For instance the "generic" Child SA code for up-down
+	 * expects .st_seen_cfg_dns, and the "generic" show-status
+	 * code expects the NAT result.
 	 */
-	jam_str(child->sa.st_xauth_username, sizeof(child->sa.st_xauth_username), ike->sa.st_xauth_username);
-
+	child->sa.hidden_variables = ike->sa.hidden_variables;
 	child->sa.st_seen_cfg_dns = clone_str(ike->sa.st_seen_cfg_dns, "child st_seen_cfg_dns");
 	child->sa.st_seen_cfg_domains = clone_str(ike->sa.st_seen_cfg_domains, "child st_seen_cfg_domains");
 	child->sa.st_seen_cfg_banner = clone_str(ike->sa.st_seen_cfg_banner, "child st_seen_cfg_banner");
+	jam_str(child->sa.st_xauth_username, sizeof(child->sa.st_xauth_username), ike->sa.st_xauth_username);
 
 	return child;
 }
 
 struct child_sa *new_v1_child_sa(struct connection *c,
-				 struct ike_sa *isakmp,
+				 struct ike_sa *ike,
 				 enum sa_role sa_role)
 {
-	return duplicate_state(c, isakmp, CHILD_SA, sa_role);
+	struct child_sa *child = duplicate_state(c, ike, CHILD_SA, sa_role);
+
+	/*
+	 * Propogate IKEv1's IKE (ISAKMP) bits to the child so that
+	 * they are still available after the ISAKMP has been deleted.
+	 */
+	child->sa.st_v1_seen_fragmentation_supported =
+		ike->sa.st_v1_seen_fragmentation_supported;
+	child->sa.st_v1_seen_fragments =
+		ike->sa.st_v1_seen_fragments;
+
+	child->sa.st_v1_quirks = ike->sa.st_v1_quirks;
+
+#   define clone_nss_symkey_field(field) child->sa.field = symkey_addref(ike->sa.logger, #field, ike->sa.field)
+	clone_nss_symkey_field(st_skeyid_nss);
+	clone_nss_symkey_field(st_skeyid_a_nss); /* aka st_skey_ai_nss */
+	clone_nss_symkey_field(st_skeyid_e_nss); /* aka st_skey_ei_nss */
+	clone_nss_symkey_field(st_enc_key_nss);
+#   undef clone_nss_symkey_field
+
+	return child;
 }
 
 struct child_sa *new_v2_child_sa(struct connection *c,
@@ -1276,37 +1289,30 @@ struct child_sa *new_v2_child_sa(struct connection *c,
 				 /* const struct v2_transition *transition */
 				 enum state_kind kind)
 {
+	struct child_sa *child = duplicate_state(c, ike, sa_type, sa_role);
+
 	/* XXX: transitions should be parameter */
 	const struct finite_state *fs = finite_states[kind];
 	const struct v2_transition *transition = fs->v2.child_transition;
 	PASSERT(c->logger, transition != NULL);
-	struct child_sa *child = duplicate_state(c, ike, sa_type, sa_role);
-	change_state(&child->sa, transition->from[0]->kind);
+
+	/*
+	 * Propogate IKEv2 IKE SA bits to the larval IKE SA replacement.
+	 *
+	 * XXX: Handle this in emancipate?
+	 * XXX: are these all really IKE SA bits?
+	 */
+	child->sa.st_ikev2_anon = ike->sa.st_ikev2_anon;
+	child->sa.st_v2_ike_fragmentation_enabled = ike->sa.st_v2_ike_fragmentation_enabled;
+	child->sa.st_seen_redirect_sup = ike->sa.st_seen_redirect_sup;
+	child->sa.st_sent_redirect = ike->sa.st_sent_redirect;
+
+	change_state(&child->sa, fs->kind);
 	set_v2_transition(&child->sa, transition, HERE);
+
 	return child;
 }
 
-#ifdef USE_IKEv1
-struct state *find_state_ikev1(const ike_spis_t *ike_spis, msgid_t msgid)
-{
-	return state_by_ike_spis(IKEv1,
-				 NULL /*ignore-clonedfrom*/,
-				 &msgid/*check v1 msgid*/,
-				 NULL /*ignore-role*/,
-				 ike_spis, NULL, NULL, __func__);
-}
-
-struct state *find_state_ikev1_init(const ike_spi_t *ike_initiator_spi,
-				    msgid_t msgid)
-{
-	return state_by_ike_initiator_spi(IKEv1,
-					  NULL /*ignore-clonedfrom*/,
-					  &msgid /*check v1 msgid*/,
-					  NULL /*ignore-role*/,
-					  ike_initiator_spi, __func__);
-}
-#endif
-
 /*
  * Find the IKEv2 IKE SA with the specified SPIs.
  */
@@ -1432,8 +1438,14 @@ struct child_sa *find_v2_child_sa_by_spi(ipsec_spi_t spi, int8_t protoid,
 		.inbound_spi = spi,
 		.dst = &dst,
 	};
-	struct state_filter sf = { .where = HERE, };
-	while (next_state(NEW2OLD, &sf)) {
+	struct state_filter sf = {
+		.search = {
+			.order = NEW2OLD,
+			.verbose.logger = &global_logger,
+			.where = HERE,
+		},
+	};
+	while (next_state(&sf)) {
 		struct state *st = sf.st;
 		if (v2_spi_predicate(st, &filter))
 			break;
@@ -1458,49 +1470,20 @@ struct child_sa *find_v2_child_sa_by_outbound_spi(struct ike_sa *ike,
 	return pexpect_child_sa(st);
 }
 
-#ifdef USE_IKEv1
-struct v1_msgid_filter {
-	msgid_t msgid;
-};
-
-static bool v1_msgid_predicate(struct state *st, void *context)
-{
-	struct v1_msgid_filter *filter = context;
-	dbg("peer and cookies match on #%lu; msgid=%08" PRIx32 " st_msgid=%08" PRIx32 " st_v1_msgid.phase15=%08" PRIx32,
-	    st->st_serialno, filter->msgid,
-	    st->st_v1_msgid.id, st->st_v1_msgid.phase15);
-	if ((st->st_v1_msgid.phase15 != v1_MAINMODE_MSGID &&
-	     filter->msgid == st->st_v1_msgid.phase15) ||
-	    filter->msgid == st->st_v1_msgid.id) {
-		dbg("p15 state object #%lu found, in %s",
-		    st->st_serialno, st->st_state->name);
-		return true;
-	}
-	return false;
-}
-
-struct state *find_v1_info_state(const ike_spis_t *ike_spis, msgid_t msgid)
-{
-	struct v1_msgid_filter filter = {
-		.msgid = msgid,
-	};
-	return state_by_ike_spis(IKEv1,
-				 NULL /* ignore-clonedfrom */,
-				 NULL /* ignore v1 msgid; see predicate */,
-				 NULL /* ignore-role */,
-				 ike_spis, v1_msgid_predicate,
-				 &filter, __func__);
-}
-#endif
-
 struct ike_sa *find_ike_sa_by_connection(const struct connection *c,
 					 lset_t ok_states,
 					 bool viable_parent)
 {
 	struct ike_sa *best = NULL;
 
-	struct state_filter sf = { .where = HERE, };
-	while (next_state(NEW2OLD, &sf)) {
+	struct state_filter sf = {
+		.search = {
+			.order = NEW2OLD,
+			.verbose.logger = &global_logger,
+			.where = HERE,
+		},
+	};
+	while (next_state(&sf)) {
 		struct state *st = sf.st;
 		if (!IS_PARENT_SA(st)) {
 			continue;
@@ -1553,8 +1536,14 @@ struct ike_sa *find_viable_parent_for_connection(const struct connection *c)
 
 	struct ike_sa *best = NULL;
 
-	struct state_filter sf = { .where = HERE, };
-	while (next_state(NEW2OLD, &sf)) {
+	struct state_filter sf = {
+		.search = {
+			.order = NEW2OLD,
+			.verbose.logger = &global_logger,
+			.where = HERE,
+		},
+	};
+	while (next_state(&sf)) {
 		struct state *st = sf.st;
 		if (!IS_PARENT_SA(st)) {
 			continue;
@@ -1617,8 +1606,8 @@ void show_brief_status(struct show *s)
 {
 	show_separator(s);
 	show(s, "State Information: DDoS cookies %s, %s new IKE connections",
-		     require_ddos_cookies() ? "REQUIRED" : "not required",
-		     drop_new_exchanges() ? "NOT ACCEPTING" : "Accepting");
+	     require_ddos_cookies() ? "REQUIRED" : "not required",
+	     (drop_new_exchanges(show_logger(s)) == NULL ? "Accepting" : "NOT ACCEPTING"));
 
 	show(s, "IKE SAs: total("PRI_CAT"), half-open("PRI_CAT"), open("PRI_CAT"), authenticated("PRI_CAT"), anonymous("PRI_CAT")",
 		  total_ike_sa(),
@@ -1653,8 +1642,14 @@ ipsec_spi_t uniquify_peer_cpi(ipsec_spi_t cpi, const struct state *st, int tries
 	 * Make sure that the result is unique.
 	 * Hard work.  If there is no unique value, we'll loop forever!
 	 */
-	struct state_filter sf = { .where = HERE, };
-	while (next_state(NEW2OLD, &sf)) {
+	struct state_filter sf = {
+		.search = {
+			.order = NEW2OLD,
+			.verbose.logger = &global_logger,
+			.where = HERE,
+		},
+	};
+	while (next_state(&sf)) {
 		struct state *s = sf.st;
 		if (s->st_ipcomp.protocol == &ip_protocol_ipcomp &&
 		    sameaddr(&s->st_connection->remote->host.addr,
@@ -1669,19 +1664,6 @@ ipsec_spi_t uniquify_peer_cpi(ipsec_spi_t cpi, const struct state *st, int tries
 	return cpi;
 }
 
-void merge_quirks(struct state *st, const struct msg_digest *md)
-{
-	struct isakmp_quirks *dq = &st->quirks;
-	const struct isakmp_quirks *sq = &md->quirks;
-
-	dq->xauth_ack_msgid   |= sq->xauth_ack_msgid;
-	dq->modecfg_pull_mode |= sq->modecfg_pull_mode;
-	/* ??? st->quirks.qnat_traversal is never used */
-	if (dq->qnat_traversal_vid < sq->qnat_traversal_vid)
-		dq->qnat_traversal_vid = sq->qnat_traversal_vid;
-	dq->xauth_vid |= sq->xauth_vid;
-}
-
 /*
  * see https://tools.ietf.org/html/rfc7296#section-2.23
  *
@@ -1758,9 +1740,13 @@ void send_n_log_delete_ike_family_now(struct ike_sa **ike,
 
 	struct state_filter cf = {
 		.clonedfrom = (*ike)->sa.st_serialno,
-		.where = where,
+		.search = {
+			.order = NEW2OLD,
+			.verbose.logger = &global_logger,
+			.where = where,
+		},
 	};
-	while(next_state(NEW2OLD, &cf)) {
+	while(next_state(&cf)) {
 		struct child_sa *child = pexpect_child_sa(cf.st);
 		state_attach(&child->sa, logger); /* no detach, going down */
 		switch (child->sa.st_ike_version) {
@@ -1769,11 +1755,11 @@ void send_n_log_delete_ike_family_now(struct ike_sa **ike,
 			if (established_isakmp != NULL) {
 				send_v1_delete(established_isakmp, &child->sa, where);
 			}
-			connection_delete_child(&child, where);
+			connection_teardown_child(&child, REASON_DELETED, where);
 			break;
 		case IKEv2:
 			/* nothing to say? */
-			connection_delete_child(&child, where);
+			connection_teardown_child(&child, REASON_DELETED, where);
 			break;
 		}
 	}
@@ -1783,7 +1769,7 @@ void send_n_log_delete_ike_family_now(struct ike_sa **ike,
 		send_v1_delete(established_isakmp, &established_isakmp->sa, where);
 	}
 
-	connection_delete_ike_family(ike, where);
+	terminate_ike_family(ike, REASON_DELETED, where);
 }
 
 bool require_ddos_cookies(void)
@@ -1793,15 +1779,15 @@ bool require_ddos_cookies(void)
 		 cat_count[CAT_HALF_OPEN_IKE_SA] >= pluto_ddos_threshold);
 }
 
-bool drop_new_exchanges(void)
+err_t drop_new_exchanges(struct logger *logger)
 {
 	if (exiting_pluto) {
-		dbg("%s() exiting_pluto!", __func__);
-		return true;
+		ldbg(logger, "%s() exiting_pluto!", __func__);
+		return "exiting pluto";
 	}
 	if (cat_count[CAT_HALF_OPEN_IKE_SA] >= pluto_max_halfopen) {
-		dbg("%s() half open count >= %u", __func__, pluto_max_halfopen);
-		return true;
+		ldbg(logger, "%s() half open count >= %u", __func__, pluto_max_halfopen);
+		return "too many half open IKE SAs";
 	}
 	return false;
 }
@@ -1884,7 +1870,7 @@ static void list_state_event(struct show *s, struct state *st,
 			jam_string(buf, " schd: ");
 			jam(buf, "%jds", monosecs(pe->ev_time));
 			jam_string(buf, " (in ");
-			jam(buf, "%jds", deltasecs(monotimediff(pe->ev_time, now)));
+			jam(buf, "%jds", deltasecs(monotime_diff(pe->ev_time, now)));
 			jam_string(buf, ") ");
 			jam_prefix(buf, st->logger);
 		}
@@ -1894,9 +1880,13 @@ static void list_state_event(struct show *s, struct state *st,
 void list_state_events(struct show *s, const monotime_t now)
 {
 	struct state_filter sf = {
-		.where = HERE,
+		.search = {
+			.order = OLD2NEW,
+			.verbose.logger = &global_logger,
+			.where = HERE,
+		},
 	};
-	while (next_state(OLD2NEW, &sf)) {
+	while (next_state(&sf)) {
 		struct state *st = sf.st;
 		/* order makes no sense */
 		FOR_EACH_ELEMENT(event, st->st_events) {
@@ -2046,7 +2036,7 @@ void wipe_old_connections(const struct ike_sa *ike)
 		.that_id_eq = &c->remote->host.id,
 		.search = {
 			.order = NEW2OLD,
-			.logger = ike->sa.logger,
+			.verbose.logger = ike->sa.logger,
 			.where = HERE,
 		},
 	};
@@ -2137,73 +2127,77 @@ void wipe_old_connections(const struct ike_sa *ike)
  *
  * DANGER: this intentionally leaks cryptographic secrets.
  */
-void DBG_tcpdump_ike_sa_keys(const struct state *st)
+void LDBG_tcpdump_ike_sa_keys(struct logger *logger, const struct ike_sa *ike)
 {
 	passert(DBGP(DBG_PRIVATE));
 	passert(!is_fips_mode());
 
-	if (st->st_oakley.ta_integ == NULL ||
-	    st->st_oakley.ta_encrypt == NULL)
+	if (ike->sa.st_oakley.ta_integ == NULL ||
+	    ike->sa.st_oakley.ta_encrypt == NULL) {
 		return;
+	}
 
 	/* format initiator SPI */
 	char tispi[3 + 2*IKE_SA_SPI_SIZE];
-	datatot(st->st_ike_spis.initiator.bytes, sizeof(st->st_ike_spis.initiator.bytes),
+	datatot(ike->sa.st_ike_spis.initiator.bytes,
+		sizeof(ike->sa.st_ike_spis.initiator.bytes),
 		'x', tispi, sizeof(tispi));
 
 	/* format responder SPI */
 	char trspi[3 + 2*IKE_SA_SPI_SIZE];
-	datatot(st->st_ike_spis.responder.bytes, sizeof(st->st_ike_spis.responder.bytes),
+	datatot(ike->sa.st_ike_spis.responder.bytes,
+		sizeof(ike->sa.st_ike_spis.responder.bytes),
 		'x', trspi, sizeof(trspi));
 
-	const char *authalgo = st->st_oakley.ta_integ->integ_tcpdump_name;
-	const char *encalgo = st->st_oakley.ta_encrypt->encrypt_tcpdump_name;
+	const char *authalgo = ike->sa.st_oakley.ta_integ->integ_tcpdump_name;
+	const char *encalgo = ike->sa.st_oakley.ta_encrypt->encrypt_tcpdump_name;
 
 	/*
 	 * Text of encryption key length (suffix for encalgo).
 	 * No more than 3 digits, but compiler fears it might be 5.
 	 */
 	char tekl[6] = "";
-	if (st->st_oakley.enckeylen != 0)
+	if (ike->sa.st_oakley.enckeylen != 0) {
 		snprintf(tekl, sizeof(tekl), "%u",
-			 st->st_oakley.enckeylen);
+			 ike->sa.st_oakley.enckeylen);
+	}
 
 	/* v2 IKE authentication key for initiator (256 bit bound) */
-	chunk_t ai = chunk_from_symkey("ai", st->st_skey_ai_nss,
-				       st->logger);
+	chunk_t ai = chunk_from_symkey("ai", ike->sa.st_skey_ai_nss,
+				       ike->sa.logger);
 	char tai[3 + 2 * BYTES_FOR_BITS(256)] = "";
 	datatot(ai.ptr, ai.len, 'x', tai, sizeof(tai));
 	free_chunk_content(&ai);
 
 	/* v2 IKE encryption key for initiator (256 bit bound) */
-	chunk_t ei = chunk_from_symkey("ei", st->st_skey_ei_nss,
-				       st->logger);
+	chunk_t ei = chunk_from_symkey("ei", ike->sa.st_skey_ei_nss,
+				       ike->sa.logger);
 	char tei[3 + 2 * BYTES_FOR_BITS(256)] = "";
 	datatot(ei.ptr, ei.len, 'x', tei, sizeof(tei));
 	free_chunk_content(&ei);
 
-	DBG_log("ikev%d I %s %s %s:%s %s%s:%s",
-		st->st_ike_version,
+	LDBG_log(logger, "ikev%d I %s %s %s:%s %s%s:%s",
+		ike->sa.st_ike_version,
 		tispi, trspi,
 		authalgo, tai,
 		encalgo, tekl, tei);
 
 	/* v2 IKE authentication key for responder (256 bit bound) */
-	chunk_t ar = chunk_from_symkey("ar", st->st_skey_ar_nss,
-				       st->logger);
+	chunk_t ar = chunk_from_symkey("ar", ike->sa.st_skey_ar_nss,
+				       ike->sa.logger);
 	char tar[3 + 2 * BYTES_FOR_BITS(256)] = "";
 	datatot(ar.ptr, ar.len, 'x', tar, sizeof(tar));
 	free_chunk_content(&ar);
 
 	/* v2 IKE encryption key for responder (256 bit bound) */
-	chunk_t er = chunk_from_symkey("er", st->st_skey_er_nss,
-				       st->logger);
+	chunk_t er = chunk_from_symkey("er", ike->sa.st_skey_er_nss,
+				       ike->sa.logger);
 	char ter[3 + 2 * BYTES_FOR_BITS(256)] = "";
 	datatot(er.ptr, er.len, 'x', ter, sizeof(ter));
 	free_chunk_content(&er);
 
-	DBG_log("ikev%d R %s %s %s:%s %s%s:%s",
-		st->st_ike_version,
+	LDBG_log(logger, "ikev%d R %s %s %s:%s %s%s:%s",
+		ike->sa.st_ike_version,
 		tispi, trspi,
 		authalgo, tar,
 		encalgo, tekl, ter);
