@@ -18,6 +18,7 @@
  * Copyright (C) 2017 Mayank Totale <mtotale@gmail.com>
  * Copyright (C) 2020 Yulia Kuzovkova <ukuzovkova@gmail.com>
  * Copyright (C) 2021 Paul Wouters <paul.wouters@aiven.io>
+ * Copyright (C) 2020 Nupur Agrawal <nupur202000@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -39,6 +40,7 @@
 #include <pk11pub.h>
 #include <x509.h>
 
+#include "verbose.h"
 #include "deltatime.h"
 #include "monotime.h"
 #include "reqid.h"
@@ -60,6 +62,7 @@
 #include "sa_role.h"
 #include "packet.h"
 #include "state_category.h"
+#include "terminate_reason.h"
 
 struct whack_message;
 struct v2_transition;
@@ -171,23 +174,6 @@ struct hidden_variables {
 	ip_address st_natd;
 };
 
-/*
- * For auditing, why an SA is being deleted.
- */
-enum delete_reason {
-#define DELETE_REASON_FLOOR 0
-	REASON_UNKNOWN = DELETE_REASON_FLOOR, /* aka other */
-	REASON_CRYPTO_TIMEOUT,
-	REASON_EXCHANGE_TIMEOUT,
-	REASON_TOO_MANY_RETRANSMITS,
-	REASON_SUPERSEDED_BY_NEW_SA,
-	REASON_CRYPTO_FAILED,
-	REASON_AUTH_FAILED,
-	REASON_TRAFFIC_SELECTORS_FAILED,
-	REASON_COMPLETED,
-#define DELETE_REASON_ROOF (REASON_COMPLETED + 1)
-};
-
 /*
  * Abstract state machine that drives the parent and child SA.
  *
@@ -276,6 +262,12 @@ struct state {
 
 	ip_endpoint st_remote_endpoint;        /* where to send packets to */
 
+	/*
+	 * IKE SA is from Session Resumption; IKE_AUTH uses this to
+	 * skip non resume stuff.
+	 */
+	struct resume_session *st_v2_resume_session;
+
 	/*
 	 * Digital Signature authentication.
 	 *
@@ -350,9 +342,8 @@ struct state {
 
 	/* Initialization Vectors for IKEv1 IKE encryption */
 
-	struct crypt_mac st_v1_new_iv;	/* tentative IV (calculated from current packet) */
-	struct crypt_mac st_v1_iv;		/* accepted IV (after packet passes muster) */
-	struct crypt_mac st_v1_ph1_iv;	/* IV at end of phase 1 */
+	struct crypt_mac st_v1_phase_1_iv;	/* IV for Phase 1 exchange */
+	struct crypt_mac st_v1_phase_2_iv;	/* IV for Phase 2 (and Phase 1.5 exchanges */
 
 	/* end of IKEv1-only things */
 #endif
@@ -376,7 +367,7 @@ struct state {
 	struct p_dns_req *ipseckey_dnsr;    /* ipseckey of that end */
 	struct p_dns_req *ipseckey_fwd_dnsr;/* validate IDi that IP in forward A/AAAA */
 
-	chunk_t st_active_redirect_gw;		/* needed for sending of REDIRECT in informational */
+	char *st_active_redirect_gw;		/* needed for sending of REDIRECT in informational */
 
 	/*
 	 * IKEv2 intermediate exchange.
@@ -513,7 +504,7 @@ struct state {
 	 */
 	struct {
 		enum sa_type sa_type;
-		enum delete_reason delete_reason;
+		enum terminate_reason terminate_reason;
 	} st_pstats;
 
 	retransmit_t st_retransmit;	/* retransmit counters; opaque */
@@ -558,12 +549,9 @@ struct state {
 	chunk_t st_p1isa;	/* v1 Phase 1 initiator SA (Payload) for HASH */
 
 	PK11SymKey *st_skeyid_nss;	/* v1 Key material */
+#define st_v1_isakmp_skeyid_d st_skey_d_nss
 
-	/* v1 names are aliases for subset of v2 fields (#define) */
-
-#define st_skeyid_d_nss st_skey_d_nss	/* v1 KM for non-ISAKMP key derivation */
 	PK11SymKey *st_skey_d_nss;	/* v2 KM for non-ISAKMP key derivation */
-
 #define st_skeyid_a_nss st_skey_ai_nss	/* v1 IKE authentication KM */
 	PK11SymKey *st_skey_ai_nss;	/* v2 IKE authentication key for initiator */
 	PK11SymKey *st_skey_ar_nss;	/* v2 IKE authentication key for responder */
@@ -588,8 +576,12 @@ struct state {
 	/*
 	 * Post-quantum Preshared Key variables (v2)
 	 */
-	bool st_ppk_used;			/* both ends agreed on PPK ID and PPK */
-	bool st_v2_ike_ppk_enabled;		/* does remote peer support PPK? */
+	bool st_ppk_ike_auth_used;			/* both ends agreed on PPK ID and PPK (RFC 8784) */
+	enum {
+		PPK_DISABLED = 0,
+		PPK_IKE_AUTH = 1,		/* RFC 8784 */
+		PPK_IKE_INTERMEDIATE = 2,	/* draft-ietf-ipsecme-ikev2-qr-alt-04 */
+	} st_v2_ike_ppk;
 
 	chunk_t st_no_ppk_auth;
 	PK11SymKey *st_sk_d_no_ppk;
@@ -652,13 +644,14 @@ struct state {
 	uint32_t st_dpd_rdupcount;		/* openbsd isakmpd bug workaround */
 	struct state_event *st_v1_dpd_event;	/* backpointer for IKEv1 DPD events */
 
-	struct isakmp_quirks quirks;            /* work arounds for faults in other products */
+	struct isakmp_quirks st_v1_quirks;	/* work arounds for faults in other products */
 	bool st_xauth_soft;                     /* XAUTH failed but policy is to soft fail */
 	bool st_v1_seen_fragmentation_supported;	/* v1 frag vid */
 	bool st_v2_ike_fragmentation_enabled;	/* v2 frag notify */
 	bool st_seen_hashnotify;		/* did we receive hash algo notification in IKE_INIT, then send in response as well */
 	bool st_v1_seen_fragments;              /* did we receive ike fragments from peer, if so use them in return as well */
 	bool st_seen_no_tfc;			/* did we receive ESP_TFC_PADDING_NOT_SUPPORTED */
+	bool st_seen_and_use_iptfs;		/* did we receive USE_AGGFRAG for IPTFS */
 	bool st_seen_redirect_sup;		/* did we receive IKEv2_REDIRECT_SUPPORTED */
 	bool st_sent_redirect;			/* did we send IKEv2_REDIRECT in IKE_AUTH (response) */
 	bool st_skip_revival_as_redirecting;	/* hack */
@@ -847,10 +840,6 @@ struct ike_sa *find_ike_sa_by_connection(const struct connection *c,
 					 lset_t ok_states,
 					 bool viable_parent);
 
-struct state *find_state_ikev1(const ike_spis_t *ike_spis, msgid_t msgid);
-struct state *find_state_ikev1_init(const ike_spi_t *ike_initiator_spi,
-				    msgid_t msgid);
-
 extern struct ike_sa *find_v2_ike_sa(const ike_spis_t *ike_spis,
 				     enum sa_role local_ike_role);
 extern struct ike_sa *find_v2_ike_sa_by_initiator_spi(const ike_spi_t *ike_initiator_spi,
@@ -860,9 +849,6 @@ struct child_sa *find_v2_child_sa_by_outbound_spi(struct ike_sa *ike,
 						  uint8_t protoid,
 						  ipsec_spi_t outbound_spi);
 
-extern struct state *find_v1_info_state(const ike_spis_t *ike_spis,
-					msgid_t msgid);
-
 extern void show_brief_status(struct show *s);
 
 extern ipsec_spi_t uniquify_peer_cpi(ipsec_spi_t cpi, const struct state *st, int tries);
@@ -878,7 +864,7 @@ extern void delete_cryptographic_continuation(struct state *st);
 extern void change_v1_state(struct state *st, enum state_kind new_state);
 extern void change_v2_state(struct state *st);
 
-extern bool drop_new_exchanges(void);
+err_t drop_new_exchanges(struct logger *logger);
 extern bool require_ddos_cookies(void);
 extern void show_globalstate_status(struct show *s);
 extern void update_ike_endpoints(struct ike_sa *ike, const struct msg_digest *md);
@@ -894,7 +880,7 @@ struct child_sa *find_v2_child_sa_by_spi(ipsec_spi_t spi, int8_t protoid,
 
 void connswitch_state_and_log(struct state *st, struct connection *c);
 
-void DBG_tcpdump_ike_sa_keys(const struct state *st);
+void LDBG_tcpdump_ike_sa_keys(struct logger *logger, const struct ike_sa *ike);
 
 /*
  * For iterating over the state DB.
@@ -910,22 +896,37 @@ void DBG_tcpdump_ike_sa_keys(const struct state *st);
  */
 
 struct state_filter {
-	/* filters */
-	enum ike_version ike_version;
-	const ike_spis_t *ike_spis;	/* hashed */
-	so_serial_t clonedfrom;
-	co_serial_t connection_serialno;
-	/* current result (can be safely deleted) */
+	/*
+	 * Filters.
+	 */
+	const enum ike_version ike_version;
+	const ike_spis_t *const ike_spis;	/* hashed */
+	const so_serial_t clonedfrom;
+	const co_serial_t connection_serialno;
+
+	/*
+	 * Current result (can be safely deleted).
+	 */
 	struct state *st;
-	/* internal: handle on next entry */
-	struct list_entry *internal;
-	/* internal: total matches so far */
-	unsigned count;
-	/* .where MUST BE LAST (See GCC bug 102288) */
-	where_t where;
+
+	/*
+	 * internal
+	 */
+	struct list_entry *internal;	/* handle on next entry */
+	unsigned count;			/* total matches so far */
+
+	/*
+	 * Required fields.
+	 */
+	struct /*search*/ {
+		const enum chrono order;
+		struct verbose verbose; /* writable */
+		/* .where MUST BE LAST (See GCC bug 102288) */
+		const where_t where;
+	} search;
 };
 
-bool next_state(enum chrono advance, struct state_filter *query);
+bool next_state(struct state_filter *query);
 
 extern void set_sa_expire_next_event(enum sa_expire_kind expire, struct child_sa *child);
 
