@@ -56,36 +56,33 @@
 #include "ikev1_xauth.h"
 #include "nat_traversal.h"
 #include "terminate.h"
-#include "whack_connection.h"		/* for whack_connection() */
+#include "visit_connection.h"		/* for whack_connection() */
 #include "ikev1_delete.h"
 #include "ikev2_delete.h"
 #include "pluto_stats.h"
 #include "revival.h"
 
-static void delete_v1_states(struct connection *c,
-			     struct ike_sa **ike,
-			     struct child_sa **child,
-			     enum whack_state whacamole)
+static void terminate_v1_state(struct connection *c,
+			       struct ike_sa **ike,
+			       struct child_sa **child,
+			       enum connection_visit_kind visit_kind)
 {
-	switch (whacamole) {
-	case WHACK_START_IKE:
+	switch (visit_kind) {
+
+	case CONNECTION_IKE_PREP:
 		/*
+		 * Prepare the IKE SA for deletion.
+		 *
 		 * IKEv1 announces the death of the ISAKMP SA after
 		 * all the children have gone (reverse of IKEv2).
 		 */
 		state_attach(&(*ike)->sa, c->logger);
 		(*ike)->sa.st_viable_parent = false;
 		return;
-	case WHACK_LURKING_CHILD:
-		state_attach(&(*child)->sa, c->logger);
-		connection_delete_child(child, HERE);
-		return;
-	case WHACK_LURKING_IKE:
-		state_attach(&(*ike)->sa, c->logger);
-		delete_ike_sa(ike);
-		return;
-	case WHACK_CHILD:
-	case WHACK_CUCKOO:
+
+	case CONNECTION_IKE_CHILD: /* ignore IKE */
+	case CONNECTION_CUCKOO_CHILD:
+	case CONNECTION_ORPHAN_CHILD:
 	{
 		/*
 		 * Can't always assume IKE is suitable for sending
@@ -99,98 +96,141 @@ static void delete_v1_states(struct connection *c,
 			established_isakmp_sa_for_state(&(*child)->sa, /*viable-parent*/false);
 		/* IKEv1 has cuckoos */
 		llog_n_maybe_send_v1_delete(isakmp, &(*child)->sa, HERE);
-		connection_delete_child(child, HERE);
+		connection_teardown_child(child, REASON_DELETED, HERE);
 		return;
 	}
-	case WHACK_ORPHAN:
-		/* IKEv1 has orphans */
+
+	case CONNECTION_LURKING_CHILD:
 		state_attach(&(*child)->sa, c->logger);
-		PEXPECT(c->logger, ike == NULL);
-		connection_delete_child(child, HERE);
+		connection_teardown_child(child, REASON_DELETED, HERE);
 		return;
-	case WHACK_SIBLING:
+	case CONNECTION_LURKING_IKE:
+		state_attach(&(*ike)->sa, c->logger);
+		delete_ike_sa(ike);
+		return;
+
+	case CONNECTION_CHILD_SIBLING:
 		/*
 		 * When IKEv1 deletes an IKE SA any siblings are
 		 * orphaned.
 		 */
 		return;
-	case WHACK_IKE:
+	case CONNECTION_CHILDLESS_IKE:
 		/*
 		 * When IKEv1 deletes an IKE SA it always sends a
-		 * delete notify; hence handle this in WHACK_STOP_IKE.
+		 * delete notify; hence handle this in
+		 * CONNECTION_IKE_POST.
 		 */
 		return;
-	case WHACK_STOP_IKE:
+
+	case CONNECTION_IKE_POST:
 	{
 		struct ike_sa *isakmp =
 			established_isakmp_sa_for_state(&(*ike)->sa, /*viable-parent*/false);
 		llog_n_maybe_send_v1_delete(isakmp, &(*ike)->sa, HERE);
-		connection_delete_ike(ike, HERE);
+		connection_teardown_ike(ike, REASON_DELETED, HERE);
 		return;
 	}
+
 	}
-	bad_case(whacamole);
+	bad_case(visit_kind);
 }
 
-static void delete_v2_states(struct connection *c,
-			     struct ike_sa **ike,
-			     struct child_sa **child,
-			     enum whack_state whacamole)
+static void terminate_v2_states(struct connection *c,
+				struct ike_sa **ike,
+				struct child_sa **child,
+				enum connection_visit_kind visit_kind)
 {
-	switch (whacamole) {
-	case WHACK_START_IKE:
-		/* announce to the world */
+	switch (visit_kind) {
+	case CONNECTION_IKE_PREP:
+		/*
+		 * Prepare the IKE SA for deletion.
+		 *
+		 * IKEv2 announces the death first.
+		 */
 		state_attach(&(*ike)->sa, c->logger);
 		(*ike)->sa.st_viable_parent = false;
 		record_n_send_n_log_v2_delete(*ike, HERE);
 		return;
-	case WHACK_LURKING_CHILD:
+
+	case CONNECTION_IKE_CHILD:
+		state_attach(&(*child)->sa, c->logger);
+		connection_teardown_child(child, REASON_DELETED, HERE);
+		return;
+	case CONNECTION_CUCKOO_CHILD:
+	case CONNECTION_ORPHAN_CHILD:
+		state_attach(&(*child)->sa, c->logger);
+		PEXPECT(c->logger, ike == NULL);
+		connection_teardown_child(child, REASON_DELETED, HERE);
+		return;
+
+	case CONNECTION_LURKING_CHILD:
 		state_attach(&(*child)->sa, c->logger);
 		delete_child_sa(child);
 		return;
-	case WHACK_LURKING_IKE:
+	case CONNECTION_LURKING_IKE:
+		/*
+		 * For instance, a larval IKE SA, either stuck waiting
+		 * for IKE_SA_INIT response crossed by the peer
+		 * establishing the IKE SA.
+		 *
+		 * This will strip the connection of the IKE SA.  If
+		 * the IKE SA has children then they too are deleted,
+		 * but when would this be the case?
+		 */
 		state_attach(&(*ike)->sa, c->logger);
-		connection_delete_ike_family(ike, HERE);
+		terminate_ike_family(ike, REASON_DELETED, HERE);
 		return;
-	case WHACK_CHILD:
+
+	case CONNECTION_CHILD_SIBLING:
 		state_attach(&(*child)->sa, c->logger);
-		connection_delete_child(child, HERE);
+		connection_teardown_child(child, REASON_DELETED, HERE);
 		return;
-	case WHACK_CUCKOO:
-		state_attach(&(*child)->sa, c->logger);
-		PEXPECT(c->logger, ike == NULL);
-		connection_delete_child(child, HERE);
+	case CONNECTION_CHILDLESS_IKE:
+	{
+		/*
+		 * Since connection_teardown_child() was not
+		 * dispatched the IKE SA gets to to the teardown.
+		 *
+		 * At this point the IKE SA should have no children.
+		 */
+		struct state_filter child = {
+			.clonedfrom = (*ike)->sa.st_serialno,
+			.search = {
+				.order = NEW2OLD,
+				.verbose.logger = c->logger,
+				.where = HERE,
+			},
+		};
+		if (next_state(&child)) {
+			llog_pexpect(c->logger, child.search.where,
+				     "IKE SA "PRI_SO" has unexpected Child SA "PRI_SO,
+				     pri_so((*ike)->sa.st_serialno),
+				     pri_so(child.st->st_serialno));
+			/* get out of Dodge! */
+			return;
+		}
+		connection_teardown_ike(ike, REASON_DELETED, HERE);
 		return;
-	case WHACK_ORPHAN:
-		state_attach(&(*child)->sa, c->logger);
-		llog_pexpect(c->logger, HERE, "unexpected orphan Child SA "PRI_SO,
-			     (*child)->sa.st_serialno);
-		PEXPECT(c->logger, ike == NULL);
-		delete_child_sa(child);
-		return;
-	case WHACK_SIBLING:
-		state_attach(&(*child)->sa, c->logger);
-		connection_delete_child(child, HERE);
-		return;
-	case WHACK_IKE:
-		connection_delete_ike_family(ike, HERE);
-		return;
-	case WHACK_STOP_IKE:
+	}
+
+	case CONNECTION_IKE_POST:
 		delete_ike_sa(ike);
 		return;
 	}
-	bad_case(whacamole);
+
+	bad_case(visit_kind);
 }
 
-struct whack_state_context {
+struct visit_connection_state_context {
 	unsigned count;
 };
 
-static void delete_states(struct connection *c,
-			  struct ike_sa **ike,
-			  struct child_sa **child,
-			  enum whack_state whacamole,
-			  struct whack_state_context *context)
+static void terminate_connection_states(struct connection *c,
+					struct ike_sa **ike,
+					struct child_sa **child,
+					enum connection_visit_kind visit_kind,
+					struct visit_connection_state_context *context)
 {
 	if (context->count == 0) {
 		llog(RC_LOG, c->logger, "terminating SAs using this connection");
@@ -198,10 +238,11 @@ static void delete_states(struct connection *c,
 	context->count++;
 	switch (c->config->ike_version) {
 	case IKEv1:
-		delete_v1_states(c, ike, child, whacamole);
+		terminate_v1_state(c, ike, child, visit_kind);
 		return;
 	case IKEv2:
-		delete_v2_states(c, ike, child, whacamole);
+		/* may need to delete IKE tree */
+		terminate_v2_states(c, ike, child, visit_kind);
 		return;
 	}
 	bad_case(c->config->ike_version);
@@ -209,8 +250,8 @@ static void delete_states(struct connection *c,
 
 void terminate_all_connection_states(struct connection *c, where_t where)
 {
-	struct whack_state_context context = {0};
-	whack_connection_states(c, delete_states, &context, where);
+	struct visit_connection_state_context context = {0};
+	visit_connection_states(c, terminate_connection_states, &context, where);
 	/* caller must hold a reference */
 	pmemory(c);
 }
@@ -329,7 +370,7 @@ void terminate_and_down_and_unroute_connections(struct connection *c, where_t wh
 				.ike_version = c->config->ike_version,
 				.search = {
 					.order = OLD2NEW,
-					.logger = c->logger,
+					.verbose.logger = c->logger,
 					.where = where,
 				},
 			};
@@ -365,7 +406,7 @@ void terminate_and_down_and_unroute_connections(struct connection *c, where_t wh
 			.ike_version = c->config->ike_version,
 			.search = {
 				.order = OLD2NEW,
-				.logger = c->logger,
+				.verbose.logger = c->logger,
 				.where = where,
 			},
 		};
@@ -420,7 +461,7 @@ void terminate_and_delete_connections(struct connection **cp,
 			.ike_version = (*cp)->config->ike_version,
 			.search = {
 				.order = OLD2NEW,
-				.logger = logger,
+				.verbose.logger = logger,
 				.where = where,
 			},
 		};
@@ -444,52 +485,7 @@ void terminate_and_delete_connections(struct connection **cp,
 	bad_enum((*cp)->logger, &connection_kind_names, (*cp)->local->kind);
 }
 
-/*
- * If the IKE SA's connection has a direct Child SA (shares
- * connection) that owns the route then send a delete/timeout to that
- * Child SA first.
- *
- * This way the IKE SA's connection can jump to the front of the
- * revival queue (without this an IKE SA with multiple children ends
- * up with its chilren squabbling over which SA should be revived
- * first).
- *
- * Also remember if there was a direct child.  The event only gets
- * dispatched to the IKE SA when there wasn't a child (such as during
- * IKE_SA_INIT).
- */
-
-static bool zap_connection_child(struct ike_sa **ike,
-				 void (*zap_child)(struct child_sa **child, where_t where),
-				 struct child_sa **child, where_t where)
-{
-
-	bool dispatched_to_child;
-	(*child) = child_sa_by_serialno((*ike)->sa.st_connection->negotiating_child_sa);
-	if ((*child) == NULL) {
-		dispatched_to_child = false;
-		ldbg_routing((*ike)->sa.logger, "  IKE SA's connection has no Child SA "PRI_SO,
-			     pri_so((*ike)->sa.st_connection->negotiating_child_sa));
-	} else if ((*child)->sa.st_clonedfrom != (*ike)->sa.st_serialno) {
-		dispatched_to_child = false;
-		ldbg_routing((*ike)->sa.logger, "  IKE SA is not the parent of the connection's Child SA "PRI_SO,
-			     pri_so((*child)->sa.st_serialno));
-	} else {
-		ldbg_routing((*ike)->sa.logger, "  dispatching delete to Child SA "PRI_SO,
-			     pri_so((*child)->sa.st_serialno));
-		state_attach(&(*child)->sa, (*ike)->sa.logger);
-		/* will delete child and its logger */
-		dispatched_to_child = true;
-		zap_child(child, where); /* always dispatches here*/
-		PEXPECT((*ike)->sa.logger, dispatched_to_child);
-		PEXPECT((*ike)->sa.logger, (*child) == NULL); /*gone!*/
-		PEXPECT((*ike)->sa.logger, (*ike)->sa.st_connection->negotiating_child_sa == SOS_NOBODY);
-		PEXPECT((*ike)->sa.logger, (*ike)->sa.st_connection->established_child_sa == SOS_NOBODY);
-	}
-	return dispatched_to_child;
-}
-
-static void zap_v1_child(struct ike_sa **ike, struct child_sa *child)
+static void terminate_v1_child(struct ike_sa **ike, struct child_sa *child)
 {
 	/*
 	 * With IKEv1, deleting an ISAKMP SA only deletes larval
@@ -511,9 +507,9 @@ static void zap_v1_child(struct ike_sa **ike, struct child_sa *child)
 	}
 }
 
-static void zap_v2_child(struct ike_sa **ike, struct child_sa *child,
-			 void (*zap_child)(struct child_sa **child, where_t where),
-			 where_t where)
+static void terminate_v2_child(struct ike_sa **ike, struct child_sa *child,
+			       enum terminate_reason reason,
+			       where_t where)
 {
 
 	/*
@@ -533,9 +529,9 @@ static void zap_v2_child(struct ike_sa **ike, struct child_sa *child,
 	if (cc->established_child_sa == child->sa.st_serialno) {
 		PEXPECT((*ike)->sa.logger, IS_IPSEC_SA_ESTABLISHED(&child->sa));
 		/* will delete child and its logger */
-		ldbg_routing((*ike)->sa.logger, "    zapping established Child SA "PRI_SO,
+		ldbg_routing((*ike)->sa.logger, "    teardown established Child SA "PRI_SO,
 			     pri_so(child->sa.st_serialno));
-		zap_child(&child, where);
+		connection_teardown_child(&child, reason, where);
 		return;
 	}
 
@@ -552,9 +548,9 @@ static void zap_v2_child(struct ike_sa **ike, struct child_sa *child,
 
 	if (cc->negotiating_child_sa == child->sa.st_serialno) {
 		/* will delete child and its logger */
-		ldbg_routing((*ike)->sa.logger, "    zapping larval Child SA "PRI_SO,
+		ldbg_routing((*ike)->sa.logger, "    teardown larval Child SA "PRI_SO,
 			     pri_so(child->sa.st_serialno));
-		zap_child(&child, where);
+		connection_teardown_child(&child, reason, where);
 		return;
 	}
 
@@ -569,23 +565,53 @@ static void zap_v2_child(struct ike_sa **ike, struct child_sa *child,
 		return;
 	}
 
-	ldbg_routing((*ike)->sa.logger, "    zapping Child SA "PRI_SO,
+	ldbg_routing((*ike)->sa.logger, "    delete Child SA "PRI_SO,
 		     pri_so(child->sa.st_serialno));
 	delete_child_sa(&child);
 }
 
-static void connection_zap_ike_family(struct ike_sa **ike,
-				      void (*zap_ike)(struct ike_sa **ike, where_t where),
-				      void (*zap_child)(struct child_sa **child, where_t where),
-				      where_t where)
+void terminate_ike_family(struct ike_sa **ike,
+			  enum terminate_reason reason,
+			  where_t where)
 {
 	ldbg_routing((*ike)->sa.logger, "%s()", __func__);
+	pstat_sa_failed(&(*ike)->sa, reason);
 
 	ldbg((*ike)->sa.logger, "  IKE SA is no longer viable");
 	(*ike)->sa.st_viable_parent = false;
 
-	struct child_sa *connection_child = NULL;
-	zap_connection_child(ike, zap_child, &connection_child, where);
+	/*
+	 * When the IKE SA's connection has a direct Child SA (i.e.,
+	 * shares connection) that owns the route then teardown that
+	 * Child SA first.
+	 *
+	 * This way the IKE SA's connection can jump to the front of
+	 * the revival queue (without this an IKE SA with multiple
+	 * children ends up with its chilren squabbling over which SA
+	 * should be revived first).
+	 *
+	 * When this isn't the case, the for-each-child will instead
+	 * do the terminating.
+	 */
+
+	struct child_sa *connection_child = child_sa_by_serialno((*ike)->sa.st_connection->negotiating_child_sa);
+	if (connection_child == NULL) {
+		ldbg_routing((*ike)->sa.logger, "  IKE SA's connection has no Child SA "PRI_SO,
+			     pri_so((*ike)->sa.st_connection->negotiating_child_sa));
+	} else if (connection_child->sa.st_clonedfrom != (*ike)->sa.st_serialno) {
+		ldbg_routing((*ike)->sa.logger, "  IKE SA is not the parent of the connection's Child SA "PRI_SO,
+			     pri_so(connection_child->sa.st_serialno));
+		connection_child = NULL;
+	} else {
+		ldbg_routing((*ike)->sa.logger, "  dispatching delete to Child SA "PRI_SO,
+			     pri_so(connection_child->sa.st_serialno));
+		state_attach(&connection_child->sa, (*ike)->sa.logger);
+		/* will delete child and its logger */
+		connection_teardown_child(&connection_child, reason, where); /* always dispatches here*/
+		PEXPECT((*ike)->sa.logger, connection_child == NULL); /*gone!*/
+		PEXPECT((*ike)->sa.logger, (*ike)->sa.st_connection->negotiating_child_sa == SOS_NOBODY);
+		PEXPECT((*ike)->sa.logger, (*ike)->sa.st_connection->established_child_sa == SOS_NOBODY);
+	}
 
 	/*
 	 * We are a parent: prune any remaining children and then
@@ -594,34 +620,27 @@ static void connection_zap_ike_family(struct ike_sa **ike,
 
 	struct state_filter cf = {
 		.clonedfrom = (*ike)->sa.st_serialno,
-		.where = HERE,
+		.search = {
+			.order = NEW2OLD,
+			.verbose.logger = (*ike)->sa.logger,
+			.where = HERE,
+		},
 	};
-	while(next_state(NEW2OLD, &cf)) {
+	while(next_state(&cf)) {
 		struct child_sa *child = pexpect_child_sa(cf.st);
 
 		switch (child->sa.st_ike_version) {
 		case IKEv1:
-			zap_v1_child(ike, child);
+			terminate_v1_child(ike, child);
 			break;
 		case IKEv2:
-			zap_v2_child(ike, child, zap_child, where);
+			terminate_v2_child(ike, child, reason, where);
 			break;
 		}
 	}
 
 	/* delete self */
-	zap_ike(ike, where);
-}
-
-void connection_timeout_ike_family(struct ike_sa **ike, where_t where)
-{
-	pstat_sa_failed(&(*ike)->sa, REASON_TOO_MANY_RETRANSMITS);
-	connection_zap_ike_family(ike, connection_timeout_ike, connection_timeout_child, where);
-}
-
-void connection_delete_ike_family(struct ike_sa **ike, where_t where)
-{
-	connection_zap_ike_family(ike, connection_delete_ike, connection_delete_child, where);
+	connection_teardown_ike(ike, reason, where);
 }
 
 void connection_delete_v1_state(struct state **st, where_t where)
@@ -629,10 +648,10 @@ void connection_delete_v1_state(struct state **st, where_t where)
 	PEXPECT((*st)->logger, (*st)->st_ike_version == IKEv1);
 	if (IS_PARENT_SA(*st)) {
 		struct ike_sa *ike = pexpect_parent_sa(*st);
-		connection_delete_ike(&ike, where);
+		connection_teardown_ike(&ike, REASON_DELETED, where);
 	} else {
 		struct child_sa *child = pexpect_child_sa(*st);
-		connection_delete_child(&child, where);
+		connection_teardown_child(&child, REASON_DELETED, where);
 	}
 	(*st) = NULL;
 }
