@@ -278,31 +278,35 @@ static void dispatch_event(struct state *st, enum event_type event_type,
 
 #ifdef USE_IKEv1
 	case EVENT_v1_SEND_XAUTH:
+	{
 		ldbg(st->logger, "XAUTH: event EVENT_v1_SEND_XAUTH #%lu %s",
 		     st->st_serialno, st->st_state->name);
-		xauth_send_request(st);
+		struct ike_sa *ike = pexpect_ike_sa(st);
+		if (ike != NULL) {
+			xauth_send_request(ike);
+		}
 		break;
+	}
 #endif
 
 	case EVENT_v2_LIVENESS:
-		liveness_check(st);
+		event_v2_liveness(st);
 		break;
 
 	case EVENT_v2_REKEY:
-		pexpect(st->st_ike_version == IKEv2);
 		event_v2_rekey(st, detach_whack);
 		break;
 
+	case EVENT_v2_REPLACE:
+		event_v2_replace(st, detach_whack);
+		break;
+
 #ifdef USE_IKEv1
 	case EVENT_v1_REPLACE:
 		event_v1_replace(st, now);
 		break;
 #endif
 
-	case EVENT_v2_REPLACE:
-		event_v2_replace(st, now);
-		break;
-
 	case EVENT_v1_EXPIRE:
 	{
 		struct connection *c = st->st_connection;
@@ -373,7 +377,7 @@ static void dispatch_event(struct state *st, enum event_type event_type,
 			llog_pexpect(child->sa.logger, HERE,
 				     "Child SA lost its IKE SA #%lu",
 				     child->sa.st_clonedfrom);
-			connection_delete_child(&child, HERE);
+			connection_teardown_child(&child, REASON_DELETED, HERE);
 			st = NULL;
 		} else if (IS_IKE_SA_ESTABLISHED(st)) {
 			/* IKEv2 parent, delete children too */
@@ -395,7 +399,7 @@ static void dispatch_event(struct state *st, enum event_type event_type,
 		} else {
 			struct child_sa *child = pexpect_child_sa(st);
 			state_attach(&child->sa, logger);
-			connection_delete_child(&child, HERE);
+			connection_teardown_child(&child, REASON_DELETED, HERE);
 			st = NULL;
 		}
 		break;
@@ -471,10 +475,10 @@ static void dispatch_event(struct state *st, enum event_type event_type,
 		 */
 		if (IS_IKE_SA(st)) {
 			struct ike_sa *ike = pexpect_ike_sa(st);
-			connection_delete_ike_family(&ike, HERE);
+			terminate_ike_family(&ike, REASON_DELETED, HERE);
 		} else {
 			struct child_sa *child = pexpect_child_sa(st);
-			connection_delete_child(&child, HERE);
+			connection_teardown_child(&child, REASON_DELETED, HERE);
 		}
 		st = NULL;
 		break;
@@ -494,7 +498,7 @@ static void dispatch_event(struct state *st, enum event_type event_type,
 		     "initiator/responder/response processor timeout after %s seconds",
 		     str_deltatime(event_delay, &dtb));
 		struct ike_sa *ike = pexpect_ike_sa(st);
-		connection_timeout_ike_family(&ike, HERE);
+		terminate_ike_family(&ike, REASON_EXCHANGE_TIMEOUT, HERE);
 		st = NULL;
 		break;
 	}
@@ -503,35 +507,44 @@ static void dispatch_event(struct state *st, enum event_type event_type,
 	case EVENT_v1_DPD:
 		event_v1_dpd(st);
 		break;
+#endif
 
+#ifdef USE_IKEv1
 	case EVENT_v1_DPD_TIMEOUT:
 		event_v1_dpd_timeout(st);
 		break;
+#endif
 
+#ifdef USE_IKEv1
 #ifdef USE_PAM_AUTH
 	case EVENT_v1_PAM_TIMEOUT:
+	{
 		ldbg(st->logger, "PAM thread timeout on state #%lu", st->st_serialno);
-		pam_auth_abort(st, "timeout");
-		/*
-		 * Things get cleaned up when the PAM process exits.
-		 *
-		 * Should this schedule an event for the case when the
-		 * child process (which is SIGKILLed) doesn't exit!?!
-		 */
+		struct ike_sa *ike = pexpect_ike_sa(st);
+		if (ike != NULL) {
+			pam_auth_abort(ike, "timeout");
+			/*
+			 * Things get cleaned up when the PAM process exits.
+			 *
+			 * Should this schedule an event for the case when the
+			 * child process (which is SIGKILLed) doesn't exit!?!
+			 */
+		}
 		break;
+	}
 #endif
 #endif
 	case EVENT_v1_CRYPTO_TIMEOUT:
 		state_attach(st, logger);
 		ldbg(st->logger, "event crypto_failed on state #%lu, aborting",
 		     st->st_serialno);
-		pstat_sa_failed(st, REASON_CRYPTO_TIMEOUT);
 		if (IS_PARENT_SA(st)) {
 			struct ike_sa *ike = pexpect_ike_sa(st);
-			connection_delete_ike_family(&ike, HERE);
+			terminate_ike_family(&ike, REASON_CRYPTO_TIMEOUT, HERE);
 		} else {
 			struct child_sa *child = pexpect_child_sa(st);
-			connection_delete_child(&child, HERE);
+			pstat_sa_failed(&child->sa, REASON_CRYPTO_TIMEOUT);
+			connection_teardown_child(&child, REASON_DELETED, HERE);
 		}
 		/* note: no md->st to clear */
 		break;
