@@ -51,12 +51,10 @@ void delete_state_event(struct state_event **evp, where_t where);
 struct state_event **state_event_slot(struct state *st, enum event_type type);
 extern void event_force(enum event_type type, struct state *st);
 
-extern void delete_event(struct state *st);
-extern void handle_next_timer_event(void);
-extern void init_timer(void);
+extern void delete_v1_event(struct state *st);
 
-void call_state_event_handler(struct logger *logger, struct state *st,
-			      enum event_type type);
+void whack_impair_call_state_event_handler(struct logger *logger, struct state *st,
+					   enum event_type type, bool detach_whack);
 
 extern void list_timers(struct show *s, const monotime_t now);
 extern char *revive_conn;
