@@ -26,15 +26,16 @@
 #include "refcnt.h"
 #include "ip_info.h"
 #include "show.h"
-
-#define F_VIRTUAL_NO		1	/* %no (subnet must be host/32) */
-#define F_VIRTUAL_PRIVATE	2	/* %priv (list held in private_net_{incl,excl} */
-#define F_VIRTUAL_ALL		4	/* %all [only for testing] */
-#define F_VIRTUAL_HOST		8	/* vhost (vnet has no representation) */
+#include "kernel.h"		/* for .overlap_supported */
 
 struct virtual_ip {
 	refcnt_t refcnt;
-	unsigned short flags;	/* union of F_VIRTUAL_* */
+	struct virtual_flags {
+		bool no;	/* %no (subnet must be host/32) */
+		bool private;	/* %priv (list held in private_net_{incl,excl} */
+		bool all;	/* %all [only for testing] */
+		bool host;	/* vhost (vnet has no representation) */
+	} virtual;	/* union of F_VIRTUAL_* */
 	unsigned short n_net;
 	ip_subnet net[0 /*n_net*/];	/* 0-length array is a GCC extension */
 };
@@ -217,11 +218,11 @@ diag_t create_virtual(const char *leftright, const char *string, struct virtual_
 {
 	passert(string != NULL && string[0] != '\0');
 
-	unsigned short flags = 0;
+	struct virtual_flags flags = {0};
 	const char *str = string;
 
 	if (eat(str, "vhost:")) {
-		flags |= F_VIRTUAL_HOST;
+		flags.host = true;
 	} else if (eat(str, "vnet:")) {
 		/* represented in flags by the absence of F_VIRTUAL_HOST */
 	} else {
@@ -248,11 +249,11 @@ diag_t create_virtual(const char *leftright, const char *string, struct virtual_
 		ip_subnet sub;	/* sink -- value never used */
 
 		if (eat(str, "%no")) {
-			flags |= F_VIRTUAL_NO;
+			flags.no = true;
 		} else if (eat(str, "%priv")) {
-			flags |= F_VIRTUAL_PRIVATE;
+			flags.private = true;
 		} else if (eat(str, "%all")) {
-			flags |= F_VIRTUAL_ALL;
+			flags.all = true;
 		} else {
 			/* don't allow ! form */
 			err_t e = read_subnet(shunk2(str, len), &sub, NULL);
@@ -286,7 +287,7 @@ diag_t create_virtual(const char *leftright, const char *string, struct virtual_
 						/*extra*/(n_net * sizeof(ip_subnet)),
 						HERE);
 
-	v->flags = flags;
+	v->virtual = flags;
 	v->n_net = n_net;
 	if (n_net > 0) {
 		passert(first_net != NULL);
@@ -326,13 +327,14 @@ diag_t create_virtual(const char *leftright, const char *string, struct virtual_
  * @param that end structure
  * @return bool True if we do
  */
-bool is_virtual_spd_end(const struct spd_end *end)
+bool is_virtual_spd_end(const struct spd_end *end,
+			struct verbose verbose)
 {
 	bool virt = (end->virt != NULL);
-	dbg("virt: %s() %s spd=%s config=%s",
-	    __func__, bool_str(virt),
-	    bool_str(end->virt != NULL),
-	    bool_str(end->config->child.virt != NULL));
+	vdbg("virt: %s() %s spd=%s config=%s",
+	     __func__, bool_str(virt),
+	     bool_str(end->virt != NULL),
+	     bool_str(end->config->child.virt != NULL));
 	pexpect((end->config->child.virt != NULL) >= (end->virt != NULL));
 	return virt;
 }
@@ -343,7 +345,8 @@ bool is_virtual_spd_end(const struct spd_end *end)
  * @param c Active Connection struct
  * @return bool True if we do
  */
-bool is_virtual_remote(const struct connection *c)
+bool is_virtual_remote(const struct connection *c,
+		       struct verbose verbose)
 {
 	bool virt = false;
 	FOR_EACH_ITEM(spd, &c->child.spds) {
@@ -352,7 +355,7 @@ bool is_virtual_remote(const struct connection *c)
 			break;
 		}
 	}
-	ldbg(c->logger, "virt: %s() %s local/remote spd %s/%s; config %s/%s",
+	vdbg("virt: %s() %s local/remote spd %s/%s; config %s/%s",
 	     __func__, bool_str(virt),
 	     bool_str(c->spd->local->virt != NULL),
 	     bool_str(c->spd->remote->virt != NULL),
@@ -363,14 +366,20 @@ bool is_virtual_remote(const struct connection *c)
 }
 
 /*
- * is_virtual_vhost - is the virt set to a host or a net?
+ * is_virtual_host - is the virt set to a host or a net?
  *
  * @param that end structure
  * @return bool True if we do
  */
-bool is_virtual_vhost(const struct spd_end *end)
+
+bool is_virtual_host(const struct virtual_ip *virt)
 {
-	return is_virtual_spd_end(end) && (end->virt->flags & F_VIRTUAL_HOST) != 0;
+	return (virt != NULL && virt->virtual.host);
+}
+
+bool is_virtual_net(const struct virtual_ip *virt)
+{
+	return (virt != NULL && !virt->virtual.host);
 }
 
 /*
@@ -400,11 +409,13 @@ static bool net_in_list(const ip_subnet peer_net, const ip_subnet *list,
  * @param peers_addr Peers IP Address
  * @return err_t NULL if allowed, diagnostic otherwise
  */
-err_t check_virtual_net_allowed(const struct connection *c,
-				const ip_subnet peer_net,
-				const ip_address peers_addr)
+
+static err_t check_virtual_net_allowed(const struct connection *c,
+				       const ip_subnet peer_net,
+				       const ip_address peers_addr,
+				       struct verbose verbose)
 {
-	ldbg(c->logger, "virt: %s() spd %s/%s; config %s/%s",
+	vdbg("virt: %s() spd %s/%s; config %s/%s",
 	     __func__,
 	     bool_str(c->spd->local->virt != NULL),
 	     bool_str(c->spd->remote->virt != NULL),
@@ -414,14 +425,14 @@ err_t check_virtual_net_allowed(const struct connection *c,
 	if (virt == NULL)
 		return NULL;
 
-	if (virt->flags & F_VIRTUAL_HOST && subnet_size(peer_net) != 1) {
+	if (virt->virtual.host && subnet_size(peer_net) != 1) {
 		return "only virtual host single IPs are allowed";
 	}
 
 	if (private_net_incl == NULL)
 		return NULL;
 
-	if (virt->flags & F_VIRTUAL_NO) {
+	if (virt->virtual.no) {
 		if (subnet_eq_address(peer_net, peers_addr)) {
 			return NULL;
 		}
@@ -431,7 +442,7 @@ err_t check_virtual_net_allowed(const struct connection *c,
 	/* last failure; ignored on subsequent success; ??? default is success */
 	err_t why = NULL;
 
-	if (virt->flags & F_VIRTUAL_PRIVATE) {
+	if (virt->virtual.private) {
 		if (!net_in_list(peer_net, private_net_incl,
 				private_net_incl_len)) {
 			why = "a private network virtual IP was required, but the proposed IP did not match our list (virtual-private=) since it is in use elsewhere";
@@ -450,11 +461,11 @@ err_t check_virtual_net_allowed(const struct connection *c,
 		why = "a specific network IP was required, but the proposed IP did not match our list (subnet=vhost:list)";
 	}
 
-	if (virt->flags & F_VIRTUAL_ALL) {
+	if (virt->virtual.all) {
 		/* %all must only be used for testing - log it */
 		llog(RC_LOG, c->logger,
 		     "WARNING: v%s:%%all must only be used for testing",
-		     (virt->flags & F_VIRTUAL_HOST) ? "host" : "net");
+		     (virt->virtual.host ? "host" : "net"));
 
 		return NULL;	/* success */
 	}
@@ -463,6 +474,125 @@ err_t check_virtual_net_allowed(const struct connection *c,
 	return why;
 }
 
+/*
+ * With virtual addressing, we must not allow someone to use an
+ * already used (by another id) addr/net.
+ */
+
+static err_t is_virtual_net_used(const ip_selector remote_client,
+				 const struct id *peer_id,
+				 struct verbose verbose)
+{
+	vdbg("%s()", __func__);
+	verbose.level++;
+
+	struct connection_filter cq = {
+		.ike_version = IKEv1,
+		.search = {
+			.order = NEW2OLD,
+			.verbose = verbose,
+			.where = HERE,
+		},
+	};
+	while (next_connection(&cq)) {
+		struct verbose verbose = cq.search.verbose;
+
+		struct connection *d = cq.c;
+		switch (d->local->kind) {
+		case CK_PERMANENT:
+		case CK_TEMPLATE:
+		case CK_INSTANCE:
+
+			if (is_template(d) &&
+			    d->remote->config->child.selectors.len > 0) {
+				/*
+				 * For instance when the template''s
+				 * peer's protoport=udp/%any but
+				 * peers' subnet is not set.  The
+				 * peer's .client is constructed from
+				 * %any:udp/%any.
+				 *
+				 * Since this has to be narrowed, any
+				 * comparison is pointless.
+				 */
+				connection_buf dcb;
+				enum_buf kb;
+				vdbg("skipping %s "PRI_CONNECTION" as remote's %ssubnet is wild (not set)",
+				     str_enum_short(&connection_kind_names, d->local->kind, &kb),
+				     pri_connection(d, &dcb),
+				     d->remote->config->leftright);
+				continue;
+			}
+
+			if (!selector_overlaps_selector(remote_client, d->spd->remote->client)) {
+				/*
+				 * For instance when REMOTE_CLIENT is IPv6
+				 * and remote .client is IPv4 (but can
+				 * be pretty much anything that
+				 * doesn't intersect).
+				 */
+				connection_buf dcb;
+				enum_buf kb;
+				vdbg("skipping %s "PRI_CONNECTION" as there is no overlap",
+				     str_enum_short(&connection_kind_names, d->local->kind, &kb),
+				     pri_connection(d, &dcb));
+				continue;
+			}
+
+			if (same_id(&d->remote->host.id, peer_id)) {
+				/*
+				 * Assumed to be a replace?
+				 */
+				connection_buf dcb;
+				enum_buf kb;
+				id_buf idb;
+				vdbg("skipping %s "PRI_CONNECTION" as it has the same id: %s",
+				     str_enum_short(&connection_kind_names, d->local->kind, &kb),
+				     pri_connection(d, &dcb),
+				     str_id(&d->remote->host.id, &idb));
+				continue;
+			}
+
+			connection_buf cbuf;
+			subnet_buf pcb, dcb;
+			llog(RC_LOG, verbose.logger,
+			     "peer Virtual IP %s overlapping %s from "PRI_CONNECTION" is not supported by the kernel interface %s",
+			     str_selector_range(&remote_client, &pcb),
+			     str_selector_range(&d->spd->remote->client, &dcb),
+			     pri_connection(d, &cbuf),
+			     kernel_ops->interface_name);
+
+			return "Virtual IP overlap";
+
+		default:
+			break;
+		}
+	}
+
+	return NULL;
+}
+
+err_t is_virtual_net_allowed(const struct connection *d,
+			     const ip_selector remote_client,
+			     struct verbose verbose)
+{
+	vdbg("%s()", __func__);
+	verbose.level++;
+
+	err_t oops = check_virtual_net_allowed(d, selector_subnet(remote_client),
+					       d->remote->host.addr, verbose);
+	if (oops != NULL) {
+		return oops;
+	}
+
+	oops = is_virtual_net_used(remote_client, &d->remote->host.id, verbose);
+	if (oops != NULL) {
+		return oops;
+	}
+
+	return NULL;
+}
+
 static void show_virtual_private_kind(struct show *s,
 				      const char *kind,
 				      const ip_subnet *private_net,
