@@ -18,10 +18,11 @@
 #include "err.h"
 #include "ip_address.h"
 #include "ip_subnet.h"
+#include "verbose.h"
 
 struct connection;
-struct spd_route;
-struct end;
+struct spd;
+struct spd_end;
 struct show;
 
 extern void show_virtual_private(struct show *s);
@@ -29,19 +30,25 @@ extern void show_virtual_private(struct show *s);
 extern void init_virtual_ip(const char *private_list, struct logger *logger);
 extern void free_virtual_ip(void);
 
-struct virtual_ip *create_virtual(const char *string, struct logger *logger);
+diag_t create_virtual(const char *leftright, const char *string, struct virtual_ip **);
 struct virtual_ip *virtual_ip_addref_where(struct virtual_ip *vip, where_t where);
 #define virtual_ip_addref(VIP) virtual_ip_addref_where(VIP, HERE)
 void virtual_ip_delref_where(struct virtual_ip **vip, where_t where);
 #define virtual_ip_delref(IP) virtual_ip_delref_where(IP, HERE)
 
-extern bool is_virtual_end(const struct end *that);
-extern bool is_virtual_connection(const struct connection *c);
-extern bool is_virtual_sr(const struct spd_route *sr);
-extern bool is_virtual_vhost(const struct end *that);
-extern err_t check_virtual_net_allowed(const struct connection *c,
-				       const ip_subnet peer_net,
-				       const ip_address peers_addr);
+bool is_virtual_spd_end(const struct spd_end *that, struct verbose verbose);
+bool is_virtual_remote(const struct connection *c, struct verbose verbose);
+
+bool is_virtual_host(const struct virtual_ip *that);
+bool is_virtual_net(const struct virtual_ip *that);
+
+/*
+ * With virtual addressing, we must not allow someone to use an
+ * already used (by another id) addr/net.
+ */
+
+err_t is_virtual_net_allowed(const struct connection *d,
+			     const ip_selector remote_client,
+			     struct verbose verbose);
 
 #endif /* VIRTUAL_IP_H */
-
