@@ -19,6 +19,7 @@
  * Copyright (C) 2016-2022 Andrew Cagney <cagney@gnu.org>
  * Copyright (C) 2017 Mayank Totale <mtotale@gmail.com>
  * Copyright (C) 20212-2022 Paul Wouters <paul.wouters@aiven.io>
+ * Copyright (C) 2020 Nupur Agrawal <nupur202000@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -32,7 +33,8 @@
  */
 
 #include "whack_connectionstatus.h"
-#include "whack_connection.h"
+
+#include "visit_connection.h"
 
 #include "ike_alg.h"
 
@@ -49,6 +51,8 @@
 #include "plutoalg.h"	/* for show_kernel_alg_connection() */
 #include "kernel.h"		/* for enum direction */
 #include "sparse_names.h"
+#include "monotime.h"
+#include "ikev2_ike_session_resume.h"	/* for show_session_resume() */
 
 /* Passed in to jam_end_client() */
 static const char END_SEPARATOR[] = "===";
@@ -60,7 +64,7 @@ static const char END_SEPARATOR[] = "===";
  *      [this]LOCAL_END ...END_REMOTE[that]
  *
  * where END_REMOTE is roughly formatted as the mirror image of
- * LOCAL_END.  LEFT_RIGHT is used to determine if the LHS or RHS
+ * LOCAL_END.  SIDE is used to determine if the LHS or RHS
  * string is being emitted (however, note that the LHS here is _not_
  * the configuration file's left*=).
  *
@@ -131,8 +135,37 @@ void jam_end_host(struct jambuf *buf,
 	}
 }
 
-void jam_end_client(struct jambuf *buf, const struct connection *c,
-		    const struct spd_end *this, enum left_right left_right,
+static struct connection_client connection_spd_client(const struct spd_end *spd)
+{
+	struct connection_client client = {
+		.client = spd->client,
+		.virt = spd->virt,
+		.host = spd->host,
+		.child = spd->child,
+		.sourceip = spd_end_sourceip(spd),
+		.is_addresspool = (spd->child->config->addresspools.len > 0),
+	};
+	return client;
+}
+
+static struct connection_client connection_config_client(const struct connection_end *this,
+							 const ip_selector *this_selector)
+{
+	struct connection_client client = {
+		.client = *this_selector,
+		.virt = this->child.config->virt,
+		.host = &this->host,
+		.child = &this->child,
+		.sourceip = config_end_sourceip(*this_selector, this->child.config),
+		.is_addresspool = (this->child.config->addresspools.len > 0),
+	};
+	return client;
+}
+
+void jam_end_client(struct jambuf *buf,
+		    const struct connection *c,
+		    const struct connection_client *this,
+		    enum end side,
 		    const char *separator)
 {
 	/* left: [CLIENT/PROTOCOL:PORT===] or right: [===CLIENT/PROTOCOL:PORT] */
@@ -150,7 +183,7 @@ void jam_end_client(struct jambuf *buf, const struct connection *c,
 			/* booring */
 			return;
 		}
-		if (this->host->config->pool_ranges.len > 0) {
+		if (this->child->config->addresspools.len > 0) {
 			/*
 			 * Suppress zero selectors that were probably derived
 			 * from the address pool.
@@ -159,44 +192,51 @@ void jam_end_client(struct jambuf *buf, const struct connection *c,
 		}
 	}
 
-	if (left_right == RIGHT_END && separator != NULL) {
+	if (side == RIGHT_END && separator != NULL) {
 		jam_string(buf, separator);
 	}
 
-	if (is_virtual_spd_end(this)) {
-		if (is_virtual_vhost(this))
-			jam_string(buf, "vhost:?");
-		else
-			jam_string(buf,  "vnet:?");
+	if (is_virtual_host(this->virt)) {
+		jam_string(buf, "vhost:?");
+	} else if (is_virtual_net(this->virt)) {
+		jam_string(buf,  "vnet:?");
 	} else {
+		if (this->is_addresspool) {
+			jam_string(buf, "{");
+		}
 		jam_selector(buf, &this->client);
 		if (selector_is_zero(this->client)) {
 			jam_string(buf, "?");
 		}
+		if (this->is_addresspool) {
+			jam_string(buf, "}");
+		}
 	}
 
-	if (left_right == LEFT_END && separator != NULL) {
+	if (side == LEFT_END && separator != NULL) {
 		jam_string(buf, separator);
 	}
 }
 
-static void jam_end_id(struct jambuf *buf, const struct spd_end *this)
+static void jam_end_id(struct jambuf *buf,
+		       const struct host_end *host,
+		       const struct child_end *child)
 {
 	/* id, if different from host */
 	bool open_paren = false;
-	if (!(this->host->id.kind == ID_NONE ||
-	      (id_is_ipaddr(&this->host->id) &&
-	       sameaddr(&this->host->id.ip_addr, &this->host->addr)))) {
+	if (!(host->id.kind == ID_NONE ||
+	      (id_is_ipaddr(&host->id) &&
+	       sameaddr(&host->id.ip_addr, &host->addr)))) {
 		open_paren = true;
 		jam_string(buf, "[");
-		jam_id_bytes(buf, &this->host->id, jam_sanitized_bytes);
+		jam_id_bytes(buf, &host->id, jam_sanitized_bytes);
 	}
 
-	if (this->host->config->modecfg.server ||
-	    this->host->config->modecfg.client ||
-	    this->host->config->xauth.server ||
-	    this->host->config->xauth.client ||
-	    this->host->config->sendcert != cert_defaultcertpolicy) {
+	if (host->config->modecfg.server ||
+	    host->config->modecfg.client ||
+	    host->config->xauth.server ||
+	    host->config->xauth.client ||
+	    host->config->sendcert != cert_defaultcertpolicy) {
 
 		if (open_paren) {
 			jam_string(buf, ",");
@@ -205,18 +245,18 @@ static void jam_end_id(struct jambuf *buf, const struct spd_end *this)
 			jam_string(buf, "[");
 		}
 
-		if (this->host->config->modecfg.server)
+		if (host->config->modecfg.server)
 			jam_string(buf, "MS");
-		if (this->host->config->modecfg.client)
+		if (host->config->modecfg.client)
 			jam_string(buf, "+MC");
-		if (this->child->config->has_client_address_translation)
+		if (child->config->has_client_address_translation)
 			jam_string(buf, "+CAT");
-		if (this->host->config->xauth.server)
+		if (host->config->xauth.server)
 			jam_string(buf, "+XS");
-		if (this->host->config->xauth.client)
+		if (host->config->xauth.client)
 			jam_string(buf, "+XC");
 
-		switch (this->host->config->sendcert) {
+		switch (host->config->sendcert) {
 		case CERT_NEVERSEND:
 			jam(buf, "+S-C");
 			break;
@@ -236,86 +276,109 @@ static void jam_end_id(struct jambuf *buf, const struct spd_end *this)
 	}
 }
 
-static void jam_end_nexthop(struct jambuf *buf, const struct spd_end *this,
-			    const struct spd_end *that, bool skip_next_hop,
-			    enum left_right left_right)
+static void jam_end_nexthop(struct jambuf *buf,
+			    const struct host_end *this,
+			    const struct host_end *that,
+			    bool skip_next_hop,
+			    enum end side)
 {
 	/* [---hop] */
 	if (!skip_next_hop &&
-	    address_is_specified(this->host->nexthop) &&
-	    !address_eq_address(this->host->nexthop, that->host->addr)) {
-		if (left_right == LEFT_END) {
+	    address_is_specified(this->nexthop) &&
+	    !address_eq_address(this->nexthop, that->addr)) {
+		if (side == LEFT_END) {
 			jam_string(buf, "---");
 		}
-		jam_address(buf, &this->host->nexthop);
-		if (left_right == RIGHT_END) {
+		jam_address(buf, &this->nexthop);
+		if (side == RIGHT_END) {
 			jam_string(buf, "---");
 		}
 	}
 }
 
-void jam_spd_end(struct jambuf *buf, const struct connection *c,
-		 const struct spd_end *this, const struct spd_end *that,
-		 enum left_right left_right, bool skip_next_hop)
+static void jam_client_end(struct jambuf *buf, const struct connection *c,
+			   const struct connection_client *this,
+			   const struct connection_client *that,
+			   enum end side, bool skip_next_hop)
 {
-	switch (left_right) {
+	switch (side) {
 	case LEFT_END:
 		/* CLIENT/PROTOCOL:PORT=== */
-		jam_end_client(buf, c, this, left_right, END_SEPARATOR);
+		jam_end_client(buf, c, this, side, END_SEPARATOR);
 		/* HOST */
 		jam_end_host(buf, c, this->host);
 		/* [ID+OPTS] */
-		jam_end_id(buf, this);
+		jam_end_id(buf, this->host, this->child);
 		/* ---NEXTHOP */
-		jam_end_nexthop(buf, this, that, skip_next_hop, left_right);
+		jam_end_nexthop(buf, this->host, that->host, skip_next_hop, side);
 		break;
 	case RIGHT_END:
 		/* HOPNEXT--- */
-		jam_end_nexthop(buf, this, that, skip_next_hop, left_right);
+		jam_end_nexthop(buf, this->host, that->host, skip_next_hop, side);
 		/* HOST */
 		jam_end_host(buf, c, this->host);
 		/* [ID+OPTS] */
-		jam_end_id(buf, this);
+		jam_end_id(buf, this->host, this->child);
 		/* ===CLIENT/PROTOCOL:PORT */
-		jam_end_client(buf, c, this, left_right, END_SEPARATOR);
+		jam_end_client(buf, c, this, side, END_SEPARATOR);
 		break;
 	}
 }
 
+static void jam_client_ends(struct jambuf *buf, const struct connection *c,
+			    const struct connection_client *this,
+			    const char *sep,
+			    const struct connection_client *that)
+{
+	jam_client_end(buf, c, this, that, LEFT_END, false);
+	jam_string(buf, sep);
+	jam_client_end(buf, c, that, this, RIGHT_END, oriented(c));
+}
+
+void jam_spd_ends(struct jambuf *buf, const struct connection *c,
+		  const struct spd_end *this_spd,
+		  const char *sep,
+		  const struct spd_end *that_spd)
+{
+	struct connection_client this = connection_spd_client(this_spd);
+	struct connection_client that = connection_spd_client(that_spd);
+	jam_client_ends(buf, c, &this, sep, &that);
+}
+
+static void jam_routing(struct jambuf *buf, const struct connection *c)
+{
+	if (!oriented(c)) {
+		jam_string(buf, "unoriented");
+ 		pexpect(c->routing.state == RT_UNROUTED);
+	} else {
+		jam_enum_human(buf, &routing_names, c->routing.state);
+	}
+}
+
 /*
  * format topology of a connection.
  * Two symmetric ends separated by ...
  */
 
-static void jam_spd_topology(struct jambuf *buf, const struct spd *spd)
-{
-	jam_spd_end(buf, spd->connection, spd->local, spd->remote,
-		    LEFT_END, false);
-	jam_string(buf, "...");
-	jam_spd_end(buf, spd->connection, spd->remote, spd->local,
-		    RIGHT_END, oriented(spd->connection));
-}
-
-static void show_one_spd(struct show *s,
-			 const struct connection *c,
-			 const struct spd *spd)
+static void show_connection_client(struct show *s,
+				   const struct connection *c,
+				   const struct connection_client *this,
+				   const struct connection_client *that)
 {
 	SHOW_JAMBUF(s, buf) {
 		jam_connection_short(buf, c);
 		jam_string(buf, ":");
+
 		/* one SPD */
 		jam_string(buf, " ");
-		jam_spd_topology(buf, spd);
+		jam_client_ends(buf, c, this, "...", that);
 		jam_string(buf, ";");
+
 		/* routing/orienting */
 		jam_string(buf, " ");
-		if (!oriented(c)) {
-			jam_string(buf, "unoriented");
-			PEXPECT(show_logger(s), c->routing.state == RT_UNROUTED);
-		} else {
-			jam_enum_human(buf, &routing_names, c->routing.state);
-		}
+		jam_routing(buf, c);
 		jam_string(buf, ";");
+
 #define OPT_HOST(H)					\
 		if (address_is_specified(H)) {		\
 			jam_address(buf, &(H));		\
@@ -323,19 +386,15 @@ static void show_one_spd(struct show *s,
 			jam_string(buf, "unset");	\
 		}
 		/* my_ip */
-		ip_address my_sourceip = spd_end_sourceip(spd->local);
 		jam_string(buf, " my_ip=");
-		OPT_HOST(my_sourceip);
+		OPT_HOST(this->sourceip);
 		jam_string(buf, ";");
 		/* their_ip */
-		ip_address their_sourceip = spd_end_sourceip(spd->remote);
 		jam_string(buf, " their_ip=");
-		OPT_HOST(their_sourceip);
+		OPT_HOST(that->sourceip);
 		jam_string(buf, ";");
 #undef OPT_HOST
 	}
-
-
 }
 
 static void jam_connection_owners(struct jambuf *buf,
@@ -387,12 +446,38 @@ static void jam_connection_owners(struct jambuf *buf,
 	}
 }
 
+void show_connection_clients(struct show *s, const struct connection *c,
+			     void (*show_client)(struct show *s,
+						 const struct connection *c,
+						 const struct connection_client *this,
+						 const struct connection_client *that))
+{
+	/* Show topology. */
+	if (oriented(c)) {
+		FOR_EACH_ITEM(spd, &c->child.spds) {
+			struct connection_client this =
+				connection_spd_client(spd->local);
+			struct connection_client that =
+				connection_spd_client(spd->remote);
+			show_client(s, c, &this, &that);
+		}
+	} else {
+		FOR_EACH_ITEM(local, &c->local->child.selectors.proposed) {
+			struct connection_client this =
+				connection_config_client(c->local, local);
+			FOR_EACH_ITEM(remote, &c->remote->child.selectors.proposed) {
+				struct connection_client that =
+					connection_config_client(c->remote, remote);
+				show_client(s, c, &this, &that);
+			}
+		}
+	}
+}
+
 static void show_connection_status(struct show *s, const struct connection *c)
 {
 	/* Show topology. */
-	FOR_EACH_ITEM(spd, &c->child.spds) {
-		show_one_spd(s, c, spd);
-	}
+	show_connection_clients(s, c, show_connection_client);
 
 	SHOW_JAMBUF(s, buf) {
 		jam_connection_short(buf, c);
@@ -650,11 +735,30 @@ static void show_connection_status(struct show *s, const struct connection *c)
 		jam(buf, " rekey_fuzz: %lu%%;", c->config->sa_rekey_fuzz);
 	}
 
+	SHOW_JAMBUF(s, buf) {
+		jam_connection_short(buf, c);
+		jam_string(buf, ":  ");
+		jam(buf, " iptfs: %s;", bool_str(c->config->child_sa.iptfs.enabled));
+		jam(buf, " fragmentation: %s;", bool_str(c->config->child_sa.iptfs.fragmentation));
+		jam(buf, " packet-size: %ju;", c->config->child_sa.iptfs.packet_size);
+		jam(buf, " max-queue-size: %ju;", c->config->child_sa.iptfs.max_queue_size);
+		/* */
+		jam(buf, " drop-time: ");
+		jam_deltatime(buf, c->config->child_sa.iptfs.drop_time);
+		jam_string(buf, ";");
+		/* */
+		jam_string(buf, " init-delay: ");
+		jam_deltatime(buf, c->config->child_sa.iptfs.init_delay);
+		jam_string(buf, ";");
+		/* */
+		jam(buf, " reorder-window: %ju;", c->config->child_sa.iptfs.reorder_window);
+	}
+
 	SHOW_JAMBUF(s, buf) {
 		jam_connection_short(buf, c);
 		jam_string(buf, ":  ");
 		jam(buf, " retransmit-interval: %jdms;",
-		    deltamillisecs(c->config->retransmit_interval));
+		    milliseconds_from_deltatime(c->config->retransmit_interval));
 		jam(buf, " retransmit-timeout: %jds;",
 		    deltasecs(c->config->retransmit_timeout));
 		/* tcp? */
@@ -709,11 +813,12 @@ static void show_connection_status(struct show *s, const struct connection *c)
 	}
 
 	SHOW_JAMBUF(s, buf) {
-		connection_priority_buf prio;
 		jam_connection_short(buf, c);
 		jam_string(buf, ":  ");
 		/* priority */
-		jam(buf, " conn_prio: %s;", str_connection_priority(c, &prio));
+		jam_string(buf, " conn_prio: ");
+		jam_connection_priority(buf, c);
+		jam_string(buf, ";");
 		/* .interface: [id_rname@][id_vname] */
 		jam_string(buf, " interface: ");
 		if (c->config->ipsec_interface.enabled) {
@@ -894,13 +999,23 @@ static void show_connection_status(struct show *s, const struct connection *c)
 		jam_connection_owners(buf, c, CHILD_SA_OWNER_FLOOR, CHILD_SA_OWNER_ROOF);
 	}
 
+	if (c->config->session_resumption) {
+		SHOW_JAMBUF(s, buf) {
+			jam_connection_short(buf, c);
+			jam_string(buf, ":  ");
+			/* ticket */
+			jam_string(buf, " ");
+			jam_resume_ticket(buf, c->session);
+		}
+	}
+
 	SHOW_JAMBUF(s, buf) {
 		jam_connection_short(buf, c);
 		jam_string(buf, ":  ");
 		/* serial */
 		jam(buf, " conn serial: "PRI_CO,
 		    pri_co(c->serialno));
-		if (c->clonedfrom != UNSET_CO_SERIAL) {
+		if (c->clonedfrom != COS_NOBODY) {
 			jam(buf, ", instantiated from: "PRI_CO,
 			    pri_connection_co(c->clonedfrom));
 		}
