@@ -89,7 +89,6 @@ bool crl_strict = false;
 bool ocsp_strict = false;
 bool ocsp_enable = false;
 bool ocsp_post = false;
-char *curl_iface = NULL;
 
 SECItem same_shunk_as_dercert_secitem(shunk_t shunk)
 {
@@ -104,9 +103,11 @@ static const char *dntoasi(dn_buf *dst, SECItem si)
 /*
  * does our CA match one of the requested CAs?
  */
-bool match_requested_ca(const generalName_t *requested_ca, chunk_t our_ca,
-			int *our_pathlen)
+bool match_v1_requested_ca(const struct ike_sa *ike, chunk_t our_ca,
+			   int *our_pathlen, struct verbose verbose)
 {
+	const generalName_t *requested_ca = ike->sa.st_v1_requested_ca;
+
 	/* if no ca is requested than any ca will match */
 	if (requested_ca == NULL) {
 		*our_pathlen = 0;
@@ -118,7 +119,8 @@ bool match_requested_ca(const generalName_t *requested_ca, chunk_t our_ca,
 	while (requested_ca != NULL) {
 		int pathlen;
 
-		if (trusted_ca(ASN1(our_ca), ASN1(requested_ca->name), &pathlen) &&
+		if (trusted_ca(ASN1(our_ca), ASN1(requested_ca->name),
+			       &pathlen, verbose) &&
 		    pathlen < *our_pathlen)
 			*our_pathlen = pathlen;
 		requested_ca = requested_ca->next;
@@ -183,23 +185,24 @@ static void same_nss_gn_as_pluto_gn(CERTGeneralName *nss_gn,
 
 /*
  * Checks if CA a is trusted by CA b
- * This very well could end up being condensed into
- * an NSS call or two. TBD.
+ *
+ * This very well could end up being condensed into an NSS call or
+ * two. TBD.
  */
-bool trusted_ca(asn1_t a, asn1_t b, int *pathlen)
+
+bool trusted_ca(asn1_t a, asn1_t b, int *pathlen, struct verbose verbose)
 {
-	if (DBGP(DBG_BASE)) {
-		if (a.ptr != NULL) {
-			dn_buf abuf;
-	    		DBG_log("%s: trustee A = '%s'", __func__,
-				str_dn(a, &abuf));
-		}
-		if (b.ptr != NULL) {
-			dn_buf bbuf;
-	    		DBG_log("%s: trustor B = '%s'", __func__,
-				str_dn(b, &bbuf));
-		}
+	if (verbose.rc_flags != LEMPTY) {
+		dn_buf abuf;
+		llog(verbose.rc_flags, verbose.logger,
+		     PRI_VERBOSE"trustee A = '%s'",
+		     pri_verbose, str_dn(a, &abuf));
+		dn_buf bbuf;
+		llog(verbose.rc_flags, verbose.logger,
+		     PRI_VERBOSE"trustor B = '%s'",
+		     pri_verbose, str_dn(b, &bbuf));
 	}
+	verbose.level++;
 
 	/* no CA b specified => any CA a is accepted */
 	if (b.ptr == NULL) {
@@ -249,7 +252,7 @@ bool trusted_ca(asn1_t a, asn1_t b, int *pathlen)
 
 		if (match) {
 			/* we have a match: exit the loop */
-			dbg("%s: A is a subordinate of B", __func__);
+			vdbg("%s: A is a subordinate of B", __func__);
 			break;
 		}
 
@@ -268,20 +271,6 @@ bool trusted_ca(asn1_t a, asn1_t b, int *pathlen)
 	return match;
 }
 
-/*
- * choose either subject DN or a subjectAltName as connection end ID
- */
-void select_nss_cert_id(CERTCertificate *cert, struct id *end_id)
-{
-	if (end_id->kind == ID_FROMCERT) {
-		dbg("setting ID to ID_DER_ASN1_DN: \'%s\'", cert->subjectName);
-		chunk_t name = clone_secitem_as_chunk(cert->derSubject, "cert id");
-		end_id->name = ASN1(name);
-		end_id->scratch = name.ptr;
-		end_id->kind = ID_DER_ASN1_DN;
-	}
-}
-
 generalName_t *collect_rw_ca_candidates(ip_address local_address,
 					enum ike_version ike_version)
 {
@@ -295,7 +284,7 @@ generalName_t *collect_rw_ca_candidates(ip_address local_address,
 		.ike_version = ike_version,
 		.search = {
 			.order = NEW2OLD,
-			.logger = &global_logger,
+			.verbose.logger = &global_logger,
 			.where = HERE,
 		},
 	};
@@ -434,17 +423,6 @@ static void get_pluto_gn_from_nss_cert(CERTCertificate *cert, generalName_t **gn
 	*gn_out = pgn_list;
 }
 
-static diag_t create_cert_subjectdn_pubkey(CERTCertificate *cert,
-					   struct pubkey **pk,
-					   struct logger *logger)
-{
-	struct id id = {
-		.kind = ID_DER_ASN1_DN,
-		.name = same_secitem_as_shunk(cert->derSubject),
-	};
-	return create_pubkey_from_cert(&id, cert, pk, logger);
-}
-
 static void add_cert_san_pubkeys(struct pubkey_list **pubkey_db,
 				 CERTCertificate *cert,
 				 struct logger *logger)
@@ -487,25 +465,48 @@ bool add_pubkey_from_nss_cert(struct pubkey_list **pubkey_db,
 			      const struct id *keyid, CERTCertificate *cert,
 			      struct logger *logger)
 {
+	/*
+	 * Create a pubkey with ID set to the subject and add it.
+	 *
+	 * Note: the SUBJECT ID sharing NAME with .derSubject is ok.
+	 * create_pubkey_from_cert() clones SUBJECT removing the
+	 * shared link.
+	 */
+
+	ldbg(logger, "create and add/replace pubkey using cert subject name");
+	struct id subject = {
+		.kind = ID_DER_ASN1_DN,
+		.name = same_secitem_as_shunk(cert->derSubject),
+	};
 	struct pubkey *pk = NULL;
-	diag_t d = create_cert_subjectdn_pubkey(cert, &pk, logger);
+	diag_t d = create_pubkey_from_cert(&subject, cert, &pk, logger);
 	if (d != NULL) {
 		llog(RC_LOG, logger, "%s", str_diag(d));
 		pfree_diag(&d);
-		dbg("failed to create subjectdn_pubkey from cert");
+		ldbg(logger, "failed to create subjectdn_pubkey from cert");
 		return false;
 	}
 
-	ldbg(logger, "adding cert using subject name");
 	replace_public_key(pubkey_db, &pk);
-	passert(pk == NULL); /*stolen*/
+	PASSERT(logger, pk == NULL); /*stolen*/
 
-	ldbg(logger, "adding cert using general names");
+	/*
+	 * Add further pubkeys using the cert's subject-alt-name aka
+	 * SAN or general name.
+	 */
+
+	ldbg(logger, "adding/replacing pubkey using cert's subject-alt-names");
 	add_cert_san_pubkeys(pubkey_db, cert, logger);
 
-	if (keyid != NULL && keyid->kind != ID_DER_ASN1_DN &&
-			     keyid->kind != ID_NONE &&
-			     keyid->kind != ID_FROMCERT) {
+	/*
+	 * Finally, when the keyid isn't the cert's subject name (or
+	 * could be), add a pubkey with that ID name.
+	 */
+
+	if (keyid != NULL &&
+	    keyid->kind != ID_DER_ASN1_DN &&
+	    keyid->kind != ID_NONE &&
+	    keyid->kind != ID_FROMCERT) {
 		id_buf idb;
 		ldbg(logger, "adding cert using keyid %s", str_id(keyid, &idb));
 		struct pubkey *pk2 = NULL;
@@ -684,11 +685,11 @@ diag_t match_peer_id_cert(const struct certs *peer_certs,
 				end_cert->subjectName);
 		}
 
+		struct verbose verbose = { .logger = &global_logger, };
 		int wildcards;
-		bool m = match_dn_any_order_wild("",
-						 end_cert_der_subject,
+		bool m = match_dn_any_order_wild(end_cert_der_subject,
 						 peer_id->name,
-						 &wildcards);
+						 &wildcards, verbose);
 		if (!m) {
 			id_buf idb;
 			return diag("peer ID_DER_ASN1_DN '%s' does not match expected '%s'",
@@ -724,12 +725,13 @@ diag_t match_peer_id_cert(const struct certs *peer_certs,
 /*
  * Look for the existence of a non-expiring preloaded public key.
  */
-bool remote_has_preloaded_pubkey(const struct state *st)
+bool remote_has_preloaded_pubkey(const struct ike_sa *ike)
 {
-	const struct connection *c = st->st_connection;
+	const struct connection *c = ike->sa.st_connection;
 
-	/* do not consider rw connections since
-	 * the peer's identity must be known
+	/*
+	 * Do not consider rw connections since the peer's identity
+	 * must be known
 	 */
 	if (is_permanent(c)) {
 		/* look for a matching RSA public key */
