@@ -134,10 +134,9 @@ static void UpdateNSS_RNG(int seedbits, struct logger *logger)
 int main(int argc, char *argv[])
 {
 	log_to_stderr = false;
-	struct logger *logger = tool_init_log("ipsec rsasigkey");
+	struct logger *logger = tool_logger(argc, argv);
 
 	int opt;
-	int nbits = 0;
 	int seedbits = DEFAULT_SEED_BITS;
 
 	while ((opt = getopt_long(argc, argv, "", opts, NULL)) != EOF)
@@ -190,10 +189,7 @@ int main(int argc, char *argv[])
 	 */
 	const struct lsw_conf_options *oco = lsw_init_options();
 
-	diag_t d = lsw_nss_setup(oco->nssdir, 0, logger);
-	if (d != NULL) {
-		fatal_diag(1, logger, &d, "%s", "");
-	}
+	init_nss(oco->nssdir, (struct nss_flags){0}, logger);
 
 	/*
 	 * RSA-PSS requires keysize to be a multiple of 8 bits
@@ -201,39 +197,41 @@ int main(int argc, char *argv[])
 	 *
 	 * We require a multiple of 16.  (??? why?)
 	 */
+	uintmax_t nbits = 0;
 	if (argv[optind] == NULL) {
 		/*
 		 * Pick a default keysize in [3072, 3072+512+256);
 		 * multiple of 16; don't roll the top digit.
 		 */
-		nbits = 3072 + get_rnd_uintmax((512 + 256) / 16) * 16;
+		nbits = 3072 + get_rnd_uintmax() % ((512 + 256) / 16) * 16;
 	} else {
-		unsigned long u;
-		err_t ugh = ttoulb(argv[optind], 0, 10, INT_MAX, &u);
-
+		err_t ugh = shunk_to_uintmax(shunk1(argv[optind]), NULL/*all*/, /*base*/10, &nbits);
 		if (ugh != NULL) {
 			fprintf(stderr,
-				"%s: keysize specification is malformed: %s\n",
-				progname, ugh);
+				"%s: keysize specification '%s' is malformed: %s\n",
+				progname, argv[optind], ugh);
 			exit(1);
 		}
-		nbits = u;
 	}
 
 	if (nbits < MIN_KEYBIT ) {
 		fprintf(stderr,
-			"%s: requested RSA key size (%d) is too small - use %d or more\n",
+			"%s: requested RSA key size (%ju) is too small - use %d or more\n",
 			progname, nbits, MIN_KEYBIT);
 		exit(1);
-	} else if (nbits > MAXBITS) {
+	}
+
+	if (nbits > MAXBITS) {
 		fprintf(stderr,
-			"%s: requested RSA key size (%d) is too large - (max %d)\n",
+			"%s: requested RSA key size (%ju) is too large - (max %d)\n",
 			progname, nbits, MAXBITS);
 		exit(1);
-	} else if (nbits % (BITS_PER_BYTE * 2) != 0) {
+	}
+
+	if (nbits % (BITS_IN_BYTE * 2) != 0) {
 		fprintf(stderr,
-			"%s: requested RSA key size (%d) is not a multiple of %d\n",
-			progname, nbits, (int)BITS_PER_BYTE * 2);
+			"%s: requested RSA key size (%ju) is not a multiple of %d\n",
+			progname, nbits, (int)BITS_IN_BYTE * 2);
 		exit(1);
 	}
 
@@ -255,7 +253,7 @@ void rsasigkey(int nbits, int seedbits, struct logger *logger)
 	PK11SlotInfo *slot = lsw_nss_get_authenticated_slot(logger);
 	if (slot == NULL) {
 		/* already logged */
-		lsw_nss_shutdown();
+		shutdown_nss();
 		exit(1);
 	}
 
@@ -299,7 +297,7 @@ void rsasigkey(int nbits, int seedbits, struct logger *logger)
 	if (pubkey != NULL)
 		SECKEY_DestroyPublicKey(pubkey);
 
-	lsw_nss_shutdown();
+	shutdown_nss();
 }
 
 /*
@@ -321,7 +319,7 @@ void lsw_random(size_t nbytes, unsigned char *buf, struct logger *logger)
 
 	ndone = 0;
 	llog(RC_LOG, logger, "getting %d random seed bytes for NSS from %s...\n",
-		    (int) nbytes * BITS_PER_BYTE, device);
+		    (int) nbytes * BITS_IN_BYTE, device);
 	while (ndone < nbytes) {
 		got = read(dev, buf + ndone, nbytes - ndone);
 		if (got < 0) {
