@@ -22,10 +22,42 @@
 #include "addr_lookup.h"
 #include "ip_address.h"
 #include "ip_info.h"
+#include "optarg.h"
+
+enum opt {
+	OPT_EOF = -1,
+	OPT_FLAG = 0,
+	OPT_MISSING = ':',
+	OPT_INVALID = '?',
+
+	OPT_HELP = 'h',
+	OPT_VERBOSE = 'v',
+	OPT_DEBUG = 'd',
+	OPT_IPv4 = '4',
+	OPT_IPv6 = '6',
+};
+
+static struct optarg_family family;
+
+static int show_source = false;
+static int show_gateway = false;
+static int show_destination = false;
+
+const struct option optarg_options[] = {
+	{ "source", no_argument, &show_source, true, },
+	{ "gateway", no_argument, &show_gateway, true, },
+	{ "destination", no_argument, &show_destination, true, },
+	{ "debug", no_argument, NULL, OPT_DEBUG, },
+	{ "ipv4", no_argument, NULL, OPT_IPv4, },
+	{ "ipv6", no_argument, NULL, OPT_IPv6, },
+	{ "verbose", no_argument, NULL, OPT_VERBOSE, },
+	{0},
+};
 
 int main(int argc, char **argv)
 {
 	struct logger *logger = tool_logger(argc, argv);
+	optarg_init(logger);
 
 	if (argc == 1) {
 		llog(WHACK_STREAM|NO_PREFIX, logger, "Usage:");
@@ -36,63 +68,35 @@ int main(int argc, char **argv)
 		exit(1);
 	}
 
-	int show_source = false;
-	int show_gateway = false;
-	int show_destination = false;
-
-	const struct option options[] = {
-		{ "source", no_argument, &show_source, true, },
-		{ "gateway", no_argument, &show_gateway, true, },
-		{ "destination", no_argument, &show_destination, true, },
-		{ "debug", no_argument, NULL, 'd', },
-		{ "ipv4", no_argument, NULL, '4', },
-		{ "ipv6", no_argument, NULL, '6', },
-#if 0
-		{ "verbose", no_argument, NULL, 'v', },
-#endif
-		{0},
-	};
-
-	opterr = 0; /* handle options locally */
-
-	const struct ip_info *afi = NULL;
-
 	while (true) {
-		/*
-		 * Need to save current optind before call (after it
-		 * points to next entry); optind can be 0 at start.
-		 */
-		int current_optind = optind > 0 ? optind : 1;
-		int option_index;
-		int c = getopt_long(argc, argv, "vd46", options, &option_index);
+		enum opt c = getopt_long(argc, argv, "vd46", optarg_options, &optarg_index);
 		if (c == -1) {
 			break;
 		}
 		switch (c) {
-		case 'd':
-			cur_debugging = DBG_ALL;
+		case OPT_DEBUG:
+			optarg_debug(true);
 			break;
-		case '4':
-			afi = &ipv4_info;
+		case OPT_IPv4:
+			optarg_family(&family, &ipv4_info);
 			break;
-		case '6':
-			afi = &ipv6_info;
+		case OPT_IPv6:
+			optarg_family(&family, &ipv4_info);
 			break;
-#if 0
-		case 'v':
-			verbose = true;
+		case OPT_VERBOSE:
+			optarg_verbose(LEMPTY);
 			break;
-#endif
-		case 0:
+		case OPT_EOF:
 			break;
-		case '?':
-			llog(ERROR_STREAM, logger, "invalid option: %s", argv[current_optind]);
+		case OPT_FLAG:
+			/* flag updated */
+			break;
+		case OPT_INVALID:
+			llog(ERROR_STREAM, logger, "invalid option: %s", argv[optarg_index]);
 			exit(1);
-#if 0
-		case ':'
-			llog(ERROR_STREAM, logger, "missing parameter: %s", argv[current_optind]);
+		case OPT_MISSING:
+			llog(ERROR_STREAM, logger, "missing parameter: %s", argv[optarg_index]);
 			exit(1);
-#endif
 		default:
 			bad_case(c);
 		}
@@ -113,7 +117,7 @@ int main(int argc, char **argv)
 	}
 
 	ip_address dst;
-	err_t e = ttoaddress_dns(shunk1(argv[optind]), afi, &dst);
+	err_t e = ttoaddress_dns(shunk1(argv[optind]), family.type, &dst);
 	if (e != NULL) {
 		llog(WHACK_STREAM, logger, "%s: %s", argv[1], e);
 		exit(1);
