@@ -1,5 +1,4 @@
-/*
- * command interface to Pluto
+/* command interface to Pluto
  *
  * Copyright (C) 1997 Angelos D. Keromytis.
  * Copyright (C) 1998-2003  D. Hugh Redelmeier.
@@ -19,6 +18,7 @@
  * Copyright (C) 2019 Tuomo Soini <tis@foobar.fi>
  * Copyright (C) 2020 Yulia Kuzovkova <ukuzovkova@gmail.com>
  * Copyright (C) 20212-2022 Paul Wouters <paul.wouters@aiven.io>
+ * Copyright (C) 2020 Nupur Agrawal <nupur202000@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -41,12 +41,11 @@
 #include <sys/un.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
-#include <getopt.h>
 #include <assert.h>
 #include <limits.h>	/* for INT_MAX */
 
 #include "lsw_socket.h"
-
+#include "optarg.h"
 #include "ttodata.h"
 #include "lswversion.h"
 #include "lswtool.h"
@@ -113,12 +112,20 @@ static void help(void)
 		"	[--mtu <mtu>] \\\n"
 		"	[--priority <prio>] [--reqid <reqid>] \\\n"
 		"	[--tfc <size>] [--send-no-esp-tfc] \\\n"
+		"	[--iptfs[={yes,no}] \\\n"
+		"         [--iptfs-fragmentation[={yes,no}]] \\\n"
+		"         [--iptfs-packet-size <size>] \\\n"
+		"	  [--iptfs-max-queue-size <size>] \\\n"
+		"         [--iptfs-init-delay <s>] \\\n"
+		"         [--iptfs-drop-time <s> ] \\\n"
+		"	  [--iptfs-reorder-window <window>] \\\n"
 		"	[--ikev1 | --ikev2] \\\n"
 		"	[--narrowing {yes,no}] \\\n"
 		"	[--fragmentation {yes,no,force}] [--no-ikepad]  \\\n"
 		"	[--ikefrag-allow | --ikefrag-force] \\\n"
 		"	[--esn ] [--no-esn] [--decap-dscp] [--encap-dscp] [--nopmtudisc] [--mobike] \\\n"
 		"	[--tcp <no|yes|fallback>] --tcp-remote-port <port>\\\n"
+		"	[--session-resumption[={yes,no}]] \\\n"
 #ifdef HAVE_NM
 		"	[--nm-configured] \\\n"
 #endif
@@ -132,11 +139,14 @@ static void help(void)
 		"	[--ikev1-natt <both|rfc|drafts>] [--no-nat_keepalive] \\\n"
 		"	[--dpddelay <seconds> --dpdtimeout <seconds>] \\\n"
 		"	[--xauthserver | --xauthclient] \\\n"
-		"	[--modecfgserver | --modecfgclient] [--modecfgpull] \\\n"
 		"	[--addresspool <network range>] \\\n"
+		"	[--modecfgserver[={yes,no}] | --modecfgclient[={yes,no}]] [--modecfgpull[={yes,no}]] \\\n"
 		"	[--modecfgdns <ip-address, ip-address>  \\\n"
 		"	[--modecfgdomains <dns-domain, dns-domain, ..>] \\\n"
 		"	[--modecfgbanner <login banner>] \\\n"
+#ifdef USE_CAT
+		"	[--cat[={yes,no}]] \\\n"
+#endif
 		"	[--metric <metric>] \\\n"
 #ifdef USE_NFLOG
 		"	[--nflog-group <groupnum>] \\\n"
@@ -212,6 +222,8 @@ static void help(void)
 		"\n"
 		"refresh dns: whack --ddns\n"
 		"\n"
+		"suspend: whack --suspend --name <connection_name>\n"
+		"\n"
 #ifdef USE_SECCOMP
 		"testing: whack --seccomp-crashtest (CAREFUL!)\n"
 		"\n"
@@ -350,6 +362,8 @@ enum option_enums {
 	OPT_ROUTE,
 	OPT_UNROUTE,
 
+	OPT_SUSPEND,
+
 	OPT_INITIATE,
 	OPT_DOWN,
 	OPT_DELETE,
@@ -460,6 +474,7 @@ enum option_enums {
 	END_VTIIP,
 	END_AUTHBY,
 	END_AUTHEAP,
+	END_CAT,
 	END_UPDOWN,
 
 #define LAST_END_OPT		END_UPDOWN	/* last end description*/
@@ -502,6 +517,15 @@ enum option_enums {
 
 	CD_DONT_REKEY,
 	CD_REAUTH,
+
+	CD_IPTFS,
+	CD_IPTFS_FRAGMENTATION,
+	CD_IPTFS_PACKET_SIZE,
+	CD_IPTFS_MAX_QUEUE_SIZE,
+	CD_IPTFS_INIT_DELAY,
+	CD_IPTFS_REORDER_WINDOW,
+	CD_IPTFS_DROP_TIME,
+
 	CD_RETRANSMIT_TIMEOUT,
 	CD_RETRANSMIT_INTERVAL,
 	CD_IKE_LIFETIME,
@@ -525,6 +549,7 @@ enum option_enums {
 	CD_CISCO_UNITY,
 	CD_FAKE_STRONGSWAN,
 	CD_MOBIKE,
+	CD_SESSION_RESUMPTION,
 	CD_IKE,
 	CD_IKE_TCP,
 	CD_IKE_TCP_REMOTE_PORT,
@@ -640,9 +665,7 @@ enum option_enums {
 #define OPTION_ENUMS_ROOF	(OPTION_ENUMS_LAST+1)
 };
 
-int long_index;
-
-static const struct option long_opts[] = {
+const struct option optarg_options[] = {
 
 	/* name, has_arg, flag, val */
 
@@ -736,6 +759,9 @@ static const struct option long_opts[] = {
 	{ "down-ike", no_argument, NULL, OPT_DOWN_IKE },
 	{ "down-child", no_argument, NULL, OPT_DOWN_CHILD },
 
+	{ "suspend", no_argument, NULL, OPT_SUSPEND, },
+	{ "session-resumption", optional_argument, NULL, CD_SESSION_RESUMPTION, },
+
 	/* list options */
 
 	{ "utc", no_argument, NULL, LST_UTC },
@@ -814,6 +840,15 @@ static const struct option long_opts[] = {
 	{ "reauth", no_argument, NULL, CD_REAUTH, },
 	{ "encaps", required_argument, NULL, CD_ENCAPSULATION },
 	{ "encapsulation", optional_argument, NULL, CD_ENCAPSULATION },
+
+	{ "iptfs", optional_argument, NULL, CD_IPTFS, },
+	{ "iptfs-fragmentation", optional_argument, NULL, CD_IPTFS_FRAGMENTATION, },
+	{ "iptfs-packet-size", required_argument, NULL, CD_IPTFS_PACKET_SIZE },
+	{ "iptfs-max-queue-size", required_argument, NULL, CD_IPTFS_MAX_QUEUE_SIZE },
+	{ "iptfs-init-delay", required_argument, NULL, CD_IPTFS_INIT_DELAY },
+	{ "iptfs-reorder-window", required_argument, NULL, CD_IPTFS_REORDER_WINDOW },
+	{ "iptfs-drop-time", required_argument, NULL, CD_IPTFS_DROP_TIME },
+
 	{ "no-nat_keepalive", no_argument, NULL,  CD_NO_NAT_KEEPALIVE },
 	{ "ikev1_natt", required_argument, NULL, CD_IKEV1_NATT },	/* obsolete _ */
 	{ "ikev1-natt", required_argument, NULL, CD_IKEV1_NATT },
@@ -835,9 +870,10 @@ static const struct option long_opts[] = {
 	{ "xauthclient", no_argument, NULL, END_XAUTHCLIENT },
 	{ "xauthby", required_argument, NULL, CD_XAUTHBY },
 	{ "xauthfail", required_argument, NULL, CD_XAUTHFAIL },
-	{ "modecfgpull", no_argument, NULL, CD_MODECFGPULL },
-	{ "modecfgserver", no_argument, NULL, END_MODECFGSERVER },
-	{ "modecfgclient", no_argument, NULL, END_MODECFGCLIENT },
+	{ "modecfgpull", optional_argument, NULL, CD_MODECFGPULL },
+	{ "modecfgserver", optional_argument, NULL, END_MODECFGSERVER },
+	{ "modecfgclient", optional_argument, NULL, END_MODECFGCLIENT },
+	{ "cat", optional_argument, NULL, END_CAT },
 	{ "addresspool", required_argument, NULL, END_ADDRESSPOOL },
 	{ "modecfgdns", required_argument, NULL, CD_MODECFGDNS },
 	{ "modecfgdomains", required_argument, NULL, CD_MODECFGDOMAINS },
@@ -946,114 +982,41 @@ static const struct option long_opts[] = {
 	{ 0, 0, 0, 0 }
 };
 
-/*
- * figure out an address family.
- */
-struct family {
-	const char *used_by;
-	const struct ip_info *type;
-};
-
-static err_t opt_ttoaddress_num(struct family *family, ip_address *address)
-{
-	err_t err = ttoaddress_num(shunk1(optarg), family->type, address);
-	if (err == NULL && family->type == NULL) {
-		family->type = address_type(address);
-		family->used_by = long_opts[long_index].name;
-	}
-	return err;
-}
-
-static err_t opt_ttoaddress_dns(struct family *family, ip_address *address)
-{
-	err_t err = ttoaddress_dns(shunk1(optarg), family->type, address);
-	if (err == NULL && family->type == NULL) {
-		family->type = address_type(address);
-		family->used_by = long_opts[long_index].name;
-	}
-	return err;
-}
-
-static void opt_to_address(struct family *family, ip_address *address)
-{
-	diagq(opt_ttoaddress_dns(family, address), optarg);
-}
-
-static void opt_to_cidr(struct family *family, ip_cidr *cidr)
-{
-	diagq(ttocidr_num(shunk1(optarg), family->type, cidr), optarg);
-	if (family->type == NULL) {
-		family->type = cidr_type(cidr);
-		family->used_by = long_opts[long_index].name;
-	}
-}
-
-static const struct ip_info *get_address_family(struct family *family)
-{
-	if (family->type == NULL) {
-		family->type = &ipv4_info;
-		family->used_by = long_opts[long_index].name;
-	}
-	return family->type;
-}
-
-static ip_address get_address_any(struct family *family)
-{
-	return get_address_family(family)->address.unspec;
-}
-
 static char *ctlsocket = NULL;
 
-static deltatime_t optarg_deltatime(const struct timescale *timescale)
-{
-	passert((long_opts[long_index].has_arg == required_argument) ||
-		(long_opts[long_index].has_arg == optional_argument && optarg != NULL));
-	deltatime_t deltatime;
-	diag_t diag = ttodeltatime(optarg, &deltatime, timescale);
-	if (diag != NULL) {
-		diagq(str_diag(diag), optarg);
-	}
-	return deltatime;
-}
-
-static uintmax_t optarg_uintmax(void)
-{
-	passert((long_opts[long_index].has_arg == required_argument) ||
-		(long_opts[long_index].has_arg == optional_argument && optarg != NULL));
-	uintmax_t val;
-	err_t err = shunk_to_uintmax(shunk1(optarg), NULL, /*base*/0, &val);
-	if (err != NULL) {
-		diagq(err, optarg);
-	}
-	return val;
-}
-
 /*
- * Lookup OPTARG in NAMES.
+ * If the numeric address is valid, accept it.  Otherwise try to parse
+ * it using DNS, and regardless throw the name at pluto.
  *
- * When optional_argument OPTARG is missing, return OPTIONAL (pass
- * optional=0 when required_argument).
+ * This is pretty bespoke.
  */
 
-static uintmax_t optarg_sparse(unsigned optional, const struct sparse_names *names)
+static void msg_host_name(struct optarg_family *family, ip_address *address, char **dns_name)
 {
-	if (optarg == NULL) {
-		passert(long_opts[long_index].has_arg == optional_argument);
-		passert(optional != 0);
-		return optional;
+	if (ttoaddress_num(shunk1(optarg), family->type, address) == NULL) {
+		/*
+		 * we have a proper numeric IP address.  Update the
+		 * host's family.
+		 */
+		optarg_family(family, address_type(address));
+		return;
 	}
 
-	const struct sparse_name *name = sparse_lookup(names, shunk1(optarg));
-	if (name == NULL) {
-		JAMBUF(buf) {
-			const char *msg = jambuf_cursor(buf); /* hack */
-			jam(buf, "unrecognized --%s '%s', valid arguments are: ",
-			    long_opts[long_index].name, optarg);
-			jam_sparse_names(buf, names, ", ");
-			diagw(msg);
-		}
+	/*
+	 * We assume that we have a DNS name.
+	 *
+	 * This logic matches confread.c.  ??? it would be kind to
+	 * check the syntax.
+	 *
+	 * we don't fail here.  Pluto will re-check the DNS later
+	 * (begging the question of why bother here!).
+	 *
+	 * Shouldn't this be per-end.
+	 */
+	(*dns_name) = optarg;
+	if (ttoaddress_dns(shunk1(optarg), family->type, address) == NULL) {
+		optarg_family(family, address_type(address));
 	}
-	return name->value;
 }
 
 /* This is a hack for initiating ISAKMP exchanges. */
@@ -1061,6 +1024,7 @@ static uintmax_t optarg_sparse(unsigned optional, const struct sparse_names *nam
 int main(int argc, char **argv)
 {
 	struct logger *logger = tool_logger(argc, argv);
+	optarg_init(logger); /* merge into tool logger? */
 
 	struct whack_message msg;
 	char esp_buf[256];	/* uses snprintf */
@@ -1076,12 +1040,12 @@ int main(int argc, char **argv)
 
 	zero(&msg);	/* ??? pointer fields might not be NULLed */
 
-	clear_end("left", &msg.left);
-	clear_end("right", &msg.right);
-	struct whack_end *end = &msg.left;
+	clear_end("left", &msg.end[LEFT_END]);
+	clear_end("right", &msg.end[RIGHT_END]);
+	struct whack_end *end = &msg.end[LEFT_END];
 
-	struct family host_family = { 0, };
-	struct family child_family = { 0, };
+	struct optarg_family host_family = { 0, };
+	struct optarg_family child_family = { 0, };
 
 	msg.whack_from = WHACK_FROM_WHACK;		/* use whack defaults */
 
@@ -1101,13 +1065,10 @@ int main(int argc, char **argv)
 
 	msg.sa_ipsec_max_bytes = IPSEC_SA_MAX_OPERATIONS; /* max uint_64_t */
 	msg.sa_ipsec_max_packets = IPSEC_SA_MAX_OPERATIONS; /* max uint_64_t */
-	msg.sa_rekey_margin = deltatime(SA_REPLACEMENT_MARGIN_DEFAULT);
 	msg.sa_rekeyfuzz_percent = SA_REPLACEMENT_FUZZ_DEFAULT;
 	msg.keyingtries.set = false;
 	/* whack cannot access kernel_ops->replay_window */
 	msg.replay_window = IPSEC_SA_DEFAULT_REPLAY_WINDOW;
-	msg.retransmit_timeout = deltatime(RETRANSMIT_TIMEOUT_DEFAULT);
-	msg.retransmit_interval = deltatime_ms(RETRANSMIT_INTERVAL_DEFAULT_MS);
 
 	msg.host_afi = NULL;
 	msg.child_afi = NULL;
@@ -1127,7 +1088,7 @@ int main(int argc, char **argv)
 		 * by getopt_long, so we simply pass an empty string as
 		 * the list.  It could be "hp:d:c:o:eatfs" "NARXPECK".
 		 */
-		int c = getopt_long(argc, argv, "", long_opts, &long_index);
+		int c = getopt_long(argc, argv, "", optarg_options, &optarg_index);
 
 		/* end of flags? exit loop */
 		if (c < 0) {
@@ -1148,7 +1109,7 @@ int main(int argc, char **argv)
 			 */
 			if (seen[c]) {
 				diagq("duplicated flag",
-				      long_opts[long_index].name);
+				      optarg_options[optarg_index].name);
 			}
 			seen[c] = true;
 			opts_seen |= NORMAL_OPT_SEEN;
@@ -1176,7 +1137,7 @@ int main(int argc, char **argv)
 			 */
 			if (seen[c])
 				diagq("duplicated flag",
-				      long_opts[long_index].name);
+				      optarg_options[optarg_index].name);
 			seen[c] = true;
 			opts_seen |= END_OPT_SEEN;
 			opts_seen |= CONN_OPT_SEEN;
@@ -1187,7 +1148,7 @@ int main(int argc, char **argv)
 			 */
 			if (seen[c])
 				diagq("duplicated flag",
-				      long_opts[long_index].name);
+				      optarg_options[optarg_index].name);
 			seen[c] = true;
 			opts_seen |= CONN_OPT_SEEN;
 #if 0
@@ -1364,6 +1325,13 @@ int main(int argc, char **argv)
 			msg.whack_sa_type = CHILD_SA;
 			continue;
 
+		case OPT_SUSPEND: /* --suspend */
+			msg.whack_suspend = true;
+			continue;
+		case CD_SESSION_RESUMPTION:
+			msg.session_resumption = optarg_sparse(YN_YES, &yn_option_names);
+			break;
+
 		case OPT_DELETE:	/* --delete */
 			msg.whack_delete = true;
 			continue;
@@ -1379,9 +1347,9 @@ int main(int argc, char **argv)
 
 		case OPT_DELETECRASH:	/* --crash <ip-address> */
 			msg.whack_crash = true;
-			opt_to_address(&host_family, &msg.whack_crash_peer);
+			msg.whack_crash_peer = optarg_address_dns(&host_family);
 			if (!address_is_specified(msg.whack_crash_peer)) {
-				/* either :: or 0.0.0.0; unset rejected */
+				/* either :: or 0.0.0.0; unset already rejected */
 				address_buf ab;
 				diagq("invalid --crash <address>",
 				      str_address(&msg.whack_crash_peer, &ab));
@@ -1395,7 +1363,7 @@ int main(int argc, char **argv)
 
 		case OPT_REDIRECT_TO:	/* --redirect-to */
 			/* either active, or or add */
-			msg.redirect_to = strdup(optarg);
+			msg.redirect_to = optarg;
 			continue;
 
 		case OPT_GLOBAL_REDIRECT:	/* --global-redirect */
@@ -1414,7 +1382,7 @@ int main(int argc, char **argv)
 			if (!strlen(optarg)) {
 				msg.global_redirect_to = strdup("<none>");
 			} else {
-				msg.global_redirect_to = strdup(optarg);
+				msg.global_redirect_to = optarg;
 			}
 			continue;
 
@@ -1539,9 +1507,9 @@ int main(int argc, char **argv)
 			continue;
 
 		case OPT_OPPO_HERE:	/* --oppohere <ip-address> */
-			opt_to_address(&child_family, &msg.oppo.local.address);
+			msg.oppo.local.address = optarg_address_dns(&child_family);
 			if (!address_is_specified(msg.oppo.local.address)) {
-				/* either :: or 0.0.0.0; unset rejected */
+				/* either :: or 0.0.0.0; unset already rejected */
 				address_buf ab;
 				diagq("invalid --opphere <address>",
 				      str_address(&msg.oppo.local.address, &ab));
@@ -1549,9 +1517,9 @@ int main(int argc, char **argv)
 			continue;
 
 		case OPT_OPPO_THERE:	/* --oppothere <ip-address> */
-			opt_to_address(&child_family, &msg.oppo.remote.address);
+			msg.oppo.remote.address = optarg_address_dns(&child_family);
 			if (!address_is_specified(msg.oppo.remote.address)) {
-				/* either :: or 0.0.0.0; unset rejected */
+				/* either :: or 0.0.0.0; unset already rejected */
 				address_buf ab;
 				diagq("invalid --oppothere <address>",
 				      str_address(&msg.oppo.remote.address, &ab));
@@ -1610,47 +1578,37 @@ int main(int argc, char **argv)
 		case END_HOST:	/* --host <ip-address> */
 		{
 			if (streq(optarg, "%any")) {
-				end->host_addr = get_address_any(&host_family);
+				end->host_addr = optarg_any(&host_family);
 				end->host_type = KH_ANY;
 			} else if (streq(optarg, "%opportunistic")) {
 				/* always use tunnel mode; mark as opportunistic */
 				msg.type = KS_TUNNEL;
 				end->host_type = KH_OPPO;
-				end->host_addr = get_address_any(&host_family);
+				end->host_addr = optarg_any(&host_family);
 				end->key_from_DNS_on_demand = true;
 			} else if (streq(optarg, "%group")) {
 				/* always use tunnel mode; mark as group */
 				msg.type = KS_TUNNEL;
 				end->host_type = KH_GROUP;
-				end->host_addr = get_address_any(&host_family);
+				end->host_addr = optarg_any(&host_family);
 			} else if (streq(optarg, "%opportunisticgroup")) {
 				/* always use tunnel mode; mark as opportunistic */
 				msg.type = KS_TUNNEL;
 				end->host_type = KH_OPPOGROUP;
-				end->host_addr = get_address_any(&host_family);
+				end->host_addr = optarg_any(&host_family);
 				end->key_from_DNS_on_demand = true;
-			} else if (msg.left.id != NULL && !streq(optarg, "%null")) {
-				if (opt_ttoaddress_num(&host_family, &end->host_addr) == NULL) {
-					/*
-					 * we have a proper numeric IP
-					 * address.
-					 */
-				} else {
-					/*
-					 * We assume that we have a DNS name.
-					 * This logic matches confread.c.
-					 * ??? it would be kind to check
-					 * the syntax.
-					 */
-					msg.dnshostname = optarg;
-					opt_ttoaddress_dns(&host_family, &end->host_addr);
-					/*
-					 * we don't fail here.  pluto
-					 * will re-check the DNS later
-					 */
-				}
+			} else if (msg.end[LEFT_END].id != NULL && !streq(optarg, "%null")) {
+				/*
+				 * This is pretty bespoke.
+				 *
+				 * If the numeric address is valid,
+				 * accept it.  Otherwise try to parse
+				 * it using DNS, and regardless throw
+				 * the name at pluto.
+				 */
+				msg_host_name(&host_family, &end->host_addr, &msg.dnshostname);
 			} else {
-				opt_to_address(&host_family, &end->host_addr);
+				end->host_addr = optarg_address_dns(&host_family);
 			}
 
 			if (end->host_type == KH_GROUP ||
@@ -1722,9 +1680,9 @@ int main(int argc, char **argv)
 
 		case END_NEXTHOP:	/* --nexthop <ip-address> */
 			if (streq(optarg, "%direct")) {
-				end->host_nexthop = get_address_any(&host_family);
+				end->nexthop = optarg_any(&host_family);
 			} else {
-				opt_to_address(&host_family, &end->host_nexthop);
+				end->nexthop = optarg_address_dns(&host_family);
 			}
 			continue;
 
@@ -1737,7 +1695,7 @@ int main(int argc, char **argv)
 			continue;
 
 		case END_VTIIP:	/* --vtiip <ip-address/mask> */
-			opt_to_cidr(&child_family, &end->host_vtiip);
+			end->host_vtiip = optarg_cidr_num(&child_family);
 			continue;
 
 		/*
@@ -1801,7 +1759,7 @@ int main(int argc, char **argv)
 				diagw("connection missing --host before --to");
 			}
 
-			end = &msg.right;
+			end = &msg.end[RIGHT_END];
 			for (enum option_enums e = FIRST_END_OPT; e <= LAST_END_OPT; e++) {
 				seen[e] = false;
 			}
@@ -1834,6 +1792,28 @@ int main(int argc, char **argv)
 			msg.reauth = YN_YES;
 			continue;
 
+		case CD_IPTFS: /* --iptfs[={yes,no}] */
+			msg.iptfs = optarg_sparse(YN_YES, &yn_option_names);
+			continue;
+		case CD_IPTFS_FRAGMENTATION: /* --iptfs-fragmentation={yes,no} */
+			msg.iptfs_fragmentation = optarg_sparse(YN_YES, &yn_option_names);
+			continue;
+		case CD_IPTFS_PACKET_SIZE:	/* --iptfs-packet-size */
+			msg.iptfs_packet_size = optarg_uintmax();
+			continue;
+		case CD_IPTFS_MAX_QUEUE_SIZE: /* --iptfs-max-queue-size */
+			msg.iptfs_max_queue_size = optarg_uintmax();
+			continue;
+		case CD_IPTFS_DROP_TIME: /* --iptfs-drop-time */
+			msg.iptfs_drop_time = optarg_deltatime(TIMESCALE_SECONDS);
+			continue;
+		case CD_IPTFS_INIT_DELAY: /* --iptfs-init-delay */
+			msg.iptfs_init_delay = optarg_deltatime(TIMESCALE_SECONDS);
+			continue;
+		case CD_IPTFS_REORDER_WINDOW: /* --iptfs-reorder-window */
+			msg.iptfs_reorder_window = optarg_uintmax();
+			continue;
+
 		case CD_COMPRESS:	/* --compress */
 			msg.compress = optarg_sparse(YN_YES, &yn_option_names);
 			continue;
@@ -1904,11 +1884,6 @@ int main(int argc, char **argv)
 			msg.aggressive = optarg_sparse(YN_YES, &yn_option_names);
 			continue;
 
-		/* --modecfgpull */
-		case CD_MODECFGPULL:
-			msg.modecfgpull = YN_YES;
-			continue;
-
 		/* --allow-cert-without-san-id */
 		case CD_ALLOW_CERT_WITHOUT_SAN_ID:
 			msg.require_id_on_certificate = YN_NO;
@@ -1979,19 +1954,19 @@ int main(int argc, char **argv)
 			continue;
 
 		case CD_RETRANSMIT_TIMEOUT:	/* --retransmit-timeout <seconds> */
-			msg.retransmit_timeout = optarg_deltatime(&timescale_seconds);
+			msg.retransmit_timeout = optarg_deltatime(TIMESCALE_SECONDS);
 			continue;
 
 		case CD_RETRANSMIT_INTERVAL:	/* --retransmit-interval <milliseconds> (not seconds) */
-			msg.retransmit_interval = optarg_deltatime(&timescale_milliseconds);
+			msg.retransmit_interval = optarg_deltatime(TIMESCALE_MILLISECONDS);
 			continue;
 
 		case CD_IKE_LIFETIME:	/* --ike-lifetime <seconds> */
-			msg.ikelifetime = optarg_deltatime(&timescale_seconds);
+			msg.ikelifetime = optarg_deltatime(TIMESCALE_SECONDS);
 			continue;
 
 		case CD_IPSEC_LIFETIME:	/* --ipsec-lifetime <seconds> */
-			msg.ipsec_lifetime = optarg_deltatime(&timescale_seconds);
+			msg.ipsec_lifetime = optarg_deltatime(TIMESCALE_SECONDS);
 			continue;
 
 		case CD_IPSEC_MAX_BYTES:	/* --ipsec-max-bytes <bytes> */
@@ -2003,7 +1978,7 @@ int main(int argc, char **argv)
 			continue;
 
 		case CD_REKEYMARGIN:	/* --rekeymargin <seconds> */
-			msg.sa_rekey_margin = optarg_deltatime(&timescale_seconds);
+			msg.rekeymargin = optarg_deltatime(TIMESCALE_SECONDS);
 			continue;
 
 		case CD_RKFUZZ:	/* --rekeyfuzz <percentage> */
@@ -2063,16 +2038,16 @@ int main(int argc, char **argv)
 			continue;
 
 		case CD_DPDDELAY:	/* --dpddelay <seconds> */
-			msg.dpddelay = strdup(optarg);
+			msg.dpddelay = optarg;
 			continue;
 
 		case CD_DPDTIMEOUT:	/* --dpdtimeout <seconds> */
-			msg.dpdtimeout = strdup(optarg);
+			msg.dpdtimeout = optarg;
 			continue;
 
 		case CD_OBSOLETE:
 			llog(RC_LOG, logger,
-			     "obsolete --%s option ignored", long_opts[long_index].name);
+			     "obsolete --%s option ignored", optarg_options[optarg_index].name);
 			continue;
 
 		case CD_SEND_REDIRECT:	/* --send-redirect */
@@ -2084,7 +2059,7 @@ int main(int argc, char **argv)
 			continue;
 
 		case CD_ACCEPT_REDIRECT_TO:	/* --accept-redirect-to */
-			msg.accept_redirect_to = strdup(optarg);
+			msg.accept_redirect_to = optarg;
 			continue;
 
 		case CD_IKE:	/* --ike <ike_alg1,ike_alg2,...> */
@@ -2207,7 +2182,7 @@ int main(int argc, char **argv)
 				/* i.e., --host ::1 --ipv4; useful? wrong message? */
 				diagq("--ipv4 must precede", host_family.used_by);
 			}
-			host_family.used_by = long_opts[long_index].name;
+			host_family.used_by = optarg_options[optarg_index].name;
 			host_family.type = &ipv4_info;
 			continue;
 
@@ -2226,7 +2201,7 @@ int main(int argc, char **argv)
 				/* i.e., --host 0.0.0.1 --ipv6; useful? wrong message? */
 				diagq("--ipv6 must precede", host_family.used_by);
 			}
-			host_family.used_by = long_opts[long_index].name;
+			host_family.used_by = optarg_options[optarg_index].name;
 			host_family.type = &ipv6_info;
 			continue;
 
@@ -2236,7 +2211,7 @@ int main(int argc, char **argv)
 			}
 			if (child_family.used_by != NULL)
 				diagq("--tunnelipv4 must precede", child_family.used_by);
-			child_family.used_by = long_opts[long_index].name;
+			child_family.used_by = optarg_options[optarg_index].name;
 			child_family.type = &ipv4_info;
 			continue;
 
@@ -2246,7 +2221,7 @@ int main(int argc, char **argv)
 			}
 			if (child_family.used_by != NULL)
 				diagq("--tunnelipv6 must precede", child_family.used_by);
-			child_family.used_by = long_opts[long_index].name;
+			child_family.used_by = optarg_options[optarg_index].name;
 			child_family.type = &ipv6_info;
 			continue;
 
@@ -2276,40 +2251,46 @@ int main(int argc, char **argv)
 			xauthpasslen = jam_str(xauthpass, sizeof(xauthpass), optarg) - xauthpass + 1;
 			continue;
 
-		case END_MODECFGCLIENT:	/* --modeconfigclient */
-			end->modecfg_client = true;
-			continue;
-
-		case END_MODECFGSERVER:	/* --modeconfigserver */
-			end->modecfg_server = true;
+		case END_CAT:		/* --cat */
+			end->cat = optarg_sparse(YN_YES, &yn_option_names);
 			continue;
 
 		case END_ADDRESSPOOL:	/* --addresspool */
-			end->addresspool = strdup(optarg);
+			end->addresspool = optarg;
+			continue;
+
+		case END_MODECFGCLIENT:	/* --modeconfigclient */
+			end->modecfgclient = optarg_sparse(YN_YES, &yn_option_names);
+			continue;
+		case END_MODECFGSERVER:	/* --modeconfigserver */
+			end->modecfgserver = optarg_sparse(YN_YES, &yn_option_names);
+			continue;
+		case CD_MODECFGPULL:	/* --modecfgpull */
+			msg.modecfgpull = optarg_sparse(YN_YES, &yn_option_names);
 			continue;
 
 		case CD_MODECFGDNS:	/* --modecfgdns */
-			msg.modecfgdns = strdup(optarg);
+			msg.modecfgdns = optarg;
 			continue;
 		case CD_MODECFGDOMAINS:	/* --modecfgdomains */
-			msg.modecfgdomains = strdup(optarg);
+			msg.modecfgdomains = optarg;
 			continue;
 		case CD_MODECFGBANNER:	/* --modecfgbanner */
-			msg.modecfgbanner = strdup(optarg);
+			msg.modecfgbanner = optarg;
 			continue;
 
 		case CD_CONN_MARK:      /* --conn-mark */
-			msg.mark = strdup(optarg);
+			msg.mark = optarg;
 			continue;
 		case CD_CONN_MARK_IN:      /* --conn-mark-in */
-			msg.mark_in = strdup(optarg);
+			msg.mark_in = optarg;
 			continue;
 		case CD_CONN_MARK_OUT:      /* --conn-mark-out */
-			msg.mark_out = strdup(optarg);
+			msg.mark_out = optarg;
 			continue;
 
 		case CD_VTI_INTERFACE:      /* --vti-interface=IFACE */
-			msg.vti_interface = strdup(optarg);
+			msg.vti_interface = optarg;
 			continue;
 		case CD_VTI_ROUTING:	/* --vti-routing[=yes|no] */
 			msg.vti_routing = optarg_sparse(YN_YES, &yn_option_names);
@@ -2319,7 +2300,7 @@ int main(int argc, char **argv)
 			continue;
 
 		case CD_IPSEC_INTERFACE:      /* --ipsec-interface=... */
-			msg.ipsec_interface = strdup(optarg);
+			msg.ipsec_interface = optarg;
 			continue;
 
 		case CD_XAUTHBY:	/* --xauthby */
@@ -2447,42 +2428,11 @@ int main(int argc, char **argv)
 			continue;
 
 		case DBGOPT_DEBUG:	/* --debug */
+			optarg_debug_lmod(/*enable*/true, &msg.debugging);
+			continue;
 		case DBGOPT_NO_DEBUG:	/* --no-debug */
-		{
-			bool enable = (c == DBGOPT_DEBUG);
-			if (streq(optarg, "list") || streq(optarg, "help") || streq(optarg, "?")) {
-				fprintf(stderr, "aliases:\n");
-				for (struct lmod_alias *a = debug_lmod_info.aliases;
-				     a->name != NULL; a++) {
-					JAMBUF(buf) {
-						jam(buf, "  %s: ", a->name);
-						jam_lset_short(buf, debug_lmod_info.names, "+", a->bits);
-						fprintf(stderr, PRI_SHUNK"\n",
-							pri_shunk(jambuf_as_shunk(buf)));
-					}
-				}
-				fprintf(stderr, "bits:\n");
-				for (long e = next_enum(&debug_names, -1);
-				     e != -1; e = next_enum(&debug_names, e)) {
-					JAMBUF(buf) {
-						jam(buf, "  ");
-						jam_enum_short(buf, &debug_names, e);
-						enum_buf help;
-						if (enum_name(&debug_help, e, &help)) {
-							jam(buf, ": ");
-							jam_string(buf, help.buf);
-						}
-						fprintf(stderr, PRI_SHUNK"\n",
-							pri_shunk(jambuf_as_shunk(buf)));
-					}
-				}
-				exit(1);
-			} else if (!lmod_arg(&msg.debugging, &debug_lmod_info, optarg, enable)) {
-				fprintf(stderr, "whack: unrecognized -%s-debug '%s' option ignored\n",
-					enable ? "" : "-no", optarg);
-			}
+			optarg_debug_lmod(/*enable*/false, &msg.debugging);
 			continue;
-		}
 
 		case DBGOPT_IMPAIR:	/* --impair */
 		case DBGOPT_NO_IMPAIR:	/* --no-impair */
@@ -2622,8 +2572,8 @@ int main(int argc, char **argv)
 		} else {
 			/* not just a shunt: a real ipsec connection */
 			if (!authby_is_set(msg.authby) &&
-			    msg.left.auth == AUTH_NEVER &&
-			    msg.right.auth == AUTH_NEVER)
+			    msg.end[LEFT_END].auth == AUTH_NEVER &&
+			    msg.end[RIGHT_END].auth == AUTH_NEVER)
 				diagw("must specify connection authentication, eg --rsasig, --psk or --auth-null for non-shunt connection");
 			/*
 			 * ??? this test can never fail:
@@ -2631,7 +2581,8 @@ int main(int argc, char **argv)
 			 * These interlocking tests should be redone.
 			 */
 			if (msg.never_negotiate_shunt != SHUNT_UNSET &&
-			    (msg.left.subnet != NULL || msg.right.subnet != NULL))
+			    (msg.end[LEFT_END].subnet != NULL ||
+			     msg.end[RIGHT_END].subnet != NULL))
 				diagw("must not specify clients for ISAKMP-only connection");
 		}
 
@@ -2654,6 +2605,7 @@ int main(int argc, char **argv)
 	    seen[OPT_DELETE_CHILD] ||
 	    seen[OPT_DOWN_IKE] ||
 	    seen[OPT_DOWN_CHILD] ||
+	    seen[OPT_SUSPEND] ||
 	    (opts_seen & CONN_OPT_SEEN)) {
 		if (!seen[OPT_NAME]) {
 			diagw("missing --name <connection_name>");
@@ -2720,6 +2672,7 @@ int main(int argc, char **argv)
 	      msg.whack_seccomp_crashtest ||
 	      msg.whack_showstates ||
 	      msg.whack_sa ||
+	      msg.whack_suspend ||
 	      msg.whack_listpubkeys ||
 	      msg.whack_checkpubkeys)) {
 		diagw("no action specified; try --help for hints");
