@@ -2,21 +2,20 @@
 
 set -e
 
-run() {
-    echo -n "$@"": "
-    "$@"
-}
+run() (
+    set -x ; "$@"
+)
 
 for file in "$@" ; do
     domain=$(basename ${file})
     if state=$(sudo virsh domstate ${domain} 2>&1); then
 	case "${state}" in
 	    "running" | "in shutdown" | "paused" )
-		run sudo virsh destroy ${domain}
-		run sudo virsh undefine ${domain} --managed-save --snapshots-metadata
+		run sudo virsh destroy ${domain} || true
+		run sudo virsh undefine ${domain}
 		;;
 	    "shut off" )
-		run sudo virsh undefine ${domain} --managed-save --snapshots-metadata
+		run sudo virsh undefine ${domain}
 		;;
 	    * )
 		echo "Unknown state ${state} for ${domain}"
@@ -25,6 +24,6 @@ for file in "$@" ; do
     else
 	echo "No domain ${domain}"
     fi
-    rm -f ${file}.qcow2
-    rm -f ${file}
+    rm -vf ${file}.qcow2
+    rm -vf ${file}
 done
