@@ -24,6 +24,7 @@ ln -s "${pkgdir}" /var/db/pkgin/cache
 : pkg_add git crashes, do not know why
 pkgin -y install git
 pkgin -y install gmake
+pkgin -y install gawk
 pkgin -y install nss
 pkgin -y install unbound
 pkgin -y install bison
@@ -34,7 +35,7 @@ pkgin -y install pkgconf
 pkgin -y install fping
 pkgin -y install bash
 pkgin -y install racoon2
-pkgin -y install pkg_developer
+#pkgin -y install pkg_developer
 
 pkg_admin fetch-pkg-vulnerabilities
 pkg_admin audit || true
