@@ -1,6 +1,6 @@
 /testing/guestbin/swan-prep
 ipsec start
-/testing/pluto/bin/wait-until-pluto-started
+../../guestbin/wait-until-pluto-started
 ipsec auto --add westnet-eastnet-esp-sha1-pfs
 ipsec auto --add westnet-eastnet-esp-md5-pfs
 ipsec auto --status
