@@ -1,6 +1,8 @@
 ipsec auto --up westnet-eastnet-esp-sha1-pfs
-../../pluto/bin/ipsec-look.sh
+../../guestbin/ipsec-kernel-state.sh
+../../guestbin/ipsec-kernel-policy.sh
 ipsec auto --delete  westnet-eastnet-esp-sha1-pfs
 ipsec auto --up westnet-eastnet-esp-md5-pfs
-../../pluto/bin/ipsec-look.sh
+../../guestbin/ipsec-kernel-state.sh
+../../guestbin/ipsec-kernel-policy.sh
 echo done
