@@ -1,3 +1,4 @@
-../../guestbin/ipsec-look.sh
+../../guestbin/ipsec-kernel-state.sh
+../../guestbin/ipsec-kernel-policy.sh
 # normally xfrmcheck should never fail, but this tests the test (on east) :)
 ../../guestbin/xfrmcheck.sh
