@@ -1,14 +1,13 @@
 /testing/guestbin/swan-prep
 # confirm that the network is alive
-../../pluto/bin/wait-until-alive -I 192.0.1.254 192.0.2.254
+../../guestbin/wait-until-alive -I 192.0.1.254 192.0.2.254
 # ensure that clear text does not get through
 iptables -A INPUT -i eth1 -s 192.0.2.0/24 -j DROP
 iptables -I INPUT -m policy --dir in --pol ipsec -j ACCEPT
 # confirm clear text does not get through
-../../pluto/bin/ping-once.sh --down -I 192.0.1.254 192.0.2.254
-ipsec _stackmanager start
+../../guestbin/ping-once.sh --down -I 192.0.1.254 192.0.2.254
 valgrind --leak-check=full /usr/local/libexec/ipsec/pluto --config /etc/ipsec.conf  &
-/testing/pluto/bin/wait-until-pluto-started
+../../guestbin/wait-until-pluto-started
 ipsec auto --add westnet-eastnet
 ipsec auto --status | grep westnet-eastnet
 echo "initdone"
