@@ -1,6 +1,6 @@
 ../../guestbin/ping-once.sh --up -I 192.0.1.254 192.0.2.254
 ipsec trafficstatus
-ip route list
+../../guestbin/ip.sh route list
 # testing re-orienting
 ipsec auto --replace westnet-all
 ipsec auto --status |grep westnet
