@@ -1,7 +1,7 @@
 /testing/guestbin/swan-prep
 ip tunnel add test0 mode gre local 192.1.2.23 remote 192.1.2.45
-ip addr add 172.29.1.2/24 dev test0
-ip link set dev test0 up
+../../guestbin/ip.sh address add 172.29.1.2/24 dev test0
+../../guestbin/ip.sh link set dev test0 up
 ipsec start
 ../../guestbin/wait-until-pluto-started
 ipsec auto --add test1
