@@ -1,9 +1,9 @@
 /testing/guestbin/swan-prep --x509
-certutil -D -n east -d sql:/etc/ipsec.d 
+ipsec certutil -D -n east 
 cp policies/* /etc/ipsec.d/policies/
 echo "192.1.2.0/24" >> /etc/ipsec.d/policies/private-or-clear
 ipsec start
-/testing/pluto/bin/wait-until-pluto-started
+../../guestbin/wait-until-pluto-started
 # give OE policies time to load
-sleep 5
+../../guestbin/wait-for.sh --match 'loaded 10,' -- ipsec auto --status
 echo "initdone"
