@@ -1,4 +1,5 @@
-../../guestbin/ipsec-look.sh
+../../guestbin/ipsec-kernel-state.sh
+../../guestbin/ipsec-kernel-policy.sh
 hostname | grep nic > /dev/null || ipsec whack --trafficstatus
 # A tunnel should have established
 grep "^[^|].* established Child SA" /tmp/pluto.log
