@@ -1,12 +1,12 @@
 /testing/guestbin/swan-prep  --x509
-certutil -D -n road -d sql:/etc/ipsec.d
-certutil -D -n east -d sql:/etc/ipsec.d
+ipsec certutil -D -n road
+ipsec certutil -D -n east
 cp east-ikev2-oe.conf /etc/ipsec.d/ikev2-oe.conf
 cp policies/* /etc/ipsec.d/policies/
 echo "192.1.3.0/24"  >> /etc/ipsec.d/policies/private-or-clear
 ipsec start
-/testing/pluto/bin/wait-until-pluto-started
-ipsec whack --impair suppress-retransmits
+../../guestbin/wait-until-pluto-started
+ipsec whack --impair suppress_retransmits
 # give OE policies time to load
-sleep 5
+../../guestbin/wait-for.sh --match 'loaded 10,' -- ipsec auto --status
 echo "initdone"
