@@ -1,19 +1,19 @@
 /testing/guestbin/swan-prep --x509
-certutil -D -n road -d sql:/etc/ipsec.d
-certutil -D -n north -d sql:/etc/ipsec.d
-certutil -D -n east -d sql:/etc/ipsec.d
+ipsec certutil -D -n road
+ipsec certutil -D -n north
+ipsec certutil -D -n east
 cp road-ikev2-oe.conf /etc/ipsec.d/ikev2-oe.conf
 cp policies/* /etc/ipsec.d/policies/
 echo "192.1.2.0/24"  >> /etc/ipsec.d/policies/private-or-clear
 restorecon -R /etc/ipsec.d
 ipsec start
 ../../guestbin/wait-until-pluto-started
-ipsec whack --impair suppress-retransmits
+ipsec whack --impair suppress_retransmits
 # ensure for tests acquires expire before our failureshunt=2m
 echo 30 > /proc/sys/net/core/xfrm_acq_expires
 
 # give OE policies time to load
-../../guestbin/wait-for.sh --match 'loaded 10' -- ipsec auto --status
+../../guestbin/wait-for.sh --match 'loaded 9,' -- ipsec auto --status
 
 # one packet, which gets eaten by XFRM, so east does not initiate
 ../../guestbin/ping-once.sh --forget -I 192.1.3.33 192.1.2.23
@@ -27,5 +27,6 @@ echo 30 > /proc/sys/net/core/xfrm_acq_expires
 ../../guestbin/ping-once.sh --up -I 192.1.3.33 192.1.2.23
 ipsec whack --trafficstatus
 ipsec whack --shuntstatus
-../../guestbin/ipsec-look.sh
+../../guestbin/ipsec-kernel-state.sh
+../../guestbin/ipsec-kernel-policy.sh
 iptables -t nat -L -n
