@@ -1,11 +1,11 @@
 /testing/guestbin/swan-prep  --x509
-certutil -D -n road -d sql:/etc/ipsec.d
+ipsec certutil -D -n road
 cp east-ikev2-oe.conf /etc/ipsec.d/ikev2-oe.conf
 cp policies/* /etc/ipsec.d/policies/
 echo "192.1.3.209/32"  >> /etc/ipsec.d/policies/private-or-clear
 ipsec start
 ../../guestbin/wait-until-pluto-started
-ipsec whack --impair suppress-retransmits
+ipsec whack --impair suppress_retransmits
 # give OE policies time to load
-../../guestbin/wait-for.sh --match 'loaded 11,' -- ipsec auto --status
+../../guestbin/wait-for.sh --match 'loaded 10,' -- ipsec auto --status
 echo "initdone"
