@@ -1,14 +1,14 @@
 /testing/guestbin/swan-prep --x509
-certutil -D -n east -d sql:/etc/ipsec.d
+ipsec certutil -D -n east
 cp road-ikev2-oe.conf /etc/ipsec.d/ikev2-oe.conf
 cp policies/* /etc/ipsec.d/policies/
 echo "192.1.2.0/24"  >> /etc/ipsec.d/policies/private-or-clear
 ipsec start
 ../../guestbin/wait-until-pluto-started
-ipsec whack --impair suppress-retransmits
+ipsec whack --impair suppress_retransmits
 # ensure for tests acquires expire before our failureshunt=2m
 echo 1 > /proc/sys/net/core/xfrm_acq_expires
 # give OE policies time to load
-../../guestbin/wait-for.sh --match 'loaded 11,' -- ipsec auto --status
-ip -s xfrm monitor > /tmp/xfrm-monitor.out &
+../../guestbin/wait-for.sh --match 'loaded 10,' -- ipsec auto --status
+ip -s xfrm monitor > /tmp/xfrm-monitor.out & sleep 1
 echo "initdone"
