@@ -1,7 +1,7 @@
 /testing/guestbin/swan-prep --x509
-ip route del default
-ip route add default via 192.9.4.1
-certutil -D -n east -d sql:/etc/ipsec.d
+../../guestbin/ip.sh route del default
+../../guestbin/ip.sh route add default via 192.9.4.1
+ipsec certutil -D -n east
 cp west-ikev2-oe.conf /etc/ipsec.d/ikev2-oe.conf
 cp policies/* /etc/ipsec.d/policies/
 # specific /32 to test replacement of /32 oppo-instance with oppo-group
@@ -9,8 +9,8 @@ echo "192.1.2.23/32"  >> /etc/ipsec.d/policies/private
 restorecon -R /etc/ipsec.d
 ipsec start
 ../../guestbin/wait-until-pluto-started
-ipsec whack --impair suppress-retransmits
+ipsec whack --impair suppress_retransmits
 # give OE policies time to load
-../../guestbin/wait-for.sh --match 'loaded 11,' -- ipsec auto --status
-ip -s xfrm monitor > /tmp/xfrm-monitor.out &
+../../guestbin/wait-for.sh --match 'loaded 10,' -- ipsec auto --status
+ip -s xfrm monitor > /tmp/xfrm-monitor.out & sleep 1
 echo "initdone"
