@@ -1,5 +1,5 @@
 # a tunnel should have established
-grep "^[^|].*: established Child SA" /tmp/pluto.log
+grep "^[^|].* established Child SA" /tmp/pluto.log
 : ==== cut ====
 ipsec auto --status
 : ==== tuc ====
