@@ -1,11 +1,10 @@
 /testing/guestbin/swan-prep --x509
-certutil -D -n east -d sql:/etc/ipsec.d
-cp road-ikev2-oe.conf /etc/ipsec.d/ikev2-oe.conf
+ipsec certutil -D -n east
 cp policies/* /etc/ipsec.d/policies/
 echo "192.1.2.0/24"  >> /etc/ipsec.d/policies/private-or-clear
 ipsec start
 ../../guestbin/wait-until-pluto-started
-ipsec whack --impair suppress-retransmits
+ipsec whack --impair suppress_retransmits
 # give OE policies time to load
 ../../guestbin/wait-for.sh --match 'loaded 7' -- ipsec auto --status
 echo "initdone"
