@@ -3,4 +3,5 @@
 ipsec showstates
 ipsec trafficstatus
 ipsec shuntstatus
-../../guestbin/ipsec-look.sh
+../../guestbin/ipsec-kernel-state.sh
+../../guestbin/ipsec-kernel-policy.sh
