@@ -1,4 +1,5 @@
-../../guestbin/ipsec-look.sh
+../../guestbin/ipsec-kernel-state.sh
+../../guestbin/ipsec-kernel-policy.sh
 # ==== cut ====
 ipsec auto --status | grep westnet-eastnet-compress
 # ==== tuc ====
