@@ -1,13 +1,14 @@
-/testing/guestbin/swan-prep
+/testing/guestbin/swan-prep --hostkeys
 # confirm that the network is alive
-../../pluto/bin/wait-until-alive -I 192.0.1.254 192.0.2.254
+../../guestbin/wait-until-alive -I 192.0.1.254 192.0.2.254
 # ensure that clear text does not get through
 iptables -A INPUT -i eth1 -s 192.0.2.0/24 -j DROP
 # confirm clear text does not get through
-../../pluto/bin/ping-once.sh --down -I 192.0.1.254 192.0.2.254
+../../guestbin/ping-once.sh --down -I 192.0.1.254 192.0.2.254
 ipsec start
-/testing/pluto/bin/wait-until-pluto-started
-ipsec whack --impair delete-on-retransmit
+../../guestbin/wait-until-pluto-started
+ipsec whack --impair timeout_on_retransmit
 ipsec auto --add westnet-eastnet-compress
 ipsec auto --status | grep westnet-eastnet-compress
 echo "initdone"
+ipsec whack --impair revival
