@@ -1,6 +1,7 @@
-/testing/guestbin/swan-prep
+/testing/guestbin/swan-prep --hostkeys
+../../guestbin/mount-bind.sh /etc/hosts /etc/hosts
 ipsec start
-/testing/pluto/bin/wait-until-pluto-started
+../../guestbin/wait-until-pluto-started
 echo 192.1.2.23 east-from-hosts-file.example.com east-from-hosts-file >> /etc/hosts
 ipsec auto --add westnet-eastnet
 ipsec auto --status
