@@ -1,9 +1,9 @@
 # we can transmit in the clear
-ping -n -q -c 4 192.1.2.23
+../../guestbin/ping-once.sh --up 192.1.2.23
 # bring up the tunnel
 ipsec auto --up west-east
 # use the tunnel
-ping -n -q -c 4 192.1.2.23
+../../guestbin/ping-once.sh --up 192.1.2.23
 # show the tunnel!
 echo "Tunnel should be up"
 ipsec whack --trafficstatus
