@@ -1,12 +1,12 @@
-../../guestbin/swan-prep --fips
+../../guestbin/swan-prep --fips --nokeys
 # confirm that the network is alive
-../bin/wait-until-alive -I 192.0.1.254 192.0.2.254
+../../guestbin/wait-until-alive -I 192.0.1.254 192.0.2.254
 # ensure that clear text does not get through
 iptables -A INPUT -i eth1 -s 192.0.2.0/24 -j DROP
 iptables -I INPUT -m policy --dir in --pol ipsec -j ACCEPT
 # confirm clear text does not get through
-../../pluto/bin/ping-once.sh --down -I 192.0.1.254 192.0.2.254
+../../guestbin/ping-once.sh --down -I 192.0.1.254 192.0.2.254
 ipsec start
-../bin/wait-until-pluto-started
-ipsec whack --impair suppress-retransmits
+../../guestbin/wait-until-pluto-started
+ipsec whack --impair suppress_retransmits
 echo "initdone"
