@@ -1,4 +1,4 @@
-/testing/guestbin/swan-prep --fips
+/testing/guestbin/swan-prep --fips --hostkeys
 ipsec start
 ../../guestbin/wait-until-pluto-started
 ipsec auto --add westnet-eastnet-ikev2
