@@ -1,4 +1,4 @@
-/testing/guestbin/swan-prep
+/testing/guestbin/swan-prep --nokeys
 # Start only one, so it is easier to spot a crash
 ipsec pluto --config /etc/ipsec.conf 
 ../../guestbin/wait-until-pluto-started
