@@ -1,4 +1,4 @@
 # temp while the test still fails
-ipsec whack --debug-all --impair retransmits
+ipsec whack --impair suppress_retransmits
 ipsec auto --up north-east
 echo done
