@@ -1,9 +1,4 @@
-ip xfrm state
+../../guestbin/ipsec-kernel-state.sh
 HEAD=$(ip xfrm state |grep "enc "|head -1)
 TAIL=$(ip xfrm state |grep "enc "|tail -1)
 if [ "$HEAD" = "$TAIL" ]; then echo "ERROR: inbound and outbound key are the same!"; fi
-: ==== cut ====
-ipsec auto --status
-: ==== tuc ====
-../bin/check-for-core.sh
-if [ -f /sbin/ausearch ]; then ausearch -r -m avc -ts recent ; fi
