@@ -1,4 +1,4 @@
-/testing/guestbin/swan-prep
+/testing/guestbin/swan-prep --nokeys
 ifconfig eth1:1 192.1.2.46 netmask 255.255.255.0
 ifconfig eth1:2 192.1.2.47 netmask 255.255.255.0
 ifconfig eth1:3 192.1.2.48 netmask 255.255.255.0
