@@ -1,4 +1,4 @@
-/testing/guestbin/swan-prep
+/testing/guestbin/swan-prep --hostkeys
 ipsec start
 ../../guestbin/wait-until-pluto-started
 # connection is loaded and initiated via auto=start
