@@ -1,7 +1,7 @@
 /testing/guestbin/swan-prep --x509
 ipsec start
 ../../guestbin/wait-until-pluto-started
-ipsec whack --impair suppress-retransmits
+ipsec whack --impair suppress_retransmits
 ipsec whack --impair revival
 ipsec auto --add westnet-eastnet-ipv4-psk-ikev1
 echo "initdone"
