@@ -1,6 +1,6 @@
-/testing/guestbin/swan-prep
+/testing/guestbin/swan-prep --hostkeys
 ipsec start
-/testing/pluto/bin/wait-until-pluto-started
+../../guestbin/wait-until-pluto-started
 ipsec auto --add westnet-eastnet1
 ipsec auto --add westnet-eastnet2
 echo "initdone"
