@@ -1,6 +1,6 @@
-/testing/guestbin/swan-prep
+/testing/guestbin/swan-prep --hostkeys
 ipsec start
 ../../guestbin/wait-until-pluto-started
-ipsec whack --impair replay-duplicates
+ipsec whack --impair duplicate_inbound
 ipsec auto --add westnet-eastnet
 echo "initdone"
