@@ -1,10 +1,10 @@
 # should fail fast
-ipsec whack --impair ke-payload:0
-ipsec whack --impair delete-on-retransmit
+ipsec whack --impair ke_payload:0
+ipsec whack --impair timeout_on_retransmit
 ipsec auto --up westnet-eastnet-ipv4-psk
 ipsec whack --impair none
 # expected to fail with a timeout
-ipsec whack --impair suppress-retransmits
+ipsec whack --impair suppress_retransmits
 ipsec whack --impair revival
 ipsec auto --add westnet-eastnet-ipv4-psk
 ipsec auto --up westnet-eastnet-ipv4-psk
