@@ -1,11 +1,11 @@
 # don't come back
 ipsec whack --impair revival
 # expect quick fail
-ipsec whack --impair ke-payload:omit
-ipsec whack --impair delete-on-retransmit
+ipsec whack --impair ke_payload:omit
+ipsec whack --impair timeout_on_retransmit
 ipsec auto --up westnet-eastnet-ipv4-psk-ikev2
 ipsec whack --impair none
 # expect slower fail
-ipsec whack --impair suppress-retransmits
+ipsec whack --impair suppress_retransmits
 ipsec auto --up westnet-eastnet-ipv4-psk-ikev2
 echo done
