@@ -11,5 +11,5 @@ iptables -A INPUT -m policy --dir in --pol ipsec -j ACCEPT
 iptables -A OUTPUT -o eth1 -s 192.1.2.23 -m policy --dir out --pol none -p udp --sport 1701 -j REJECT
 iptables -A OUTPUT -m policy --dir out --pol ipsec -j ACCEPT
 ../../guestbin/l2tpd.sh
-../../guestbin/echod.sh
+../../guestbin/echo-server.sh -tcp -4 7 -daemon
 echo done
