@@ -1,4 +1,4 @@
-/testing/guestbin/swan-prep
+/testing/guestbin/swan-prep --nokeys
 ipsec start
 ../../guestbin/wait-until-pluto-started
 ../../guestbin/block-non-ipsec.sh
@@ -6,5 +6,5 @@ ipsec auto --add north-east
 ipsec whack --xauthname 'use1' --xauthpass 'use1pass' --name north-east --initiate
 ../../guestbin/ping-once.sh --up -I 192.0.2.101 192.0.2.254
 ipsec whack --trafficstatus
-ipsec whack --impair suppress-retransmits
+ipsec whack --impair suppress_retransmits
 echo initdone
