@@ -1,7 +1,7 @@
 /testing/guestbin/swan-prep --x509
-certutil -D -n west -d sql:/etc/ipsec.d
+ipsec certutil -D -n west
 ipsec start
 ../../guestbin/wait-until-pluto-started
-ipsec whack --impair suppress-retransmits
+ipsec whack --impair suppress_retransmits
 ipsec auto --add san
 echo "initdone"
