@@ -1,5 +1,7 @@
-/testing/guestbin/swan-prep --x509
-ipsec certutil -D -n "Libreswan test CA for mainca - Libreswan"
+/testing/guestbin/swan-prep --nokeys
+/testing/x509/import.sh real/mainca/west.end.p12
+/testing/x509/import.sh real/mainca/east.end.cert
+
 ipsec start
 ../../guestbin/wait-until-pluto-started
 ipsec whack --impair send_pkcs7_thingie
