@@ -1,8 +1,8 @@
 /testing/guestbin/swan-prep --x509
-certutil -D -n east -d sql:/etc/ipsec.d
+ipsec certutil -D -n east
 ipsec start
 ../../guestbin/wait-until-pluto-started
 ipsec auto --add ikev2-westnet-eastnet-x509-cr
 ipsec status | grep idtype
-ipsec whack --impair suppress-retransmits
+ipsec whack --impair suppress_retransmits
 echo "initdone"
