@@ -1,8 +1,8 @@
 /testing/guestbin/swan-prep --x509 --x509name west-nosan
-certutil -D -n east -d sql:/etc/ipsec.d
+ipsec certutil -D -n east
 ipsec start
-/testing/pluto/bin/wait-until-pluto-started
+../../guestbin/wait-until-pluto-started
 ipsec auto --add ikev2-westnet-eastnet-x509-cr
 ipsec status | grep idtype
-#ipsec whack --impair suppress-retransmits
+#ipsec whack --impair suppress_retransmits
 echo "initdone"
