@@ -1,4 +1,4 @@
-ipsec whack --impair retransmits
+ipsec whack --impair timeout_on_retransmit
 # this should fail
 ipsec auto --up san
 echo "done"
