@@ -2,6 +2,6 @@
 echo 0 > /proc/sys/net/ipv4/conf/all/rp_filter
 ipsec start
 ../../guestbin/wait-until-pluto-started
-ipsec whack --impair suppress-retransmits
+ipsec whack --impair suppress_retransmits
 ipsec auto --add road
 echo "initdone"
