@@ -1,13 +1,13 @@
-/testing/guestbin/swan-prep
+/testing/guestbin/swan-prep --nokeys
 
 # scrub the nssdb (is there a swan-prep option?)
 rm /etc/ipsec.d/*.db
 modutil -create -dbdir /etc/ipsec.d -force
 
 # import fiddled keys
-pk12util -d /etc/ipsec.d/ -i OUTPUT/west.p12 -W foobar
-pk12util -d /etc/ipsec.d/ -i OUTPUT/east.p12 -W foobar
-certutil -K -d /etc/ipsec.d/
+ipsec pk12util -i OUTPUT/west.p12 -W foobar
+ipsec pk12util -i OUTPUT/east.p12 -W foobar
+ipsec certutil -K
 
 # patch up ipsec.conf
 sed -i -e "s/@east-ckaid@/`cat OUTPUT/east.ckaid`/" /etc/ipsec.conf
@@ -24,5 +24,5 @@ ipsec start
 ../../guestbin/wait-until-pluto-started
 ipsec auto --add westnet-eastnet-ikev2
 ipsec auto --status | grep westnet-eastnet-ikev2
-ipsec whack --impair suppress-retransmits
+ipsec whack --impair suppress_retransmits
 echo "initdone"
