@@ -1,7 +1,7 @@
-/testing/guestbin/swan-prep
+/testing/guestbin/swan-prep --hostkeys
 : > /etc/ipsec.secrets
 ipsec start
-/testing/pluto/bin/wait-until-pluto-started
+../../guestbin/wait-until-pluto-started
 ipsec auto --add west-rsasigkey-east-rsasigkey
 ipsec auto --status
 echo "initdone"
