@@ -1,4 +1,4 @@
-/testing/guestbin/swan-prep
+/testing/guestbin/swan-prep --hostkeys
 : > /etc/ipsec.secrets
 ipsec start
 ../../guestbin/wait-until-pluto-started
