@@ -1,7 +1,8 @@
-/testing/guestbin/swan-prep --x509
-# delete the CA, both ends hardcode both certificates
-ipsec certutil -D -n "Libreswan test CA for mainca - Libreswan"
-ipsec certutil -D -n "east-ec"
+/testing/guestbin/swan-prep --nokeys
+# no CA, both ends hardcode both certificates
+/testing/x509/import.sh real/mainca/west.end.p12
+/testing/x509/import.sh real/mainca/east.end.cert
+
 # confirm that the network is alive
 ../../guestbin/wait-until-alive -I 192.0.1.254 192.0.2.254
 # ensure that clear text does not get through
