@@ -1,6 +1,6 @@
 /testing/guestbin/swan-prep --x509 --nsspw
 ipsec start
-/testing/pluto/bin/wait-until-pluto-started
+../../guestbin/wait-until-pluto-started
 ipsec auto --add westnet-eastnet-ikev2
 ipsec auto --status | grep westnet-eastnet-ikev2
 echo "initdone"
