@@ -1,6 +1,6 @@
-/testing/guestbin/swan-prep
+/testing/guestbin/swan-prep --nokeys
 ipsec start
 ../../guestbin/wait-until-pluto-started
 ipsec auto --add westnet-eastnet-ipv4-psk-ikev2
-ipsec whack --impair delete-on-retransmit
+ipsec whack --impair timeout_on_retransmit
 echo "initdone"
