@@ -1,4 +1,4 @@
-ping -n -q -c 4 -I 192.1.3.209 192.1.2.23
+../../guestbin/ping-once.sh --up -I 192.1.3.209 192.1.2.23
 ipsec whack --trafficstatus
 ipsec whack --shuntstatus
 ipsec status
