@@ -1,5 +1,5 @@
 /testing/guestbin/swan-prep --x509
 ipsec start
 ../../guestbin/wait-until-pluto-started
-ipsec whack --impair suppress-retransmits
+ipsec whack --impair suppress_retransmits
 ipsec auto --add road
