@@ -1,8 +1,8 @@
 /testing/guestbin/swan-prep --x509
-ip addr add 192.0.22.254/24 dev eth0
+../../guestbin/ip.sh address add 192.0.22.254/24 dev eth0
 ipsec start
-/testing/pluto/bin/wait-until-pluto-started
+../../guestbin/wait-until-pluto-started
 ipsec auto --add northnet-eastnets
 ipsec auto --status | grep northnet-eastnets
-ipsec whack --impair suppress-retransmits
+ipsec whack --impair suppress_retransmits
 echo "initdone"
