@@ -1,5 +1,5 @@
-ipsec auto --up  westnet-eastnet-ikev2
-ping -n -c 4 -I 192.0.1.254 192.0.2.254
+ipsec auto --up westnet-eastnet-ikev2
+../../guestbin/ping-once.sh --up -I 192.0.1.254 192.0.2.254
 ipsec whack --trafficstatus
 # wait for rekey event
 sleep 20
