@@ -1,5 +1,5 @@
 ipsec auto --up northnet-eastnet-ipv4
-ping -n -q -c 2 -I 192.0.3.254 192.0.2.254
+../../guestbin/ping-once.sh --up -I 192.0.3.254 192.0.2.254
 ipsec whack --trafficstatus
 # waiting 2 minutes in chunks of 15 seconds
 sleep 15
@@ -13,5 +13,5 @@ sleep 15
 sleep 15
 echo two minutes
 ipsec auto --status | grep northnet-eastnet-ipv4
-ping -n -q -c 2 -I 192.0.3.254 192.0.2.254
+../../guestbin/ping-once.sh --up -I 192.0.3.254 192.0.2.254
 echo done
