@@ -1,3 +1,3 @@
-ipsec whack --impair suppress-retransmits
+ipsec whack --impair suppress_retransmits
 ipsec auto --up ipv4-psk-ikev2-transport
 echo done
