@@ -1,6 +1,5 @@
 /testing/guestbin/swan-prep
-export PLUTO_CRYPTO_HELPER_DELAY=10
-ipsec pluto --config /etc/ipsec.conf --leak-detective
+ipsec pluto --impair helper_thread_delay:10 --config /etc/ipsec.conf --leak-detective
 ../../guestbin/wait-until-pluto-started
 ipsec auto --add multi
 echo "initdone"
