@@ -6,5 +6,5 @@ ipsec start
 ../../guestbin/wait-until-pluto-started
 ipsec auto --add road-east-x509-ipv4
 ipsec auto --add road-west-x509-ipv4
-ipsec whack --impair suppress-retransmits
+ipsec whack --impair suppress_retransmits
 echo "initdone"
