@@ -1,7 +1,7 @@
-/testing/guestbin/swan-prep
+/testing/guestbin/swan-prep --nokeys
 ipsec start
-/testing/pluto/bin/wait-until-pluto-started
-ipsec whack --impair suppress-retransmits
+../../guestbin/wait-until-pluto-started
+ipsec whack --impair suppress_retransmits
 ipsec auto --add northnet-westnet-ipv4-psk
 ipsec auto --up northnet-westnet-ipv4-psk
 ipsec auto --status
