@@ -2,6 +2,6 @@
 ipsec auto --listpubkeys
 ipsec auto --up road-east-ikev2
 # there should be two public keys. including road
-ping -n -q -c 4 -I 192.1.3.209 192.1.2.23
+../../guestbin/ping-once.sh --down -I 192.1.3.209 192.1.2.23
 ipsec whack --trafficstatus
 echo done
