@@ -1,8 +1,8 @@
 /testing/guestbin/swan-prep
 ipsec start
-/testing/pluto/bin/wait-until-pluto-started
+../../guestbin/wait-until-pluto-started
 ipsec auto --add north-east
-ipsec whack --impair suppress-retransmits
+ipsec whack --impair suppress_retransmits
 # road should have only one public key of its own
 ipsec auto --listpubkeys
 echo "initdone"
