@@ -1,6 +1,6 @@
-/testing/guestbin/swan-prep
+/testing/guestbin/swan-prep --hostkeys
 ipsec start
-/testing/pluto/bin/wait-until-pluto-started
+../../guestbin/wait-until-pluto-started
 ipsec auto --add east-any
 ipsec auto --status | grep east-any
 # east should have only one public key of its own
