@@ -1,6 +1,6 @@
 /testing/guestbin/swan-prep
 ipsec start
-/testing/pluto/bin/wait-until-pluto-started
+../../guestbin/wait-until-pluto-started
 ipsec auto --add road-east
 ipsec auto --status | grep road-east-ikev2
 # eash should have only one pub key not road.
