@@ -2,6 +2,6 @@
 ipsec auto --listpubkeys
 ipsec auto --up road-east
 # there should be two public keys. including road
-ping -n -c 4 -I 192.1.3.209 192.1.2.23
+../../guestbin/ping-once.sh --up -I 192.1.3.209 192.1.2.23
 ipsec whack --trafficstatus
 echo done
