@@ -1,7 +1,7 @@
-ipsec whack --impair suppress-retransmits
+ipsec whack --impair suppress_retransmits
 # this connection will fail
 ipsec auto --up west-westnet-eastnet
-ping -n -q -c 4 -I 192.0.1.254 192.0.2.254
+../../guestbin/ping-once.sh --up -I 192.0.1.254 192.0.2.254
 ipsec whack --trafficstatus
 # note the ID is @west
 ipsec auto --listpubkeys
@@ -14,5 +14,5 @@ ipsec auto --add east-westnet-eastnet
 ipsec auto --listpubkeys
 # this should succeed
 ipsec auto --up east-westnet-eastnet
-ping -n -q -c 4 -I 192.0.1.254 192.0.2.254
+../../guestbin/ping-once.sh --up -I 192.0.1.254 192.0.2.254
 echo done
