@@ -1,5 +1,5 @@
 /testing/guestbin/swan-prep
 rm -fr /var/run/pluto/pluto.pid
-PLUTO_CRYPTO_HELPER_DELAY=1 /usr/local/libexec/ipsec/pluto --config /etc/ipsec.conf
+/usr/local/libexec/ipsec/pluto --impair helper_thread_delay:1 --config /etc/ipsec.conf
 ../../guestbin/wait-until-pluto-started
 echo "initdone"
