@@ -1,5 +1,5 @@
 /testing/guestbin/swan-prep
 #add address from to be extruded subnet on road.
 ipsec start
-/testing/pluto/bin/wait-until-pluto-started
+../../guestbin/wait-until-pluto-started
 echo "initdone"
