@@ -1,10 +1,10 @@
 /testing/guestbin/swan-prep --hostname east
 ipsec start
-/testing/pluto/bin/wait-until-pluto-started
-ipsec whack --impair allow-null-none
+../../guestbin/wait-until-pluto-started
+ipsec whack --impair allow_null_none
 # normally NONE is not emitted
-ipsec whack --impair v2-proposal-integ:allow-none
-ipsec whack --impair suppress-retransmits
+ipsec whack --impair v2_proposal_integ:allow-none
+ipsec whack --impair suppress_retransmits
 ipsec auto --add west-east
 ipsec auto --status | grep west-east
 taskset 0x1 iperf3 -s -p 5001 & echo "$!"  > OUTPUT/$(hostname)-iperf3-5001.pid
