@@ -4,11 +4,11 @@ ipsec start
 ipsec auto --add west-east-ikev2
 ipsec auto --status | grep west
 
-../../guestbin/echod.sh 7
+../../guestbin/echo-server.sh -tcp -4 7 -daemon
 sleep 1
 echo 7 | ncat localhost 7
 
-../../guestbin/echod.sh 333
+../../guestbin/echo-server.sh -tcp -4 333 -daemon
 sleep 1
 echo 333 | ncat localhost 333
 
