@@ -1,4 +1,8 @@
-/testing/guestbin/swan-prep --x509
+/testing/guestbin/swan-prep --hostkeys
+
+/testing/x509/import.sh real/mainca/west.end.p12
+ipsec certutil -L
+
 # confirm that the network is alive
 ../../guestbin/wait-until-alive -I 192.0.1.254 192.0.2.254
 # ensure that clear text does not get through
@@ -9,5 +13,4 @@ iptables -I INPUT -m policy --dir in --pol ipsec -j ACCEPT
 ipsec start
 ../../guestbin/wait-until-pluto-started
 ipsec auto --add westnet-eastnet-ikev2
-ipsec auto --status | grep westnet-eastnet-ikev2
 echo "initdone"
