@@ -3,5 +3,5 @@ ipsec start
 ../../guestbin/wait-until-pluto-started
 ipsec auto --add east
 ipsec whack --impair revival
-ipsec whack --impair rekey-respond-subnet
+ipsec whack --impair rekey_respond_subnet
 echo "initdone"
