@@ -1,5 +1,5 @@
 ipsec auto --up westnet-eastnet-foo
 # expected to fail
-ipsec whack --impair delete-on-retransmit
+ipsec whack --impair timeout_on_retransmit
 ipsec auto --up westnet-eastnet-bar
 echo done
