@@ -1,8 +1,7 @@
-/testing/guestbin/swan-prep
+/testing/guestbin/swan-prep --nokeys
 # confirm that the network is alive
 ipsec start
-/testing/pluto/bin/wait-until-pluto-started
-ipsec whack --whackrecord /var/tmp/ikev2.record
+../../guestbin/wait-until-pluto-started
 ipsec auto --add west-east-delete1
 ipsec auto --status | grep west-east
 echo "initdone"
