@@ -2,5 +2,5 @@
 ipsec start
 ../../guestbin/wait-until-pluto-started
 ipsec auto --add westnet-eastnet-ikev2
-ipsec whack --impair replay-forward
+ipsec whack --impair replay_inbound
 echo "initdone"
