@@ -2,8 +2,8 @@
 # expect KE:0 in response
 ipsec whack --impair none
 ipsec whack --impair revival
-ipsec whack --impair ke-payload:0
-ipsec whack --impair suppress-retransmits
+ipsec whack --impair ke_payload:0
+ipsec whack --impair suppress_retransmits
 # DH should fail
 ipsec auto --up westnet-eastnet-ipv4-psk-ikev2
 
@@ -11,7 +11,7 @@ ipsec auto --up westnet-eastnet-ipv4-psk-ikev2
 # expect KE:0 in response (which is invalid)
 ipsec whack --impair none
 ipsec whack --impair revival
-ipsec whack --impair suppress-retransmits
+ipsec whack --impair suppress_retransmits
 # DH should fail
 ipsec auto --up westnet-eastnet-ipv4-psk-ikev2
 
