@@ -1,6 +1,6 @@
-/testing/guestbin/swan-prep
+/testing/guestbin/swan-prep --nokeys
 ipsec start
 ../../guestbin/wait-until-pluto-started
-ipsec whack --impair ke-payload:empty
+ipsec whack --impair ke_payload:empty
 ipsec auto --add westnet-eastnet-ipv4-psk-ikev2
 echo "initdone"
