@@ -1,3 +1,3 @@
-ipsec whack --impair omit-v2-ike-auth-child
+ipsec whack --impair omit_v2_ike_auth_child
 ipsec auto --up westnet-eastnet-ipv4-psk-ikev2
 echo done
