@@ -1,13 +1,13 @@
 /testing/guestbin/swan-prep
 # confirm that the network is alive
-../../pluto/bin/wait-until-alive -I 192.0.1.254 192.0.2.254
+../../guestbin/wait-until-alive -I 192.0.1.254 192.0.2.254
 # ensure that clear text does not get through
 iptables -I INPUT -m policy --dir in --pol ipsec -j ACCEPT
 iptables -A INPUT -i eth1 -s 192.0.2.0/24 -j DROP
 # confirm clear text does not get through
-../../pluto/bin/ping-once.sh --down -I 192.0.1.254 192.0.2.254
+../../guestbin/ping-once.sh --down -I 192.0.1.254 192.0.2.254
 ipsec start
-/testing/pluto/bin/wait-until-pluto-started
+../../guestbin/wait-until-pluto-started
 ipsec auto --add westnet-eastnet
 ipsec auto --status
 echo "initdone"
