@@ -1,4 +1,4 @@
-/testing/guestbin/swan-prep
+/testing/guestbin/swan-prep --hostkeys
 echo 3 > /proc/sys/net/core/xfrm_acq_expires
 # build install se module (west only)
 ../../guestbin/semodule.sh ipsecspd-full-perm.te
