@@ -1,4 +1,4 @@
-/testing/guestbin/swan-prep
+/testing/guestbin/swan-prep --hostkeys
 echo 3 > /proc/sys/net/core/xfrm_acq_expires
 # build install se module
 ../../guestbin/semodule.sh ipsecspd.te
@@ -6,5 +6,5 @@ ipsec start
 ../../guestbin/wait-until-pluto-started
 ipsec auto --add labeled
 # start the server
-ipsec getpeercon_server -d 4300
+ipsec _getpeercon_server -d 4300
 echo "initdone"
