@@ -1,4 +1,4 @@
-/testing/guestbin/swan-prep
+/testing/guestbin/swan-prep --hostkeys
 echo 3 > /proc/sys/net/core/xfrm_acq_expires
 # install selinux; generated in OUTPUT by east
 semodule -i OUTPUT/ipsecspd-full-perm.pp
