@@ -1,8 +1,8 @@
-/testing/guestbin/swan-prep
+/testing/guestbin/swan-prep --nokeys
 # confirm that the network is alive
 ../../guestbin/wait-until-alive -I 192.0.1.254 192.0.2.254
 ipsec start
 ../../guestbin/wait-until-pluto-started
 ipsec auto --add westnet-eastnet-ikev2-major
-ipsec whack --impair delete-on-retransmit
+ipsec whack --impair timeout_on_retransmit
 echo "initdone"
