@@ -1,9 +1,9 @@
 /testing/guestbin/swan-prep --x509
-certutil -D -d sql:/etc/ipsec.d -n road
-certutil -D -d sql:/etc/ipsec.d -n east
+ipsec certutil -D -n road
+ipsec certutil -D -n east
 ipsec start
 ../../guestbin/wait-until-pluto-started
 ipsec auto --add nss-cert-incorrect
 ipsec auto --add nss-cert-correct
-ipsec whack --impair suppress-retransmits
+ipsec whack --impair suppress_retransmits
 echo "initdone"
