@@ -1,5 +1,6 @@
 # road is redirected to 1 - 192.1.2.44 - which is down
 ipsec whack --impair none
-ipsec whack --impair revival --impair delete-on-retransmit
+ipsec whack --impair revival --impair timeout_on_retransmit
 ipsec auto --add road-east
 ipsec auto --up road-east
+ipsec whack --impair trigger_revival:2
