@@ -1,5 +1,5 @@
 ipsec auto --status | grep interface
-ipsec auto --status | grep orient
+ipsec auto --status | grep "[.][.][.]"
 ipsec auto --ready
 ipsec auto --status | grep interface
-ipsec auto --status | grep orient
+ipsec auto --status | grep "[.][.][.]"
