@@ -1,2 +1,2 @@
-ipsec whack --impair suppress-retransmits
+ipsec whack --impair suppress_retransmits
 ../../guestbin/libreswan-up-down.sh westnet-eastnet-ikev2 -I 192.0.1.254 192.0.2.254
