@@ -1,5 +1,5 @@
 /testing/guestbin/swan-prep
 ipsec start
 ../../guestbin/wait-until-pluto-started
-ipsec auto --add ikev2-west-east
+ipsec add ikev2-west-east
 echo "initdone"
