@@ -1,7 +1,7 @@
 /testing/guestbin/swan-prep --x509
-certutil -D -n west -d sql:/etc/ipsec.d
+ipsec certutil -D -n west
 ipsec start
-/testing/pluto/bin/wait-until-pluto-started
+../../guestbin/wait-until-pluto-started
 # down'ed conn must remain down
 ipsec whack --impair revival
 ipsec auto --add san
