@@ -1,8 +1,11 @@
-/testing/guestbin/swan-prep --x509
-ipsec certutil -D -n west
-# add second identity/cert
-ipsec pk12util -W foobar -K '' -i /testing/x509/pkcs12/otherca/othereast.p12
+/testing/guestbin/swan-prep --nokeys
+
+/testing/x509/import.sh real/mainca/east.all.p12
+/testing/x509/import.sh real/otherca/othereast.all.p12
 ipsec checknss --settrusts
+# check
+ipsec certutil -L
+
 ipsec start
 ../../guestbin/wait-until-pluto-started
 # other is loaded in another test case, ikev2-x509-17-multicert-03
