@@ -1,10 +1,10 @@
 /testing/guestbin/swan-prep --x509
-certutil -D -n west -d sql:/etc/ipsec.d
+ipsec certutil -D -n west
 # add second identity/cert
-pk12util -W foobar -K '' -d sql:/etc/ipsec.d -i /testing/x509/pkcs12/otherca/othereast.p12
+/testing/x509/import.sh real/otherca/othereast.all.p12
 ipsec checknss --settrusts
 ipsec start
-/testing/pluto/bin/wait-until-pluto-started
+../../guestbin/wait-until-pluto-started
 # this causes main to match first, it should not switch since west uses main
 ipsec auto --add other
 ipsec auto --add main
