@@ -1,6 +1,6 @@
 /testing/guestbin/swan-prep --x509
 ipsec certutil -D -n east
-ipsec pk12util -W foobar -K '' -i /testing/x509/pkcs12/otherca/otherwest.p12
+/testing/x509/import.sh real/otherca/otherwest.all.p12
 ipsec checknss --settrusts
 ipsec start
 ../../guestbin/wait-until-pluto-started
