@@ -1,7 +1,7 @@
 /testing/guestbin/swan-prep --x509
-certutil -D -n east -d sql:/etc/ipsec.d
+ipsec certutil -D -n east
 ipsec start
 ../../guestbin/wait-until-pluto-started
 ipsec auto --add main
-ipsec whack --impair suppress-retransmits
+ipsec whack --impair suppress_retransmits
 echo "initdone"
