@@ -2,5 +2,5 @@
 ipsec start
 ../../guestbin/wait-until-pluto-started
 ipsec auto --add ikev2-westnet-eastnet-x509-cr
-ipsec whack --impair send-pkcs7-thingie
+ipsec whack --impair send_pkcs7_thingie
 echo "initdone"
