@@ -1,13 +1,13 @@
 /testing/guestbin/swan-prep --x509
 # confirm that the network is alive
-../../pluto/bin/wait-until-alive -I 192.0.1.254 192.0.2.254
+../../guestbin/wait-until-alive -I 192.0.1.254 192.0.2.254
 # ensure that clear text does not get through
 iptables -A INPUT -i eth1 -s 192.0.2.0/24 -j DROP
 iptables -I INPUT -m policy --dir in --pol ipsec -j ACCEPT
 # confirm clear text does not get through
-../../pluto/bin/ping-once.sh --down -I 192.0.1.254 192.0.2.254
+../../guestbin/ping-once.sh --down -I 192.0.1.254 192.0.2.254
 ipsec start
-/testing/pluto/bin/wait-until-pluto-started
-ipsec whack --impair send-pkcs7-thingie
+../../guestbin/wait-until-pluto-started
+ipsec whack --impair send_pkcs7_thingie
 ipsec auto --add ikev2-westnet-eastnet-x509-cr
 echo "initdone"
