@@ -1,5 +1,10 @@
-/testing/guestbin/swan-prep --x509
-ipsec certutil -D -n east
+/testing/guestbin/swan-prep --nokeys
+
+/testing/x509/import.sh real/mainca/west.all.p12
+ipsec certutil -M -n mainca -t CT,,
+# check
+ipsec certutil -L
+
 # confirm that the network is alive
 ../../guestbin/wait-until-alive -I 192.0.1.254 192.0.2.254
 # ensure that clear text does not get through
