@@ -1,3 +1,4 @@
-../../guestbin/ipsec-look.sh
+../../guestbin/ipsec-kernel-state.sh
+../../guestbin/ipsec-kernel-policy.sh
 # on east, it should show it failed the NSS IPsec profile and used the NSS TLS profile
 hostname | grep east > /dev/null && grep "verify_end_cert trying profile" /tmp/pluto.log
