@@ -1,11 +1,11 @@
-/testing/guestbin/swan-prep --eccert
+/testing/guestbin/swan-prep --nokeys
 
-# import real west+mainca
-ipsec pk12util -W foobar -K '' -i /testing/x509/pkcs12/mainca/west.p12
-# delete real main CA
-ipsec certutil -D -n "Libreswan test CA for mainca - Libreswan"
-# so only CA is bogus
-ipsec pk12util -W foobar -K '' -i /testing/x509/fake/pkcs12/mainec/mainec.p12
+# import real west+key (no CA)
+/testing/x509/import.sh real/mainca/west.end.p12
+# import the fake EC ca (could also use real EC CA?)
+/testing/x509/import.sh fake/mainec/root.cert
+# confirm
+ipsec certutil -L
 
 # confirm that the network is alive
 ../../guestbin/wait-until-alive -I 192.0.1.254 192.0.2.254
