@@ -1,9 +1,9 @@
-/testing/guestbin/swan-prep
+/testing/guestbin/swan-prep --nokeys
 
 # Always zap and recreate special comma certs. Saved under OUTPUT/nss
 # and left waiting for clients (west et.al.) which an import using:
-# pk12util -i OUTPUT/nss/client.p12 -d sql:/etc/ipsec.d -K 'foobar' -W 'foobar'
-# certutil -M -d sql:/etc/ipsec.d -n cacert -t 'CT,,'
+# ipsec pk12util -i OUTPUT/nss/client.p12 -K 'foobar' -W 'foobar'
+# ipsec certutil -M -n cacert -t 'CT,,'
 
 # new scratch DB
 rm -f OUTPUT/nss
@@ -25,8 +25,8 @@ pk12util -o OUTPUT/nss/serverX.p12 -n serverX -d sql:OUTPUT/nss -W 'foobar' -K '
 
 # init and import real server
 ipsec initnss > /dev/null 2> /dev/null
-pk12util -i OUTPUT/nss/server.p12 -d sql:/etc/ipsec.d -K 'foobar' -W 'foobar'
-certutil -M -d sql:/etc/ipsec.d -n cacert -t 'CT,,'
+ipsec pk12util -i OUTPUT/nss/server.p12 -K 'foobar' -W 'foobar'
+ipsec certutil -M -n cacert -t 'CT,,'
 
 ipsec start
 ../../guestbin/wait-until-pluto-started
