@@ -1,8 +1,8 @@
-/testing/guestbin/swan-prep
+/testing/guestbin/swan-prep --nokeys
 rm -f /etc/ipsec.d/*.*
 ipsec initnss > /dev/null 2> /dev/null
-pk12util -i OUTPUT/nss/client.p12 -d sql:/etc/ipsec.d -K 'foobar' -W 'foobar'
-certutil -M -d sql:/etc/ipsec.d -n cacert -t 'CT,,'
+ipsec pk12util -i OUTPUT/nss/client.p12 -K 'foobar' -W 'foobar'
+ipsec certutil -M -n cacert -t 'CT,,'
 ipsec start
 ../../guestbin/wait-until-pluto-started
 ipsec auto --add x509
