@@ -1,4 +1,8 @@
-/testing/guestbin/swan-prep --x509
+/testing/guestbin/swan-prep --nokeys
+
+# RSA, peer is ECDSA
+/testing/x509/import.sh real/mainca/`hostname`.all.p12
+
 ipsec start
 ../../guestbin/wait-until-pluto-started
 ipsec auto --add westnet-eastnet-ikev2
