@@ -1,9 +1,8 @@
 /testing/guestbin/swan-prep --nokeys
-ipsec pk12util -i /testing/x509/strongswan/strongEast.p12 -w /testing/x509/nss-pw
-# Tuomo: why doesn't ipsec checknss --settrust work here?
-ipsec certutil -M -n "strongSwan CA - strongSwan" -t CT,,
-#ipsec start
-ipsec pluto --config /etc/ipsec.conf --leak-detective
+
+/testing/x509/import.sh real/mainec/`hostname`.all.p12
+
+ipsec start
 ../../guestbin/wait-until-pluto-started
 ipsec auto --add westnet-eastnet-ikev2
 ipsec whack --impair suppress_retransmits
