@@ -1,6 +1,6 @@
 ipsec auto --up eastnet-any
 # did we get our IP
-ip addr show dev ipsec1
+../../guestbin/ip.sh address show dev ipsec1
 ../../guestbin/ping-once.sh --up 192.0.2.254
 ipsec whack --trafficstatus
 # check to see if our resolv.conf got updated
@@ -9,5 +9,5 @@ cat /etc/resolv.conf
 ipsec auto --down eastnet-any
 cat /etc/resolv.conf
 # did we get our IP cleaned up
-ip addr show dev ipsec1
+../../guestbin/ip.sh address show dev ipsec1
 echo done
