@@ -1,6 +1,6 @@
-/testing/guestbin/swan-prep
-ip route get to 192.0.2.254 | grep eth1 && ip route del 192.0.2.0/24 via 192.1.3.254 dev eth1
-ip link show ipsec2 || echo "ipsec2 should not exist"
+/testing/guestbin/swan-prep --nokeys
+../../guestbin/ip.sh route get to 192.0.2.254 | grep eth1 && ip route del 192.0.2.0/24 via 192.1.3.254 dev eth1
+../../guestbin/ip.sh link show ipsec2 || echo "ipsec2 should not exist"
 cp north.ipsec2.netdev  /etc/systemd/network/ipsec2.netdev
 : ==== cut ====
 # we need this to check systemd creating interface and hard to sanitize
@@ -12,7 +12,7 @@ systemctl restart systemd-networkd
 # Sep 12 05:37:23 systemd-networkd[489]: ipsec2: netdev ready
 journalctl  --unit=systemd-networkd --no-hostname
 : ==== tuc ====
-ip link show ipsec2
+../../guestbin/ip.sh link show ipsec2
 echo 0 > /proc/sys/net/ipv4/conf/all/rp_filter
 ipsec start
 ../../guestbin/wait-until-pluto-started
