@@ -1,10 +1,10 @@
-/testing/guestbin/swan-prep
+/testing/guestbin/swan-prep --nokeys
 ip address del 192.0.1.254/24 dev eth0 >/dev/null
-ip link set dev eth0 down 2>/dev/null
-ip link set dev ipsec17 down 2>/dev/null
-ip link delete ipsec17 2>/dev/null
-ip route get to 192.0.2.254 | grep eth1 && ip route del 192.0.2.0/24 via 192.1.2.23 dev eth1
-ip link add ipsec17 type xfrm if_id 17 dev eth1
+../../guestbin/ip.sh link set dev eth0 down 2>/dev/null
+../../guestbin/ip.sh link set dev ipsec17 down 2>/dev/null
+../../guestbin/ip.sh link delete ipsec17 2>/dev/null
+../../guestbin/ip.sh route get to 192.0.2.254 | grep eth1 && ip route del 192.0.2.0/24 via 192.1.2.23 dev eth1
+../../guestbin/ip.sh link add ipsec17 type xfrm if_id 17 dev eth1
 ../../guestbin/ip-addr-show.sh ipsec17
 ip address add 192.0.1.254/24 dev ipsec17
 ../../guestbin/ip-addr-show.sh
