@@ -1,6 +1,6 @@
-/testing/guestbin/swan-prep
+/testing/guestbin/swan-prep --nokeys
 ipsec start
 ../../guestbin/wait-until-pluto-started
-# ipsec whack --impair key-length-attribute:0
+# ipsec whack --impair key_length_attribute:0
 ipsec auto --add conf
 echo "initdone"
