@@ -1,4 +1,4 @@
-../../guestbin/netbsd-prep.sh
+../../guestbin/prep.sh
 
 # confirm that the network is alive
 ../../guestbin/wait-until-alive -I 192.1.2.45 192.1.2.23
@@ -6,7 +6,7 @@
 ipsec start
 ../../guestbin/wait-until-pluto-started
 
-ipsec whack --impair suppress-retransmits
+ipsec whack --impair suppress_retransmits
 ipsec auto --add eastnet-westnet-ikev2
 
 echo "initdone"
