@@ -1,5 +1,5 @@
 /testing/guestbin/swan-prep --userland strongswan
-ip addr add 192.0.4.254/32 dev eth0
+../../guestbin/ip.sh address add 192.0.4.254/32 dev eth0
 # ensure that clear text does not get through
 iptables -A INPUT -i eth1 -s 192.0.4.0/24 -j DROP
 iptables -I INPUT -m policy --dir in --pol ipsec -j ACCEPT
