@@ -1,4 +1,4 @@
 ipsec auto --up road-eastnet-ikev2
-ping -n -c 4 192.0.2.254
+../../guestbin/ping-once.sh --up 192.0.2.254
 ipsec whack --trafficstatus
 echo done
