@@ -1,4 +1,4 @@
-ipsec whack --impair suppress-retransmits
+ipsec whack --impair suppress_retransmits
 ipsec auto --up westnet-eastnet-ikev2
 ../../guestbin/ping-once.sh --up -I 192.0.1.254 192.0.2.254
 echo done
