@@ -1,4 +1,4 @@
-/testing/guestbin/swan-prep
+/testing/guestbin/swan-prep --nokeys
 ipsec start
 ipsec auto --add westnet-eastnet-ikev2
 echo "initdone"
